@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
- * distribution and is available at http://eclipse.org/legal/epl-2.0
+ * distribution and is available at https://www.eclipse.org/legal/epl-2.0/
  * or the Apache License, Version 2.0 which accompanies this distribution
  * and is available at https://www.apache.org/licenses/LICENSE-2.0.
  *
@@ -14,9 +14,9 @@
  * License, version 2 with the OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef VALUEPROPAGATIONTABLE_INCL
@@ -48,7 +48,7 @@ TR::Node *constrainBu2l(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainByteConst(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainCall(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainCase(OMR::ValuePropagation *vp, TR::Node *node);
-//TR::Node *constrainCharConst(OMR::ValuePropagation *vp, TR::Node *node);
+// TR::Node *constrainCharConst(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainCheckcast(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainCheckcastNullChk(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainCmp(OMR::ValuePropagation *vp, TR::Node *node);
@@ -67,7 +67,7 @@ TR::Node *constrainFload(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainFloatCmp(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainGoto(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainI2l(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node *constrainIaload(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainAloadi(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainIand(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainIdiv(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainIfcmpeq(OMR::ValuePropagation *vp, TR::Node *node);
@@ -76,7 +76,7 @@ TR::Node *constrainIfcmpgt(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainIfcmple(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainIfcmplt(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainIfcmpne(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node *constrainIiload(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainIloadi(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainImul(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainIneg(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainIabs(OMR::ValuePropagation *vp, TR::Node *node);
@@ -148,18 +148,16 @@ TR::Node *constrainBCDAggrLoad(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainIntegralToBCD(OMR::ValuePropagation *vp, TR::Node *node);
 TR::Node *constrainBCDToIntegral(OMR::ValuePropagation *vp, TR::Node *node);
 
-
-TR::Node * constrainIntegerHighestOneBit(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node * constrainIntegerLowestOneBit(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node * constrainIntegerNumberOfLeadingZeros(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node * constrainIntegerNumberOfTrailingZeros(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node * constrainIntegerBitCount(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node * constrainLongHighestOneBit(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node * constrainLongLowestOneBit(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node * constrainLongNumberOfLeadingZeros(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node * constrainLongNumberOfTrailingZeros(OMR::ValuePropagation *vp, TR::Node *node);
-TR::Node * constrainLongBitCount(OMR::ValuePropagation *vp, TR::Node *node);
-
+TR::Node *constrainIntegerHighestOneBit(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainIntegerLowestOneBit(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainIntegerNumberOfLeadingZeros(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainIntegerNumberOfTrailingZeros(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainIntegerBitCount(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainLongHighestOneBit(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainLongLowestOneBit(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainLongNumberOfLeadingZeros(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainLongNumberOfTrailingZeros(OMR::ValuePropagation *vp, TR::Node *node);
+TR::Node *constrainLongBitCount(OMR::ValuePropagation *vp, TR::Node *node);
 
 /*
  * One-to-one mapping between opcodes and their value propagation handlers
@@ -187,17 +185,17 @@ TR::Node * constrainLongBitCount(OMR::ValuePropagation *vp, TR::Node *node);
 #define brdbarVPHandler constrainIntLoad
 #define srdbarVPHandler constrainShortLoad
 #define lrdbarVPHandler constrainLload
-#define iloadiVPHandler constrainIiload
+#define iloadiVPHandler constrainIloadi
 #define floadiVPHandler constrainFload
 #define dloadiVPHandler constrainDload
-#define aloadiVPHandler constrainIaload
+#define aloadiVPHandler constrainAloadi
 #define bloadiVPHandler constrainIntLoad
 #define sloadiVPHandler constrainShortLoad
 #define lloadiVPHandler constrainLload
-#define irdbariVPHandler constrainIiload
+#define irdbariVPHandler constrainIloadi
 #define frdbariVPHandler constrainFload
 #define drdbariVPHandler constrainDload
-#define ardbariVPHandler constrainIaload
+#define ardbariVPHandler constrainAloadi
 #define brdbariVPHandler constrainIntLoad
 #define srdbariVPHandler constrainShortLoad
 #define lrdbariVPHandler constrainLload
@@ -336,13 +334,13 @@ TR::Node * constrainLongBitCount(OMR::ValuePropagation *vp, TR::Node *node);
 #define f2iVPHandler constrainChildren
 #define f2lVPHandler constrainChildren
 #define f2dVPHandler constrainChildren
-#define f2bVPHandler constrainNarrowToByte
-#define f2sVPHandler constrainNarrowToShort
+#define f2bVPHandler constrainChildren
+#define f2sVPHandler constrainChildren
 #define d2iVPHandler constrainChildren
 #define d2lVPHandler constrainChildren
 #define d2fVPHandler constrainChildren
-#define d2bVPHandler constrainNarrowToByte
-#define d2sVPHandler constrainNarrowToShort
+#define d2bVPHandler constrainChildren
+#define d2sVPHandler constrainChildren
 #define b2iVPHandler constrainB2i
 #define b2lVPHandler constrainB2l
 #define b2fVPHandler constrainChildren
@@ -546,39 +544,43 @@ TR::Node * constrainLongBitCount(OMR::ValuePropagation *vp, TR::Node *node);
 #define compressedRefsVPHandler constrainChildren
 #define BBStartVPHandler constrainChildren
 #define BBEndVPHandler constrainChildren
-#define viminVPHandler constrainChildren
-#define vimaxVPHandler constrainChildren
-#define vigetelemVPHandler constrainChildren
-#define visetelemVPHandler constrainChildren
-#define vimergelVPHandler constrainChildren
-#define vimergehVPHandler constrainChildren
-#define vicmpeqVPHandler constrainChildren
-#define vicmpgtVPHandler constrainChildren
-#define vicmpgeVPHandler constrainChildren
-#define vicmpltVPHandler constrainChildren
-#define vicmpleVPHandler constrainChildren
+#define mAnyTrueVPHandler constrainChildren
+#define mAllTrueVPHandler constrainChildren
+#define mmAnyTrueVPHandler constrainChildren
+#define mmAllTrueVPHandler constrainChildren
+#define mloadVPHandler constrainChildren
+#define mloadiVPHandler constrainChildren
+#define mstoreVPHandler constrainStore
+#define mstoreiVPHandler constrainStore
+#define msplatsVPHandler constrainChildren
+#define mTrueCountVPHandler constrainChildren
+#define mFirstTrueVPHandler constrainChildren
+#define mLastTrueVPHandler constrainChildren
+#define mToLongBitsVPHandler constrainChildren
+#define mLongBitsToMaskVPHandler constrainChildren
+#define mRegLoadVPHandler constrainChildren
+#define mRegStoreVPHandler constrainChildren
+#define mandVPHandler constrainChildren
+#define morVPHandler constrainChildren
+#define mxorVPHandler constrainChildren
+#define b2mVPHandler constrainChildren
+#define s2mVPHandler constrainChildren
+#define i2mVPHandler constrainChildren
+#define l2mVPHandler constrainChildren
+#define v2mVPHandler constrainChildren
+#define m2bVPHandler constrainChildren
+#define m2sVPHandler constrainChildren
+#define m2iVPHandler constrainChildren
+#define m2lVPHandler constrainChildren
+#define m2vVPHandler constrainChildren
 #define vnotVPHandler constrainChildren
-#define vbitselectVPHandler constrainChildren
-#define vpermVPHandler constrainChildren
 #define vsplatsVPHandler constrainChildren
-#define vdmergelVPHandler constrainChildren
-#define vdmergehVPHandler constrainChildren
-#define vdsetelemVPHandler constrainChildren
-#define vdgetelemVPHandler constrainChildren
-#define vdselVPHandler constrainChildren
-#define vdmaxVPHandler constrainChildren
-#define vdminVPHandler constrainChildren
-#define vdcmpeqVPHandler constrainCmp
-#define vdcmpneVPHandler constrainCmp
-#define vdcmpgtVPHandler constrainCmp
-#define vdcmpgeVPHandler constrainCmp
-#define vdcmpltVPHandler constrainCmp
-#define vdcmpleVPHandler constrainCmp
-#define vdsqrtVPHandler constrainChildren
 #define vfmaVPHandler constrainChildren
 #define vnegVPHandler constrainChildren
 #define vabsVPHandler constrainChildren
 #define vsqrtVPHandler constrainChildren
+#define vminVPHandler constrainChildren
+#define vmaxVPHandler constrainChildren
 #define vaddVPHandler constrainAdd
 #define vsubVPHandler constrainSubtract
 #define vmulVPHandler constrainChildren
@@ -596,35 +598,99 @@ TR::Node * constrainLongBitCount(OMR::ValuePropagation *vp, TR::Node *node);
 #define vloadiVPHandler constrainChildren
 #define vstoreVPHandler constrainStore
 #define vstoreiVPHandler constrainStore
+#define vreductionAddVPHandler constrainChildren
+#define vreductionAndVPHandler constrainChildren
+#define vreductionFirstNonZeroVPHandler constrainChildren
+#define vreductionMaxVPHandler constrainChildren
+#define vreductionMinVPHandler constrainChildren
+#define vreductionMulVPHandler constrainChildren
+#define vreductionOrVPHandler constrainChildren
+#define vreductionOrUncheckedVPHandler constrainChildren
+#define vreductionXorVPHandler constrainChildren
 #define vreturnVPHandler constrainReturn
 #define vcallVPHandler constrainCall
 #define vcalliVPHandler constrainCall
-#define vselectVPHandler constrainChildrenFirstToLast
-#define v2vVPHandler constrainChildren
-#define vl2vdVPHandler constrainChildren
-#define vconstVPHandler constrainChildren
-#define getvelemVPHandler constrainChildren
+#define vbitselectVPHandler constrainChildrenFirstToLast
+#define vblendVPHandler constrainChildrenFirstToLast
+#define vcastVPHandler constrainChildren
+#define vconvVPHandler constrainChildren
 #define vsetelemVPHandler constrainChildren
-#define vbRegLoadVPHandler constrainChildren
-#define vsRegLoadVPHandler constrainChildren
-#define viRegLoadVPHandler constrainChildren
-#define vlRegLoadVPHandler constrainChildren
-#define vfRegLoadVPHandler constrainChildren
-#define vdRegLoadVPHandler constrainChildren
-#define vbRegStoreVPHandler constrainChildren
-#define vsRegStoreVPHandler constrainChildren
-#define viRegStoreVPHandler constrainChildren
-#define vlRegStoreVPHandler constrainChildren
-#define vfRegStoreVPHandler constrainChildren
-#define vdRegStoreVPHandler constrainChildren
+#define vindexVectorVPHandler constrainChildren
+#define vorUncheckedVPHandler constrainChildren
+#define vRegLoadVPHandler constrainChildren
+#define vRegStoreVPHandler constrainChildren
+#define vfirstNonZeroVPHandler constrainChildren
+#define vgetelemVPHandler constrainChildren
+
+#define vmabsVPHandler constrainChildren
+#define vmaddVPHandler constrainChildren
+#define vmandVPHandler constrainChildren
+#define vmcmpeqVPHandler constrainChildren
+#define vmcmpneVPHandler constrainChildren
+#define vmcmpgtVPHandler constrainChildren
+#define vmcmpgeVPHandler constrainChildren
+#define vmcmpltVPHandler constrainChildren
+#define vmcmpleVPHandler constrainChildren
+#define vmdivVPHandler constrainChildren
+#define vmfmaVPHandler constrainChildren
+#define vmindexVectorVPHandler constrainChildren
+#define vmloadiVPHandler constrainChildren
+#define vmmaxVPHandler constrainChildren
+#define vmminVPHandler constrainChildren
+#define vmmulVPHandler constrainChildren
+#define vmnegVPHandler constrainChildren
+#define vmnotVPHandler constrainChildren
+#define vmorVPHandler constrainChildren
+#define vmorUncheckedVPHandler constrainChildren
+#define vmreductionAddVPHandler constrainChildren
+#define vmreductionAndVPHandler constrainChildren
+#define vmreductionFirstNonZeroVPHandler constrainChildren
+#define vmreductionMaxVPHandler constrainChildren
+#define vmreductionMinVPHandler constrainChildren
+#define vmreductionMulVPHandler constrainChildren
+#define vmreductionOrVPHandler constrainChildren
+#define vmreductionOrUncheckedVPHandler constrainChildren
+#define vmreductionXorVPHandler constrainChildren
+#define vmsqrtVPHandler constrainChildren
+#define vmstoreiVPHandler constrainChildren
+#define vmsubVPHandler constrainChildren
+#define vmxorVPHandler constrainChildren
+#define vmfirstNonZeroVPHandler constrainChildren
+
+#define vpopcntVPHandler constrainChildren
+#define vmpopcntVPHandler constrainChildren
+#define vcompressVPHandler constrainChildren
+#define vexpandVPHandler constrainChildren
+#define vshlVPHandler constrainChildren
+#define vmshlVPHandler constrainChildren
+#define vshrVPHandler constrainChildren
+#define vmshrVPHandler constrainChildren
+#define vushrVPHandler constrainChildren
+#define vmushrVPHandler constrainChildren
+#define vrolVPHandler constrainChildren
+#define vmrolVPHandler constrainChildren
+#define mcompressVPHandler constrainChildren
+#define vnotzVPHandler constrainChildren
+#define vmnotzVPHandler constrainChildren
+#define vnolzVPHandler constrainChildren
+#define vmnolzVPHandler constrainChildren
+#define vbitswapVPHandler constrainChildren
+#define vmbitswapVPHandler constrainChildren
+#define vbyteswapVPHandler constrainChildren
+#define vmbyteswapVPHandler constrainChildren
+#define vcompressbitsVPHandler constrainChildren
+#define vmcompressbitsVPHandler constrainChildren
+#define vexpandbitsVPHandler constrainChildren
+#define vmexpandbitsVPHandler constrainChildren
+
 #define f2iuVPHandler constrainChildren
 #define f2luVPHandler constrainChildren
 #define f2buVPHandler constrainChildren
-#define f2cVPHandler constrainNarrowToChar
+#define f2cVPHandler constrainChildren
 #define d2iuVPHandler constrainChildren
 #define d2luVPHandler constrainChildren
 #define d2buVPHandler constrainChildren
-#define d2cVPHandler constrainNarrowToChar
+#define d2cVPHandler constrainChildren
 #define monentVPHandler constrainMonent
 #define monexitVPHandler constrainMonexit
 #define monexitfenceVPHandler constrainMonexitfence
@@ -686,6 +752,7 @@ TR::Node * constrainLongBitCount(OMR::ValuePropagation *vp, TR::Node *node);
 #define long2StringVPHandler constrainChildren
 #define bitOpMemVPHandler constrainChildren
 #define arraycmpVPHandler constrainChildren
+#define arraycmplenVPHandler constrainChildren
 #define allocationFenceVPHandler constrainChildren
 #define loadFenceVPHandler constrainChildren
 #define storeFenceVPHandler constrainChildren
@@ -756,6 +823,14 @@ TR::Node * constrainLongBitCount(OMR::ValuePropagation *vp, TR::Node *node);
 #define sbitpermuteVPHandler constrainChildren
 #define ibitpermuteVPHandler constrainChildren
 #define lbitpermuteVPHandler constrainChildren
+#define lcompressbitsVPHandler constrainChildren
+#define icompressbitsVPHandler constrainChildren
+#define scompressbitsVPHandler constrainChildren
+#define bcompressbitsVPHandler constrainChildren
+#define lexpandbitsVPHandler constrainChildren
+#define iexpandbitsVPHandler constrainChildren
+#define sexpandbitsVPHandler constrainChildren
+#define bexpandbitsVPHandler constrainChildren
 #define PrefetchVPHandler constrainChildren
 
 #ifdef J9_PROJECT_SPECIFIC
@@ -854,52 +929,30 @@ TR::Node * constrainLongBitCount(OMR::ValuePropagation *vp, TR::Node *node);
 #define pdModifyPrecisionVPHandler constrainChildren
 #define countDigitsVPHandler constrainChildren
 #define BCDCHKVPHandler constrainBCDCHK
+#define zdchkVPHandler constrainChildren
 #endif
 
 const ValuePropagationPointerTable constraintHandlers;
 
-const ValuePropagationPtr ValuePropagationPointerTable::table[] =
-   {
-#define OPCODE_MACRO(\
-   opcode, \
-   name, \
-   prop1, \
-   prop2, \
-   prop3, \
-   prop4, \
-   dataType, \
-   typeProps, \
-   childProps, \
-   swapChildrenOpcode, \
-   reverseBranchOpcode, \
-   boolCompareOpcode, \
-   ifCompareOpcode, \
-   ...) opcode ## VPHandler,
+const ValuePropagationPtr ValuePropagationPointerTable::table[] = {
+#define OPCODE_MACRO(opcode, name, prop1, prop2, prop3, prop4, dataType, typeProps, childProps, swapChildrenOpcode, \
+    reverseBranchOpcode, boolCompareOpcode, ifCompareOpcode, ...)                                                   \
+    opcode##VPHandler,
 
-   BadILOpVPHandler,
+    BadILOpVPHandler,
 
 #include "il/Opcodes.enum"
 #undef OPCODE_MACRO
 
-#define VECTOR_OPERATION_MACRO(\
-   operation, \
-   name, \
-   prop1, \
-   prop2, \
-   prop3, \
-   prop4, \
-   dataType, \
-   typeProps, \
-   childProps, \
-   swapChildrenOpcode, \
-   reverseBranchOpcode, \
-   boolCompareOpcode, \
-   ifCompareOpcode, \
-   ...) operation ## VPHandler,
+#define VECTOR_OPERATION_MACRO(operation, name, prop1, prop2, prop3, prop4, dataType, typeProps, childProps, \
+    swapChildrenOpcode, reverseBranchOpcode, boolCompareOpcode, ifCompareOpcode, ...)                        \
+    operation##VPHandler,
+
+    BadILOpVPHandler,
 
 #include "il/VectorOperations.enum"
 #undef VECTOR_OPERATION_MACRO
 
-   };
+};
 
 #endif
