@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
- * distribution and is available at http://eclipse.org/legal/epl-2.0
+ * distribution and is available at https://www.eclipse.org/legal/epl-2.0/
  * or the Apache License, Version 2.0 which accompanies this distribution
  * and is available at https://www.apache.org/licenses/LICENSE-2.0.
  *
@@ -14,9 +14,9 @@
  * License, version 2 with the OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /*
@@ -24,16 +24,5 @@
  * definitions are permitted.
  */
 
-   Unknown390Processor,
-   s370,
-   s370gp1,
-   s370gp2,
-   s370gp3,
-   s370gp4,
-   s370gp5,
-   s370gp6,
-   s370gp7,
-   s370gp8,
-   s370gp9,
-   s370gp10,
-   s370gp11,
+Unknown390Processor, s370, s370gp1, s370gp2, s370gp3, s370gp4, s370gp5, s370gp6, s370gp7, s370gp8, s370gp9, s370gp10,
+    s370gp11,
