@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2015, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2015
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
- * distribution and is available at http://eclipse.org/legal/epl-2.0
+ * distribution and is available at https://www.eclipse.org/legal/epl-2.0/
  * or the Apache License, Version 2.0 which accompanies this distribution
  * and is available at https://www.apache.org/licenses/LICENSE-2.0.
  *
@@ -14,9 +14,9 @@
  * License, version 2 with the OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "ddr/scanner/dwarf/DwarfScanner.hpp"
@@ -41,6 +41,157 @@
 #include <stack>
 #include <utility>
 
+#define DW_LIBDWARF_MAKE_VERSION(major, minor) (((major) * 100) + (minor))
+
+#if defined(DW_LIBDWARF_VERSION_MAJOR) && defined(DW_LIBDWARF_VERSION_MINOR)
+#define OMR_LIBDWARF_VERSION DW_LIBDWARF_MAKE_VERSION(DW_LIBDWARF_VERSION_MAJOR, DW_LIBDWARF_VERSION_MINOR)
+#else /* defined(DW_LIBDWARF_VERSION_MAJOR) && defined(DW_LIBDWARF_VERSION_MINOR) */
+#define OMR_LIBDWARF_VERSION 0
+#endif /* defined(DW_LIBDWARF_VERSION_MAJOR) && defined(DW_LIBDWARF_VERSION_MINOR) */
+
+/* Wrappers to adapt to differences across libdwarf API versions. */
+
+#if OMR_LIBDWARF_VERSION >= DW_LIBDWARF_MAKE_VERSION(0, 9)
+
+static int
+ddr_dw_finish(
+	Dwarf_Debug  dbg,
+	Dwarf_Error *error)
+{
+	if (NULL != error) {
+		*error = DW_DLV_OK;
+	}
+
+	return dwarf_finish(dbg);
+}
+
+static int
+ddr_dw_init(
+	int           fd,
+	Dwarf_Handler errhand,
+	Dwarf_Ptr     errarg,
+	Dwarf_Debug  *dbg,
+	Dwarf_Error  *error)
+{
+	unsigned int groupnumber = DW_GROUPNUMBER_ANY;
+
+	return dwarf_init_b(fd, groupnumber, errhand, errarg, dbg, error);
+}
+
+static int
+ddr_dw_next_cu_header(
+	Dwarf_Debug     dbg,
+	Dwarf_Bool      is_info,
+	Dwarf_Die      *cu_die,
+	Dwarf_Unsigned *cu_header_length,
+	Dwarf_Half     *version_stamp,
+	Dwarf_Off      *abbrev_offset,
+	Dwarf_Half     *address_size,
+	Dwarf_Half     *length_size,
+	Dwarf_Half     *extension_size,
+	Dwarf_Unsigned *typeoffset,
+	Dwarf_Unsigned *next_cu_header_offset,
+	Dwarf_Half     *header_cu_type,
+	Dwarf_Error    *error)
+{
+	Dwarf_Sig8 type_signature = { 0 };
+
+	return dwarf_next_cu_header_e(
+			dbg,
+			is_info,
+			cu_die,
+			cu_header_length,
+			version_stamp,
+			abbrev_offset,
+			address_size,
+			length_size,
+			extension_size,
+			&type_signature,
+			typeoffset,
+			next_cu_header_offset,
+			header_cu_type,
+			error);
+}
+
+static int
+ddr_dw_siblingof(
+	Dwarf_Debug  dbg,
+	Dwarf_Die    die,
+	Dwarf_Die   *return_siblingdie,
+	Dwarf_Error *error)
+{
+	return dwarf_siblingof_c(die, return_siblingdie, error);
+}
+
+#else /* OMR_LIBDWARF_VERSION >= DW_LIBDWARF_MAKE_VERSION(0, 9) */
+
+#define DW_GROUPNUMBER_ANY 0
+
+static int
+ddr_dw_finish(
+		Dwarf_Debug  dbg,
+		Dwarf_Error *error)
+{
+	return dwarf_finish(dbg, error);
+}
+
+static int
+ddr_dw_init(
+	int           fd,
+	Dwarf_Handler errhand,
+	Dwarf_Ptr     errarg,
+	Dwarf_Debug  *dbg,
+	Dwarf_Error  *error)
+{
+	Dwarf_Unsigned access = DW_DLC_READ;
+
+	return dwarf_init(fd, access, errhand, errarg, dbg, error);
+}
+
+static int
+ddr_dw_next_cu_header(
+	Dwarf_Debug     dbg,
+	Dwarf_Bool      is_info,
+	Dwarf_Die      *cu_die,
+	Dwarf_Unsigned *cu_header_length,
+	Dwarf_Half     *version_stamp,
+	Dwarf_Off      *abbrev_offset,
+	Dwarf_Half     *address_size,
+	Dwarf_Half     *length_size,
+	Dwarf_Half     *extension_size,
+	Dwarf_Unsigned *typeoffset,
+	Dwarf_Unsigned *next_cu_header_offset,
+	Dwarf_Half     *header_cu_type,
+	Dwarf_Error    *error)
+{
+	if (NULL != cu_die) {
+		*cu_die = NULL;
+	}
+
+	return dwarf_next_cu_header(
+			dbg,
+			cu_header_length,
+			version_stamp,
+			abbrev_offset,
+			address_size,
+			next_cu_header_offset,
+			error);
+}
+
+static int
+ddr_dw_siblingof(
+	Dwarf_Debug  dbg,
+	Dwarf_Die    die,
+	Dwarf_Die   *return_siblingdie,
+	Dwarf_Error *error)
+{
+	return dwarf_siblingof(dbg, die, return_siblingdie, error);
+}
+
+#endif /* OMR_LIBDWARF_VERSION >= DW_LIBDWARF_MAKE_VERSION(0, 9) */
+
+static DDR_RC getConstValue(Dwarf_Debug debug, Dwarf_Attribute attr, const char *attrName, bool assumeSigned, uint64_t *value);
+
 class DwarfVisitor : public TypeVisitor
 {
 private:
@@ -232,19 +383,17 @@ DwarfScanner::excludedDie(Dwarf_Die die, bool *dieExcluded)
 				goto Done;
 			}
 
-			Dwarf_Unsigned declFile = 0;
-			int ret = dwarf_formudata(attr, &declFile, &err);
+			uint64_t declFile = 0;
+			rc = getConstValue(_debug, attr, "decl_file", false, &declFile);
 			dwarf_dealloc(_debug, attr, DW_DLA_ATTR);
-			if (DW_DLV_ERROR == ret) {
-				ERRMSG("Getting attr value decl_file: %s\n", dwarf_errmsg(err));
-				rc = DDR_RC_ERROR;
+			if (DDR_RC_OK != rc) {
 				goto Done;
 			}
 
 			if (0 == declFile) {
 				/* no declaring file is specified */
 				excludedFile = false;
-			} else if (declFile <= (Dwarf_Unsigned)_fileNameCount) {
+			} else if (declFile <= (uint64_t)(Dwarf_Unsigned)_fileNameCount) {
 				const string fileName(_fileNamesTable[declFile - 1]);
 				excludedFile = checkExcludedFile(fileName);
 			} else {
@@ -402,19 +551,17 @@ DwarfScanner::getBitField(Dwarf_Die die, size_t *bitField)
 		if (hasAttr) {
 			Dwarf_Attribute attr = NULL;
 			if (DW_DLV_ERROR == dwarf_attr(die, DW_AT_bit_size, &attr, &err)) {
-				ERRMSG("Getting bit size attribute: %s\n", dwarf_errmsg(err));
+				ERRMSG("Getting bit_size attribute: %s\n", dwarf_errmsg(err));
 				rc = DDR_RC_ERROR;
 				goto Done;
 			}
-			Dwarf_Off bitSize = 0;
-			int ret = dwarf_formudata(attr, &bitSize, &err);
+			uint64_t bitSize = 0;
+			rc = getConstValue(_debug, attr, "bit_size", false, &bitSize);
 			dwarf_dealloc(_debug, attr, DW_DLA_ATTR);
-			if (DW_DLV_ERROR == ret) {
-				ERRMSG("Getting formudata of bit size: %s\n", dwarf_errmsg(err));
-				rc = DDR_RC_ERROR;
+			if (DDR_RC_OK != rc) {
 				goto Done;
 			}
-			*bitField = bitSize;
+			*bitField = (size_t)bitSize;
 		}
 	}
 Done:
@@ -456,7 +603,7 @@ DwarfScanner::getTypeInfo(Dwarf_Die die, Dwarf_Die *dieOut, string *typeName, Mo
 				modifiers->_modifierFlags |= Modifiers::SHARED_TYPE;
 			} else if (DW_TAG_array_type == tag && !foundTypedef) {
 				Dwarf_Die child = NULL;
-				Dwarf_Unsigned upperBound = 0;
+				uint64_t upperBound = 0;
 				int ret = dwarf_child(typeDie, &child, &err);
 
 				/* Get the array size if it is an array. */
@@ -482,11 +629,9 @@ DwarfScanner::getTypeInfo(Dwarf_Die die, Dwarf_Die *dieOut, string *typeName, Mo
 								rc = DDR_RC_ERROR;
 								break;
 							}
-							ret = dwarf_formudata(attr, &upperBound, &err);
+							rc = getConstValue(_debug, attr, "upper_bound", false, &upperBound);
 							dwarf_dealloc(_debug, attr, DW_DLA_ATTR);
-							if (DW_DLV_ERROR == ret) {
-								ERRMSG("Getting array upper bound value: %s\n", dwarf_errmsg(err));
-								rc = DDR_RC_ERROR;
+							if (DDR_RC_OK != rc) {
 								break;
 							}
 							/* New array length is upperBound + 1. */
@@ -655,14 +800,11 @@ DwarfScanner::getTypeSize(Dwarf_Die die, size_t *typeSize)
 				ERRMSG("Getting size attr of type: %s\n", dwarf_errmsg(err));
 				rc = DDR_RC_ERROR;
 			} else {
-				Dwarf_Unsigned byteSize = 0;
-				int ret = dwarf_formudata(attr, &byteSize, &err);
+				uint64_t byteSize = 0;
+				rc = getConstValue(_debug,  attr, "byte_size", false, &byteSize);
 				dwarf_dealloc(_debug, attr, DW_DLA_ATTR);
-				if (DW_DLV_ERROR == ret) {
-					ERRMSG("Getting size attr value: %s\n", dwarf_errmsg(err));
-					rc = DDR_RC_ERROR;
-				} else {
-					*typeSize = byteSize;
+				if (DDR_RC_OK == rc) {
+					*typeSize = (size_t)byteSize;
 				}
 			}
 		} else {
@@ -1101,10 +1243,12 @@ DwarfVisitor::visitUnion(UnionUDT *newType) const
 }
 
 /*
- * Given an attribute of type DW_AT_const_value, return its value.
+ * Extract the constant value of the given attribute.
+ * Some forms are used for both signed and unsigned attributes;
+ * assumeSigned indicates how ambiguous forms should be handled.
  */
 static DDR_RC
-getConstValue(Dwarf_Debug debug, Dwarf_Attribute attr, uint64_t *value)
+getConstValue(Dwarf_Debug debug, Dwarf_Attribute attr, const char *attrName, bool assumeSigned, uint64_t *value)
 {
 	DDR_RC rc = DDR_RC_ERROR;
 	Dwarf_Error error = NULL;
@@ -1113,12 +1257,12 @@ getConstValue(Dwarf_Debug debug, Dwarf_Attribute attr, uint64_t *value)
 	/* Get the literal value form. */
 	/* Get the literal value. */
 	if (DW_DLV_ERROR == dwarf_whatform(attr, &form, &error)) {
-		ERRMSG("Getting form of const_value attribute: %s\n", dwarf_errmsg(error));
+		ERRMSG("Getting form of %s attribute: %s\n", attrName, dwarf_errmsg(error));
 	} else if ((DW_FORM_block  == form) || (DW_FORM_block1 == form)
 			|| (DW_FORM_block2 == form) || (DW_FORM_block4 == form)) {
 		Dwarf_Block *block = NULL;
 		if (DW_DLV_ERROR == dwarf_formblock(attr, &block, &error)) {
-			ERRMSG("Getting block of const_value attribute: %s\n", dwarf_errmsg(error));
+			ERRMSG("Getting block of %s attribute: %s\n", attrName, dwarf_errmsg(error));
 		} else {
 			/* Note: This assumes the host byte-order. */
 			switch (block->bl_len) {
@@ -1139,28 +1283,57 @@ getConstValue(Dwarf_Debug debug, Dwarf_Attribute attr, uint64_t *value)
 				rc = DDR_RC_OK;
 				break;
 			default:
-				ERRMSG("Unsupported const_value block size: %d\n", (int )block->bl_len);
+				ERRMSG("Unsupported block size of %s attribute: %d\n", attrName, (int)block->bl_len);
 				break;
 			}
 			dwarf_dealloc(debug, block, DW_DLA_BLOCK);
 		}
+	} else if (DW_FORM_sdata == form) {
+		Dwarf_Signed svalue = 0;
+
+		if (DW_DLV_OK == dwarf_formsdata(attr, &svalue, &error)) {
+			*value = (uint64_t)(int64_t)svalue;
+			rc = DDR_RC_OK;
+		} else {
+			ERRMSG("Getting formudata of %s attribute: %s\n", attrName, dwarf_errmsg(error));
+		}
 	} else if (DW_FORM_udata == form) {
 		Dwarf_Unsigned uvalue = 0;
 
-		if (DW_DLV_ERROR == dwarf_formudata(attr, &uvalue, &error)) {
-			ERRMSG("Getting const_value of enum: %s\n", dwarf_errmsg(error));
-		} else {
+		if (DW_DLV_OK == dwarf_formudata(attr, &uvalue, &error)) {
 			*value = uvalue;
 			rc = DDR_RC_OK;
+		} else {
+			ERRMSG("Getting formudata of %s attribute: %s\n", attrName, dwarf_errmsg(error));
 		}
 	} else {
 		Dwarf_Signed svalue = 0;
+		Dwarf_Unsigned uvalue = 0;
 
-		if (DW_DLV_ERROR == dwarf_formsdata(attr, &svalue, &error)) {
-			ERRMSG("Getting const_value of enum: %s\n", dwarf_errmsg(error));
+		/*
+		 * For other forms, e.g. DW_FORM_data{1,2,4,8}, assumeSigned
+		 * indicates whether the value is signed or not by default.
+		 */
+		if (assumeSigned) {
+			if (DW_DLV_OK == dwarf_formsdata(attr, &svalue, &error)) {
+				*value = (uint64_t)(int64_t)svalue;
+				rc = DDR_RC_OK;
+			} else if (DW_DLV_OK == dwarf_formudata(attr, &uvalue, &error)) {
+				*value = uvalue;
+				rc = DDR_RC_OK;
+			} else {
+				ERRMSG("Getting formsdata of %s attribute: %s\n", attrName, dwarf_errmsg(error));
+			}
 		} else {
-			*value = (uint64_t)(int64_t)svalue;
-			rc = DDR_RC_OK;
+			if (DW_DLV_OK == dwarf_formudata(attr, &uvalue, &error)) {
+				*value = uvalue;
+				rc = DDR_RC_OK;
+			} else if (DW_DLV_OK == dwarf_formsdata(attr, &svalue, &error)) {
+				*value = (uint64_t)(int64_t)svalue;
+				rc = DDR_RC_OK;
+			} else {
+				ERRMSG("Getting formsdata of %s attribute: %s\n", attrName, dwarf_errmsg(error));
+			}
 		}
 	}
 
@@ -1208,7 +1381,7 @@ DwarfScanner::addEnumMember(Dwarf_Die die, NamespaceUDT *outerUDT, EnumUDT *newE
 		goto AddEnumMemberError;
 	}
 
-	rc = getConstValue(_debug, attr, &enumValue);
+	rc = getConstValue(_debug, attr, "const_value", true, &enumValue);
 	dwarf_dealloc(_debug, attr, DW_DLA_ATTR);
 
 	if (DDR_RC_OK == rc) {
@@ -1404,6 +1577,7 @@ DwarfScanner::addClassField(Dwarf_Die die, ClassType *newClass, const string &fi
 			if (hasAttrBytes || hasAttrBits) {
 				Dwarf_Attribute attr = NULL;
 				Dwarf_Unsigned offset = 0;
+				uint64_t offset64 = 0;
 				Dwarf_Half attrType = hasAttrBytes ? DW_AT_data_member_location : DW_AT_data_bit_offset;
 				if (DW_DLV_ERROR == dwarf_attr(die, attrType, &attr, &error)) {
 					ERRMSG("Getting offset attribute: %s\n", dwarf_errmsg(error));
@@ -1432,8 +1606,9 @@ DwarfScanner::addClassField(Dwarf_Die die, ClassType *newClass, const string &fi
 								newClass->_name.c_str(), fieldName.c_str());
 						goto FreeAttrThenDone;
 					}
-				} else if (DW_DLV_ERROR == dwarf_formudata(attr, &offset, &error)) {
-					ERRMSG("Getting value of offset attribute: %s\n", dwarf_errmsg(error));
+				} else if (DDR_RC_OK == getConstValue(_debug, attr, "offset", false, &offset64)) {
+					offset = (Dwarf_Unsigned)offset64;
+				} else {
 FreeAttrThenDone:
 					dwarf_dealloc(_debug, attr, DW_DLA_ATTR);
 					goto AddUDTFieldDone;
@@ -1504,7 +1679,26 @@ DwarfScanner::traverse_cu_in_debug_section(Symbol_IR *ir)
 		_typeOffsetMap.clear();
 		_ir = &newIR;
 
-		int ret = dwarf_next_cu_header(_debug, &cuHeaderLength, &versionStamp, &abbrevOffset, &addressSize, &nextCUheader, &error);
+		Dwarf_Die cuDie = NULL;
+		Dwarf_Bool is_info = true;
+		Dwarf_Half length_size = 0;
+		Dwarf_Half extension_size = 0;
+		Dwarf_Unsigned typeoffset = 0;
+		Dwarf_Half header_cu_type = 0;
+		int ret = ddr_dw_next_cu_header(
+				_debug,
+				is_info,
+				&cuDie,
+				&cuHeaderLength,
+				&versionStamp,
+				&abbrevOffset,
+				&addressSize,
+				&length_size,
+				&extension_size,
+				&typeoffset,
+				&nextCUheader,
+				&header_cu_type,
+				&error);
 		if (DW_DLV_ERROR == ret) {
 			ERRMSG("Failed to get next dwarf CU header.");
 			rc = DDR_RC_ERROR;
@@ -1514,11 +1708,10 @@ DwarfScanner::traverse_cu_in_debug_section(Symbol_IR *ir)
 			/* No more CU's. */
 			break;
 		}
-		Dwarf_Die cuDie = NULL;
 		Dwarf_Die childDie = NULL;
 
 		/* Expect the CU to have a single sibling - a DIE */
-		if (DW_DLV_ERROR == dwarf_siblingof(_debug, NULL, &cuDie, &error)) {
+		if (DW_DLV_ERROR == ddr_dw_siblingof(_debug, cuDie, &cuDie, &error)) {
 			ERRMSG("Getting sibling of CU: %s\n", dwarf_errmsg(error));
 			rc = DDR_RC_ERROR;
 			break;
@@ -1617,12 +1810,11 @@ DwarfScanner::scanFile(OMRPortLibrary *portLibrary, Symbol_IR *ir, const char *f
 	}
 
 	if (DDR_RC_OK == rc) {
-		Dwarf_Unsigned access = DW_DLC_READ;
 		Dwarf_Handler errhand = 0;
 		Dwarf_Ptr errarg = NULL;
 		intptr_t native_fd = omrfile_convert_omrfile_fd_to_native_fd(fd);
 		DwarfScanner::scanFileName = filepath;
-		res = dwarf_init((int)native_fd, access, errhand, errarg, &_debug, &error);
+		res = ddr_dw_init((int)native_fd, errhand, errarg, &_debug, &error);
 		if (DW_DLV_OK != res) {
 			ERRMSG("Failed to initialize libDwarf scanning %s: %s\nExiting...\n", filepath, dwarf_errmsg(error));
 			if (NULL != error) {
@@ -1640,7 +1832,7 @@ DwarfScanner::scanFile(OMRPortLibrary *portLibrary, Symbol_IR *ir, const char *f
 
 		DEBUGPRINTF("Unloading libDwarf");
 
-		res = dwarf_finish(_debug, &error);
+		res = ddr_dw_finish(_debug, &error);
 		if (DW_DLV_OK != res) {
 			ERRMSG("Failed to Unload libDwarf: %s\nExiting...\n", dwarf_errmsg(error));
 			if (NULL != error) {
@@ -1681,7 +1873,7 @@ DwarfScanner::getNextSibling(Dwarf_Die *die)
 	Dwarf_Error err = NULL;
 
 	/* Get the next sibling and free the previous one if successful. */
-	int ret = dwarf_siblingof(_debug, *die, &nextSibling, &err);
+	int ret = ddr_dw_siblingof(_debug, *die, &nextSibling, &err);
 	if (DW_DLV_ERROR == ret) {
 		ERRMSG("Getting sibling of die:%s\n", dwarf_errmsg(err));
 	} else if (DW_DLV_OK == ret) {
