@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2018 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
- * distribution and is available at http://eclipse.org/legal/epl-2.0
+ * distribution and is available at https://www.eclipse.org/legal/epl-2.0/
  * or the Apache License, Version 2.0 which accompanies this distribution
  * and is available at https://www.apache.org/licenses/LICENSE-2.0.
  *
@@ -14,9 +14,9 @@
  * License, version 2 with the OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /*
@@ -56,8 +56,8 @@ extern PortTestEnvironment *portTestEnv;
 #endif
 
 #if defined(OMR_ENV_DATA64)
-/* this macro corresponds to the one defined in omrmem32helpers */
-#define HEAP_SIZE_BYTES 8*1024*1024
+/* This macro corresponds to SUBALLOCATOR_INCREMENT_SIZE defined in omrgcconsts.h. */
+#define HEAP_SIZE_BYTES (8 * 1024 * 1024)
 #endif
 
 #define COMPLETE_LARGE_REGION 1
@@ -632,6 +632,57 @@ TEST(PortMemTest, mem_test7_allocate32)
 }
 
 
+
+/**
+ * Verify port library memory management.
+ *
+ * Ensure large allocation sizes do not overflow.
+ * @ref omrmem.c::omrmem_allocate_memory "omrmem_reallocate_memory()".
+ * @ref omrmem.c::omrmem_allocate_memory32 "omrmem_reallocate_memory32()".
+ */
+TEST(PortMemTest, mem_test8)
+{
+	OMRPORT_ACCESS_FROM_OMRPORT(portTestEnv->getPortLibrary());
+	const char *testName = "omrmem_test8";
+
+	reportTestEntry(OMRPORTLIB, testName);
+
+	/* Ensure that allocating a large size does not overflow into a reasonable size. */
+	uintptr_t byteAmount = UINTPTR_MAX;
+	void *memPtr = omrmem_allocate_memory(byteAmount, OMRMEM_CATEGORY_PORT_LIBRARY);
+	if (NULL != memPtr){
+		outputErrorMessage(PORTTEST_ERROR_ARGS, "Unexpected allocation success\n");
+		omrmem_free_memory(memPtr);
+	}
+
+	/* Check for overflow in realloc. First we need a valid allocation. */
+	memPtr = omrmem_allocate_memory(32, OMRMEM_CATEGORY_PORT_LIBRARY);
+	if (NULL == memPtr) {
+		outputErrorMessage(PORTTEST_ERROR_ARGS, "Allocation error while testing realloc\n");
+	} else {
+		void *memPtr2 = omrmem_reallocate_memory(memPtr, byteAmount, OMRMEM_CATEGORY_PORT_LIBRARY);
+		if (NULL != memPtr2) {
+			outputErrorMessage(PORTTEST_ERROR_ARGS, "Unexpected reallocation success\n");
+			omrmem_free_memory(memPtr2);
+		} else {
+			omrmem_free_memory(memPtr);
+		}
+	}
+
+#if defined(OMR_ENV_DATA64)
+	/* Check for overflow behaviour in 32 bit allocations. */
+	byteAmount = UINT32_MAX;
+	memPtr = omrmem_allocate_memory32(byteAmount, OMRMEM_CATEGORY_PORT_LIBRARY);
+	if (NULL != memPtr){
+		outputErrorMessage(PORTTEST_ERROR_ARGS, "Unexpected 32-bit allocation success\n");
+		omrmem_free_memory(memPtr);
+	}
+#endif /* defined(OMR_ENV_DATA64) */
+
+	reportTestExit(OMRPORTLIB, testName);
+}
+
+
 /* Dummy categories for omrmem_test8_categories */
 
 #define DUMMY_CATEGORY_ONE 0
@@ -1058,6 +1109,7 @@ TEST(PortMemTest, mem_test8_categories)
 		}
 	}
 
+#if !defined(OSX) || !defined(OMR_ARCH_AARCH64)
 	/* Try allocating with allocate32 */
 	{
 		uintptr_t initialBlocks = 0;
@@ -1219,6 +1271,7 @@ TEST(PortMemTest, mem_test8_categories)
 		}
 	}
 #endif /* defined(OMR_ENV_DATA64) */
+#endif /* !defined(OSX) || !defined(OMR_ARCH_AARCH64) */
 
 	/* Try allocating and reallocating */
 	{
