@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2020 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "AllocateDescription.hpp"
@@ -194,15 +194,16 @@ MM_Collector::recordExcessiveStatsForGCEnd(MM_EnvironmentBase* env)
  * @param aggressive True if this is an aggressive collect, otherwise false
  */
 void
-MM_Collector::preCollect(MM_EnvironmentBase* env, MM_MemorySubSpace* subSpace, MM_AllocateDescription* allocDescription, uint32_t gcCode)
+MM_Collector::preCollect(MM_EnvironmentBase *env, MM_MemorySubSpace *subSpace, MM_AllocateDescription *allocDescription, uint32_t gcCode)
 {
-	MM_GCExtensionsBase* extensions = env->getExtensions();
+	MM_GCExtensionsBase *extensions = env->getExtensions();
 
 	/* There might be a colliding concurrent cycle in progress, that must be completed before we start this one.
 	 * Specific Collector subclass will have exact knowledge if that is the case.
 	 */
-	completeExternalConcurrentCycle(env);
+	completeExternalConcurrentCycle(env, subSpace, allocDescription, gcCode);
 
+	Assert_MM_false(_stwCollectionInProgress);
 	_stwCollectionInProgress = true;
 	
 	/* Record the main GC thread CPU time at the start to diff later */
@@ -258,10 +259,15 @@ MM_Collector::preCollect(MM_EnvironmentBase* env, MM_MemorySubSpace* subSpace, M
 		}
 	}
 
-	/* If this is a global collection then set the globalgc flag.  This will allow us to
-	 * trigger excessiveGC checks from a local collection that has recursed into a global gc .
+	/* If this is a global collection then set the didGlobalGC flag.  This will allow us to
+	 * trigger excessiveGC checks from a local collection that has recursed into a global GC.
+	 * Only include GCs that collect the whole heap to correctly evaluate the reclaimed
+	 * memory criteria.
+	 * There may be collectors that have global domain (any part of heap may be collected),
+	 * but not necessarily all the heap is collected, so all possible global GC criteria
+	 * are included.
 	 */
-	if (_globalCollector) {
+	if (_globalCollector && (MM_CycleState::CT_GLOBAL_GARBAGE_COLLECTION == env->_cycleState->_collectionType)) {
 		extensions->didGlobalGC = true;
 	}
 }
@@ -283,17 +289,7 @@ MM_Collector::checkForExcessiveGC(MM_EnvironmentBase* env, MM_Collector *collect
 	Assert_MM_true(extensions->excessiveGCEnabled._valueSpecified);
 
 	/* Get gc count now collect has happened */
-	UDATA gcCount = 0;
-	if (extensions->isStandardGC()) {
-		gcCount += extensions->globalGCStats.gcCount;
-#if defined(OMR_GC_MODRON_SCAVENGER)
-		gcCount += extensions->scavengerStats._gcCount;
-#endif /* defined(OMR_GC_MODRON_SCAVENGER) */
-	} else if (extensions->isVLHGC()) {
-#if defined(OMR_GC_VLHGC)
-		gcCount += extensions->globalVLHGCStats.gcCount;
-#endif /* defined(OMR_GC_VLHGC) */
-	}
+	UDATA gcCount = extensions->getUniqueGCCycleCount();
 
 	OMRPORT_ACCESS_FROM_ENVIRONMENT(env);
 	TRIGGER_J9HOOK_MM_PRIVATE_EXCESSIVEGC_CHECK_GC_ACTIVITY(extensions->privateHookInterface,
@@ -484,6 +480,8 @@ MM_Collector::garbageCollect(MM_EnvironmentBase* env, MM_MemorySubSpace* calling
 {
 	Assert_MM_mustHaveExclusiveVMAccess(env->getOmrVMThread());
 
+	uintptr_t vmState = env->pushVMstate(getVMStateID());
+
 	Assert_MM_true(NULL == env->_cycleState);
 	preCollect(env, callingSubSpace, allocateDescription, gcCode);
 	Assert_MM_true(NULL != env->_cycleState);
@@ -491,16 +489,12 @@ MM_Collector::garbageCollect(MM_EnvironmentBase* env, MM_MemorySubSpace* calling
 	/* ensure that we aren't trying to collect while in a NoGC allocation */
 	Assert_MM_false(env->_isInNoGCAllocationCall);
 
-	uintptr_t vmState = env->pushVMstate(getVMStateID());
-
 	/* First do any pre-collection initialization of the collector*/
 	setupForGC(env);
 
 	/* perform the collection */
 	_gcCompleted = internalGarbageCollect(env, callingSubSpace, allocateDescription);
 
-	env->popVMstate(vmState);
-
 	/* now, see if we need to resume an allocation or replenishment attempt */
 	void* postCollectAllocationResult = NULL;
 	if (NULL != allocateDescription) {
@@ -522,6 +516,8 @@ MM_Collector::garbageCollect(MM_EnvironmentBase* env, MM_MemorySubSpace* calling
 	Assert_MM_true(NULL != env->_cycleState);
 	env->_cycleState = NULL;
 
+	env->popVMstate(vmState);
+
 	return postCollectAllocationResult;
 }
 
