@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 
@@ -30,10 +30,15 @@
 #include "MemcheckWrapper.hpp"
 #endif /* defined(OMR_VALGRIND_MEMCHECK) */
 
-MM_MemoryManager*
-MM_MemoryManager::newInstance(MM_EnvironmentBase* env)
+#define TWO_GB_ADDRESS ((void *)((uintptr_t)2 * 1024 * 1024 * 1024))
+#define EIGHT_GB_ADDRESS ((void *)((uintptr_t)8 * 1024 * 1024 * 1024))
+#define SIXTEEN_GB_ADDRESS ((void *)((uintptr_t)16 * 1024 * 1024 * 1024))
+#define THIRTY_TWO_GB_ADDRESS ((void *)((uintptr_t)32 * 1024 * 1024 * 1024))
+
+MM_MemoryManager *
+MM_MemoryManager::newInstance(MM_EnvironmentBase *env)
 {
-	MM_MemoryManager* memoryManager = (MM_MemoryManager*)env->getForge()->allocate(sizeof(MM_MemoryManager), OMR::GC::AllocationCategory::FIXED, OMR_GET_CALLSITE());
+	MM_MemoryManager *memoryManager = (MM_MemoryManager *)env->getForge()->allocate(sizeof(MM_MemoryManager), OMR::GC::AllocationCategory::FIXED, OMR_GET_CALLSITE());
 
 	if (NULL != memoryManager) {
 		new (memoryManager) MM_MemoryManager(env);
@@ -47,24 +52,24 @@ MM_MemoryManager::newInstance(MM_EnvironmentBase* env)
 }
 
 void
-MM_MemoryManager::kill(MM_EnvironmentBase* env)
+MM_MemoryManager::kill(MM_EnvironmentBase *env)
 {
 	env->getForge()->free(this);
 }
 
 bool
-MM_MemoryManager::initialize(MM_EnvironmentBase* env)
+MM_MemoryManager::initialize(MM_EnvironmentBase *env)
 {
 	return true;
 }
 
 bool
-MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryHandle* handle, uintptr_t heapAlignment, uintptr_t size, uintptr_t tailPadding, void* preferredAddress, void* ceiling)
+MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase *env, MM_MemoryHandle *handle, uintptr_t heapAlignment, uintptr_t size, uintptr_t tailPadding, void *preferredAddress, void *ceiling)
 {
 	Assert_MM_true(NULL != handle);
-	MM_GCExtensionsBase* extensions = env->getExtensions();
+	MM_GCExtensionsBase *extensions = env->getExtensions();
 
-	MM_VirtualMemory* instance = NULL;
+	MM_VirtualMemory *instance = NULL;
 	uintptr_t mode = (OMRPORT_VMEM_MEMORY_MODE_READ | OMRPORT_VMEM_MEMORY_MODE_WRITE);
 	uintptr_t options = 0;
 	uint32_t memoryCategory = OMRMEM_CATEGORY_MM_RUNTIME_HEAP;
@@ -102,7 +107,7 @@ MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryH
 	 * huge pages). Nonetheless, this is a safe operation because in case page size equals system's huge page
 	 * the mode flag OMRPORT_VMEM_MEMORY_MODE_SHARE_FILE_OPEN will be ignored.
 	 */
-	if(extensions->isVLHGC() && extensions->isArrayletDoubleMapRequested) {
+	if (extensions->isVLHGC() && extensions->isArrayletDoubleMapRequested) {
 		mode |= OMRPORT_VMEM_MEMORY_MODE_SHARE_FILE_OPEN;
 	}
 #endif /* defined(OMR_GC_DOUBLE_MAP_ARRAYLETS) */
@@ -137,7 +142,7 @@ MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryH
 		if (!env->compressObjectReferences()) {
 			if (1 == extensions->fvtest_enableReadBarrierVerification) {
 				MM_VirtualMemory* instanceShadow = MM_VirtualMemory::newInstance(env, heapAlignment, allocateSize, pageSize, pageFlags,
-						tailPadding, preferredAddress, (void*)OMR_MIN(NON_SCALING_LOW_MEMORY_HEAP_CEILING,
+						tailPadding, preferredAddress, (void *)OMR_MIN(NON_SCALING_LOW_MEMORY_HEAP_CEILING,
 						(uintptr_t)ceiling), mode, options, memoryCategory);
 
 				extensions->shadowHeapBase = instanceShadow->getHeapBase();
@@ -164,11 +169,17 @@ MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryH
 			 * to avoid possible interference with requested heap location
 			 */
 			bool shouldHeapBeAllocatedFirst = (NULL != preferredAddress);
-			void* startAllocationAddress = preferredAddress;
+			void *startAllocationAddress = preferredAddress;
 
-			/* Set the commit size for the sub allocator. This needs to be completed before the call to omrmem_ensure_capacity32 */
+			/* Set the commit size for the suballocator. This needs to be completed before the call to omrmem_ensure_capacity32. */
 			omrport_control(OMRPORT_CTLDATA_ALLOCATE32_COMMIT_SIZE, extensions->suballocatorCommitSize);
 
+			/* Set the increment size for the suballocator. This needs to be completed before the call to omrmem_ensure_capacity32. */
+			omrport_control(OMRPORT_CTLDATA_ALLOCATE32_INCREMENT_SIZE, extensions->suballocatorIncrementSize);
+
+			/* Set if the suballocator should use ALLOC_QUICK. This needs to be completed before the call to omrmem_ensure_capacity32. */
+			omrport_control(OMRPORT_CTLDATA_ALLOCATE32_QUICK_ALLOC, extensions->suballocatorQuickAlloc ? 1 : 0);
+
 			if (!shouldHeapBeAllocatedFirst) {
 				if (OMRPORT_ENSURE_CAPACITY_FAILED == omrmem_ensure_capacity32(extensions->suballocatorInitialSize)) {
 					extensions->heapInitializationFailureReason = MM_GCExtensionsBase::HEAP_INITIALIZATION_FAILURE_REASON_CAN_NOT_ALLOCATE_LOW_MEMORY_RESERVE;
@@ -185,16 +196,15 @@ MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryH
 			/*
 			 * On ZOS an address space below 2G can not be taken for virtual memory
 			 */
-#define TWO_GB_ADDRESS ((void*)((uintptr_t)2 * 1024 * 1024 * 1024))
 			if (NULL == preferredAddress) {
 				startAllocationAddress = TWO_GB_ADDRESS;
 			}
 #endif /* defined(J9ZOS39064) */
 
-			void* requestedTopAddress = (void*)((uintptr_t)startAllocationAddress + allocateSize + tailPadding);
+			void *requestedTopAddress = (void *)((uintptr_t)startAllocationAddress + allocateSize + tailPadding);
 
 			if (extensions->isConcurrentScavengerHWSupported()) {
-				void * ceilingToRequest = ceiling;
+				void *ceilingToRequest = ceiling;
 				/* Requested top address might be higher then ceiling because of added chunk */
 				if ((requestedTopAddress > ceiling) && ((void *)((uintptr_t)requestedTopAddress - concurrentScavengerPageSize) <= ceiling)) {
 					/* ZOS 2_TO_64/2_TO_32 options would not allow memory request larger then 64G/32G so total requested size including tail padding should not exceed it */
@@ -219,33 +229,74 @@ MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryH
 				if (requestedTopAddress <= ceiling) {
 					bool allocationTopDown = true;
 					/* define the scan direction when reserving the GC heap in the range of (4G, 32G) */
-#if defined(S390) || defined(J9ZOS390)
-					/* s390 benefits from smaller shift values so allocate direction is bottom up */
+#if defined(J9ZOS390)
+					/*
+					 * zOS does not allow to choose direction of allocation and uses Bottom Up.
+					 * So, this flag is not necessary. However set it to match actual behavior.
+					 */
 					options |= OMRPORT_VMEM_ALLOC_DIR_BOTTOM_UP;
 					allocationTopDown = false;
-#else /* defined(S390) || defined(J9ZOS390) */
+#else /* defined(J9ZOS390) */
+					/*
+					 * Use Top Down for all platforms except zOS.
+					 */
 					options |= OMRPORT_VMEM_ALLOC_DIR_TOP_DOWN;
-#endif /* defined(S390) || defined(J9ZOS390) */
+#endif /* defined(J9ZOS390) */
 
 					if (allocationTopDown && extensions->shouldForceSpecifiedShiftingCompression) {
 						/* force to allocate heap top-down from correspondent to shift address */
-						void* maxAddress = (void *)(((uintptr_t)1 << 32) << extensions->forcedShiftingCompressionAmount);
+						void *maxAddress = (void *)(((uintptr_t)1 << 32) << extensions->forcedShiftingCompressionAmount);
 
-						instance = MM_VirtualMemory::newInstance(env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
+						instance = MM_VirtualMemory::newInstance(
+								env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
 								maxAddress, mode, options, memoryCategory);
 					} else {
-						if (requestedTopAddress < (void*)NON_SCALING_LOW_MEMORY_HEAP_CEILING) {
-							/*
-							 * Attempt to allocate heap below 4G
-							 */
-							instance = MM_VirtualMemory::newInstance(env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
-																	 (void*)OMR_MIN(NON_SCALING_LOW_MEMORY_HEAP_CEILING, (uintptr_t)ceiling), mode, options, memoryCategory);
+						void *adjustedCeiling = NULL;
+
+						/*
+						 * Attempt to allocate heap below 4G.
+						 */
+						if (requestedTopAddress < (void *)NON_SCALING_LOW_MEMORY_HEAP_CEILING) {
+							adjustedCeiling = (void *)OMR_MIN(NON_SCALING_LOW_MEMORY_HEAP_CEILING, (uintptr_t)ceiling);
+
+							instance = MM_VirtualMemory::newInstance(
+									env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
+									adjustedCeiling, mode, options, memoryCategory);
 						}
 
-						if ((NULL == instance) && (ceiling > (void*)NON_SCALING_LOW_MEMORY_HEAP_CEILING)) {
+						if ((NULL == instance) && (ceiling > (void *)NON_SCALING_LOW_MEMORY_HEAP_CEILING)) {
 
-#define THIRTY_TWO_GB_ADDRESS ((uintptr_t)32 * 1024 * 1024 * 1024)
-							if (requestedTopAddress <= (void *)THIRTY_TWO_GB_ADDRESS) {
+#if defined(LINUX) && defined(S390)
+							/*
+							 * Z platform executes smaller shift faster, so it is beneficial to have smaller one.
+							 * Attempting to allocate object heap below 8G or 16G to get Compressed
+							 * References shift 1 or 2 for zLinux only.
+							 */
+
+							/*
+							 * Attempt to allocate heap below 8G.
+							 */
+							if ((NULL == instance) && (requestedTopAddress <= EIGHT_GB_ADDRESS)) {
+								adjustedCeiling = (void *)OMR_MIN((uintptr_t)EIGHT_GB_ADDRESS, (uintptr_t)ceiling);
+
+								instance = MM_VirtualMemory::newInstance(
+										env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
+										adjustedCeiling, mode, options, memoryCategory);
+							}
+
+							/*
+							 * Attempt to allocate heap below 16G.
+							 */
+							if ((NULL == instance) && (requestedTopAddress <= SIXTEEN_GB_ADDRESS)) {
+								adjustedCeiling = (void *)OMR_MIN((uintptr_t)SIXTEEN_GB_ADDRESS, (uintptr_t)ceiling);
+
+								instance = MM_VirtualMemory::newInstance(
+										env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
+										adjustedCeiling, mode, options, memoryCategory);
+							}
+#endif /* defined(LINUX) && defined(S390) */
+
+							if ((NULL == instance) && (requestedTopAddress <= THIRTY_TWO_GB_ADDRESS)) {
 								/*
 								 * If requested object heap size is in range 28G-32G its allocation with 3-bit shift might compromise amount of low memory below 4G
 								 * To prevent this go straight to 4-bit shift if it possible.
@@ -254,25 +305,29 @@ MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryH
 								 *  - requested size is larger then 28G (32 minus 4)
 								 *  - allocation direction is top-down, otherwise it does not make sense
 								 */
-								bool skipAllocationBelow32G = (ceiling > (void*)THIRTY_TWO_GB_ADDRESS)
-									 && (requestedTopAddress > (void*)(THIRTY_TWO_GB_ADDRESS - NON_SCALING_LOW_MEMORY_HEAP_CEILING))
+								bool skipAllocationBelow32G = (ceiling > THIRTY_TWO_GB_ADDRESS)
+									 && (requestedTopAddress > (void *)((uintptr_t)THIRTY_TWO_GB_ADDRESS - NON_SCALING_LOW_MEMORY_HEAP_CEILING))
 									 && allocationTopDown;
 
 								if (!skipAllocationBelow32G) {
 									/*
-									 * Attempt to allocate heap below 32G
+									 * Attempt to allocate heap below 32G.
 									 */
-									instance = MM_VirtualMemory::newInstance(env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
-																			 (void*)OMR_MIN((uintptr_t)THIRTY_TWO_GB_ADDRESS, (uintptr_t)ceiling), mode, options, memoryCategory);
+									adjustedCeiling = (void *)OMR_MIN((uintptr_t)THIRTY_TWO_GB_ADDRESS, (uintptr_t)ceiling);
+
+									instance = MM_VirtualMemory::newInstance(
+											env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
+											adjustedCeiling, mode, options, memoryCategory);
 								}
 							}
 
 							/*
-							 * Attempt to allocate above 32G
+							 * Attempt to allocate heap at any location below ceiling.
 							 */
-							if ((NULL == instance) && (ceiling > (void *)THIRTY_TWO_GB_ADDRESS)) {
-								instance = MM_VirtualMemory::newInstance(env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
-																		 ceiling, mode, options, memoryCategory);
+							if (NULL == instance) {
+								instance = MM_VirtualMemory::newInstance(
+										env, heapAlignment, allocateSize, pageSize, pageFlags, tailPadding, preferredAddress,
+										ceiling, mode, options, memoryCategory);
 							}
 						}
 					}
@@ -310,7 +365,7 @@ MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryH
 		}
 	}
 
-	if((NULL != instance) && extensions->largePageFailOnError && (instance->getPageSize() != extensions->requestedPageSize)) {
+	if ((NULL != instance) && extensions->largePageFailOnError && (instance->getPageSize() != extensions->requestedPageSize)) {
 		extensions->heapInitializationFailureReason = MM_GCExtensionsBase::HEAP_INITIALIZATION_FAILURE_REASON_CAN_NOT_SATISFY_REQUESTED_PAGE_SIZE;
 		instance->kill(env);
 		instance = NULL;
@@ -404,22 +459,22 @@ MM_MemoryManager::createVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryH
 }
 
 bool
-MM_MemoryManager::createVirtualMemoryForMetadata(MM_EnvironmentBase* env, MM_MemoryHandle* handle, uintptr_t alignment, uintptr_t size)
+MM_MemoryManager::createVirtualMemoryForMetadata(MM_EnvironmentBase *env, MM_MemoryHandle *handle, uintptr_t alignment, uintptr_t size)
 {
 	Assert_MM_true(NULL != handle);
 	Assert_MM_true(NULL == handle->getVirtualMemory());
-	MM_GCExtensionsBase* extensions = env->getExtensions();
+	MM_GCExtensionsBase *extensions = env->getExtensions();
 
 	/*
 	 * Can we take already preallocated memory?
 	 */
 	if (NULL != _preAllocated.getVirtualMemory()) {
 		/* base might be not aligned */
-		void* base = (void*)MM_Math::roundToCeiling(alignment, (uintptr_t)_preAllocated.getMemoryBase());
-		void* top = (void*)((uintptr_t)base + MM_Math::roundToCeiling(alignment, size));
+		void *base = (void *)MM_Math::roundToCeiling(alignment, (uintptr_t)_preAllocated.getMemoryBase());
+		void *top = (void *)((uintptr_t)base + MM_Math::roundToCeiling(alignment, size));
 		if (top <= _preAllocated.getMemoryTop()) {
 			/* it is enough room in preallocated memory - take chunk of it */
-			MM_VirtualMemory* instance = _preAllocated.getVirtualMemory();
+			MM_VirtualMemory *instance = _preAllocated.getVirtualMemory();
 			/* Add one more consumer to Virtual Memory instance */
 			instance->incrementConsumerCount();
 
@@ -444,7 +499,7 @@ MM_MemoryManager::createVirtualMemoryForMetadata(MM_EnvironmentBase* env, MM_Mem
 	 */
 	if (NULL == handle->getVirtualMemory()) {
 		uint32_t memoryCategory = OMRMEM_CATEGORY_MM;
-		MM_VirtualMemory* instance = NULL;
+		MM_VirtualMemory *instance = NULL;
 		bool isOverAllocationRequested = false;
 
 		/* memory consumer might expect memory to be aligned so allocate a little bit more */
@@ -452,8 +507,8 @@ MM_MemoryManager::createVirtualMemoryForMetadata(MM_EnvironmentBase* env, MM_Mem
 
 		if (isMetadataAllocatedInVirtualMemory(env)) {
 			uintptr_t tailPadding = 0;
-			void* preferredAddress = NULL;
-			void* ceiling = NULL;
+			void *preferredAddress = NULL;
+			void *ceiling = NULL;
 			uintptr_t mode = (OMRPORT_VMEM_MEMORY_MODE_READ | OMRPORT_VMEM_MEMORY_MODE_WRITE);
 			uintptr_t options = 0;
 
@@ -520,13 +575,13 @@ MM_MemoryManager::createVirtualMemoryForMetadata(MM_EnvironmentBase* env, MM_Mem
 }
 
 void
-MM_MemoryManager::destroyVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_MemoryHandle* handle)
+MM_MemoryManager::destroyVirtualMemoryForHeap(MM_EnvironmentBase *env, MM_MemoryHandle *handle)
 {
 	destroyVirtualMemory(env, handle);
 
 #if defined(OMR_ENV_DATA64) && defined(OMR_GC_FULL_POINTERS)
-	MM_GCExtensionsBase* extensions = env->getExtensions();
-	MM_VirtualMemory* shadowMemory = extensions->shadowHeapHandle.getVirtualMemory();
+	MM_GCExtensionsBase *extensions = env->getExtensions();
+	MM_VirtualMemory *shadowMemory = extensions->shadowHeapHandle.getVirtualMemory();
 	if (NULL != shadowMemory) {
 		shadowMemory->kill(env);
 		extensions->shadowHeapHandle.setVirtualMemory(NULL);
@@ -537,10 +592,10 @@ MM_MemoryManager::destroyVirtualMemoryForHeap(MM_EnvironmentBase* env, MM_Memory
 }
 
 void
-MM_MemoryManager::destroyVirtualMemory(MM_EnvironmentBase* env, MM_MemoryHandle* handle)
+MM_MemoryManager::destroyVirtualMemory(MM_EnvironmentBase *env, MM_MemoryHandle *handle)
 {
 	Assert_MM_true(NULL != handle);
-	MM_VirtualMemory* memory = handle->getVirtualMemory();
+	MM_VirtualMemory *memory = handle->getVirtualMemory();
 	if (NULL != memory) {
 		Assert_MM_true(memory->getConsumerCount() > 0);
 		memory->decrementConsumerCount();
@@ -568,36 +623,36 @@ MM_MemoryManager::destroyVirtualMemory(MM_EnvironmentBase* env, MM_MemoryHandle*
 }
 
 int
-MM_MemoryManager::getHeapFileDescriptor(MM_MemoryHandle* handle)
+MM_MemoryManager::getHeapFileDescriptor(MM_MemoryHandle *handle)
 {
 	Assert_MM_true(NULL != handle);
-	MM_VirtualMemory* memory = handle->getVirtualMemory();
+	MM_VirtualMemory *memory = handle->getVirtualMemory();
 	Assert_MM_true(NULL != memory);
 	return memory->getHeapFileDescriptor();
 }
 
 #if defined(OMR_GC_DOUBLE_MAP_ARRAYLETS)
-void*
-MM_MemoryManager::doubleMapArraylet(MM_MemoryHandle* handle, MM_EnvironmentBase *env, void* arrayletLeaves[], UDATA arrayletLeafCount, UDATA arrayletLeafSize, UDATA byteAmount, struct J9PortVmemIdentifier *newIdentifier, UDATA pageSize)
+void *
+MM_MemoryManager::doubleMapArraylet(MM_MemoryHandle *handle, MM_EnvironmentBase *env, void *arrayletLeaves[], UDATA arrayletLeafCount, UDATA arrayletLeafSize, UDATA byteAmount, struct J9PortVmemIdentifier *newIdentifier, UDATA pageSize)
 {
 	Assert_MM_true(NULL != handle);
-	MM_VirtualMemory* memory = handle->getVirtualMemory();
+	MM_VirtualMemory *memory = handle->getVirtualMemory();
 	Assert_MM_true(NULL != memory);
 	return memory->doubleMapArraylet(env, arrayletLeaves, arrayletLeafCount, arrayletLeafSize, byteAmount, newIdentifier, pageSize);
 }
 
-void*
-MM_MemoryManager::doubleMapRegions(MM_MemoryHandle* handle, MM_EnvironmentBase *env, void* regions[], UDATA regionsCount, UDATA regionSize, UDATA byteAmount, struct J9PortVmemIdentifier *newIdentifier, UDATA pageSize, void *preferredAddress)
+void *
+MM_MemoryManager::doubleMapRegions(MM_MemoryHandle *handle, MM_EnvironmentBase *env, void *regions[], UDATA regionsCount, UDATA regionSize, UDATA byteAmount, struct J9PortVmemIdentifier *newIdentifier, UDATA pageSize, void *preferredAddress)
 {
 	Assert_MM_true(NULL != handle);
-	MM_VirtualMemory* memory = handle->getVirtualMemory();
+	MM_VirtualMemory *memory = handle->getVirtualMemory();
 	Assert_MM_true(NULL != memory);
 	return memory->doubleMapRegions(env, regions, regionsCount, regionSize, byteAmount, newIdentifier, pageSize, preferredAddress);
 }
 #endif /* defined(OMR_GC_DOUBLE_MAP_ARRAYLETS) */
 
 bool
-MM_MemoryManager::commitMemory(MM_MemoryHandle* handle, void* address, uintptr_t size)
+MM_MemoryManager::commitMemory(MM_MemoryHandle *handle, void *address, uintptr_t size)
 {
 	Assert_MM_true(NULL != handle);
 	MM_VirtualMemory* memory = handle->getVirtualMemory();
@@ -606,28 +661,28 @@ MM_MemoryManager::commitMemory(MM_MemoryHandle* handle, void* address, uintptr_t
 }
 
 bool
-MM_MemoryManager::decommitMemory(MM_MemoryHandle* handle, void* address, uintptr_t size, void* lowValidAddress, void* highValidAddress)
+MM_MemoryManager::decommitMemory(MM_MemoryHandle *handle, void *address, uintptr_t size, void *lowValidAddress, void *highValidAddress)
 {
 	Assert_MM_true(NULL != handle);
-	MM_VirtualMemory* memory = handle->getVirtualMemory();
+	MM_VirtualMemory *memory = handle->getVirtualMemory();
 	Assert_MM_true(NULL != memory);
 	return memory->decommitMemory(address, size, lowValidAddress, highValidAddress);
 }
 
 bool
-MM_MemoryManager::isLargePage(MM_EnvironmentBase* env, uintptr_t pageSize)
+MM_MemoryManager::isLargePage(MM_EnvironmentBase *env, uintptr_t pageSize)
 {
 	OMRPORT_ACCESS_FROM_OMRPORT(env->getPortLibrary());
-	uintptr_t* pageSizes = omrvmem_supported_page_sizes();
+	uintptr_t *pageSizes = omrvmem_supported_page_sizes();
 	return pageSize > pageSizes[0];
 }
 
 #if defined(OMR_GC_VLHGC) || defined(OMR_GC_MODRON_SCAVENGER)
 bool
-MM_MemoryManager::setNumaAffinity(const MM_MemoryHandle* handle, uintptr_t numaNode, void* address, uintptr_t byteAmount)
+MM_MemoryManager::setNumaAffinity(const MM_MemoryHandle *handle, uintptr_t numaNode, void *address, uintptr_t byteAmount)
 {
 	Assert_MM_true(NULL != handle);
-	MM_VirtualMemory* memory = handle->getVirtualMemory();
+	MM_VirtualMemory *memory = handle->getVirtualMemory();
 	Assert_MM_true(NULL != memory);
 	return memory->setNumaAffinity(numaNode, address, byteAmount);
 }
