@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #if !defined(MEMORYPOOL_HPP_)
@@ -418,12 +418,22 @@ public:
 		return _nonScannableBytes;
 	}
 
-#if defined(OMR_GC_IDLE_HEAP_MANAGER)
 	/**
 	 * @return bytes of free memory in the pool released/decommited back to OS
 	 */
 	virtual uintptr_t releaseFreeMemoryPages(MM_EnvironmentBase* env);
-#endif
+
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+	/**
+	 * Make adjustments to the Memory Pool to accommodate the restore configuration.
+	 * It is expected that subclasses will implement this method.
+	 *
+	 * @param[in] env the current environment.
+	 * @return boolean indicating whether the memory pool was successfully updated.
+	 */
+	virtual bool reinitializeForRestore(MM_EnvironmentBase *env) { return true; }
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+
 	/**
 	 * Create a MemoryPool object.
 	 */
