@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2019 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "omrcfg.h"
@@ -1296,13 +1296,12 @@ MM_MemoryPoolSplitAddressOrderedList::removeFreeEntriesWithinRange(MM_Environmen
 	return true;
 }
 
-#if defined(OMR_GC_IDLE_HEAP_MANAGER)
 uintptr_t
 MM_MemoryPoolSplitAddressOrderedList::releaseFreeMemoryPages(MM_EnvironmentBase* env)
 {
 	uintptr_t releasedMemory = 0;
 
-	for (uintptr_t i = 0; i < _heapFreeListCountExtended; i++) {
+	for (uintptr_t i = 0; i < _heapFreeListCount; i++) {
 		_heapFreeLists[i]._lock.acquire();
 		_heapFreeLists[i]._timesLocked += 1;
 		releasedMemory += releaseFreeEntryMemoryPages(env, _heapFreeLists[i]._freeList);
@@ -1311,4 +1310,44 @@ MM_MemoryPoolSplitAddressOrderedList::releaseFreeMemoryPages(MM_EnvironmentBase*
 
 	return releasedMemory;
 }
-#endif
+
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+bool
+MM_MemoryPoolSplitAddressOrderedList::reinitializeForRestore(MM_EnvironmentBase *env)
+{
+	if (_extensions->splitFreeListSplitAmount > _heapFreeListCount) {
+		/* Free lists must not exceed the pre-allocated _maximumHeapFreeListCount number of entries. */
+		_extensions->splitFreeListSplitAmount = OMR_MIN(
+				_extensions->splitFreeListSplitAmount, _maximumHeapFreeListCount);
+
+		/* Initialize values from _heapFreeListCount to the newly set splitFreeListSplitAmount. */
+		for (uintptr_t i = _heapFreeListCount; i < _extensions->splitFreeListSplitAmount; ++i) {
+			_currentThreadFreeList[i] = 0;
+
+			_heapFreeLists[i] = J9ModronFreeList();
+
+			if (!_heapFreeLists[i].initialize(env)) {
+				return false;
+			}
+
+			new (&_largeObjectAllocateStatsForFreeList[i]) MM_LargeObjectAllocateStats(env);
+
+			if (!_largeObjectAllocateStatsForFreeList[i].initialize(
+					env, (uint16_t)_extensions->largeObjectAllocationProfilingTopK,
+					_extensions->largeObjectAllocationProfilingThreshold,
+					_extensions->largeObjectAllocationProfilingVeryLargeObjectThreshold,
+					(float)_extensions->largeObjectAllocationProfilingSizeClassRatio / (float)100.0,
+					_extensions->heap->getMaximumMemorySize(), getTlhMaximumSize() + _minimumFreeEntrySize,
+					_extensions->tlhMinimumSize, 2)) {
+				return false;
+			}
+		}
+
+		_heapFreeListCount = _extensions->splitFreeListSplitAmount;
+	} else {
+		Assert_MM_true(_extensions->splitFreeListSplitAmount == _heapFreeListCount);
+	}
+
+	return true;
+}
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
