@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2018, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2018
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -329,10 +329,7 @@ MM_ConcurrentGCSATB::setupForConcurrent(MM_EnvironmentBase *env)
 {
 	GC_OMRVMInterface::flushCachesForGC(env);
 
-	/* Activate SATB Write Barrier
-	 * TODO: reserve/restoreGlobalFragmentIndex should have disable/enableWritterBarrier wrappers. Consider moving
-	 * much of the setupForConcurrent/completeConcurrentTracing in wrappers. */
-	_extensions->sATBBarrierRememberedSet->restoreGlobalFragmentIndex(env);
+	enableSATB(env);
 
 	_extensions->newThreadAllocationColor = GC_MARK;
 	_concurrentDelegate.setupClassScanning(env);
@@ -348,6 +345,36 @@ MM_ConcurrentGCSATB::setupForConcurrent(MM_EnvironmentBase *env)
 	_stats.switchExecutionMode(CONCURRENT_INIT_COMPLETE, CONCURRENT_TRACE_ONLY);
 }
 
+void
+MM_ConcurrentGCSATB::enableSATB(MM_EnvironmentBase *env) {
+	OMRPORT_ACCESS_FROM_ENVIRONMENT(env);
+
+	/* Activate SATB Write Barrier */
+	_extensions->sATBBarrierRememberedSet->restoreGlobalFragmentIndex(env);
+
+	TRIGGER_J9HOOK_MM_PRIVATE_CONCURRENT_SATB_TOGGLED(
+		_extensions->privateHookInterface,
+		env->getOmrVMThread(),
+		omrtime_hires_clock(),
+		J9HOOK_MM_PRIVATE_CONCURRENT_SATB_TOGGLED,
+		true);
+}
+
+void
+MM_ConcurrentGCSATB::disableSATB(MM_EnvironmentBase *env) {
+	OMRPORT_ACCESS_FROM_ENVIRONMENT(env);
+
+	/* Deactivate barrier */
+	_extensions->sATBBarrierRememberedSet->preserveGlobalFragmentIndex(env);
+
+	TRIGGER_J9HOOK_MM_PRIVATE_CONCURRENT_SATB_TOGGLED(
+		_extensions->privateHookInterface,
+		env->getOmrVMThread(),
+		omrtime_hires_clock(),
+		J9HOOK_MM_PRIVATE_CONCURRENT_SATB_TOGGLED,
+		false);
+}
+
 uintptr_t
 MM_ConcurrentGCSATB::doConcurrentTrace(MM_EnvironmentBase *env, MM_AllocateDescription *allocDescription, uintptr_t sizeToTrace, MM_MemorySubSpace *subspace, bool threadAtSafePoint)
 {
@@ -455,8 +482,7 @@ MM_ConcurrentGCSATB::completeConcurrentTracing(MM_EnvironmentBase *env, uintptr_
 			_extensions->sATBBarrierRememberedSet->flushFragments(env);
 	}
 
-	/* Deactivate barrier */
-	_extensions->sATBBarrierRememberedSet->preserveGlobalFragmentIndex(env);
+	disableSATB(env);
 
 	_extensions->newThreadAllocationColor = GC_UNMARK;
 
@@ -551,7 +577,7 @@ MM_ConcurrentGCSATB::reportConcurrentCollectionStart(MM_EnvironmentBase *env)
 /**
  * TLH is about to be cleared, we must premark it if SATB is active. This is
  * non-trival to do, we must know the start of the last object to mark the proper range of the TLH.
- * Arbitrary bits at the end of the TLH can’t be marked, this is problematic for sweep.
+ * Arbitrary bits at the end of the TLH can't be marked, this is problematic for sweep.
  *
  * To get around this issue we seal the TLH with a bogus/dummy obj at the
  * end of the TLH (done before call to this method). With this, we know the precise addr to batch mark to. We have
