@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #if !defined(SCAVENGER_HPP_)
@@ -249,9 +249,11 @@ public:
 	 * @param objectptr the object to scan
 	 * @param objectScannerState points to space for inline allocation of scanner
 	 * @param flags scanner flags
+	 * @param[in] reason See MM_ScavengeScanReason
+	 * @param[out]shouldRemember
 	 * @return the object scanner
 	 */
-	MMINLINE GC_ObjectScanner *getObjectScanner(MM_EnvironmentStandard *env, omrobjectptr_t objectptr, void *objectScannerState, uintptr_t flags);
+	MMINLINE GC_ObjectScanner *getObjectScanner(MM_EnvironmentStandard *env, omrobjectptr_t objectptr, void *objectScannerState, uintptr_t flags, MM_ScavengeScanReason reason, bool *shouldRemember);
 
 	uintptr_t calculateCopyScanCacheSizeForWaitingThreads(uintptr_t maxCacheSize, uintptr_t threadCount, uintptr_t waitingThreads);
 	uintptr_t calculateCopyScanCacheSizeForQueueLength(uintptr_t maxCacheSize, uintptr_t threadCount, uintptr_t scanCacheCount);
@@ -368,6 +370,7 @@ public:
 	 * @return True If Object should be remembered
 	 */
 	bool shouldRememberObject(MM_EnvironmentStandard *env, omrobjectptr_t objectPtr);
+	bool shouldRememberSlot(omrobjectptr_t *slotPtr);
 
 	/**
 	 * BackOutFixSlot implementation
@@ -382,7 +385,16 @@ public:
 #if defined(OMR_GC_CONCURRENT_SCAVENGER)
 	void fixupNurserySlots(MM_EnvironmentStandard *env);
 	void fixupObjectScan(MM_EnvironmentStandard *env, omrobjectptr_t objectPtr);
-	bool fixupSlot(GC_SlotObject *slotObject);
+	bool fixupSlot(omrobjectptr_t *slotPtr);
+	MMINLINE bool fixupSlot(GC_SlotObject *slotObject)
+	{
+		omrobjectptr_t objectPtr = slotObject->readReferenceFromSlot();
+		if (fixupSlot(&objectPtr)) {
+			slotObject->writeReferenceToSlot(objectPtr);
+			return true;
+		}
+		return false;
+	}
 	bool fixupSlotWithoutCompression(volatile omrobjectptr_t *slotPtr);
 	
 	void scavengeRememberedSetListIndirect(MM_EnvironmentStandard *env);
@@ -424,7 +436,25 @@ public:
 	MMINLINE uintptr_t scanCacheDistanceMetric(MM_CopyScanCacheStandard* cache, GC_SlotObject *scanSlot);
 	MMINLINE uintptr_t copyCacheDistanceMetric(MM_CopyScanCacheStandard* cache);
 
+
+	/**
+	 * Try to get a thread local scan cache. It's un-contended and preferred over the global scan list.
+	 *
+	 * @param env - current thread environment
+	 * @return scan cache or null
+	 */
+	MMINLINE MM_CopyScanCacheStandard *getNextScanCacheFromThread(MM_EnvironmentStandard *env);
+	/**
+	 * Try to get a scan cache from the global scan cache list. It's potentially blocking, if the list is empty.
+	 * Hence, not to be used by mutators, but only by dedicated GC threads.
+	 * It also serves as an implicit thread synchronization barrier, when all threads block on the empty list.
+	 * Hence, returning NULL means the work cycle is complete.
+	 *
+	 * @param env - current thread environment
+	 * @return scan cache or null
+	 */
 	MMINLINE MM_CopyScanCacheStandard *getNextScanCacheFromList(MM_EnvironmentStandard *env);
+
 	/**
 	 * Called at the end of a task to return empty caches to the global free pool
 	 */
@@ -523,9 +553,9 @@ public:
 
 	void clearRememberedSetLists(MM_EnvironmentStandard *env);
 
-	MMINLINE bool isRememberedSetInOverflowState() { return _extensions->isRememberedSetInOverflowState(); }
-	MMINLINE void setRememberedSetOverflowState() { _extensions->setRememberedSetOverflowState(); }
-	MMINLINE void clearRememberedSetOverflowState() { _extensions->clearRememberedSetOverflowState(); }
+	MMINLINE bool isRememberedSetInOverflowState() { return _extensions->isScavengerRememberedSetInOverflowState(); }
+	MMINLINE void setRememberedSetOverflowState() { _extensions->setScavengerRememberedSetOverflowState(); }
+	MMINLINE void clearRememberedSetOverflowState() { _extensions->clearScavengerRememberedSetOverflowState(); }
 
 	/* Auto-remember stack objects so JIT can omit generational barriers */
 	void rescanThreadSlots(MM_EnvironmentStandard *env);
@@ -551,31 +581,40 @@ public:
 
 	/**
 	 * Clear global (not thread local) stats for current phase/increment
-	 * @param firstIncrement true if first increment in a cycle
+	 * @param env[in] the current thread
+	 * @param firstIncrement[in] true if first increment in a cycle
 	 */
 	void clearIncrementGCStats(MM_EnvironmentBase *env, bool firstIncrement);
 	/**
-	 * Clear global (not thread local) cumulative cycle stats 
+	 * Clear global (not thread local) cumulative cycle stats
+	 * @param env[in] the current thread
 	 */
 	void clearCycleGCStats(MM_EnvironmentBase *env);
 	/**
 	 * Clear thread local stats for current phase/increment
-	 * @param firstIncrement true if first increment in a cycle
+	 * @param env[in] the current thread
+	 * @param firstIncrement[in] true if first increment in a cycle
 	 */
 	void clearThreadGCStats(MM_EnvironmentBase *env, bool firstIncrement);
 	/**
 	 * Merge thread local stats for current phase/increment in to global current increment stats
+	 * @param env[in] the current thread
 	 */	
 	void mergeThreadGCStats(MM_EnvironmentBase *env);
 	/**
 	 * Merge global current increment stats in to global cycle stats
-	 * @param firstIncrement true if last increment in a cycle
+	 * @param env[in] the current thread
+	 * @param firstIncrement[in] true if last increment in a cycle
 	 */		
 	void mergeIncrementGCStats(MM_EnvironmentBase *env, bool lastIncrement);
 	/**
 	 * Common merge logic used for both thread and increment level merges.
+	 * @param env[in] the current thread
+	 * @param finalGCStats[in/out] stats being added to
+	 * @param scavStats[in] stats being added from
 	 */
 	void mergeGCStatsBase(MM_EnvironmentBase *env, MM_ScavengerStats *finalGCStats, MM_ScavengerStats *scavStats);
+
 	bool canCalcGCStats(MM_EnvironmentStandard *env);
 	void calcGCStats(MM_EnvironmentStandard *env);
 
@@ -588,6 +627,13 @@ public:
 	 */
 	void calculateRecommendedWorkingThreads(MM_EnvironmentStandard *env);
 
+	/**
+	 * Sets the collector recommended thread count to UDATA_MAX (default value).
+	 *
+	 * @return void
+	 */
+	void resetRecommendedThreads() { _recommendedThreads = UDATA_MAX; };
+
 	void scavenge(MM_EnvironmentBase *env);
 	bool scavengeCompletedSuccessfully(MM_EnvironmentStandard *env);
 	virtual	void mainThreadGarbageCollect(MM_EnvironmentBase *env, MM_AllocateDescription *allocDescription, bool initMarkMap = false, bool rebuildMarkBits = false);
@@ -704,7 +750,7 @@ protected:
 	
 public:
 
-	static MM_Scavenger *newInstance(MM_EnvironmentStandard *env, MM_HeapRegionManager *regionManager);
+	static MM_Scavenger *newInstance(MM_EnvironmentStandard *env);
 	virtual void kill(MM_EnvironmentBase *env);
 
 	MM_ScavengerDelegate* getDelegate() { return &_delegate; }
@@ -725,6 +771,14 @@ public:
 	virtual uintptr_t mainThreadConcurrentCollect(MM_EnvironmentBase *env);
 	virtual void postConcurrentUpdateStatsAndReport(MM_EnvironmentBase *env, MM_ConcurrentPhaseStatsBase *stats, UDATA bytesConcurrentlyScanned);
 
+	/**
+	 * Merge RB increment stats to general increment stats.
+	 * RB occasionally updates separate copy stats fields in the increment stats,
+	 * but eventually those values need to be added to the general copy increment stats.
+	 * @param env[in] the current thread
+	 */
+	void mergeReadBarrierStats(MM_EnvironmentBase *env);
+
 	/* main thread specific methods */
 	bool scavengeIncremental(MM_EnvironmentBase *env);
 	bool scavengeInit(MM_EnvironmentBase *env);
@@ -752,7 +806,7 @@ public:
 	/**
 	 * complete (trigger end) of a Concurrent Scavenger Cycle
 	 */
-	void completeConcurrentCycle(MM_EnvironmentBase *envBase);
+	void completeConcurrentCycle(MM_EnvironmentBase *envBase, MM_MemorySubSpace *subSpace, MM_AllocateDescription *allocDescription, uint32_t gcCode);
 
 	/* worker thread */
 	void workThreadProcessRoots(MM_EnvironmentStandard *env);
@@ -787,6 +841,8 @@ public:
 	void reportConcurrentScavengeStart(MM_EnvironmentStandard *env);
 	void reportConcurrentScavengeEnd(MM_EnvironmentStandard *env);
 	
+	virtual void payAllocationTax(MM_EnvironmentBase *env, MM_MemorySubSpace *subspace, MM_MemorySubSpace *baseSubSpace, MM_AllocateDescription *allocDescription);
+
 #endif /* OMR_GC_CONCURRENT_SCAVENGER */
 
 	/**
@@ -906,7 +962,18 @@ public:
 	virtual bool canCollectorExpand(MM_EnvironmentBase *env, MM_MemorySubSpace *subSpace, uintptr_t expandSize);
 	virtual uintptr_t getCollectorExpandSize(MM_EnvironmentBase *env);
 
-	MM_Scavenger(MM_EnvironmentBase *env, MM_HeapRegionManager *regionManager) :
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+	/**
+	 * Reinitialize the CopyScanCacheLists (used by Scavenger) by splitting them
+	 * to optimize for the number of restore GC threads.
+	 *
+	 * @param[in] env the current environment.
+	 * @return boolean indicating whether the CopyScanCacheLists were successfully updated.
+	 */
+	virtual bool reinitializeForRestore(MM_EnvironmentBase *env);
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+
+	MM_Scavenger(MM_EnvironmentBase *env) :
 		MM_Collector()
 		, _cycleTimes()
 		, _delegate(env)
@@ -946,7 +1013,7 @@ public:
 		, _backOutDoneIndex(0)
 		, _heapBase(NULL)
 		, _heapTop(NULL)
-		, _regionManager(regionManager)
+		, _regionManager(_extensions->heapRegionManager)
 #if defined(OMR_GC_CONCURRENT_SCAVENGER)
 		, _mainGCThread(env)
 		, _concurrentPhase(concurrent_phase_idle)
