@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "AllocateDescription.hpp"
@@ -31,6 +31,9 @@
 #include "HeapRegionManager.hpp"
 #include "ObjectAllocationInterface.hpp"
 #include "ParallelDispatcher.hpp"
+#if defined(OMR_GC_SPARSE_HEAP_ALLOCATION)
+#include "SparseVirtualMemory.hpp"
+#endif /* defined(OMR_GC_SPARSE_HEAP_ALLOCATION) */
 #include "VerboseHandlerOutput.hpp"
 #include "VerboseManager.hpp"
 #include "VerboseWriterChain.hpp"
@@ -39,6 +42,9 @@
 #include "gcutils.h"
 
 static void verboseHandlerInitialized(J9HookInterface** hook, uintptr_t eventNum, void* eventData, void* userData);
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+static void verboseHandlerReinitialized(J9HookInterface** hook, uintptr_t eventNum, void* eventData, void* userData);
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
 static void verboseHandlerHeapResize(J9HookInterface** hook, uintptr_t eventNum, void* eventData, void* userData);
 
 MM_VerboseHandlerOutput *
@@ -102,6 +108,9 @@ MM_VerboseHandlerOutput::enableVerbose()
 {
 	/* Initialized */
 	(*_mmOmrHooks)->J9HookRegisterWithCallSite(_mmOmrHooks, J9HOOK_MM_OMR_INITIALIZED, verboseHandlerInitialized, OMR_GET_CALLSITE(), (void *)this);
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+	(*_mmOmrHooks)->J9HookRegisterWithCallSite(_mmOmrHooks, J9HOOK_MM_OMR_REINITIALIZED, verboseHandlerReinitialized, OMR_GET_CALLSITE(), (void *)this);
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
 	(*_mmPrivateHooks)->J9HookRegisterWithCallSite(_mmPrivateHooks, J9HOOK_MM_PRIVATE_HEAP_RESIZE, verboseHandlerHeapResize, OMR_GET_CALLSITE(), (void *)this);
 
 	return ;
@@ -112,6 +121,9 @@ MM_VerboseHandlerOutput::disableVerbose()
 {
 	/* Initialized */
 	(*_mmOmrHooks)->J9HookUnregister(_mmOmrHooks, J9HOOK_MM_OMR_INITIALIZED, verboseHandlerInitialized, NULL);
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+	(*_mmOmrHooks)->J9HookUnregister(_mmOmrHooks, J9HOOK_MM_OMR_REINITIALIZED, verboseHandlerReinitialized, NULL);
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
 	(*_mmPrivateHooks)->J9HookUnregister(_mmPrivateHooks, J9HOOK_MM_PRIVATE_HEAP_RESIZE, verboseHandlerHeapResize, NULL);
 
 	return ;
@@ -256,6 +268,27 @@ MM_VerboseHandlerOutput::getConcurrentTerminationReason(MM_ConcurrentPhaseStatsB
 	return reasonForTermination;
 }
 
+/**
+ * Return the reason for heap fixup as a string
+ * @param reason reason code
+ */
+const char *
+MM_VerboseHandlerOutput::getHeapFixupReasonString(uintptr_t reason)
+{
+	switch((FixUpReason)reason) {
+		case FIXUP_NONE:
+			return "no fixup";
+		case FIXUP_CLASS_UNLOADING:
+			return "class unloading";
+		case FIXUP_DEBUG_TOOLING:
+			return "debug tooling";
+		case FIXUP_AND_CLEAR_HEAP:
+			return "fixup and clear heap";
+		default:
+			return "unknown";
+	}
+}
+
 void
 MM_VerboseHandlerOutput::outputInitializedStanza(MM_EnvironmentBase *env, MM_VerboseBuffer *buffer)
 {
@@ -282,6 +315,9 @@ MM_VerboseHandlerOutput::outputInitializedStanza(MM_EnvironmentBase *env, MM_Ver
 #endif /* OMR_GC_CONCURRENT_SCAVENGER */
 
 	buffer->formatAndOutput(env, 1, "<attribute name=\"maxHeapSize\" value=\"0x%zx\" />", _extensions->memoryMax);
+	if (0 < _extensions->softMx) {
+		buffer->formatAndOutput(env, 1, "<attribute name=\"softMx\" value=\"0x%zx\" />", _extensions->softMx);
+	}
 	buffer->formatAndOutput(env, 1, "<attribute name=\"initialHeapSize\" value=\"0x%zx\" />", _extensions->initialMemorySize);
 
 #if defined(OMR_GC_COMPRESSED_POINTERS)
@@ -320,12 +356,24 @@ MM_VerboseHandlerOutput::outputInitializedStanza(MM_EnvironmentBase *env, MM_Ver
 #endif /* OMR_GC_MODRON_SCAVENGER */
 	buffer->formatAndOutput(env, 1, "<attribute name=\"splitFreeListSplitAmount\" value=\"%zu\" />", _extensions->splitFreeListSplitAmount);
 	buffer->formatAndOutput(env, 1, "<attribute name=\"numaNodes\" value=\"%zu\" />", _extensions->_numaManager.getAffinityLeaderCount());
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+	if (_extensions->reinitializationInProgress()) {
+		buffer->formatAndOutput(env, 1, "<attribute name=\"Restored Snapshot\" value=\"%s\" />", "true");
+	}
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
 
 	outputInitializedInnerStanza(env, buffer);
 
 	buffer->formatAndOutput(env, 1, "<system>");
 	buffer->formatAndOutput(env, 2, "<attribute name=\"physicalMemory\" value=\"%llu\" />", omrsysinfo_get_physical_memory());
+	buffer->formatAndOutput(env, 2, "<attribute name=\"addressablePhysicalMemory\" value=\"%llu\" />", omrsysinfo_get_addressable_physical_memory());
+	bool containerLimitSet = false;
+	if (OMR_CGROUP_SUBSYSTEM_MEMORY == omrsysinfo_cgroup_are_subsystems_enabled(OMR_CGROUP_SUBSYSTEM_MEMORY)) {
+		containerLimitSet = omrsysinfo_cgroup_is_memlimit_set();
+	}
+	buffer->formatAndOutput(env, 2, "<attribute name=\"container memory limit set\" value=\"%s\" />", containerLimitSet ? "true" : "false");
 	buffer->formatAndOutput(env, 2, "<attribute name=\"numCPUs\" value=\"%zu\" />", omrsysinfo_get_number_CPUs_by_type(OMRPORT_CPU_ONLINE));
+	buffer->formatAndOutput(env, 2, "<attribute name=\"numCPUs active\" value=\"%zu\" />", omrsysinfo_get_number_CPUs_by_type(OMRPORT_CPU_TARGET));
 	buffer->formatAndOutput(env, 2, "<attribute name=\"architecture\" value=\"%s\" />", omrsysinfo_get_CPU_architecture());
 	buffer->formatAndOutput(env, 2, "<attribute name=\"os\" value=\"%s\" />", omrsysinfo_get_OS_type());
 	buffer->formatAndOutput(env, 2, "<attribute name=\"osVersion\" value=\"%s\" />", omrsysinfo_get_OS_version());
@@ -345,21 +393,25 @@ MM_VerboseHandlerOutput::outputInitializedRegion(MM_EnvironmentBase *env, MM_Ver
 	const char *arrayletDoubleMappingStatus = _extensions->indexableObjectModel.isDoubleMappingEnabled() ? "enabled" : "disabled";
 	const char *arrayletDoubleMappingRequested = isArrayletDoubleMapRequested ? "true" : "false";
 #endif /* OMR_GC_DOUBLE_MAP_ARRAYLETS */
-	bool isVirtualLargeObjectHeapRequested = _extensions->isVirtualLargeObjectHeapRequested;
-	const char *virtualLargeObjectHeapStatus = _extensions->isVirtualLargeObjectHeapEnabled ? "enabled" : "disabled";
-	const char *virtualLargeObjectHeapRequested = isVirtualLargeObjectHeapRequested ? "true" : "false";
-
 	buffer->formatAndOutput(env, 1, "<region>");
 	buffer->formatAndOutput(env, 2, "<attribute name=\"regionSize\" value=\"%zu\" />", _extensions->getHeap()->getHeapRegionManager()->getRegionSize());
 	buffer->formatAndOutput(env, 2, "<attribute name=\"regionCount\" value=\"%zu\" />", _extensions->getHeap()->getHeapRegionManager()->getTableRegionCount());
-	buffer->formatAndOutput(env, 2, "<attribute name=\"arrayletLeafSize\" value=\"%zu\" />", omrVM->_arrayletLeafSize);
+	if (!_extensions->isVirtualLargeObjectHeapEnabled) {
+		buffer->formatAndOutput(env, 2, "<attribute name=\"arrayletLeafSize\" value=\"%zu\" />", omrVM->_arrayletLeafSize);
+	}
 	if (_extensions->isVLHGC()) {
 #if defined(OMR_GC_DOUBLE_MAP_ARRAYLETS)
-		buffer->formatAndOutput(env, 2, "<attribute name=\"arrayletDoubleMappingRequested\" value=\"%s\"/>", arrayletDoubleMappingRequested);
-		buffer->formatAndOutput(env, 2, "<attribute name=\"arrayletDoubleMapping\" value=\"%s\"/>", arrayletDoubleMappingStatus);
+		if (!_extensions->isVirtualLargeObjectHeapEnabled) {
+			buffer->formatAndOutput(env, 2, "<attribute name=\"arrayletDoubleMappingRequested\" value=\"%s\"/>", arrayletDoubleMappingRequested);
+			buffer->formatAndOutput(env, 2, "<attribute name=\"arrayletDoubleMapping\" value=\"%s\"/>", arrayletDoubleMappingStatus);
+		}
 #endif /* OMR_GC_DOUBLE_MAP_ARRAYLETS */
-		buffer->formatAndOutput(env, 2, "<attribute name=\"virtualLargeObjectHeapRequested\" value=\"%s\"/>", virtualLargeObjectHeapRequested);
-		buffer->formatAndOutput(env, 2, "<attribute name=\"virtualLargeObjectHeapStatus\" value=\"%s\"/>", virtualLargeObjectHeapStatus);
+#if defined(OMR_GC_SPARSE_HEAP_ALLOCATION)
+		if (_extensions->isVirtualLargeObjectHeapEnabled) {
+			MM_SparseVirtualMemory *largeObjectVirtualMemory = _extensions->largeObjectVirtualMemory;
+			buffer->formatAndOutput(env, 2, "<attribute name=\"virtualLargeObjectHeapSize\" value=\"0x%zx\"/>", (uintptr_t)largeObjectVirtualMemory->getHeapTop() - (uintptr_t)largeObjectVirtualMemory->getHeapBase());
+		}
+#endif /* defined(OMR_GC_SPARSE_HEAP_ALLOCATION) */
 	}
 	buffer->formatAndOutput(env, 1, "</region>");
 }
@@ -379,6 +431,23 @@ MM_VerboseHandlerOutput::handleInitialized(J9HookInterface** hook, uintptr_t eve
 	exitAtomicReportingBlock();
 }
 
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+void
+MM_VerboseHandlerOutput::handleReinitialized(J9HookInterface** hook, uintptr_t eventNum, void* eventData)
+{
+	MM_ReinitializedEvent* event = (MM_ReinitializedEvent*)eventData;
+	MM_VerboseWriterChain* writer = _manager->getWriterChain();
+	MM_EnvironmentBase* env = MM_EnvironmentBase::getEnvironment(event->currentThread);
+
+	_manager->setInitializedTime(event->timestamp);
+
+	enterAtomicReportingBlock();
+	outputInitializedStanza(env, writer->getBuffer());
+	writer->flush(env);
+	exitAtomicReportingBlock();
+}
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+
 void
 MM_VerboseHandlerOutput::handleCycleStart(J9HookInterface** hook, uintptr_t eventNum, void* eventData)
 {
@@ -472,6 +541,7 @@ MM_VerboseHandlerOutput::handleCycleEnd(J9HookInterface** hook, uintptr_t eventN
 
 	const char* cycleType = getCurrentCycleType(env);
 	char tagTemplate[200];
+	char fixupTagTemplate[100];
 	getTagTemplate(tagTemplate, sizeof(tagTemplate), _manager->getIdAndIncrement(), cycleType, env->_cycleState->_verboseContextID, omrtime_current_time_millis());
 
 	enterAtomicReportingBlock();
@@ -482,6 +552,13 @@ MM_VerboseHandlerOutput::handleCycleEnd(J9HookInterface** hook, uintptr_t eventN
 	} else {
 		writer->formatAndOutput(env, 0, "<cycle-end %s />", tagTemplate);
 	}
+
+	if ((OMR_GC_CYCLE_TYPE_GLOBAL == event->cycleType) && (FIXUP_NONE != event->fixHeapForWalkReason)) {
+		uint64_t fixupDuration = event->fixHeapForWalkTime;
+		getTagTemplate(fixupTagTemplate, sizeof(fixupTagTemplate), omrtime_current_time_millis());
+		writer->formatAndOutput(env, 0, "<heap-fixup timems=\"%llu.%03llu\" reason=\"%s\"  %s />", fixupDuration / 1000, fixupDuration % 1000, getHeapFixupReasonString(event->fixHeapForWalkReason), fixupTagTemplate);
+	}
+
 	writer->flush(env);
 	exitAtomicReportingBlock();
 }
@@ -536,8 +613,12 @@ MM_VerboseHandlerOutput::handleExclusiveStart(J9HookInterface** hook, uintptr_t
 	manager->setLastExclusiveAccessStartTime(currentTime);
 
 	OMR_VMThread* lastResponder = event->lastResponder;
-	char escapedLastResponderName[64];
-	getThreadName(escapedLastResponderName,sizeof(escapedLastResponderName),lastResponder);
+	char escapedLastResponderName[64] = "";
+
+	/* Last Responder thread can be passed NULL in the case of Safe Point Exclusive */
+	if (NULL != lastResponder) {
+		getThreadName(escapedLastResponderName, sizeof(escapedLastResponderName), lastResponder);
+	}
 
 	char tagTemplate[200];
 	getTagTemplate(tagTemplate, sizeof(tagTemplate), manager->getIdAndIncrement(), omrtime_current_time_millis());
@@ -546,8 +627,12 @@ MM_VerboseHandlerOutput::handleExclusiveStart(J9HookInterface** hook, uintptr_t
 		writer->formatAndOutput(env, 0, "<warning details=\"clock error detected, following timing may be inaccurate\" />");
 	}	
 	writer->formatAndOutput(env, 0, "<exclusive-start %s intervalms=\"%llu.%03.3llu\">", tagTemplate, deltaTime / 1000, deltaTime % 1000);
-	writer->formatAndOutput(env, 1, "<response-info timems=\"%llu.%03.3llu\" idlems=\"%llu.%03.3llu\" threads=\"%zu\" lastid=\"%p\" lastname=\"%s\" />",
-			exclusiveAccessTime / 1000, exclusiveAccessTime % 1000, meanIdleTime / 1000, meanIdleTime % 1000, event->haltedThreads, (NULL == lastResponder ? NULL : lastResponder->_language_vmthread), escapedLastResponderName);
+
+	writer->formatAndOutput(
+		env, 1, "<response-info timems=\"%llu.%03.3llu\" idlems=\"%llu.%03.3llu\" threads=\"%zu\" lastid=\"%p\" lastname=\"%s\" />",
+		exclusiveAccessTime / 1000, exclusiveAccessTime % 1000, meanIdleTime / 1000, meanIdleTime % 1000, event->haltedThreads,
+		(NULL == lastResponder ? NULL : lastResponder->_language_vmthread), escapedLastResponderName);
+
 	writer->formatAndOutput(env, 0, "</exclusive-start>");
 	writer->flush(env);
 	exitAtomicReportingBlock();
@@ -809,12 +894,18 @@ MM_VerboseHandlerOutput::printAllocationStats(MM_EnvironmentBase* env)
 	OMRPORT_ACCESS_FROM_OMRPORT(env->getPortLibrary());
 
 	enterAtomicReportingBlock();
-	writer->formatAndOutput(env, 0, "<allocation-stats totalBytes=\"%zu\" >", systemStats->bytesAllocated());
+	writer->formatAndOutput(env, 0, "<allocation-stats totalBytes=\"%zu\" discardedBytes=\"%zu\" >", systemStats->bytesAllocated(), systemStats->_tlhDiscardedBytes);
 
 	if (_extensions->isVLHGC()) {
 #if defined(OMR_GC_VLHGC)
-		writer->formatAndOutput(env, 1, "<allocated-bytes non-tlh=\"%zu\" tlh=\"%zu\" arrayletleaf=\"%zu\"/>",
+		if (_extensions->isVirtualLargeObjectHeapEnabled) {
+			writer->formatAndOutput(env, 1, "<allocated-bytes non-tlh=\"%zu\" tlh=\"%zu\" offheap=\"%zu\"/>",
 				systemStats->nontlhBytesAllocated(), systemStats->tlhBytesAllocated(), systemStats->_arrayletLeafAllocationBytes);
+
+		} else {
+			writer->formatAndOutput(env, 1, "<allocated-bytes non-tlh=\"%zu\" tlh=\"%zu\" arrayletleaf=\"%zu\"/>",
+				systemStats->nontlhBytesAllocated(), systemStats->tlhBytesAllocated(), systemStats->_arrayletLeafAllocationBytes);
+		}
 #endif /* OMR_GC_VLHGC */
 	} else if (_extensions->isStandardGC()) {
 #if defined(OMR_GC_MODRON_STANDARD)
@@ -856,6 +947,11 @@ MM_VerboseHandlerOutput::handleGCStart(J9HookInterface** hook, uintptr_t eventNu
 
 	enterAtomicReportingBlock();
 	writer->formatAndOutput(env, 0, "<gc-start %s>", tagTemplate);
+
+	if (stats->_cpuUtilStats._validData) {
+		writer->formatAndOutput(env, 1, "<cpu-util id=\"%zu\" total=\"%.2f\" process=\"%.2f\" />",
+				_manager->getIdAndIncrement(), stats->_cpuUtilStats._avgCpuUtil * 100, stats->_cpuUtilStats._avgProcUtil * 100);
+	}
 	outputMemoryInfo(env, _manager->getIndentLevel() + 1, stats);
 	writer->formatAndOutput(env, 0, "</gc-start>");
 	exitAtomicReportingBlock();
@@ -1094,6 +1190,14 @@ verboseHandlerInitialized(J9HookInterface** hook, uintptr_t eventNum, void* even
 	((MM_VerboseHandlerOutput*)userData)->handleInitialized(hook, eventNum, eventData);
 }
 
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+void
+verboseHandlerReinitialized(J9HookInterface** hook, uintptr_t eventNum, void* eventData, void* userData)
+{
+	((MM_VerboseHandlerOutput*)userData)->handleReinitialized(hook, eventNum, eventData);
+}
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+
 void
 verboseHandlerHeapResize(J9HookInterface** hook, uintptr_t eventNum, void* eventData, void* userData)
 {
