@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef OMRCOMP_H
@@ -38,7 +38,7 @@
 
 #include <stdint.h>
 #include "omrcfg.h"
-#if defined(__cplusplus) && (defined(__xlC__) || defined(J9ZOS390))
+#if defined(__cplusplus) && (defined(__xlC__) || defined(J9ZOS390) || defined(__open_xl__))
 #include <builtins.h>
 #endif
 
@@ -108,6 +108,16 @@ EXE_EXTENSION_CHAR: the executable has a delimiter that we want to stop at as pa
 
 */
 
+/* GCC will respect the always_inline attribute even in unoptimized builds,
+ * where one might expect inlining to be inhibited regardless, so for VMINLINE,
+ * apply the attribute only in optimized builds.
+ */
+#if defined(__GNUC__) && defined(__OPTIMIZE__)
+#define OMR_GNUC_ALWAYS_INLINE_WHEN_OPTIMIZED __attribute__((__always_inline__))
+#else /* defined(__GNUC__) && defined(__OPTIMIZE__) */
+#define OMR_GNUC_ALWAYS_INLINE_WHEN_OPTIMIZED
+#endif /* defined(__GNUC__) && defined(__OPTIMIZE__) */
+
 /* Linux ANSI compiler (gcc) and OSX (clang). */
 #if defined(LINUX) || defined (OSX)
 
@@ -150,12 +160,7 @@ typedef double SYS_FLOAT;
 #endif
 
 #if defined(__GNUC__)
-#define VMINLINE_ALWAYS inline __attribute((always_inline))
-/* If -O0 is in effect, define VMINLINE to be empty */
-#if !defined(__OPTIMIZE__)
-#define VMINLINE
-#endif
-
+#define VMINLINE_ALWAYS inline OMR_GNUC_ALWAYS_INLINE_WHEN_OPTIMIZED
 #elif defined(__xlC__)
 /*
  * xlC11 C++ compiler reportedly supports attributes before function names, but we've only tested xlC12.
@@ -279,11 +284,7 @@ typedef double 					SYS_FLOAT;
 	THREAD_PRIORITY_TIME_CRITICAL			/*11 */}
 
 #if defined(__GNUC__)
-#define VMINLINE_ALWAYS inline __attribute((always_inline))
-/* If -O0 is in effect, define VMINLINE to be empty */
-#if !defined(__OPTIMIZE__)
-#define VMINLINE
-#endif
+#define VMINLINE_ALWAYS inline OMR_GNUC_ALWAYS_INLINE_WHEN_OPTIMIZED
 #define HAS_BUILTIN_EXPECT
 #else /* __GNUC__ */
 /* Only for use on static functions */
@@ -548,13 +549,8 @@ typedef struct U_128 {
 #if !defined(VMINLINE_ALWAYS)
 #define VMINLINE_ALWAYS
 #endif
-#if !defined(VMINLINE)
+
 #define VMINLINE VMINLINE_ALWAYS
-#endif
-#if defined(DEBUG)
-#undef VMINLINE
-#define VMINLINE
-#endif
 
 /* DDR cannot parse __builtin_expect */
 #if defined(TYPESTUBS_H)
@@ -603,7 +599,7 @@ typedef struct U_128 {
 
 #if defined(_MSC_VER) && (1900 > _MSC_VER) /* MSVC versions prior to Visual Studio 2015 (14.0) */
 #define OMR_ALIGNOF(x) __alignof(x)
-#elif defined(__IBMC__) || defined(__IBMCPP__) /* XL C/C++ versions prior to xlclang/xlclang++ */
+#elif defined(__IBMC__) || defined(__IBMCPP__) || defined(__open_xl__) /* XL C/C++ versions prior to xlclang/xlclang++ */
 #define OMR_ALIGNOF(x) __alignof__(x)
 #else /* All other compilers that support C11 and C++11 */
 #define OMR_ALIGNOF(x) alignof(x)
@@ -617,4 +613,10 @@ typedef struct U_128 {
 #endif /* defined(__GNUC__) && (__GNUC__ < 5) */
 #endif /* defined(__cplusplus) && (__cplusplus >= 201103L) */
 
+#if defined(J9ZOS390) && !defined(OMR_EBCDIC)
+#define OMR_ISDIGIT(x) __isdigit_a(x)
+#else /* defined(J9ZOS390) && !defined(OMR_EBCDIC) */
+#define OMR_ISDIGIT(x) isdigit(x)
+#endif /* defined(J9ZOS390) && !defined(OMR_EBCDIC) */
+
 #endif /* OMRCOMP_H */
