@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #if !defined(OMRGCCONSTS_H_)
@@ -121,10 +121,23 @@ typedef enum MM_MarkingSchemeScanReason {
 	SCAN_REASON_OVERFLOWED_OBJECT = 4, /**< Indicates the object being scanned was in an overflowed region */
 } MM_MarkingSchemeScanReason;
 
+typedef enum MM_ScavengeScanReason {
+	SCAN_REASON_SCAVENGE = 10,
+	SCAN_REASON_FIXUP  = 11,
+	SCAN_REASON_BACKOUT = 12,
+	SCAN_REASON_SHOULDREMEMBER = 13,
+} MM_ScavengeScanReason;
+
 #define OMR_GC_CYCLE_TYPE_DEFAULT     0
 #define OMR_GC_CYCLE_TYPE_GLOBAL      1
 #define OMR_GC_CYCLE_TYPE_SCAVENGE    2
 #define OMR_GC_CYCLE_TYPE_EPSILON	 6
+/**
+ * This bit represents the state to abort a cycle. It is only used in the cycleType of CycleEnd event.
+ * MM_CycleState->_type has uintptr_t type. This code still needs to be supported on 32-bit platforms,
+ * which will require the bit to fit in a 32 bit word.
+ */
+#define OMR_GC_CYCLE_TYPE_STATE_UNSUCCESSFUL 0x80000000
 
 /* Core allocation flags defined for OMR are < OMR_GC_ALLOCATE_OBJECT_LANGUAGE_DEFINED_BASE */
 #define OMR_GC_ALLOCATE_OBJECT_NON_INSTRUMENTABLE 0x0
@@ -287,7 +300,8 @@ typedef enum {
 	COMPACT_CONTRACT = 11,
 	COMPACT_AGGRESSIVE= 12,
 	COMPACT_PAGE = 13,
-	COMPACT_MICRO_FRAG = 14
+	COMPACT_MICRO_FRAG = 14,
+	COMPACT_RASDUMP = 15
 } CompactReason;
 
 typedef enum {
@@ -356,7 +370,8 @@ typedef enum {
 	ABORT_COLLECTION_REMEMBERSET_OVERFLOW,
 	ABORT_COLLECTION_SCAVENGE_REMEMBEREDSET_OVERFLOW,
 	ABORT_COLLECTION_PREPARE_HEAP_FOR_WALK,
-	ABORT_COLLECTION_IDLE_GC
+	ABORT_COLLECTION_IDLE_GC,
+	ABORT_COLLECTION_PREPARE_FOR_CHECKPOINT_GC
 } CollectionAbortReason;
 
 /**
@@ -456,7 +471,8 @@ typedef enum {
 typedef enum {
 	FIXUP_NONE = 0,
 	FIXUP_CLASS_UNLOADING,
-	FIXUP_DEBUG_TOOLING
+	FIXUP_DEBUG_TOOLING,
+	FIXUP_AND_CLEAR_HEAP
 } FixUpReason;
 
 typedef enum {
@@ -539,15 +555,28 @@ typedef enum {
 #define PREFERRED_HEAP_BASE 0x0
 #endif
 
-#define SUBALLOCATOR_INITIAL_SIZE (200*1024*1024)
-#define SUBALLOCATOR_COMMIT_SIZE (50*1024*1024)
+#define SUBALLOCATOR_INITIAL_SIZE (200 * 1024 * 1024)
+#define SUBALLOCATOR_COMMIT_SIZE (50 * 1024 * 1024)
 #if defined(AIXPPC)
 /* virtual memory is assigned in segment of 256M, so grab the entire segment */
-#define SUBALLOCATOR_ALIGNMENT (256*1024*1024)
+#define SUBALLOCATOR_ALIGNMENT (256 * 1024 * 1024)
 #else /* defined(AIXPPC) */
-#define SUBALLOCATOR_ALIGNMENT (8*1024*1024)
+#define SUBALLOCATOR_ALIGNMENT (8 * 1024 * 1024)
 #endif /* defined(AIXPPC) */
 
+/* VMDESIGN 1761 The size of a suballocation heap.
+ * See VMDESIGN 1761 for the rationale behind the selection of this size.
+ * An 8MB heap is used to provide more capacity in the cases where an application loads a greater
+ * number of classes than typical.
+ * For testing purposes, this value is mirrored in the port library test. omrmemTest.cpp should be
+ * updated if this value is changed.
+ */
+#if defined(AIXPPC) && defined(OMR_GC_COMPRESSED_POINTERS)
+#define SUBALLOCATOR_INCREMENT_SIZE (256 * 1024 * 1024)
+#else /* defined(AIXPPC) && defined(OMR_GC_COMPRESSED_POINTERS) */
+#define SUBALLOCATOR_INCREMENT_SIZE (8 * 1024 * 1024)
+#endif /* defined(AIXPPC) && defined(OMR_GC_COMPRESSED_POINTERS) */
+
 #define MAXIMUM_HEAP_SIZE_RECOMMENDED_FOR_COMPRESSEDREFS            ((U_64)57 * 1024 * 1024 * 1024)
 #define MAXIMUM_HEAP_SIZE_RECOMMENDED_FOR_3BIT_SHIFT_COMPRESSEDREFS ((U_64)25 * 1024 * 1024 * 1024)
 
@@ -557,9 +586,9 @@ typedef enum {
 #define METRONOME_DEFAULT_TIME_WINDOW_MICRO 60000
 #endif /* OMR_GC_REALTIME */
 
-#if defined(J9ZTPF)
+#if defined(OMRZTPF)
 #define ZTPF_MEMORY_RESERVE_RATIO .8
-#endif /* defined(J9ZTPF) */
+#endif /* defined(OMRZTPF) */
 
 /**
  * Bit geometry within header flags byte.
@@ -573,10 +602,6 @@ typedef enum {
 #define OMR_OBJECT_METADATA_AGE_MASK		0xF0
 #define OMR_OBJECT_METADATA_AGE_SHIFT		4
 
-#if (0 != (OMR_OBJECT_METADATA_FLAGS_MASK & COPY_PROGRESS_INFO_MASK))
-#error "mask overlap: OMR_OBJECT_METADATA_FLAGS_MASK, COPY_PROGRESS_INFO_MASK"
-#endif
-
 /**
  * Remembered bit states overlay tenured header age flags. These are normalized to low-order byte, as above.
  */
