@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef thread_api_h
@@ -33,17 +33,22 @@
 *
 * This file contains public function prototypes and
 * type definitions for the THREAD module.
-*
 */
 
 #include "omrcfg.h"
 #include "omrthread.h"
 #include "omrcomp.h"
 
+#if defined(OSX) && defined(AARCH64)
+#include <pthread.h> /* for pthread_jit_write_protect_np() */
+#endif /* defined(OSX) && defined(AARCH64) */
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+typedef void (*omrthread_monitor_wait_callback)(void *userData);
+
 #define J9THREAD_RWMUTEX_OK		 	 0
 #define J9THREAD_RWMUTEX_FAIL	 	 1
 #define J9THREAD_RWMUTEX_WOULDBLOCK -1
@@ -59,6 +64,11 @@ typedef struct omrthread_process_time_t {
 	int64_t _userTime;
 } omrthread_process_time_t;
 
+typedef struct omrthread_thread_time_t {
+	int64_t userTime;
+	int64_t sysTime;
+} omrthread_thread_time_t;
+
 typedef struct omrthread_state_t {
 	uintptr_t flags;
 	omrthread_monitor_t blocker;
@@ -166,7 +176,7 @@ uintptr_t
 omrthread_get_stack_range(omrthread_t thread, void **stackStart, void **stackEnd);
 
 
-#if (defined(OMR_THR_JLM))
+#if defined(OMR_THR_JLM)
 /**
 * @brief
 * @param monitor
@@ -174,7 +184,7 @@ omrthread_get_stack_range(omrthread_t thread, void **stackStart, void **stackEnd
 */
 J9ThreadMonitorTracing *
 omrthread_monitor_get_tracing(omrthread_monitor_t monitor);
-#endif /* OMR_THR_JLM */
+#endif /* defined(OMR_THR_JLM) */
 
 
 /**
@@ -282,16 +292,14 @@ omrthread_rwmutex_try_enter_write(omrthread_rwmutex_t mutex);
 intptr_t
 omrthread_rwmutex_exit_read(omrthread_rwmutex_t mutex);
 
-
 /**
-* @brief
-* @param mutex
-* @return intptr_t
-*/
+ * @brief
+ * @param mutex
+ * @return intptr_t
+ */
 intptr_t
 omrthread_rwmutex_exit_write(omrthread_rwmutex_t mutex);
 
-
 /**
 * @brief
 * @param handle
@@ -524,7 +532,7 @@ omrthread_monitor_t
 omrthread_global_monitor(void);
 
 /**
- * Return monitor that thread is currently waiting to acquire. 
+ * Return monitor that thread is currently waiting to acquire.
  * If thread is not waiting on a monitor return null.
  * @param thread
  * @return omrthread_monitor_t
@@ -551,8 +559,8 @@ intptr_t
 omrthread_monitor_get_acquired_count(omrthread_monitor_t monitor);
 
 /**
- * Return thread that currently owns the monitor. Null 
- * will be returned if monitor is not owned.
+ * Return thread that currently owns the monitor.
+ * Null will be returned if monitor is not owned.
  * @param monitor
  * @return omrthread_t
  */
@@ -577,7 +585,7 @@ uintptr_t
 omrthread_interrupted(omrthread_t thread);
 
 
-#if (defined(OMR_THR_JLM))
+#if defined(OMR_THR_JLM)
 /**
 * @brief
 * @param void
@@ -585,10 +593,9 @@ omrthread_interrupted(omrthread_t thread);
 */
 J9ThreadMonitorTracing *
 omrthread_jlm_get_gc_lock_tracing(void);
-#endif /* OMR_THR_JLM */
+#endif /* defined(OMR_THR_JLM) */
 
-
-#if (defined(OMR_THR_JLM))
+#if defined(OMR_THR_JLM)
 /**
 * @brief
 * @param flags
@@ -596,7 +603,7 @@ omrthread_jlm_get_gc_lock_tracing(void);
 */
 intptr_t
 omrthread_jlm_init(uintptr_t flags);
-#endif /* OMR_THR_JLM */
+#endif /* defined(OMR_THR_JLM) */
 
 #if defined(OMR_THR_ADAPTIVE_SPIN)
 /**
@@ -866,6 +873,16 @@ omrthread_monitor_unlock(omrthread_t self, omrthread_monitor_t monitor);
 intptr_t
 omrthread_monitor_wait(omrthread_monitor_t monitor);
 
+/**
+* @brief
+* @param monitor
+* @param callbackFunction
+* @param userData
+* @return intptr_t
+*/
+intptr_t
+omrthread_monitor_wait_with_callback(omrthread_monitor_t monitor, omrthread_monitor_wait_callback callbackFunction, void *userData);
+
 /**
 * @brief
 * @param monitor
@@ -897,6 +914,18 @@ omrthread_monitor_wait_interruptable(omrthread_monitor_t monitor, int64_t millis
 intptr_t
 omrthread_monitor_wait_timed(omrthread_monitor_t monitor, int64_t millis, intptr_t nanos);
 
+/**
+* @brief
+* @param monitor
+* @param millis
+* @param nanos
+* @param callbackFunction
+* @param userData
+* @return intptr_t
+*/
+intptr_t
+omrthread_monitor_wait_timed_with_callback(omrthread_monitor_t monitor, int64_t millis, intptr_t nanos, omrthread_monitor_wait_callback callbackFunction, void *userData);
+
 /**
 * @brief
 * @param nanos
@@ -1240,6 +1269,15 @@ omrthread_get_jvm_cpu_usage_info(J9ThreadsCpuUsage *cpuUsage);
 void
 omrthread_get_jvm_cpu_usage_info_error_recovery(void);
 
+/**
+ * Gets the system and user CPU time of the current thread.
+ *
+ * @param[out] threadTime the pointer to the thread time structure
+ * @return 0 on success or -1 on failure
+ */
+intptr_t
+omrthread_get_thread_times(omrthread_thread_time_t *threadTime);
+
 /* ---------------- omrthreadattr.c ---------------- */
 
 /**
@@ -1428,7 +1466,20 @@ omrthread_monitor_unpin(omrthread_monitor_t monitor, omrthread_t self);
 struct J9ThreadLibrary;
 
 #ifdef __cplusplus
-}
+} /* extern "C" */
 #endif
 
+/*
+ * @brief Control write access to memory containing dynamically generated code.
+ *
+ * Currently only meaningful on macOS using Aarch64 processors.
+ */
+#if defined(OSX) && defined(AARCH64)
+#define omrthread_jit_write_protect_disable() pthread_jit_write_protect_np(0)
+#define omrthread_jit_write_protect_enable()  pthread_jit_write_protect_np(1)
+#else /* defined(OSX) && defined(AARCH64) */
+#define omrthread_jit_write_protect_disable() /* nop */
+#define omrthread_jit_write_protect_enable()  /* nop */
+#endif /* defined(OSX) && defined(AARCH64) */
+
 #endif /* thread_api_h */
