@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -185,6 +185,12 @@ omrvmem_startup(struct OMRPortLibrary *portLibrary)
 				Trc_PRT_vmem_omrvmem_startup_pagessize_exceeds_physical_memory(psizes[counter], physicalMemory);
 				break;
 			}
+#if !defined(OMR_ENV_DATA64)
+			if (psizes[counter] > UINT_MAX) {
+				/* Don't advertise sizes larger than the 32-bit address space. */
+				break;
+			}
+#endif /* !defined(OMR_ENV_DATA64) */
 			PPG_vmem_pageSize[counter] = psizes[counter];
 			PPG_vmem_pageFlags[counter] = OMRPORT_VMEM_PAGE_FLAG_NOT_USED;
 		}
@@ -316,7 +322,7 @@ omrvmem_free_memory(struct OMRPortLibrary *portLibrary, void *userAddress, uintp
 	uintptr_t allocator = identifier->allocator;
 	uintptr_t size = identifier->size;
 	OMRMemCategory *category = identifier->category;
-	
+
 	update_vmemIdentifier(identifier, NULL, NULL, 0, 0, 0, 0, 0, NULL);
 
 	if (OMRPORT_VMEM_RESERVE_USED_J9MEM_ALLOCATE_MEMORY == allocator) {
@@ -399,8 +405,11 @@ omrvmem_reserve_memory_ex(struct OMRPortLibrary *portLibrary, struct J9PortVmemI
 		/* Invalid input */
 		update_vmemIdentifier(identifier, NULL, NULL, 0, 0, 0, 0, 0, NULL);
 		Trc_PRT_vmem_omrvmem_reserve_memory_invalid_input();
-	} else if (PPG_vmem_pageSize[0] == params->pageSize) {
-		/* Handle default page size differently, don't use shmget */
+	} else if ((PPG_vmem_pageSize[0] == params->pageSize) || (params->byteAmount < params->pageSize)) {
+		/* Use default page size if byteAmount is smaller than the requested pageSize.
+		 * In this case, handle the allocation differently, and avoid using shmget,
+		 * which is not optimal for small allocations.
+		 */
 		uintptr_t alignment = OMR_MAX(params->pageSize, params->alignmentInBytes);
 		uintptr_t minimumGranule = OMR_MIN(params->pageSize, params->alignmentInBytes);
 
@@ -756,13 +765,18 @@ default_pageSize_reserve_memory(struct OMRPortLibrary *portLibrary, void *addres
 	int protMask;
 	void *result = NULL;
 
-	if(mode & OMRPORT_VMEM_MEMORY_MODE_SHARE_FILE_OPEN) {
-		portLibrary->error_set_last_error(portLibrary,  errno, OMRPORT_ERROR_VMEM_NOT_SUPPORTED);
+	if (OMR_ARE_ANY_BITS_SET(mode, OMRPORT_VMEM_MEMORY_MODE_SHARE_FILE_OPEN | OMRPORT_VMEM_MEMORY_MODE_SHARE_TMP_FILE_OPEN)) {
+		portLibrary->error_set_last_error(portLibrary, errno, OMRPORT_ERROR_VMEM_NOT_SUPPORTED);
 		return result;
 	}
 
 	Trc_PRT_vmem_default_reserve_entry(address, byteAmount);
 
+#if !defined(OMR_ENV_DATA64)
+	/* On 32 bit systems, starting a new 256MB segment with mmap during allocating can
+	 * significantly impact the address space due to its limited size. This is not a
+	 * concern on 64-bit systems, which have a much larger virtual address space.
+	 */
 	if (0 != (OMRPORT_VMEM_MEMORY_MODE_EXECUTE & mode)) {
 		/* Allocate code memory  */
 		result = portLibrary->mem_allocate_memory(portLibrary, byteAmount, OMR_GET_CALLSITE(), category->categoryCode);
@@ -780,6 +794,7 @@ default_pageSize_reserve_memory(struct OMRPortLibrary *portLibrary, void *addres
 		Trc_PRT_vmem_default_reserve_exit(result, address, byteAmount);
 		return result;
 	}
+#endif /* !defined(OMR_ENV_DATA64) */
 
 #if defined(MAP_ANONYMOUS)
 	flags |= MAP_ANONYMOUS;
@@ -948,32 +963,30 @@ omrvmem_find_valid_page_size(struct OMRPortLibrary *portLibrary, uintptr_t mode,
 	Assert_PRT_true_wrapper(OMRPORT_VMEM_PAGE_FLAG_NOT_USED == validPageFlags);
 
 	if (0 != validPageSize) {
-		/* For executable pages search through the list of supported page sizes only if
-		 * - request is for 64K pages, or
-		 * - request is for 16M pages, and
-		 * - 64 bit system OR (32 bit system AND CodeCacheConsolidation is enabled)
+		/* For 32-bit executable pages, search through the list of page sizes only if:
+		 * - The request is for 64K pages, or
+		 * - The request is for 16M pages and CodeCacheConsolidation is enabled.
+		 *
+		 * For 64-bit, accept any request supported by the OS.
 		 */
-#if defined(OMR_ENV_DATA64)
-		if ((OMRPORT_VMEM_MEMORY_MODE_EXECUTE != (OMRPORT_VMEM_MEMORY_MODE_EXECUTE & mode))
-			|| (SIXTY_FOUR_K == validPageSize)
-			|| (SIXTEEN_M == validPageSize)
-#else
+#if !defined(OMR_ENV_DATA64)
 		BOOLEAN codeCacheConsolidationEnabled = FALSE;
 
 		/* Check if the TR_ppcCodeCacheConsolidationEnabled env variable is set.
 		 * TR_ppcCodeCacheConsolidationEnabled is a manually set env variable used to indicate
 		 * that Code Cache Consolidation is enabled in the JIT.
 		 */
-		if (OMRPORT_VMEM_MEMORY_MODE_EXECUTE == (OMRPORT_VMEM_MEMORY_MODE_EXECUTE & mode)) {
-			if (portLibrary->sysinfo_get_env(portLibrary, "TR_ppcCodeCacheConsolidationEnabled", NULL, 0) != -1) {
-				codeCacheConsolidationEnabled = TRUE;
-			}
-		}
-		if ((OMRPORT_VMEM_MEMORY_MODE_EXECUTE != (OMRPORT_VMEM_MEMORY_MODE_EXECUTE & mode))
-			|| ((TRUE == codeCacheConsolidationEnabled) && (SIXTEEN_M == validPageSize))
-			|| (SIXTY_FOUR_K == validPageSize)
-#endif /* defined(OMR_ENV_DATA64) */
+		if (OMR_ARE_ANY_BITS_SET(mode, OMRPORT_VMEM_MEMORY_MODE_EXECUTE)
+			&& (-1 != portLibrary->sysinfo_get_env(portLibrary, "TR_ppcCodeCacheConsolidationEnabled", NULL, 0))
 		) {
+			codeCacheConsolidationEnabled = TRUE;
+		}
+
+		if (OMR_ARE_NO_BITS_SET(mode, OMRPORT_VMEM_MEMORY_MODE_EXECUTE)
+			|| (codeCacheConsolidationEnabled && (SIXTEEN_M == validPageSize))
+			|| (SIXTY_FOUR_K == validPageSize))
+#endif /* !defined(OMR_ENV_DATA64) */
+		{
 			uintptr_t pageIndex = 0;
 			uintptr_t *supportedPageSizes = portLibrary->vmem_supported_page_sizes(portLibrary);
 			uintptr_t *supportedPageFlags = portLibrary->vmem_supported_page_flags(portLibrary);
