@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2015, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2015
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -85,6 +85,7 @@ static OMRPortLibrary MainPortLibraryTable = {
 	omrsysinfo_get_load_average, /* sysinfo_get_load_average */
 	omrsysinfo_get_CPU_utilization, /* sysinfo_get_CPU_utilization */
 	omrsysinfo_get_CPU_load, /* sysinfo_get_CPU_load */
+	omrsysinfo_get_processes, /* sysinfo_get_processes */
 	omrsysinfo_limit_iterator_init, /* sysinfo_limit_iterator_next */
 	omrsysinfo_limit_iterator_hasNext, /* sysinfo_limit_iterator_hasNext */
 	omrsysinfo_limit_iterator_next, /* sysinfo_limit_iterator_next */
@@ -139,6 +140,7 @@ static OMRPortLibrary MainPortLibraryTable = {
 	omrsl_close_shared_library, /* sl_close_shared_library */
 	omrsl_open_shared_library, /* sl_open_shared_library */
 	omrsl_lookup_name, /* sl_lookup_name */
+	omrsl_get_libraries, /* sl_get_libraries */
 	omrtty_startup, /* tty_startup */
 	omrtty_shutdown, /* tty_shutdown */
 	omrtty_printf, /* tty_printf */
@@ -335,6 +337,8 @@ static OMRPortLibrary MainPortLibraryTable = {
 	omrsysinfo_cgroup_subsystem_iterator_metricKey, /* sysinfo_cgroup_subsystem_iterator_metricKey */
 	omrsysinfo_cgroup_subsystem_iterator_next, /* sysinfo_cgroup_subsystem_iterator_next */
 	omrsysinfo_cgroup_subsystem_iterator_destroy, /* sysinfo_cgroup_subsystem_iterator_destroy */
+	omrsysinfo_get_process_start_time, /* sysinfo_get_process_start_time */
+	omrsysinfo_get_number_context_switches, /* sysinfo_get_number_context_switches */
 	omrport_init_library, /* port_init_library */
 	omrport_startup_library, /* port_startup_library */
 	omrport_create_library, /* port_create_library */
@@ -347,6 +351,7 @@ static OMRPortLibrary MainPortLibraryTable = {
 	omrintrospect_threads_nextDo, /* introspect_threads_nextDo */
 	omrintrospect_backtrace_thread, /* introspect_backtrace_thread */
 	omrintrospect_backtrace_symbols, /* introspect_backtrace_symbols */
+	omrintrospect_backtrace_symbols_ex, /* introspect_backtrace_symbols_ex */
 	omrsyslog_query, /* syslog_query */
 	omrsyslog_set, /* syslog_set */
 	omrmem_walk_categories, /* mem_walk_categories */
