@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -165,7 +165,8 @@ omrsysinfo_get_processor_feature_name(struct OMRPortLibrary *portLibrary, uint32
 
 /**
  * Generate the corresponding string literals for the provided OMRProcessorDesc. The buffer will be zero 
- * initialized and overwritten with the processor feature output string.
+ * initialized and overwritten with the processor feature output string. The string will not contain "null",
+ * which corresponds to unknown feature names.
  *
  * @param[in] portLibrary The port library.
  * @param[in] desc The struct that contains the list of processor features to be converted to string.
@@ -465,7 +466,6 @@ omrsysinfo_shutdown(struct OMRPortLibrary *portLibrary)
 int32_t
 omrsysinfo_startup(struct OMRPortLibrary *portLibrary)
 {
-	PPG_isRunningInContainer = FALSE;
 	return 0;
 }
 
@@ -625,8 +625,9 @@ omrsysinfo_get_load_average(struct OMRPortLibrary *portLibrary, struct J9PortSys
 
 /**
  * Obtain the cumulative CPU utilization of all CPUs on the system.
- * The cpuTime and timestamp values have no absolute significance: they should be used only to compute
- * differences from previous values.
+ * The cpuTime, timestamp, userTime, systemTime and idleTime values have no absolute significance.
+ * They should be used only to compute differences from previous values.
+ * userTime, systemTime and idleTime are set to -1 if they are unavailable.
  * On an N-processor  system, cpuTimeStats.cpuTime may increase up to N times faster than real time.
  *
  * @param[in] OMRPortLibrary portLibrary The port library.
@@ -652,10 +653,10 @@ omrsysinfo_get_CPU_utilization(struct OMRPortLibrary *portLibrary, struct J9Sysi
  * @param[out] cpuLoad the cumulative CPU utilization of all CPUs on the system
  *
  * @return
- * 	- 0 on success
- *		- \arg OMRPORT_ERROR_OPFAILED on the first two invocations of this API
- *		- \arg OMRPORT_ERROR_OPFAILED if less than 10ns have passed since the second call to this API
- *		- negative portable error code on other failures
+ *  \arg 0 on success
+ *  \arg OMRPORT_ERROR_INSUFFICIENT_DATA if only one data point has been recorded
+ *  \arg OMRPORT_ERROR_OPFAILED if less than 10 ms have passed since the last call to this API
+ *  \arg negative portable error code on other failures
  */
 intptr_t
 omrsysinfo_get_CPU_load(struct OMRPortLibrary *portLibrary, double *cpuLoad)
@@ -1095,7 +1096,7 @@ omrsysinfo_get_cgroup_subsystem_list(struct OMRPortLibrary *portLibrary)
 BOOLEAN
 omrsysinfo_is_running_in_container(struct OMRPortLibrary *portLibrary)
 {
-	return PPG_isRunningInContainer;
+	return FALSE;
 }
 
 /**
@@ -1179,3 +1180,41 @@ omrsysinfo_cgroup_subsystem_iterator_destroy(struct OMRPortLibrary *portLibrary,
 {
 	return;
 }
+
+/**
+ * Get the process start time in ns precision epoch time.
+ * @param[in] portLibrary The port library
+ * @param[in] pid The process ID
+ * @param[in/out] processStartTimeInNanoseconds The pointer to uint64_t that stores the process start time in ns precision epoch time
+ * @return 0 on success, error code on failure
+ */
+int32_t
+omrsysinfo_get_process_start_time(struct OMRPortLibrary *portLibrary, uintptr_t pid, uint64_t *processStartTimeInNanoseconds)
+{
+	return OMRPORT_ERROR_NOT_SUPPORTED_ON_THIS_PLATFORM;
+}
+
+/**
+ * Get the number of context switches of the OS.
+ * @param[in] portLibrary The port library
+ * @param[out] numSwitches Pointer to the number of context switches
+ * @return 0 on success, error code on failure
+ */
+int32_t
+omrsysinfo_get_number_context_switches(struct OMRPortLibrary *portLibrary, uint64_t *numSwitches)
+{
+	return OMRPORT_ERROR_NOT_SUPPORTED_ON_THIS_PLATFORM;
+}
+
+/**
+ * Get the process ID and commandline for each process.
+ * @param[in] portLibrary The port library.
+ * @param[in] callback The function to be invoked for each process with the process ID and command info.
+ * @param[in] userData Data passed to the callback.
+ * @return 0 on success, or the first non-zero value returned by the callback.
+ */
+uintptr_t
+omrsysinfo_get_processes(struct OMRPortLibrary *portLibrary, OMRProcessInfoCallback callback, void *userData)
+{
+	return OMRPORT_ERROR_NOT_SUPPORTED_ON_THIS_PLATFORM;
+}
