@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2016, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2016
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -117,7 +117,11 @@ upcallHandler(int signal, siginfo_t *siginfo, void *context)
 	char *data = "A";
 
 	stateForBacktrace->portLibrary->introspect_backtrace_thread(stateForBacktrace->portLibrary, stateForBacktrace->current_thread, stateForBacktrace->heap, NULL);
-	stateForBacktrace->portLibrary->introspect_backtrace_symbols(stateForBacktrace->portLibrary, stateForBacktrace->current_thread, stateForBacktrace->heap);
+
+	if (OMR_ARE_NO_BITS_SET(stateForBacktrace->options, OMR_INTROSPECT_NO_SYMBOLS)) {
+		stateForBacktrace->portLibrary->introspect_backtrace_symbols_ex(stateForBacktrace->portLibrary, stateForBacktrace->current_thread, stateForBacktrace->heap, 0);
+	}
+
 	write(pipeFileDescriptor[1], data, 1);
 }
 
@@ -289,9 +293,11 @@ setupNativeThread(J9ThreadWalkState *state, thread_context *sigContext)
 				CLEAR_ERROR(state);
 			}
 
-			if ((NULL != state->current_thread->callstack) && (NULL == state->current_thread->callstack->symbol)) {
+			if (OMR_ARE_ANY_BITS_SET(state->options, OMR_INTROSPECT_NO_SYMBOLS)) {
+				/* The caller asked us not to resolve symbols, so we don't expect to have a symbol for the top frame. */
+			} else if ((NULL != state->current_thread->callstack) && (NULL == state->current_thread->callstack->symbol)) {
 				SPECULATE_ERROR(state, FAULT_DURING_BACKTRACE, 3);
-				state->portLibrary->introspect_backtrace_symbols(state->portLibrary, state->current_thread, state->heap);
+				state->portLibrary->introspect_backtrace_symbols_ex(state->portLibrary, state->current_thread, state->heap, 0);
 				CLEAR_ERROR(state);
 			}
 		}
