@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2016, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2016
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -215,7 +215,7 @@ omrvmem_commit_memory(struct OMRPortLibrary *portLibrary, void *address, uintptr
 				rc = address;
 			} else {
 				Trc_PRT_vmem_omrvmem_commit_memory_mprotect_failure(errno);
-				portLibrary->error_set_last_error(portLibrary,  errno, OMRPORT_ERROR_VMEM_OPFAILED);
+				portLibrary->error_set_last_error(portLibrary, errno, OMRPORT_ERROR_VMEM_OPFAILED);
 			}
 		} else if (PPG_vmem_pageSize[2] == identifier->pageSize || PPG_vmem_pageFlags[1] == identifier->pageFlags) {
 			rc = address;
@@ -446,7 +446,12 @@ reserveMemory(struct OMRPortLibrary *portLibrary, void *address, uintptr_t byteA
 	int protectionFlags = PROT_NONE;
 	BOOLEAN useBackingSharedFile = FALSE;
 
-	if(mode & OMRPORT_VMEM_MEMORY_MODE_SHARE_FILE_OPEN) {
+	if (OMR_ARE_ANY_BITS_SET(mode, OMRPORT_VMEM_MEMORY_MODE_SHARE_TMP_FILE_OPEN)) {
+		portLibrary->error_set_last_error(portLibrary, errno, OMRPORT_ERROR_VMEM_NOT_SUPPORTED);
+		return result;
+	}
+
+	if (OMR_ARE_ANY_BITS_SET(mode, OMRPORT_VMEM_MEMORY_MODE_SHARE_FILE_OPEN)) {
 		flags = MAP_SHARED;
 		useBackingSharedFile = TRUE;
 	}
@@ -478,9 +483,9 @@ reserveMemory(struct OMRPortLibrary *portLibrary, void *address, uintptr_t byteA
 		char omrtime_current_time_str[FILE_NAME_SIZE];
 		char pthread_self_str[FILE_NAME_SIZE];
 		char filename[FILE_NAME_SIZE];
-		sprintf(omrtime_current_time_str, "%zu", (uintptr_t)omrtime_current_time_millis(portLibrary) % (uintptr_t)TRUCANTE_TIME_CONST);
-		sprintf(pthread_self_str, "%zu", (uintptr_t)pthread_self() % (uintptr_t)TRUNCATE_THREAD_CONST);
-		sprintf(filename, "omrvmem_%.12s_%.9s", omrtime_current_time_str, pthread_self_str);
+		snprintf(omrtime_current_time_str, sizeof(omrtime_current_time_str), "%zu", (uintptr_t)omrtime_current_time_millis(portLibrary) % (uintptr_t)TRUCANTE_TIME_CONST);
+		snprintf(pthread_self_str, sizeof(pthread_self_str), "%zu", (uintptr_t)pthread_self() % (uintptr_t)TRUNCATE_THREAD_CONST);
+		snprintf(filename, sizeof(filename), "omrvmem_%.12s_%.9s", omrtime_current_time_str, pthread_self_str);
 
 		fd = shm_open(filename, O_RDWR | O_CREAT, 0600);
 		shm_unlink(filename);
@@ -498,7 +503,7 @@ reserveMemory(struct OMRPortLibrary *portLibrary, void *address, uintptr_t byteA
 	}
 
 #if defined(AARCH64)
-	if (category->categoryCode == OMRMEM_CATEGORY_JIT_CODE_CACHE) {
+	if (OMR_ARE_ANY_BITS_SET(mode, OMRPORT_VMEM_MEMORY_MODE_EXECUTE)) {
 		flags |= MAP_JIT;
 		protectionFlags &= ~PROT_EXEC;
 	}
@@ -683,7 +688,7 @@ getMemoryInRange(struct OMRPortLibrary *portLibrary, struct J9PortVmemIdentifier
 			currentAddress = (void *)alignmentInBytes;
 		}
 	}
-	
+
 #if defined(VM_FLAGS_SUPERPAGE_SIZE_2MB)
 	if (PPG_vmem_pageSize[2] == pageSize) {
 		/* Don't bother scanning the entire address space if the request can't be satisfied. */
@@ -694,7 +699,7 @@ getMemoryInRange(struct OMRPortLibrary *portLibrary, struct J9PortVmemIdentifier
 		omrvmem_free_memory(portLibrary, tmpPointer, byteAmount, identifier);
 	}
 #endif /* defined(VM_FLAGS_SUPERPAGE_SIZE_2MB) */
-	
+
 	/* Try all addresses within range */
 	while ((startAddress <= currentAddress) && (endAddress >= currentAddress)) {
 		memoryPointer = reserveMemory(portLibrary, currentAddress, byteAmount, identifier, mode, pageSize, pageFlags, category);
@@ -935,7 +940,7 @@ omrvmem_release_double_mapped_region(struct OMRPortLibrary *portLibrary, void *a
  *  @param uintptr_t                    pageSize            [in] Constant describing page size
  *  @param OMRMemCategory               *category           [in] Memory allocation category
  *  @param void                         *preferredAddress    [in] Prefered address of contiguous region to be double mapped
- * 
+ *
  * @return pointer to contiguous region to which regions were double mapped into, NULL is returned if unsuccessful
  */
 
@@ -975,7 +980,7 @@ omrvmem_create_double_mapped_region(struct OMRPortLibrary *portLibrary, void* re
 	if ((contiguousMap == MAP_FAILED) || (NULL == contiguousMap)) {
 		Trc_PRT_double_map_regions_Create_Failure();
 		contiguousMap = NULL;
-		portLibrary->error_set_last_error(portLibrary,  errno, OMRPORT_ERROR_VMEM_DOUBLE_MAP_ADRESS_RESERVE_FAILED);
+		portLibrary->error_set_last_error(portLibrary, errno, OMRPORT_ERROR_VMEM_DOUBLE_MAP_ADRESS_RESERVE_FAILED);
 	} else {
 		Trc_PRT_double_map_regions_Create_Success(contiguousMap, preferredAddress);
 		shouldUnmapAddr = TRUE;
@@ -1029,7 +1034,7 @@ omrvmem_create_double_mapped_region(struct OMRPortLibrary *portLibrary, void* re
 			if (address == MAP_FAILED) {
 				Trc_PRT_double_map_regions_Create_Failure2();
 				successfulContiguousMap = FALSE;
-				portLibrary->error_set_last_error(portLibrary,  errno, OMRPORT_ERROR_VMEM_DOUBLE_MAP_FAILED);
+				portLibrary->error_set_last_error(portLibrary, errno, OMRPORT_ERROR_VMEM_DOUBLE_MAP_FAILED);
 #if defined(OMRVMEM_DEBUG)
 				printf("***************************** errno: %d\n", errno);
 				printf("Failed to mmap address[%zu] at mmapContiguous()\n", i);
@@ -1039,7 +1044,7 @@ omrvmem_create_double_mapped_region(struct OMRPortLibrary *portLibrary, void* re
 			} else if (nextAddress != address) {
 				Trc_PRT_double_map_regions_Create_Failure3(nextAddress, address);
 				successfulContiguousMap = FALSE;
-				portLibrary->error_set_last_error(portLibrary,  errno, OMRPORT_ERROR_VMEM_DOUBLE_MAP_FAILED);
+				portLibrary->error_set_last_error(portLibrary, errno, OMRPORT_ERROR_VMEM_DOUBLE_MAP_FAILED);
 #if defined(OMRVMEM_DEBUG)
 				printf("Map failed to provide the correct address. nextAddress %p != %p\n", nextAddress, address);
 				fflush(stdout);
@@ -1067,6 +1072,6 @@ omrvmem_create_double_mapped_region(struct OMRPortLibrary *portLibrary, void* re
 void *
 omrvmem_get_contiguous_region_memory(struct OMRPortLibrary *portLibrary, void* addresses[], uintptr_t addressesCount, uintptr_t addressSize, uintptr_t byteAmount, struct J9PortVmemIdentifier *oldIdentifier, struct J9PortVmemIdentifier *newIdentifier, uintptr_t mode, uintptr_t pageSize, OMRMemCategory *category)
 {
-	portLibrary->error_set_last_error(portLibrary,  errno, OMRPORT_ERROR_VMEM_NOT_SUPPORTED);
+	portLibrary->error_set_last_error(portLibrary, errno, OMRPORT_ERROR_VMEM_NOT_SUPPORTED);
 	return NULL;
 }
