@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2019 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -51,11 +51,15 @@
 #include "ut_omrport.h"
 #include <sys/stat.h>
 
-#ifdef J9ZOS390
+#if defined(J9ZOS390)
 /* The following undef is to address CMVC 95221 */
 #undef fwrite
 #undef fread
-#endif
+
+#if !defined(OMR_EBCDIC)
+#include "atoe.h"
+#endif /* !defined(OMR_EBCDIC) */
+#endif /* defined(J9ZOS390) */
 
 #if defined(LINUX) || defined(OSX)
 typedef struct statfs PlatformStatfs;
