@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2015, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2015
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 /**
  * @file
@@ -30,6 +30,8 @@
 #include <stdio.h>
 #include <windows.h>
 #include <WinSDKVer.h>
+#include <tlhelp32.h>
+#include <psapi.h>
 /* Undefine the winsockapi because winsock2 defines it.  Removes warnings. */
 #if defined(_WINSOCKAPI_) && !defined(_WINSOCK2API_)
 #undef _WINSOCKAPI_
@@ -46,6 +48,10 @@
 #include "ut_omrport.h"
 #include "omrsysinfo_helpers.h"
 
+#define OMRPORT_SYSINFO_WINDOWS_TICK 10000000ULL
+#define OMRPORT_SYSINFO_SEC_TO_UNIX_EPOCH 11644473600ULL
+#define OMRPORT_SYSINFO_NS100_PER_SEC 10000000ULL
+
 static int32_t copyEnvToBuffer(struct OMRPortLibrary *portLibrary, void *args);
 
 typedef struct CopyEnvToBufferArgs {
@@ -186,7 +192,9 @@ omrsysinfo_get_processor_feature_string(struct OMRPortLibrary *portLibrary, OMRP
 				uint32_t feature = (uint32_t)(i * numberOfBits + j);
 				const char * featureName = omrsysinfo_get_processor_feature_name(portLibrary, feature);
 				size_t featureLength = strlen(featureName);
-
+				if ((4 == featureLength) && (0 == strncmp("null", featureName, 4))) {
+					continue;
+				}
 				if (start == FALSE) {
 					strncat(buffer, " ", length - bufferLength - 1);
 					bufferLength += 1;
@@ -538,11 +546,13 @@ WIN32_WINNT version constants :
 								}
 							}
 							else if (isServerMajorVersion10) {
-								/* Windows Server 2022 build number cutoff is 20348, Server 2019 cutoff is 17763
-								 * and Server 2016 cutoff is 14393. These versions are currently ones with major
-								 * version 10.
+								/* Windows Server 2025 build number cutoff is 26100, Server 2022 build number cutoff is 20348,
+								 * Server 2019 cutoff is 17763 and Server 2016 cutoff is 14393. These versions are currently
+								 * ones with major version 10.
 								 */
-								if (currentBuildNumber >= 20348) {
+								if (currentBuildNumber >= 26100) {
+									PPG_si_osType = "Windows Server 2025";
+								} else if (currentBuildNumber >= 20348) {
 									PPG_si_osType = "Windows Server 2022";
 								} else if (currentBuildNumber >= 17763) {
 									PPG_si_osType = "Windows Server 2019";
@@ -901,6 +911,7 @@ omrsysinfo_get_memory_info(struct OMRPortLibrary *portLibrary, struct J9MemoryIn
 	memInfo->availSwap = OMRPORT_MEMINFO_NOT_AVAILABLE;
 	memInfo->cached = OMRPORT_MEMINFO_NOT_AVAILABLE;
 	memInfo->buffered = OMRPORT_MEMINFO_NOT_AVAILABLE;
+	memInfo->swappiness = OMRPORT_MEMINFO_NOT_AVAILABLE;
 
 	aMemStatusEx.dwLength = sizeof(aMemStatusEx);
 	rc = GlobalMemoryStatusEx(&aMemStatusEx);
@@ -1083,7 +1094,6 @@ omrsysinfo_shutdown(struct OMRPortLibrary *portLibrary)
 int32_t
 omrsysinfo_startup(struct OMRPortLibrary *portLibrary)
 {
-	PPG_isRunningInContainer = FALSE;
 	/* Obtain and cache executable name; if this fails, executable name remains NULL, but
 	 * shouldn't cause failure to startup port library.  Failure will be noticed only
 	 * when the omrsysinfo_get_executable_name() actually gets invoked.
@@ -1222,14 +1232,23 @@ omrsysinfo_get_load_average(struct OMRPortLibrary *portLibrary, struct J9PortSys
 intptr_t
 omrsysinfo_get_CPU_utilization(struct OMRPortLibrary *portLibrary, struct J9SysinfoCPUTime *cpuTime)
 {
-
 	FILETIME lpIdleTime;
+	ULARGE_INTEGER lpIdleTimeU64;
 	FILETIME lpKernelTime;
+	ULARGE_INTEGER lpKernelTimeU64;
 	FILETIME lpUserTime;
+	ULARGE_INTEGER lpUserTimeU64;
 	uint64_t idleTime = 0;
 	uint64_t kernelActiveTime = 0;
 	uint64_t userTime = 0;
 
+	memset(&lpIdleTime, 0, sizeof(lpIdleTime));
+	memset(&lpIdleTimeU64, 0, sizeof(lpIdleTimeU64));
+	memset(&lpKernelTime, 0, sizeof(lpKernelTime));
+	memset(&lpKernelTimeU64, 0, sizeof(lpKernelTimeU64));
+	memset(&lpUserTime, 0, sizeof(lpUserTime));
+	memset(&lpUserTimeU64, 0, sizeof(lpUserTimeU64));
+
 	/*
 	 * GetSystemTimes returns the sum of CPU times across all CPUs
 	 */
@@ -1238,13 +1257,27 @@ omrsysinfo_get_CPU_utilization(struct OMRPortLibrary *portLibrary, struct J9Sysi
 		Trc_PRT_sysinfo_get_CPU_utilization_GetSystemTimesFailed(GetLastError());
 		return OMRPORT_ERROR_SYSINFO_GET_STATS_FAILED;
 	}
-	idleTime = (lpIdleTime.dwLowDateTime + (((uint64_t) lpIdleTime.dwHighDateTime) << 32)) * 100; /* FILETIME resolution is 100 ns */
-	kernelActiveTime = (lpKernelTime.dwLowDateTime + (((uint64_t) lpKernelTime.dwHighDateTime) << 32)) * 100
-					   - idleTime; /* kernel time includes idle time */
+
+	lpIdleTimeU64.LowPart = lpIdleTime.dwLowDateTime;
+	lpIdleTimeU64.HighPart = lpIdleTime.dwHighDateTime;
+
+	lpKernelTimeU64.LowPart = lpKernelTime.dwLowDateTime;
+	lpKernelTimeU64.HighPart = lpKernelTime.dwHighDateTime;
+
+	lpUserTimeU64.LowPart = lpUserTime.dwLowDateTime;
+	lpUserTimeU64.HighPart = lpUserTime.dwHighDateTime;
+
+	idleTime = lpIdleTimeU64.QuadPart * 100; /* FILETIME resolution is 100 ns */
+	kernelActiveTime = (lpKernelTimeU64.QuadPart * 100) - idleTime; /* kernel time includes idle time */
 	Trc_PRT_sysinfo_get_CPU_utilization_GST_result("kernel active", kernelActiveTime);
-	userTime = (lpUserTime.dwLowDateTime + (((uint64_t) lpUserTime.dwHighDateTime) << 32)) * 100;
+	userTime = lpUserTimeU64.QuadPart * 100;
 	Trc_PRT_sysinfo_get_CPU_utilization_GST_result("user", userTime);
 	cpuTime->cpuTime = kernelActiveTime + userTime;
+
+	cpuTime->userTime = lpUserTimeU64.QuadPart;
+	cpuTime->systemTime = lpKernelTimeU64.QuadPart - lpIdleTimeU64.QuadPart;
+	cpuTime->idleTime = lpIdleTimeU64.QuadPart;
+
 	cpuTime->timestamp = portLibrary->time_nano_time(portLibrary);
 	if (0 == cpuTime->timestamp) {
 		Trc_PRT_sysinfo_get_CPU_utilization_timeFailed();
@@ -1271,12 +1304,12 @@ omrsysinfo_get_CPU_load(struct OMRPortLibrary *portLibrary, double *cpuLoad)
 	if (oldestCPUTime->timestamp == 0) {
 		*oldestCPUTime = currentCPUTime;
 		*latestCPUTime = currentCPUTime;
-		return OMRPORT_ERROR_OPFAILED;
+		return OMRPORT_ERROR_INSUFFICIENT_DATA;
 	}
 
 	/* Calculate using the most recent value in the history */
 	if (((currentCPUTime.timestamp - latestCPUTime->timestamp) >= 10000000) && (currentCPUTime.numberOfCpus != 0)) {
-		*cpuLoad = OMR_MIN((currentCPUTime.cpuTime - latestCPUTime->cpuTime) / ((double)currentCPUTime.numberOfCpus * (currentCPUTime.timestamp - latestCPUTime->timestamp)), 1.0);
+		*cpuLoad = omrsysinfo_calculate_cpu_load(&currentCPUTime, latestCPUTime);
 		if (*cpuLoad >= 0.0) {
 			*oldestCPUTime = *latestCPUTime;
 			*latestCPUTime = currentCPUTime;
@@ -1288,7 +1321,7 @@ omrsysinfo_get_CPU_load(struct OMRPortLibrary *portLibrary, double *cpuLoad)
 	}
 	
 	if (((currentCPUTime.timestamp - oldestCPUTime->timestamp) >= 10000000) && (currentCPUTime.numberOfCpus != 0)) {
-		*cpuLoad = OMR_MIN((currentCPUTime.cpuTime - oldestCPUTime->cpuTime) / ((double)currentCPUTime.numberOfCpus * (currentCPUTime.timestamp - oldestCPUTime->timestamp)), 1.0);
+		*cpuLoad = omrsysinfo_calculate_cpu_load(&currentCPUTime, oldestCPUTime);
 		if (*cpuLoad >= 0.0) {
 			return 0;
 		} else {
@@ -1957,7 +1990,7 @@ omrsysinfo_get_cgroup_subsystem_list(struct OMRPortLibrary *portLibrary)
 BOOLEAN
 omrsysinfo_is_running_in_container(struct OMRPortLibrary *portLibrary)
 {
-	return PPG_isRunningInContainer;
+	return FALSE;
 }
 
 int32_t
@@ -1990,3 +2023,139 @@ omrsysinfo_cgroup_subsystem_iterator_destroy(struct OMRPortLibrary *portLibrary,
 	return;
 }
 
+int32_t
+omrsysinfo_get_process_start_time(struct OMRPortLibrary *portLibrary, uintptr_t pid, uint64_t *processStartTimeInNanoseconds)
+{
+	int32_t rc = 0;
+	uint64_t computedProcessStartTimeInNanoseconds = 0;
+	Trc_PRT_sysinfo_get_process_start_time_enter(pid);
+	if (0 != omrsysinfo_process_exists(portLibrary, pid)) {
+		double seconds = 0;
+		FILETIME createTime;
+		ULARGE_INTEGER createTimeU64;
+		FILETIME exitTime;
+		FILETIME kernelTime;
+		FILETIME userTime;
+		HANDLE process = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, (DWORD)pid);
+
+		memset(&createTime, 0, sizeof(createTime));
+		memset(&createTimeU64, 0, sizeof(createTimeU64));
+		memset(&exitTime, 0, sizeof(exitTime));
+		memset(&kernelTime, 0, sizeof(kernelTime));
+		memset(&userTime, 0, sizeof(userTime));
+
+		if (NULL == process) {
+			rc = OMRPORT_ERROR_SYSINFO_NONEXISTING_PROCESS;
+			goto done;
+		}
+		if (!GetProcessTimes(process, &createTime, &exitTime, &kernelTime, &userTime)) {
+			rc = OMRPORT_ERROR_SYSINFO_ERROR_GETTING_PROCESS_START_TIME;
+			goto cleanup;
+		}
+		createTimeU64.LowPart = createTime.dwLowDateTime;
+		createTimeU64.HighPart = createTime.dwHighDateTime;
+		seconds = ((double)createTimeU64.QuadPart) / OMRPORT_SYSINFO_WINDOWS_TICK;
+		computedProcessStartTimeInNanoseconds = (uint64_t)((seconds - OMRPORT_SYSINFO_SEC_TO_UNIX_EPOCH) * OMRPORT_SYSINFO_NS100_PER_SEC);
+		computedProcessStartTimeInNanoseconds *= 100;
+cleanup:
+		CloseHandle(process);
+	} else {
+		rc = OMRPORT_ERROR_SYSINFO_NONEXISTING_PROCESS;
+	}
+done:
+	*processStartTimeInNanoseconds = computedProcessStartTimeInNanoseconds;
+	Trc_PRT_sysinfo_get_process_start_time_exit(pid, computedProcessStartTimeInNanoseconds, rc);
+	return rc;
+}
+
+int32_t
+omrsysinfo_get_number_context_switches(struct OMRPortLibrary *portLibrary, uint64_t *numSwitches)
+{
+	return OMRPORT_ERROR_SYSINFO_NOT_SUPPORTED;
+}
+
+/**
+ * Get the process ID and commandline for each process.
+ * @param[in] portLibrary The port library.
+ * @param[in] callback The function to be invoked for each process with the process ID and command info.
+ * @param[in] userData Data passed to the callback.
+ * @return 0 on success, or the first non-zero value returned by the callback.
+ */
+uintptr_t
+omrsysinfo_get_processes(struct OMRPortLibrary *portLibrary, OMRProcessInfoCallback callback, void *userData)
+{
+	DWORD *processes = NULL;
+	DWORD bytesReturned = 0;
+	DWORD numProcesses = 0;
+	DWORD bufferSize = 1024 * sizeof(DWORD);
+	DWORD i = 0;
+	uintptr_t callbackResult = 0;
+	if (NULL == callback) {
+		portLibrary->error_set_last_error_with_message(
+				portLibrary,
+				OMRPORT_ERROR_OPFAILED,
+				"Callback function is NULL.");
+		return (uintptr_t)(intptr_t)OMRPORT_ERROR_OPFAILED;
+	}
+	processes = (DWORD *)portLibrary->mem_allocate_memory(
+			portLibrary,
+			bufferSize,
+			OMR_GET_CALLSITE(),
+			OMRMEM_CATEGORY_PORT_LIBRARY);
+	if (NULL == processes) {
+		goto alloc_failed;
+	}
+	for (;;) {
+		DWORD *newBuffer = NULL;
+		if (0 == EnumProcesses(processes, bufferSize, &bytesReturned)) {
+			Trc_PRT_failed_to_call_EnumProcesses(OMRPORT_ERROR_SYSINFO_OPFAILED);
+			callbackResult = (uintptr_t)(intptr_t)OMRPORT_ERROR_SYSINFO_OPFAILED;
+			goto done;
+		}
+		/* Break if the buffer is large enough; otherwise, grow the buffer. */
+		if (bytesReturned < bufferSize) {
+			break;
+		}
+		/* Buffer may be too small, increase and retry. */
+		bufferSize *= 2;
+		newBuffer = (DWORD *)portLibrary->mem_reallocate_memory(
+				portLibrary,
+				processes,
+				bufferSize,
+				OMR_GET_CALLSITE(),
+				OMRMEM_CATEGORY_PORT_LIBRARY);
+		if (NULL == newBuffer) {
+			goto alloc_failed;
+		}
+		processes = newBuffer;
+	}
+	numProcesses = bytesReturned / sizeof(DWORD);
+	for (i = 0; i < numProcesses; i++) {
+		char exePath[MAX_PATH];
+		DWORD pathLen = sizeof(exePath);
+		DWORD pid = processes[i];
+		HANDLE hProcess = OpenProcess(PROCESS_QUERY_LIMITED_INFORMATION, FALSE, pid);
+		exePath[0] = '\0';
+		if (NULL == hProcess) {
+			continue;
+		}
+		if (!QueryFullProcessImageName(hProcess, 0, exePath, &pathLen)) {
+			pathLen = 0;
+		}
+		CloseHandle(hProcess);
+		/* Skip entries with no name. */
+		if (0 == pathLen) {
+			continue;
+		}
+		callbackResult = callback((uintptr_t)pid, exePath, userData);
+		if (0 != callbackResult) {
+			goto done;
+		}
+	}
+done:
+	portLibrary->mem_free_memory(portLibrary, processes);
+	return callbackResult;
+alloc_failed:
+	callbackResult = (uintptr_t)(intptr_t)OMRPORT_ERROR_SYSINFO_MEMORY_ALLOC_FAILED;
+	goto done;
+}
