@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -28,6 +28,11 @@
  * APIs for querying per-thread statistics: CPU usage, stack usage.
  */
 
+#if defined(LINUX)
+#define _GNU_SOURCE
+#include <sys/resource.h>
+#endif /* defined(LINUX) */
+
 #include <string.h> /* for memset() */
 #include "omrcfg.h"
 
@@ -116,14 +121,33 @@ omrthread_get_cpu_time_ex(omrthread_t thread, int64_t *cpuTime)
 #if defined(OMR_OS_WINDOWS) && !defined(BREW)
 	{
 		intptr_t ret = 0;
-		FILETIME creationTime, exitTime, kernelTime, userTime;
-		int64_t totalTime;
+		FILETIME creationTime;
+		FILETIME exitTime;
+		FILETIME kernelTime;
+		FILETIME userTime;
+		int64_t totalTime = 0;
+
+		memset(&creationTime, 0, sizeof(creationTime));
+		memset(&exitTime, 0, sizeof(exitTime));
+		memset(&kernelTime, 0, sizeof(kernelTime));
+		memset(&userTime, 0, sizeof(userTime));
 
 		/* WARNING! Not supported on Win95!  Need to test to ensure this fails gracefully */
 
 		if (GetThreadTimes(thread->handle, &creationTime, &exitTime, &kernelTime, &userTime)) {
-			totalTime = ((int64_t)kernelTime.dwLowDateTime | ((int64_t)kernelTime.dwHighDateTime << 32))
-						+ ((int64_t)userTime.dwLowDateTime | ((int64_t)userTime.dwHighDateTime << 32));
+			ULARGE_INTEGER kernelTimeU64;
+			ULARGE_INTEGER userTimeU64;
+
+			memset(&kernelTimeU64, 0, sizeof(kernelTimeU64));
+			memset(&userTimeU64, 0, sizeof(userTimeU64));
+
+			kernelTimeU64.LowPart = kernelTime.dwLowDateTime;
+			kernelTimeU64.HighPart = kernelTime.dwHighDateTime;
+
+			userTimeU64.LowPart = userTime.dwLowDateTime;
+			userTimeU64.HighPart = userTime.dwHighDateTime;
+
+			totalTime = (int64_t)(kernelTimeU64.QuadPart + userTimeU64.QuadPart);
 
 			/* totalTime is in 100's of nanos.  Convert to nanos */
 			*cpuTime = totalTime * 100;
@@ -349,6 +373,7 @@ omrthread_get_self_cpu_time(omrthread_t self)
 
 #if defined(J9ZOS390)
 	{
+		extern void _CPUTIME(uint64_t *); /* defined in thrcputime.s */
 		uint64_t time = 0;
 
 		/* _CPUTIME returns time in TOD format (see z/Architecture Principles of Operation) */
@@ -403,13 +428,27 @@ omrthread_get_user_time(omrthread_t thread)
 	 *
 	 */
 
-	FILETIME creationTime, exitTime, kernelTime, userTime;
-	int64_t totalTime;
+	FILETIME creationTime;
+	FILETIME exitTime;
+	FILETIME kernelTime;
+	FILETIME userTime;
+	int64_t totalTime = 0;
+
+	memset(&creationTime, 0, sizeof(creationTime));
+	memset(&exitTime, 0, sizeof(exitTime));
+	memset(&kernelTime, 0, sizeof(kernelTime));
+	memset(&userTime, 0, sizeof(userTime));
 
 	/* WARNING! Not supported on Win95!  Need to test to ensure this fails gracefully */
 
 	if (GetThreadTimes(thread->handle, &creationTime, &exitTime, &kernelTime, &userTime)) {
-		totalTime = ((int64_t)userTime.dwLowDateTime | ((int64_t)userTime.dwHighDateTime << 32));
+		ULARGE_INTEGER userTimeU64;
+
+		memset(&userTimeU64, 0, sizeof(userTimeU64));
+
+		userTimeU64.LowPart = userTime.dwLowDateTime;
+		userTimeU64.HighPart = userTime.dwHighDateTime;
+		totalTime = (int64_t)userTimeU64.QuadPart;
 
 		/* totalTime is in 100's of nanos.  Convert to nanos */
 		return totalTime * 100;
@@ -463,17 +502,11 @@ uintptr_t
 omrthread_get_handle(omrthread_t thread)
 {
 #if defined(J9ZOS390)
-	/* Hack!! - If we do the simple cast (in the #else case) we get the following
-		compiler error in z/OS:
-			"ERROR CBC3117 ./thrprof.c:79    Operand must be a scalar type."
-		In order to work around the compiler error, we have to reach inside
-		the structure do the dirty work. The handle may not even be correct! */
-	uintptr_t *tempHandle;
-	tempHandle = (uintptr_t *)&(thread->handle.__[0]);
-	return *tempHandle;
-#else
+	OSTHREAD tempHandle = thread->handle;
+	return (uintptr_t)*(unsigned long long *)&tempHandle;
+#else /* defined(J9ZOS390) */
 	return (uintptr_t)thread->handle;
-#endif
+#endif /* defined(J9ZOS390) */
 }
 
 
@@ -667,12 +700,31 @@ int64_t
 omrthread_get_process_cpu_time(void)
 {
 #if defined(OMR_OS_WINDOWS) && !defined(BREW)
-	FILETIME creationTime, exitTime, kernelTime, userTime;
-	int64_t totalTime;
+	FILETIME creationTime;
+	FILETIME exitTime;
+	FILETIME kernelTime;
+	FILETIME userTime;
+	int64_t totalTime = 0;
+
+	memset(&creationTime, 0, sizeof(creationTime));
+	memset(&exitTime, 0, sizeof(exitTime));
+	memset(&kernelTime, 0, sizeof(kernelTime));
+	memset(&userTime, 0, sizeof(userTime));
 
 	if (GetProcessTimes(GetCurrentProcess(), &creationTime, &exitTime, &kernelTime, &userTime)) {
-		totalTime = ((int64_t)kernelTime.dwLowDateTime | ((int64_t)kernelTime.dwHighDateTime << 32))
-					+ ((int64_t)userTime.dwLowDateTime | ((int64_t)userTime.dwHighDateTime << 32));
+		ULARGE_INTEGER kernelTimeU64;
+		ULARGE_INTEGER userTimeU64;
+
+		memset(&kernelTimeU64, 0, sizeof(kernelTimeU64));
+		memset(&userTimeU64, 0, sizeof(userTimeU64));
+
+		kernelTimeU64.LowPart = kernelTime.dwLowDateTime;
+		kernelTimeU64.HighPart = kernelTime.dwHighDateTime;
+
+		userTimeU64.LowPart = userTime.dwLowDateTime;
+		userTimeU64.HighPart = userTime.dwHighDateTime;
+
+		totalTime = (int64_t)(kernelTimeU64.QuadPart + userTimeU64.QuadPart);
 
 		/* totalTime is in 100's of nanos.  Convert to nanos */
 		return totalTime * GET_PROCESS_TIMES_IN_NANO;
@@ -709,10 +761,20 @@ omrthread_get_process_times(omrthread_process_time_t *processTime)
 		 * WARNING: GetProcessTimes does not exist on pre-Win 98
 		 */
 		if (GetProcessTimes(GetCurrentProcess(), &creationTime, &exitTime, &systemTime, &userTime)) {
-			processTime->_userTime = GET_PROCESS_TIMES_IN_NANO *
-									 ((int64_t)userTime.dwLowDateTime | ((int64_t)userTime.dwHighDateTime << 32));
-			processTime->_systemTime = GET_PROCESS_TIMES_IN_NANO *
-									   ((int64_t)systemTime.dwLowDateTime | ((int64_t)systemTime.dwHighDateTime << 32));
+			ULARGE_INTEGER systemTimeU64;
+			ULARGE_INTEGER userTimeU64;
+
+			memset(&systemTimeU64, 0, sizeof(systemTimeU64));
+			memset(&userTimeU64, 0, sizeof(userTimeU64));
+
+			systemTimeU64.LowPart = systemTime.dwLowDateTime;
+			systemTimeU64.HighPart = systemTime.dwHighDateTime;
+
+			userTimeU64.LowPart = userTime.dwLowDateTime;
+			userTimeU64.HighPart = userTime.dwHighDateTime;
+
+			processTime->_userTime = (int64_t)(GET_PROCESS_TIMES_IN_NANO * userTimeU64.QuadPart);
+			processTime->_systemTime = (int64_t)(GET_PROCESS_TIMES_IN_NANO * systemTimeU64.QuadPart);
 			return 0;
 		} else {
 			/* GetLastError() indicates reason for failure in GetProcessTimes(). */
@@ -1024,3 +1086,76 @@ omrthread_get_jvm_cpu_usage_info_error_recovery(void)
 		GLOBAL_UNLOCK_SIMPLE(lib);
 	}
 }
+
+intptr_t
+omrthread_get_thread_times(omrthread_thread_time_t *threadTime)
+{
+#if defined(LINUX)
+	struct rusage rUsage;
+	memset(&rUsage, 0, sizeof(rUsage));
+
+	if (0 == getrusage(RUSAGE_THREAD, &rUsage)) {
+		threadTime->userTime = (SEC_TO_NANO_CONVERSION_CONSTANT * (int64_t)rUsage.ru_utime.tv_sec)
+			+ (MICRO_TO_NANO_CONVERSION_CONSTANT * (int64_t)rUsage.ru_utime.tv_usec);
+		threadTime->sysTime = (SEC_TO_NANO_CONVERSION_CONSTANT * (int64_t)rUsage.ru_stime.tv_sec)
+			+ (MICRO_TO_NANO_CONVERSION_CONSTANT * (int64_t)rUsage.ru_stime.tv_usec);
+
+		return 0;
+	}
+
+	return -1;
+#elif defined(OMR_OS_WINDOWS) && !defined(BREW) /* defined(LINUX) */
+	omrthread_t self = omrthread_self();
+	FILETIME creationTime;
+	FILETIME exitTime;
+	FILETIME kernelTime;
+	FILETIME userTime;
+	memset(&creationTime, 0, sizeof(creationTime));
+	memset(&exitTime, 0, sizeof(exitTime));
+	memset(&kernelTime, 0, sizeof(kernelTime));
+	memset(&userTime, 0, sizeof(userTime));
+
+	if (GetThreadTimes(self->handle, &creationTime, &exitTime, &kernelTime, &userTime)) {
+		ULARGE_INTEGER kernelTimeU64;
+		ULARGE_INTEGER userTimeU64;
+
+		memset(&kernelTimeU64, 0, sizeof(kernelTimeU64));
+		memset(&userTimeU64, 0, sizeof(userTimeU64));
+
+		kernelTimeU64.LowPart = kernelTime.dwLowDateTime;
+		kernelTimeU64.HighPart = kernelTime.dwHighDateTime;
+
+		userTimeU64.LowPart = userTime.dwLowDateTime;
+		userTimeU64.HighPart = userTime.dwHighDateTime;
+
+		/* Time is in 100's of nanos. Convert to nanos. */
+		threadTime->sysTime = (int64_t)(kernelTimeU64.QuadPart * 100);
+		threadTime->userTime = (int64_t)(userTimeU64.QuadPart * 100);
+
+		return 0;
+	}
+
+	return -1;
+#elif defined(AIXPPC) /* defined(OMR_OS_WINDOWS) && !defined(BREW) */
+	omrthread_t self = omrthread_self();
+
+	/* AIX provides a function call that returns an entire structure of
+	 * information about the thread.
+	 */
+	struct rusage rUsage;
+	memset(&rUsage, 0, sizeof(rUsage));
+
+	if (0 == pthread_getrusage_np(self->handle, &rUsage, PTHRDSINFO_RUSAGE_COLLECT)) {
+		threadTime->userTime = (SEC_TO_NANO_CONVERSION_CONSTANT * (int64_t)rUsage.ru_utime.tv_sec)
+			+ (MICRO_TO_NANO_CONVERSION_CONSTANT * (int64_t)rUsage.ru_utime.tv_usec);
+		threadTime->sysTime = (SEC_TO_NANO_CONVERSION_CONSTANT * (int64_t)rUsage.ru_stime.tv_sec)
+			+ (MICRO_TO_NANO_CONVERSION_CONSTANT * (int64_t)rUsage.ru_stime.tv_usec);
+		return 0;
+	}
+
+	return -1;
+#else /* defined(AIXPPC) */
+	/* Return -1 since the user time can only be retrieved on Windows, Linux, and AIX. */
+	return -1;
+#endif /* defined(LINUX) */
+}
