@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2015 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available 
  *  under the terms of the Eclipse Public License 2.0 which 
@@ -16,9 +16,9 @@
  * OpenJDK Assembly Exception [2].
  *   
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  ******************************************************************************/
 
 #include "omrutil.h"
@@ -26,6 +26,7 @@
 uintptr_t
 getStorageKey(void)
 {
+	extern uintptr_t _GETPSW(); /* defined in getpsw.s */
 	/* The storage key is returned in bits 8-11 of the PSW */
 	return ((_GETPSW() >> 20) & 0xF);
 }
