@@ -1,9 +1,8 @@
-/*[INCLUDE-IF Sidecar17]*/
-
+/*[INCLUDE-IF JAVA_SPEC_VERSION >= 8]*/
 package com.ibm.jit;
 
-/*******************************************************************************
- * Copyright (c) 1998, 2021 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 1998
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -19,27 +18,27 @@ package com.ibm.jit;
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 
 import com.ibm.oti.vm.J9UnmodifiableClass;
 import java.lang.reflect.Field;
 import java.lang.reflect.Array;
 import com.ibm.oti.vm.VM;
-/*[IF Sidecar19-SE]
+/*[IF JAVA_SPEC_VERSION >= 9]
 import jdk.internal.misc.Unsafe;
 import jdk.internal.reflect.Reflection;
 import jdk.internal.reflect.CallerSensitive;
-/*[ELSE]*/
+/*[ELSE] JAVA_SPEC_VERSION >= 9 */
 import sun.misc.Unsafe;
 import sun.reflect.Reflection;
 import sun.reflect.CallerSensitive;
-/*[ENDIF]*/
+/*[ENDIF] JAVA_SPEC_VERSION >= 9 */
 
 /**
- * The <code>JITHelpers</code> class contains methods used by the JIT to optimize certain primitive operations.
+ * The {@code JITHelpers} class contains methods used by the JIT to optimize certain primitive operations.
  */
 @SuppressWarnings("javadoc")
 @J9UnmodifiableClass
@@ -97,7 +96,7 @@ public final class JITHelpers {
 
 	/**
 	 * Java implementation of the native getSuperclass
-	 * 
+	 *
 	 * @param clazz
 	 *           The class to introspect (must not be null).
 	 * @return The superclass, or null for primitive types, interfaces and java.lang.Object.
@@ -163,7 +162,6 @@ public final class JITHelpers {
 		return getJ9ClassFromClass32(clazz);
 	}
 
-
 	/*
 	 * To be recognized by the JIT and returns true if the hardware supports SIMD case conversion.
 	 */
@@ -263,7 +261,7 @@ public final class JITHelpers {
 	 * checks for and handles: 1) standard fields in normal objects 2) static fields from classes 3) entries from arrays This sequence is branchy and
 	 * hard for us to optimize. The better solution in cases where we know the type of object is to use case specific accessors which are the methods
 	 * below. Be careful that you know the type of the object you are getting from before using these.
-	 * 
+	 *
 	 * NOTE: JIT assumes that obj is non-null and offset is positive - breaking this assumption is just asking for trouble.
 	 */
 	public int getIntFromObject(Object obj, long offset) {
@@ -283,11 +281,19 @@ public final class JITHelpers {
 	}
 
 	public Object getObjectFromObject(Object obj, long offset) {
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		return unsafe.getReference(obj, offset);
+		/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 		return unsafe.getObject(obj, offset);
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public Object getObjectFromObjectVolatile(Object obj, long offset) {
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		return unsafe.getReferenceVolatile(obj, offset);
+		/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 		return unsafe.getObjectVolatile(obj, offset);
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public void putIntInObject(Object obj, long offset, int value) {
@@ -307,35 +313,45 @@ public final class JITHelpers {
 	}
 
 	public void putObjectInObject(Object obj, long offset, Object value) {
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		unsafe.putReference(obj, offset, value);
+		/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 		unsafe.putObject(obj, offset, value);
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public void putObjectInObjectVolatile(Object obj, long offset, Object value) {
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		unsafe.putReferenceVolatile(obj, offset, value);
+		/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 		unsafe.putObjectVolatile(obj, offset, value);
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public boolean compareAndSwapIntInObject(Object obj, long offset, int expected, int value) {
-/*[IF Sidecar19-SE-OpenJ9]*/				
+		/*[IF JAVA_SPEC_VERSION >= 9]*/
 		return unsafe.compareAndSetInt(obj, offset, expected, value);
-/*[ELSE]
+		/*[ELSE] JAVA_SPEC_VERSION >= 9 */
 		return unsafe.compareAndSwapInt(obj, offset, expected, value);
-/*[ENDIF]*/		
+		/*[ENDIF] JAVA_SPEC_VERSION >= 9 */
 	}
 
 	public boolean compareAndSwapLongInObject(Object obj, long offset, long expected, long value) {
-/*[IF Sidecar19-SE-OpenJ9]*/				
+		/*[IF JAVA_SPEC_VERSION >= 9]*/
 		return unsafe.compareAndSetLong(obj, offset, expected, value);
-/*[ELSE]
+		/*[ELSE] JAVA_SPEC_VERSION >= 9 */
 		return unsafe.compareAndSwapLong(obj, offset, expected, value);
-/*[ENDIF]*/		
+		/*[ENDIF] JAVA_SPEC_VERSION >= 9 */
 	}
 
 	public boolean compareAndSwapObjectInObject(Object obj, long offset, Object expected, Object value) {
-/*[IF Sidecar19-SE-OpenJ9]*/				
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		return unsafe.compareAndSetReference(obj, offset, expected, value);
+		/*[ELSEIF JAVA_SPEC_VERSION >= 9]*/
 		return unsafe.compareAndSetObject(obj, offset, expected, value);
-/*[ELSE]
+		/*[ELSE] JAVA_SPEC_VERSION >= 9 */
 		return unsafe.compareAndSwapObject(obj, offset, expected, value);
-/*[ENDIF]*/		
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public byte getByteFromArray(Object obj, long offset) {
@@ -371,11 +387,19 @@ public final class JITHelpers {
 	}
 
 	public Object getObjectFromArray(Object obj, long offset) {
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		return unsafe.getReference(obj, offset);
+		/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 		return unsafe.getObject(obj, offset);
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public Object getObjectFromArrayVolatile(Object obj, long offset) {
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		return unsafe.getReferenceVolatile(obj, offset);
+		/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 		return unsafe.getObjectVolatile(obj, offset);
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public void putByteInArray(Object obj, long offset, byte value) {
@@ -411,35 +435,45 @@ public final class JITHelpers {
 	}
 
 	public void putObjectInArray(Object obj, long offset, Object value) {
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		unsafe.putReference(obj, offset, value);
+		/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 		unsafe.putObject(obj, offset, value);
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public void putObjectInArrayVolatile(Object obj, long offset, Object value) {
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		unsafe.putReferenceVolatile(obj, offset, value);
+		/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 		unsafe.putObjectVolatile(obj, offset, value);
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public boolean compareAndSwapIntInArray(Object obj, long offset, int expected, int value) {
-/*[IF Sidecar19-SE-OpenJ9]*/				
+		/*[IF JAVA_SPEC_VERSION >= 9]*/
 		return unsafe.compareAndSetInt(obj, offset, expected, value);
-/*[ELSE]
+		/*[ELSE] JAVA_SPEC_VERSION >= 9 */
 		return unsafe.compareAndSwapInt(obj, offset, expected, value);
-/*[ENDIF]*/		
+		/*[ENDIF] JAVA_SPEC_VERSION >= 9 */
 	}
 
 	public boolean compareAndSwapLongInArray(Object obj, long offset, long expected, long value) {
-/*[IF Sidecar19-SE-OpenJ9]*/				
+		/*[IF JAVA_SPEC_VERSION >= 9]*/
 		return unsafe.compareAndSetLong(obj, offset, expected, value);
-/*[ELSE]
+		/*[ELSE] JAVA_SPEC_VERSION >= 9 */
 		return unsafe.compareAndSwapLong(obj, offset, expected, value);
-/*[ENDIF]*/		
+		/*[ENDIF] JAVA_SPEC_VERSION >= 9 */
 	}
 
 	public boolean compareAndSwapObjectInArray(Object obj, long offset, Object expected, Object value) {
-/*[IF Sidecar19-SE-OpenJ9]*/				
+		/*[IF JAVA_SPEC_VERSION >= 23]*/
+		return unsafe.compareAndSetReference(obj, offset, expected, value);
+		/*[ELSEIF JAVA_SPEC_VERSION >= 9]*/
 		return unsafe.compareAndSetObject(obj, offset, expected, value);
-/*[ELSE]
+		/*[ELSE] JAVA_SPEC_VERSION >= 9 */
 		return unsafe.compareAndSwapObject(obj, offset, expected, value);
-/*[ENDIF]*/		
+		/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 	}
 
 	public char byteToCharUnsigned(byte b) {
@@ -504,7 +538,7 @@ public final class JITHelpers {
 
 	public byte getByteFromArrayByIndex(Object obj, int index) {
 		Class<?> clazz = obj.getClass();
-		
+
 		if (clazz == byte[].class) {
 			return ((byte[]) obj)[index];
 		} else if (clazz == char[].class) {
@@ -567,19 +601,34 @@ public final class JITHelpers {
 		}
 	}
 
+/*[IF JAVA_SPEC_VERSION < 17]*/
+	public boolean canEncodeAsLatin1(byte[] array, int start, int length) {
+		int index = start << 1;
+		if (!IS_BIG_ENDIAN) {
+			index += 1;
+		}
+		for (int end = index + (length << 1); index < end; index += 2) {
+			if (array[index] != 0) {
+				return false;
+			}
+		}
+		return true;
+	}
+/*[ENDIF] JAVA_SPEC_VERSION < 17 */
+
 	/**
 	 * Returns the first index of the target character array within the source character array starting from the specified
 	 * offset.
 	 *
 	 * <p>This API implicitly assumes the following:
-	 * <blockquote><pre>
-	 *     - s1Value != null
-	 *     - s2Value != null
-	 *     - 0 <= s1len <= s1Value.length * 2
-	 *     - 1 <= s2len <= s2Value.length * 2
-	 *     - 0 <= start < s1len
-	 * <blockquote><pre>
-	 * 
+	 * <ul>
+	 * <li>{@code s1Value != null}</li>
+	 * <li>{@code s2Value != null}</li>
+	 * <li>{@code 0 <= s1len <= s1Value.length * 2}</li>
+	 * <li>{@code 1 <= s2len <= s2Value.length * 2}</li>
+	 * <li>{@code 0 <= start < s1len}</li>
+	 * </ul>
+	 *
 	 * @param s1Value the source character array to search in.
 	 * @param s1len   the length (in number of characters) of the source array.
 	 * @param s2Value the target character array to search for.
@@ -593,7 +642,7 @@ public final class JITHelpers {
 
 		while (true) {
 			int i = intrinsicIndexOfLatin1(s1Value, (byte)firstChar, start, s1len);
-						
+
 			// Handles subCount > count || start >= count
 			if (i == -1 || s2len + i > s1len) {
 				return -1;
@@ -612,20 +661,20 @@ public final class JITHelpers {
 			start = i + 1;
 		}
 	}
-	
+
 	/**
 	 * Returns the first index of the target character array within the source character array starting from the specified
 	 * offset.
 	 *
 	 * <p>This API implicitly assumes the following:
-	 * <blockquote><pre>
-	 *     - s1Value != null
-	 *     - s2Value != null
-	 *     - 0 <= s1len <= s1Value.length
-	 *     - 1 <= s2len <= s2Value.length
-	 *     - 0 <= start < s1len
-	 * <blockquote><pre>
-	 * 
+	 * <ul>
+	 * <li>{@code s1Value != null}</li>
+	 * <li>{@code s2Value != null}</li>
+	 * <li>{@code 0 <= s1len <= s1Value.length}</li>
+	 * <li>{@code 1 <= s2len <= s2Value.length}</li>
+	 * <li>{@code 0 <= start < s1len}</li>
+	 * </ul>
+	 *
 	 * @param s1Value the source character array to search in.
 	 * @param s1len   the length (in number of characters) of the source array.
 	 * @param s2Value the target character array to search for.
@@ -639,7 +688,7 @@ public final class JITHelpers {
 
 		while (true) {
 			int i = intrinsicIndexOfUTF16(s1Value, firstChar, start, s1len);
-			
+
 			// Handles subCount > count || start >= count
 			if (i == -1 || s2len + i > s1len) {
 				return -1;
@@ -663,12 +712,12 @@ public final class JITHelpers {
 	 * Returns the first index of the character within the source character array starting from the specified offset.
 	 *
 	 * <p>This API implicitly assumes the following:
-	 * <blockquote><pre>
-	 *     - array != null
-	 *     - 0 <= offset < length <= array.length * 1 (if array instanceof byte[])
-	 *     - 0 <= offset < length <= array.length * 2 (if array instanceof char[])
-	 * <blockquote><pre>
-	 * 
+	 * <ul>
+	 * <li>{@code array != null}</li>
+	 * <li>{@code 0 <= offset < length <= array.length * 1 (if array instanceof byte[])}</li>
+	 * <li>{@code 0 <= offset < length <= array.length * 2 (if array instanceof char[])}</li>
+	 * </ul>
+	 *
 	 * @param array  the source character array to search in.
 	 * @param ch     the character to search for.
 	 * @param offset the starting offset (in number of characters) to search from.
@@ -689,12 +738,12 @@ public final class JITHelpers {
 	 * Returns the first index of the character within the source character array starting from the specified offset.
 	 *
 	 * <p>This API implicitly assumes the following:
-	 * <blockquote><pre>
-	 *     - array != null
-	 *     - 0 <= offset < length <= array.length * 1 (if array instanceof byte[])
-	 *     - 0 <= offset < length <= array.length * 2 (if array instanceof char[])
-	 * <blockquote><pre>
-	 * 
+	 * <ul>
+	 * <li>{@code array != null}</li>
+	 * <li>{@code 0 <= offset < length <= array.length * 1 (if array instanceof byte[])}</li>
+	 * <li>{@code 0 <= offset < length <= array.length * 2 (if array instanceof char[])}</li>
+	 * </ul>
+	 *
 	 * @param array  the source character array to search in.
 	 * @param ch     the character to search for.
 	 * @param offset the starting offset (in number of characters) to search from.
@@ -729,7 +778,7 @@ public final class JITHelpers {
 
 	/**
 	 * Called by the JIT on 32bit platforms to copy a source object's contents to a destination object as part of a clone. This is a shallow copy only.
-	 * 
+	 *
 	 * @param srcObj
 	 *           The object whose fields are to be copied
 	 * @param destObj
@@ -782,7 +831,7 @@ public final class JITHelpers {
 
 	/**
 	 * Called by the JIT on 64bit platforms to copy a source object's contents to a destination object as part of a clone. This is a shallow copy only.
-	 * 
+	 *
 	 * @param srcObj
 	 *           The object whose fields are to be copied
 	 * @param destObj
@@ -846,7 +895,7 @@ public final class JITHelpers {
 
 	/**
 	 * Java implementation of optimized clone.
-	 * 
+	 *
 	 * @param srcObj
 	 *           The source object of the clone.
 	 * @return The cloned object.
@@ -892,8 +941,13 @@ public final class JITHelpers {
 			int countSlots = 0;
 			for (int index = 0; numSlotsInObject != 0; index++) {
 				if (isDescriptorPointerTagged(descriptorWord)) {
+					/*[IF JAVA_SPEC_VERSION >= 23]*/
+					Object fieldValue = unsafe.getReference(srcObj, VM.OBJECT_HEADER_SIZE + (countSlots * SLOT_SIZE));
+					unsafe.putReference(clnObj, VM.OBJECT_HEADER_SIZE + (index * SLOT_SIZE), fieldValue);
+					/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 					Object fieldValue = unsafe.getObject(srcObj, VM.OBJECT_HEADER_SIZE + (countSlots * SLOT_SIZE));
 					unsafe.putObject(clnObj, VM.OBJECT_HEADER_SIZE + (index * SLOT_SIZE), fieldValue);
+					/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 				} else {
 					int fieldValue = unsafe.getInt(srcObj, VM.OBJECT_HEADER_SIZE + (countSlots * SLOT_SIZE));
 					unsafe.putInt(clnObj, VM.OBJECT_HEADER_SIZE + (index * SLOT_SIZE), fieldValue);
@@ -934,8 +988,13 @@ public final class JITHelpers {
 			int countSlots = 0;
 			for (int index = 0; numSlotsInObject != 0; index++) {
 				if (isDescriptorPointerTagged(descriptorWord)) {
+					/*[IF JAVA_SPEC_VERSION >= 23]*/
+					Object fieldValue = unsafe.getReference(srcObj, VM.OBJECT_HEADER_SIZE + (countSlots * SLOT_SIZE));
+					unsafe.putReference(clnObj, VM.OBJECT_HEADER_SIZE + (index * SLOT_SIZE), fieldValue);
+					/*[ELSE] JAVA_SPEC_VERSION >= 23 */
 					Object fieldValue = unsafe.getObject(srcObj, VM.OBJECT_HEADER_SIZE + (countSlots * SLOT_SIZE));
 					unsafe.putObject(clnObj, VM.OBJECT_HEADER_SIZE + (index * SLOT_SIZE), fieldValue);
+					/*[ENDIF] JAVA_SPEC_VERSION >= 23 */
 				} else {
 					long fieldValue = unsafe.getLong(srcObj, VM.OBJECT_HEADER_SIZE + (countSlots * SLOT_SIZE));
 					unsafe.putLong(clnObj, VM.OBJECT_HEADER_SIZE + (index * SLOT_SIZE), fieldValue);
@@ -1091,11 +1150,11 @@ public final class JITHelpers {
 		return lwValue;
 	}
 
-	/**		
-	 * Determines whether the underlying platform's memory model is big-endian.		
-	 * 		
-	 * @return True if the underlying platform's memory model is big-endian, false otherwise.		
-	 */		
+	/**
+	 * Determines whether the underlying platform's memory model is big-endian.
+	 *
+	 * @return True if the underlying platform's memory model is big-endian, false otherwise.
+	 */
 	private native static final boolean isBigEndian();
 
 	/* Placeholder for JIT GPU optimizations - this method never actually gets run */
