@@ -1,7 +1,7 @@
 /*[INCLUDE-IF Sidecar18-SE]*/
 package java.lang;
-/*******************************************************************************
- * Copyright (c) 2017, 2021 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2017
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -17,10 +17,10 @@ package java.lang;
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 
 /**
  * Represents the internal J9ConstantPool
@@ -28,9 +28,9 @@ package java.lang;
  */
 final class InternalConstantPool {
 	@SuppressWarnings("unused")
-	private final long vmref;
+	private final long vmRef;
 
 	public InternalConstantPool(long addr) {
-		vmref = addr;
+		vmRef = addr;
 	}
 }
