@@ -1,6 +1,6 @@
 /*[INCLUDE-IF JAVA_SPEC_VERSION > 8]*/
-/*******************************************************************************
- * Copyright (c) 2016, 2022 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2016
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -16,21 +16,21 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package java.lang;
 
-import java.lang.StackWalker.StackFrameImpl;
 /*[IF JAVA_SPEC_VERSION >= 10]*/
 import java.lang.invoke.MethodType;
 /*[ENDIF] JAVA_SPEC_VERSION >= 10 */
 import java.lang.module.ModuleDescriptor;
 import java.lang.module.ModuleDescriptor.Version;
+/*[IF JAVA_SPEC_VERSION < 24]*/
 import java.security.Permission;
+/*[ENDIF] JAVA_SPEC_VERSION < 24 */
 import java.util.Collections;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Objects;
 import java.util.Optional;
@@ -39,6 +39,10 @@ import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
+/*[IF JAVA_SPEC_VERSION >= 19]*/
+import jdk.internal.vm.Continuation;
+import jdk.internal.vm.ContinuationScope;
+/*[ENDIF] JAVA_SPEC_VERSION >= 19 */
 
 /**
  * This provides a facility for iterating over the call stack of the current
@@ -51,62 +55,126 @@ import java.util.stream.Stream;
 public final class StackWalker {
 
 	private static final int DEFAULT_BUFFER_SIZE = 1;
-	private final static int J9_RETAIN_CLASS_REFERENCE = 1;
-	private final static int J9_SHOW_REFLECT_FRAMES = 2;
-	private final static int J9_SHOW_HIDDEN_FRAMES = 4;
 
-	final Set<Option> walkerOptions;
+	/* Java StackWalker flag constants cloned from java_lang_StackWalker.cpp. */
+	private static final int J9_RETAIN_CLASS_REFERENCE = 0x01;
+	private static final int J9_SHOW_REFLECT_FRAMES    = 0x02;
+	private static final int J9_SHOW_HIDDEN_FRAMES     = 0x04;
+	/*[IF JAVA_SPEC_VERSION >= 21]*/
+	private static final int J9_GET_MONITORS           = 0x08;
+	/*[ENDIF] JAVA_SPEC_VERSION >= 21 */
+	/*[IF JAVA_SPEC_VERSION >= 22]*/
+	private static final int J9_DROP_METHOD_INFO       = 0x10;
+	/*[ENDIF] JAVA_SPEC_VERSION >= 22 */
+	private static final int J9_GET_CALLER_CLASS       = 0x20;
+
+	/* Map the given options to the corresponding set of flags. */
+	private static int flagsFor(Set<Option> options) {
+		int flags = 0;
+
+		if (options.contains(Option.RETAIN_CLASS_REFERENCE)) {
+			flags |= J9_RETAIN_CLASS_REFERENCE;
+		}
+
+		if (options.contains(Option.SHOW_REFLECT_FRAMES)) {
+			flags |= J9_SHOW_REFLECT_FRAMES;
+		}
+
+		if (options.contains(Option.SHOW_HIDDEN_FRAMES)) {
+			flags |= J9_SHOW_HIDDEN_FRAMES;
+		}
+
+		/* There is no option corresponding to J9_GET_MONITORS. */
+
+		/*[IF JAVA_SPEC_VERSION >= 22]*/
+		if (options.contains(Option.DROP_METHOD_INFO)) {
+			flags |= J9_DROP_METHOD_INFO;
+		}
+		/*[ENDIF] JAVA_SPEC_VERSION >= 22 */
+
+		return flags;
+	}
 
 	private final int bufferSize;
-	private int flags;
+	private final int flags;
 
-	private StackWalker(Set<Option> options, int estimatedDepth) {
+	/*[IF JAVA_SPEC_VERSION >= 19]*/
+	private StackWalker(int flags, int estimatedDepth) {
+		this(flags, null, null, null, estimatedDepth);
+	}
+	/*[ENDIF] JAVA_SPEC_VERSION >= 19 */
+
+	private StackWalker(
+			int flags,
+			/*[IF JAVA_SPEC_VERSION >= 19]*/
+			ExtendedOption extendedOption,
+			ContinuationScope contScope,
+			Continuation continuation,
+			/*[ENDIF] JAVA_SPEC_VERSION >= 19 */
+			int estimatedDepth) {
 		super();
-		/* Caller is responsible for copying the client set (if any) */
-		walkerOptions = options; 
-		bufferSize = estimatedDepth;
+		this.flags = flags;
+		/*[IF JAVA_SPEC_VERSION >= 19]*/
+		this.extendedOption = extendedOption;
+		this.scope = contScope;
+		this.cont = continuation;
+		/*[ENDIF] JAVA_SPEC_VERSION >= 19 */
+		this.bufferSize = estimatedDepth;
+
 		if (estimatedDepth <= 0) {
 			/*[MSG "K0641", "estimatedDepth must be greater than 0"]*/
 			throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K0641")); //$NON-NLS-1$
 		}
-		if (options.contains(Option.RETAIN_CLASS_REFERENCE)) {
+
+		if ((flags & J9_RETAIN_CLASS_REFERENCE) != 0) {
+			/*[IF JAVA_SPEC_VERSION < 24]*/
 			@SuppressWarnings("removal")
 			SecurityManager securityMgr = System.getSecurityManager();
 			if (null != securityMgr) {
 				securityMgr.checkPermission(PermissionSingleton.perm);
 			}
-		}
-		flags = 0;
-		if (walkerOptions.contains(Option.RETAIN_CLASS_REFERENCE)) {
-			flags |= J9_RETAIN_CLASS_REFERENCE;
-		}
-		if (walkerOptions.contains(Option.SHOW_REFLECT_FRAMES)) {
-			flags |= J9_SHOW_REFLECT_FRAMES;
-		}
-		if (walkerOptions.contains(Option.SHOW_HIDDEN_FRAMES)) {
-			flags |= J9_SHOW_HIDDEN_FRAMES;
+			/*[ENDIF] JAVA_SPEC_VERSION < 24 */
+			/*[IF JAVA_SPEC_VERSION >= 19]*/
+			this.retainClassRef = true;
+		} else {
+			this.retainClassRef = false;
+			/*[ENDIF] JAVA_SPEC_VERSION >= 19 */
 		}
 	}
 
+	/*[IF JAVA_SPEC_VERSION >= 21]*/
+	/**
+	 * Return a new StackWalker suitable for use by PinnedThreadPrinter
+	 * with appropriate flags set (including J9_GET_MONITORS).
+	 */
+	static StackWalker newInstanceWithMonitors() {
+		int flags = J9_GET_MONITORS | J9_SHOW_REFLECT_FRAMES | J9_RETAIN_CLASS_REFERENCE;
+
+		return new StackWalker(flags, DEFAULT_BUFFER_SIZE);
+	}
+	/*[ENDIF] JAVA_SPEC_VERSION >= 21 */
+
 	/**
 	 * Factory method to create a StackWalker instance with no options set.
-	 * 
+	 *
 	 * @return StackWalker StackWalker object
 	 */
 	public static StackWalker getInstance() {
-		return getInstance(Collections.emptySet(), DEFAULT_BUFFER_SIZE);
+		return new StackWalker(0, DEFAULT_BUFFER_SIZE);
 	}
 
 	/**
 	 * Factory method to create a StackWalker with one option. This is provided
 	 * for the case where only a single option is required.
-	 * 
+	 *
 	 * @param option
 	 *            select the type of information to include
 	 * @return StackWalker instance configured with the value of option
+	/*[IF JAVA_SPEC_VERSION < 24]
 	 * @throws SecurityException
 	 *             if option is RETAIN_CLASS_REFERENCE and the security manager
-	 *             check fails.
+	 *             check fails
+	/*[ENDIF] JAVA_SPEC_VERSION < 24
 	 */
 	public static StackWalker getInstance(Option option) {
 		Objects.requireNonNull(option);
@@ -115,33 +183,38 @@ public final class StackWalker {
 
 	/**
 	 * Factory method to create a StackWalker with any number of options.
-	 * 
+	 *
 	 * @param options
-	 *            select the types of information to include.
+	 *            select the types of information to include
 	 * @return StackWalker instance configured with the given options
+	/*[IF JAVA_SPEC_VERSION < 24]
 	 * @throws SecurityException
 	 *             if RETAIN_CLASS_REFERENCE is requested and not permitted by
 	 *             the security manager
+	/*[ENDIF] JAVA_SPEC_VERSION < 24
 	 */
 	public static StackWalker getInstance(Set<Option> options) {
-		return getInstance(new HashSet<>(options), DEFAULT_BUFFER_SIZE);
+		return getInstance(options, DEFAULT_BUFFER_SIZE);
 	}
 
 	/**
 	 * Factory method to create a StackWalker.
-	 * 
+	 *
 	 * @param options
-	 *            select the types of information to include.
+	 *            select the types of information to include
 	 * @param estimatedDepth
-	 *            Hint for the size of buffer to use. Must be 1 or greater.
+	 *            hint for the size of buffer to use. Must be 1 or greater
 	 * @return StackWalker instance with the given options specifying the stack
-	 *         frame information it can access.
+	 *         frame information it can access
+	/*[IF JAVA_SPEC_VERSION < 24]
 	 * @throws SecurityException
 	 *             if RETAIN_CLASS_REFERENCE is requested and not permitted by
 	 *             the security manager
+	/*[ENDIF] JAVA_SPEC_VERSION < 24
 	 */
 	public static StackWalker getInstance(Set<Option> options, int estimatedDepth) {
-		return new StackWalker(new HashSet<>(options), estimatedDepth);
+		Objects.requireNonNull(options);
+		return new StackWalker(flagsFor(options), estimatedDepth);
 	}
 
 	/**
@@ -160,16 +233,16 @@ public final class StackWalker {
 	/**
 	 * Get the caller of the caller of this function, eliding any reflection or
 	 * hidden frames.
-	 * 
+	 *
 	 * @return Class object for the method calling the current method.
 	 * @throws UnsupportedOperationException
 	 *             if the StackWalker was not created with
 	 *             {@link Option#RETAIN_CLASS_REFERENCE}
 	 * @throws IllegalStateException
-	 *             if the caller is at the bottom of the stack.
+	 *             if the caller is at the bottom of the stack
 	 */
 	public Class<?> getCallerClass() {
-		if (!walkerOptions.contains(Option.RETAIN_CLASS_REFERENCE)) {
+		if ((flags & J9_RETAIN_CLASS_REFERENCE) == 0) {
 			/*[MSG "K0639", "Stack walker not configured with RETAIN_CLASS_REFERENCE"]*/
 			throw new UnsupportedOperationException(com.ibm.oti.util.Msg.getString("K0639")); //$NON-NLS-1$
 		}
@@ -177,7 +250,7 @@ public final class StackWalker {
 		 * Get the top two stack frames: the client calling getCallerClass and
 		 * the client's caller. Ignore reflection and special frames.
 		 */
-		List<StackFrame> result = StackWalker.walkWrapperImpl(J9_RETAIN_CLASS_REFERENCE, "getCallerClass", //$NON-NLS-1$
+		List<StackFrame> result = StackWalker.walkWrapperImpl(J9_RETAIN_CLASS_REFERENCE | J9_GET_CALLER_CLASS, "getCallerClass", //$NON-NLS-1$
 				s -> s.limit(2).collect(Collectors.toList()));
 		if (result.size() < 2) {
 			/*[MSG "K0640", "getCallerClass() called from method with no caller"]*/
@@ -192,17 +265,17 @@ public final class StackWalker {
 		return clientsCaller.getDeclaringClass();
 	}
 
-	private native static <T> T walkWrapperImpl(int flags, String walkerMethod,
+	private static native <T> T walkWrapperImpl(int flags, String walkerMethod,
 			Function<? super Stream<StackFrame>, ? extends T> function);
 
 	/**
 	 * Traverse the calling thread's stack at the time this method is called and
 	 * apply {@code function} to each stack frame.
-	 * 
-	 * @param <T> the type of the return value from applying function to the stream 
+	 *
+	 * @param <T> the type of the return value from applying function to the stream
 	 * @param function operation to apply to the stream
 	 * @param walkState Pointer to a J9StackWalkState struct
-	 * @return the value returned by {@code function}.
+	 * @return the value returned by {@code function}
 	 */
 	private static <T> T walkImpl(Function<? super Stream<StackFrame>, ? extends T> function, long walkState) {
 		T result;
@@ -217,25 +290,72 @@ public final class StackWalker {
 	/**
 	 * Traverse the calling thread's stack at the time this method is called and
 	 * apply {@code function} to each stack frame.
-	 * 
-	 * @param <T> the type of the return value from applying function to the stream 
+	 *
+	 * @param <T> the type of the return value from applying function to the stream
 	 * @param function operation to apply to the stream
-	 * @return the value returned by {@code function}.
+	 * @return the value returned by {@code function}
 	 */
 	public <T> T walk(Function<? super Stream<StackFrame>, ? extends T> function) {
+		/*[IF JAVA_SPEC_VERSION >= 19]*/
+		if (null != cont) {
+			if (cont.trylockAccess()) {
+				try {
+					return walkContinuationImpl(flags, function, cont);
+				} finally {
+					cont.unlockAccess();
+				}
+			} else {
+				throw new IllegalStateException("Continuation is mounted.");
+			}
+		}
+		/*[ENDIF] JAVA_SPEC_VERSION >= 19 */
 		return walkWrapperImpl(flags, "walk", function); //$NON-NLS-1$
 	}
 
+	/*[IF JAVA_SPEC_VERSION >= 19]*/
+	final boolean retainClassRef;
+
+	private final ExtendedOption extendedOption;
+	private final ContinuationScope scope;
+	private final Continuation cont;
+
+	private static native <T> T walkContinuationImpl(int flags, Function<? super Stream<StackFrame>, ? extends T> function, Continuation cont);
+
+	static StackWalker newInstance(Set<Option> options, ExtendedOption extendedOption) {
+		return newInstance(options, extendedOption, null, null);
+	}
+
+	static StackWalker newInstance(Set<Option> options, ExtendedOption extendedOption, ContinuationScope contScope) {
+		return newInstance(options, extendedOption, contScope, null);
+	}
+
+	static StackWalker newInstance(Set<Option> options, ExtendedOption extendedOption, ContinuationScope contScope, Continuation continuation) {
+		Objects.requireNonNull(options);
+		return new StackWalker(flagsFor(options), extendedOption, contScope, continuation, DEFAULT_BUFFER_SIZE);
+	}
+
+	enum ExtendedOption {
+		LOCALS_AND_OPERANDS;
+	}
+	/*[ENDIF] JAVA_SPEC_VERSION >= 19 */
+
 	/**
 	 * Selects what type of stack and method information is provided by the
-	 * StackWalker
-	 *
+	 * StackWalker.
 	 */
 	public static enum Option {
 		/**
 		 * Allow clients to obtain a method's Class object.
 		 */
 		RETAIN_CLASS_REFERENCE,
+		/*[IF JAVA_SPEC_VERSION >= 22]*/
+		/**
+		 * A client may use this option to signal that method information is not
+		 * required. UnsupportedOperationException is thrown if an attempt is
+		 * made to access that suppressed information.
+		 */
+		DROP_METHOD_INFO,
+		/*[ENDIF] JAVA_SPEC_VERSION >= 22 */
 		/**
 		 * Include stack frames for reflection methods.
 		 */
@@ -254,19 +374,19 @@ public final class StackWalker {
 
 		/**
 		 * @return the offset of the current bytecode in the method represented
-		 *         by this frame.
+		 *         by this frame
 		 */
 		int getByteCodeIndex();
 
 		/**
 		 * @return the binary name of the declaring class of this frame's
-		 *         method.
+		 *         method
 		 */
 		String getClassName();
 
 		/**
 		 * @return the Class object of the declaring class of this frame's
-		 *         method.
+		 *         method
 		 * @throws UnsupportedOperationException
 		 *             if the StackWalker was not created with
 		 *             Option.RETAIN_CLASS_REFERENCE
@@ -274,15 +394,15 @@ public final class StackWalker {
 		Class<?> getDeclaringClass();
 
 		/**
-		 * @return File name of the class containing the current method. May be
-		 *         null.
+		 * @return file name of the class containing the current method (may be
+		 *         null)
 		 */
 		String getFileName();
 
 		/**
-		 * @return Location of the current point of execution in the source
+		 * @return location of the current point of execution in the source
 		 *         file, or a negative number if this information is unavailable
-		 *         or the method is native.
+		 *         or the method is native
 		 */
 		int getLineNumber();
 
@@ -299,7 +419,7 @@ public final class StackWalker {
 
 		/**
 		 * Converts this StackFrame into a StackTraceElement.
-		 * 
+		 *
 		 * @return StackTraceElement
 		 */
 		StackTraceElement toStackTraceElement();
@@ -307,7 +427,7 @@ public final class StackWalker {
 		/*[IF JAVA_SPEC_VERSION >= 10]*/
 		/**
 		 * @throws UnsupportedOperationException if this method is not overridden
-		 * @return MethodType containing the parameter and return types for the associated method.
+		 * @return MethodType containing the parameter and return types for the associated method
 		 * @since 10
 		 */
 		default MethodType getMethodType() {
@@ -316,18 +436,17 @@ public final class StackWalker {
 
 		/**
 		 * @throws UnsupportedOperationException if this method is not overridden or the StackWalker
-		 * instance is not configured with RETAIN_CLASS_REFERENCE.
-		 * @return method descriptor string representing the type of this frame's method.
+		 * instance is not configured with RETAIN_CLASS_REFERENCE
+		 * @return method descriptor string representing the type of this frame's method
 		 * @since 10
 		 */
 		default String getDescriptor() {
 			throw new UnsupportedOperationException();
 		}
-
 		/*[ENDIF] JAVA_SPEC_VERSION >= 10 */
 	}
 
-	final static class StackFrameImpl implements StackFrame {
+	static final class StackFrameImpl implements StackFrame {
 
 		private Class<?> declaringClass;
 		private String fileName;
@@ -338,10 +457,30 @@ public final class StackWalker {
 		private Module frameModule;
 		private String methodName;
 		private String methodSignature;
-		boolean callerSensitive;
+		/*[IF JAVA_SPEC_VERSION >= 21]*/
+		private Object[] monitors;
+		/*[ENDIF] JAVA_SPEC_VERSION >= 21 */
+		/*[IF JAVA_SPEC_VERSION >= 22]*/
+		private int flags; /* a copy of StackWalker.flags */
+		/*[ENDIF] JAVA_SPEC_VERSION >= 22 */
+		private boolean callerSensitive;
+
+		/*
+		 * Throw UnsupportedOperationException if Option.DROP_METHOD_INFO
+		 * was specified when the StackWalker was created (Java 22+).
+		 */
+		private void ensureMethodInfo() {
+			/*[IF JAVA_SPEC_VERSION >= 22]*/
+			if ((flags & J9_DROP_METHOD_INFO) != 0) {
+				/*[MSG "K0639D","Stack walker configured with DROP_METHOD_INFO"]*/
+				throw new UnsupportedOperationException(com.ibm.oti.util.Msg.getString("K0639D")); //$NON-NLS-1$
+			}
+			/*[ENDIF] JAVA_SPEC_VERSION >= 22 */
+		}
 
 		@Override
 		public int getByteCodeIndex() {
+			ensureMethodInfo();
 			return bytecodeIndex;
 		}
 
@@ -362,26 +501,32 @@ public final class StackWalker {
 
 		@Override
 		public String getFileName() {
+			ensureMethodInfo();
 			return fileName;
 		}
 
 		@Override
 		public int getLineNumber() {
+			ensureMethodInfo();
 			return lineNumber;
 		}
 
 		@Override
 		public String getMethodName() {
+			ensureMethodInfo();
 			return methodName;
 		}
 
 		@Override
 		public boolean isNativeMethod() {
+			ensureMethodInfo();
 			return -2 == lineNumber;
 		}
 
 		@Override
 		public StackTraceElement toStackTraceElement() {
+			ensureMethodInfo();
+
 			String moduleName = null;
 			String moduleVersion = null;
 			if (null != frameModule && frameModule.isNamed()) {
@@ -393,7 +538,8 @@ public final class StackWalker {
 				}
 			}
 
-			StackTraceElement element = new StackTraceElement(classLoaderName, moduleName, moduleVersion, className, methodName, fileName,
+			StackTraceElement element = new StackTraceElement(classLoaderName,
+					moduleName, moduleVersion, className, methodName, fileName,
 					lineNumber);
 
 			/**
@@ -406,7 +552,7 @@ public final class StackWalker {
 
 			return element;
 		}
-		
+
 		@Override
 		public String toString() {
 			StackTraceElement stackTraceElement = toStackTraceElement();
@@ -426,6 +572,7 @@ public final class StackWalker {
 				/*[MSG "K0639","Stack walker not configured with RETAIN_CLASS_REFERENCE"]*/
 				throw new UnsupportedOperationException(com.ibm.oti.util.Msg.getString("K0639")); //$NON-NLS-1$
 			}
+			ensureMethodInfo();
 			return MethodType.fromMethodDescriptorString(methodSignature, declaringClass.internalGetClassLoader());
 		}
 
@@ -435,15 +582,23 @@ public final class StackWalker {
 		 * @since 10
 		 */
 		@Override
-		public java.lang.String getDescriptor() {
+		public String getDescriptor() {
+			ensureMethodInfo();
 			return methodSignature;
 		}
 		/*[ENDIF] JAVA_SPEC_VERSION >= 10 */
 
+		/*[IF JAVA_SPEC_VERSION >= 21]*/
+		Object[] getMonitors() {
+			return monitors;
+		}
+		/*[ENDIF] JAVA_SPEC_VERSION >= 21 */
 	}
 
-	static class PermissionSingleton {
+	/*[IF JAVA_SPEC_VERSION < 24]*/
+	static final class PermissionSingleton {
 		static final Permission perm =
 				new RuntimePermission("getStackWalkerWithClassReference"); //$NON-NLS-1$
 	}
+	/*[ENDIF] JAVA_SPEC_VERSION < 24 */
 }
