@@ -1,6 +1,6 @@
-/*[INCLUDE-IF Sidecar18-SE]*/
-/*******************************************************************************
- * Copyright (c) 1998, 2022 IBM Corp. and others
+/*[INCLUDE-IF JAVA_SPEC_VERSION >= 8]*/
+/*
+ * Copyright IBM Corp. and others 1998
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -16,20 +16,22 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package java.security;
 
-import java.security.AccessControlContext.AccessCache;
+import com.ibm.oti.util.Msg;
+/*[IF JAVA_SPEC_VERSION < 24]*/
 import sun.security.util.SecurityConstants;
+/*[ENDIF] JAVA_SPEC_VERSION < 24 */
 
-/*[IF Sidecar19-SE]
+/*[IF JAVA_SPEC_VERSION >= 9]
 import jdk.internal.reflect.CallerSensitive;
-/*[ELSE]*/
+/*[ELSE] JAVA_SPEC_VERSION >= 9 */
 import sun.reflect.CallerSensitive;
-/*[ENDIF]*/
+/*[ENDIF] JAVA_SPEC_VERSION >= 9 */
 
 /**
  * Checks access to system resources. Supports marking of code
@@ -41,8 +43,13 @@ import sun.reflect.CallerSensitive;
  */
 /*[IF JAVA_SPEC_VERSION >= 17]*/
 @Deprecated(since="17", forRemoval=true)
+@SuppressWarnings("removal")
 /*[ENDIF] JAVA_SPEC_VERSION >= 17 */
 public final class AccessController {
+/*[IF JAVA_SPEC_VERSION >= 24]*/
+	private static final AccessControlContext ACC_NO_PERM = new AccessControlContext(
+			new ProtectionDomain[] { new ProtectionDomain(null, null) });
+/*[ELSE] JAVA_SPEC_VERSION >= 24 */
 	static {
 		// Initialize vm-internal caches
 		initializeInternal();
@@ -53,15 +60,16 @@ public final class AccessController {
 	static final int OBJS_ARRAY_SIZE = 3;
 	static final int OBJS_INDEX_PERMS_OR_CACHECHECKED = 2;
 
-private static native void initializeInternal();
+	private static native void initializeInternal();
 
-/* [PR CMVC 188787] Enabling -Djava.security.debug option within WAS keeps JVM busy */
-static final class DebugRecursionDetection {
-	private static ThreadLocal<String> tlDebug = new ThreadLocal<>();
-	static ThreadLocal<String> getTlDebug() {
-		return tlDebug;
+	/* [PR CMVC 188787] Enabling -Djava.security.debug option within WAS keeps JVM busy */
+	static final class DebugRecursionDetection {
+		private static ThreadLocal<String> tlDebug = new ThreadLocal<>();
+		static ThreadLocal<String> getTlDebug() {
+			return tlDebug;
+		}
 	}
-}
+/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 
 /*[PR 1FDIC6B] J9JCL:WIN95 - AccessController missing private no-arg constructor */
 /**
@@ -71,6 +79,7 @@ private AccessController() {
 	super();
 }
 
+/*[IF JAVA_SPEC_VERSION < 24]*/
 /**
  * The object array returned has following format:
  *
@@ -105,6 +114,10 @@ private AccessController() {
  *           or just the ProtectionDomain of the caller of doPrivileged in case of flag forDoPrivilegedWithCombiner is true
  *          Permission object array
  *       Until a full permission privileged frame or the end of the stack reached.
+ *  There is a special case that the limited doPrivileged method is in a Java main() method, there is no caller frame,
+ *  the first element is an AccessControlContext object inherited from current thread,
+ *  the second element (ProtectionDomain object array) is NULL,
+ * 	the third element is NULL as well.
  *
  * Note: 1. The reason to have Pre-JEP140 and JEP 140 format is to keep similar format and processing logic
  *          when there is no limited doPrivileged method (JEP 140 implementation) involved.
@@ -179,10 +192,10 @@ private static void throwACE(boolean debug, Permission perm, ProtectionDomain pD
 	}
 	if (createACCdenied) {
 		/*[MSG "K002d", "Access denied {0} due to untrusted AccessControlContext since {1} is denied"]*/
-		throw new AccessControlException(com.ibm.oti.util.Msg.getString("K002d", perm, SecurityConstants.CREATE_ACC_PERMISSION), perm); //$NON-NLS-1$
+		throw new AccessControlException(Msg.getString("K002d", perm, SecurityConstants.CREATE_ACC_PERMISSION), perm); //$NON-NLS-1$
 	} else {
 		/*[MSG "K002c", "Access denied {0}"]*/
-		throw new AccessControlException(com.ibm.oti.util.Msg.getString("K002c", perm), perm); //$NON-NLS-1$
+		throw new AccessControlException(Msg.getString("K002c", perm), perm); //$NON-NLS-1$
 	}
 }
 
@@ -204,7 +217,7 @@ private static void throwACE(boolean debug, Permission perm, ProtectionDomain pD
  *
  * @return true if access is granted by a limited permission, otherwise return false
  */
-private static boolean checkPermissionHelper(Permission perm, AccessControlContext acc, DomainCombiner activeDC, Object[] objects, int frame, AccessCache checked, Object[] objPDomains, int debug, int startPos) {
+private static boolean checkPermissionHelper(Permission perm, AccessControlContext acc, DomainCombiner activeDC, Object[] objects, int frame, AccessControlContext.AccessCache checked, Object[] objPDomains, int debug, int startPos) {
 	boolean limitedPermImplied = false;
 	boolean debugEnabled = (debug & AccessControlContext.DEBUG_ENABLED) != 0;
 	ProtectionDomain[] pDomains = generatePDarray(activeDC, acc, objPDomains, debugEnabled, startPos);
@@ -240,17 +253,19 @@ private static boolean checkPermissionHelper(Permission perm, AccessControlConte
 
 	if (2 == startPos) { // Pre-JEP140 format
 		if (null != acc && (null != acc.doPrivilegedAcc || null != acc.nextStackAcc || acc.isLimitedContext)) {
-			checked = new AccessCache(); /* checked was null initially when Pre-JEP140 format */
+			checked = new AccessControlContext.AccessCache(); /* checked was null initially when Pre-JEP140 format */
 			return AccessControlContext.checkPermissionWithCache(perm, activeDC, pDomains, debug, acc, false, null, null, checked);
 		} else {
 			if (pDomains != null) {
 				for (int i = 0; i < length ; ++i) {
 					// invoke PD within acc.context first
-					/*[IF Sidecar19-SE]*/
-					if ((pDomains[length - i - 1] != null) && !pDomains[length - i - 1].impliesWithAltFilePerm(perm)) {
-					/*[ELSE]*/
-					if ((pDomains[length - i - 1] != null) && !pDomains[length - i - 1].implies(perm)) {
-					/*[ENDIF] Sidecar19-SE*/
+					if ((pDomains[length - i - 1] != null)
+					/*[IF JAVA_SPEC_VERSION >= 9]*/
+						&& !pDomains[length - i - 1].impliesWithAltFilePerm(perm)
+					/*[ELSE] JAVA_SPEC_VERSION >= 9 */
+						&& !pDomains[length - i - 1].implies(perm)
+					/*[ENDIF] JAVA_SPEC_VERSION >= 9 */
+					) {
 						throwACE((debug & AccessControlContext.DEBUG_ACCESS_DENIED) != 0, perm, pDomains[length - i - 1], false);
 					}
 				}
@@ -357,8 +372,15 @@ private static boolean debugHelperJEP140(Object[] objects, Permission perm) {
 	debugPrintStack(debug, perm);
 	return debug;
 }
+/*[ENDIF] JAVA_SPEC_VERSION < 24 */
 
 /**
+/*[IF JAVA_SPEC_VERSION >= 24]
+ * Throws AccessControlException
+ *
+ * @param       perm                    is ignored
+ * @exception   AccessControlException  is always thrown
+/*[ELSE] JAVA_SPEC_VERSION >= 24
  * Checks whether the running program is allowed to
  * access the resource being guarded by the given
  * Permission argument.
@@ -366,8 +388,13 @@ private static boolean debugHelperJEP140(Object[] objects, Permission perm) {
  * @param       perm                    the permission to check
  * @exception   AccessControlException  if access is not allowed.
  *              NullPointerException if perm is null
+/*[ENDIF] JAVA_SPEC_VERSION >= 24
  */
 public static void checkPermission(Permission perm) throws AccessControlException {
+/*[IF JAVA_SPEC_VERSION >= 24]*/
+	/*[MSG "K002e", "checking permissions is not supported"]*/
+	throw new AccessControlException(Msg.getString("K002e")); //$NON-NLS-1$
+/*[ELSE] JAVA_SPEC_VERSION >= 24 */
 	if (perm == null) {
 		throw new NullPointerException();
 	}
@@ -392,7 +419,7 @@ public static void checkPermission(Permission perm) throws AccessControlExceptio
 		 */
 		activeDC = topACC.domainCombiner;
 	}
-	
+
 	if (isPreJEP140Format) {
 		if ((debug != AccessControlContext.DEBUG_DISABLED) && !debugHelperPreJEP140(objects, perm)) {
 			debug = AccessControlContext.DEBUG_ACCESS_DENIED; // Disable DEBUG_ENABLED
@@ -406,7 +433,7 @@ public static void checkPermission(Permission perm) throws AccessControlExceptio
 			debug = AccessControlContext.DEBUG_ACCESS_DENIED; // Disable DEBUG_ENABLED
 		}
 
-		AccessCache checked = new AccessCache();
+		AccessControlContext.AccessCache checked = new AccessControlContext.AccessCache();
 		for (int j = 0; j < frameNbr; ++j) {
 			AccessControlContext acc = (AccessControlContext) objects[j * OBJS_ARRAY_SIZE];
 			Object[] objPDomains = (Object[]) objects[j * OBJS_ARRAY_SIZE + OBJS_INDEX_PDS];
@@ -421,8 +448,32 @@ public static void checkPermission(Permission perm) throws AccessControlExceptio
 		System.err.println("access allowed " + perm); //$NON-NLS-1$
 		DebugRecursionDetection.getTlDebug().remove();
 	}
+/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 }
 
+/**
+/*[IF JAVA_SPEC_VERSION >= 24]
+ * @return an AccessControlContext with no permissions
+/*[ELSE] JAVA_SPEC_VERSION >= 24
+ * Answers the access controller context of the current thread,
+ * including the inherited ones. It basically retrieves all the
+ * protection domains from the calling stack and creates an
+ * <code>AccessControlContext</code> with them.
+ *
+ * @return an AccessControlContext which captures the current state
+ *
+ * @see         AccessControlContext
+/*[ENDIF] JAVA_SPEC_VERSION >= 24
+ */
+public static AccessControlContext getContext() {
+/*[IF JAVA_SPEC_VERSION >= 24]*/
+	return ACC_NO_PERM;
+/*[ELSE] JAVA_SPEC_VERSION >= 24 */
+	return getContextHelper(false);
+/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
+}
+
+/*[IF JAVA_SPEC_VERSION < 24]*/
 /**
  * Used to keep the context live during doPrivileged().
  *
@@ -441,20 +492,6 @@ private static void keepalive(Permission... perms) {
 	return;
 }
 
-/**
- * Answers the access controller context of the current thread,
- * including the inherited ones. It basically retrieves all the
- * protection domains from the calling stack and creates an
- * <code>AccessControlContext</code> with them.
- *
- * @return an AccessControlContext which captures the current state
- *
- * @see         AccessControlContext
- */
-public static AccessControlContext getContext() {
-	return getContextHelper(false);
-}
-
 /**
  * This is a helper method for getContext() and doPrivilegedWithCombiner methods.
  * Answers the access controller context of the current thread including the inherited ones.
@@ -510,7 +547,9 @@ private static AccessControlContext getContextHelper(boolean forDoPrivilegedWith
 			/*[PR JAZZ 66930] j.s.AccessControlContext.checkPermission() invoke untrusted ProtectionDomain.implies */
 			// the actual ProtectionDomain element starts at index 1
 			pDomains = generatePDarray(activeDC, acc, (Object[]) domains[j * OBJS_ARRAY_SIZE + OBJS_INDEX_PDS], false, 1);
-			newAuthorizedState = getNewAuthorizedState(acc, (ProtectionDomain)((Object[]) domains[j * OBJS_ARRAY_SIZE + OBJS_INDEX_PDS])[0]);
+			Object[] objDomains = (Object[])domains[j * OBJS_ARRAY_SIZE + OBJS_INDEX_PDS];
+			ProtectionDomain callerPD = ((objDomains == null) || (objDomains.length == 0)) ? null : (ProtectionDomain)objDomains[0];
+			newAuthorizedState = getNewAuthorizedState(acc, callerPD);
 		}
 		if (((null != acc) && acc.isLimitedContext) || (1 < frameNbr)) {
 			// there is a limited doPrivilege frame
@@ -572,7 +611,7 @@ private static ProtectionDomain[] generatePDarray(DomainCombiner activeDC, Acces
 			actDC = acc.domainCombiner;
 		}
 	}
-	
+
 	if (actDC != null) {
 		if (debug) {
 			DebugRecursionDetection.getTlDebug().set(""); //$NON-NLS-1$
@@ -619,7 +658,7 @@ private static int getNewAuthorizedState(AccessControlContext acc, ProtectionDom
 		newAuthorizedState = AccessControlContext.STATE_AUTHORIZED;
 	}
 	return newAuthorizedState;
- }
+}
 
 /**
  * Helper method to combine the ProtectionDomain objects
@@ -687,6 +726,7 @@ static ProtectionDomain[] toArrayOfProtectionDomains(Object[] domains, AccessCon
 
 	return answer;
 }
+/*[ENDIF] JAVA_SPEC_VERSION < 24 */
 
 /**
  * Performs the privileged action specified by <code>action</code>.
@@ -735,10 +775,14 @@ public static <T> T doPrivileged(PrivilegedAction<T> action) {
  */
 @CallerSensitive
 public static <T> T doPrivileged(PrivilegedAction<T> action, AccessControlContext context) {
+	/*[IF JAVA_SPEC_VERSION >= 24]*/
+	return action.run();
+	/*[ELSE] JAVA_SPEC_VERSION >= 24 */
 	T result = action.run();
 	/*[PR 108112] context is not kept alive*/
 	keepalive(context);
 	return result;
+	/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 }
 
 /**
@@ -802,10 +846,14 @@ public static <T> T doPrivileged (PrivilegedExceptionAction<T> action, AccessCon
 	throws PrivilegedActionException
 {
 	try {
+		/*[IF JAVA_SPEC_VERSION >= 24]*/
+		return action.run();
+		/*[ELSE] JAVA_SPEC_VERSION >= 24 */
 		T result = action.run();
 		/*[PR 108112] context is not kept alive*/
 		keepalive(context);
 		return result;
+		/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 	} catch (RuntimeException ex) {
 		throw ex;
 	} catch (Exception ex) {
@@ -832,7 +880,11 @@ public static <T> T doPrivileged (PrivilegedExceptionAction<T> action, AccessCon
  */
 @CallerSensitive
 public static <T> T doPrivilegedWithCombiner(PrivilegedAction<T> action) {
-	return doPrivileged(action, getContextHelper(true));
+	/*[IF JAVA_SPEC_VERSION >= 24]*/
+	return doPrivileged(action, null);
+	/*[ELSE] JAVA_SPEC_VERSION >= 24 */
+	return doPrivileged(action, doPrivilegedWithCombinerHelper(null));
+	/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 }
 
 /**
@@ -859,9 +911,14 @@ public static <T> T doPrivilegedWithCombiner(PrivilegedAction<T> action) {
 public static <T> T doPrivilegedWithCombiner(PrivilegedExceptionAction<T> action)
 	throws PrivilegedActionException
 {
-	return doPrivileged(action, getContextHelper(true));
+	/*[IF JAVA_SPEC_VERSION >= 24]*/
+	return doPrivileged(action, null);
+	/*[ELSE] JAVA_SPEC_VERSION >= 24 */
+	return doPrivileged(action, doPrivilegedWithCombinerHelper(null));
+	/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 }
 
+/*[IF JAVA_SPEC_VERSION < 24]*/
 /**
  * Helper method to check if any permission is null
  *
@@ -877,6 +934,7 @@ private static void checkPermsNPE(Permission... perms) {
 		}
 	}
 }
+/*[ENDIF] JAVA_SPEC_VERSION < 24 */
 
 /**
  * Performs the privileged action specified by <code>action</code>.
@@ -905,11 +963,15 @@ private static void checkPermsNPE(Permission... perms) {
 public static <T> T doPrivileged(PrivilegedAction<T> action,
 		AccessControlContext context, Permission... perms)
 {
+	/*[IF JAVA_SPEC_VERSION >= 24]*/
+	return action.run();
+	/*[ELSE] JAVA_SPEC_VERSION >= 24 */
 	checkPermsNPE(perms);
 	T result = action.run();
 	keepalive(context);
 	keepalive(perms);
 	return result;
+	/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 }
 
 /**
@@ -937,16 +999,12 @@ public static <T> T doPrivileged(PrivilegedAction<T> action,
 public static <T> T doPrivilegedWithCombiner(PrivilegedAction<T> action,
 		AccessControlContext context, Permission... perms)
 {
+	/*[IF JAVA_SPEC_VERSION >= 24]*/
+	return doPrivileged(action, context, perms);
+	/*[ELSE] JAVA_SPEC_VERSION >= 24 */
 	checkPermsNPE(perms);
-	ProtectionDomain domain = getCallerPD(1);
-	ProtectionDomain[] pdArray = (domain == null) ? null : new ProtectionDomain[] { domain };
-	AccessControlContext fixedContext = new AccessControlContext(context, pdArray, getNewAuthorizedState(context, domain));
-	if (null == context) {
-		AccessControlContext parentContext = getContextHelper(true);
-		fixedContext.domainCombiner = parentContext.domainCombiner;
-		fixedContext.nextStackAcc = parentContext;
-	}
-	return doPrivileged(action, fixedContext, perms);
+	return doPrivileged(action, doPrivilegedWithCombinerHelper(context), perms);
+	/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 }
 
 /**
@@ -980,11 +1038,15 @@ public static <T> T doPrivileged(PrivilegedExceptionAction<T> action,
 	throws PrivilegedActionException
 {
 	try {
+		/*[IF JAVA_SPEC_VERSION >= 24]*/
+		return action.run();
+		/*[ELSE] JAVA_SPEC_VERSION >= 24 */
 		checkPermsNPE(perms);
 		T result = action.run();
 		keepalive(context);
 		keepalive(perms);
 		return result;
+		/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 	} catch (RuntimeException ex) {
 		throw ex;
 	} catch (Exception ex) {
@@ -1020,16 +1082,30 @@ public static <T> T doPrivilegedWithCombiner(PrivilegedExceptionAction<T> action
 		AccessControlContext context, Permission... perms)
 	throws PrivilegedActionException
 {
+	/*[IF JAVA_SPEC_VERSION >= 24]*/
+	return doPrivileged(action, context, perms);
+	/*[ELSE] JAVA_SPEC_VERSION >= 24 */
 	checkPermsNPE(perms);
-	ProtectionDomain domain = getCallerPD(1);
-	ProtectionDomain[] pdArray = (domain == null) ? null : new ProtectionDomain[] { domain };
-	AccessControlContext fixedContext = new AccessControlContext(context, pdArray, getNewAuthorizedState(context, domain));
-	if (null == context) {
-		AccessControlContext parentContext = getContextHelper(true);
-		fixedContext.domainCombiner = parentContext.domainCombiner;
-		fixedContext.nextStackAcc = parentContext;
-	}
-	return doPrivileged(action, fixedContext, perms);
+	return doPrivileged(action, doPrivilegedWithCombinerHelper(context), perms);
+	/*[ENDIF] JAVA_SPEC_VERSION >= 24 */
 }
 
+/*[IF JAVA_SPEC_VERSION < 24]*/
+/**
+ * Helper method to construct an AccessControlContext for doPrivilegedWithCombiner methods.
+ *
+ * @param   acc an AccessControlContext, if it is null, use getContextHelper() to construct a context.
+ *
+ * @return  An AccessControlContext to be applied to the doPrivileged(action, context, perms).
+ */
+@CallerSensitive
+private static AccessControlContext doPrivilegedWithCombinerHelper(AccessControlContext acc) {
+	ProtectionDomain domain = getCallerPD(2);
+	ProtectionDomain[] context = (domain == null) ? null : new ProtectionDomain[] { domain };
+	AccessControlContext parentAcc = getContextHelper(acc == null);
+
+	return new AccessControlContext(context, parentAcc, acc, getNewAuthorizedState(acc, domain));
+}
+/*[ENDIF] JAVA_SPEC_VERSION < 24 */
+
 }
