@@ -1,6 +1,6 @@
 /*[INCLUDE-IF JAVA_SPEC_VERSION >= 8]*/
-/*******************************************************************************
- * Copyright (c) 2009, 2022 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2009
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -16,29 +16,32 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
-
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package openj9.internal.tools.attach.target;
 
 import static openj9.internal.tools.attach.target.IPC.LOGGING_DISABLED;
 import static openj9.internal.tools.attach.target.IPC.LOGGING_ENABLED;
 import static openj9.internal.tools.attach.target.IPC.loggingStatus;
 
+import com.ibm.oti.vm.VM;
+
 import java.io.File;
 import java.io.IOException;
 import java.io.PrintStream;
+import java.nio.file.Files;
+import java.nio.file.Paths;
+import java.nio.file.attribute.FileTime;
 import java.util.Properties;
 import java.util.Vector;
-
-import com.ibm.oti.vm.VM;
+import java.util.concurrent.TimeUnit;
 
 /**
  * This class handles incoming attachment requests from other VMs. Must be
  * public because it is used by java.lang.System
- * 
+ *
  */
 public class AttachHandler extends Thread {
 
@@ -55,7 +58,7 @@ public class AttachHandler extends Thread {
 	/**
 	 * Time delay before we give up trying to terminate the wait loop.
 	 */
-	static final long shutdownTimeoutMs = Integer.getInteger("com.ibm.tools.attach.shutdown_timeout", 10000).longValue(); //$NON-NLS-1$
+	static final long shutdownTimeoutMs = Long.getLong("com.ibm.tools.attach.shutdown_timeout", 10000).longValue(); //$NON-NLS-1$
 	private enum AttachStateValues {
 		ATTACH_UNINITIALIZED, ATTACH_TERMINATED, ATTACH_STARTING, ATTACH_INITIALIZED
 	}
@@ -91,7 +94,7 @@ public class AttachHandler extends Thread {
 	}
 
 	private static boolean doCancelNotify;
-	
+
 	/**
 	 * used to force an attacher to ignore its own notifications
 	 */
@@ -103,33 +106,38 @@ public class AttachHandler extends Thread {
 	private static int numberOfTargets;
 	/**
 	 * As of Java 9, a VM cannot attach to itself unless explicitly enabled.
-	 * Grab the setting before the application has a chance to change it, 
-	 * but parse it lazily because we rarely need the value.
+	 * Grab the setting before the application has a chance to change it.
 	 */
-	public final static String allowAttachSelf = 
-			VM.getVMLangAccess().internalGetProperties().getProperty("jdk.attach.allowAttachSelf", //$NON-NLS-1$
-/*[IF Sidecar19-SE]*/
-					"false" //$NON-NLS-1$
-/*[ELSE]
-					"true" //$NON-NLS-1$
-/*[ENDIF]*/
-					);  
-	
+	public static final boolean selfAttachAllowed;
+
 	/* only the attach handler thread uses syncFileLock */
 	/* [PR Jazz 30075] Make syncFileLock an instance variable since it is accessed only by the attachHandler singleton. */
 	FileLock syncFileLock;
 
+	static volatile Thread fileAccessTimeUpdaterThread;
+
+	static {
+		String allowAttachSelf = VM.internalGetProperties().getProperty("jdk.attach.allowAttachSelf" //$NON-NLS-1$
+		/*[IF JAVA_SPEC_VERSION >= 9]*/
+				, "false" //$NON-NLS-1$
+		/*[ELSE] JAVA_SPEC_VERSION >= 9 */
+				, "true" //$NON-NLS-1$
+		/*[ENDIF] JAVA_SPEC_VERSION >= 9 */
+		);
+		selfAttachAllowed = "".equals(allowAttachSelf) || Boolean.parseBoolean(allowAttachSelf); //$NON-NLS-1$
+	}
+
 	/**
 	 * Keep the constructor private
 	 */
 	private AttachHandler() {
+		super("Attach API initializer"); //$NON-NLS-1$
 		setDaemon(true);
-		setName("Attach API initializer"); //$NON-NLS-1$
 	}
 
 	/**
 	 * start the attach handler, create files , semaphore, etc.
-	 * 
+	 *
 	 * @note This is on the VM startup critical path.
 	 */
 	static void initializeAttachAPI() {
@@ -142,12 +150,12 @@ public class AttachHandler extends Thread {
 		/*
 		 *  set default behaviour:
 		 *  Java 6 R24 and later: disabled by default on z/OS, enabled on all other platforms
-		 *  Java 5: disabled by default on all platforms 
+		 *  Java 5: disabled by default on all platforms
 		 */
 		/*[PR Jazz 59196 LIR: Disable attach API by default on z/OS (31972)]*/
 		boolean enableAttach = !IPC.isZOS;
 		/* the system property overrides the default */
-		String enableAttachProp = com.ibm.oti.vm.VM.getVMLangAccess().internalGetProperties().getProperty("com.ibm.tools.attach.enable");  //$NON-NLS-1$
+		String enableAttachProp = VM.internalGetProperties().getProperty("com.ibm.tools.attach.enable");  //$NON-NLS-1$
 		if (null != enableAttachProp) {
 			if (enableAttachProp.equalsIgnoreCase("no")) { //$NON-NLS-1$
 				enableAttach = false;
@@ -187,10 +195,10 @@ public class AttachHandler extends Thread {
 		try {
 			/*[PR Jazz 31593] stress failures caused by long wait times for lock file ]*/
 			if (CommonDirectory.tryObtainControllerLock("AttachHandler.createFiles(" + newDisplayName + ")_1")) { //$NON-NLS-1$ //$NON-NLS-2$
-				/* 
-				 * Non-blocking lock attempt succeeded. 
+				/*
+				 * Non-blocking lock attempt succeeded.
 				 * clean up garbage files from crashed processes or other failures.
-				 * This operation is optional: if there is contention for the lock, 
+				 * This operation is optional: if there is contention for the lock,
 				 * the first process in will do the cleanup.
 				 */
 				CommonDirectory.deleteStaleDirectories(pidProperty);
@@ -203,12 +211,12 @@ public class AttachHandler extends Thread {
 			}
 			CommonDirectory.createNotificationFile();
 			/*[PR Jazz 31593 create the file artifacts after we have cleaned the directory]*/
-			if (isAttachApiTerminated()) { 
+			if (isAttachApiTerminated()) {
 				return false; /* abort if we decided to shut down */
 			}
 			String myId = TargetDirectory.createMyDirectory(pidProperty, true);
 			/*[PR RTC 80844 problem in initialization, or we are shutting down ]*/
-			if (null == myId) { 
+			if (null == myId) {
 				return false;
 			}
 			setVmId(myId); /* may need to tweak the ID */
@@ -227,7 +235,7 @@ public class AttachHandler extends Thread {
 	 */
 	public void run() {
 		/* Set  the current thread as a System Thread */
-		com.ibm.oti.vm.VM.markCurrentThreadAsSystem();
+		VM.markCurrentThreadAsSystem();
 
 		synchronized (stateSync) { /* CMVC 161729 : shutdown hook may already be running */
 			if (AttachStateValues.ATTACH_UNINITIALIZED == getAttachState()) {
@@ -258,27 +266,28 @@ public class AttachHandler extends Thread {
 		} catch (IOException e) {
 			setAttachState(AttachStateValues.ATTACH_TERMINATED);
 			return;
-		}	
+		}
 	}
 
 	private boolean initialize() throws IOException {
-		String loggingProperty = com.ibm.oti.vm.VM.getVMLangAccess().internalGetProperties().getProperty(LOGGING_ENABLE_PROPERTY);
-		String logName = com.ibm.oti.vm.VM.getVMLangAccess().internalGetProperties().getProperty(LOG_NAME_PROPERTY);
+		final Properties internalProperties = VM.internalGetProperties();
+		String loggingProperty = internalProperties.getProperty(LOGGING_ENABLE_PROPERTY);
+		String logName = internalProperties.getProperty(LOG_NAME_PROPERTY);
 		if ((null != logName) && !logName.equals("")) { //$NON-NLS-1$
 			logName = logName + '_';
 		} else {
 			logName = ""; //$NON-NLS-1$
 		}
-		
-		nameProperty = com.ibm.oti.vm.VM.getVMLangAccess().internalGetProperties().getProperty(DISPLAYNAME_PROPERTY);
-		pidProperty = validateVmId(com.ibm.oti.vm.VM.getVMLangAccess().internalGetProperties().getProperty(VMID_PROPERTY));
+
+		nameProperty = internalProperties.getProperty(DISPLAYNAME_PROPERTY);
+		pidProperty = validateVmId(internalProperties.getProperty(VMID_PROPERTY));
 		if ((null == pidProperty) || (0 == pidProperty.length())) {
 			/* CMVC 161414 - PIDs and UIDs are long */
 			long pid = IPC.getProcessId();
 			pidProperty = Long.toString(pid);
 		}
 		if (null == nameProperty) {
-			nameProperty = com.ibm.oti.vm.VM.getVMLangAccess().internalGetProperties().getProperty("sun.java.command"); //$NON-NLS-1$
+			nameProperty = internalProperties.getProperty("sun.java.command"); //$NON-NLS-1$
 		}
 		synchronized (IPC.accessorMutex) {
 			if ((null == IPC.logStream) && (null != loggingProperty)
@@ -289,11 +298,10 @@ public class AttachHandler extends Thread {
 				IPC.printMessageWithHeader("AttachHandler initialized", IPC.logStream); //$NON-NLS-1$
 				loggingStatus = LOGGING_ENABLED;
 			} else {
-				loggingStatus = LOGGING_DISABLED;			
-			}			
+				loggingStatus = LOGGING_DISABLED;
+			}
 		}
-		
-		
+
 		synchronized (stateSync) {
 			if (isAttachApiTerminated()) {
 				IPC.logMessage("cancel initialize before prepareCommonDirectory"); //$NON-NLS-1$
@@ -313,20 +321,101 @@ public class AttachHandler extends Thread {
 			terminate(false);
 			return false; /* CMVC 161135. Force the run() method to stop. */
 		}
-		
+
 		File syncFileObjectTemp = TargetDirectory.getSyncFileObject();
 		if (isAttachApiTerminated() || (null == syncFileObjectTemp)) {
 			IPC.logMessage("cancel initialize before creating syncFileLock"); //$NON-NLS-1$
 			return false;
-		}		
+		}
 		/* the syncFileObject was created by createFiles() (above) */
-		syncFileLock = new FileLock(syncFileObjectTemp.getAbsolutePath(), TargetDirectory.SYNC_FILE_PERMISSIONS); 
+		syncFileLock = new FileLock(syncFileObjectTemp.getAbsolutePath(), TargetDirectory.SYNC_FILE_PERMISSIONS);
 		/* give other users read permission to the sync file */
 
+		if (IPC.isLinux) {
+			// Update AttachAPI control file access time every sleepDays (8 days by default).
+			// This is to prevent Linux systemd from deleting the aging files within /tmp after 10 days.
+			// More details at https://github.com/eclipse-openj9/openj9/issues/18720
+			int sleepDays = 8;
+			String fileAccessUpdateTime = internalProperties.getProperty("com.ibm.tools.attach.fileAccessUpdateTime", "8"); //$NON-NLS-1$ //$NON-NLS-2$
+			IPC.logMessage("fileAccessUpdateTime = " + fileAccessUpdateTime); //$NON-NLS-1$
+			try {
+				sleepDays = Integer.parseInt(fileAccessUpdateTime);
+			} catch (NumberFormatException nfe) {
+				// ignore this non-fatal exception, and use the default value
+			}
+			IPC.logMessage("fileAccessTimeUpdaterThread sleepDays = " + sleepDays); //$NON-NLS-1$
+			if (sleepDays > 0) {
+				final long targetIntervalMillis = TimeUnit.DAYS.toMillis(sleepDays);
+				final String lastAccessTime = "lastAccessTime"; //$NON-NLS-1$
+				final String commonDir = CommonDirectory.getCommonDirFileObject().getPath();
+				fileAccessTimeUpdaterThread = new Thread("Attach API update file access time") { //$NON-NLS-1$
+					@Override
+					public void run() {
+						VM.markCurrentThreadAsSystem();
+
+						long lastMillis = System.currentTimeMillis();
+						long sleepMillis = targetIntervalMillis;
+						for (;;) {
+							try {
+								Thread.sleep(sleepMillis);
+							} catch (InterruptedException ie) {
+								// ignore this non-fatal exception
+								IPC.logMessage("fileAccessTimeUpdaterThread received an InterruptedException: " + ie.getMessage()); //$NON-NLS-1$
+							}
+							if (isAttachApiTerminated()) {
+								IPC.logMessage("AttachAPI was terminated, fileAccessTimeUpdaterThread exits"); //$NON-NLS-1$
+								break;
+							}
+							long currentMillis = System.currentTimeMillis();
+							long passedMillis = currentMillis - lastMillis;
+							if (passedMillis >= sleepMillis) {
+								FileTime fileTime = FileTime.fromMillis(currentMillis);
+								IPC.logMessage("fileAccessTimeUpdaterThread currentMillis = " + currentMillis); //$NON-NLS-1$
+								try {
+									// Common directory : _attachlock
+									File attachLock = new File(commonDir, CommonDirectory.ATTACH_LOCK);
+									if (attachLock.exists()) {
+										// _attachlock only appears when there was an attaching request
+										Files.setAttribute(Paths.get(attachLock.getPath()), lastAccessTime, fileTime);
+										IPC.logMessage("fileAccessTimeUpdaterThread updated access time = " + attachLock); //$NON-NLS-1$
+									}
+									// Common directory : _controller
+									Files.setAttribute(Paths.get(commonDir, CommonDirectory.CONTROLLER_LOCKFILE), lastAccessTime, fileTime);
+									// Common directory : _notifier
+									Files.setAttribute(Paths.get(commonDir, CommonDirectory.CONTROLLER_NOTIFIER), lastAccessTime, fileTime);
+									// Target directory : attachNotificationSync
+									Files.setAttribute(Paths.get(TargetDirectory.getSyncFileObject().getPath()), lastAccessTime, fileTime);
+									// Target directory : attachInfo
+									Files.setAttribute(Paths.get(TargetDirectory.getAdvertisementFileObject().getPath()), lastAccessTime, fileTime);
+								} catch (IOException ioe) {
+									// ignore this non-fatal exception
+									IPC.logMessage("fileAccessTimeUpdaterThread received an IOException : " + ioe.getMessage()); //$NON-NLS-1$
+								}
+								// reset for next update
+								sleepMillis = targetIntervalMillis;
+								lastMillis = currentMillis;
+							} else {
+								// continue sleep()
+								sleepMillis = targetIntervalMillis - passedMillis;
+								IPC.logMessage("fileAccessTimeUpdaterThread currentMillis = " + currentMillis //$NON-NLS-1$
+										+ ", passedMillis = " + passedMillis //$NON-NLS-1$
+										+ ", sleepMillis = " + sleepMillis); //$NON-NLS-1$
+							}
+						}
+					}
+				};
+				fileAccessTimeUpdaterThread.setDaemon(true);
+				if (isAttachApiTerminated()) {
+					IPC.logMessage("AttachAPI was already terminated, no need to start fileAccessTimeUpdaterThread"); //$NON-NLS-1$
+					return false;
+				}
+				fileAccessTimeUpdaterThread.start();
+			}
+		}
+
 		return true;
 	}
 
-
 	/**
 	 * Try to read the reply. If it exists, connect to the attacher. This may be called from tryAttachTarget() in the case
 	 * of a VM attaching to itself.
@@ -372,11 +461,10 @@ public class AttachHandler extends Thread {
 	 * @return true if the caller should destroy the semaphore
 	 */
 	protected boolean terminate(boolean wakeHandler) {
-		
 		if (LOGGING_DISABLED != loggingStatus) {
 			IPC.logMessage("AttachHandler terminate: Attach API is being shut down, currentAttachThread = " + currentAttachThread); //$NON-NLS-1$
 		}
-		
+
 		synchronized (stateSync) {
 			/*[PR CMVC 161729 : shutdown hook may run before the initializer]*/
 			AttachStateValues myAttachState = getAttachState();
@@ -391,7 +479,12 @@ public class AttachHandler extends Thread {
 				break;
 			}
 		}
-		currentAttachThread.interrupt(); /* do this after we change the attachState */
+		/* do this after we change the attachState */
+		if ((fileAccessTimeUpdaterThread != null) && fileAccessTimeUpdaterThread.isAlive()) {
+			IPC.logMessage("fileAccessTimeUpdaterThread interrupt"); //$NON-NLS-1$
+			fileAccessTimeUpdaterThread.interrupt();
+		}
+		currentAttachThread.interrupt();
 		if (wakeHandler) {
 			if (LOGGING_DISABLED != loggingStatus) {
 				IPC.logMessage("AttachHandler terminate removing contents of directory : ", TargetDirectory.getTargetDirectoryPath(getVmId())); //$NON-NLS-1$
@@ -409,7 +502,7 @@ public class AttachHandler extends Thread {
 			}
 		}
 		FileLock.shutDown();
-		
+
 		return terminateWaitLoop(wakeHandler, 0);
 	}
 
@@ -426,12 +519,12 @@ public class AttachHandler extends Thread {
 		/*[PR CMVC 187777 : non-clean shutdown in life cycle tests]*/
 		/*
 		 * If multiple VMs shut down simultaneously, there is contention for the lock file.
-		 * If someone else has the lock file, it is likely but not guaranteed 
+		 * If someone else has the lock file, it is likely but not guaranteed
 		 * that they will post to the semaphore and wake up this VM.
-		 * 
-		 * Try to get the lock file, but give up after a timeout. If this VM is 
-		 * notified and terminates in the meantime, don't bother notifying. 
-		 * 
+		 *
+		 * Try to get the lock file, but give up after a timeout. If this VM is
+		 * notified and terminates in the meantime, don't bother notifying.
+		 *
 		 * Note that the controller lock is not held while waiting to shut down.
 		 */
 		long lockDeadline = System.nanoTime() + shutdownTimeoutMs*1000000/10; /* let the file lock use only a fraction of the timeout budget */
@@ -453,16 +546,16 @@ public class AttachHandler extends Thread {
 				IPC.logMessage("VM already notified for termination, abandoning controller lock"); //$NON-NLS-1$
 			}
 			if (gotLock) {
-				/* 
-				 * We grabbed the lock to notify the wait loop.  
-				 * Release it because the wait loop does not need to be notified. 
-				 */ 
+				/*
+				 * We grabbed the lock to notify the wait loop.
+				 * Release it because the wait loop does not need to be notified.
+				 */
 				CommonDirectory.releaseControllerLock("AttachHandler.terminateWaitLoop(" + wakeHandler + "," + retryNumber + ")_3"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
 				gotLock = false;
 			}
 		}
 
-		if (gotLock) {	
+		if (gotLock) {
 			try {
 				if (LOGGING_DISABLED != loggingStatus) {
 					IPC.logMessage("AttachHandler terminate obtained controller lock"); //$NON-NLS-1$
@@ -474,14 +567,14 @@ public class AttachHandler extends Thread {
 				int numTargets = CommonDirectory.countTargetDirectories() + retryNumber;
 				setNumberOfTargets(numTargets);
 				if (numTargets <= 1) {
-					/* 
+					/*
 					 * if this VM is the only one running, don't need to lock sync files.
 					 * numTargets may be 0 if my directory was deleted accidentally.
 					 */
 					setDoCancelNotify(false); /* cancelNotify re-opens the semaphore */
-					/* 
-					 * CMVC 162086: destroy does not always interrupt a semaphore wait. 
-					 * Therefore, notify myself 
+					/*
+					 * CMVC 162086: destroy does not always interrupt a semaphore wait.
+					 * Therefore, notify myself
 					 */
 					if (wakeHandler) {
 						CommonDirectory.notifyVm(1, true, "AttachHandler.terminateWaitLoop_1"); //$NON-NLS-1$
@@ -505,9 +598,9 @@ public class AttachHandler extends Thread {
 					IPC.logMessage("AttachHandler terminate released controller lock"); //$NON-NLS-1$
 				}
 			}
-		} else { 
+		} else {
 			/*
-			 * Either the handler has already terminated, or we cannot get the lock. In the latter case, clean up as best we can 
+			 * Either the handler has already terminated, or we cannot get the lock. In the latter case, clean up as best we can
 			 */
 			IPC.logMessage("AttachHandler tryObtainControllerLock failed"); //$NON-NLS-1$
 		}
@@ -518,21 +611,76 @@ public class AttachHandler extends Thread {
 	 * wind up the attach API on termination
 	 */
 	static final class teardownHook extends Thread {
+		teardownHook() {
+			super("Attach API teardown"); //$NON-NLS-1$)
+		}
+
+		private static void threadJoinHelper(Thread thread, long shutdownDeadlineNs) {
+			long timeout = 100;
+			int retryNumber = 0;
+			for (;;) {
+				long currentNanoTime = System.nanoTime();
+				long tempTimeout = TimeUnit.NANOSECONDS.toMillis(shutdownDeadlineNs - currentNanoTime);
+				if (tempTimeout <= 0) {
+					// already reached shutdownDeadlineNs
+					// or ignore the NANOSECONDS.excessNanos() which is less than 1ms
+					IPC.logMessage(thread + " already reached shutdownDeadlineNs : currentNanoTime = " //$NON-NLS-1$
+							+ currentNanoTime + ", tempTimeout = " + tempTimeout); //$NON-NLS-1$
+					break;
+				}
+				if (timeout > tempTimeout) {
+					// not wait beyond shutdownDeadlineNs
+					timeout = tempTimeout;
+				}
+				IPC.logMessage(thread + ": currentNanoTime = " + currentNanoTime //$NON-NLS-1$
+						+ ", tempTimeout = " + tempTimeout //$NON-NLS-1$
+						+ ", timeout = " + timeout); //$NON-NLS-1$
+				try {
+					thread.join(timeout);
+				} catch (InterruptedException e) {
+					IPC.logMessage(thread + ": join() interrupted"); //$NON-NLS-1$
+					break;
+				}
+				State state = thread.getState();
+				if (state == Thread.State.TERMINATED) {
+					// exit if thread is terminated
+					IPC.logMessage(thread + " is terminated, exit"); //$NON-NLS-1$
+					break;
+				}
+				IPC.logMessage(thread + ": state = " + state //$NON-NLS-1$
+						+ ", timeout waiting for termination. Retry #" //$NON-NLS-1$
+						+ retryNumber + ", timeout = " + timeout); //$NON-NLS-1$
+				timeout *= 2;
+				retryNumber += 1;
+				if (thread == currentAttachThread) {
+					IPC.logMessage(thread + ": currentAttachThread requires AttachHandler.terminateWaitLoop()"); //$NON-NLS-1$
+					AttachHandler.terminateWaitLoop(true, retryNumber);
+				} else if (thread == fileAccessTimeUpdaterThread) {
+					IPC.logMessage(thread + ": fileAccessTimeUpdaterThread requires thread.interrupt()"); //$NON-NLS-1$
+					thread.interrupt();
+				} else {
+					throw new InternalError(thread + ": unexpected"); //$NON-NLS-1$
+				}
+			}
+		}
 
 		@Override
 		public void run() {
 			/*[PR CMVC 188652]  Suppress OOM messages from attach API*/
 			try {
-				Thread.currentThread().setName("Attach API teardown"); //$NON-NLS-1$
 				/* Set  the current thread as a System Thread */
-				com.ibm.oti.vm.VM.markCurrentThreadAsSystem();
+				VM.markCurrentThreadAsSystem();
 				if (LOGGING_DISABLED != loggingStatus) {
 					IPC.logMessage("shutting down attach API : " + mainHandler); //$NON-NLS-1$
 				}
 				if (null == mainHandler) {
 					return; /* the constructor failed */
 				}
-				long shutdownDeadlineNs = System.nanoTime() + shutdownTimeoutMs*1000000;
+				long currentNanoTime = System.nanoTime();
+				long shutdownDeadlineNs = currentNanoTime + TimeUnit.MILLISECONDS.toNanos(shutdownTimeoutMs);
+				IPC.logMessage("currentNanoTime = " + currentNanoTime //$NON-NLS-1$
+						+ ", shutdownTimeoutMs = " + shutdownTimeoutMs //$NON-NLS-1$
+						+ ", shutdownDeadlineNs = " + shutdownDeadlineNs); //$NON-NLS-1$
 				boolean destroySemaphore = mainHandler.terminate(true);
 				try {
 					/*[PR CMVC 172177] Ensure the initializer thread has terminated */
@@ -542,22 +690,13 @@ public class AttachHandler extends Thread {
 					 */
 					mainHandler.join(shutdownTimeoutMs/2); /* don't let the join use all the time */
 					/*[PR CMVC 164479] AIX sometimes hangs in File.exists.  See if a ridiculously long timeout makes a difference*/
-					/* 
+					/*
 					 * currentAttachThread is the same as mainHandler before the wait loop launches.
-					 * In that case, the join is redundant but harmless. 
+					 * In that case, the join is redundant but harmless.
 					 */
-					int timeout = 100;
-					int retryNumber = 0;
-					while (System.nanoTime() < shutdownDeadlineNs) {
-						currentAttachThread.join(timeout); /* timeout in milliseconds */
-						if (currentAttachThread.getState() != Thread.State.TERMINATED) {
-							IPC.logMessage(currentAttachThread + "Timeout waiting for wait loop termination.  Retry #" + retryNumber); //$NON-NLS-1$
-							timeout *= 2;
-							AttachHandler.terminateWaitLoop(true, retryNumber);
-							++retryNumber;
-						} else {
-							break;
-						}
+					threadJoinHelper(currentAttachThread, shutdownDeadlineNs);
+					if ((fileAccessTimeUpdaterThread != null) && fileAccessTimeUpdaterThread.isAlive()) {
+						threadJoinHelper(fileAccessTimeUpdaterThread, shutdownDeadlineNs);
 					}
 				} catch (InterruptedException e) {
 					IPC.logMessage("teardown join with attach handler interrupted"); //$NON-NLS-1$
@@ -584,34 +723,34 @@ public class AttachHandler extends Thread {
 				} else {
 					CommonDirectory.closeSemaphore();
 					if (LOGGING_DISABLED != loggingStatus) {
-						IPC.logMessage("AttachHandler closed semaphore"); //$NON-NLS-1$ 
+						IPC.logMessage("AttachHandler closed semaphore"); //$NON-NLS-1$
 					}
 				}
 				if (null != IPC.logStream) {
-					/* 
+					/*
 					 * Defer closing the file as long as possible.
 					 * logStream is a PrintStream, which never throws IOException, so any further writes will be silently ignored.
 					 */
 					IPC.logStream.close();
 				}
 			} catch (OutOfMemoryError e) {
-				/* 
+				/*
 				 * note that this may leave the wait loop running or fail to clean up file system artifacts.
 				 * These are recoverable conditions, though.
 				 */
 				IPC.tracepoint(IPC.TRACEPOINT_STATUS_OOM_DURING_TERMINATE, e.getMessage());
 				return;
 			}
-		} 
+		}
 	}
 
 	/**
 	 * Checks if the attach API is initialized.  If not, it waits until it changes to the initialized or terminated state.
 	 * This is called by an application thread. attachState is initialized to ATTACH_UNINITIALIZED during class initialization.
-	 * 
-	 * This is provided for the benefit of applications which use attach API to load JVMTI agents 
+	 *
+	 * This is provided for the benefit of applications which use attach API to load JVMTI agents
 	 * into their own JVMs.
-	 * 
+	 *
 	 * @return true if the attach API was successfully initialized, false if the API is stopped, encountered an error, or timed out.
 	 */
 	public static boolean waitForAttachApiInitialization() {
@@ -623,9 +762,9 @@ public class AttachHandler extends Thread {
 			} else if (AttachStateValues.ATTACH_TERMINATED == currentState) {
 				status = false; /* initialization failed or was not enabled */
 			} else {
-				/* 
-				 * normal state sequence: UNINITIALIZED->STARTING->INITIALIZED->TERMINATED. 
-				 * May skip from UNINITIALIZED or STARTING directly to TERMINATED. 
+				/*
+				 * normal state sequence: UNINITIALIZED->STARTING->INITIALIZED->TERMINATED.
+				 * May skip from UNINITIALIZED or STARTING directly to TERMINATED.
 				 */
 				int waitCycles = 2; /* at most 2 state transitions are required to get to INITIALIZED. */
 				status = false; /* default value */
@@ -706,8 +845,9 @@ public class AttachHandler extends Thread {
 		Attachment.saveLocalConnectorAddress();
 		//If we don't have a local connector address, just continue and get the others
 		Properties agentProperties = new Properties();
+		final Properties internalProperties = VM.internalGetProperties();
 		for (String pName: agentPropertyNames) {
-			String pValue = com.ibm.oti.vm.VM.getVMLangAccess().internalGetProperties().getProperty(pName);
+			String pValue = internalProperties.getProperty(pName);
 			if (null != pValue) {
 				agentProperties.put(pName, pValue);
 			}
@@ -716,9 +856,9 @@ public class AttachHandler extends Thread {
 	}
 
 	/**
-	 * This is provided for the benefit of applications which use attach API to load JVMTI agents 
+	 * This is provided for the benefit of applications which use attach API to load JVMTI agents
 	 * into their own JVMs.
-	 * 
+	 *
 	 * @return True if the Attach API is set up and the VM can attach to other
 	 *         VMs and receive attach requests.
 	 */
@@ -729,7 +869,7 @@ public class AttachHandler extends Thread {
 	/*
 	 * @return true if the attach API has not started launching (i.e. disabled)
 	 * or is shutting down
-	 * 
+	 *
 	 */
 	public static boolean isAttachApiTerminated() {
 		return (AttachStateValues.ATTACH_TERMINATED == getAttachState());
@@ -757,7 +897,7 @@ public class AttachHandler extends Thread {
 	 * Set the waitingForSemaphore true if the attach API is not being shut down and return true,
 	 * otherwise set it false and return false. This is thread safe and coordinated with the overall
 	 * attach API state.
-	 * 
+	 *
 	 * @return true is the attach API is not being shut down
 	 */
 	static boolean startWaitingForSemaphore() {
@@ -818,7 +958,7 @@ public class AttachHandler extends Thread {
 
 	/**
 	 * This field is static final, so no synchronization required.
-	 * 
+	 *
 	 * @return the main AttachHandler created by the factory
 	 */
 	public static AttachHandler getMainHandler() {
@@ -826,9 +966,9 @@ public class AttachHandler extends Thread {
 	}
 
 	/**
-	 * This is provided for the benefit of applications which use attach API to load JVMTI agents 
+	 * This is provided for the benefit of applications which use attach API to load JVMTI agents
 	 * into their own JVMs.
-	 * 
+	 *
 	 * @return ID of this VM
 	 */
 	public static String getVmId() {
