@@ -1,6 +1,6 @@
 /*[INCLUDE-IF JAVA_SPEC_VERSION == 8]*/
-/*******************************************************************************
- * Copyright (c) 2005, 2021 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2005
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -16,10 +16,10 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package java.lang.management;
 
 import java.io.IOException;
@@ -30,6 +30,7 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Set;
 
+import javax.management.DynamicMBean;
 import javax.management.InstanceAlreadyExistsException;
 import javax.management.InstanceNotFoundException;
 import javax.management.MBeanRegistrationException;
@@ -40,6 +41,8 @@ import javax.management.MBeanServerPermission;
 import javax.management.NotCompliantMBeanException;
 import javax.management.NotificationEmitter;
 import javax.management.ObjectName;
+import javax.management.StandardEmitterMBean;
+import javax.management.StandardMBean;
 
 import com.ibm.java.lang.management.internal.ClassLoadingMXBeanImpl;
 import com.ibm.java.lang.management.internal.CompilationMXBeanImpl;
@@ -512,7 +515,15 @@ public class ManagementFactory {
 				}
 
 				try {
-					server.registerMBean(bean, objectName);
+					final DynamicMBean dmbean;
+					if (bean instanceof DynamicMBean) {
+						dmbean = (DynamicMBean)bean;
+					} else if (bean instanceof NotificationEmitter) {
+						dmbean = new StandardEmitterMBean(bean, null, true, (NotificationEmitter)bean);
+					} else {
+						dmbean = new StandardMBean(bean, null, true);
+					}
+					server.registerMBean(dmbean, objectName);
 				} catch (InstanceAlreadyExistsException | MBeanRegistrationException | NullPointerException e) {
 					if (ManagementUtils.VERBOSE_MODE) {
 						e.printStackTrace(System.err);
