@@ -1,6 +1,6 @@
 /*[INCLUDE-IF Sidecar18-SE]*/
-/*******************************************************************************
- * Copyright (c) 2007, 2018 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2007
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -16,10 +16,10 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package com.ibm.dtfj.javacore.builder.javacore;
 
 import java.util.HashMap;
@@ -57,7 +57,6 @@ import com.ibm.dtfj.javacore.builder.IJavaRuntimeBuilder;
 
 public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJavaRuntimeBuilder {
 
-
 	private JCImageProcess fImageProcess = null;
 	private JCJavaRuntime fJavaRuntime = null;
 	private JCImageAddressSpace fAddressSpace = null;
@@ -76,11 +75,11 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 		if (fId == null) fJavaRuntime = new JCJavaRuntime(fImageProcess, fId);
 		fAddressSpace = fImageProcess.getImageAddressSpace();
 	}
-	
+
 	/**
 	 * Must pass a valid class loader ID in order to generate a class loader object. Else exception
-	 * is thrown. 
-	 * 
+	 * is thrown.
+	 *
 	 * @param classLoaderName optional
 	 * @param clID required, or else exception is throw and class loader is not created.
 	 * @param objectID optional (although generally this is the same as the clID for most javacores)
@@ -93,14 +92,14 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 			if (classLoader == null) {
 				classLoader = new JCJavaClassLoader(getJavaRuntime(), clID);
 			}
-			
+
 			JCJavaObject javaObject = classLoader.getInternalObject();
 			if (javaObject == null) {
 				JCJavaClass javaClass = getJavaRuntime().findJavaClass(classLoaderName);
 				if (javaClass == null) {
 					javaClass = new JCJavaClass(getJavaRuntime(), classLoaderName);
 				}
-					
+
 				ImagePointer objectPointer = fAddressSpace.getPointer(objectID);
 				javaObject = new JCJavaObject(objectPointer, javaClass);
 				classLoader.setObject(javaObject);
@@ -110,10 +109,10 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 			throw new BuilderFailureException(e);
 		}
 	}
-	
+
 	/**
 	 * Note that even if a class was already registered with a class loader previously, it may contain incomplete data,
-	 * so this case has to be considered. 
+	 * so this case has to be considered.
 	 */
 	public JavaClass addClass(JavaClassLoader jClassLoader, String name, long classID, long superClassID, String fileName) throws BuilderFailureException {
 		try {
@@ -134,9 +133,9 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 			throw new BuilderFailureException(e);
 		}
 	}
-	
+
 	/**
-	 * Either retrieves an existing java class registered with the java runtime or it creates one and it fills in missing information, 
+	 * Either retrieves an existing java class registered with the java runtime or it creates one and it fills in missing information,
 	 * if the latter is available.
 	 * <br><br>
 	 * A valid runtime and class name must be passed in order to successfully generate a java class. An invalid argument exception is
@@ -144,7 +143,7 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 	 * <br><br>
 	 * A null value should not be returned, as this indicates an error was encountered. Instead, throw the
 	 * appropriate exception.
-	 * 
+	 *
 	 * @param runtime required
 	 * @param name required
 	 * @param classID optional
@@ -171,8 +170,8 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 					}
 				}
 				return jClass;
-			}			
-			
+			}
+
 			JCJavaClass jClass = runtime.findJavaClass(name);
 			if (jClass == null) {
 				jClass = new JCJavaClass(runtime, name);
@@ -188,7 +187,7 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 	}
 
 	/**
-	 * 
+	 *
 	 * @return imageprocess containing the runtime.
 	 */
 	public ImageProcess getImageProcess() {
@@ -200,7 +199,7 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 	 * throw an exception. Note that a javathread cannot be successfully added if no valid threadID is passed.
 	 * <br><br>
 	 * REQUIREMENT: Thread Id must be valid to properly create a Java Thread, or exception is thrown.
-	 * 
+	 *
 	 * @throws BuilderFailureException if arguments lead to an invalid DTFJ object, particularly an invalid threadID
 	 * @return a non null JavaThread that was successfully added or updated.
 	 */
@@ -246,7 +245,7 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 			}
 			if( fAddressSpace.isValidAddressID(blockingObject)) {
 				JCJavaClass jClass = generateJavaClass(getJavaRuntime(), blockingObjectClass, IBuilderData.NOT_AVAILABLE);
-				ImagePointer pointerObjectID = fAddressSpace.getPointer(javaObjID);
+				ImagePointer pointerObjectID = fAddressSpace.getPointer(blockingObject);
 				JCJavaObject jobject = new JCJavaObject(pointerObjectID, jClass);
 				javaThread.setBlockingObject(jobject);
 			}
@@ -257,7 +256,7 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 	}
 
 	/**
-	 * 
+	 *
 	 */
 	public JavaStackFrame addJavaStackFrame(JavaThread javaThread, String className, String classFileName, String methodName, String methodType, String compilationLevel, int lineNumber) throws BuilderFailureException {
 		try {
@@ -275,7 +274,7 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 			if ("run".equals(methodName)) {
 				/*
 				 * Perhaps the type of the thread is the type of the last run() method.
-				 * Not strictly accurate if the class was extended without overriding run(). 
+				 * Not strictly accurate if the class was extended without overriding run().
 				 * Add the object only if the object ID is valid.
 				 */
 				try {
@@ -317,7 +316,7 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 			 * Adding a class and monitor object are optional, therefore
 			 * if the data is not present don't assume it is an error.
 			 */
-			
+
 			if (className != null) {
 				JCJavaClass jClass = generateJavaClass(getJavaRuntime(), className, IBuilderData.NOT_AVAILABLE);
 				/*
@@ -336,7 +335,7 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 	}
 
 	/**
-	 * 
+	 *
 	 */
 	public void addBlockedThread(JavaMonitor monitor, long threadID) throws BuilderFailureException {
 		JCJavaMonitor jmonitor = (JCJavaMonitor) monitor;
@@ -345,23 +344,23 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 	}
 
 	/**
-	 * 
+	 *
 	 */
 	public void addWaitOnNotifyThread(JavaMonitor monitor, long threadID) throws BuilderFailureException {
 		JCJavaMonitor jmonitor = (JCJavaMonitor) monitor;
 		ImagePointer pointer = fAddressSpace.getPointer(threadID);
 		jmonitor.addNotifyWaiter(pointer);
 	}
-	
+
 	/**
 	 * Adds an (empty) JavaVMInitArgs
-	 */	
+	 */
 	public void addVMInitArgs() throws BuilderFailureException {
 		if (fJavaVMInitArgs != null) {
 			throw new BuilderFailureException("JCJavaVMInitArgs already created for this JavaRuntime");
 		}
 		try {
-			// Create JavaVMInitArgs object. Note that JNI version and ignoreUnrecognized 
+			// Create JavaVMInitArgs object. Note that JNI version and ignoreUnrecognized
 			// flags are not available in javacore - JCJavaVMInitArgs returns DataUnavailable
 			// if those fields are requested.
 			fJavaVMInitArgs = new JCJavaVMInitArgs(getJavaRuntime(), 0, true);
@@ -369,10 +368,10 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 			throw new BuilderFailureException(e);
 		}
 	}
-	
+
 	/**
 	 * Adds an individual VM option to JavaVMInitArgs
-	 */	
+	 */
 	public void addVMOption(String option) throws BuilderFailureException {
 		if (fJavaVMInitArgs == null) {
 			throw new BuilderFailureException("JCJavaVMInitArgs must be created before options added");
@@ -385,8 +384,8 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 	}
 
 	/**
-	 * Adds an individual VM option to JavaVMInitArgs, with 'extra information' field 
-	 */	
+	 * Adds an individual VM option to JavaVMInitArgs, with 'extra information' field
+	 */
 	public void addVMOption(String option, long extraInfo) throws BuilderFailureException {
 		if (fJavaVMInitArgs == null) {
 			throw new BuilderFailureException("JCJavaVMInitArgs must be created before options added");
@@ -398,7 +397,7 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 			throw new BuilderFailureException(e);
 		}
 	}
-	
+
 	/**
 	 * Sets the Java version string.
 	 */
@@ -427,13 +426,13 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 			JavaRuntimeMemoryCategory parent)
 	{
 		JCJavaRuntimeMemoryCategory category = new JCJavaRuntimeMemoryCategory(name,deepBytes,deepAllocations);
-		
+
 		if (parent != null) {
 			( (JCJavaRuntimeMemoryCategory) parent).addChild(category);
 		} else {
 			getJavaRuntime().addTopLevelMemoryCategory(category);
 		}
-		
+
 		return category;
 	}
 
@@ -451,11 +450,11 @@ public class JavaRuntimeBuilder extends AbstractBuilderComponent implements IJav
 	public void addJITProperty(String name, String value) {
 		getJavaRuntime().addJITProperty(name, value);
 	}
-	
+
 	public void setStartTime(long startTime) {
 		getJavaRuntime().setStartTime(startTime);
 	}
-	
+
 	public void setStartTimeNanos(long startTimeNanos) {
 		getJavaRuntime().setStartTimeNanos(startTimeNanos);
 	}
