@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include <string.h>
@@ -812,7 +812,7 @@ checkBytecodeStructure (J9CfrClassFile * classfile, UDATA methodIndex, UDATA len
 			break;
 
 		case CFR_BC_areturn:
-			if (!IS_REF_OR_VAL_SIGNATURE(sigChar) && (sigChar != '[')) {
+			if (!IS_CLASS_SIGNATURE(sigChar) && (sigChar != '[')) {
 				/* fail, modify the bytecode to be incompatible in the second pass of verification */
 				if (sigChar != 'V') {
 					*(bcIndex - 1) = CFR_BC_return;
@@ -829,9 +829,6 @@ checkBytecodeStructure (J9CfrClassFile * classfile, UDATA methodIndex, UDATA len
 */
 			break;
 
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-		case CFR_BC_withfield:
-#endif /* J9VM_OPT_VALHALLA_VALUE_TYPES */
 		case CFR_BC_getstatic:
 		case CFR_BC_putstatic:
 		case CFR_BC_getfield:
@@ -887,20 +884,13 @@ checkBytecodeStructure (J9CfrClassFile * classfile, UDATA methodIndex, UDATA len
 			info = &(classfile->constantPool[classfile->constantPool[index].slot1]);
 			U_16 returnChar = getReturnTypeFromSignature(classfile->constantPool[classfile->constantPool[index].slot2].bytes, classfile->constantPool[classfile->constantPool[index].slot2].slot1, NULL);
 			if (info->bytes[0] == '<') {
-				if ((info->tag != CFR_CONSTANT_Utf8)
-					|| !J9UTF8_DATA_EQUALS("<init>", 6, info->bytes, info->slot1)
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-#if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD)
-					|| (J9UTF8_DATA_EQUALS("<new>", 5, info->bytes, info->slot1) && (bc != CFR_BC_invokestatic))
-					|| (!J9UTF8_DATA_EQUALS("<new>", 5, info->bytes, info->slot1) && (bc != CFR_BC_invokespecial))
-#else /* #if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD) */
-					|| ((IS_QTYPE(returnChar) || IS_LTYPE(returnChar)) && (bc != CFR_BC_invokestatic))
-					|| (!(IS_QTYPE(returnChar) || IS_LTYPE(returnChar)) && (bc != CFR_BC_invokespecial))
-#endif /* #if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD) */
-#else /* #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-					|| (bc != CFR_BC_invokespecial)
-#endif /* #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-				) {
+				BOOLEAN legal = FALSE;
+				if (CFR_CONSTANT_Utf8 == info->tag) {
+					if (CFR_BC_invokespecial == bc) {
+						legal = J9UTF8_DATA_EQUALS("<init>", 6, info->bytes, info->slot1);
+					}
+				}
+				if (!legal) {
 					errorType = J9NLS_CFR_ERR_BC_METHOD_INVALID__ID;
 					goto _verifyError;
 				}
@@ -978,10 +968,6 @@ checkBytecodeStructure (J9CfrClassFile * classfile, UDATA methodIndex, UDATA len
 			}
 			break;
 
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-		case CFR_BC_aconst_init:
-			/* fall through */
-#endif /* J9VM_OPT_VALHALLA_VALUE_TYPES */
 		case CFR_BC_new:
 			NEXT_U16(index, bcIndex);
 			if ((!index) || (index >= cpCount)) {
@@ -997,6 +983,7 @@ checkBytecodeStructure (J9CfrClassFile * classfile, UDATA methodIndex, UDATA len
 				errorDataIndex = index;
 				goto _verifyError;
 			}
+			info = &(classfile->constantPool[classfile->constantPool[index].slot1]);
 			break;
 
 		case CFR_BC_newarray:
@@ -1554,11 +1541,6 @@ checkMethodStructure (J9PortLibrary * portLib, J9CfrClassFile * classfile, UDATA
 	/* Throw a class format error if we are given a static <init> method (otherwise later we will throw a verify error due to back stack shape) */
 	info = &(classfile->constantPool[method->nameIndex]);
 	if ((method->accessFlags & CFR_ACC_STATIC)
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-#if !defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD)
-		&& (!J9_IS_CLASSFILE_VALUETYPE(classfile))
-#endif /* !defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD) */
-#endif /* #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
 	) {
 		if (CFR_CONSTANT_Utf8 == info->tag) {
 			if (J9UTF8_DATA_EQUALS("<init>", 6, info->bytes, info->slot1)) {
@@ -1780,37 +1762,12 @@ j9bcv_verifyClassStructure (J9PortLibrary * portLib, J9CfrClassFile * classfile,
 			&& !isConstantInvokeDynamic
 #endif /* JAVA_SPEC_VERSION >= 18 */
 			) {
-				BOOLEAN invalidRetType = FALSE;
-				if (
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-#if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD)
-				(CFR_METHOD_NAME_NEW == isInit)
-#else /* #if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD) */
-				(CFR_METHOD_NAME_INIT == isInit) && J9_IS_CLASSFILE_VALUETYPE(classfile)
-#endif /* #if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD) */
-#else /* #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-				FALSE
-#endif /* #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-				) {
-					U_16 returnChar = getReturnTypeFromSignature(info->bytes, info->slot1, NULL);
-					if (J9_IS_CLASSFILE_PRIMITIVE_VALUETYPE(classfile)) {
-						if (!IS_QTYPE(returnChar)) {
-							invalidRetType = TRUE;
-						}
-					} else {
-						if (!IS_LTYPE(returnChar)) {
-							invalidRetType = TRUE;
-						}
-					}
-				} else {
+				{
 					if (info->bytes[info->slot1 - 1] != 'V') {
-						invalidRetType = TRUE;
+						errorType = J9NLS_CFR_ERR_BC_METHOD_INVALID_SIG__ID;
+						goto _formatError;
 					}
 				}
-				if (invalidRetType) {
-					errorType = J9NLS_CFR_ERR_BC_METHOD_INVALID_SIG__ID;
-					goto _formatError;
-				}
 			}
 			if (argCount > 255) {
 				errorType = J9NLS_CFR_ERR_TOO_MANY_ARGS__ID;
@@ -1832,7 +1789,7 @@ j9bcv_verifyClassStructure (J9PortLibrary * portLib, J9CfrClassFile * classfile,
 				J9CfrConstantPoolInfo *methodref = &classfile->constantPool[info->slot2];
 				nameAndSig = &classfile->constantPool[methodref->slot2];
 				utf8 = &classfile->constantPool[nameAndSig->slot1];
-				isInit = bcvIsInitOrClinitOrNew(utf8);
+				isInit = bcvIsInitOrClinit(utf8);
 				if (CFR_METHOD_NAME_CLINIT == isInit) {
 					errorType = J9NLS_CFR_ERR_BAD_METHOD_NAME__ID;
 					goto _formatError;
@@ -1840,26 +1797,11 @@ j9bcv_verifyClassStructure (J9PortLibrary * portLib, J9CfrClassFile * classfile,
 				if (CFR_METHOD_NAME_INIT == isInit) {
 					if (
 						(info->slot1 != MH_REF_NEWINVOKESPECIAL)
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-#if !defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD)
-						&& (info->slot1 != MH_REF_INVOKESTATIC)
-#endif /* #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-#endif /* #if !defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD) */
 					) {
 						errorType = J9NLS_CFR_ERR_BAD_METHOD_NAME__ID;
 						goto _formatError;
 					}
 				}
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-#if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD)
-				if (CFR_METHOD_NAME_NEW == isInit) {
-					if (info->slot1 != MH_REF_INVOKESTATIC) {
-						errorType = J9NLS_CFR_ERR_BAD_METHOD_NAME__ID;
-						goto _formatError;
-					}
-				}
-#endif /* #if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD) */
-#endif /* #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
 			}
 			break;
 
@@ -1932,35 +1874,7 @@ j9bcv_verifyClassStructure (J9PortLibrary * portLib, J9CfrClassFile * classfile,
 			}
 		}
 		if (isInit) {
-			BOOLEAN invalidRetType = FALSE;
-			if (
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-#if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD)
-				(CFR_METHOD_NAME_NEW == isInit)
-#else /* #if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD) */
-				/* Check if classfile is a valuetype to differentiate between a regular <init> and an <init> that behaves like a <new> */
-				(CFR_METHOD_NAME_INIT == isInit) && J9_IS_CLASSFILE_VALUETYPE(classfile)
-#endif /* #if defined(J9VM_OPT_VALHALLA_NEW_FACTORY_METHOD) */
-#else /* #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-				FALSE
-#endif /* #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-			) {
-				U_16 returnChar = getReturnTypeFromSignature(info->bytes, info->slot1, NULL);
-				if (J9_IS_CLASSFILE_PRIMITIVE_VALUETYPE(classfile)) {
-					if (!IS_QTYPE(returnChar)) {
-						invalidRetType = TRUE;
-					}
-				} else {
-					if (!IS_LTYPE(returnChar)) {
-						invalidRetType = TRUE;
-					}
-				}
-			} else {
-				if (info->bytes[info->slot1 - 1] != 'V') {
-					invalidRetType = TRUE;
-				}
-			}
-			if (invalidRetType) {
+			if (info->bytes[info->slot1 - 1] != 'V') {
 				Trc_STV_j9bcv_verifyClassStructure_MethodError(J9NLS_CFR_ERR_BC_METHOD_INVALID_SIG__ID, i);
 				buildMethodError((J9CfrError *)segment, errorType, CFR_ThrowClassFormatError, (I_32) i, 0, method, classfile->constantPool);
 				result = -1;
