@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2019, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2019
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "codegen/ARM64JNILinkage.hpp"
@@ -78,7 +78,7 @@ void J9::ARM64::JNILinkage::releaseVMAccess(TR::Node *callNode, TR::Register *vm
    //    addimmx scratch0, vmThreadReg, #publicFlagsOffset
    //    movzx   scratch1, constReleaseVMAccessOutOfLineMask
    //
-   //    dmb ishst
+   //    dmb ish
    // loopHead:
    //    ldxrx   scratch2, [scratch0]
    //    tst     scratch2, scratch1
@@ -93,11 +93,11 @@ void J9::ARM64::JNILinkage::releaseVMAccess(TR::Node *callNode, TR::Register *vm
 
    generateTrg1Src1ImmInstruction(cg(), TR::InstOpCode::addimmx, callNode, scratchReg0, vmThreadReg, fej9->thisThreadGetPublicFlagsOffset());
    loadConstant64(cg(), callNode, fej9->constReleaseVMAccessOutOfLineMask(), scratchReg1);
-   // dmb ishst (Inner Shareable store barrier)
+   // dmb ish (Inner Shareable full barrier)
    // Arm Architecture Reference Manual states:
    // "This architecture assumes that all PEs that use the same operating system or hypervisor are in the same Inner Shareable shareability domain"
    // thus, inner shareable dmb suffices
-   generateSynchronizationInstruction(cg(), TR::InstOpCode::dmb, callNode, 0xb);
+   generateSynchronizationInstruction(cg(), TR::InstOpCode::dmb, callNode, TR::InstOpCode::ish);
 
    TR::LabelSymbol *loopHead = generateLabelSymbol(cg());
    generateLabelInstruction(cg(), TR::InstOpCode::label, callNode, loopHead);
@@ -165,7 +165,7 @@ void J9::ARM64::JNILinkage::acquireVMAccess(TR::Node *callNode, TR::Register *vm
    generateTrg1MemSrc1Instruction(cg(), TR::InstOpCode::stxrx, callNode, scratchReg2, TR::MemoryReference::createWithDisplacement(cg(), scratchReg0, 0), scratchReg1);
    generateCompareBranchInstruction(cg(), TR::InstOpCode::cbnzx, callNode, scratchReg2, loopHead);
    // dmb ishld (Inner Shareable load barrier)
-   generateSynchronizationInstruction(cg(), TR::InstOpCode::dmb, callNode, 0x9);
+   generateSynchronizationInstruction(cg(), TR::InstOpCode::dmb, callNode, TR::InstOpCode::ishld);
 
    generateLabelInstruction(cg(), TR::InstOpCode::label, callNode, reacquireVMAccessRestartLabel);
    }
@@ -230,7 +230,7 @@ void J9::ARM64::JNILinkage::acquireVMAccessAtomicFree(TR::Node *callNode, TR::Re
    static_assert(static_cast<uint64_t>(J9_PUBLIC_FLAGS_VM_ACCESS) < (1 << 12), "J9_PUBLIC_FLAGS_VM_ACCESS must fit in immediate");
    auto strInNativeInstr = generateMemSrc1Instruction(cg(), TR::InstOpCode::strimmx, callNode, TR::MemoryReference::createWithDisplacement(cg(), vmThreadReg, offsetof(J9VMThread, inNative)), zeroReg);
 #ifndef J9VM_INTERP_ATOMIC_FREE_JNI_USES_FLUSH
-   generateSynchronizationInstruction(cg(), TR::InstOpCode::dmb, callNode, 0xb);
+   generateSynchronizationInstruction(cg(), TR::InstOpCode::dmb, callNode, TR::InstOpCode::ish);
 #endif
    auto ldrPubliFlagsInstr = generateTrg1MemInstruction(cg(), TR::InstOpCode::ldrimmx, callNode, scratchReg0, TR::MemoryReference::createWithDisplacement(cg(), vmThreadReg, fej9->thisThreadGetPublicFlagsOffset()));
    if (debugObj)
@@ -258,6 +258,16 @@ void J9::ARM64::JNILinkage::buildJNICallOutFrame(TR::Node *callNode, bool isWrap
    {
    TR_J9VMBase *fej9 = reinterpret_cast<TR_J9VMBase *>(fe());
 
+#if JAVA_SPEC_VERSION >= 19
+   /**
+     * For virtual threads, bump the callOutCounter.  It is safe and most efficient to
+     * do this unconditionally.  No need to check for overflow.
+     */
+   generateTrg1MemInstruction(cg(), TR::InstOpCode::ldrimmx, callNode, scratchReg1, TR::MemoryReference::createWithDisplacement(cg(), vmThreadReg, fej9->thisThreadGetCallOutCountOffset()));
+   generateTrg1Src1ImmInstruction(cg(), TR::InstOpCode::addimmx, callNode, scratchReg1, scratchReg1, 1);
+   generateMemSrc1Instruction(cg(), TR::InstOpCode::strimmx, callNode, TR::MemoryReference::createWithDisplacement(cg(), vmThreadReg, fej9->thisThreadGetCallOutCountOffset()), scratchReg1);
+#endif
+
    // begin: mask out the magic bit that indicates JIT frames below
    loadConstant64(cg(), callNode, 0, scratchReg1);
    generateMemSrc1Instruction(cg(), TR::InstOpCode::strimmx, callNode, TR::MemoryReference::createWithDisplacement(cg(), vmThreadReg, fej9->thisThreadGetJavaFrameFlagsOffset()), scratchReg1);
@@ -344,6 +354,16 @@ void J9::ARM64::JNILinkage::restoreJNICallOutFrame(TR::Node *callNode, bool tear
    {
    TR_J9VMBase *fej9 = reinterpret_cast<TR_J9VMBase *>(fe());
 
+#if JAVA_SPEC_VERSION >= 19
+   /**
+     * For virtual threads, decrement the callOutCounter.  It is safe and most efficient to
+     * do this unconditionally.  No need to check for underflow.
+     */
+   generateTrg1MemInstruction(cg(), TR::InstOpCode::ldrimmx, callNode, scratchReg, TR::MemoryReference::createWithDisplacement(cg(), vmThreadReg, fej9->thisThreadGetCallOutCountOffset()));
+   generateTrg1Src1ImmInstruction(cg(), TR::InstOpCode::subimmx, callNode, scratchReg, scratchReg, 1);
+   generateMemSrc1Instruction(cg(), TR::InstOpCode::strimmx, callNode, TR::MemoryReference::createWithDisplacement(cg(), vmThreadReg, fej9->thisThreadGetCallOutCountOffset()), scratchReg);
+#endif
+
    // restore stack pointer: need to deal with growable stack -- stack may already be moved.
    generateTrg1MemInstruction(cg(), TR::InstOpCode::ldrimmx, callNode, scratchReg, TR::MemoryReference::createWithDisplacement(cg(), vmThreadReg, fej9->thisThreadGetJavaLiteralsOffset()));
    generateTrg1MemInstruction(cg(),TR::InstOpCode::ldrimmx, callNode, javaStackReg, TR::MemoryReference::createWithDisplacement(cg(), vmThreadReg, fej9->thisThreadGetJavaSPOffset()));
@@ -375,7 +395,6 @@ size_t J9::ARM64::JNILinkage::buildJNIArgs(TR::Node *callNode, TR::RegisterDepen
    {
    const TR::ARM64LinkageProperties &properties = _systemLinkage->getProperties();
    TR::ARM64MemoryArgument *pushToMemory = NULL;
-   TR::Register *argMemReg;
    TR::Register *tempReg;
    int32_t argIndex = 0;
    int32_t numMemArgs = 0;
@@ -483,12 +502,18 @@ size_t J9::ARM64::JNILinkage::buildJNIArgs(TR::Node *callNode, TR::RegisterDepen
    if (numMemArgs > 0)
       {
       pushToMemory = new (trStackMemory()) TR::ARM64MemoryArgument[numMemArgs];
-
-      argMemReg = cg()->allocateRegister();
       }
 
    // align to 16-byte boundary
    totalSize = (totalSize + 15) & (~15);
+   /*
+    * We have J9_INLINE_JNI_MAX_ARG_COUNT (32) slots in interpreter frame.
+    * If the size of the native stack used for passing parameters exceeds this size,
+    * it will overwrite preserved registers.
+    * The number of parameters is guaranteed to be <= J9_INLINE_JNI_MAX_ARG_COUNT
+    * when HasFixedFrameC_CallingConvention codegen flag is set.
+    */
+   TR_ASSERT_FATAL(totalSize <= sizeof(UDATA) * J9_INLINE_JNI_MAX_ARG_COUNT, "totalSize must not be larger than J9_INLINE_JNI_MAX_ARG_COUNT slots");
 
    numIntegerArgs = 0;
    numFloatArgs = 0;
@@ -502,6 +527,7 @@ size_t J9::ARM64::JNILinkage::buildJNIArgs(TR::Node *callNode, TR::RegisterDepen
 #if defined(OSX)
    sigCursor = (char *)sig;
 #endif
+   TR::RealRegister *sp = cg()->machine()->getRealRegister(properties.getStackPointerRegister());
 
    for (i = firstArgumentChild; i < callNode->getNumChildren(); i++)
       {
@@ -529,17 +555,17 @@ size_t J9::ARM64::JNILinkage::buildJNIArgs(TR::Node *callNode, TR::RegisterDepen
             else
                argRegister = pushIntegerWordArg(child);
 
+            if (checkSplit && !cg()->canClobberNodesRegister(child, 0))
+               {
+               if (argRegister->containsCollectedReference())
+                  tempReg = cg()->allocateCollectedReferenceRegister();
+               else
+                  tempReg = cg()->allocateRegister();
+               generateMovInstruction(cg(), callNode, tempReg, argRegister);
+               argRegister = tempReg;
+               }
             if (numIntegerArgs < properties.getNumIntArgRegs())
                {
-               if (checkSplit && !cg()->canClobberNodesRegister(child, 0))
-                  {
-                  if (argRegister->containsCollectedReference())
-                     tempReg = cg()->allocateCollectedReferenceRegister();
-                  else
-                     tempReg = cg()->allocateRegister();
-                  generateMovInstruction(cg(), callNode, tempReg, argRegister);
-                  argRegister = tempReg;
-                  }
                if (numIntegerArgs == 0 &&
                   (resType.isAddress() || resType.isInt32() || resType.isInt64()))
                   {
@@ -591,7 +617,7 @@ size_t J9::ARM64::JNILinkage::buildJNIArgs(TR::Node *callNode, TR::RegisterDepen
 #else
 #error Unsupported platform
 #endif
-               getOutgoingArgumentMemRef(argMemReg, argOffset, argRegister, op, pushToMemory[argIndex++]);
+               getOutgoingArgumentMemRef(sp, argOffset, argRegister, op, pushToMemory[argIndex++]);
                argOffset += offsetInc;
                }
             numIntegerArgs++;
@@ -604,15 +630,15 @@ size_t J9::ARM64::JNILinkage::buildJNIArgs(TR::Node *callNode, TR::RegisterDepen
             else
                argRegister = pushDoubleArg(child);
 
+            if (!cg()->canClobberNodesRegister(child, 0))
+               {
+               tempReg = cg()->allocateRegister(TR_FPR);
+               op = (childType == TR::Float) ? TR::InstOpCode::fmovs : TR::InstOpCode::fmovd;
+               generateTrg1Src1Instruction(cg(), op, callNode, tempReg, argRegister);
+               argRegister = tempReg;
+               }
             if (numFloatArgs < properties.getNumFloatArgRegs())
                {
-               if (!cg()->canClobberNodesRegister(child, 0))
-                  {
-                  tempReg = cg()->allocateRegister(TR_FPR);
-                  op = (childType == TR::Float) ? TR::InstOpCode::fmovs : TR::InstOpCode::fmovd;
-                  generateTrg1Src1Instruction(cg(), op, callNode, tempReg, argRegister);
-                  argRegister = tempReg;
-                  }
                if ((numFloatArgs == 0 && resType.isFloatingPoint()))
                   {
                   TR::Register *resultReg;
@@ -652,7 +678,7 @@ size_t J9::ARM64::JNILinkage::buildJNIArgs(TR::Node *callNode, TR::RegisterDepen
 #error Unsupported platform
 #endif
                   }
-               getOutgoingArgumentMemRef(argMemReg, argOffset, argRegister, op, pushToMemory[argIndex++]);
+               getOutgoingArgumentMemRef(sp, argOffset, argRegister, op, pushToMemory[argIndex++]);
                argOffset += offsetInc;
                }
             numFloatArgs++;
@@ -726,17 +752,12 @@ size_t J9::ARM64::JNILinkage::buildJNIArgs(TR::Node *callNode, TR::RegisterDepen
 
    if (numMemArgs > 0)
       {
-      TR::RealRegister *sp = cg()->machine()->getRealRegister(properties.getStackPointerRegister());
-      generateTrg1Src1ImmInstruction(cg(), TR::InstOpCode::subimmx, callNode, argMemReg, sp, totalSize);
-
       for (argIndex = 0; argIndex < numMemArgs; argIndex++)
          {
          TR::Register *aReg = pushToMemory[argIndex].argRegister;
          generateMemSrc1Instruction(cg(), pushToMemory[argIndex].opCode, callNode, pushToMemory[argIndex].argMemory, aReg);
          cg()->stopUsingRegister(aReg);
          }
-
-      cg()->stopUsingRegister(argMemReg);
       }
 
    return totalSize;
@@ -1038,20 +1059,7 @@ TR::Register *J9::ARM64::JNILinkage::buildDirectDispatch(TR::Node *callNode)
 #endif
    TR::RegisterDependencyConditions *deps = new (trHeapMemory()) TR::RegisterDependencyConditions(maxRegisters, maxPostRegisters, trMemory());
 
-   size_t spSize = buildJNIArgs(callNode, deps, passThread, passReceiver, killNonVolatileGPRs);
-   TR::RealRegister *sp = machine()->getRealRegister(_systemLinkage->getProperties().getStackPointerRegister());
-
-   if (spSize > 0)
-      {
-      if (constantIsUnsignedImm12(spSize))
-         {
-         generateTrg1Src1ImmInstruction(cg(), TR::InstOpCode::subimmx, callNode, sp, sp, spSize);
-         }
-      else
-         {
-         TR_ASSERT_FATAL(false, "Too many arguments.");
-         }
-      }
+   buildJNIArgs(callNode, deps, passThread, passReceiver, killNonVolatileGPRs);
 
    TR::Register *returnRegister = getReturnRegisterFromDeps(callNode, deps);
 
@@ -1091,18 +1099,6 @@ TR::Register *J9::ARM64::JNILinkage::buildDirectDispatch(TR::Node *callNode)
    TR::Instruction *callInstr = generateMethodDispatch(callNode, isJNIGCPoint, deps, targetAddress, x9Reg);
    generateLabelInstruction(cg(), TR::InstOpCode::label, callNode, returnLabel, callInstr);
 
-   if (spSize > 0)
-      {
-      if (constantIsUnsignedImm12(spSize))
-         {
-         generateTrg1Src1ImmInstruction(cg(), TR::InstOpCode::addimmx, callNode, sp, sp, spSize);
-         }
-      else
-         {
-         TR_ASSERT_FATAL(false, "Too many arguments.");
-         }
-      }
-
    if (dropVMAccess)
       {
 #ifdef J9VM_INTERP_ATOMIC_FREE_JNI
