@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef J9_RECOGNIZEDMETHODS_ENUM_INCL
@@ -58,12 +58,17 @@
    java_lang_Class_newInstancePrototype,
    java_lang_Class_isArray,
    java_lang_Class_isPrimitive,
+   java_lang_Class_isValue,
+   java_lang_Class_isIdentity,
    java_lang_Class_getComponentType,
    java_lang_Class_getModifiersImpl,
+   java_lang_Class_getStackClass,
    java_lang_Class_getSuperclass,
    java_lang_Class_isAssignableFrom,
    java_lang_Class_isInstance,
    java_lang_Class_isInterface,
+   java_lang_Class_cast,
+   java_lang_Class_all,
    java_lang_ClassLoader_callerClassLoader,
    java_lang_ClassLoader_getCallerClassLoader,
    java_lang_ClassLoader_getStackClassLoader,
@@ -118,15 +123,22 @@
    java_lang_Math_tanh,
    java_lang_Math_fma_D,
    java_lang_Math_fma_F,
+   java_lang_Math_multiplyHigh,
    java_lang_Object_init,
    java_lang_Object_getClass,
    java_lang_Object_clone,
    java_lang_Object_newInstancePrototype,
    java_lang_Object_getAddressAsPrimitive,
+   java_lang_Object_hashCode,
    java_lang_ref_Reference_getImpl,
    java_lang_ref_Reference_reachabilityFence,
+   java_lang_ref_Reference_refersTo,
    java_lang_ref_SoftReference_get,
    java_lang_Thread_currentThread,
+   java_lang_Thread_onSpinWait,
+   java_lang_Thread_runWith,
+   java_lang_VirtualThread_runWith,
+   java_lang_ScopedValue_runWith,
    java_lang_System_arraycopy,
    java_lang_System_currentTimeMillis,
    java_lang_System_nanoTime,
@@ -145,15 +157,26 @@
    java_lang_Float_init,
    java_lang_Double_init,
    java_lang_String_init,
-
    // When adding a new recognized constructor for one of the immutable classes listed above you must also
    // properly alias the recognized constructor in TR::SymbolReferenceTable::immutableConstructorId. Otherwise
    // the JIT may consider the new constructor as side-effect free which will result in invalid transformations.
 
+   // Unboxing methods
+   java_lang_Boolean_booleanValue,
+   java_lang_Byte_byteValue,
+   java_lang_Character_charValue,
+   java_lang_Short_shortValue,
+   java_lang_Integer_intValue,
+   java_lang_Long_longValue,
+   java_lang_Float_floatValue,
+   java_lang_Double_doubleValue,
+
    java_lang_String_init_String_char,
    java_lang_String_init_String,
    java_lang_String_init_int_String_int_String_String,
    java_lang_String_init_int_int_char_boolean,
+   java_lang_String_init_StringBuilder,
+   java_lang_String_init_AbstractStringBuilder_Void,
 
    java_lang_String_trim,
    java_lang_String_charAt,
@@ -167,7 +190,6 @@
    java_lang_String_decompressedArrayCopy_BICII,
    java_lang_String_decompressedArrayCopy_CIBII,
    java_lang_String_decompressedArrayCopy_CICII,
-   java_lang_StringLatin1_inflate,
    java_lang_String_concat,
    java_lang_String_length,
    java_lang_String_lengthInternal,
@@ -197,9 +219,6 @@
    java_lang_String_equalsIgnoreCase,
    java_lang_String_encodeASCII,
    java_lang_String_compareToIgnoreCase,
-   java_lang_String_compress,
-   java_lang_String_andOR,
-   java_lang_String_compressNoCheck,
    java_lang_String_unsafeCharAt,
    java_lang_String_split_str_int,
    java_lang_String_getChars_charArray,
@@ -212,6 +231,9 @@
    java_lang_String_startsWith,
 
    java_lang_StringLatin1_indexOf,
+   java_lang_StringLatin1_indexOfChar,
+   java_lang_StringLatin1_inflate_BICII,
+   java_lang_StringLatin1_inflate_BIBII,
 
    java_lang_StringUTF16_charAt,
    java_lang_StringUTF16_checkIndex,
@@ -220,6 +242,8 @@
    java_lang_StringUTF16_compareValues,
    java_lang_StringUTF16_getChar,
    java_lang_StringUTF16_indexOf,
+   java_lang_StringUTF16_indexOfCharUnsafe,
+   java_lang_StringUTF16_indexOfUnsafe,
    java_lang_StringUTF16_length,
    java_lang_StringUTF16_newBytesFor,
    java_lang_StringUTF16_putChar,
@@ -273,38 +297,26 @@
 
    java_nio_ByteOrder_nativeOrder,
 
-   // routines from the memory Fence library.  See JIT design 1598
-   java_util_concurrent_atomic_Fences_postLoadFence,
-   java_util_concurrent_atomic_Fences_preStoreFence,
-   java_util_concurrent_atomic_Fences_postStorePreLoadFence,
-   // same 3 with java.lang.Object parm
-   java_util_concurrent_atomic_Fences_postLoadFence_jlObject,
-   java_util_concurrent_atomic_Fences_preStoreFence_jlObject,
-   java_util_concurrent_atomic_Fences_postStorePreLoadFence_jlObject,
-   // as above, with java.lang.reflect.Field parm
-   java_util_concurrent_atomic_Fences_postLoadFence_jlObjectjlrField,
-   java_util_concurrent_atomic_Fences_preStoreFence_jlObjectjlrField,
-   java_util_concurrent_atomic_Fences_postStorePreLoadFence_jlObjectjlrField,
-   // for array element
-   java_util_concurrent_atomic_Fences_postLoadFence_jlObjectI,
-   java_util_concurrent_atomic_Fences_preStoreFence_jlObjectI,
-   java_util_concurrent_atomic_Fences_postStorePreLoadFence_jlObjectI,
-
-   // JSR 166y
-   //
-   java_util_concurrent_atomic_Fences_orderAccesses,
-   java_util_concurrent_atomic_Fences_orderReads,
-   java_util_concurrent_atomic_Fences_orderWrites,
-   java_util_concurrent_atomic_Fences_reachabilityFence,
-
    java_util_regex_Matcher_init,
    java_util_regex_Matcher_usePattern,
 
-   java_util_HashMap_all,
+   java_util_AbstractCollection_all,
+   java_util_ArrayDeque_all,
    java_util_ArrayList_all,
-   java_util_Hashtable_all,
+   java_util_ComparableTimSort_all,
    java_util_concurrent_ConcurrentHashMap_all,
+   java_util_HashMap_all,
+   java_util_Hashtable_all,
+   java_util_IdentityHashMap_all,
+   java_util_ImmutableCollections_all,
+   java_util_LinkedHashMap_all,
+   java_util_LinkedList_all,
+   java_util_Map_all,
+   java_util_regex_Pattern_all,
+   java_util_stream_Nodes_all,
+   java_util_TimSort_all,
    java_util_Vector_all,
+   java_util_WeakHashMap_all,
 
    java_util_Hashtable_get,
    java_util_Hashtable_put,
@@ -332,7 +344,10 @@
    java_util_HashMap_findNullKeyEntry,
    java_util_HashMap_get,
    java_util_HashMap_getNode,
+   java_util_HashMap_getNode_Object,
    java_util_HashMap_findNonNullKeyEntry,
+   java_util_HashMap_hash,
+   java_util_HashMap_put,
    java_util_HashMap_putImpl,
    java_util_HashMap_resize,
    java_util_HashMap_prepareArray,
@@ -342,13 +357,14 @@
    java_util_HashMapHashIterator_init,
    java_util_zip_CRC32_update,
    java_util_zip_CRC32_updateBytes,
+   java_util_zip_CRC32_updateBytes0,
    java_util_zip_CRC32_updateByteBuffer,
+   java_util_zip_CRC32_updateByteBuffer0,
+   java_util_zip_CRC32C_updateBytes,
+   java_util_zip_CRC32C_updateDirectByteBuffer,
    sun_misc_Unsafe_compareAndSwapInt_jlObjectJII_Z,
    sun_misc_Unsafe_compareAndSwapLong_jlObjectJJJ_Z,
    sun_misc_Unsafe_compareAndSwapObject_jlObjectJjlObjectjlObject_Z,
-   sun_misc_Unsafe_compareAndExchangeInt_jlObjectJII_Z,
-   sun_misc_Unsafe_compareAndExchangeLong_jlObjectJJJ_Z,
-   sun_misc_Unsafe_compareAndExchangeObject_jlObjectJjlObjectjlObject_Z,
 
    sun_misc_Unsafe_putBoolean_jlObjectJZ_V,
    sun_misc_Unsafe_putByte_jlObjectJB_V,
@@ -441,29 +457,93 @@
 
    sun_misc_Unsafe_ensureClassInitialized,
    sun_misc_Unsafe_allocateInstance,
+   sun_misc_Unsafe_allocateUninitializedArray0,
+
+   jdk_internal_misc_Unsafe_getBooleanAcquire_jlObjectJ_Z,
+   jdk_internal_misc_Unsafe_getByteAcquire_jlObjectJ_B,
+   jdk_internal_misc_Unsafe_getCharAcquire_jlObjectJ_C,
+   jdk_internal_misc_Unsafe_getShortAcquire_jlObjectJ_S,
+   jdk_internal_misc_Unsafe_getIntAcquire_jlObjectJ_I,
+   jdk_internal_misc_Unsafe_getLongAcquire_jlObjectJ_J,
+   jdk_internal_misc_Unsafe_getFloatAcquire_jlObjectJ_F,
+   jdk_internal_misc_Unsafe_getDoubleAcquire_jlObjectJ_D,
+   jdk_internal_misc_Unsafe_getReferenceAcquire_jlObjectJ_jlObject,
+
+   jdk_internal_misc_Unsafe_getBooleanOpaque_jlObjectJ_Z,
+   jdk_internal_misc_Unsafe_getByteOpaque_jlObjectJ_B,
+   jdk_internal_misc_Unsafe_getCharOpaque_jlObjectJ_C,
+   jdk_internal_misc_Unsafe_getShortOpaque_jlObjectJ_S,
+   jdk_internal_misc_Unsafe_getIntOpaque_jlObjectJ_I,
+   jdk_internal_misc_Unsafe_getLongOpaque_jlObjectJ_J,
+   jdk_internal_misc_Unsafe_getFloatOpaque_jlObjectJ_F,
+   jdk_internal_misc_Unsafe_getDoubleOpaque_jlObjectJ_D,
+   jdk_internal_misc_Unsafe_getReferenceOpaque_jlObjectJ_jlObject,
+
+   jdk_internal_misc_Unsafe_putBooleanOpaque_jlObjectJZ_V,
+   jdk_internal_misc_Unsafe_putByteOpaque_jlObjectJB_V,
+   jdk_internal_misc_Unsafe_putCharOpaque_jlObjectJC_V,
+   jdk_internal_misc_Unsafe_putShortOpaque_jlObjectJS_V,
+   jdk_internal_misc_Unsafe_putIntOpaque_jlObjectJI_V,
+   jdk_internal_misc_Unsafe_putLongOpaque_jlObjectJJ_V,
+   jdk_internal_misc_Unsafe_putFloatOpaque_jlObjectJF_V,
+   jdk_internal_misc_Unsafe_putDoubleOpaque_jlObjectJD_V,
+   jdk_internal_misc_Unsafe_putReferenceOpaque_jlObjectJjlObject_V,
+
+   jdk_internal_misc_Unsafe_compareAndExchangeInt,
+   jdk_internal_misc_Unsafe_compareAndExchangeLong,
+   jdk_internal_misc_Unsafe_compareAndExchangeObject,
+   jdk_internal_misc_Unsafe_compareAndExchangeReference,
 
    jdk_internal_misc_Unsafe_copyMemory0,
+   jdk_internal_misc_Unsafe_getCharUnaligned,
+   jdk_internal_misc_Unsafe_getShortUnaligned,
+   jdk_internal_misc_Unsafe_getIntUnaligned,
+   jdk_internal_misc_Unsafe_getLongUnaligned,
+   jdk_internal_misc_Unsafe_putCharUnaligned,
+   jdk_internal_misc_Unsafe_putShortUnaligned,
+   jdk_internal_misc_Unsafe_putIntUnaligned,
+   jdk_internal_misc_Unsafe_putLongUnaligned,
    jdk_internal_loader_NativeLibraries_load,
+   jdk_internal_util_ArraysSupport_vectorizedMismatch,
+   jdk_internal_util_ArraysSupport_vectorizedHashCode,
+   jdk_internal_util_Preconditions_checkIndex,
 
    FirstVectorMethod,
    jdk_internal_vm_vector_VectorSupport_load = FirstVectorMethod,
    jdk_internal_vm_vector_VectorSupport_store,
    jdk_internal_vm_vector_VectorSupport_binaryOp,
    jdk_internal_vm_vector_VectorSupport_blend,
-   jdk_internal_vm_vector_VectorSupport_broadcastCoerced,
+   jdk_internal_vm_vector_VectorSupport_broadcastInt,
    jdk_internal_vm_vector_VectorSupport_compare,
+   jdk_internal_vm_vector_VectorSupport_compressExpandOp,
+   jdk_internal_vm_vector_VectorSupport_convert,
+   jdk_internal_vm_vector_VectorSupport_extract,
+   jdk_internal_vm_vector_VectorSupport_fromBitsCoerced,
+   jdk_internal_vm_vector_VectorSupport_indexPartiallyInUpperRange,
+   jdk_internal_vm_vector_VectorSupport_indexVector,
+   jdk_internal_vm_vector_VectorSupport_insert,
+   jdk_internal_vm_vector_VectorSupport_loadMasked,
+   jdk_internal_vm_vector_VectorSupport_loadWithMap,
+   jdk_internal_vm_vector_VectorSupport_maskReductionCoerced,
+   jdk_internal_vm_vector_VectorSupport_rearrangeOp,
+   jdk_internal_vm_vector_VectorSupport_reductionCoerced,
+   jdk_internal_vm_vector_VectorSupport_selectFromOp,
+   jdk_internal_vm_vector_VectorSupport_selectFromTwoVectorOp,
+   jdk_internal_vm_vector_VectorSupport_shuffleIota,
+   jdk_internal_vm_vector_VectorSupport_shuffleToVector,
+   jdk_internal_vm_vector_VectorSupport_storeMasked,
+   jdk_internal_vm_vector_VectorSupport_storeWithMap,
    jdk_internal_vm_vector_VectorSupport_ternaryOp,
+   jdk_internal_vm_vector_VectorSupport_test,
    jdk_internal_vm_vector_VectorSupport_unaryOp,
-   LastVectorIntrinsicMethod = jdk_internal_vm_vector_VectorSupport_unaryOp,
-   jdk_incubator_vector_FloatVector_fromArray,
-   jdk_incubator_vector_FloatVector_intoArray,
-   jdk_incubator_vector_FloatVector_fromArray_mask,
-   jdk_incubator_vector_FloatVector_intoArray_mask,
-   jdk_incubator_vector_FloatVector_add,
-   jdk_incubator_vector_VectorSpecies_indexInRange,
-   LastVectorMethod = jdk_incubator_vector_VectorSpecies_indexInRange,
+   jdk_internal_vm_vector_VectorSupport_wrapShuffleIndexes,
+   LastVectorIntrinsicMethod = jdk_internal_vm_vector_VectorSupport_wrapShuffleIndexes,
+   LastVectorMethod = LastVectorIntrinsicMethod,
 
    java_lang_reflect_Array_getLength,
+   jdk_internal_value_ValueClass_newArrayInstance,
+   jdk_internal_value_ValueClass_newNullRestrictedArray,
+   jdk_internal_value_NullRestrictedCheckedType_of,
    java_lang_reflect_Method_invoke,
    java_util_Arrays_fill,
    java_util_Arrays_equals,
@@ -482,9 +562,11 @@
    sun_nio_cs_UTF_8_Decoder_decodeUTF_8,
    sun_nio_cs_UTF_8_Encoder_encodeUTF_8,
    sun_nio_cs_ext_IBM1388_Encoder_encodeArrayLoop,
+   sun_nio_cs_SingleByteDecoder_decodeToLatin1Impl,
+   sun_nio_cs_SingleByteEncoder_encodeFromLatin1Impl,
 
-   sun_nio_cs_UTF_16_Encoder_encodeUTF16Big,
-   sun_nio_cs_UTF_16_Encoder_encodeUTF16Little,
+   sun_nio_cs_UTF16_Encoder_encodeUTF16Big,
+   sun_nio_cs_UTF16_Encoder_encodeUTF16Little,
    com_ibm_jit_JITHelpers_transformedEncodeUTF16Big,
    com_ibm_jit_JITHelpers_transformedEncodeUTF16Little,
 
@@ -496,6 +578,8 @@
    java_lang_Integer_reverseBytes,
    java_lang_Integer_rotateLeft,
    java_lang_Integer_rotateRight,
+   java_lang_Integer_compress,
+   java_lang_Integer_expand,
    java_lang_Integer_valueOf,
    java_lang_Integer_toUnsignedLong,
    java_lang_Integer_stringSize,
@@ -513,6 +597,8 @@
    java_lang_Long_reverseBytes,
    java_lang_Long_rotateLeft,
    java_lang_Long_rotateRight,
+   java_lang_Long_compress,
+   java_lang_Long_expand,
    java_lang_Short_reverseBytes,
    java_lang_Long_stringSize,
    java_lang_Long_toString,
@@ -689,6 +775,12 @@
    com_ibm_dataaccess_PackedDecimal_movePackedDecimal_,
    com_ibm_dataaccess_PackedDecimal_checkPackedDecimal_,
 
+   // wrapper methods
+   com_ibm_dataaccess_ExternalDecimal_checkExternalDecimal,
+
+   //inline methods
+   com_ibm_dataaccess_ExternalDecimal_checkExternalDecimal_,
+
    com_ibm_Compiler_Internal__TR_Prefetch,
 
    com_ibm_Compiler_Internal_Quad_enableQuadOptimization,
@@ -817,6 +909,12 @@
    java_math_BigInteger_add,
    java_math_BigInteger_subtract,
    java_math_BigInteger_multiply,
+   java_math_BigInteger_init_long,
+   java_math_BigInteger_toByteArray,
+   java_math_BigInteger_stripLeadingZeroBytes1,
+   java_math_BigInteger_stripLeadingZeroBytes2,
+   java_math_BigInteger_bitCount,
+   java_math_BigInteger_bitLength,
 
    java_text_NumberFormat_format,
 
@@ -943,6 +1041,7 @@
    java_util_concurrent_ConcurrentHashMap_tabAt,
    java_util_concurrent_ConcurrentHashMap_casTabAt,
    java_util_concurrent_ConcurrentHashMap_setTabAt,
+   java_util_concurrent_ConcurrentHashMap_get,
 
    java_util_concurrent_ConcurrentHashMap_TreeBin_lockRoot,
    java_util_concurrent_ConcurrentHashMap_TreeBin_contendedLock,
@@ -1026,6 +1125,7 @@
    java_lang_invoke_Invokers_checkCustomized,
    java_lang_invoke_Invokers_checkExactType,
    java_lang_invoke_Invokers_getCallSiteTarget,
+   java_lang_invoke_Invokers_checkVarHandleGenericType,
    java_lang_invoke_MethodHandle_doCustomizationLogic,
    java_lang_invoke_MethodHandle_asType,
    java_lang_invoke_MethodHandle_asType_instance,
@@ -1037,11 +1137,41 @@
    java_lang_invoke_MethodHandle_linkToSpecial,
    java_lang_invoke_MethodHandle_linkToVirtual,
    java_lang_invoke_MethodHandle_linkToInterface,
+   java_lang_invoke_MethodHandle_linkToNative,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_getElementI,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_getElementJ,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_getElementF,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_getElementD,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_getElementZ,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_getElementB,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_getElementS,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_getElementC,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_getElementL,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_setElementI,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_setElementJ,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_setElementF,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_setElementD,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_setElementZ,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_setElementB,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_setElementS,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_setElementC,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_setElementL,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_lengthI,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_lengthJ,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_lengthF,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_lengthD,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_lengthZ,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_lengthB,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_lengthS,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_lengthC,
+   java_lang_invoke_MethodHandleImpl_ArrayAccessor_lengthL,
    java_lang_invoke_MethodHandleImpl_CountingWrapper_getTarget,
    java_lang_invoke_DelegatingMethodHandle_getTarget,
    java_lang_invoke_DirectMethodHandle_internalMemberName,
    java_lang_invoke_DirectMethodHandle_internalMemberNameEnsureInit,
    java_lang_invoke_DirectMethodHandle_constructorMethod,
+   java_lang_invoke_DirectMethodHandle_checkCast,
+   java_lang_invoke_DirectMethodHandle_Accessor_checkCast,
    java_lang_invoke_MethodHandles_getStackClass,
    java_lang_invoke_MethodHandle_type,
    java_lang_invoke_MethodHandle_undoCustomizationLogic,
@@ -1058,6 +1188,7 @@
    java_lang_invoke_FieldGetterHandle_invokeExact,
    java_lang_invoke_FieldSetterHandle_invokeExact,
    java_lang_invoke_FilterArgumentsHandle_invokeExact,
+   java_lang_invoke_VarHandle_asDirect,
    java_lang_invoke_VarHandle_get,
    java_lang_invoke_VarHandle_set,
    java_lang_invoke_VarHandle_getVolatile,
@@ -1100,6 +1231,17 @@
    java_lang_invoke_MethodHandleImpl_profileBoolean,
    java_lang_invoke_MethodHandleImpl_isCompileConstant,
 
+   java_lang_invoke_VarHandleX_Array_method,
+   java_lang_invoke_VarHandleX_FieldInstanceReadOnlyOrReadWrite_method,
+   java_lang_invoke_VarHandleX_FieldStaticReadOnlyOrReadWrite_method,
+
+   java_lang_invoke_VarHandleByteArrayAsX_ArrayHandle_method,
+   java_lang_invoke_VarHandleByteArrayAsX_ByteBufferHandle_method,
+
+   jdk_internal_foreign_layout_ValueLayouts_AbstractValueLayout_accessHandle,
+   jdk_internal_foreign_AbstractMemorySegmentImpl_reinterpret,
+   java_lang_foreign_MemorySegment_method,
+
    // Clone and Deep Copy
    java_lang_J9VMInternals_is32Bit,
    java_lang_J9VMInternals_isClassModifierPublic,
@@ -1121,9 +1263,10 @@
    java_lang_J9VMInternals_getInstanceDescriptionFromJ9Class64,
    java_lang_J9VMInternals_getDescriptionWordFromPtr64,
    java_lang_J9VMInternals_getSuperclass,
+   java_lang_J9VMInternals_primitiveClone,
+
    java_lang_J9VMInternals_identityHashCode,
    java_lang_J9VMInternals_fastIdentityHashCode,
-   java_lang_J9VMInternals_primitiveClone,
 
    java_util_GregorianCalendar_computeFields,
 
@@ -1145,7 +1288,10 @@
    java_lang_StringCoding_encode,
    java_lang_StringCoding_StringDecoder_decode,
    java_lang_StringCoding_StringEncoder_encode,
+   java_lang_StringCoding_hasNegatives,
+   java_lang_StringCoding_countPositives,
    java_lang_StringCoding_implEncodeISOArray,
+   java_lang_StringCoding_implEncodeAsciiArray,
    java_lang_StringCoding_encode8859_1,
    java_lang_StringCoding_encodeASCII,
    java_lang_StringCoding_encodeUTF8,
@@ -1171,6 +1317,8 @@
    java_util_Arrays_copyOfRange_Object1,
    java_util_Arrays_copyOfRange_Object2,
 
+   java_util_Arrays_copyOfRangeByte,
+
    sun_nio_ch_NativeThread_current,
 
    // Multi-Tenancy Tenant Context switch methods
