@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef COMPILATION_RUNTIME_HPP
@@ -40,6 +40,7 @@
 #include "control/rossa.h"
 #include "runtime/RelocationRuntime.hpp"
 #if defined(J9VM_OPT_JITSERVER)
+#include <vector>
 #include "control/JITServerHelpers.hpp"
 #include "env/PersistentCollections.hpp"
 #include "net/ServerStream.hpp"
@@ -80,6 +81,9 @@ class JITServerAOTCacheMap;
 class JITServerAOTDeserializer;
 class JITServerSharedROMClassCache;
 #endif /* defined(J9VM_OPT_JITSERVER) */
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+namespace TR { class CRRuntime; }
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
 
 struct TR_SignatureCountPair
 {
@@ -119,6 +123,7 @@ class TR_LowPriorityCompQueue
       bool createLowPriorityCompReqAndQueueIt(TR::IlGeneratorMethodDetails &details, void *startPC, uint8_t reason);
       bool addFirstTimeCompReqToLPQ(J9Method *j9method, uint8_t reason);
       bool addUpgradeReqToLPQ(TR_MethodToBeCompiled*, uint8_t reason = TR_MethodToBeCompiled::REASON_UPGRADE);
+      bool addUpgradeReqToLPQ(J9Method *j9method, void *startPC, uint8_t reason);
       int32_t getLowPriorityQueueSize() const { return _sizeLPQ; }
       int32_t getLPQWeight() const { return _LPQWeight; }
       void increaseLPQWeightBy(uint8_t weight) { _LPQWeight += (int32_t)weight; }
@@ -171,6 +176,10 @@ class TR_LowPriorityCompQueue
       uint32_t _STAT_LPQcompFromIprofiler; // first time compilations coming from LPQ
       uint32_t _STAT_LPQcompFromInterpreter;
       uint32_t _STAT_LPQcompUpgrade;
+#if defined(J9VM_OPT_JITSERVER)
+      uint32_t _STAT_compReqQueuedByJITServer;
+      uint32_t _STAT_LPQcompServerUnavailable;
+#endif /* defined(J9VM_OPT_JITSERVER) */
       // stats written by application threads
       uint32_t _STAT_compReqQueuedByInterpreter;
       uint32_t _STAT_numFailedToEnqueueInLPQ;
@@ -373,6 +382,8 @@ public:
       SAMPLE_THR_FAILED_TO_ATTACH,
       SAMPLE_THR_ATTACHED,
       SAMPLE_THR_INITIALIZED,
+      SAMPLE_THR_SUSPENDED,
+      SAMPLE_THR_RESUMING,
       SAMPLE_THR_STOPPING,
       SAMPLE_THR_DESTROYED,
       SAMPLE_THR_LAST_STATE // must be the last one
@@ -389,15 +400,6 @@ public:
       UNDEFINED_ACTION
       };
 
-#if defined(J9VM_OPT_CRIU_SUPPORT)
-   enum TR_CheckpointStatus
-      {
-      NO_CHECKPOINT_IN_PROGRESS,
-      CHECKPOINT_IN_PROGRESS,
-      INTERRUPT_CHECKPOINT
-      };
-#endif
-
    struct DLT_record
       {
       DLT_record         *_next;
@@ -451,12 +453,13 @@ public:
    static bool createCompilationInfo(J9JITConfig * jitConfig);
    static void freeCompilationInfo(J9JITConfig *jitConfig);
    static TR::CompilationInfo *get(J9JITConfig * = 0) { return _compilationRuntime; }
-   static bool shouldRetryCompilation(TR_MethodToBeCompiled *entry, TR::Compilation *comp);
+   static bool shouldRetryCompilation(J9VMThread *vmThread, TR_MethodToBeCompiled *entry, TR::Compilation *comp);
    static bool shouldAbortCompilation(TR_MethodToBeCompiled *entry, TR::PersistentInfo *persistentInfo);
    static bool canRelocateMethod(TR::Compilation * comp);
    static int computeCompilationThreadPriority(J9JavaVM *vm);
    static void *compilationEnd(J9VMThread *context, TR::IlGeneratorMethodDetails & details, J9JITConfig *jitConfig, void * startPC,
-                               void *oldStartPC, TR_FrontEnd *vm=0, TR_MethodToBeCompiled *entry=NULL, TR::Compilation *comp=NULL);
+                               void *oldStartPC, bool preventFutureMethodCountingOnFailure = true, TR_FrontEnd *vm=0,
+                               TR_MethodToBeCompiled *entry=NULL, TR::Compilation *comp=NULL);
 #if defined(J9VM_OPT_JITSERVER)
    static JITServer::ServerStream *getStream();
 #endif /* defined(J9VM_OPT_JITSERVER) */
@@ -700,24 +703,6 @@ public:
    void waitOnCompMonitor(J9VMThread *vmThread);
    intptr_t waitOnCompMonitorTimed(J9VMThread *vmThread, int64_t millis, int32_t nanos);
 
-#if defined(J9VM_OPT_CRIU_SUPPORT)
-   /* The CR Monitor (Checkpoint/Restore Monitor) must always be acquired with the Comp Monitor
-    * in hand. If waiting on the CR Monitor, the Comp Monitor should be released. After being
-    * notified, the CR Monitor should be released before re-acquiring the Comp Monitor.
-    */
-   TR::Monitor *getCRMonitor() { return _crMonitor; }
-   void acquireCRMonitor();
-   void releaseCRMonitor();
-   void waitOnCRMonitor();
-
-   /* The following APIs should only be invoked with the Comp Monitor in hand. */
-   bool isCheckpointInProgress()        { return _checkpointStatus != TR_CheckpointStatus::NO_CHECKPOINT_IN_PROGRESS; }
-   void setCheckpointInProgress()       {        _checkpointStatus  = TR_CheckpointStatus::CHECKPOINT_IN_PROGRESS;    }
-   void resetCheckpointInProgress()     {        _checkpointStatus  = TR_CheckpointStatus::NO_CHECKPOINT_IN_PROGRESS; }
-   bool shouldCheckpointBeInterrupted() { return _checkpointStatus == TR_CheckpointStatus::INTERRUPT_CHECKPOINT;      }
-   void interruptCheckpoint()           {        _checkpointStatus  = TR_CheckpointStatus::INTERRUPT_CHECKPOINT;      }
-#endif
-
    TR_PersistentMemory *     persistentMemory() { return _persistentMemory; }
 
    TR::PersistentInfo    * getPersistentInfo() { return persistentMemory()->getPersistentInfo(); }
@@ -756,7 +741,7 @@ public:
    // It makes sure the number of usable compilation threads is within allowed bounds.
    // If not, set it to the upper bound based on the mode: JITClient/non-JITServer or JITServer.
    void updateNumUsableCompThreads(int32_t &numUsableCompThreads);
-   bool allocateCompilationThreads(int32_t numUsableCompThreads);
+   bool allocateCompilationThreads(int32_t numCompThreads);
    void freeAllCompilationThreads();
    void freeAllResources();
 
@@ -789,134 +774,6 @@ public:
     */
    void resumeCompilationThread();
 
-#if defined(J9VM_OPT_CRIU_SUPPORT)
-   /**
-    * @brief Work that is necessary prior to taking a snapshot. This includes:
-    *        - Setting the _checkpointStatus state.
-    *        - Suspending all compilation threads.
-    *        - Waiting until all compilation threads are suspended.
-    *
-    * Normal Execution (steps 5&6 can be interchanged)
-    * ================================================
-    * 1. Hook Thread acquires Comp Monitor
-    * 2. Hook Thread signals Comp Threads to suspend
-    * 3. Hook Thread acquires CR Monitor, releases Comp Monitor, and waits on
-    *    CR Monitor
-    * 4. Comp Threads acquire Comp Monitor and CompThread Monitor, and change
-    *    state to COMPTHREAD_SUSPENDED
-    * 5. Comp Threads acquire CR Monitor, call notifyAll, and release CR
-    *    Monitor
-    * 6. Hook Thread wakes up with CR Monitor in hand, releases CR Monitor, and
-    *    blocks on Comp Monitor
-    * 7. Comp Threads release Comp Monitor and wait on CompThread Monitor
-    * 8. Hook Thread acquires Comp Monitor, ensures state has changed to
-    *    COMPTHREAD_SUSPENDED for the current compInfoPT
-    * 9. Hook Thread checks the next compInfoPT, waiting on the CR Monitor if
-    *    needed (it will release the Comp Monitor prior to waiting)
-    * 10. Hook Thread releases Comp Monitor, returns from the hook
-    *
-    *
-    * JIT Dump
-    * ========
-    * - Crash on application thread:
-    *    - Hook Thread running prepareForCheckpoint will run as normal
-    *       - Comp Threads will suspend themselves
-    *       - Hook Thread will wait till all Comp Threads are suspended
-    *       - Hook Thread will return from the jit hook
-    *       - VM will need to ensure it doesn't invoke criu API
-    *
-    * - Crash on compilation thread:
-    *    - Crashing Comp Thread will return to the VM and terminate process
-    *      (https://github.com/eclipse-openj9/openj9/blob/500e0a26e2c5be6ead0f838495ae8d8cc34821e1/runtime/compiler/control/CompilationThread.cpp#L3442-L3447)
-    *    - Possible scenarios for Hook Thread:
-    *       1. Hook Thread will try to acquire the Comp Monitor to signal Comp
-    *          Threads to suspend but will end up blocking until the JVM
-    *          terminates
-    *       2. Hook Thread will manage to signal Comp Threads to suspend, but
-    *          will remain waiting on the CR Monitor for the crashed Comp
-    *          Thread to suspend until the JVM terminates
-    *    - VM will need to ensure it doesn't invoke criu API
-    *
-    *
-    * Normal Shutdown
-    * ===============
-    * - Scenario 1
-    *    1. Hook Thread waits on CR Monitor
-    *    2. Comp Thread goes to suspend, already has Comp Monitor in hand
-    *    3. Shutdown Thread blocks on the Comp Monitor
-    *    4. Comp Thread updates state to COMPTHREAD_SUSPENDED, acquires CR
-    *       Monitor, and calls notifyAll
-    *    5. Comp Thread releases CR Monitor, releases Comp Monitor, and waits
-    *       on CompThread Monitor
-    *    6. Hook Thread wakes up with CR Monitor in hand, releases CR Monitor,
-    *       acquires Comp Monitor and continues on to next thread
-    *    7. Hook Thread checks state and moves onto the next threads possibly
-    *       finding them all suspended.
-    *    8. Hook Thread release Comp Monitor and returns from hook
-    *    9. Shutdown Thread goes through the sequence of stopping all threads
-    *
-    * - Scenario 2
-    *    Repeat steps 1-5 from Scenario 1
-    *    6. Shutdown Thread acquires Comp Monitor
-    *    7. Hook Thread wakes up with CR Monitor in hand, releases CR Monitor
-    *       and blocks on the Comp Monitor
-    *    8. Shutdown Thread sets checkpoint to be interrupted flag.
-    *       Additionally (though irrelevant in this scenario) it also
-    *       acquires the CR Monitor, calls notify, and releases CR Monitor.
-    *    9. Shutdown Thread finishes stopping all threads, releases Comp
-    *       Monitor
-    *    10. Hook Thread acquires Comp Monitor, sees that the checkpoint should
-    *        be interrupted
-    *    11. Hook Thread breaks out of the loops, releases Comp Monitor, and
-    *        returns from hook
-    *
-    * - Scenario 3
-    *    1. Hook Thread waits on CR Monitor
-    *    2. Shutdown Thread acquires Comp Monitor
-    *    3. Comp Thread blocks on the Comp Monitor in order to (eventually)
-    *       suspend itself
-    *    4. Shutdown Thread sets checkpoint to be interrupted flag, acquires CR
-    *       Monitor, calls notify, and releases CR Monitor
-    *    5. Shutdown Thread goes about the task of stopping Comp Threads
-    *    6. Hook Thread wakes up with CR Monitor in hand, releases CR Monitor
-    *       and blocks on the Comp Monitor
-    *    7. Shutdown Thread finishes stopping all threads, releases Comp
-    *       Monitor (steps 5 & 6 can be interchanged with the same following
-    *       steps)
-    *    8. Hook Thread acquires Comp Monitor, sees that the checkpoint should
-    *       be interrupted
-    *    9. Hook Thread breaks out of the loops, releases Comp Monitor, and
-    *       returns from hook
-    *
-    * It should be noted that when the Hook Thread runs prepareForCheckpoint,
-    * either it will succeed in signalling the Comp Threads to suspend, or it
-    * will wait indefinitely on the Comp Monitor (in the case of a JIT Dump) or
-    * it will abort after acquring the Comp Monitor if the Shutdown Thread
-    * already finished its task. Once the Hook Thread reaches the point where
-    * it waits on the CR Monitor, the scenarioes above can occur.
-    *
-    *
-    * Shutdown during JIT Dump
-    * ========================
-    * - Crash on application thread
-    *    - same as Normal Shutdown
-    *
-    * - Crash on compilation thread
-    *    - In all of the above scenarios:
-    *       - Hook Thread remains waiting on CR Monitor
-    *       - Shutdown Thread blocks on the Comp Monitor
-    *       - Comp Thread returns to VM and terminates the process
-    */
-   void prepareForCheckpoint();
-
-   /**
-    * @brief Work that is necessary after the JVM has been restored. This includes:
-    *        - Resetting the _checkpointStatus state.
-    *        - Resuming all suspended compilation threads.
-    */
-   void prepareForRestore();
-#endif
-
    void purgeMethodQueue(TR_CompilationErrorCode errorCode);
    void *compileMethod(J9VMThread * context, TR::IlGeneratorMethodDetails &details, void *oldStartPC,
       TR_YesNoMaybe async, TR_CompilationErrorCode *, bool *queued, TR_OptimizationPlan *optPlan);
@@ -932,6 +789,9 @@ public:
    void  cleanDLTRecordOnUnload();
    DLTTracking *getDLT_HT() const { return _dltHT; }
    void setDLT_HT(DLTTracking *dltHT) { _dltHT = dltHT; }
+#if defined(J9VM_OPT_JITSERVER)
+   std::vector<J9Method*> collectDLTedMethods();
+#endif /* defined(J9VM_OPT_JITSERVER) */
 #else
    DLTTracking *getDLT_HT() const { return NULL; }
 #endif // J9VM_JIT_DYNAMIC_LOOP_TRANSFER
@@ -1016,6 +876,9 @@ public:
    bool              isInShutdownMode() {return _isInShutdownMode;}
    void              setIsInShutdownMode() {_isInShutdownMode = true;}
 
+   bool              isSwapMemoryDisabled() {return _isSwapMemoryDisabled;}
+   void              setIsSwapMemoryDisabled(bool b) {_isSwapMemoryDisabled = b;}
+
    TR_LinkHead0<TR_ClassHolder> *getListOfClassesToCompile() { return &_classesToCompileList; }
    int32_t getCompilationLag();
    int32_t getCompilationLagUnlocked() { return getCompilationLag(); } // will go away
@@ -1052,10 +915,18 @@ public:
    TR::CompilationInfoPerThread *getCompInfoForThread(J9VMThread *vmThread);
    int32_t getNumUsableCompilationThreads() const { return _numCompThreads; }
    int32_t getNumTotalCompilationThreads() const { return _numCompThreads + _numDiagnosticThreads; }
+   int32_t getNumAllocatedCompilationThreads() const { return _numAllocatedCompThreads; }
+   int32_t getNumTotalAllocatedCompilationThreads() const { return _numAllocatedCompThreads + _numDiagnosticThreads; }
+   int32_t getFirstCompThreadID() const { return _firstCompThreadID; }
+   int32_t getFirstDiagThreadID() const { return _firstDiagnosticThreadID; }
+   int32_t getLastCompThreadID() const { return _lastCompThreadID; }
+   int32_t getLastDiagThreadID() const { return _lastDiagnosticTheadID; }
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+   void setNumUsableCompilationThreadsPostRestore(int32_t &numUsableCompThreads);
+#endif
    TR::CompilationInfoPerThreadBase *getCompInfoWithID(int32_t ID);
    TR::Compilation *getCompilationWithID(int32_t ID);
    TR::CompilationInfoPerThread *getFirstSuspendedCompilationThread();
-   bool useMultipleCompilationThreads() { return (getNumUsableCompilationThreads() + _numDiagnosticThreads) > 1; }
    bool getRampDownMCT() const { return _rampDownMCT; }
    void setRampDownMCT() { _rampDownMCT = true; } // cannot be reset
    static void printMethodNameToVlog(J9Method *method);
@@ -1075,28 +946,29 @@ public:
    bool importantMethodForStartup(J9Method *method);
    bool shouldDowngradeCompReq(TR_MethodToBeCompiled *entry);
 
+   bool isMethodIneligibleForAot(J9Method *method);
 
    int32_t computeDynamicDumbInlinerBytecodeSizeCutoff(TR::Options *options);
    TR_YesNoMaybe shouldActivateNewCompThread();
 #if DEBUG
-   void debugPrint(char *debugString);
-   void debugPrint(J9VMThread *, char *);
-   void debugPrint(char *, intptr_t);
-   void debugPrint(J9VMThread *, char *, IDATA);
+   void debugPrint(const char *debugString);
+   void debugPrint(J9VMThread *, const char *);
+   void debugPrint(const char *, intptr_t);
+   void debugPrint(J9VMThread *, const char *, IDATA);
    void debugPrint(J9Method*);
-   void debugPrint(char *, J9Method *);
-   void debugPrint(J9VMThread *, char *, TR_MethodToBeCompiled *);
-   void debugPrint(char * debugString, TR::IlGeneratorMethodDetails & details, J9VMThread * vmThread);
+   void debugPrint(const char *, J9Method *);
+   void debugPrint(J9VMThread *, const char *, TR_MethodToBeCompiled *);
+   void debugPrint(const char * debugString, TR::IlGeneratorMethodDetails & details, J9VMThread * vmThread);
    void printQueue();
 #else // !DEBUG
-   void debugPrint(char *debugString) {}
-   void debugPrint(J9VMThread *, char *) {}
-   void debugPrint(char *, intptr_t) {}
-   void debugPrint(J9VMThread *, char *, IDATA) {}
+   void debugPrint(const char *debugString) {}
+   void debugPrint(J9VMThread *, const char *) {}
+   void debugPrint(const char *, intptr_t) {}
+   void debugPrint(J9VMThread *, const char *, IDATA) {}
    void debugPrint(J9Method*) {}
-   void debugPrint(char *, J9Method *) {}
-   void debugPrint(J9VMThread *, char *, TR_MethodToBeCompiled *) {}
-   void debugPrint(char * debugString, TR::IlGeneratorMethodDetails & details, J9VMThread * vmThread) { }
+   void debugPrint(const char *, J9Method *) {}
+   void debugPrint(J9VMThread *, const char *, TR_MethodToBeCompiled *) {}
+   void debugPrint(const char * debugString, TR::IlGeneratorMethodDetails & details, J9VMThread * vmThread) { }
    void printQueue() {}
 #endif // DEBUG
    void debugPrint(TR_OpaqueMethodBlock *omb){ debugPrint((J9Method*)omb); }
@@ -1192,7 +1064,7 @@ public:
    void setAppSleepNano(int32_t t) { _appSleepNano = t; }
    int32_t computeAppSleepNano() const;
    TR_YesNoMaybe detectCompThreadStarvation();
-   bool getStarvationDetected() const { return _starvationDetected; }
+   bool getStarvationDetected() const { return _starvationDetected; } // This refers to compilation threads starvation
    void setStarvationDetected(bool b) { _starvationDetected = b; }
    int32_t getTotalCompThreadCpuUtilWhenStarvationComputed() const { return _totalCompThreadCpuUtilWhenStarvationComputed; }
    int32_t getNumActiveCompThreadsWhenStarvationComputed() const { return _numActiveCompThreadsWhenStarvationComputed; }
@@ -1200,6 +1072,18 @@ public:
    bool getSuspendThreadDueToLowPhysicalMemory() const { return _suspendThreadDueToLowPhysicalMemory; }
    void setSuspendThreadDueToLowPhysicalMemory(bool b) { _suspendThreadDueToLowPhysicalMemory = b; }
 
+   void initCPUEntitlement();
+
+   bool isJVMStarved() const { return _jvmIsStarved; }
+   void setIsJVMStarved(bool val) { _jvmIsStarved = val; }
+
+   bool getLowCompDensityMode() const { return _lowCompDensityMode; }
+   void enterLowCompDensityMode() { _lowCompDensityMode = true; _hasEnteredLowCompDensityModeInThePast = true;}
+   void exitLowCompDensityMode() { _lowCompDensityMode = false; }
+   bool hasEnteredLowCompDensityModeInThePast() const { return _hasEnteredLowCompDensityModeInThePast; }
+   bool compileFromLPQRegardlessOfCPU() const { return _compileFromLPQRegardlessOfCPU; }
+   void setCompileFromLPQRegardlessOfCPU(bool b) { _compileFromLPQRegardlessOfCPU = b; }
+
 #if defined(J9VM_OPT_JITSERVER)
    ClientSessionHT *getClientSessionHT() const { return _clientSessionHT; }
    void setClientSessionHT(ClientSessionHT *ht) { _clientSessionHT = ht; }
@@ -1236,6 +1120,13 @@ public:
    void  addJITServerSslCert(const std::string &cert) { _sslCerts.push_back(cert); }
    const std::string &getJITServerSslRootCerts() const { return _sslRootCerts; }
    void  setJITServerSslRootCerts(const std::string &cert) { _sslRootCerts = cert; }
+   void  freeClientSslCertificates() { _sslRootCerts.clear(); }
+   const PersistentVector<std::string> &getJITServerMetricsSslKeys() const { return _metricsSslKeys; }
+   void  addJITServerMetricsSslKey(const std::string &key) { _metricsSslKeys.push_back(key); }
+   const PersistentVector<std::string> &getJITServerMetricsSslCerts() const { return _metricsSslCerts; }
+   void  addJITServerMetricsSslCert(const std::string &cert) { _metricsSslCerts.push_back(cert); }
+   bool  useSSL() const { return (_sslKeys.size() || _sslCerts.size() || _sslRootCerts.size() ||
+                                  _metricsSslKeys.size() || _metricsSslCerts.size()); }
 
    void setCompThreadActivationPolicy(JITServer::CompThreadActivationPolicy newPolicy) { _activationPolicy = newPolicy; }
    JITServer::CompThreadActivationPolicy getCompThreadActivationPolicy() const { return _activationPolicy; }
@@ -1249,7 +1140,24 @@ public:
 
    JITServerAOTDeserializer *getJITServerAOTDeserializer() const { return _JITServerAOTDeserializer; }
    void setJITServerAOTDeserializer(JITServerAOTDeserializer *deserializer) { _JITServerAOTDeserializer = deserializer; }
+
+   bool methodCanBeJITServerAOTCacheStored(const char *methodSig, TR::Method::Type ty);
+   bool methodCanBeJITServerAOTCacheLoaded(const char *methodSig, TR::Method::Type ty);
+
+   bool methodCanBeRemotelyCompiled(const char *methodSig, TR::Method::Type ty);
+
+   /**
+   * @brief Notify every compilation thread that a JITServer AOT deserializer reset has occurred
+   *
+   * This function sets the _deserializerWasReset flag in every compilation thread. This function
+   * must be called with the appropriate deserializer monitors in hand. See
+   * JITServerAOTDeserializer::reset() for details.
+   *
+   */
+   void notifyCompilationThreadsOfDeserializerReset();
 #endif /* defined(J9VM_OPT_JITSERVER) */
+   uint32_t getNumTotalCompilations() const { return _numSyncCompilations + _numAsyncCompilations; }
+   uint32_t getNumCompsUsedForCompDensityCalculations() const { return _numCompsUsedForCompDensityCalculations; }
 
    static void replenishInvocationCount(J9Method* method, TR::Compilation* comp);
 
@@ -1259,6 +1167,11 @@ public:
    static void storeAOTInSharedCache(J9VMThread *vmThread, J9ROMMethod *romMethod, const U_8 *dataStart, UDATA dataSize, const U_8 *codeStart, UDATA codeSize, TR::Compilation *comp, J9JITConfig *jitConfig, TR_MethodToBeCompiled *entry);
 #endif
 
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+   TR::CRRuntime *getCRRuntime() { return _crRuntime; }
+   void setCRRuntime(TR::CRRuntime *crRuntime) { _crRuntime = crRuntime; }
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+
    static int32_t         VERY_SMALL_QUEUE;
    static int32_t         SMALL_QUEUE;
    static int32_t         MEDIUM_LARGE_QUEUE;
@@ -1323,21 +1236,18 @@ private:
    TR::Monitor *_dltMonitor;
    struct DLT_record     *_freeDLTRecord;
    struct DLT_record     *_dltHash[DLT_HASHSIZE];
+   int32_t _numDLTRecords;
 #endif
    DLTTracking           *_dltHT;
 
-#if defined(J9VM_OPT_CRIU_SUPPORT)
-   TR::Monitor *_crMonitor;
-   TR_CheckpointStatus _checkpointStatus;
-#endif
-
    TR::Monitor *_vlogMonitor;
    TR::Monitor *_rtlogMonitor;
    TR::Monitor *_iprofilerBufferArrivalMonitor;
    TR::MonitorTable *_j9MonitorTable; // used only for RAS (debuggerExtensions); no accessor; use TR_J9MonitorTable::get() everywhere else
    TR_LinkHead0<TR_ClassHolder> _classesToCompileList; // used by compileClasses; adjusted by unload hooks
-   intptr_t               _numSyncCompilations;
-   intptr_t               _numAsyncCompilations;
+   uint32_t               _numSyncCompilations;
+   uint32_t               _numAsyncCompilations;
+   uint32_t               _numCompsUsedForCompDensityCalculations;
    int32_t                _numCompThreadsActive;
    int32_t                _numCompThreadsJobless; // threads are not suspended, but have no work to do
    int32_t                _numCompThreadsCompilingHotterMethods; // allow only one at a time; use compQmonitor to change
@@ -1416,8 +1326,15 @@ private:
    bool                   _traceCompiling;
 #endif
    bool                   _isInShutdownMode;
+   bool                   _isSwapMemoryDisabled;
    int32_t                _numCompThreads; // Number of usable compilation threads that does not include the diagnostic thread
    int32_t                _numDiagnosticThreads;
+   int32_t                _numAllocatedCompThreads; // Number of allocated compilation threads that does not include the diagnostic thread
+   int32_t                _firstCompThreadID;
+   int32_t                _firstDiagnosticThreadID;
+   int32_t                _lastCompThreadID;
+   int32_t                _lastDiagnosticTheadID;
+   int32_t                _lastAllocatedCompThreadID;
    int32_t                _iprofilerMaxCount;
    int32_t                _numGCRQueued; // how many GCR requests are in the queue
                                          // We should disable GCR counting if too many
@@ -1426,7 +1343,7 @@ private:
    //----------------
    int32_t                _appSleepNano; // make app threads sleep when sampling
 
-   bool                   _starvationDetected;
+   bool                   _starvationDetected; // This refers to compilation threads starvation
    int32_t                _totalCompThreadCpuUtilWhenStarvationComputed;   // for RAS purposes
    int32_t                _numActiveCompThreadsWhenStarvationComputed; // for RAS purposes
    //--------------
@@ -1450,6 +1367,11 @@ private:
    bool _suspendThreadDueToLowPhysicalMemory;
    TR_InterpreterSamplingTracking *_interpSamplTrackingInfo;
 
+   bool _lowCompDensityMode; // set to true when compilations occur infrequently and are unlikely to contribute to JVM performance
+   bool _hasEnteredLowCompDensityModeInThePast; // set to true when _lowCompDensityMode is set to true at least once
+   bool _compileFromLPQRegardlessOfCPU;
+   bool _jvmIsStarved; // set to true if the JVM uses very little CPU because of external factors
+
 #if defined(J9VM_OPT_JITSERVER)
    ClientSessionHT               *_clientSessionHT; // JITServer hashtable that holds session information about JITClients
    PersistentUnorderedSet<J9Class*> _classesCachedAtServer;
@@ -1465,14 +1387,19 @@ private:
    std::string                   _sslRootCerts;
    PersistentVector<std::string> _sslKeys;
    PersistentVector<std::string> _sslCerts;
+   PersistentVector<std::string> _metricsSslKeys;
+   PersistentVector<std::string> _metricsSslCerts;
    JITServer::CompThreadActivationPolicy _activationPolicy;
    JITServerSharedROMClassCache *_sharedROMClassCache;
    JITServerAOTCacheMap *_JITServerAOTCacheMap;
    JITServerAOTDeserializer *_JITServerAOTDeserializer;
 #endif /* defined(J9VM_OPT_JITSERVER) */
+
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+   TR::CRRuntime *_crRuntime;
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
    }; // CompilationInfo
 }
 
 
 #endif // COMPILATIONRUNTIME_HPP
-
