@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef TR_RECOMPILATION_INFO_INCL
@@ -46,6 +46,7 @@ class TR_OptimizationPlan;
 class TR_ResolvedMethod;
 namespace TR { class Instruction; }
 namespace TR { class SymbolReference; }
+namespace J9 { class CompilationStrategy; }
 
 // Bits to represent sampling mechanism in method return info field.
 // Bits 3, 4, 5, 6, 7 are reserved for this purpose (could use fewer)
@@ -66,11 +67,28 @@ static int32_t profilingFreqTable  [] = {  19,  29,   47,   47,   47,    53 }; /
 #define DEFAULT_PROFILING_FREQUENCY (profilingFreqTable  [MAX_BACKEDGES])
 #define DEFAULT_PROFILING_COUNT     (profilingCountsTable[MAX_BACKEDGES])
 
-namespace TR { class DefaultCompilationStrategy; }
-namespace TR { class ThresholdCompilationStrategy; }
 namespace OMR { class Recompilation; }
 namespace J9 { class Recompilation; }
 
+class TR_JitBodyInvalidations
+   {
+   public:
+   enum Reason
+      {
+      HCR, // outermost method has been obsoleted by HCR
+      Preexistence, // preexistence assumption has been invalidated
+      PostRestoreExclude, // method is excluded post-restore, should be interpreted
+      };
+
+   bool isEmpty() const { return _flags.getValue() == 0; }
+   bool contains(Reason reason) const { return _flags.testAny(1 << reason); }
+   void add(Reason reason) { _flags.set(1 << reason); }
+   void add(TR_JitBodyInvalidations reasons) { _flags.set(reasons._flags.getValue()); }
+
+   private:
+   flags8_t _flags;
+   };
+
 // Persistent information associated with a method for recompilation
 //
 class TR_PersistentMethodInfo
@@ -80,8 +98,7 @@ class TR_PersistentMethodInfo
    friend class TR::CompilationInfo;
    friend class TR_S390Recompilation;  // FIXME: ugly
    friend class ::OMR::Options;
-   friend class TR::DefaultCompilationStrategy;
-   friend class TR::ThresholdCompilationStrategy;
+   friend class J9::CompilationStrategy;
 
    public:
    TR_PERSISTENT_ALLOC(TR_Memory::PersistentMethodInfo);
@@ -159,26 +176,39 @@ class TR_PersistentMethodInfo
 
    bool doesntKillAnything() { return _flags.testAll(RefinedAliasesMask); }
 
-   // Accessor methods for the "cpoCounter".  This does not really
-   // need to be its own counter, as it is conceptually the same as
-   // "_counter".  However, the original _counter is still during instrumentation, so
-   // it was simplest to keep them separate
-   //
-   int32_t cpoGetCounter()                {return _cpoSampleCounter;}
-   int32_t cpoIncCounter()                {return ++_cpoSampleCounter;}
-   int32_t cpoSetCounter(int newCount)    {return _cpoSampleCounter = newCount;}
+   bool isExcludedPostRestore()
+      {
+      return _invalidationReasons.contains(TR_JitBodyInvalidations::PostRestoreExclude);
+      }
+
+   /**
+    * \brief Get the set of all reasons for prior invalidations of JIT bodies
+    * belonging to this method.
+    */
+   TR_JitBodyInvalidations invalidationReasons() { return _invalidationReasons; }
 
    uint16_t getTimeStamp() { return _timeStamp; }
 
    TR_OptimizationPlan * getOptimizationPlan() {return _optimizationPlan;}
    void setOptimizationPlan(TR_OptimizationPlan *optPlan) { _optimizationPlan = optPlan; }
-   uint8_t getNumberOfInvalidations() {return _numberOfInvalidations;}
-   void incrementNumberOfInvalidations() {_numberOfInvalidations++;}
+   uint32_t getCatchBlockCounter() const { return _catchBlockCounter; }
+   uint32_t *getCatchBlockCounterAddress() { return &_catchBlockCounter; }
+   void incrementCatchBlockCounter() { _catchBlockCounter++; }
+   uint8_t getNumberOfPreexistenceInvalidations() {return _numberOfPreexistenceInvalidations;}
    uint8_t getNumberOfInlinedMethodRedefinition() {return _numberOfInlinedMethodRedefinition;}
    void incrementNumberOfInlinedMethodRedefinition() {_numberOfInlinedMethodRedefinition++;}
    int16_t getNumPrexAssumptions() {return _numPrexAssumptions;}
    void incNumPrexAssumptions() {_numPrexAssumptions++;}
 
+   void addInvalidationReasons(TR_JitBodyInvalidations reasons)
+      {
+      _invalidationReasons.add(reasons);
+      if (reasons.contains(TR_JitBodyInvalidations::Preexistence))
+         {
+         _numberOfPreexistenceInvalidations++;
+         }
+      }
+
    enum InfoBits
       {
       // Normally set by the previous compilation to indicate that the next
@@ -234,7 +264,9 @@ class TR_PersistentMethodInfo
       RecompDueToRI                        = 0x000A0000,
       RecompDueToJProfiling                = 0x000B0000,
       RecompDueToInlinedMethodRedefinition = 0x000C0000,
-      // NOTE: recompilations due to EDO decrementation cannot be tracked
+      RecompDueToCRIU                      = 0x000D0000,
+
+      // NOTE: recompilations due to EDO decrementation cannot be tracked precisely
       // because they are triggered from a snippet (must change the code for snippet)
       // Also, the recompilations after a profiling step cannot be marked as such.
       // NOTE: recompilations can be triggered by invalidations too, but this
@@ -247,6 +279,7 @@ class TR_PersistentMethodInfo
                                                        // Attention: this is not always accurate
       WasScannedForInlining                = 0x00400000, // New scanning for warm method inlining
       IsInDataCache                        = 0x00800000, // This TR_PersistentMethodInfo is stored in the datacache for AOT
+
       lastFlag                             = 0x80000000
       };
 
@@ -291,12 +324,12 @@ class TR_PersistentMethodInfo
 
 
    TR_OptimizationPlan            *_optimizationPlan;
-
-   int32_t                         _cpoSampleCounter; // TODO remove this field
+   uint32_t                        _catchBlockCounter; // how many times a catch block was executed
    uint16_t                        _timeStamp;
-   uint8_t                         _numberOfInvalidations; // how many times this method has been invalidated
+   uint8_t                         _numberOfPreexistenceInvalidations; // how many times this method has been invalidated due to preexistence
    uint8_t                         _numberOfInlinedMethodRedefinition; // how many times this method triggers recompilation because of its inlined callees being redefined
    int16_t                         _numPrexAssumptions;
+   TR_JitBodyInvalidations         _invalidationReasons;
 
    TR_PersistentProfileInfo       *_bestProfileInfo;
    TR_PersistentProfileInfo       *_recentProfileInfo;
@@ -305,7 +338,6 @@ class TR_PersistentMethodInfo
    void setForSharedInfo(TR_PersistentProfileInfo** ptr, TR_PersistentProfileInfo *newInfo);
    };
 
-
 // This information is kept for every jitted method that can be recompiled
 // It may be garbage collected along with the jitted method
 // The only way to get the following information is via a pointer that is kept
@@ -317,8 +349,7 @@ class TR_PersistentJittedBodyInfo
    friend class J9::Recompilation;
    friend class TR::CompilationInfo;
    friend class TR_S390Recompilation; // FIXME: ugly
-   friend class TR::DefaultCompilationStrategy;
-   friend class TR_EmilyPersistentJittedBodyInfo;
+   friend class J9::CompilationStrategy;
    friend class ::OMR::Options;
    friend class J9::Options;
 
@@ -332,6 +363,8 @@ class TR_PersistentJittedBodyInfo
    static TR_PersistentJittedBodyInfo *get(void *startPC);
 
    bool getHasLoops()               { return _flags.testAny(HasLoops); }
+   bool getHasEdoSnippet()          { return _flags.testAny(HasEdoSnippet); }
+   void setHasEdoSnippet()          { _flags.set(HasEdoSnippet, true); } // set by codegen when the recompilation snippet is created
    bool getUsesPreexistence()       { return _flags.testAny(UsesPreexistence); }
    bool getDisableSampling()        { return _flags.testAny(DisableSampling);  }
    void setDisableSampling(bool b)  { _flags.set(DisableSampling, b); }
@@ -344,8 +377,18 @@ class TR_PersistentJittedBodyInfo
    void setSamplingRecomp()         { _flags.set(SamplingRecomp, true); }
    bool getIsPushedForRecompilation(){ return _flags.testAny(IsPushedForRecompilation); }
    void setIsPushedForRecompilation(){ _flags.set(IsPushedForRecompilation, true); }
-   bool getIsInvalidated()          { return _isInvalidated; }
-   void setIsInvalidated()          { _isInvalidated = true; }
+
+   /**
+    * \brief Get the set of all reasons (there may be multiple) for which this
+    * JIT body has been invalidated.
+    */
+   TR_JitBodyInvalidations invalidationReasons() { return _invalidationReasons; }
+
+   bool getIsInvalidated() { return !_invalidationReasons.isEmpty(); }
+   void setIsInvalidated(TR_JitBodyInvalidations::Reason reason)
+      {
+      _invalidationReasons.add(reason);
+      }
 
    bool getFastHotRecompilation()   { return _flags.testAny(FastHotRecompilation); }
    void setFastHotRecompilation(bool b){ _flags.set(FastHotRecompilation, b); }
@@ -410,7 +453,7 @@ class TR_PersistentJittedBodyInfo
    enum
       {
       HasLoops                = 0x0001,
-      //HasManyIterationsLoops  = 0x0002, // Available
+      HasEdoSnippet           = 0x0002,
       UsesPreexistence        = 0x0004,
       DisableSampling         = 0x0008, // This flag disables sampling of this method even though its recompilable
       IsProfilingBody         = 0x0010,
@@ -470,7 +513,7 @@ class TR_PersistentJittedBodyInfo
    uint8_t                  _aggressiveRecompilationChances;
    TR_Hotness               _hotness;
    uint8_t                  _numScorchingIntervals; // How many times we reached scorching recompilation decision points
-   bool                     _isInvalidated;
+   TR_JitBodyInvalidations  _invalidationReasons;
    bool                     _longRunningInterpreted; // This cannot be moved into _flags due to synchronization issues
    TR_PersistentProfileInfo * _profileInfo;
    public:
