@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef J9_NODE_INCL
@@ -75,9 +75,39 @@ public:
    /// given a direct call to Object.clone node, return the class of the receiver.
    ///
    TR_OpaqueClassBlock* getCloneClassInNode();
-   TR::Node *           processJNICall(TR::TreeTop *, TR::ResolvedMethodSymbol *);
 
-   void                    devirtualizeCall(TR::TreeTop*);
+   /**
+    * @brief Stub method for OMR callers using the old API.  This function is
+    *    deprecated and will be removed once upstream OMR code is changed.
+    */
+   TR::Node *processJNICall(TR::TreeTop *callNodeTreeTop, TR::ResolvedMethodSymbol *owningSymbol);
+
+   /**
+    * @brief Transform a call node to prepare it for JNI dispatch.
+    *
+    * @param[in] callNodeTreeTop : \c TR::TreeTop for the call node
+    * @param[in] owningSymbol : \c TR::ResolvedMethodSymbol of the method to call
+    * @param[in] comp : \c TR::Compilation object
+    *
+    * @return Transformed call  \c TR::Node
+    */
+   TR::Node *processJNICall(TR::TreeTop *callNodeTreeTop, TR::ResolvedMethodSymbol *owningSymbol, TR::Compilation *comp);
+
+   /**
+    * @brief Stub method for OMR callers using the old API.  This function is
+    *    deprecated and will be removed once upstream OMR code is changed.
+    */
+   void devirtualizeCall(TR::TreeTop *treeTop);
+
+   /**
+    * @brief Devirtualize the call under the given treetop.
+    *
+    * @param[in] treeTop : \c TR::TreeTop of the call to devirtualize
+    * @param[in] comp : \c TR::Compilation object
+    *
+    * @return None
+    */
+   void devirtualizeCall(TR::TreeTop *treeTop, TR::Compilation *comp);
 
    /**
     * @return the signature of the node's type if applicable.
@@ -109,8 +139,8 @@ public:
    bool alwaysGeneratesAKnownCleanSign();
    bool alwaysGeneratesAKnownPositiveCleanSign();
 #ifdef TR_TARGET_S390
-   int32_t getStorageReferenceSize();
-   int32_t getStorageReferenceSourceSize();
+   int32_t getStorageReferenceSize(TR::Compilation *comp);
+   int32_t getStorageReferenceSourceSize(TR::Compilation *comp);
 #endif
 
    bool         isEvenPrecision();
@@ -132,13 +162,13 @@ public:
 
    bool canRemoveArithmeticOperand();
    bool canGCandReturn();
+   bool canGCandReturn(TR::Compilation *comp);
 
    static uint32_t hashOnBCDOrAggrLiteral(char *lit, size_t litSize);
 
    bool referencesSymbolInSubTree(TR::SymbolReference * symRef, vcount_t visitCount);
-   bool referencesMayKillAliasInSubTree(TR::Node * rootNode, vcount_t visitCount);
+   bool referencesMayKillAliasInSubTree(TR::Node * rootNode, vcount_t visitCount, TR::Compilation *comp);
    void getSubTreeReferences(TR::SparseBitVector &references, vcount_t visitCount);
-   TR_ParentOfChildNode * referencesSymbolExactlyOnceInSubTree(TR::Node *, int32_t, TR::SymbolReference *, vcount_t);
 
    /**
     * Node field functions
@@ -264,16 +294,8 @@ public:
 
    // Flag used by TR::BNDCHK nodes
    bool isSpineCheckWithArrayElementChild();
-   void setSpineCheckWithArrayElementChild(bool v);
+   void setSpineCheckWithArrayElementChild(bool v, TR::Compilation *comp);
    bool chkSpineCheckWithArrayElementChild();
-   const char *printSpineCheckWithArrayElementChild();
-
-   // Flags used by call nodes
-   bool isUnsafePutOrderedCall();
-   bool isDontInlinePutOrderedCall();
-   void setDontInlinePutOrderedCall();
-   bool chkDontInlineUnsafePutOrderedCall();
-   const char * printIsDontInlineUnsafePutOrderedCall();
 
    /**
     * Checks  and return true if the callNode is JNI Call to Unsafe.copyMemory
@@ -282,6 +304,7 @@ public:
 
    bool isUnsafeGetPutCASCallOnNonArray();
    void setUnsafeGetPutCASCallOnNonArray();
+   void setUnsafeGetPutCASCallOnNonArray(TR::Compilation *comp);
 
    bool isProcessedByCallCloneConstrain();
    void setProcessedByCallCloneConstrain();
@@ -297,56 +320,48 @@ public:
    bool cleanSignDuringPackedLeftShift();
    void setCleanSignDuringPackedLeftShift(bool v);
    bool chkCleanSignDuringPackedLeftShift();
-   const char *printCleanSignDuringPackedLeftShift();
 
    // Flag used by non-store and non-call binary coded decimal and aggregate nodes
    bool chkOpsSkipCopyOnLoad();
    bool skipCopyOnLoad();
    void setSkipCopyOnLoad(bool v);
    bool chkSkipCopyOnLoad();
-   const char *printSkipCopyOnLoad();
 
    // Flag used by binary coded decimal and aggregate store nodes
    bool chkOpsSkipCopyOnStore();
    bool skipCopyOnStore();
    void setSkipCopyOnStore(bool v);
    bool chkSkipCopyOnStore();
-   const char *printSkipCopyOnStore();
 
    // Flag used by packed decimal stores
    bool chkOpsCleanSignInPDStoreEvaluator();
    bool mustCleanSignInPDStoreEvaluator();
    void setCleanSignInPDStoreEvaluator(bool v);
    bool chkCleanSignInPDStoreEvaluator();
-   const char *printCleanSignInPDStoreEvaluator();
 
    // Flag used by binary coded decimal stores and aggregate stores
    bool chkOpsUseStoreAsAnAccumulator();
    bool useStoreAsAnAccumulator();
    void setUseStoreAsAnAccumulator(bool v);
    bool chkUseStoreAsAnAccumulator();
-   const char *printUseStoreAsAnAccumulator();
 
    // Flag used by non-store packed types
    bool canSkipPadByteClearing();
    void setSkipPadByteClearing(bool v);
    bool chkSkipPadByteClearing();
-   const char *printSkipPadByteClearing();
 
    // Flag used for BCD and Aggr type stores
    bool chkOpsIsInMemoryCopyProp();
    bool isInMemoryCopyProp();
    void setIsInMemoryCopyProp(bool v);
    bool chkIsInMemoryCopyProp();
-   const char *printIsInMemoryCopyProp();
 
    // Flag used by address type nodes in Java
    bool chkSharedMemory();
    void setSharedMemory(bool v);
-   const char *printSharedMemory();
-
 
    bool isArrayCopyCall();
+   bool isJitDispatchJ9MethodCall(TR::Compilation *comp);
 
    /**
     * Node flag functions end
@@ -436,7 +451,6 @@ protected:
       spineCHKWithArrayElementChild         = 0x00004000,
 
       // Flags used by call nodes
-      dontInlineUnsafePutOrderedCall        = 0x00000800, ///< unsafe putOrdered calls
       processedByCallCloneConstrain         = 0x00100000,
       unsafeGetPutOnNonArray                = 0x00200000,
       DAAVariableSlowCall                   = 0x00400000, ///< Used to avoid Variable precision DAA optimization
