@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "optimizer/IdiomRecognitionUtils.hpp"
@@ -47,6 +47,7 @@
 #include "optimizer/IdiomRecognition.hpp"
 #include "optimizer/Structure.hpp"
 #include "optimizer/TranslateTable.hpp"
+#include "optimizer/TransformUtil.hpp"
 
 /************************************/
 /************ Utilities *************/
@@ -822,22 +823,8 @@ createArrayHeaderConst(TR::Compilation *comp, bool is64bit, TR::Node *baseNode)
 TR::Node*
 createArrayTopAddressTree(TR::Compilation *comp, bool is64bit, TR::Node *baseNode)
    {
-   TR::Node *top, *c2;
    TR::Node *aload = createLoad(baseNode);
-   if (is64bit)
-      {
-      top = TR::Node::create(baseNode, TR::aladd, 2);
-      c2 = TR::Node::create(baseNode, TR::lconst, 0);
-      c2->setLongInt((int32_t)TR::Compiler->om.contiguousArrayHeaderSizeInBytes());
-      }
-   else
-      {
-      top = TR::Node::create(baseNode, TR::aiadd, 2);
-      c2 = TR::Node::create(baseNode, TR::iconst, 0, (int32_t)TR::Compiler->om.contiguousArrayHeaderSizeInBytes());
-      }
-   top->setAndIncChild(0, aload);
-   top->setAndIncChild(1, c2);
-   return top;
+   return TR::TransformUtil::generateFirstArrayElementAddressTrees(comp, aload);
    }
 
 
@@ -902,23 +889,7 @@ createBytesFromElement(TR::Compilation *comp, bool is64bit, TR::Node *indexNode,
 TR::Node*
 createIndexOffsetTree(TR::Compilation *comp, bool is64bit, TR::Node *indexNode, int multiply)
    {
-   TR::Node *top, *c1, *c2;
-   c1 = createBytesFromElement(comp, is64bit, indexNode, multiply);
-
-   if (is64bit)
-      {
-      c2 = TR::Node::create(indexNode, TR::lconst, 0);
-      c2->setLongInt(-(int32_t)TR::Compiler->om.contiguousArrayHeaderSizeInBytes());
-      top = TR::Node::create(indexNode, TR::lsub, 2);
-      }
-   else
-      {
-      c2 = TR::Node::create(indexNode, TR::iconst, 0, -(int32_t)TR::Compiler->om.contiguousArrayHeaderSizeInBytes());
-      top = TR::Node::create(indexNode, TR::isub, 2);
-      }
-   top->setAndIncChild(0, c1);
-   top->setAndIncChild(1, c2);
-   return top;
+   return createBytesFromElement(comp, is64bit, indexNode, multiply);
    }
 
 
@@ -934,13 +905,9 @@ createArrayAddressTree(TR::Compilation *comp, bool is64bit, TR::Node *baseNode,
       }
    else
       {
-      TR::Node *top, *c2;
+      TR::Node *c2 = createIndexOffsetTree(comp, is64bit, indexNode, multiply);
       TR::Node *aload = createLoad(baseNode);
-      c2 = createIndexOffsetTree(comp, is64bit, indexNode, multiply);
-      top = TR::Node::create(baseNode, is64bit ? TR::aladd : TR::aiadd, 2);
-      top->setAndIncChild(0, aload);
-      top->setAndIncChild(1, c2);
-      return top;
+      return TR::TransformUtil::generateArrayElementAddressTrees(comp, aload, c2);
       }
    }
 
