@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef INLINERTEMPFORJ9_INCL
@@ -69,17 +69,12 @@ class TR_MultipleCallTargetInliner : public TR_InlinerBase
    {
    public:
 
-      template <typename FunctObj>
-      void recursivelyWalkCallTargetAndPerformAction(TR_CallTarget *ct, FunctObj &action);
-
-      //void generateNodeEstimate(TR_CallTarget *ct, TR::Compilation *comp);
-
-      class generateNodeEstimate
+      struct NodeEstimate
          {
-         public:
-            generateNodeEstimate() : _nodeEstimate(0){ }
-            void operator()(TR_CallTarget *ct, TR::Compilation *comp);
-            int32_t getNodeEstimate() { return _nodeEstimate; }
+         NodeEstimate() : _nodeEstimate(0){ }
+         void operator()(TR_CallTarget *ct, TR::Compilation *comp);
+         int32_t getNodeEstimate() { return _nodeEstimate; }
+
          private:
             int32_t _nodeEstimate;
          };
@@ -91,15 +86,21 @@ class TR_MultipleCallTargetInliner : public TR_InlinerBase
 
       TR_LinkHead<TR_CallTarget> _callTargets; // This list only contains the call targets from top most level
 
+      /*
+       * \brief Recursively walks call target and estimates the number of nodes of a call graph.
+       *
+       * \param ct
+       *    the TR_CallTarget to evaluate
+       * \param estimate
+       *    the NodeEstimate to keep track of the number of nodes
+       */
+      void recursivelyWalkCallTargetAndGenerateNodeEstimate(TR_CallTarget *ct, NodeEstimate &estimate);
+
    protected:
       virtual int32_t scaleSizeBasedOnBlockFrequency(int32_t bytecodeSize, int32_t frequency, int32_t borderFrequency, TR_ResolvedMethod * calleeResolvedMethod, TR::Node *callNode, int32_t coldBorderFrequency = 0);
       float getScalingFactor(float factor);
       virtual bool supportsMultipleTargetInlining () { return true ; }
 
-      void walkCallSites(TR::ResolvedMethodSymbol *, TR_CallStack *, TR_InnerPreexistenceInfo *, int32_t walkDepth);
-      void walkCallSite( TR::ResolvedMethodSymbol * calleeSymbol, TR_CallStack * callStack,
-                         TR::TreeTop * callNodeTreeTop, TR::Node * parent, TR::Node * callNode, TR_VirtualGuardSelection *guard,
-                         TR_OpaqueClassBlock * thisClass, bool inlineNonRecursively, int32_t walkDepth);
    private:
       bool analyzeCallSite(TR::ResolvedMethodSymbol *, TR_CallStack *, TR::TreeTop *, TR::Node *, TR::Node *);
       void weighCallSite( TR_CallStack * callStack , TR_CallSite *callsite, bool currentBlockHasExceptionSuccessors,bool dontAddCalls=false);
@@ -107,7 +108,7 @@ class TR_MultipleCallTargetInliner : public TR_InlinerBase
       int32_t applyArgumentHeuristics(TR_LinkHead<TR_ParameterMapping> &map, int32_t originalWeight, TR_CallTarget *target);
       bool eliminateTailRecursion(TR::ResolvedMethodSymbol *, TR_CallStack *, TR::TreeTop *, TR::Node *, TR::Node *, TR_VirtualGuardSelection *);
       void assignArgumentsToParameters(TR::ResolvedMethodSymbol *, TR::TreeTop *, TR::Node *);
-      bool isLargeCompiledMethod(TR_ResolvedMethod *calleeResolvedMethod, int32_t bytecodeSize, int32_t freq);
+      bool isLargeCompiledMethod(TR_ResolvedMethod *calleeResolvedMethod, int32_t bytecodeSize, int32_t freq, int32_t exemptionFreqCutoff, int32_t veryLargeCompiledMethodThreshold, int32_t veryLargeCompiledMethodFaninThreshold);
       /* \brief
        *    This API processes the call targets got chopped off from \ref _calltargets
        *
@@ -132,6 +133,20 @@ class TR_MultipleCallTargetInliner : public TR_InlinerBase
        *    True if the given calltarget should be inlined
        */
       bool inlineSubCallGraph(TR_CallTarget* calltarget);
+
+      /*
+       * \brief
+       *   For some call targets and their sub call graphs, it may be possible to simplify them into simple operations in
+       *   certain situations, such as when known object info is being passed as arg. In such cases, the node count
+       *   obtained via NodeEstimate would not truly reflect the number of nodes that are actually introduced. This
+       *   function provides a mechanism for examining call targets and evaluating whether it is safe to skip counting nodes.
+       *
+       * \param callTarget
+       *    the call target to examine
+       * \return
+       *    true if node counting can be skipped for callTarget, false otherwise
+       */
+      bool canSkipCountingNodes(TR_CallTarget* callTarget);
    };
 
 class TR_J9InlinerUtil: public OMR_InlinerUtil
@@ -161,7 +176,6 @@ class TR_J9InlinerUtil: public OMR_InlinerUtil
    protected:
       virtual void refineColdness (TR::Node* node, bool& isCold);
       virtual void computeMethodBranchProfileInfo (TR::Block * cfgBlock, TR_CallTarget* calltarget, TR::ResolvedMethodSymbol* callerSymbol);
-      virtual int32_t getCallCount(TR::Node *callNode);
       virtual TR_InnerPreexistenceInfo *createInnerPrexInfo(TR::Compilation * c, TR::ResolvedMethodSymbol *methodSymbol, TR_CallStack *callStack, TR::TreeTop *callTree, TR::Node *callNode, TR_VirtualGuardKind guardKind);
       virtual void estimateAndRefineBytecodeSize(TR_CallSite* callsite, TR_CallTarget* target, TR_CallStack *callStack, int32_t &bytecodeSize);
       virtual TR_TransformInlinedFunction *getTransformInlinedFunction(TR::ResolvedMethodSymbol *, TR::ResolvedMethodSymbol *, TR::Block *, TR::TreeTop *,
@@ -174,10 +188,12 @@ class TR_J9InlinerPolicy : public OMR_InlinerPolicy
    friend class TR_J9InlinerUtil;
    friend class TR_InlinerBase;
    friend class TR_MultipleCallTargetInliner;
+   friend class TR_J9EstimateCodeSize;
    public:
       TR_J9InlinerPolicy(TR::Compilation *comp);
       virtual bool inlineRecognizedMethod(TR::RecognizedMethod method);
       virtual bool tryToInlineTrivialMethod (TR_CallStack* callStack, TR_CallTarget* calltarget);
+      virtual bool trivialInliningOnly(TR_CallStack *callStack, TR_CallTarget *callTarget);
       bool isInlineableJNI(TR_ResolvedMethod *method,TR::Node *callNode);
       virtual bool alwaysWorthInlining(TR_ResolvedMethod * calleeMethod, TR::Node *callNode);
       bool adjustFanInSizeInExceedsSizeThreshold(int bytecodeSize,
@@ -213,21 +229,53 @@ class TR_J9InlinerPolicy : public OMR_InlinerPolicy
       void createTempsForUnsafeCall( TR::TreeTop *callNodeTreeTop, TR::Node * unsafeCallNode );
       TR::Node *   inlineGetClassAccessFlags(TR::ResolvedMethodSymbol *, TR::ResolvedMethodSymbol *, TR::TreeTop *, TR::Node *);
       bool         inlineUnsafeCall(TR::ResolvedMethodSymbol *, TR::ResolvedMethodSymbol *, TR::TreeTop *, TR::Node *);
-      TR::Block * addNullCheckForUnsafeGetPut(TR::Node* unsafeAddress, TR::SymbolReference* newSymbolReferenceForAddress, TR::TreeTop* callNodeTreeTop, TR::TreeTop* directAccessTreeTop, TR::TreeTop* arrayDirectAccessTreeTop, TR::TreeTop* indirectAccessTreeTop);
-      bool createUnsafePutWithOffset(TR::ResolvedMethodSymbol *, TR::ResolvedMethodSymbol *, TR::TreeTop *, TR::Node *, TR::DataType, bool, bool needNullCheck = false, bool isOrdered = false);
+
+      /**
+       * \brief Generates a diamond that will be used as the outline of the IL generated
+       *        for inlining a call to \c Unsafe.get* or \c Unsafe.put*.  It splits the block
+       *        that contains \c callNodeTreeTop at that point, adds \c comparisonTree
+       *        as the branch for the diamond, branching to a block containing \c ifTree
+       *
+       * \param callNodeTreeTop A pointer to the \ref TR::TreeTop for the call that is
+       *           being inlined
+       * \param comparisonTree A pointer to a \ref TR::TreeTop for a \c if node that will
+       *           be the root of the diamond
+       * \param branchTargetTree A pointer to a \ref TR::TreeTop representing the taken branch of
+       *           \c comparisonTree
+       * \param fallThroughTree A pointer to a \ref TR::TreeTop reprenting the fall-through branch
+       *           of \c comparisonTree
+       *
+       * \return A pointer to a \ref TR::Block representing the join point of the diamond
+       *         after executing either \c branchTargetTree or \c fallThroughTree
+       */
+      TR::Block * createUnsafeGetPutCallDiamond(TR::TreeTop* callNodeTreeTop, TR::TreeTop* comparisonTree, TR::TreeTop* branchTargetTree, TR::TreeTop* fallThroughTree);
+      bool createUnsafePutWithOffset(TR::ResolvedMethodSymbol *, TR::ResolvedMethodSymbol *, TR::TreeTop *, TR::Node *, TR::DataType, TR::Symbol::MemoryOrdering ordering = TR::Symbol::MemoryOrdering::Transparent, bool needNullCheck = false, bool isUnaligned = false);
       TR::TreeTop* genDirectAccessCodeForUnsafeGetPut(TR::Node* callNode, bool conversionNeeded, bool isUnsafeGet);
       void createTempsForUnsafePutGet(TR::Node*& unsafeAddress, TR::Node* unsafeCall, TR::TreeTop* callNodeTreeTop, TR::Node*& offset, TR::SymbolReference*& newSymbolReferenceForAddress, bool isUnsafeGet);
       bool         createUnsafeGet(TR::ResolvedMethodSymbol *, TR::ResolvedMethodSymbol *, TR::TreeTop *, TR::Node *, TR::DataType, bool compress = true);
       bool         createUnsafePut(TR::ResolvedMethodSymbol *, TR::ResolvedMethodSymbol *, TR::TreeTop *, TR::Node *, TR::DataType, bool compress = true);
       TR::Node *    createUnsafeAddress(TR::Node *);
-      bool         createUnsafeGetWithOffset(TR::ResolvedMethodSymbol *, TR::ResolvedMethodSymbol *, TR::TreeTop *, TR::Node *, TR::DataType, bool, bool needNullCheck = false);
+      bool         createUnsafeGetWithOffset(TR::ResolvedMethodSymbol *, TR::ResolvedMethodSymbol *, TR::TreeTop *, TR::Node *, TR::DataType, TR::Symbol::MemoryOrdering ordering = TR::Symbol::MemoryOrdering::Transparent, bool needNullCheck = false, bool isUnaligned = false);
       TR::Node *    createUnsafeAddressWithOffset(TR::Node *);
       bool         createUnsafeFence(TR::TreeTop *, TR::Node *, TR::ILOpCodes);
 
       TR::Node *    createUnsafeMonitorOp(TR::ResolvedMethodSymbol *calleeSymbol, TR::ResolvedMethodSymbol *callerSymbol, TR::TreeTop * callNodeTreeTop, TR::Node * unsafeCall, bool isEnter);
       bool         createUnsafeCASCallDiamond(TR::TreeTop *, TR::Node *);
-      TR::TreeTop* genClassCheckForUnsafeGetPut(TR::Node* offset);
-      TR::TreeTop* genClassCheckForUnsafeGetPut(TR::Node* offset, bool isNotLowTagged);
+
+      /**
+       * \brief Generates an \c if node that will test whether the low-order bit of
+       *           the supplied \c offset is set.  The branch target must be set
+       *           by the caller.
+       *
+       * \param offset A pointer to a \ref TR::Node representing the offset value
+       *           whose low-order bit is to be tested
+       * \param branchIfLowTagged A value of type \c bool that indicates whether the
+       *           branch should be taken if the low-order bit of \c offset is set
+       *           or not set
+       *
+       * \return A pointer to a \ref TR::TreeTop holding the generated \c if node
+       */
+      TR::TreeTop* genClassCheckForUnsafeGetPut(TR::Node* offset, bool branchIfLowTagged);
 
       /** \brief
        *     Generates the indirect access for an unsafe get/put operation given a direct access. This function will
@@ -248,13 +296,66 @@ class TR_J9InlinerPolicy : public OMR_InlinerPolicy
 
       void createAnchorNodesForUnsafeGetPut(TR::TreeTop* treeTop, TR::DataType type, bool isUnsafeGet);
       TR::Node *     genCompressedRefs(TR::Node *, bool genTT = true, int32_t isLoad = 1);
-      void genCodeForUnsafeGetPut(TR::Node* unsafeAddress, TR::TreeTop* callNodeTreeTop, TR::TreeTop* prevTreeTop, TR::SymbolReference* newSymbolReferenceForAddress, TR::TreeTop* directAccessTreeTop, TR::TreeTop* lowTagCmpTree, bool needNullCheck, bool isUnsafeGet, bool conversionNeeded, TR::Block * joinBlock, TR_OpaqueClassBlock *javaLangClass, TR::Node* orderedCallNode);
+
+      /**
+       * \brief Puts together fragments of IL that have been generated for parts of the
+       *        IL that are needed to generate \c Unsafe.get* or \c Unsafe.put* inline.
+       *        Resultant IL is placed in the \ref TR:CFG in place of the existing call.
+       *
+       * \param unsafeAddress Pointer to a \ref TR::Node representing the \c Object
+       *           argument of the call
+       * \param unsafeOffset Pointer to a \ref TR::Node representing the \c long
+       *           value offset argument of the call
+       * \param type A \ref TR::DataType indicating the OMR data type corresponding to
+       *           the Java type of the \c Unsafe method call
+       * \param callNodeTreeTop A pointer to a \ref TR::TreeTop representing the call
+       *           that is being inlined
+       * \param prevTreeTop  A pointer to a \ref TR::TreeTop preceding the call in the IL
+       * \param newSymbolReferenceForAddress A \ref TR::SymbolReference representing the
+       *           the \c Object argument of the \c Unsafe call
+       * \param directAccessTreeTop A pointer to a \ref TR::TreeTop for the IL needed
+       *           to perform a nearly "direct" reference loading or storing a value
+       *           for the \c Unsafe method call, without any conversion for one or
+       *           two byte values
+       * \param arraydirectAccessTreeTop A pointer to a \ref TR::TreeTop for the IL
+       *           needed to perform a nearly "direct" reference loading or storing a
+       *           value for the \c Unsafe method call, without any conversion for one
+       *           or two byte values
+       * \param indirectAccessTreeTop A pointer to a \ref TR::TreeTop for the IL
+       *           needed to perform a more "indirect" reference loading or storing a
+       *           static field value for the \c Unsafe method call
+       * \param directAccessWithConversionTreeTop A pointer to a \ref TR::TreeTop for
+       *           the IL needed to perform a nearly "direct" reference loading or
+       *           storing a value for the \c Unsafe method call, with a conversion
+       *           for one or two byte values
+       * \param needNullCheck A \bool value indicating whether a \ref TR::NULLCHK
+       *           needs to be generated for the value of \c unsafeAddress
+       * \param isUnsafeGet A \bool value indicating whether the call represents an
+       *           \c Unsafe.get* operation &mdash; \c true &mdash; or an
+       *           \c Unsafe.put* operation &mdash; \c false.
+       * \param conversionNeeded Indicates whether the call reprents an \c Unsafe
+       *           method call involving any of Java \c char, \c short, \c byte or
+       *           \c boolean.
+       * \param arrayBlockNeeded Indicates whether a separate access block needs to be
+       *           generated to handle the case where the \c Object is an array
+       * \param typeTestsNeeded Indicates whether any type tests for \c Object need to
+       *           be generated (i.e.: if the type of the \c Object is unknown)
+       * \param orderedCallNode Indicates whether the call represents an \c Unsafe
+       *           ordered method
+       */
+      void genCodeForUnsafeGetPut(TR::Node* unsafeAddress, TR::Node *unsafeOffset, TR::DataType type, TR::TreeTop* callNodeTreeTop,
+                                  TR::TreeTop* prevTreeTop, TR::SymbolReference* newSymbolReferenceForAddress,
+                                  TR::TreeTop* directAccessTreeTop, TR::TreeTop* arraydirectAccessTreeTop,
+                                  TR::TreeTop* indirectAccessTreeTop, TR::TreeTop* directAccessWithConversionTreeTop,
+                                  bool needNullCheck, bool isUnsafeGet, bool conversionNeeded,
+                                  bool arrayBlockNeeded, bool typeTestsNeeded, TR::Node* orderedCallNode);
       virtual bool callMustBeInlined(TR_CallTarget *calltarget);
+      virtual bool callMustBeInlinedInCold(TR_ResolvedMethod *method);
       bool mustBeInlinedEvenInDebug(TR_ResolvedMethod * calleeMethod, TR::TreeTop *callNodeTreeTop);
       bool _tryToGenerateILForMethod (TR::ResolvedMethodSymbol* calleeSymbol, TR::ResolvedMethodSymbol* callerSymbol, TR_CallTarget* calltarget);
       bool doCorrectnessAndSizeChecksForInlineCallTarget(TR_CallStack *callStack, TR_CallTarget *calltarget, bool inlinefromgraph, TR_PrexArgInfo *argInfo);
       bool validateArguments(TR_CallTarget *calltarget, TR_LinkHead<TR_ParameterMapping> &map);
-      virtual bool supressInliningRecognizedInitialCallee(TR_CallSite* callsite, TR::Compilation* comp);
+      virtual bool suppressInliningRecognizedInitialCallee(TR_CallSite* callsite, TR::Compilation* comp);
       virtual TR_InlinerFailureReason checkIfTargetInlineable(TR_CallTarget* target, TR_CallSite* callsite, TR::Compilation* comp);
       /** \brief
        *     This query decides whether the given method is JSR292 related
@@ -275,6 +376,22 @@ class TR_J9InlinerPolicy : public OMR_InlinerPolicy
        *     This query defines a group of methods that are small helpers in the java/lang/invoke package
        */
       static bool isJSR292SmallHelperMethod(TR_ResolvedMethod *resolvedMethod);
+
+      /**
+       * \brief
+       *    This query answers whether the method is a simple non-native Unsafe method that contain a call to
+       *    a native Unsafe method that would normally be handled in TR_J9InlinerPolicy::inlineUnsafeCall. If
+       *    we can determine that the runtime checks in the wrapper method can be determined at compile time,
+       *    it may be possible to treat the wrapper method as its underlying native Unsafe method and have it
+       *    inlined in TR_J9InlinerPolicy::inlineUnsafeCall.
+       *
+       * \param
+       *    resolvedMethod the TR_ResolvedMethod
+       * \return
+       *    true if the method is a simple wrapper method for a native unsafe method, false otherwise
+       */
+      static bool isSimpleWrapperForInlineableUnsafeNativeMethod(TR_ResolvedMethod *resolvedMethod);
+
    };
 
 class TR_J9JSR292InlinerPolicy : public TR_J9InlinerPolicy
