@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #if defined(J9ZOS390)
@@ -85,6 +85,7 @@
 #include "optimizer/HandleRecompilationOps.hpp"
 #include "optimizer/MethodHandleTransformer.hpp"
 #include "optimizer/VectorAPIExpansion.hpp"
+#include "optimizer/CatchBlockProfiler.hpp"
 
 
 static const OptimizationStrategy J9EarlyGlobalOpts[] =
@@ -264,7 +265,7 @@ static const OptimizationStrategy coldStrategyOpts[] =
    { OMR::trivialInlining                                                       },
    { OMR::jProfilingBlock                                                       },
    { OMR::virtualGuardTailSplitter                                              },
-   { OMR::recompilationModifier,                     OMR::IfEnabled                  },
+   { OMR::recompilationModifier,                     OMR::IfEnabled             },
    { OMR::samplingJProfiling                                                    },
    { OMR::treeSimplification                                                    }, // cleanup before basicBlockExtension
 #if defined(J9VM_OPT_OPENJDK_METHODHANDLE)
@@ -295,12 +296,115 @@ static const OptimizationStrategy coldStrategyOpts[] =
    { OMR::jProfilingValue,                           OMR::MustBeDone                 },
    { OMR::treeLowering,                              OMR::MustBeDone            },
    { OMR::globalLiveVariablesForGC,                  OMR::IfAggressiveLiveness  },
-   { OMR::profilingGroup,                            OMR::IfProfiling                },
+   { OMR::jitProfilingGroup,                         OMR::IfJitProfiling             },
+   { OMR::catchBlockProfiler,                        OMR::IfExceptionHandlers   },
    { OMR::regDepCopyRemoval                                                     },
    { OMR::hotFieldMarking                                                       },
    { OMR::endOpts                                                               }
    };
 
+// ***************************************************************************
+//
+// Strategy for warm methods.
+//
+// ***************************************************************************
+//
+static const OptimizationStrategy warmStrategyOpts[] =
+   {
+   { OMR::trivialDeadTreeRemoval,                    OMR::IfEnabled             },
+   { OMR::coldBlockOutlining                                                    },
+   { OMR::stringBuilderTransformer                                              },
+   { OMR::stringPeepholes                                                       }, // need stringpeepholes to catch bigdecimal patterns
+   { OMR::inlining                                                              },
+   { OMR::methodHandleInvokeInliningGroup,           OMR::IfEnabled             },
+   { OMR::staticFinalFieldFolding,                                              },
+   { OMR::osrGuardInsertion,                         OMR::MustBeDone            },
+   { OMR::osrExceptionEdgeRemoval                                               }, // most inlining is done by now
+   { OMR::jProfilingBlock                                                       },
+   { OMR::virtualGuardTailSplitter                                              }, // merge virtual guards
+   { OMR::treeSimplification                                                    },
+#ifdef TR_HOST_S390
+   { OMR::sequentialLoadAndStoreWarmGroup,           OMR::IfEnabled                  },
+#endif
+   { OMR::cheapGlobalValuePropagationGroup                                      },
+   { OMR::localCSE,                                  OMR::IfVectorAPI           },
+   { OMR::dataAccessAccelerator                                                 },
+#ifdef TR_HOST_S390
+   { OMR::globalCopyPropagation,                     OMR::IfVoluntaryOSR        },
+#endif
+   { OMR::lastLoopVersionerGroup,                    OMR::IfLoopsAndNotCompileTimeSensitive},
+#ifdef TR_HOST_S390
+   { OMR::globalDeadStoreElimination,                OMR::IfEnabledAndLoops          },
+   { OMR::deadTreesElimination                                                       },
+   { OMR::recompilationModifier,                     OMR::IfEnabledAndNotProfiling   },
+   { OMR::localReordering,                           OMR::IfNoLoopsOREnabledAndLoops },
+   { OMR::basicBlockOrdering,                        OMR::IfLoops                    },
+   { OMR::treeSimplification                                                         },
+   { OMR::loopReduction                                                              },
+   { OMR::blockShuffling                                                             },
+#endif
+   { OMR::localCSE,                                  OMR::IfLoopsAndNotProfiling     },
+   { OMR::idiomRecognition,                          OMR::IfLoopsAndNotProfiling     },
+   { OMR::treeSimplification                                                    },
+   { OMR::redundantGotoElimination,                  OMR::IfEnabledAndNotJitProfiling     },
+   { OMR::blockSplitter                                                         },
+   { OMR::treeSimplification                                                    }, // revisit; not really required ?
+   { OMR::virtualGuardHeadMerger                                                },
+#if defined(J9VM_OPT_OPENJDK_METHODHANDLE)
+   { OMR::recognizedCallTransformer,                 OMR::MarkLastRun           },
+#endif
+   { OMR::basicBlockExtension,                       OMR::MarkLastRun                }, // extend blocks; move trees around if reqd
+   { OMR::localValuePropagationGroup                                            },
+   { OMR::explicitNewInitialization,                 OMR::IfNews                },
+   { OMR::arraycopyTransformation                                               },
+   { OMR::treeSimplification,                        OMR::IfEnabled                  },
+   { OMR::asyncCheckInsertion,                       OMR::IfNotJitProfiling          },
+   { OMR::localCSE                                                              },
+   { OMR::treeSimplification,                        OMR::MarkLastRun                },
+   { OMR::andSimplification,                         OMR::IfEnabled                  },  //clean up after versioner
+   { OMR::compactNullChecks                                                     }, // cleanup at the end
+   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // cleanup at the end
+   { OMR::globalCopyPropagation,                     OMR::IfMethodHandleInvokes      }, // Does a lot of good after methodHandleInvokeInliningGroup
+   { OMR::treesCleansing,                            OMR::IfEnabled                  },
+   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // cleanup at the end
+   { OMR::localCSE,                                  OMR::IfEnabled                  }, // common up expressions for sunk stores
+   { OMR::treeSimplification,                        OMR::IfEnabledMarkLastRun       }, // cleanup the trees after sunk store and localCSE
+
+   /** \breif
+    *      This optimization is performance critical on z Systems. On z Systems a literal pool register is blocked off
+    *      by default at the start of the compilation since materializing this address could be expensive depending on
+    *      the architecture level we are executing on. This optimization pass validates support for dynamically
+    *      materializing the literal pool address and frees up the literal pool register for register allocation.
+   */
+   { OMR::dynamicLiteralPool,                        OMR::IfNotProfiling             },
+   { OMR::samplingJProfiling                                                         },
+   { OMR::trivialBlockExtension                                                      },
+   { OMR::localCSE,                                  OMR::IfEnabled                  },  //common up lit pool refs in the same block
+   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // cleanup at the end
+   { OMR::treeSimplification,                        OMR::IfEnabledMarkLastRun       }, // Simplify non-normalized address computations introduced by prefetch insertion
+   { OMR::trivialDeadTreeRemoval,                    OMR::IfEnabled                  }, // final cleanup before opcode expansion
+   { OMR::jProfilingRecompLoopTest,                  OMR::IfLoops                    },
+   { OMR::globalDeadStoreElimination,                OMR::IfVectorAPI                }, // global dead store removal
+   { OMR::deadTreesElimination,                      OMR::IfVectorAPI                }, // cleanup after dead store removal
+   { OMR::vectorAPIExpansion,                        OMR::IfVectorAPI                },
+   { OMR::osrGuardRemoval,                           OMR::IfVectorAPI                },
+   { OMR::cheapTacticalGlobalRegisterAllocatorGroup, OMR::IfEnabled                  },
+   { OMR::jProfilingValue,                           OMR::MustBeDone                 },
+   { OMR::treeLowering,                              OMR::MustBeDone                 },
+   { OMR::globalDeadStoreGroup,                                                      },
+   { OMR::compactNullChecks,                         OMR::IfEnabled                  }, // cleanup at the end
+   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // remove dead anchors created by check/store removal
+   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // remove dead RegStores produced by previous deadTrees pass
+   { OMR::redundantGotoElimination,                  OMR::IfEnabledAndNotJitProfiling }, // dead store and dead tree elimination may have left empty blocks
+   { OMR::compactLocals,                             OMR::IfNotJitProfiling          }, // analysis results are invalidated by jitProfilingGroup
+   { OMR::globalLiveVariablesForGC                                                   },
+   { OMR::jitProfilingGroup,                         OMR::IfJitProfiling             },
+   { OMR::catchBlockProfiler,                        OMR::IfExceptionHandlers        },
+   { OMR::regDepCopyRemoval                                                          },
+   { OMR::hotFieldMarking                                                            },
+   { OMR::endOpts                                                                    }
+   };
+
 
 // ***************************************************************************
 //
@@ -309,7 +413,7 @@ static const OptimizationStrategy coldStrategyOpts[] =
 //
 // ***************************************************************************
 //
-static const OptimizationStrategy warmStrategyOpts[] =
+static const OptimizationStrategy oldWarmStrategyOpts[] =
    {
    { OMR::trivialDeadTreeRemoval,                                      OMR::IfEnabled},
    { OMR::coldBlockOutlining                                                    },
@@ -323,13 +427,13 @@ static const OptimizationStrategy warmStrategyOpts[] =
    { OMR::jProfilingBlock                                                       },
    { OMR::virtualGuardTailSplitter                                              }, // merge virtual guards
    { OMR::treeSimplification                                                    },
-   { OMR::sequentialLoadAndStoreWarmGroup,           OMR::IfEnabled                  }, // disabled by default, enabled by -Xjit:enableSequentialLoadStoreWarm
+   { OMR::sequentialLoadAndStoreWarmGroup,           OMR::IfEnabled             }, // disabled by default, enabled by -Xjit:enableSequentialLoadStoreWarm
    { OMR::cheapGlobalValuePropagationGroup                                      },
-   { OMR::localCSE,                                    OMR::IfVectorAPI },
+   { OMR::localCSE,                                    OMR::IfVectorAPI         },
    { OMR::dataAccessAccelerator                                                 }, // globalValuePropagation and inlining might expose opportunities for dataAccessAccelerator
-   { OMR::globalCopyPropagation,                       OMR::IfVoluntaryOSR          },
-   { OMR::lastLoopVersionerGroup,                      OMR::IfLoops                  },
-   { OMR::globalDeadStoreElimination,                  OMR::IfEnabledAndLoops},
+   { OMR::globalCopyPropagation,                       OMR::IfVoluntaryOSR      },
+   { OMR::lastLoopVersionerGroup,                      OMR::IfLoops             },
+   { OMR::globalDeadStoreElimination,                  OMR::IfEnabledAndLoops   },
    { OMR::deadTreesElimination                                                  },
    { OMR::recompilationModifier,                     OMR::IfEnabledAndNotProfiling   },
    { OMR::localReordering,                           OMR::IfNoLoopsOREnabledAndLoops }, // if required or if not done earlier
@@ -382,6 +486,8 @@ static const OptimizationStrategy warmStrategyOpts[] =
    { OMR::globalDeadStoreElimination,                OMR::IfVectorAPI                }, // global dead store removal
    { OMR::deadTreesElimination,                      OMR::IfVectorAPI                }, // cleanup after dead store removal
    { OMR::vectorAPIExpansion,                        OMR::IfVectorAPI                },
+   { OMR::osrGuardRemoval,                           OMR::IfVectorAPI                },
+
    { OMR::cheapTacticalGlobalRegisterAllocatorGroup, OMR::IfEnabled                  },
    { OMR::jProfilingValue,                           OMR::MustBeDone                 },
    { OMR::treeLowering,                              OMR::MustBeDone                 },
@@ -390,9 +496,10 @@ static const OptimizationStrategy warmStrategyOpts[] =
    { OMR::compactNullChecks,                         OMR::IfEnabled                  }, // cleanup at the end
    { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // remove dead anchors created by check/store removal
    { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // remove dead RegStores produced by previous deadTrees pass
-   { OMR::compactLocals,                             OMR::IfNotJitProfiling          }, // analysis results are invalidated by profilingGroup
+   { OMR::compactLocals,                             OMR::IfNotJitProfiling          }, // analysis results are invalidated by jitProfilingGroup
    { OMR::globalLiveVariablesForGC                                              },
-   { OMR::profilingGroup,                            OMR::IfProfiling                },
+   { OMR::jitProfilingGroup,                         OMR::IfJitProfiling             },
+   { OMR::catchBlockProfiler,                        OMR::IfExceptionHandlers   },
    { OMR::regDepCopyRemoval                                                     },
    { OMR::hotFieldMarking                                                       },
    { OMR::endOpts                                                               }
@@ -425,7 +532,8 @@ static const OptimizationStrategy reducedWarmStrategyOpts[] =
    { OMR::jProfilingRecompLoopTest,                  OMR::IfLoops                    },
    { OMR::globalDeadStoreElimination,                OMR::IfVectorAPI           }, // global dead store removal
    { OMR::deadTreesElimination,                      OMR::IfVectorAPI           }, // cleanup after dead store removal
-   { OMR::vectorAPIExpansion,                        OMR::IfVectorAPI},
+   { OMR::vectorAPIExpansion,                        OMR::IfVectorAPI           },
+   { OMR::osrGuardRemoval,                           OMR::IfVectorAPI           },
    { OMR::cheapTacticalGlobalRegisterAllocatorGroup, OMR::IfEnabled                  },
    { OMR::treeLowering,                              OMR::MustBeDone                 },
    { OMR::jProfilingValue,                           OMR::MustBeDone                 },
@@ -448,14 +556,16 @@ const OptimizationStrategy hotStrategyOpts[] =
    { OMR::earlyLocalGroup                                                 },
    { OMR::stripMiningGroup,                      OMR::IfLoops                  }, // strip mining in loops
    { OMR::loopReplicator,                        OMR::IfLoops                  }, // tail-duplication in loops
-   { OMR::blockSplitter,                         OMR::IfNews                   }, // treeSimplification + blockSplitter + VP => opportunity for EA
    { OMR::expensiveGlobalValuePropagationGroup                            },
+   { OMR::redundantGotoElimination,              OMR::IfEnabledAndNotJitProfiling     },
+   { OMR::blockSplitter,                         OMR::IfNews                   }, // treeSimplification + blockSplitter + VP => opportunity for EA
    { OMR::localCSE,                              OMR::IfVectorAPI },
    { OMR::loopCanonicalization,                  OMR::IfVectorAPI },
    { OMR::partialRedundancyEliminationGroup,     OMR::IfVectorAPI },
    { OMR::globalDeadStoreElimination,            OMR::IfVectorAPI }, // global dead store removal
    { OMR::deadTreesElimination,                  OMR::IfVectorAPI }, // cleanup after dead store removal
    { OMR::vectorAPIExpansion,                    OMR::IfVectorAPI },
+   { OMR::osrGuardRemoval,                       OMR::IfVectorAPI },
    { OMR::dataAccessAccelerator                                           },
    { OMR::osrGuardRemoval,                       OMR::IfEnabled           }, // run after calls/monents/asyncchecks have been removed
    { OMR::globalDeadStoreGroup,                                           },
@@ -464,9 +574,9 @@ const OptimizationStrategy hotStrategyOpts[] =
    { OMR::loopCanonicalizationGroup,             OMR::IfLoops                  }, // canonicalize loops (improve fall throughs)
    { OMR::inductionVariableAnalysis,             OMR::IfLoops                  },
    { OMR::redundantInductionVarElimination,      OMR::IfLoops                  },
-   { OMR::loopAliasRefinerGroup,                 OMR::IfLoops     },
+   { OMR::loopAliasRefinerGroup,                 OMR::IfLoops                  },
    { OMR::recompilationModifier,                 OMR::IfEnabledAndNotProfiling },
-   { OMR::partialRedundancyEliminationGroup                               },
+   { OMR::partialRedundancyEliminationGroup                                    },
    { OMR::globalDeadStoreElimination,            OMR::IfLoopsAndNotProfiling   },
    { OMR::inductionVariableAnalysis,             OMR::IfLoopsAndNotProfiling   },
    { OMR::loopSpecializerGroup,                  OMR::IfLoopsAndNotProfiling   },
@@ -505,10 +615,11 @@ const OptimizationStrategy hotStrategyOpts[] =
    { OMR::deadTreesElimination                                            }, // cleanup after dead store removal
    { OMR::compactNullChecks                                               }, // cleanup at the end
    { OMR::finalGlobalGroup                                                }, // done just before codegen
-   { OMR::profilingGroup,                        OMR::IfProfiling              },
-   { OMR::regDepCopyRemoval                                               },
-   { OMR::hotFieldMarking                                                       },
-   { OMR::endOpts                                                         }
+   { OMR::jitProfilingGroup,                     OMR::IfJitProfiling           },
+   { OMR::catchBlockProfiler,                    OMR::IfExceptionHandlers      },
+   { OMR::regDepCopyRemoval                                                    },
+   { OMR::hotFieldMarking                                                      },
+   { OMR::endOpts                                                              }
    };
 
 // ***************************************************************************
@@ -554,6 +665,7 @@ const OptimizationStrategy scorchingStrategyOpts[] =
    { OMR::globalDeadStoreElimination,            OMR::IfVectorAPI }, // global dead store removal
    { OMR::deadTreesElimination,                  OMR::IfVectorAPI }, // cleanup after dead store removal
    { OMR::vectorAPIExpansion,                    OMR::IfVectorAPI },
+   { OMR::osrGuardRemoval,                       OMR::IfVectorAPI },
    { OMR::loopCanonicalizationGroup,             OMR::IfLoops     }, // canonicalize loops (improve fall throughs)
    { OMR::inductionVariableAnalysis,             OMR::IfLoops     },
    { OMR::redundantInductionVarElimination,      OMR::IfLoops     },
@@ -595,7 +707,7 @@ const OptimizationStrategy scorchingStrategyOpts[] =
    { OMR::deadTreesElimination                               }, // cleanup after dead store removal
    { OMR::compactNullChecks                                  }, // cleanup at the end
    { OMR::finalGlobalGroup                                   }, // done just before codegen
-   { OMR::profilingGroup,                        OMR::IfProfiling },
+   { OMR::jitProfilingGroup,                     OMR::IfJitProfiling },
    { OMR::regDepCopyRemoval                                  },
    { OMR::hotFieldMarking                                                       },
    { OMR::endOpts                                            }
@@ -648,6 +760,7 @@ static const OptimizationStrategy AOTStrategyOpts[] =
    { OMR::globalDeadStoreElimination,            OMR::IfVectorAPI }, // global dead store removal
    { OMR::deadTreesElimination,                  OMR::IfVectorAPI }, // cleanup after dead store removal
    { OMR::vectorAPIExpansion,                    OMR::IfVectorAPI },
+   { OMR::osrGuardRemoval,                       OMR::IfVectorAPI },
    { OMR::globalDeadStoreGroup,                            },
    { OMR::globalCopyPropagation,                 OMR::IfNoLoops },
    { OMR::loopCanonicalizationGroup,             OMR::IfLoops   }, // canonicalize loops (improve fall throughs) and versioning
@@ -688,106 +801,13 @@ static const OptimizationStrategy *j9CompilationStrategies[] =
    };
 
 
-// ***************************************************************************
-//
-// Cheaper strategy for warm methods.
-//
-// ***************************************************************************
-//
-static const OptimizationStrategy cheapWarmStrategyOpts[] =
-   {
-   { OMR::trivialDeadTreeRemoval,                    OMR::IfEnabled                  },
-   { OMR::coldBlockOutlining                                                    },
-   { OMR::stringBuilderTransformer                                              },
-   { OMR::stringPeepholes                                                       }, // need stringpeepholes to catch bigdecimal patterns
-   { OMR::inlining                                                              },
-   { OMR::methodHandleInvokeInliningGroup,           OMR::IfEnabled             },
-   { OMR::staticFinalFieldFolding,                                              },
-   { OMR::osrGuardInsertion,                         OMR::MustBeDone        },
-   { OMR::osrExceptionEdgeRemoval                                               }, // most inlining is done by now
-   { OMR::jProfilingBlock                                                       },
-   { OMR::virtualGuardTailSplitter                                              }, // merge virtual guards
-   { OMR::treeSimplification                                                    },
-#ifdef TR_HOST_S390
-   { OMR::sequentialLoadAndStoreWarmGroup,           OMR::IfEnabled                  },
-#endif
-   { OMR::cheapGlobalValuePropagationGroup                                      },
-   { OMR::localCSE,                                  OMR::IfVectorAPI },
-   { OMR::dataAccessAccelerator                                                 },
-#ifdef TR_HOST_S390
-   { OMR::globalCopyPropagation,                     OMR::IfVoluntaryOSR            },
-#endif
-   { OMR::lastLoopVersionerGroup,                    OMR::IfLoops                    },
-#ifdef TR_HOST_S390
-   { OMR::globalDeadStoreElimination,                OMR::IfEnabledAndLoops          },
-   { OMR::deadTreesElimination                                                       },
-   { OMR::recompilationModifier,                     OMR::IfEnabledAndNotProfiling   },
-   { OMR::localReordering,                           OMR::IfNoLoopsOREnabledAndLoops },
-   { OMR::basicBlockOrdering,                        OMR::IfLoops                    },
-   { OMR::treeSimplification                                                         },
-   { OMR::loopReduction                                                              },
-   { OMR::blockShuffling                                                             },
-#endif
-   { OMR::localCSE,                                  OMR::IfLoopsAndNotProfiling     },
-   { OMR::idiomRecognition,                          OMR::IfLoopsAndNotProfiling     },
-   { OMR::blockSplitter                                                         },
-   { OMR::treeSimplification                                                    }, // revisit; not really required ?
-   { OMR::virtualGuardHeadMerger                                                },
-#if defined(J9VM_OPT_OPENJDK_METHODHANDLE)
-   { OMR::recognizedCallTransformer,                 OMR::MarkLastRun           },
-#endif
-   { OMR::basicBlockExtension,                       OMR::MarkLastRun                }, // extend blocks; move trees around if reqd
-   { OMR::localValuePropagationGroup                                            },
-   { OMR::explicitNewInitialization,                 OMR::IfNews                },
-   { OMR::arraycopyTransformation                                               },
-   { OMR::treeSimplification,                        OMR::IfEnabled                  },
-   { OMR::asyncCheckInsertion,                       OMR::IfNotJitProfiling          },
-   { OMR::localCSE                                                              },
-   { OMR::treeSimplification,                        OMR::MarkLastRun                },
-   { OMR::andSimplification,                         OMR::IfEnabled                  },  //clean up after versioner
-   { OMR::compactNullChecks                                                     }, // cleanup at the end
-   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // cleanup at the end
-   { OMR::globalCopyPropagation,                     OMR::IfMethodHandleInvokes      }, // Does a lot of good after methodHandleInvokeInliningGroup
-   { OMR::treesCleansing,                            OMR::IfEnabled                  },
-   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // cleanup at the end
-   { OMR::localCSE,                                  OMR::IfEnabled                  }, // common up expressions for sunk stores
-   { OMR::treeSimplification,                        OMR::IfEnabledMarkLastRun       }, // cleanup the trees after sunk store and localCSE
 
-   /** \breif
-    *      This optimization is performance critical on z Systems. On z Systems a literal pool register is blocked off
-    *      by default at the start of the compilation since materializing this address could be expensive depending on
-    *      the architecture level we are executing on. This optimization pass validates support for dynamically
-    *      materializing the literal pool address and frees up the literal pool register for register allocation.
-   */
-   { OMR::dynamicLiteralPool,                        OMR::IfNotProfiling             },
-   { OMR::samplingJProfiling                                                         },
-   { OMR::trivialBlockExtension                                                 },
-   { OMR::localCSE,                                  OMR::IfEnabled                  },  //common up lit pool refs in the same block
-   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // cleanup at the end
-   { OMR::treeSimplification,                        OMR::IfEnabledMarkLastRun       }, // Simplify non-normalized address computations introduced by prefetch insertion
-   { OMR::trivialDeadTreeRemoval,                    OMR::IfEnabled                  }, // final cleanup before opcode expansion
-   { OMR::jProfilingRecompLoopTest,                  OMR::IfLoops                    },
-   { OMR::globalDeadStoreElimination,                OMR::IfVectorAPI                }, // global dead store removal
-   { OMR::deadTreesElimination,                      OMR::IfVectorAPI                }, // cleanup after dead store removal
-   { OMR::vectorAPIExpansion,                        OMR::IfVectorAPI                },
-   { OMR::cheapTacticalGlobalRegisterAllocatorGroup, OMR::IfEnabled                  },
-   { OMR::jProfilingValue,                           OMR::MustBeDone                 },
-   { OMR::treeLowering,                              OMR::MustBeDone                 },
-   { OMR::globalDeadStoreGroup,                                                 },
-   { OMR::compactNullChecks,                         OMR::IfEnabled                  }, // cleanup at the end
-   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // remove dead anchors created by check/store removal
-   { OMR::deadTreesElimination,                      OMR::IfEnabled                  }, // remove dead RegStores produced by previous deadTrees pass
-   { OMR::redundantGotoElimination,                  OMR::IfEnabledAndNotJitProfiling }, // dead store and dead tree elimination may have left empty blocks
-   { OMR::compactLocals,                             OMR::IfNotJitProfiling          }, // analysis results are invalidated by profilingGroup
-   { OMR::globalLiveVariablesForGC                                              },
-   { OMR::profilingGroup,                            OMR::IfProfiling                },
-   { OMR::regDepCopyRemoval                                                     },
-   { OMR::hotFieldMarking                                                       },
-   { OMR::endOpts                                                               }
-   };
-
-
-static const OptimizationStrategy profilingOpts[] =
+// **********************************************************
+//
+// Transformations that are specific to JitProfiling mode
+//
+// **********************************************************
+static const OptimizationStrategy jitProfilingOpts[] =
    {
    { OMR::profileGenerator,          OMR::MustBeDone },
    { OMR::deadTreesElimination,      OMR::IfEnabled  },
@@ -891,6 +911,8 @@ J9::Optimizer::Optimizer(TR::Compilation *comp, TR::ResolvedMethodSymbol *method
       new (comp->allocator()) TR::OptimizationManager(self(), TR_HotFieldMarking::create, OMR::hotFieldMarking);
    _opts[OMR::vectorAPIExpansion] =
       new (comp->allocator()) TR::OptimizationManager(self(), TR_VectorAPIExpansion::create, OMR::vectorAPIExpansion);
+   _opts[OMR::catchBlockProfiler] =
+      new (comp->allocator()) TR::OptimizationManager(self(), TR::CatchBlockProfiler::create, OMR::catchBlockProfiler);
    // NOTE: Please add new J9 optimizations here!
 
    // initialize additional J9 optimization groups
@@ -921,8 +943,8 @@ J9::Optimizer::Optimizer(TR::Compilation *comp, TR::ResolvedMethodSymbol *method
       new (comp->allocator()) TR::OptimizationManager(self(), NULL, OMR::signExtendLoadsGroup, signExtendLoadsOpts);
    _opts[OMR::loopSpecializerGroup] =
       new (comp->allocator()) TR::OptimizationManager(self(), NULL, OMR::loopSpecializerGroup, loopSpecializerOpts);
-   _opts[OMR::profilingGroup] =
-      new (comp->allocator()) TR::OptimizationManager(self(), NULL, OMR::profilingGroup, profilingOpts);
+   _opts[OMR::jitProfilingGroup] =
+      new (comp->allocator()) TR::OptimizationManager(self(), NULL, OMR::jitProfilingGroup, jitProfilingOpts);
    _opts[OMR::sequentialLoadAndStoreColdGroup] =
       new (comp->allocator()) TR::OptimizationManager(self(), NULL, OMR::sequentialLoadAndStoreColdGroup, sequentialLoadAndStoreColdOpts);
    _opts[OMR::sequentialLoadAndStoreWarmGroup] =
@@ -1008,14 +1030,7 @@ J9::Optimizer::optimizationStrategy(TR::Compilation *c)
       }
 
    TR_Hotness strategy = c->getMethodHotness();
-   if (strategy == warm && !c->getOption(TR_DisableCheapWarmOpts))
-      {
-      return cheapWarmStrategyOpts;
-      }
-   else
-      {
-      return j9CompilationStrategies[strategy];
-      }
+   return j9CompilationStrategies[strategy];
    }
 
 
