@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2021, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2021
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "optimizer/MethodHandleTransformer.hpp"
@@ -367,32 +367,34 @@ TR_MethodHandleTransformer::getObjectInfoOfNode(TR::Node* node)
       {
       auto rm = symbol->castToMethodSymbol()->getMandatoryRecognizedMethod();
       switch (rm)
-        {
-        case TR::java_lang_invoke_DirectMethodHandle_internalMemberName:
-        case TR::java_lang_invoke_DirectMethodHandle_internalMemberNameEnsureInit:
-           {
-           auto mhIndex = getObjectInfoOfNode(node->getFirstArgument());
-           if (knot && isKnownObject(mhIndex) && !knot->isNull(mhIndex))
-              {
-              auto mnIndex = comp()->fej9()->getMemberNameFieldKnotIndexFromMethodHandleKnotIndex(comp(), mhIndex, "member");
-              if (trace())
-                 traceMsg(comp(), "Get DirectMethodHandle.member known object %d, update node n%dn known object\n", mnIndex, node->getGlobalIndex());
-              node->setKnownObjectIndex(mnIndex);
-              return mnIndex;
-              }
-           }
-        case TR::java_lang_invoke_DirectMethodHandle_constructorMethod:
-           {
-           auto mhIndex = getObjectInfoOfNode(node->getFirstArgument());
-           if (knot && isKnownObject(mhIndex) && !knot->isNull(mhIndex))
-              {
-              auto mnIndex = comp()->fej9()->getMemberNameFieldKnotIndexFromMethodHandleKnotIndex(comp(), mhIndex, "initMethod");
-              if (trace())
-                 traceMsg(comp(), "Get DirectMethodHandle.initMethod known object %d, update node n%dn known object\n", mnIndex, node->getGlobalIndex());
-              node->setKnownObjectIndex(mnIndex);
-              return mnIndex;
-              }
-           }
+         {
+         case TR::java_lang_invoke_DirectMethodHandle_internalMemberName:
+         case TR::java_lang_invoke_DirectMethodHandle_internalMemberNameEnsureInit:
+            {
+            auto mhIndex = getObjectInfoOfNode(node->getFirstArgument());
+            if (knot && isKnownObject(mhIndex) && !knot->isNull(mhIndex))
+               {
+               auto mnIndex = comp()->fej9()->getMemberNameFieldKnotIndexFromMethodHandleKnotIndex(comp(), mhIndex, "member");
+               if (trace())
+                  traceMsg(comp(), "Get DirectMethodHandle.member known object %d, update node n%dn known object\n", mnIndex, node->getGlobalIndex());
+               node->setKnownObjectIndex(mnIndex);
+               return mnIndex;
+               }
+            break;
+            }
+         case TR::java_lang_invoke_DirectMethodHandle_constructorMethod:
+            {
+            auto mhIndex = getObjectInfoOfNode(node->getFirstArgument());
+            if (knot && isKnownObject(mhIndex) && !knot->isNull(mhIndex))
+               {
+               auto mnIndex = comp()->fej9()->getMemberNameFieldKnotIndexFromMethodHandleKnotIndex(comp(), mhIndex, "initMethod");
+               if (trace())
+                  traceMsg(comp(), "Get DirectMethodHandle.initMethod known object %d, update node n%dn known object\n", mnIndex, node->getGlobalIndex());
+               node->setKnownObjectIndex(mnIndex);
+               return mnIndex;
+               }
+            break;
+            }
 
 #if defined(J9VM_OPT_OPENJDK_METHODHANDLE)
          case TR::java_lang_invoke_DelegatingMethodHandle_getTarget:
@@ -404,6 +406,24 @@ TR_MethodHandleTransformer::getObjectInfoOfNode(TR::Node* node)
                comp(), dmhIndex, trace());
             }
 #endif
+         case TR::java_lang_invoke_Invokers_checkVarHandleGenericType:
+            {
+            auto vhIndex = getObjectInfoOfNode(node->getFirstArgument());
+            auto adIndex = getObjectInfoOfNode(node->getLastChild());
+            if (knot
+               && isKnownObject(adIndex)
+               && isKnownObject(vhIndex)
+               && !knot->isNull(vhIndex)
+               && !knot->isNull(adIndex))
+               {
+               auto mhIndex = comp()->fej9()->getMethodHandleTableEntryIndex(comp(), vhIndex, adIndex);
+               if (trace())
+                 traceMsg(comp(), "Invokers_checkVarHandleGenericType with known VarHandle object %d, updating node n%dn with known MH object %d from MH table\n", vhIndex, node->getGlobalIndex(), mhIndex);
+               node->setKnownObjectIndex(mhIndex);
+               return mhIndex;
+               }
+            break;
+            }
 
          default:
             break;
@@ -515,6 +535,9 @@ void TR_MethodHandleTransformer::visitCall(TR::TreeTop* tt, TR::Node* node)
       case TR::java_lang_invoke_Invokers_checkExactType:
          process_java_lang_invoke_Invokers_checkExactType(tt, node);
          break;
+      case TR::java_lang_invoke_Invokers_checkVarHandleGenericType:
+         process_java_lang_invoke_Invokers_checkVarHandleGenericType(tt, node);
+         break;
 
       default:
          break;
@@ -705,6 +728,9 @@ TR_MethodHandleTransformer::process_java_lang_invoke_Invokers_checkExactType(TR:
                                                                                          true,
                                                                                          "java/lang/invoke/MethodHandle.type Ljava/lang/invoke/MethodType;");
    auto handleTypeNode = TR::Node::createWithSymRef(node, comp()->il.opCodeForIndirectLoad(TR::Address), 1, methodHandleNode, typeSymRef);
+   auto nullCheckSymRef = comp()->getSymRefTab()->findOrCreateNullCheckSymbolRef(comp()->getMethodSymbol());
+   auto nullCheckNode = TR::Node::createWithSymRef(node, TR::NULLCHK, 1, handleTypeNode, nullCheckSymRef);
+   tt->insertBefore(TR::TreeTop::create(comp(), nullCheckNode));
    auto cmpEqNode = TR::Node::create(node, TR::acmpeq, 2, expectedTypeNode, handleTypeNode);
    TR::Node* zerochkNode = TR::Node::createWithSymRef(TR::ZEROCHK, 1, 1,
                                                        cmpEqNode,
@@ -750,3 +776,158 @@ TR_MethodHandleTransformer::process_java_lang_invoke_Invokers_checkCustomized(TR
       TR::TransformUtil::transformCallNodeToPassThrough(this, node, tt, node->getFirstArgument());
       }
    }
+
+/*
+When the VarHandle object and the AccessDescriptor objects are known, we can perform a compile-time lookup of the corresponding
+entry in the VarHandle's MethodHandle table. This function replaces Invokers.checkVarHandleGenericType call nodes with a
+aloadi node performing the load of the MethodHandle object that would have been returned by Invokers.checkVarHandleGenericType.
+This is achieved through inserting trees that perform loads of the field containing the MethodHandle table. The various checks that
+Invokers.checkVarHandleGenericType would have performed are done in TR_J9VMBase::getMethodHandleTableEntryIndex which returns the
+object index of the MethodHandle entry in the table if the checks succeed, following which the Invokers.checkVarHandleGenericType
+can be folded.
+
+Blocks before transformation ==>
+start Block_A
+...
+treetop
+acall  java/lang/invoke/Invokers.checkVarHandleGenericType(Ljava/lang/invoke/VarHandle;Ljava/lang/invoke/VarHandle$AccessDescriptor;)Ljava/lang/invoke/MethodHandle;
+   aload  <VarHandle>
+   aload  <AccessDescriptor>
+...
+astore  <auto slot n>
+   ==>acall <MethodHandle returned by checkVarHandleGenericType>
+...
+end Block_A
+
+Blocks after transformation (JAVA_SPEC_VERSION <= 17, compressedRefs) ==>
+start Block_A
+...
+compressedRefs
+  aloadi  java/lang/invoke/VarHandle.typesAndInvokers Ljava/lang/invoke/VarHandle$TypesAndInvokers;
+    aload  (node known obj#) <VarHandle>
+  lconst 0 (highWordZero X==0 X>=0 X<=0 )
+compressedRefs
+  aloadi  java/lang/invoke/VarHandle$TypesAndInvokers.methodHandle_table [Ljava/lang/invoke/MethodHandle;
+    ==>aloadi
+compressedRefs
+  aloadi  unknown field [# Shadow] (known obj#) <MethodHandle>
+    aladd (internalPtr )
+        ==>aloadi  <VarHandle$TypesAndInvokers.methodHandle_table>
+      ladd (X>=0 )
+        lshl
+          i2l
+            iconst 0 (X==0 X>=0 X<=0 )
+          iconst 2 (X!=0 X>=0 )
+        lconst 8 (highWordZero X!=0 X>=0 )
+  lconst 0 (highWordZero X==0 X>=0 X<=0 )
+treetop
+  ==>aloadi <MethodHandle object>
+astore  <auto slot n>
+  ==>aloadi <MethodHandle object>
+...
+end Block_A
+
+For JDK 21 and above, the trees would look similar to the above, but not require an additional load of the
+VarHandle$TypesAndInvokers field as the MethodHandle table exists as a field of the VarHandle object.
+*/
+void
+TR_MethodHandleTransformer::process_java_lang_invoke_Invokers_checkVarHandleGenericType(TR::TreeTop* tt, TR::Node* node)
+   {
+   static const bool disableFoldingVHGenType = feGetEnv("TR_disableFoldingVHGenType") != NULL;
+   if (disableFoldingVHGenType) return;
+
+   auto vhIndex = getObjectInfoOfNode(node->getFirstArgument());
+   auto adIndex = getObjectInfoOfNode(node->getLastChild());
+   auto mhIndex = TR::KnownObjectTable::UNKNOWN;
+   auto knot = comp()->getKnownObjectTable();
+   int32_t mhEntryIndex = -1;
+   if (knot
+      && isKnownObject(adIndex)
+      && isKnownObject(vhIndex)
+      && !knot->isNull(vhIndex)
+      && !knot->isNull(adIndex))
+      {
+      mhIndex = comp()->fej9()->getMethodHandleTableEntryIndex(comp(), vhIndex, adIndex);
+      mhEntryIndex = comp()->fej9()->getVarHandleAccessDescriptorMode(comp(), adIndex);
+
+      if (mhIndex != TR::KnownObjectTable::UNKNOWN && mhEntryIndex >= 0)
+         {
+         if (trace())
+            traceMsg(comp(), "Replacing Invokers.checkVarHandleGenericType call at n%dn with its resulting MH table entry. VarHandle object %d, MH object %d\n", node->getGlobalIndex(), vhIndex, mhIndex);
+
+         // obtain symref(s) for creating load for the MH table base
+#if JAVA_SPEC_VERSION <= 17
+         uint32_t typesAndInvokersOffset = comp()->fej9()->getInstanceFieldOffsetIncludingHeader("Ljava/lang/invoke/VarHandle;", "typesAndInvokers", "Ljava/lang/invoke/VarHandle$TypesAndInvokers;", comp()->getCurrentMethod());
+         const char * tisFieldNameAndSig = "java/lang/invoke/VarHandle.typesAndInvokers Ljava/lang/invoke/VarHandle$TypesAndInvokers;";
+         TR::SymbolReference *tisSymRef = comp()->getSymRefTab()->findOrFabricateShadowSymbol(comp()->getMethodSymbol(),
+                                                                                 TR::Symbol::Java_lang_invoke_VarHandle_typesAndInvokers,
+                                                                                 TR::Address,
+                                                                                 typesAndInvokersOffset,
+                                                                                 false,
+                                                                                 false,
+                                                                                 false,
+                                                                                 tisFieldNameAndSig);
+         TR::Node *tisNode = TR::Node::createWithSymRef(TR::aloadi, 1, 1, node->getFirstArgument(), tisSymRef);
+         tisNode->copyByteCodeInfo(node);
+         if (comp()->useCompressedPointers())
+            tt->insertBefore(TR::TreeTop::create(comp(), TR::Node::createCompressedRefsAnchor(tisNode)));
+
+         uint32_t mhTableOffset = comp()->fej9()->getInstanceFieldOffsetIncludingHeader("Ljava/lang/invoke/VarHandle$TypesAndInvokers;", "methodHandle_table", "[Ljava/lang/invoke/MethodHandle;", comp()->getCurrentMethod());
+         const char * mhTableFieldNameAndSig = "java/lang/invoke/VarHandle$TypesAndInvokers.methodHandle_table [Ljava/lang/invoke/MethodHandle;";
+         TR::Node *mhTableHolder = tisNode;
+         bool mhTableIsFinal = true;
+#else
+         uint32_t mhTableOffset = comp()->fej9()->getInstanceFieldOffsetIncludingHeader("Ljava/lang/invoke/VarHandle;", "methodHandleTable", "[Ljava/lang/invoke/MethodHandle;", comp()->getCurrentMethod());
+         const char * mhTableFieldNameAndSig = "java/lang/invoke/VarHandle.methodHandleTable [Ljava/lang/invoke/MethodHandle;";
+         TR::Node *mhTableHolder = node->getFirstArgument();
+         bool mhTableIsFinal = false;
+#endif /* JAVA_SPEC_VERSION <= 17 */
+         TR::SymbolReference *mhTableSymRef = comp()->getSymRefTab()->findOrFabricateShadowSymbol(comp()->getMethodSymbol(),
+                                                                                 TR::Symbol::Java_lang_invoke_VarHandle_methodHandleTable,
+                                                                                 TR::Address,
+                                                                                 mhTableOffset,
+                                                                                 false,
+                                                                                 false,
+                                                                                 mhTableIsFinal,
+                                                                                 mhTableFieldNameAndSig);
+         TR::Node *mhTableNode = TR::Node::createWithSymRef(TR::aloadi, 1, 1, mhTableHolder, mhTableSymRef);
+         mhTableNode->copyByteCodeInfo(node);
+         if (comp()->useCompressedPointers())
+            tt->insertBefore(TR::TreeTop::create(comp(), TR::Node::createCompressedRefsAnchor(mhTableNode)));
+
+         // create node representing index for the load
+         TR::Node *arrayIndexNode = TR::Node::create(TR::iconst, 0, mhEntryIndex);
+         arrayIndexNode->copyByteCodeInfo(node);
+
+         // calculate element address, transform call node into load from array, and improve MH symref with known object info
+         TR::Node *mhAddressNode = J9::TransformUtil::calculateElementAddress(comp(), mhTableNode, arrayIndexNode, TR::Address);
+         mhAddressNode->copyByteCodeInfo(node);
+
+         anchorAllChildren(node, tt);
+         node->removeAllChildren();
+         TR::Node::recreateWithSymRef(node, TR::aloadi, comp()->getSymRefTab()->findOrCreateArrayShadowSymbolRef(TR::Address, mhTableNode));
+         node->setNumChildren(1);
+         node->setAndIncChild(0, mhAddressNode);
+         TR::SymbolReference *improvedMHSymRef = comp()->getSymRefTab()->findOrCreateSymRefWithKnownObject(node->getSymbolReference(), mhIndex);
+         node->setSymbolReference(improvedMHSymRef);
+
+         // Insert spine check for arraylets
+         if (TR::Compiler->om.canGenerateArraylets())
+            {
+            TR::SymbolReference * spineCHKSymRef = comp()->getSymRefTab()->findOrCreateArrayBoundsCheckSymbolRef(comp()->getMethodSymbol());
+            TR::Node * spineCHKNode = TR::Node::create(node, TR::SpineCHK, 3);
+            spineCHKNode->setAndIncChild(0, node);
+            spineCHKNode->setAndIncChild(1, mhTableNode);
+            spineCHKNode->setAndIncChild(2, arrayIndexNode);
+            spineCHKNode->setSymbolReference(spineCHKSymRef);
+            spineCHKNode->setSpineCheckWithArrayElementChild(true, comp());
+            tt->insertBefore(TR::TreeTop::create(comp(), spineCHKNode));
+            }
+
+         if (comp()->useCompressedPointers())
+            tt->insertBefore(TR::TreeTop::create(comp(), TR::Node::createCompressedRefsAnchor(node)));
+
+         }
+      else return;
+      }
+   }
