@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j9cfg.h"
@@ -77,6 +77,13 @@ J9::Power::CodeGenerator::initialize()
       cg->setSupportsInlineConcurrentLinkedQueue();
       }
 
+   static bool disableInlineVectorizedMismatch = feGetEnv("TR_disableInlineVectorizedMismatch") != NULL;
+   if (cg->getSupportsArrayCmpLen() &&
+         !disableInlineVectorizedMismatch)
+      {
+      cg->setSupportsInlineVectorizedMismatch();
+      }
+
    cg->setSupportsNewInstanceImplOpt();
 
    static char *disableMonitorCacheLookup = feGetEnv("TR_disableMonitorCacheLookup");
@@ -91,9 +98,41 @@ J9::Power::CodeGenerator::initialize()
       !TR::Compiler->om.canGenerateArraylets())
       cg->setSupportsInlineStringIndexOf();
 
+   static bool disableStringInflateIntrinsic = feGetEnv("TR_DisableStringInflateIntrinsic") != NULL;
+   if (comp->target().cpu.isAtLeast(OMR_PROCESSOR_PPC_P8) && comp->target().cpu.supportsFeature(OMR_FEATURE_PPC_HAS_VSX) &&
+      comp->target().is64Bit() &&
+      !TR::Compiler->om.canGenerateArraylets() &&
+      !disableStringInflateIntrinsic)
+      cg->setSupportsInlineStringLatin1Inflate();
+
+   static bool disableCASInlining = feGetEnv("TR_DisableCASInlining") != NULL;
+   if (!disableCASInlining)
+      {
+      cg->setSupportsInlineUnsafeCompareAndSet();
+      }
+
+   static bool disableCAEInlining = feGetEnv("TR_DisableCAEInlining") != NULL;
+   if (!disableCAEInlining)
+      {
+      cg->setSupportsInlineUnsafeCompareAndExchange();
+      }
+
    if (!comp->getOption(TR_DisableReadMonitors))
       cg->setSupportsReadOnlyLocks();
 
+   if (!TR::Compiler->om.canGenerateArraylets()
+         && comp->target().cpu.isAtLeast(OMR_PROCESSOR_PPC_P8)
+         && comp->target().cpu.supportsFeature(OMR_FEATURE_PPC_HAS_VSX)
+         && !comp->compileRelocatableCode()
+#ifdef J9VM_OPT_JITSERVER
+         && !comp->isOutOfProcessCompilation()
+#endif
+      )
+      {
+      cg->setSupportsInlineStringHashCode();
+      cg->setSupportsInlineVectorizedHashCode();
+      }
+
    static bool disableTLHPrefetch = (feGetEnv("TR_DisableTLHPrefetch") != NULL);
 
    // Enable software prefetch of the TLH and configure the TLH prefetching
@@ -104,6 +143,21 @@ J9::Power::CodeGenerator::initialize()
       cg->setEnableTLHPrefetching();
       }
 
+   static bool disableInlineStringCodingHasNegatives =
+      feGetEnv("TR_DisableInlineStringCodingHasNegatives") != NULL;
+   static bool disableInlineStringCodingCountPositives =
+      feGetEnv("TR_DisableInlineStringCodingCountPositives") != NULL;
+   if (comp->target().cpu.isAtLeast(OMR_PROCESSOR_PPC_P8) &&
+      comp->target().cpu.supportsFeature(OMR_FEATURE_PPC_HAS_VSX) &&
+      !TR::Compiler->om.canGenerateArraylets() &&
+      comp->target().is64Bit())
+      {
+      if (!disableInlineStringCodingHasNegatives)
+         cg->setSupportsInlineStringCodingHasNegatives();
+      if (!disableInlineStringCodingCountPositives)
+         cg->setSupportsInlineStringCodingCountPositives();
+      }
+
    //This env-var does 3 things:
    // 1. Prevents batch clear in frontend/j9/rossa.cpp
    // 2. Prevents all allocations to nonZeroTLH
@@ -118,6 +172,12 @@ J9::Power::CodeGenerator::initialize()
       cg->setIsDualTLH();
       }
 
+   static bool disableInlineMath_MaxMin_FD = feGetEnv("TR_disableInlineMaxMin") != NULL;
+   if (!disableInlineMath_MaxMin_FD && comp->target().cpu.isAtLeast(OMR_PROCESSOR_PPC_P7))
+      {
+      cg->setSupportsInlineMath_MaxMin_FD();
+      }
+
    /*
     * "Statically" initialize the FE-specific tree evaluator functions.
     * This code only needs to execute once per JIT lifetime.
@@ -353,6 +413,17 @@ bool J9::Power::CodeGenerator::suppressInliningOfRecognizedMethod(TR::Recognized
           }
       }
 
+   /* Need this check for CRC32C update* methods so that this doesn't get inlined.
+    * Unlike CRC32's update* which are JNI methods, CRC32C have Java implementation
+    * and therefore we need to prevent the inliner from eliminating the calls so
+    * we can redirect them to our optimized helpers.
+    */
+   if (method == TR::java_util_zip_CRC32C_updateBytes ||
+       method == TR::java_util_zip_CRC32C_updateDirectByteBuffer)
+      {
+      return true;
+      }
+
    return false;
 }
 
@@ -559,7 +630,7 @@ J9::Power::CodeGenerator::insertPrefetchIfNecessary(TR::Node *node, TR::Register
                      generateTrg1MemInstruction(self(), TR::InstOpCode::lwz, node, temp3Reg, TR::MemoryReference::createWithDisplacement(self(), pointerReg, (int32_t)(TR::Compiler->om.sizeofReferenceField()*2), 4));
                   }
 
-               if (comp->getOptions()->getHeapBase() != NULL)
+               if (comp->getOptions()->getHeapBase() != 0)
                   {
                   loadAddressConstant(self(), comp->compileRelocatableCode(), node, (intptr_t)(comp->getOptions()->getHeapBase()), tempReg);
                   generateTrg1Src2Instruction(self(), TR::InstOpCode::cmpl4, node, condReg, temp3Reg, tempReg);
