@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include <algorithm>
@@ -208,7 +208,7 @@ OMR::RuntimeAssumption::addToRAT(TR_PersistentMemory * persistentMemory, TR_Runt
    }
 
 void
-OMR::RuntimeAssumption::dumpInfo(char *subclassName)
+OMR::RuntimeAssumption::dumpInfo(const char *subclassName)
    {
    TR_VerboseLog::write("%s@%p: key=%p", subclassName, this, _key);
    }
@@ -385,7 +385,7 @@ void TR_RuntimeAssumptionTable::addAssumption(OMR::RuntimeAssumption *a, TR_Runt
       }
    }
 
-/** 
+/**
  * Mark an assumption for future detach and reclaiming from the RAT
  * @param assumption The assumption to be marked for removal
  * Once all assumptions are marked a call to reclaimMarkedFromRAT() will free
@@ -405,9 +405,9 @@ void TR_RuntimeAssumptionTable::markForDetachFromRAT(OMR::RuntimeAssumption *ass
 
 /**
  * Traverse the entire RAT detaching and reclaiming all marked assumptions.
- * This assumes that the assumptions have already been detached from the 
+ * This assumes that the assumptions have already been detached from the
  * metadata's linked list. Only RAT 'kinds' that have any marked assumptions
- * will be traversed, and only the hashtable linked-lists that have a non-zero 
+ * will be traversed, and only the hashtable linked-lists that have a non-zero
  * marked for detach count will be traversed.
  */
 void TR_RuntimeAssumptionTable::reclaimMarkedAssumptionsFromRAT(int32_t cleanupCount)
@@ -499,14 +499,14 @@ void TR_RuntimeAssumptionTable::markAssumptionsAndDetach(void * md, bool reclaim
             #if defined(PROD_WITH_ASSUMES) || defined(DEBUG)
             TR_RuntimeAssumptionKind kind = cursor->getAssumptionKind();
             TR_ASSERT(kind == RuntimeAssumptionOnClassRedefinitionPIC ||
-                      kind == RuntimeAssumptionOnClassRedefinitionUPIC || 
+                      kind == RuntimeAssumptionOnClassRedefinitionUPIC ||
                       kind == RuntimeAssumptionOnClassRedefinitionNOP,
                "non redefinition assumption (RA=%p kind=%d key=%p) left after metadata reclamation\n",
                cursor, kind, cursor->getKey());
             #endif
             }
          }
-      
+
       if (!entriesRemain)
          {
          sentry->markForDetach();
@@ -556,7 +556,7 @@ void TR_RuntimeAssumptionTable::reclaimAssumptions(OMR::RuntimeAssumption **sent
                 kind != RuntimeAssumptionOnClassRedefinitionNOP)
                {
                fprintf(stderr, "%d assumptions were left after metadata %p assumption reclaiming\n", numAssumptionsNotReclaimed, metaData);
-               fprintf(stderr, "RA=%p kind=%d key=%p assumingPC=%p\n", cursor, cursor->getAssumptionKind(), cursor->getKey(), cursor->getFirstAssumingPC());
+               fprintf(stderr, "RA=%p kind=%d key=%p assumingPC=%p\n", cursor, cursor->getAssumptionKind(), (void*) cursor->getKey(), cursor->getFirstAssumingPC());
                TR_ASSERT(false, "non redefinition assumptions left after metadata reclamation\n");
                }
             cursor = cursor->getNextAssumptionForSameJittedBody();
@@ -717,12 +717,33 @@ void TR_UnloadedClassPicSite::compensate(TR_FrontEnd *, bool isSMP, void *)
             }
 
          }
+      else if (((*((uint16_t *)cursor) & (uint16_t)0xFF0F) == (uint16_t)0xC00F))
+         {
+         //if LLILF we need to convert to LGFI to get sign extension
+         *(cursor+1) ^= (int8_t)0x0E;
+         }
       }
 #endif
       }
    else
       {
-      *(int64_t *)_picLocation = -1;
+#if (defined(TR_HOST_64BIT) && defined(TR_HOST_S390))
+      //Check if LLILF followed by IIHF
+      int8_t * cursor = (int8_t *)_picLocation - 2;
+      //Mask out register bits from opcodes
+      uint16_t opcode = *(uint16_t *)cursor & (uint16_t)0xFF0F;
+      uint16_t next_opcode = *(uint16_t *)(cursor + 6) & (uint16_t)0xFF0F;
+      if ((opcode == (uint16_t)0xC00F) && (next_opcode == (uint16_t)0xC008))
+         {
+         //patch both immediates
+         *(int32_t *)_picLocation     = -1;
+         *(int32_t *)(_picLocation+6) = -1;
+         }
+      else
+#endif
+         {
+         *(int64_t *)_picLocation = -1;
+         }
       }
 #elif defined(TR_HOST_POWER)
    // On PPC, the patching is on a 4-byte entity regardless of 32/64bit JIT
@@ -740,14 +761,10 @@ void TR_UnloadedClassPicSite::compensate(TR_FrontEnd *, bool isSMP, void *)
 #elif defined(TR_HOST_ARM64)
    // On aarch64, we use constant data snippet for class unloading pic site
    extern void arm64CodeSync(unsigned char *codeStart, unsigned int codeSize);
-#if defined(OSX)
-   pthread_jit_write_protect_np(0);
-#endif
+   omrthread_jit_write_protect_disable();
    *(int64_t *)_picLocation = -1;
    arm64CodeSync(_picLocation, 8);
-#if defined(OSX)
-   pthread_jit_write_protect_np(1);
-#endif
+   omrthread_jit_write_protect_enable();
 #else
    //   TR_ASSERT(0, "unloaded class PIC patching is not implemented on this platform yet");
 #endif
@@ -916,13 +933,9 @@ TR_RuntimeAssumptionTable::notifyClassRedefinitionEvent(TR_FrontEnd *vm, bool is
                if (reportDetails)
                   TR_VerboseLog::writeLineLocked(TR_Vlog_RA, "old=%p resolved=%p @ %p patching new=%p", initialKey, resolvedKey, pic_cursor->getPicLocation(), *(uintptr_t *)(pic_cursor->getPicLocation()));
 
-#if defined(OSX) && defined(AARCH64)
-               pthread_jit_write_protect_np(0);
-#endif
+               omrthread_jit_write_protect_disable();
                *(uintptr_t *)(pic_cursor->getPicLocation()) = (uintptr_t)newKey;
-#if defined(OSX) && defined(AARCH64)
-               pthread_jit_write_protect_np(1);
-#endif
+               omrthread_jit_write_protect_enable();
                }
             }
          pic_cursor = pic_next;
@@ -1164,14 +1177,29 @@ void TR_RedefinedClassPicSite::compensate(TR_FrontEnd *, bool isSMP, void *newKe
                   }
                }
             }
-
          }
       }
 #endif
       }
    else
       {
-      *(int64_t *)_picLocation = (uintptr_t)newKey;
+#if (defined(TR_HOST_64BIT) && defined(TR_HOST_S390))
+      //Check if LLILF followed by IIHF
+      int8_t * cursor = (int8_t *)_picLocation - 2;
+      //Mask out register bits from opcodes
+      uint16_t opcode = *(uint16_t *)cursor & (uint16_t)0xFF0F;
+      uint16_t next_opcode = *(uint16_t *)(cursor + 6) & (uint16_t)0xFF0F;
+      if ((opcode == (uint16_t)0xC00F) && (next_opcode == (uint16_t)0xC008))
+         {
+         //patch both immediates
+         *(int32_t *)_picLocation     = (uintptr_t)newKey;
+         *(int32_t *)(_picLocation+6) = ((uintptr_t)newKey) >> 32;
+         }
+      else
+#endif
+         {
+         *(int64_t *)_picLocation = (uintptr_t)newKey;
+         }
       }
 #elif defined(TR_HOST_POWER)
    extern void ppcCodeSync(unsigned char *codeStart, unsigned int codeSize);
@@ -1185,13 +1213,9 @@ void TR_RedefinedClassPicSite::compensate(TR_FrontEnd *, bool isSMP, void *newKe
 #elif defined(TR_HOST_ARM)
    *(int32_t *)_picLocation = (uintptr_t)newKey;
 #elif defined(TR_HOST_ARM64)
-#if defined(OSX)
-   pthread_jit_write_protect_np(0);
-#endif
+   omrthread_jit_write_protect_disable();
    *(int64_t *)_picLocation = (uintptr_t)newKey;
-#if defined(OSX)
-   pthread_jit_write_protect_np(1);
-#endif
+   omrthread_jit_write_protect_enable();
 #else
    //   TR_ASSERT(0, "redefined class PIC patching is not implemented on this platform yet");
 #endif
@@ -1278,7 +1302,7 @@ J9::PersistentInfo::isUnloadedClass(
       auto clientData = TR::compInfoPT->getClientData();
       OMR::CriticalSection isUnloadedClass(clientData->getROMMapMonitor());
       return clientData->getUnloadedClassAddresses().mayContain((uintptr_t)v);
-      } 
+      }
 #endif
    OMR::CriticalSection isUnloadedClass(assumptionTableMutex);
    bool result = (_unloadedClassAddresses && _unloadedClassAddresses->mayContain((uintptr_t)v));
@@ -1344,9 +1368,9 @@ void TR_AddressSet::setRanges(const std::vector<TR_AddressRange> &ranges)
    }
 #endif
 
-void TR_AddressSet::trace(char *format, ...)
+void TR_AddressSet::trace(const char *format, ...)
    {
-   static char *env = feGetEnv("TR_traceUnloadedClassRanges");
+   static const char *env = feGetEnv("TR_traceUnloadedClassRanges");
    if (env)
       {
       va_list args;
@@ -1357,7 +1381,7 @@ void TR_AddressSet::trace(char *format, ...)
       }
    }
 
-void TR_AddressSet::traceDetails(char *format, ...)
+void TR_AddressSet::traceDetails(const char *format, ...)
    {
    if (enableTraceDetails())
       {
