@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2020 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,19 +15,30 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include <algorithm>
-#include "runtime/DataCache.hpp"
-#include "infra/Monitor.hpp"
-#include "infra/CriticalSection.hpp"
+#include "OMR/Bytes.hpp"
+#include "control/CompilationRuntime.hpp"
 #include "control/Options.hpp"
 #include "control/Options_inlines.hpp"
-#include "env/VMJ9.h"
 #include "env/VerboseLog.hpp"
+#include "env/VMJ9.h"
+#include "infra/CriticalSection.hpp"
+#include "infra/Monitor.hpp"
+#include "runtime/DataCache.hpp"
+#ifdef LINUX
+#include <sys/mman.h> // for madvise
+#ifndef MADV_NOHUGEPAGE
+#define MADV_NOHUGEPAGE  15
+#endif // MADV_NOHUGEPAGE
+#ifndef MADV_PAGEOUT
+#define MADV_PAGEOUT     21
+#endif // MADV_PAGEOUT
+#endif // LINUX
 
 //--------------------- DataCacheManager ----------------
 
@@ -150,6 +161,7 @@ TR_DataCacheManager::TR_DataCacheManager(J9JITConfig *jitConfig, TR::Monitor *mo
    {
    TR_ASSERT( (_quantumSize % sizeof(UDATA)) == 0, "Chunks need to be aligned with pointer size");
    TR_ASSERT( (_quantumSize * _minQuanta) >= sizeof(Allocation), "Allocation won't fit in free blocks" );
+   _disclaimEnabled = TR::Options::getCmdLineOptions()->getOption(TR_DisableDataCacheDisclaiming) ? false : true;
    // Add trace point if we have disabled reclamation
 #if defined(DATA_CACHE_DEBUG)
    if (!_newImplementation)
@@ -245,13 +257,29 @@ TR_DataCache* TR_DataCacheManager::allocateNewDataCache(uint32_t minimumSize)
          if (dataCache)
             {
             // Compute the size for the segment
-            int32_t segSize = std::max(_jitConfig->dataCacheKB * 1024, static_cast<UDATA>(minimumSize));
+            UDATA segSize = std::max(_jitConfig->dataCacheKB * 1024, static_cast<UDATA>(minimumSize));
 
             //--- allocate the segment for the dataCache
             J9MemorySegment *dataCacheSeg = NULL;
+            UDATA memoryType = MEMORY_TYPE_RAM;
+#ifdef LINUX
+            if (_disclaimEnabled)
+               {
+               UDATA defaultPageSize = j9vmem_supported_page_sizes()[0];
+               segSize = OMR::align((size_t)segSize, (size_t)defaultPageSize); // Round up to a multiple of default page size if needed
+               memoryType |= MEMORY_TYPE_VIRTUAL; // Use mmap for allocation
+               // If swap is enabled, we can allocate memory with mmap(MAP_ANOYNMOUS|MAP_PRIVATE) and disclaim to swap
+               // If swap is not enabled we can disclaim to a backing file
+               TR::CompilationInfo * compInfo = TR::CompilationInfo::get(_jitConfig);
+               if (!TR::Options::getCmdLineOptions()->getOption(TR_PreferSwapForMemoryDisclaim) || compInfo->isSwapMemoryDisabled())
+                  {
+                  memoryType |= MEMORY_TYPE_DISCLAIMABLE_TO_FILE;
+                  }
+               }
+#endif
                {
                OMR::CriticalSection criticalSection(_mutex);
-               dataCacheSeg = _jitConfig->javaVM->internalVMFunctions->allocateMemorySegmentInList(_jitConfig->javaVM, _jitConfig->dataCacheList, segSize, MEMORY_TYPE_RAM, J9MEM_CATEGORY_JIT_DATA_CACHE);
+               dataCacheSeg = _jitConfig->javaVM->internalVMFunctions->allocateMemorySegmentInList(_jitConfig->javaVM, _jitConfig->dataCacheList, segSize, memoryType, J9MEM_CATEGORY_JIT_DATA_CACHE);
                if (dataCacheSeg)
                   _jitConfig->dataCache = dataCacheSeg; // for maximum compatibility with the old implementation
                }
@@ -265,16 +293,57 @@ TR_DataCache* TR_DataCacheManager::allocateNewDataCache(uint32_t minimumSize)
                dataCache->_status = 0;
                dataCache->_vmThread = NULL;
                dataCache->_allocationMark = dataCacheSeg->heapAlloc;
+               dataCache->_rssRegion = NULL;
+
+               if (OMR::RSSReport::instance())
+                  {
+                  J9JavaVM * javaVM = jitConfig->javaVM;
+                  PORT_ACCESS_FROM_JAVAVM(javaVM); // for j9vmem_supported_page_sizes
+
+                  dataCache->_rssRegion = new (PERSISTENT_NEW) OMR::RSSRegion("data cache", dataCacheSeg->heapBase, segSize, OMR::RSSRegion::lowToHigh,
+                                                         j9vmem_supported_page_sizes()[0]);
+                  OMR::RSSReport::instance()->addRegion(dataCache->_rssRegion);
+                  }
+
                _numAllocatedCaches++;
                _totalSegmentMemoryAllocated += (uint32_t)allocatedSize;
 #ifdef DATA_CACHE_DEBUG
-               fprintf(stderr, "Allocated a new segment %p of size %d for TR_DataCache %p heapAlloc=%p\n",
+               fprintf(stderr, "Allocated a new segment %p of size %zu for TR_DataCache %p heapAlloc=%p\n",
                   dataCacheSeg, segSize, dataCache, dataCacheSeg->heapAlloc);
 #endif
+               if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+                  TR_VerboseLog::writeLineLocked(TR_Vlog_INFO, "Allocated new data cache segment starting at address %p", dataCacheSeg->heapBase);
+
+#ifdef LINUX
+               if (_disclaimEnabled)
+                  {
+                  // The start address must be page aligned because we obtained it from mmap
+                  TR_ASSERT_FATAL(OMR::alignedNoCheck((uintptr_t)dataCacheSeg->heapBase, j9vmem_supported_page_sizes()[0]), "Start address of the segment is not page aligned");
+
+                  // Use small pages for data caches to improve RSS
+                  size_t segLength = dataCacheSeg->heapTop - dataCacheSeg->heapBase;
+                  if (madvise(dataCacheSeg->heapBase, segLength, MADV_NOHUGEPAGE) != 0)
+                     {
+                     if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+                        TR_VerboseLog::writeLineLocked(TR_Vlog_INFO, "Failed to set MADV_NOHUGEPAGE for data cache");
+                     }
+                  // If the memory segment is backed by a file, disable read-ahead
+                  // so that touching one byte brings a single page in
+                  if (dataCacheSeg->vmemIdentifier.allocator == OMRPORT_VMEM_RESERVE_USED_MMAP_SHM)
+                     {
+                     if (madvise(dataCacheSeg->heapBase, segLength, MADV_RANDOM) != 0)
+                        {
+                        if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+                           TR_VerboseLog::writeLineLocked(TR_Vlog_INFO, "Failed to set MADV_RANDOM for data cache");
+                        }
+                     }
+                  }
+#endif // LINUX
                }
             else
                {
-               TR_VerboseLog::writeLine(TR_Vlog_INFO, "Failed to allocate %d Kb data cache", _jitConfig->dataCacheKB);
+               if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+                  TR_VerboseLog::writeLineLocked(TR_Vlog_INFO, "Failed to allocate %d Kb data cache", _jitConfig->dataCacheKB);
                j9mem_free_memory(dataCache);
                dataCache = NULL;
                _jitConfig->runtimeFlags |= J9JIT_DATA_CACHE_FULL;  // prevent future allocations
@@ -285,7 +354,7 @@ TR_DataCache* TR_DataCacheManager::allocateNewDataCache(uint32_t minimumSize)
             }
          else // cannot allocate even a bit of memory from the JVM
             {
-            TR_VerboseLog::writeLine(TR_Vlog_INFO, "Failed to allocate %d bytes for data cache", sizeof(TR_DataCache));
+            TR_VerboseLog::writeLineLocked(TR_Vlog_INFO, "Failed to allocate %d bytes for data cache", sizeof(TR_DataCache));
             _jitConfig->runtimeFlags |= J9JIT_DATA_CACHE_FULL;  // prevent future allocations
 #ifdef DATA_CACHE_DEBUG
             fprintf(stderr, "Memory allocation for TR_DataCache failed\n");
@@ -678,6 +747,89 @@ double TR_DataCacheManager::computeDataCacheEfficiency()
    return 100.0 * (_totalSegmentMemoryAllocated - availableSpaceActive - availableSpaceRetired) / _totalSegmentMemoryAllocated;
    }
 
+// Note: this method has high overhead. Use it only for debugging
+extern "C" {
+#ifdef LINUX
+   int countPresentPagesInSegment(const J9MemorySegment *segment, int &swappedCount, int &filePageCount);
+#endif
+}
+
+//#define DEBUG_DISCLAIM
+//----------------------------- disclaimSegment -----------------------------
+// Disclaim memory for the given segment
+// Return 1 if the memory segment was disclaimed, or 0 otherwise
+// Used with DataCacheManager mutex in hand
+// Side effect: may disable disclaiming if the kernel does not support it
+//---------------------------------------------------------------------------
+int TR_DataCacheManager::disclaimSegment(J9MemorySegment *segment, bool canDisclaimOnSwap)
+   {
+   int disclaimDone = 0;
+#ifdef LINUX
+   if (segment->vmemIdentifier.allocator == OMRPORT_VMEM_RESERVE_USED_MMAP_SHM || // Can disclaim to file
+       ((segment->vmemIdentifier.mode & J9PORT_VMEM_MEMORY_MODE_VIRTUAL) && canDisclaimOnSwap)) // Can disclaim to swap
+      {
+      size_t segLength = segment->heapTop - segment->heapBase;
+#ifdef DEBUG_DISCLAIM
+      int swappedCount = 0;
+      int filePageCount = 0;
+      int numPresentPages = countPresentPagesInSegment(segment, swappedCount, filePageCount);
+      if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+         TR_VerboseLog::writeLineLocked(TR_Vlog_PERF, "Will disclaim data cache segment %p of size %zu. Present pages =%d swapped=%d fileMapped=%d",
+              segment, segLength, numPresentPages, swappedCount, filePageCount);
+#endif // DEBUG_DISCLAIM
+      int ret = madvise(segment->heapBase, segLength, MADV_PAGEOUT);
+      if (ret != 0)
+         {
+         // madvise() could fail with EINVAL if MADV_PAGEOUT is not supported by the kernel
+         if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+            TR_VerboseLog::writeLineLocked(TR_Vlog_PERF, "WARNING: Failed to use madvise to disclaim memory for data cache");
+         if (ret == EINVAL)
+            {
+            _disclaimEnabled = false; // Don't try to disclaim again, since support seems to be missing
+            if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+               TR_VerboseLog::writeLineLocked(TR_Vlog_PERF, "WARNING: Disabling data cache disclaiming from now on");
+            }
+         }
+      else
+         {
+         disclaimDone = 1;
+#ifdef DEBUG_DISCLAIM
+         swappedCount = 0;
+         filePageCount = 0;
+         numPresentPages = countPresentPagesInSegment(segment, swappedCount, filePageCount);
+         if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+            TR_VerboseLog::writeLineLocked(TR_Vlog_PERF, "Disclaimed data cache segment %p of size %zu. Present pages =%d swapped=%d fileMapped=%d",
+                 segment, segLength, numPresentPages, swappedCount, filePageCount);
+#endif // DEBUG_DISCLAIM
+         }
+      }
+   else // Cannot disclaim because the memory is not backed by a file and swap is not present
+      {
+      if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+         TR_VerboseLog::writeLineLocked(TR_Vlog_PERF, "WARNING: Data cache segment %p is not backed by a file/swap", segment);
+      }
+#endif // LINUX
+   return disclaimDone;
+   }
+
+// Return the number of segments disclaimed
+int TR_DataCacheManager::disclaimAllDataCaches()
+   {
+   if (!_disclaimEnabled)
+      return 0;
+   int numDisclaimed = 0;
+#ifdef LINUX
+   TR::CompilationInfo *compInfo = TR::CompilationInfo::get(_jitConfig);
+   bool canDisclaimOnSwap = TR::Options::getCmdLineOptions()->getOption(TR_PreferSwapForMemoryDisclaim) && !compInfo->isSwapMemoryDisabled();
+   OMR::CriticalSection criticalSection(_mutex);
+   // Traverses all dataCache segments
+   for (J9MemorySegment *dataCacheSeg = _jitConfig->dataCacheList->nextSegment; dataCacheSeg; dataCacheSeg = dataCacheSeg->nextSegment)
+      {
+      numDisclaimed += disclaimSegment(dataCacheSeg, canDisclaimOnSwap);
+      }
+#endif // LINUX
+   return numDisclaimed;
+   }
 
 
 void *
@@ -1010,17 +1162,17 @@ TR_InstrumentedDataCacheManager::printStatistics()
    double currentWaste = averageWaste * _numberOfCurrentAllocations;
    _maxConcurrentWasteEstimate = std::max( _maxConcurrentWasteEstimate, currentWaste);
    fprintf(stderr, "=== Data cache statistics ===\n");
-   fprintf(stderr, "Total data cache bytes in use = %zu\n", _totalSegmentMemoryAllocated);
-   fprintf(stderr, "Bytes converted for regluar JIT use = %zu\n", _jitSpace);
+   fprintf(stderr, "Total data cache bytes in use = %" OMR_PRIuPTR "\n", _totalSegmentMemoryAllocated);
+   fprintf(stderr, "Bytes converted for regluar JIT use = %" OMR_PRIuPTR "\n", _jitSpace);
    fprintf(stderr, "Average allocation size = %f\n", static_cast<double>(_bytesAllocated) / static_cast<double>(_numberOfAllocations) );
    fprintf(stderr, "Standard Deviation of allocation size = %f\n", sqrt(_squares) );
    fprintf(stderr, "Average waste per allocation = %f\n", averageWaste );
    fprintf(stderr, "Estimated current waste = %f\n", currentWaste);
    fprintf(stderr, "Estimated maximum waste = %f\n", _maxConcurrentWasteEstimate);
-   fprintf(stderr, "Loss = %zu\n",  _freeSpace - _bytesInPool);
-   fprintf(stderr, "Loss Error = %zu\n", _bytesInPool - calculatePoolSize());
-   fprintf(stderr, "Free Space = %zu\n",  _freeSpace);
-   fprintf(stderr, "Bytes in pool = %zu\n", _bytesInPool);
+   fprintf(stderr, "Loss = %" OMR_PRIuPTR "\n",  _freeSpace - _bytesInPool);
+   fprintf(stderr, "Loss Error = %" OMR_PRIuPTR "\n", _bytesInPool - calculatePoolSize());
+   fprintf(stderr, "Free Space = %" OMR_PRIuPTR "\n",  _freeSpace);
+   fprintf(stderr, "Bytes in pool = %" OMR_PRIuPTR "\n", _bytesInPool);
    _allocationStatistics.report(stderr);
    _wasteStatistics.report(stderr);
    printPoolContents();
@@ -1047,4 +1199,3 @@ TR_InstrumentedDataCacheManager::calculatePoolSize()
       }
    return poolSize;
    }
-
