@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "runtime/HWProfiler.hpp"
@@ -30,6 +30,7 @@
 #include "compile/CompilationTypes.hpp"
 #include "control/CompilationController.hpp"
 #include "control/MethodToBeCompiled.hpp"
+#include "control/CompilationStrategy.hpp"
 #include "env/CompilerEnv.hpp"
 #include "env/TRMemory.hpp"
 #include "env/jittypes.h"
@@ -452,7 +453,7 @@ bool TR_HWProfiler::checkAndTurnBufferProcessingOff()
       {
       if (TR::Options::isAnyVerboseOptionSet(TR_VerboseHWProfiler))
          {
-         TR_VerboseLog::writeLineLocked(TR_Vlog_HWPROFILER, "t=%6u RI continue because QSZ is large: %d\n",
+         TR_VerboseLog::writeLineLocked(TR_Vlog_HWPROFILER, "t=%6u RI continue because QSZ is large: %d",
                                         (uint32_t)_compInfo->getPersistentInfo()->getElapsedTime(),
                                         _compInfo->getMethodQueueSize());
          }
@@ -474,7 +475,7 @@ bool TR_HWProfiler::checkAndTurnBufferProcessingOff()
          if (TR::Options::isAnyVerboseOptionSet(TR_VerboseHWProfiler, TR_VerbosePerformance))
             {
             float recompFrequency = (float)newRecompDecisionsYes / (float)newRecompDecisionsTotal;
-            TR_VerboseLog::writeLineLocked(TR_Vlog_HWPROFILER, "t=%6u RI buffer processing disabled because recomp frequency is %.4f newRecompDecisionsTotal=%llu\n",
+            TR_VerboseLog::writeLineLocked(TR_Vlog_HWPROFILER, "t=%6u RI buffer processing disabled because recomp frequency is %.4f newRecompDecisionsTotal=%llu",
                                            (uint32_t)_compInfo->getPersistentInfo()->getElapsedTime(),
                                            recompFrequency, newRecompDecisionsTotal);
             }
@@ -485,7 +486,7 @@ bool TR_HWProfiler::checkAndTurnBufferProcessingOff()
          if (TR::Options::isAnyVerboseOptionSet(TR_VerboseHWProfiler))
             {
             float recompFrequency = (float)newRecompDecisionsYes / (float)newRecompDecisionsTotal;
-            TR_VerboseLog::writeLineLocked(TR_Vlog_HWPROFILER, "t=%6u RI continue. recomp frequency is %.4f newRecompDecisionsTotal=%llu\n",
+            TR_VerboseLog::writeLineLocked(TR_Vlog_HWPROFILER, "t=%6u RI continue. recomp frequency is %.4f newRecompDecisionsTotal=%llu",
                                            (uint32_t)_compInfo->getPersistentInfo()->getElapsedTime(),
                                            recompFrequency, newRecompDecisionsTotal);
             }
@@ -505,7 +506,7 @@ bool TR_HWProfiler::checkAndTurnBufferProcessingOn()
          restoreBufferProcessingFunctionality();
          if (TR::Options::isAnyVerboseOptionSet(TR_VerboseHWProfiler, TR_VerbosePerformance))
             {
-            TR_VerboseLog::writeLineLocked(TR_Vlog_HWPROFILER, "RI buffer processing re-enabled because Q_SZ=%d\n", _compInfo->getMethodQueueSize());
+            TR_VerboseLog::writeLineLocked(TR_Vlog_HWPROFILER, "RI buffer processing re-enabled because Q_SZ=%d", _compInfo->getMethodQueueSize());
             }
          return true;
          }
@@ -518,7 +519,7 @@ bool TR_HWProfiler::checkAndTurnBufferProcessingOn()
          if (TR::Options::isAnyVerboseOptionSet(TR_VerboseHWProfiler, TR_VerbosePerformance))
             {
             TR_VerboseLog::writeLineLocked(TR_Vlog_HWPROFILER,
-                                           "RI buffer processing re-enabled because we downgraded %d methods at cold since RI was turned off\n",
+                                           "RI buffer processing re-enabled because we downgraded %d methods at cold since RI was turned off",
                                            _numDowngradesSinceTurnedOff);
             }
          return true;
@@ -841,14 +842,16 @@ TR_HWProfiler::createRecords(TR::Compilation *comp)
       if (!TR::Options::getCmdLineOptions()->getOption(TR_HWProfilerDisableAOT) &&
           fej9->hardwareProfilingInstructionsNeedRelocation())
          {
-         TR::ExternalRelocation *relocation =
-               new (comp->trHeapMemory()) TR::ExternalRelocation(ia,
-                                                                target,
-                                                                target2,
-                                                                relocationTargetKind,
-                                                                cg);
-
-         cg->addExternalRelocation(relocation, __FILE__, __LINE__, node);
+         cg->addExternalRelocation(
+            TR::ExternalRelocation::create(
+               ia,
+               target,
+               target2,
+               relocationTargetKind,
+               cg),
+            __FILE__,
+            __LINE__,
+            node);
          }
       }
    }
