@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,35 +15,46 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include <algorithm>
 #include <stdlib.h>
 #include <string.h>
+#if defined(LINUX)
+#include <sys/mman.h> // for madvise
+#endif // LINUX
+#include "OMR/Bytes.hpp"
 #include "j9.h"
+#include "j9cp.h"
 #include "j9protos.h"
 #include "j9thread.h"
 #include "jitprotos.h"
-#include "j9cp.h"
 #define J9_EXTERNAL_TO_VM
-#include "runtime/J9VMAccess.hpp"
 #include "vmaccess.h"
-#include "infra/Monitor.hpp"
+#include "control/CompilationRuntime.hpp"
 #include "control/Recompilation.hpp"
 #include "control/RecompilationInfo.hpp"
 #include "env/FrontEnd.hpp"
-#include "env/ut_j9jit.h"
-#include "infra/CriticalSection.hpp"
-#include "runtime/CodeCacheManager.hpp"
-#include "runtime/CodeCache.hpp"
-#include "runtime/CodeCacheMemorySegment.hpp"
-#include "env/VMJ9.h"
-#include "runtime/ArtifactManager.hpp"
 #include "env/IO.hpp"
 #include "env/VerboseLog.hpp"
+#include "env/VMJ9.h"
+#include "env/ut_j9jit.h"
+#include "infra/CriticalSection.hpp"
+#include "infra/Monitor.hpp"
+#include "runtime/ArtifactManager.hpp"
+#include "runtime/CodeCache.hpp"
+#include "runtime/CodeCacheManager.hpp"
+#include "runtime/CodeCacheMemorySegment.hpp"
+#include "runtime/J9VMAccess.hpp"
+
+#if defined(LINUX)
+#if !defined(MADV_HUGEPAGE)
+#define MADV_HUGEPAGE 14
+#endif /* MADV_HUGEPAGE */
+#endif /* LINUX */
 
 TR::CodeCacheManager *J9::CodeCacheManager::_codeCacheManager = NULL;
 J9JavaVM *J9::CodeCacheManager::_javaVM = NULL;
@@ -76,6 +87,37 @@ J9::CodeCacheManager::initialize(bool useConsolidatedCache, uint32_t numberOfCod
    return self()->OMR::CodeCacheManager::initialize(useConsolidatedCache, numberOfCodeCachesToCreateAtStartup);
    }
 
+bool
+J9::CodeCacheManager::isSufficientPhysicalMemoryAvailableForAllocation(size_t requestedCodeCacheSize)
+   {
+   TR::CodeCacheConfig &config = self()->codeCacheConfig();
+   TR::CompilationInfo * compInfo = getCompilationInfo(_jitConfig);
+   bool incomplete;
+   // If swap memory is not available then no need to do further check
+   if (!compInfo->isSwapMemoryDisabled())
+      {
+      return true;
+      }
+
+   // Read the amount of free physical memory currently, bypassing the caching mechanism
+   uint64_t freePhysicalMemoryB = compInfo->computeAndCacheFreePhysicalMemory(incomplete, 0);
+   // Avoid consuming the last drop of physical memory for JIT; leave some for emergency situations
+   uint64_t safeMemReserve = (uint64_t)TR::Options::getSafeReservePhysicalMemoryValue();
+   uint64_t desiredMemory = requestedCodeCacheSize + safeMemReserve;
+
+   if (!incomplete && freePhysicalMemoryB < desiredMemory)
+      {
+      if (config.verboseCodeCache() || config.verbosePerformance())
+         TR_VerboseLog::writeLineLocked(TR_Vlog_CODECACHE, "Warning: low physical memory detected during code cache allocation, requestedCodeCacheSize=%zu, freePhysicalMemory=%zu, safeMemReserve=%zu",
+                                       requestedCodeCacheSize, (size_t)freePhysicalMemoryB, (size_t)safeMemReserve);
+      return false;
+      }
+   else
+      {
+      return true;
+      }
+   }
+
 void
 J9::CodeCacheManager::addCodeCache(TR::CodeCache *codeCache)
    {
@@ -215,12 +257,15 @@ TR::CodeCache*
 J9::CodeCacheManager::reserveCodeCache(bool compilationCodeAllocationsMustBeContiguous,
                                       size_t sizeEstimate,
                                       int32_t compThreadID,
-                                      int32_t *numReserved)
+                                      int32_t *numReserved,
+                                      TR::CodeCacheKind kind)
    {
    TR::CodeCache *codeCache = self()->OMR::CodeCacheManager::reserveCodeCache(compilationCodeAllocationsMustBeContiguous,
                                                                             sizeEstimate,
                                                                             compThreadID,
-                                                                            numReserved);
+                                                                            numReserved,
+                                                                            kind);
+
    if (codeCache == NULL)
       {
       J9JITConfig *jitConfig = self()->fej9()->getJ9JITConfig();
@@ -251,21 +296,34 @@ J9::CodeCacheManager::allocateCodeCacheSegment(size_t segmentSize,
 
    J9PortVmemParams vmemParams;
    j9vmem_vmem_params_init(&vmemParams);
+   UDATA *pageSizes = j9vmem_supported_page_sizes();
 
    TR::CodeCacheConfig &config = self()->codeCacheConfig();
 
-   size_t largeCodePageSize = config.largeCodePageSize();
-#if defined(TR_TARGET_POWER) && defined(TR_HOST_POWER)
-   /* Use largeCodePageSize on PPC only if its 16M.
-    If we pass in any pagesize other than the default page size, the port library picks the shared memory api to allocate which wastes memory */
-   size_t sixteenMegs = 16 * 1024 * 1024;
-   if (largeCodePageSize == sixteenMegs)
-#else
+   UDATA ccTotalSizeB = config.codeCacheTotalKB() << 10;
+
+   // Determine whether we are actually using large pages.
+   // Note that config.largeCodePageSize() could be set at the default page size (given by pageSize[0])
+   size_t largeCodePageSize = 0;
+   if (config.largeCodePageSize() > pageSizes[0])
+      largeCodePageSize = config.largeCodePageSize();
+
    if (largeCodePageSize > 0)
-#endif
       {
       vmemParams.pageSize = largeCodePageSize;
       vmemParams.pageFlags = config.largeCodePageFlags();
+
+      // Recalculate the page size if using large pages
+      if(vmemParams.pageSize > ccTotalSizeB)
+         {
+         for (UDATA pageIndex = 0; 0 != pageSizes[pageIndex]; ++pageIndex)
+            {
+            if(pageSizes[pageIndex] <= ccTotalSizeB)
+               vmemParams.pageSize = pageSizes[pageIndex];
+            }
+         if (config.verboseCodeCache() || config.verbosePerformance())
+            TR_VerboseLog::writeLineLocked(TR_Vlog_CODECACHE,"Warning: Using page size %zu instead of large page size %zu", (size_t)vmemParams.pageSize, largeCodePageSize);
+         }
       }
 
    UDATA mode = J9PORT_VMEM_MEMORY_MODE_READ |
@@ -297,20 +355,56 @@ J9::CodeCacheManager::allocateCodeCacheSegment(size_t segmentSize,
    codeCacheSizeToAllocate = (codeCacheSizeToAllocate + (vmemParams.pageSize-1)) & (~(vmemParams.pageSize-1));
    vmemParams.byteAmount = codeCacheSizeToAllocate;
 
+   // Fail code cache allocation, if the available physical memory is low
+   if (!self()->isSufficientPhysicalMemoryAvailableForAllocation(codeCacheSizeToAllocate))
+      return 0;
+
    void *defaultEndAddress = vmemParams.endAddress;
 
+   uintptr_t someJitLibraryAddress = self()->getSomeJitLibraryAddress();
+
+#if defined(TR_TARGET_POWER)
+   size_t alignment = 64 * 1024;
+#elif (defined(LINUX) && defined(TR_TARGET_X86))
+   size_t alignment = 2 * 1024 * 1024;
+#elif (defined(TR_TARGET_S390))
+   size_t alignment = 1024 * 1024;
+#else
+   size_t alignment = pageSizes[0];
+#endif
+   if (largeCodePageSize > 0)
+      alignment = largeCodePageSize;
+
    if (preferredStartAddress)
       {
-   // J9PORT_VMEM_ADDRESS_HINT is only supported for default pages and linux
+      vmemParams.options |= J9PORT_VMEM_STRICT_ADDRESS; // if we cannot allocate a block in the desired range, return NULL
+      vmemParams.alignmentInBytes = alignment;
 #if defined(LINUX)
-      if (largeCodePageSize == 0)
-         {
-         vmemParams.options |= J9PORT_VMEM_ADDRESS_HINT;
-         }
-#endif
-      vmemParams.options |= J9PORT_VMEM_STRICT_ADDRESS; // if we cannot allocate a block whose start is between preferredStartAddress and endAddress, return NULL
+      vmemParams.options |= J9PORT_VMEM_ALLOC_QUICK; // Use smaps for guidance
       vmemParams.startAddress = preferredStartAddress;
-      vmemParams.endAddress = (void *)(((UDATA)preferredStartAddress) + SAFE_DISTANCE_REPOSITORY_JITLIBRARY);
+      if ((uintptr_t)preferredStartAddress < someJitLibraryAddress) // Allocate before JIT dll
+         {
+         vmemParams.endAddress = (void *)(someJitLibraryAddress - SAFE_DISTANCE_REPOSITORY_JITLIBRARY - segmentSize);
+         }
+      else // Allocate after JIT dll
+         {
+         vmemParams.endAddress = (void *)(someJitLibraryAddress + MAX_DISTANCE_NEAR_JITLIBRARY_TO_AVOID_TRAMPOLINE - segmentSize);
+         }
+      // If the code cache repository size is very large, it may not be possible to
+      // allocate it in the vicinity of the JIT dll. If that's the case, let the OS
+      // pick any address it wants.
+      if (vmemParams.startAddress >= vmemParams.endAddress)
+         {
+         preferredStartAddress = NULL;
+         vmemParams.startAddress = NULL;
+         vmemParams.endAddress = defaultEndAddress;
+         vmemParams.options &= ~(J9PORT_VMEM_STRICT_ADDRESS);
+         vmemParams.options &= ~(J9PORT_VMEM_ALLOC_QUICK);
+         }
+#else
+     vmemParams.startAddress = preferredStartAddress;
+     vmemParams.endAddress = (void *)(((UDATA)preferredStartAddress) + SAFE_DISTANCE_REPOSITORY_JITLIBRARY);
+#endif
       }
 #if defined(J9ZOS390)
    else
@@ -329,6 +423,19 @@ J9::CodeCacheManager::allocateCodeCacheSegment(size_t segmentSize,
       }
 #endif
 
+#ifdef LINUX
+      if (_disclaimEnabled)
+         {
+         // If swap is enabled, we can allocate memory with mmap(MAP_ANOYNMOUS|MAP_PRIVATE) and disclaim to swap
+         // If swap is not enabled we can disclaim to a backing file
+         TR::CompilationInfo * compInfo = TR::CompilationInfo::get(_jitConfig);
+         if (!TR::Options::getCmdLineOptions()->getOption(TR_PreferSwapForMemoryDisclaim) || compInfo->isSwapMemoryDisabled())
+            {
+            segmentType |= MEMORY_TYPE_DISCLAIMABLE_TO_FILE;
+            }
+         }
+#endif
+
    mcc_printf("TR::CodeCache::allocate : requesting %d bytes\n", codeCacheSizeToAllocate);
    mcc_printf("TR::CodeCache::allocate : javaVM = %p\n", javaVM);
    mcc_printf("TR::CodeCache::allocate : codeCacheList = %p\n", jitConfig->codeCacheList);
@@ -345,51 +452,11 @@ J9::CodeCacheManager::allocateCodeCacheSegment(size_t segmentSize,
                                                                       segmentType,
                                                                       &vmemParams);
 
-   uintptr_t someJitLibraryAddress = self()->getSomeJitLibraryAddress();
-   // isInRange() checks whether the allocated codeCacheSegment baseAddress is in range of JitLibrary to avoid trampoline
-   if (codeCacheSegment &&
-       (vmemParams.options & J9PORT_VMEM_ADDRESS_HINT) &&
-       !(self()->isInRange((uintptr_t)(codeCacheSegment->baseAddress), someJitLibraryAddress, MAX_DISTANCE_NEAR_JITLIBRARY_TO_AVOID_TRAMPOLINE)))
-      {
-      // allocated code cache is not in range to avoid trampoline
-      // try with full address range that avoids trampoline
-      // free old segment
-      javaVM->internalVMFunctions->freeMemorySegment(javaVM, codeCacheSegment, 1);
-      size_t alignment = 2 * 1024 * 1024;
-
-      // if jit library address is greater than MAX_DISTANCE, then we search in the addresses before the jit library address
-      // if jit library address is less than MAX_DISTANCE, which means we do not have the full MAX_DISTANCE(2GB-24MB) range before the jit library
-      // in this case we search in the addresses after the jit library address
-      if (someJitLibraryAddress > MAX_DISTANCE_NEAR_JITLIBRARY_TO_AVOID_TRAMPOLINE)
-         {
-         // align the startAddress to page boundary
-         vmemParams.startAddress = (void *)OMR::align((size_t)(someJitLibraryAddress - MAX_DISTANCE_NEAR_JITLIBRARY_TO_AVOID_TRAMPOLINE), alignment);
-         vmemParams.endAddress = preferredStartAddress;
-         }
-      else
-         {
-         vmemParams.startAddress = (void *)OMR::align((size_t)(someJitLibraryAddress + SAFE_DISTANCE_REPOSITORY_JITLIBRARY), alignment);
-         vmemParams.endAddress = (void *)(someJitLibraryAddress + MAX_DISTANCE_NEAR_JITLIBRARY_TO_AVOID_TRAMPOLINE);
-         }
-      // unset STRICT_ADDRESS and ADDRESS_HINT
-      vmemParams.options &= ~(J9PORT_VMEM_STRICT_ADDRESS);
-      vmemParams.options &= ~(J9PORT_VMEM_ADDRESS_HINT);
-      // for Linux allocate using QUICK method based on smaps
-#if defined(LINUX)
-      vmemParams.options |= J9PORT_VMEM_ALLOC_QUICK;
-#endif
-      codeCacheSegment =
-          javaVM->internalVMFunctions->allocateVirtualMemorySegmentInList(javaVM,
-                                                                          jitConfig->codeCacheList,
-                                                                          codeCacheSizeToAllocate,
-                                                                          segmentType,
-                                                                          &vmemParams);
-      }
    if (!codeCacheSegment && preferredStartAddress)
       {
 #if !defined(J9ZOS390)
-      // we could have failed because we wanted a start address
-      // let's try without it
+      // We could have failed because we wanted a start address.
+      // Let's try without it.
       vmemParams.startAddress = NULL;
       vmemParams.options &= ~(J9PORT_VMEM_STRICT_ADDRESS);
       vmemParams.options &= ~(J9PORT_VMEM_ADDRESS_HINT);
@@ -422,21 +489,36 @@ J9::CodeCacheManager::allocateCodeCacheSegment(size_t segmentSize,
       mcc_printf("TR::CodeCache::allocated : codeCacheSegment is %p\n",codeCacheSegment);
       if (config.verboseCodeCache())
          {
-         char * verboseLogString = "The code cache repository was allocated between addresses %p and %p";
+         const char *verboseLogString = "The code cache repository was allocated between addresses %p and %p alignment=%zu largeCodePageSize=%zu";
          if (preferredStartAddress && self()->isInRange((uintptr_t)(codeCacheSegment->baseAddress), someJitLibraryAddress, MAX_DISTANCE_NEAR_JITLIBRARY_TO_AVOID_TRAMPOLINE))
             {
-            verboseLogString = "The code cache repository was allocated between addresses %p and %p to be near the VM/JIT modules to avoid trampolines";
+            verboseLogString = "The code cache repository was allocated between addresses %p and %p to avoid helper trampolines. alignment=%zu largeCodePageSize=%zu";
             }
-         TR_VerboseLog::writeLineLocked(TR_Vlog_CODECACHE,
-            verboseLogString,
-            vmemParams.startAddress,
-            vmemParams.endAddress);
+         TR_VerboseLog::writeLineLocked(TR_Vlog_CODECACHE, verboseLogString, codeCacheSegment->baseAddress, codeCacheSegment->heapTop, alignment, largeCodePageSize);
          }
+#ifdef LINUX
+      if (0 != madvise((void *)codeCacheSegment->baseAddress, (codeCacheSegment->heapTop - codeCacheSegment->baseAddress), MADV_HUGEPAGE))
+         {
+         if (config.verboseCodeCache() || TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+            {
+            TR_VerboseLog::writeLineLocked(TR_Vlog_PERF,"Warning: madvise failed while providing hint to use large pages for code cache");
+            }
+         }
+#endif // LINUX
+
+      if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+         TR_VerboseLog::writeLineLocked(TR_Vlog_INFO, "Allocated new code cache segment %p starting at address %p",
+                                        codeCacheSegment,
+                                        codeCacheSegment->heapBase);
       }
    else
       {
       // TODO: we should generate a trace point
       mcc_printf("TR::CodeCache::allocate : codeCacheSegment is NULL, %p\n",codeCacheSegment);
+
+      if (TR::Options::getCmdLineOptions()->getVerboseOption(TR_VerbosePerformance))
+         TR_VerboseLog::writeLineLocked(TR_Vlog_INFO, "Failed to allocate new code cache segment of %d Kb", _jitConfig->codeCacheKB);
+
       return 0;
       }
 
@@ -483,6 +565,7 @@ J9::CodeCacheManager::setupMemorySegmentFromRepository(uint8_t *start,
    {
    // create and set the j9segment object
    J9MemorySegment *j9segment = (J9MemorySegment *) self()->getMemory(sizeof(J9MemorySegment));
+   j9segment->baseAddress = start;
    j9segment->heapBase = start;
    j9segment->heapAlloc = start;
    j9segment->heapTop = end;
@@ -510,7 +593,7 @@ J9::CodeCacheManager::chooseCacheStartAddress(size_t repositorySize)
 #if defined(TR_HOST_64BIT) && defined(TR_TARGET_X86)
    if (!TR::Options::getCmdLineOptions()->getOption(TR_DisableSmartPlacementOfCodeCaches))
       {
-      size_t alignment = 2 * 1024 * 1024; // 2MB alignment
+      size_t alignment = 2 * 1024 * 1024; // 2MB alignment is preferred on x86-64
       TR::CodeCacheConfig & config = self()->codeCacheConfig();
       const size_t largeCodePageSize = config.largeCodePageSize();
 
@@ -518,31 +601,31 @@ J9::CodeCacheManager::chooseCacheStartAddress(size_t repositorySize)
       if (largeCodePageSize > alignment)
          alignment = largeCodePageSize;
 
-      size_t safeDistance = repositorySize + SAFE_DISTANCE_REPOSITORY_JITLIBRARY;
       uintptr_t someFunctionPointer = self()->getSomeJitLibraryAddress();
 
       mcc_printf("addCodeCache() function address is %p\n", someFunctionPointer);
 
-      // if we are using 1GB pages or something even larger
-      if (largeCodePageSize >= 0x40000000)
+      // Depending on where the JIT dll is placed in memory, we may want
+      // to allocate the code cache before or after the JIT dll
+      if (someFunctionPointer > MAX_DISTANCE_NEAR_JITLIBRARY_TO_AVOID_TRAMPOLINE)
          {
-         if (someFunctionPointer > largeCodePageSize * 2)
+         // Enough space is available before the JIT dll
+         // For Linux we can afford to search a larger space (~2 GB) because we use smaps to accelerate the search operation.
+         // Also, if we use very large pages (1 GB), then we can afford to search a larger space because there won't be
+         // too many attempts to find the right spot.
+         if (TR::Compiler->target.isLinux() || alignment > 2 * 1024 * 1024)
             {
-            // round down to the nearest GB first
-            startAddress = (void *)(((uintptr_t)someFunctionPointer - SAFE_DISTANCE_REPOSITORY_JITLIBRARY) & ~(largeCodePageSize-1));
-            // then move down 1GB to allocate the code cache
-            startAddress = (void *)(((uintptr_t)startAddress) - largeCodePageSize);
+            startAddress = (void *)OMR::align((size_t)(someFunctionPointer - MAX_DISTANCE_NEAR_JITLIBRARY_TO_AVOID_TRAMPOLINE), alignment);
+            }
+         else
+            {
+            size_t safeDistance = repositorySize + SAFE_DISTANCE_REPOSITORY_JITLIBRARY;
+            startAddress = (void *)OMR::align((size_t)(someFunctionPointer - safeDistance), alignment);
             }
          }
-      else
+      else // Choose a starting address after the JIT dll
          {
-         if (someFunctionPointer > safeDistance+alignment)
-            {
-            // otherwise move back some space larger than the VM DLL footprint
-            startAddress = (void *)(((uint8_t *)someFunctionPointer) - safeDistance);
-            // align so that port library returns exactly what we wanted
-            startAddress = (void *)OMR::align((size_t)startAddress, alignment);
-            }
+         startAddress = (void *)OMR::align((size_t)(someFunctionPointer + SAFE_DISTANCE_REPOSITORY_JITLIBRARY), alignment);
          }
       }
 #endif
@@ -656,13 +739,15 @@ J9::CodeCacheManager::almostOutOfCodeCache()
 
    TR::CodeCacheConfig &config = self()->codeCacheConfig();
 
-   // If we can allocate another code cache we are fine
-   // Put common case first
+   // If we can allocate another code cache we are fine.
+   // Put common case first.
    if (self()->canAddNewCodeCache())
+      {
       return false;
+      }
    else
       {
-      // Check the space in the most current code cache
+      // Check the space in the most current code cache.
       bool foundSpace = false;
 
          {
@@ -680,6 +765,7 @@ J9::CodeCacheManager::almostOutOfCodeCache()
       if (!foundSpace)
          {
          _lowCodeCacheSpaceThresholdReached = true;   // Flag can be checked under debugger
+         _jitConfig->lowCodeCacheFreeSpace = 1;
          if (config.verbosePerformance())
             {
             TR_VerboseLog::writeLineLocked(TR_Vlog_CODECACHE,"Reached code cache space threshold. Disabling JIT profiling.");
@@ -723,3 +809,26 @@ J9::CodeCacheManager::printOccupancyStats()
       codeCache->printOccupancyStats();
       }
    }
+
+
+int32_t
+J9::CodeCacheManager::disclaimAllCodeCaches()
+   {
+   if (!_disclaimEnabled)
+      return 0;
+
+   int32_t numDisclaimed = 0;
+
+#ifdef LINUX
+   TR::CompilationInfo *compInfo = TR::CompilationInfo::get(_jitConfig);
+   bool canDisclaimOnSwap = TR::Options::getCmdLineOptions()->getOption(TR_PreferSwapForMemoryDisclaim) && !compInfo->isSwapMemoryDisabled();
+
+   CacheListCriticalSection scanCacheList(self());
+   for (TR::CodeCache *codeCache = self()->getFirstCodeCache(); codeCache; codeCache = codeCache->next())
+      {
+      numDisclaimed += codeCache->disclaim(self(), canDisclaimOnSwap);
+      }
+#endif // LINUX
+
+   return numDisclaimed;
+   }
