@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include <algorithm>
@@ -51,6 +51,7 @@
 #include "runtime/DataCache.hpp"
 #include "env/FrontEnd.hpp"
 #include "infra/Monitor.hpp"
+#include "env/PersistentCHTable.hpp"
 #include "env/PersistentInfo.hpp"
 #include "env/VMAccessCriticalSection.hpp"
 #include "env/CompilerEnv.hpp"
@@ -80,7 +81,7 @@
 
 #include "exceptions/AOTFailure.hpp"
 
-char *TR_RelocationRuntime::_reloErrorCodeNames[] =
+const char *TR_RelocationRuntime::_reloErrorCodeNames[] =
    {
    "relocationOK",                                     // 0
 
@@ -89,7 +90,7 @@ char *TR_RelocationRuntime::_reloErrorCodeNames[] =
    "unknownRelocation",                                // 3
    "unknownReloAction",                                // 4
    "invalidRelocation",                                // 5
-   "exceptionThrown"                                   // 6
+   "exceptionThrown",                                  // 6
 
    "methodEnterValidationFailure",                     // 7
    "methodExitValidationFailure",                      // 8
@@ -128,23 +129,29 @@ char *TR_RelocationRuntime::_reloErrorCodeNames[] =
    "methodFromSingleInterfaceImplValidationFailure",   // 41
    "methodFromSingleAbstractImplValidationFailure",    // 42
    "j2iThunkFromMethodValidationFailure",              // 43
-   "svmValidationFailure",                             // 44
-   "wkcValidationFailure",                             // 45
+   "isClassVisibleValidationFailure",                  // 44
+   "svmValidationFailure",                             // 45
+   "wkcValidationFailure",                             // 46
+   "methodTracingValidationFailure",                   // 47
+   "dynamicMethodFromcallsiteIndexValidationFailure",  // 48
+   "handleMethodFromcallsiteIndexValidationFailure" ,  // 49
 
-   "classAddressRelocationFailure",                    // 46
-   "inlinedMethodRelocationFailure",                   // 47
-   "symbolFromManagerRelocationFailure",               // 48
-   "thunkRelocationFailure",                           // 49
-   "trampolineRelocationFailure",                      // 50
-   "picTrampolineRelocationFailure",                   // 51
-   "cacheFullRelocationFailure",                       // 52
-   "blockFrequencyRelocationFailure",                  // 53
-   "recompQueuedFlagRelocationFailure",                // 54
-   "debugCounterRelocationFailure",                    // 55
-   "directJNICallRelocationFailure",                   // 56
-   "ramMethodConstRelocationFailure",                  // 57
+   "classAddressRelocationFailure",                    // 50
+   "inlinedMethodRelocationFailure",                   // 51
+   "symbolFromManagerRelocationFailure",               // 52
+   "thunkRelocationFailure",                           // 53
+   "trampolineRelocationFailure",                      // 54
+   "picTrampolineRelocationFailure",                   // 55
+   "cacheFullRelocationFailure",                       // 56
+   "blockFrequencyRelocationFailure",                  // 57
+   "recompQueuedFlagRelocationFailure",                // 58
+   "debugCounterRelocationFailure",                    // 59
+   "directJNICallRelocationFailure",                   // 60
+   "ramMethodConstRelocationFailure",                  // 61
+   "catchBlockCounterRelocationFailure",               // 62
+   "staticDefaultValueInstanceRelocationFailure",      // 63
 
-   "maxRelocationError"                                // 58
+   "maxRelocationError"                                // 64
    };
 
 TR_RelocationRuntime::TR_RelocationRuntime(J9JITConfig *jitCfg)
@@ -211,6 +218,7 @@ TR_RelocationRuntime::TR_RelocationRuntime(J9JITConfig *jitCfg)
       }
 
       _isLoading = false;
+      _isRelocating = false;
 
 #if defined(DEBUG) || defined(PROD_WITH_ASSUMES)
       _numValidations = 0;
@@ -235,7 +243,9 @@ TR_RelocationRuntime::prepareRelocateAOTCodeAndData(J9VMThread* vmThread,
                                                     TR::Options *options,
                                                     TR::Compilation *comp,
                                                     TR_ResolvedMethod *resolvedMethod,
-                                                    uint8_t *existingCode)
+                                                    uint8_t *existingCode,
+                                                    TR_J9SharedCache *cacheOverride
+                                                    )
    {
    _currentThread = vmThread;
    _fe = theFE;
@@ -261,6 +271,8 @@ TR_RelocationRuntime::prepareRelocateAOTCodeAndData(J9VMThread* vmThread,
    TR_ASSERT(_options, "Options were not correctly initialized.");
    _reloLogger->setupOptions(_options);
 
+   TR_RelocationRuntime::IsRelocating startRelocating(this, cacheOverride);
+
    uint8_t *tempCodeStart, *tempDataStart;
    uint8_t *oldDataStart, *oldCodeStart, *newCodeStart;
    tempDataStart = (uint8_t *)cacheEntry;
@@ -272,19 +284,24 @@ TR_RelocationRuntime::prepareRelocateAOTCodeAndData(J9VMThread* vmThread,
 
    // If we want to trace this method but the AOT body is not prepared to handle it
    // we must fail this AOT load with an error code that will force retrial
-   if ((fej9->isMethodTracingEnabled((TR_OpaqueMethodBlock*)theMethod) || fej9->canMethodExitEventBeHooked())
-      &&
-       (_aotMethodHeaderEntry->flags & TR_AOTMethodHeader_IsNotCapableOfMethodExitTracing))
+   if (fej9->canMethodEnterEventBeHooked()
+       && !(_aotMethodHeaderEntry->flags & TR_AOTMethodHeader_MethodEnterEventCanBeHooked))
+      {
+      setReloErrorCode(TR_RelocationErrorCode::methodEnterValidationFailure);
+      setReturnCode(compilationRelocationFailure);
+      return NULL; // fail
+      }
+   if (fej9->canMethodExitEventBeHooked()
+       && !(_aotMethodHeaderEntry->flags & TR_AOTMethodHeader_MethodExitEventCanBeHooked))
       {
       setReloErrorCode(TR_RelocationErrorCode::methodExitValidationFailure);
       setReturnCode(compilationRelocationFailure);
       return NULL; // fail
       }
-   if ((fej9->isMethodTracingEnabled((TR_OpaqueMethodBlock*)theMethod) || fej9->canMethodEnterEventBeHooked())
-      &&
-       (_aotMethodHeaderEntry->flags & TR_AOTMethodHeader_IsNotCapableOfMethodEnterTracing))
+   if (fej9->isMethodTracingEnabled((TR_OpaqueMethodBlock *)theMethod)
+       && !(_aotMethodHeaderEntry->flags & TR_AOTMethodHeader_MethodTracingEnabled))
       {
-      setReloErrorCode(TR_RelocationErrorCode::methodEnterValidationFailure);
+      setReloErrorCode(TR_RelocationErrorCode::methodTracingValidationFailure);
       setReturnCode(compilationRelocationFailure);
       return NULL; // fail
       }
@@ -429,16 +446,12 @@ TR_RelocationRuntime::prepareRelocateAOTCodeAndData(J9VMThread* vmThread,
                // Before copying, memorize the real size of the block returned by the code cache manager
                // and fix it later
                U_32 blockSize = ((OMR::CodeCacheMethodHeader*)newCodeStart)->_size;
-#if defined(OSX) && defined(AARCH64)
-               pthread_jit_write_protect_np(0);
-#endif
+               omrthread_jit_write_protect_disable();
                memcpy(newCodeStart, tempCodeStart, codeSize);  // the real size may have been overwritten
                ((OMR::CodeCacheMethodHeader*)newCodeStart)->_size = blockSize; // fix it
                // Must fix the pointer to the metadata which is stored in the OMR::CodeCacheMethodHeader
                ((OMR::CodeCacheMethodHeader*)newCodeStart)->_metaData = _exceptionTable;
-#if defined(OSX) && defined(AARCH64)
-               pthread_jit_write_protect_np(1);
-#endif
+               omrthread_jit_write_protect_enable();
                }
             else
                {
@@ -524,6 +537,15 @@ void TR_RelocationRuntime::relocationFailureCleanup()
       }
    }
 
+static int32_t strHash(const char *str)
+   {
+   // The string hash from Java.
+   int32_t result = 0;
+   for (const unsigned char *s = reinterpret_cast<const unsigned char *>(str); '\0' != *s; ++s)
+      result = (result * 31) + *s;
+   return result;
+   }
+
 // Function called for AOT method from both JXE and Shared Classes to perform relocations
 //
 void
@@ -683,19 +705,50 @@ TR_RelocationRuntime::relocateAOTCodeAndData(U_8 *tempDataStart,
             excptEntry32->ramMethod = actualMethod;
 
             excptEntry32++;
-            if (_comp->getOption(TR_FullSpeedDebug))
+            if (_aotMethodHeaderEntry->flags & TR_AOTMethodHeader_UsesFSD)
                excptEntry32 = (J9JIT32BitExceptionTableEntry *) ((uint8_t *) excptEntry32 + 4);
 
             numExcptionRanges--;
             }
          }
 
+      if (_exceptionTable->inlinedCalls)
+         {
+         if (comp()->getOptions()->getVerboseOption(TR_VerboseInlining))
+            {
+            U_32 numInlinedCallSites = getNumInlinedCallSites(_exceptionTable);
+            int32_t jittedBodyHash = strHash(comp()->signature());
+            char callerBuf[501], calleeBuf[501];
+            TR_VerboseLog::CriticalSection vlogLock;
+            TR_VerboseLog::writeLine(TR_Vlog_INL, "%d methods inlined into %x %s @ %p", numInlinedCallSites, jittedBodyHash, comp()->signature(), _exceptionTable->startPC);
+            for (auto siteIndex = 0; siteIndex < numInlinedCallSites; siteIndex++)
+               {
+               TR_InlinedCallSite *site = (TR_InlinedCallSite *)getInlinedCallSiteArrayElement(_exceptionTable, siteIndex);
+               int32_t calleeLen = fej9()->printTruncatedSignature(calleeBuf, sizeof(calleeBuf)-1, site->_methodInfo);
+               calleeBuf[calleeLen] = 0; // null terminate
+               const char *callerSig = comp()->signature();
+               int16_t callerIndex = site->_byteCodeInfo.getCallerIndex();
+               if (callerIndex != -1)
+                  {
+                  TR_InlinedCallSite *callerCallsite = (TR_InlinedCallSite *)getInlinedCallSiteArrayElement(_exceptionTable, callerIndex);
+                  int32_t callerLen = fej9()->printTruncatedSignature(callerBuf, sizeof(callerBuf)-1, callerCallsite->_methodInfo);
+                  callerBuf[callerLen] = 0; // null terminate
+                  callerSig = callerBuf;
+                  }
+               TR_VerboseLog::writeLine(TR_Vlog_INL, "#%d: %x #%d inlined %x@%d -> %x bcsz=%d %s",
+                  siteIndex, jittedBodyHash, callerIndex,
+                  strHash(callerSig), site->_byteCodeInfo.getByteCodeIndex(),
+                  strHash(calleeBuf), TR::Compiler->mtd.bytecodeSize(site->_methodInfo), calleeBuf);
+               }
+            }
+         }
+
       // Fix RAM method and send target AFTER all relocations are complete.
       startPC = _exceptionTable->startPC;
       } //end if J9_JIT_DCE_EXCEPTION_INFO
 
 #if defined(J9VM_OPT_JITSERVER)
-   TR_ASSERT_FATAL(!TR::CompilationInfo::getStream(), "TR_RelocationRuntime::relocateAOTCodeAndData should not be called at the JITSERVER");
+   TR_ASSERT_FATAL(!_comp->getStream(), "TR_RelocationRuntime::relocateAOTCodeAndData should not be called at the JITSERVER");
 #endif /* defined(J9VM_OPT_JITSERVER) */
    if (startPC)
       {
@@ -844,6 +897,15 @@ TR_RelocationRuntime::relocateMethodMetaData(UDATA codeRelocationAmount, UDATA d
       _exceptionTable->osrInfo = (void *) (((U_8 *)_exceptionTable->osrInfo) + dataRelocationAmount);
       }
 
+#if defined(J9VM_OPT_OPENJDK_METHODHANDLE)
+   if (_exceptionTable->invokeBasicCallInfo != NULL)
+      {
+      uint8_t *infoAddr = (uint8_t*)_exceptionTable->invokeBasicCallInfo;
+      infoAddr += dataRelocationAmount;
+      _exceptionTable->invokeBasicCallInfo = (J9JITInvokeBasicCallInfo*)infoAddr;
+      }
+#endif
+
    // Reset the uninitialized bit
    _exceptionTable->flags &= ~JIT_METADATA_NOT_INITIALIZED;
    }
@@ -862,6 +924,132 @@ TR_RelocationRuntime::aotMethodHeaderVersionsMatch()
    return true;
    }
 
+void
+TR_RelocationRuntime::fillAOTHeader(J9JavaVM *vm, TR_FrontEnd *fe, TR_AOTHeader *aotHeader)
+   {
+   PORT_ACCESS_FROM_JAVAVM(vm);
+
+   TR_J9VMBase *fej9 = (TR_J9VMBase *)fe;
+
+   memset(aotHeader, 0, sizeof(TR_AOTHeader));
+   aotHeader->eyeCatcher = TR_AOTHeaderEyeCatcher;
+
+   TR_Version *aotHeaderVersion = &aotHeader->version;
+   aotHeaderVersion->structSize = sizeof(TR_Version);
+   aotHeaderVersion->majorVersion = TR_AOTHeaderMajorVersion;
+   aotHeaderVersion->minorVersion = TR_AOTHeaderMinorVersion;
+   strncpy(aotHeaderVersion->vmBuildVersion, EsBuildVersionString, sizeof(aotHeaderVersion->vmBuildVersion) - 1);
+   aotHeaderVersion->vmBuildVersion[sizeof(aotHeaderVersion->vmBuildVersion) - 1] = '\0';
+   strncpy(aotHeaderVersion->jitBuildVersion, TR_BUILD_NAME, sizeof(aotHeaderVersion->jitBuildVersion) - 1);
+   aotHeaderVersion->jitBuildVersion[sizeof(aotHeaderVersion->jitBuildVersion) - 1] = '\0';
+
+   aotHeader->gcPolicyFlag = vm->memoryManagerFunctions->j9gc_modron_getWriteBarrierType(vm);
+   aotHeader->lockwordOptionHashValue = getCurrentLockwordOptionHashValue(vm);
+   aotHeader->compressedPointerShift = vm->memoryManagerFunctions->j9gc_objaccess_compressedPointersShift(vm->internalVMFunctions->currentVMThread(vm));
+   aotHeader->processorDescription = TR::Compiler->relocatableTarget.cpu.getProcessorDescription();
+
+   // Set up other feature flags
+   aotHeader->featureFlags = generateFeatureFlags(fe);
+
+   // Set ArrayLet Size if supported
+   aotHeader->arrayLetLeafSize = TR::Compiler->om.arrayletLeafSize();
+   }
+
+// The method CS::Hash_FNV is being used to compute the hash value
+// Notice that currently CS2::Hash_FNV is a hash function that never returns 0
+uint32_t
+TR_RelocationRuntime::getCurrentLockwordOptionHashValue(J9JavaVM *vm)
+   {
+   IDATA currentLockwordArgIndex = J9::Options::getExternalOptionIndex(J9::ExternalOptions::Xlockword);
+   uint32_t currentLockwordOptionHashValue = 0;
+   if (currentLockwordArgIndex >= 0)
+      {
+      char * currentLockwordOption = NULL;
+      GET_OPTION_VALUE(currentLockwordArgIndex, ':', &currentLockwordOption);
+      currentLockwordOptionHashValue = CS2::Hash_FNV((unsigned char*)currentLockwordOption, strlen(currentLockwordOption));
+      }
+   return currentLockwordOptionHashValue;
+   }
+
+uintptr_t
+TR_RelocationRuntime::generateFeatureFlags(TR_FrontEnd *fe)
+   {
+   uintptr_t featureFlags = 0;
+   TR_J9VMBase *fej9 = (TR_J9VMBase *)fe;
+
+   featureFlags |= TR_FeatureFlag_sanityCheckBegin;
+
+   if (TR::Compiler->relocatableTarget.isSMP())
+      featureFlags |= TR_FeatureFlag_IsSMP;
+
+   if (TR::Options::useCompressedPointers())
+      featureFlags |= TR_FeatureFlag_UsesCompressedPointers;
+
+   if (TR::Options::getCmdLineOptions()->getOption(TR_DisableTraps))
+      featureFlags |= TR_FeatureFlag_DisableTraps;
+
+   if (TR::Options::getAOTCmdLineOptions()->getOption(TR_TLHPrefetch))
+      featureFlags |= TR_FeatureFlag_TLHPrefetch;
+
+   if (TR::CodeCacheManager::instance()->codeCacheConfig().needsMethodTrampolines())
+      featureFlags |= TR_FeatureFlag_MethodTrampolines;
+
+   if (TR::Options::getCmdLineOptions()->getOption(TR_FullSpeedDebug))
+      featureFlags |= TR_FeatureFlag_FSDEnabled;
+
+   if (TR::Options::getCmdLineOptions()->getOption(TR_EnableHCR))
+      featureFlags |= TR_FeatureFlag_HCREnabled;
+
+#ifdef TR_TARGET_S390
+   if (TR::Compiler->relocatableTarget.cpu.supportsFeature(OMR_FEATURE_S390_VECTOR_FACILITY))
+      featureFlags |= TR_FeatureFlag_SIMDEnabled;
+#endif
+
+   if (TR::Compiler->om.readBarrierType() != gc_modron_readbar_none)
+      {
+      featureFlags |= TR_FeatureFlag_ConcurrentScavenge;
+
+#ifdef TR_TARGET_S390
+      if (!TR::Compiler->relocatableTarget.cpu.supportsFeature(OMR_FEATURE_S390_GUARDED_STORAGE))
+         featureFlags |= TR_FeatureFlag_SoftwareReadBarrier;
+#endif
+      }
+
+   if (TR::Compiler->om.isIndexableDataAddrPresent())
+      featureFlags |= TR_FeatureFlag_ArrayHeaderShape;
+
+   if (fej9->isAsyncCompilation())
+      featureFlags |= TR_FeatureFlag_AsyncCompilation;
+
+
+   if (!TR::Options::getCmdLineOptions()->getOption(TR_DisableTM) &&
+       !TR::Options::getAOTCmdLineOptions()->getOption(TR_DisableTM))
+      {
+      if (TR::Compiler->relocatableTarget.cpu.supportsTransactionalMemoryInstructions())
+         {
+         featureFlags |= TR_FeatureFlag_UsesTM;
+         }
+      }
+
+   if (TR::Options::getCmdLineOptions()->isVariableHeapBaseForBarrierRange0())
+      featureFlags |= TR_FeatureFlag_IsVariableHeapBaseForBarrierRange0;
+
+   if (TR::Options::getCmdLineOptions()->isVariableHeapSizeForBarrierRange0())
+      featureFlags |= TR_FeatureFlag_IsVariableHeapSizeForBarrierRange0;
+
+   if (TR::Options::getCmdLineOptions()->isVariableActiveCardTableBase())
+      featureFlags |= TR_FeatureFlag_IsVariableActiveCardTableBase;
+
+   TR::CompilationInfo *compInfo = TR::CompilationInfo::get();
+   TR_PersistentCHTable *cht = compInfo->getPersistentInfo()->getPersistentCHTable();
+   if (!TR::Options::getAOTCmdLineOptions()->getOption(TR_DisableCHOpts)
+       && cht && cht->isActive())
+      {
+      featureFlags |= TR_FeatureFlag_CHTableEnabled;
+      }
+
+   return featureFlags;
+   }
 
 bool
 TR_RelocationRuntime::storeAOTHeader(TR_FrontEnd *fe, J9VMThread *curThread)
@@ -928,7 +1116,7 @@ uintptr_t TR_RelocationRuntime::_globalValueList[TR_NumGlobalValueItems] =
    0          // TR_HeapSizeForBarrierRange0
    };
 
-char *TR_RelocationRuntime::_globalValueNames[TR_NumGlobalValueItems] =
+const char *TR_RelocationRuntime::_globalValueNames[TR_NumGlobalValueItems] =
    {
    "not used (0)",
    "TR_CountForRecompile (1)",
@@ -962,7 +1150,7 @@ TR_SharedCacheRelocationRuntime::allocateSpaceInCodeCache(UDATA codeSize)
       {
       int32_t numReserved;
 
-      _codeCache = manager->reserveCodeCache(false, codeSize, compThreadID, &numReserved);  // Acquire a cold/warm code cache.
+      _codeCache = manager->reserveCodeCache(false, codeSize, compThreadID, &numReserved, _comp->codeCacheKind());  // Acquire a cold/warm code cache.
       if (!codeCache())
          {
          // TODO: how do we pass back error codes to trigger retrial?
@@ -982,7 +1170,18 @@ TR_SharedCacheRelocationRuntime::allocateSpaceInCodeCache(UDATA codeSize)
       }
 
    uint8_t *coldCode;
-   U_8 *codeStart = manager->allocateCodeMemory(codeSize, 0, &_codeCache, &coldCode, false);
+   U_8 *codeStart;
+   bool installIntoCold = TR::Options::getCmdLineOptions()->getOption(TR_InstallAOTToColdCode);
+
+   if (!installIntoCold)
+      {
+      codeStart = manager->allocateCodeMemory(codeSize, 0, &_codeCache, &coldCode, false);
+      }
+   else
+      {
+      codeStart = manager->allocateCodeMemory(0, codeSize, &_codeCache, &coldCode, false);
+      }
+
    // FIXME: the GC may unload classes if code caches have been switched
    if (compThreadID >= 0 && fej9->getCompilationShouldBeInterruptedFlag())
       {
@@ -991,7 +1190,8 @@ TR_SharedCacheRelocationRuntime::allocateSpaceInCodeCache(UDATA codeSize)
       //*returnCode = compilationInterrupted; // allow retrial
       return NULL; // fail this AOT load
       }
-   return codeStart;
+
+   return installIntoCold ? coldCode : codeStart;
    }
 
 
@@ -1026,11 +1226,11 @@ TR_SharedCacheRelocationRuntime::initializeCacheDeltas()
    }
 
 void
-TR_SharedCacheRelocationRuntime::incompatibleCache(U_32 module_name, U_32 reason, char *assumeMessage)
+TR_SharedCacheRelocationRuntime::incompatibleCache(U_32 module_name, U_32 reason, const char *assumeMessage)
    {
    if (TR::Options::isAnyVerboseOptionSet())
       {
-      TR_VerboseLog::writeLineLocked(TR_Vlog_INFO, "%s\n", assumeMessage);
+      TR_VerboseLog::writeLineLocked(TR_Vlog_INFO, "%s", assumeMessage);
       }
 
    if (javaVM()->sharedClassConfig->verboseFlags & J9SHR_VERBOSEFLAG_ENABLE_VERBOSE)
@@ -1041,7 +1241,7 @@ TR_SharedCacheRelocationRuntime::incompatibleCache(U_32 module_name, U_32 reason
    }
 
 bool
-TR_SharedCacheRelocationRuntime::generateError(U_32 module_name, U_32 reason, char *assumeMessage)
+TR_SharedCacheRelocationRuntime::generateError(U_32 module_name, U_32 reason, const char *assumeMessage)
    {
    incompatibleCache(module_name, reason, assumeMessage);
    return false;
@@ -1052,7 +1252,7 @@ TR_SharedCacheRelocationRuntime::checkAOTHeaderFlags(const TR_AOTHeader *hdrInCa
    {
    bool defaultMessage = true;
 
-   if (!TR::Compiler->target.cpu.isCompatible(hdrInCache->processorDescription))
+   if (!TR::Compiler->relocatableTarget.cpu.isCompatible(hdrInCache->processorDescription))
       defaultMessage = generateError(J9NLS_RELOCATABLE_CODE_WRONG_HARDWARE, "AOT header validation failed: Processor incompatible.");
    if ((featureFlags & TR_FeatureFlag_sanityCheckBegin) != (hdrInCache->featureFlags & TR_FeatureFlag_sanityCheckBegin))
       defaultMessage = generateError(J9NLS_RELOCATABLE_CODE_HEADER_START_SANITY_BIT_MANGLED, "AOT header validation failed: Processor feature sanity bit mangled.");
@@ -1086,6 +1286,10 @@ TR_SharedCacheRelocationRuntime::checkAOTHeaderFlags(const TR_AOTHeader *hdrInCa
       defaultMessage = generateError(J9NLS_RELOCATABLE_CODE_HEAP_SIZE_FOR_BARRIER_RANGE_MISMATCH, "AOT header validation failed: Heap Size for Barrier Range feature mismatch.");
    if ((featureFlags & TR_FeatureFlag_IsVariableActiveCardTableBase) != (hdrInCache->featureFlags & TR_FeatureFlag_IsVariableActiveCardTableBase))
       defaultMessage = generateError(J9NLS_RELOCATABLE_CODE_ACTIVE_CARD_TABLE_BASE_MISMATCH, "AOT header validation failed: Active Card Table Base feature mismatch.");
+   if ((featureFlags & TR_FeatureFlag_ArrayHeaderShape) != (hdrInCache->featureFlags & TR_FeatureFlag_ArrayHeaderShape))
+      defaultMessage = generateError(J9NLS_RELOCATABLE_CODE_ARRAY_HEADER_SHAPE_MISMATCH, "AOT header validation failed: Array header shape mismatch.");
+   if ((featureFlags & TR_FeatureFlag_CHTableEnabled) != (hdrInCache->featureFlags & TR_FeatureFlag_CHTableEnabled))
+      defaultMessage = generateError(J9NLS_RELOCATABLE_CODE_CH_TABLE_MISMATCH, "AOT header validation failed: CH Table mismatch.");
    if ((featureFlags & TR_FeatureFlag_SanityCheckEnd) != (hdrInCache->featureFlags & TR_FeatureFlag_SanityCheckEnd))
       defaultMessage = generateError(J9NLS_RELOCATABLE_CODE_HEADER_END_SANITY_BIT_MANGLED, "AOT header validation failed: Trailing sanity bit mismatch.");
 
@@ -1093,22 +1297,6 @@ TR_SharedCacheRelocationRuntime::checkAOTHeaderFlags(const TR_AOTHeader *hdrInCa
       generateError(J9NLS_RELOCATABLE_CODE_UNKNOWN_PROBLEM, "AOT header validation failed: Unkown problem with processor features.");
    }
 
-// The method CS::Hash_FNV is being used to compute the hash value
-// Notice that currently CS2::Hash_FNV is a hash function that never returns 0
-uint32_t
-TR_SharedCacheRelocationRuntime::getCurrentLockwordOptionHashValue(J9JavaVM *vm) const
-   {
-   IDATA currentLockwordArgIndex = FIND_ARG_IN_VMARGS(STARTSWITH_MATCH, "-Xlockword", NULL);
-   uint32_t currentLockwordOptionHashValue = 0;
-   if (currentLockwordArgIndex >= 0)
-      {
-      char * currentLockwordOption = NULL;
-      GET_OPTION_VALUE(currentLockwordArgIndex, ':', &currentLockwordOption);
-      currentLockwordOptionHashValue = CS2::Hash_FNV((unsigned char*)currentLockwordOption, strlen(currentLockwordOption));
-      }
-   return currentLockwordOptionHashValue;
-   }
-
 OMRProcessorDesc
 TR_SharedCacheRelocationRuntime::getProcessorDescriptionFromSCC(J9VMThread *curThread)
    {
@@ -1183,12 +1371,20 @@ TR_SharedCacheRelocationRuntime::validateAOTHeader(TR_FrontEnd *fe, J9VMThread *
          }
       else if
          (hdrInCache->featureFlags != featureFlags ||
-          !TR::Compiler->target.cpu.isCompatible(hdrInCache->processorDescription)
+          !TR::Compiler->relocatableTarget.cpu.isCompatible(hdrInCache->processorDescription)
          )
          {
          checkAOTHeaderFlags(hdrInCache, featureFlags);
          }
-      else if (hdrInCache->gcPolicyFlag != javaVM()->memoryManagerFunctions->j9gc_modron_getWriteBarrierType(javaVM()) )
+      else if (!((hdrInCache->gcPolicyFlag == javaVM()->memoryManagerFunctions->j9gc_modron_getWriteBarrierType(javaVM())) ||
+                 // it's safe to run AOT code with inline cardmark barrier even if runtime will not need it since the barrier
+                 // is only executed for active Concurrent Mark cycles (which won't ever happen with Concurrent Mark disabled
+                 // in runtime)
+                 ((hdrInCache->gcPolicyFlag == gc_modron_wrtbar_cardmark_and_oldcheck) &&
+                  (javaVM()->memoryManagerFunctions->j9gc_modron_getWriteBarrierType(javaVM()) == gc_modron_wrtbar_oldcheck)
+                 )
+                )
+              )
          {
          incompatibleCache(J9NLS_RELOCATABLE_CODE_WRONG_GC_POLICY,
                            "AOT header validation failed: incompatible gc write barrier type");
@@ -1250,44 +1446,10 @@ TR_AOTHeader *
 TR_SharedCacheRelocationRuntime::createAOTHeader(TR_FrontEnd *fe)
    {
    PORT_ACCESS_FROM_JAVAVM(javaVM());
-
-   TR_J9VMBase *fej9 = (TR_J9VMBase *)fe;
-   TR_AOTHeader * aotHeader = (TR_AOTHeader *)j9mem_allocate_memory(sizeof(TR_AOTHeader), J9MEM_CATEGORY_JIT);
+   TR_AOTHeader *aotHeader = (TR_AOTHeader *)j9mem_allocate_memory(sizeof(TR_AOTHeader), J9MEM_CATEGORY_JIT);
 
    if (aotHeader)
-      {
-      memset(aotHeader, 0, sizeof(TR_AOTHeader));
-      aotHeader->eyeCatcher = TR_AOTHeaderEyeCatcher;
-
-      TR_Version *aotHeaderVersion = &aotHeader->version;
-      aotHeaderVersion->structSize = sizeof(TR_Version);
-      aotHeaderVersion->majorVersion = TR_AOTHeaderMajorVersion;
-      aotHeaderVersion->minorVersion = TR_AOTHeaderMinorVersion;
-      strncpy(aotHeaderVersion->vmBuildVersion, EsBuildVersionString, sizeof(aotHeaderVersion->vmBuildVersion) - 1);
-      aotHeaderVersion->vmBuildVersion[sizeof(aotHeaderVersion->vmBuildVersion) - 1] = '\0';
-      strncpy(aotHeaderVersion->jitBuildVersion, TR_BUILD_NAME, sizeof(aotHeaderVersion->jitBuildVersion) - 1);
-      aotHeaderVersion->jitBuildVersion[sizeof(aotHeaderVersion->jitBuildVersion) - 1] = '\0';
-
-      aotHeader->gcPolicyFlag = javaVM()->memoryManagerFunctions->j9gc_modron_getWriteBarrierType(javaVM());
-      aotHeader->lockwordOptionHashValue = getCurrentLockwordOptionHashValue(javaVM());
-      aotHeader->compressedPointerShift = javaVM()->memoryManagerFunctions->j9gc_objaccess_compressedPointersShift(javaVM()->internalVMFunctions->currentVMThread(javaVM()));
-
-      if (J9_ARE_ANY_BITS_SET(javaVM()->extendedRuntimeFlags2, J9_EXTENDED_RUNTIME2_ENABLE_PORTABLE_SHARED_CACHE))
-         {
-         TR::Compiler->relocatableTarget.cpu = TR::CPU::detectRelocatable(TR::Compiler->omrPortLib);
-         aotHeader->processorDescription = TR::Compiler->relocatableTarget.cpu.getProcessorDescription();
-         }
-      else
-         {
-         aotHeader->processorDescription = TR::Compiler->target.cpu.getProcessorDescription();
-         }
-
-      // Set up other feature flags
-      aotHeader->featureFlags = generateFeatureFlags(fe);
-
-      // Set ArrayLet Size if supported
-      aotHeader->arrayLetLeafSize = TR::Compiler->om.arrayletLeafSize();
-      }
+      fillAOTHeader(javaVM(), fe, aotHeader);
 
    return aotHeader;
    }
@@ -1361,76 +1523,6 @@ TR_SharedCacheRelocationRuntime::storeAOTHeader(TR_FrontEnd *fe, J9VMThread *cur
       }
    }
 
-uintptr_t
-TR_SharedCacheRelocationRuntime::generateFeatureFlags(TR_FrontEnd *fe)
-   {
-   uintptr_t featureFlags = 0;
-   TR_J9VMBase *fej9 = (TR_J9VMBase *)fe;
-
-   featureFlags |= TR_FeatureFlag_sanityCheckBegin;
-
-   if (TR::Compiler->target.isSMP())
-      featureFlags |= TR_FeatureFlag_IsSMP;
-
-   if (TR::Options::useCompressedPointers())
-      featureFlags |= TR_FeatureFlag_UsesCompressedPointers;
-
-   if (TR::Options::getCmdLineOptions()->getOption(TR_DisableTraps))
-      featureFlags |= TR_FeatureFlag_DisableTraps;
-
-   if (TR::Options::getCmdLineOptions()->getOption(TR_TLHPrefetch))
-      featureFlags |= TR_FeatureFlag_TLHPrefetch;
-
-   if (TR::CodeCacheManager::instance()->codeCacheConfig().needsMethodTrampolines())
-      featureFlags |= TR_FeatureFlag_MethodTrampolines;
-
-   if (TR::Options::getCmdLineOptions()->getOption(TR_FullSpeedDebug))
-      featureFlags |= TR_FeatureFlag_FSDEnabled;
-
-   if (TR::Options::getCmdLineOptions()->getOption(TR_EnableHCR))
-      featureFlags |= TR_FeatureFlag_HCREnabled;
-
-#ifdef TR_TARGET_S390
-   if (TR::Compiler->target.cpu.supportsFeature(OMR_FEATURE_S390_VECTOR_FACILITY))
-      featureFlags |= TR_FeatureFlag_SIMDEnabled;
-#endif
-
-   if (TR::Compiler->om.readBarrierType() != gc_modron_readbar_none)
-      {
-      featureFlags |= TR_FeatureFlag_ConcurrentScavenge;
-
-#ifdef TR_TARGET_S390
-      if (!TR::Compiler->target.cpu.supportsFeature(OMR_FEATURE_S390_GUARDED_STORAGE))
-         featureFlags |= TR_FeatureFlag_SoftwareReadBarrier;
-#endif
-      }
-
-
-   if (fej9->isAsyncCompilation())
-      featureFlags |= TR_FeatureFlag_AsyncCompilation;
-
-
-   if (!TR::Options::getCmdLineOptions()->getOption(TR_DisableTM) &&
-       !TR::Options::getAOTCmdLineOptions()->getOption(TR_DisableTM))
-      {
-      if (TR::Compiler->target.cpu.supportsTransactionalMemoryInstructions())
-         {
-         featureFlags |= TR_FeatureFlag_UsesTM;
-         }
-      }
-
-   if (TR::Options::getCmdLineOptions()->isVariableHeapBaseForBarrierRange0())
-      featureFlags |= TR_FeatureFlag_IsVariableHeapBaseForBarrierRange0;
-
-   if (TR::Options::getCmdLineOptions()->isVariableHeapSizeForBarrierRange0())
-      featureFlags |= TR_FeatureFlag_IsVariableHeapSizeForBarrierRange0;
-
-   if (TR::Options::getCmdLineOptions()->isVariableActiveCardTableBase())
-      featureFlags |= TR_FeatureFlag_IsVariableActiveCardTableBase;
-
-   return featureFlags;
-   }
-
 void TR_RelocationRuntime::initializeHWProfilerRecords(TR::Compilation *comp)
    {
    assert(comp != NULL);
@@ -1469,7 +1561,7 @@ TR_JITServerRelocationRuntime::allocateSpaceInCodeCache(UDATA codeSize)
       {
       int32_t numReserved;
 
-      _codeCache = manager->reserveCodeCache(false, codeSize, compThreadID, &numReserved);  // Acquire a cold/warm code cache.
+      _codeCache = manager->reserveCodeCache(false, codeSize, compThreadID, &numReserved, _comp->codeCacheKind());  // Acquire a cold/warm code cache.
       if (!codeCache())
          {
          // TODO: How do we pass back error codes to trigger retrial?
@@ -1517,14 +1609,14 @@ TR_JITServerRelocationRuntime::allocateSpaceInDataCache(uintptr_t metaDataSize,
    }
 
 uint8_t *
-TR_JITServerRelocationRuntime::copyDataToCodeCache(const void *startAddress, size_t totalSize, TR_J9VMBase *fe)
+TR_JITServerRelocationRuntime::copyDataToCodeCache(const void *startAddress, size_t totalSize, TR_J9VMBase *fe, TR::CodeCacheKind kind)
    {
    TR::CompilationInfoPerThreadBase *compInfoPT = fe->_compInfoPT;
    int32_t numReserved;
    TR::CodeCache *codeCache = NULL;
    TR::CodeCacheManager *manager = TR::CodeCacheManager::instance();
    TR_ASSERT(!compInfoPT->getCompilation()->cg()->getCodeCache(), "No code caches should be reserved when copying a thunk");
-   codeCache = manager->reserveCodeCache(false, totalSize, compInfoPT->getCompThreadId(), &numReserved);
+   codeCache = manager->reserveCodeCache(false, totalSize, compInfoPT->getCompThreadId(), &numReserved, kind);
    if (!codeCache)
       return NULL;
 
