@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef SYMBOL_VALIDATION_MANAGER_INCL
@@ -158,7 +158,7 @@ struct ClassValidationRecord : public SymbolValidationRecord
 struct ClassValidationRecordWithChain : public ClassValidationRecord
    {
    ClassValidationRecordWithChain(TR_ExternalRelocationTargetKind kind, TR_OpaqueClassBlock *clazz)
-      : ClassValidationRecord(kind), _class(clazz), _classChain(NULL)
+      : ClassValidationRecord(kind), _class(clazz), _classChainOffset(TR_SharedCache::INVALID_CLASS_CHAIN_OFFSET)
       {
 #if defined(J9VM_OPT_JITSERVER)
       _aotCacheClassChainRecord = NULL;
@@ -168,13 +168,13 @@ struct ClassValidationRecordWithChain : public ClassValidationRecord
    virtual void printFields();
 
 #if defined(J9VM_OPT_JITSERVER)
-   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() { return _aotCacheClassChainRecord; }
+   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() const { return _aotCacheClassChainRecord; }
 #else /* defined(J9VM_OPT_JITSERVER) */
-   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() { return NULL; }
+   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() const { return NULL; }
 #endif /* defined(J9VM_OPT_JITSERVER) */
 
    TR_OpaqueClassBlock *_class;
-   void *_classChain;
+   uintptr_t _classChainOffset;
 #if defined(J9VM_OPT_JITSERVER)
    const AOTCacheClassChainRecord *_aotCacheClassChainRecord;
 #endif /* defined(J9VM_OPT_JITSERVER) */
@@ -196,10 +196,10 @@ struct ClassByNameRecord : public ClassValidationRecordWithChain
 
 struct ProfiledClassRecord : public ClassValidationRecord
    {
-   ProfiledClassRecord(TR_OpaqueClassBlock *clazz, void *classChain,
+   ProfiledClassRecord(TR_OpaqueClassBlock *clazz, uintptr_t classChainOffset,
                        const AOTCacheClassChainRecord *aotCacheClassChainRecord = NULL)
       : ClassValidationRecord(TR_ValidateProfiledClass),
-        _class(clazz), _classChain(classChain)
+        _class(clazz), _classChainOffset(classChainOffset)
       {
 #if defined(J9VM_OPT_JITSERVER)
       _aotCacheClassChainRecord = aotCacheClassChainRecord;
@@ -210,13 +210,13 @@ struct ProfiledClassRecord : public ClassValidationRecord
    virtual void printFields();
 
 #if defined(J9VM_OPT_JITSERVER)
-   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() { return _aotCacheClassChainRecord; }
+   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() const { return _aotCacheClassChainRecord; }
 #else /* defined(J9VM_OPT_JITSERVER) */
-   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() { return NULL; }
+   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() const { return NULL; }
 #endif /* defined(J9VM_OPT_JITSERVER) */
 
    TR_OpaqueClassBlock *_class;
-   void *_classChain;
+   uintptr_t _classChainOffset;
 #if defined(J9VM_OPT_JITSERVER)
    const AOTCacheClassChainRecord *_aotCacheClassChainRecord;
 #endif /* defined(J9VM_OPT_JITSERVER) */
@@ -388,11 +388,11 @@ struct ConcreteSubClassFromClassRecord : public ClassValidationRecord
 
 struct ClassChainRecord : public SymbolValidationRecord
    {
-   ClassChainRecord(TR_OpaqueClassBlock *clazz, void *classChain,
+   ClassChainRecord(TR_OpaqueClassBlock *clazz, uintptr_t classChainOffset,
                     const AOTCacheClassChainRecord *aotCacheClassChainRecord = NULL)
       : SymbolValidationRecord(TR_ValidateClassChain),
         _class(clazz),
-        _classChain(classChain)
+        _classChainOffset(classChainOffset)
       {
 #if defined(J9VM_OPT_JITSERVER)
       _aotCacheClassChainRecord = aotCacheClassChainRecord;
@@ -403,13 +403,13 @@ struct ClassChainRecord : public SymbolValidationRecord
    virtual void printFields();
 
 #if defined(J9VM_OPT_JITSERVER)
-   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() { return _aotCacheClassChainRecord; }
+   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() const { return _aotCacheClassChainRecord; }
 #else /* defined(J9VM_OPT_JITSERVER) */
-   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() { return NULL; }
+   const AOTCacheClassChainRecord *getAOTCacheClassChainRecord() const { return NULL; }
 #endif /* defined(J9VM_OPT_JITSERVER) */
 
    TR_OpaqueClassBlock *_class;
-   void *_classChain;
+   uintptr_t _classChainOffset;
 #if defined(J9VM_OPT_JITSERVER)
    const AOTCacheClassChainRecord *_aotCacheClassChainRecord;
 #endif /* defined(J9VM_OPT_JITSERVER) */
@@ -691,12 +691,69 @@ struct J2IThunkFromMethodRecord : public SymbolValidationRecord
    TR_OpaqueMethodBlock *_method;
    };
 
+struct IsClassVisibleRecord : public SymbolValidationRecord
+   {
+   IsClassVisibleRecord(TR_OpaqueClassBlock *sourceClass, TR_OpaqueClassBlock *destClass, bool isVisible)
+      : SymbolValidationRecord(TR_ValidateIsClassVisible),
+        _sourceClass(sourceClass),
+        _destClass(destClass),
+        _isVisible(isVisible)
+      {}
+
+   virtual bool isLessThanWithinKind(SymbolValidationRecord *other);
+   virtual void printFields();
+
+   TR_OpaqueClassBlock *_sourceClass;
+   TR_OpaqueClassBlock *_destClass;
+   bool _isVisible;
+   };
+
+struct DynamicMethodFromCallsiteIndexRecord : public MethodValidationRecord
+   {
+   DynamicMethodFromCallsiteIndexRecord(TR_OpaqueMethodBlock *method,
+                                        TR_OpaqueMethodBlock *caller,
+                                        int32_t callsiteIndex,
+                                        bool appendixObjectNull)
+      : MethodValidationRecord(TR_ValidateDynamicMethodFromCallsiteIndex, method),
+      _caller(caller),
+      _callsiteIndex(callsiteIndex),
+      _appendixObjectNull(appendixObjectNull)
+      {}
+
+   virtual bool isLessThanWithinKind(SymbolValidationRecord *other);
+   virtual void printFields();
+
+   TR_OpaqueMethodBlock *_caller;
+   int32_t _callsiteIndex;
+   bool _appendixObjectNull;
+   };
+
+struct HandleMethodFromCPIndex  : public MethodValidationRecord
+   {
+   HandleMethodFromCPIndex(TR_OpaqueMethodBlock *method,
+                           TR_OpaqueMethodBlock *caller,
+                           int32_t cpIndex,
+                           bool appendixObjectNull)
+      : MethodValidationRecord(TR_ValidateHandleMethodFromCPIndex, method),
+      _caller(caller),
+      _cpIndex(cpIndex),
+      _appendixObjectNull(appendixObjectNull)
+      {}
+
+   virtual bool isLessThanWithinKind(SymbolValidationRecord *other);
+   virtual void printFields();
+
+   TR_OpaqueMethodBlock *_caller;
+   int32_t _cpIndex;
+   bool _appendixObjectNull;
+   };
+
 class SymbolValidationManager
    {
 public:
-   TR_ALLOC(TR_MemoryBase::SymbolValidationManager);
+   TR_ALLOC(TR_MemoryBase::SymbolValidationManager)
 
-   SymbolValidationManager(TR::Region &region, TR_ResolvedMethod *compilee);
+   SymbolValidationManager(TR::Region &region, TR_ResolvedMethod *compilee, TR::Compilation *comp);
 
    struct SystemClassNotWorthRemembering
       {
@@ -707,7 +764,6 @@ public:
 
    #define WELL_KNOWN_CLASS_COUNT 9
 
-   static void getWellKnownClassesSCCKey(char *buffer, size_t size, unsigned int includedClasses);
    void populateWellKnownClasses();
    bool validateWellKnownClasses(const uintptr_t *wellKnownClassChainOffsets);
    bool isWellKnownClass(TR_OpaqueClassBlock *clazz);
@@ -771,10 +827,13 @@ public:
                                           TR_OpaqueClassBlock *thisClass,
                                           int32_t vftSlot,
                                           TR_OpaqueMethodBlock *callerMethod);
+   bool addDynamicMethodFromCallsiteIndex(TR_OpaqueMethodBlock *method, TR_OpaqueMethodBlock *caller, int32_t callsiteIndex, bool appendixObjectNull);
+   bool addHandleMethodFromCPIndex(TR_OpaqueMethodBlock *method, TR_OpaqueMethodBlock *caller, int32_t cpIndex, bool appendixObjectNull);
 
    bool addStackWalkerMaySkipFramesRecord(TR_OpaqueMethodBlock *method, TR_OpaqueClassBlock *methodClass, bool skipFrames);
    bool addClassInfoIsInitializedRecord(TR_OpaqueClassBlock *clazz, bool isInitialized);
    void addJ2IThunkFromMethodRecord(void *thunk, TR_OpaqueMethodBlock *method);
+   bool addIsClassVisibleRecord(TR_OpaqueClassBlock *sourceClass, TR_OpaqueClassBlock *destClass, bool isVisible);
 
 
 
@@ -817,6 +876,18 @@ public:
                                                uint16_t thisClassID,
                                                int32_t vftSlot,
                                                uint16_t callerMethodID);
+   bool validateDynamicMethodFromCallsiteIndex(uint16_t methodID,
+                                               uint16_t callerID,
+                                               int32_t callsiteIndex,
+                                               bool appendixObjectNull,
+                                               uint16_t definingClassID,
+                                               uint32_t methodIndex);
+   bool validateHandleMethodFromCPIndex(uint16_t methodID,
+                                        uint16_t callerID,
+                                        int32_t cpIndex,
+                                        bool appendixObjectNull,
+                                        uint16_t definingClassID,
+                                        uint32_t methodIndex);
 
    bool validateStackWalkerMaySkipFramesRecord(uint16_t methodID, uint16_t methodClassID, bool couldSkipFrames);
    bool validateClassInfoIsInitializedRecord(uint16_t classID, bool wasInitialized);
@@ -826,6 +897,8 @@ public:
    // that the thunk loading logic can be confined to RelocationRecord.cpp.
    bool validateJ2IThunkFromMethodRecord(uint16_t thunkID, void *thunk);
 
+   bool validateIsClassVisibleRecord(uint16_t sourceClassID, uint16_t destClassID, bool wasVisible);
+
 
    TR_OpaqueClassBlock *getBaseComponentClass(TR_OpaqueClassBlock *clazz, int32_t & numDims);
 
@@ -842,8 +915,6 @@ public:
    static int getSystemClassesNotWorthRememberingCount();
 
 #if defined(J9VM_OPT_JITSERVER)
-   std::string serializeValueToSymbolMap();
-   void deserializeValueToSymbolMap(const std::string &valueToSymbolStr);
    static void populateSystemClassesNotWorthRemembering(ClientSessionData *clientData);
 #endif /* defined(J9VM_OPT_JITSERVER) */
 
@@ -869,7 +940,7 @@ private:
    struct ClassChainInfo
       {
       ClassChainInfo()
-         : _baseComponent(NULL), _baseComponentClassChain(NULL), _arrayDims(0)
+         : _baseComponent(NULL), _baseComponentClassChainOffset(TR_SharedCache::INVALID_CLASS_CHAIN_OFFSET), _arrayDims(0)
          {
 #if defined(J9VM_OPT_JITSERVER)
          _baseComponentAOTCacheClassChainRecord = NULL;
@@ -877,7 +948,7 @@ private:
          }
 
       TR_OpaqueClassBlock *_baseComponent;
-      void *_baseComponentClassChain;
+      uintptr_t _baseComponentClassChainOffset;
       int32_t _arrayDims;
 #if defined(J9VM_OPT_JITSERVER)
       const AOTCacheClassChainRecord *_baseComponentAOTCacheClassChainRecord;
