@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j9.h"
@@ -956,9 +956,8 @@ void arm64CodeCacheConfig(int32_t ccSizeInByte, int32_t *numTempTrampolines)
 void arm64CreateHelperTrampolines(void *trampPtr, int32_t numHelpers)
    {
    uint32_t *buffer = (uint32_t *)((uint8_t *)trampPtr + TRAMPOLINE_SIZE);
-#if defined(OSX) && defined(AARCH64)
-   pthread_jit_write_protect_np(0);
-#endif
+   omrthread_jit_write_protect_disable();
+
    for (int32_t i=1; i<numHelpers; i++)
       {
       *((int32_t *)buffer) = 0x58000050; //LDR R16 PC+8
@@ -972,9 +971,7 @@ void arm64CreateHelperTrampolines(void *trampPtr, int32_t numHelpers)
 #if defined(TR_HOST_ARM64)
    arm64CodeSync((uint8_t*)trampPtr, TRAMPOLINE_SIZE * numHelpers);
 #endif
-#if defined(OSX) && defined(AARCH64)
-   pthread_jit_write_protect_np(1);
-#endif
+   omrthread_jit_write_protect_enable();
    }
 
 void arm64CreateMethodTrampoline(void *trampPtr, void *startPC, void *method)
@@ -983,9 +980,7 @@ void arm64CreateMethodTrampoline(void *trampPtr, void *startPC, void *method)
    J9::PrivateLinkage::LinkageInfo *linkInfo = J9::PrivateLinkage::LinkageInfo::get(startPC);
    intptr_t dispatcher = (intptr_t)((uint8_t *)startPC + linkInfo->getReservedWord());
 
-#if defined(OSX) && defined(AARCH64)
-   pthread_jit_write_protect_np(0);
-#endif
+   omrthread_jit_write_protect_disable();
    *buffer = 0x58000050; //LDR R16 PC+8
    buffer += 1;
    *buffer = 0xD61F0200; //BR R16
@@ -995,9 +990,7 @@ void arm64CreateMethodTrampoline(void *trampPtr, void *startPC, void *method)
 #if defined(TR_HOST_ARM64)
    arm64CodeSync((uint8_t*)trampPtr, TRAMPOLINE_SIZE);
 #endif
-#if defined(OSX) && defined(AARCH64)
-   pthread_jit_write_protect_np(1);
-#endif
+   omrthread_jit_write_protect_enable();
    }
 
 #define ARM64_INSTRUCTION_LENGTH 4
@@ -1013,59 +1006,110 @@ static bool isInterfaceCallSite(uint8_t *callSite, intptr_t& addrOfFirstClassSlo
    {
    /*
     *  Following instruction sequence is used for interface call.
-    *  We can assume tmpReg is x9.
+    *  We can assume tmpReg is x10.
     *  Searching for the last 4 instructions
     *
-    *  ldrx tmpReg, L_firstClassCacheSlot
-    *  cmpx vftReg, tmpReg
-    *  ldrx tmpReg, L_firstBranchAddressCacheSlot
-    *  beq  hitLabel
-    *  ldrx tmpReg, L_secondClassCacheSlot
-    *  cmpx vftReg, tmpReg
-    *  bne  snippetLabel
-    *  ldrx tmpReg, L_secondBranchAddressCacheSlot
-    * hitLabel:
-    *  blr  tmpReg
-    * doneLabel:
+    *  If the instruction before `blr tmpReg` is `ldr tmpReg, label`,
+    *  then the lastITable cache is not used.
+    *  In that case, we expect the below instructions before the call site.
+    *  We obtain the address of the secondBranchAddressCacheSlot from `ldrx` instruction.
+    *
+    *      cmpx vftReg, tmpReg
+    *      bne  snippetLabel
+    *      ldrx tmpReg, L_secondBranchAddressCacheSlot
+    *     hitLabel:
+    *      blr  tmpReg
+    *     doneLabel:
+    *
+    *  If the instruction before `blr tmpReg` is `ldr tmpReg, [vftReg, x9]`,
+    *  then the lastITable cache is used.
+    *  In that case, we expect the below instructions before the call site.
+    *  We get the address of the interface call snippet from `bal snippetLabel` instruction.
+    *
+    *      bal  snippetLabel                           ; probably already patched to bne
+    *      mov  w9, sizeof(J9Class)
+    *      ldr  tmp2Reg, [tmpReg, iTableOffset]        ; load vTableOffset
+    *      sub  x9, x9, tmp2Reg                        ; icallVMprJavaSendPatchupVirtual expects x9 to hold vTable index
+    *      ldr  tmpReg, [vftReg, x9]
+    *     hitLabel:
+    *      blr  tmpReg
+    *     doneLabel:
     */
 
    int32_t blrInstr = *reinterpret_cast<int32_t *>(callSite);
-   /* Check if the instruction at the callSite is 'blr x9' */
-   if (blrInstr != 0xd63f0120)
+   /* Check if the instruction at the callSite is 'blr x10' */
+   if (blrInstr != 0xd63f0140)
       {
       return false;
       }
 
-   int32_t ldrInst = *reinterpret_cast<int32_t *>(callSite - ARM64_INSTRUCTION_LENGTH);
-   /* Check if the instruction before blr is 'ldrx x9, label' */
-   if ((ldrInst & 0xff00001f) != 0x58000009)
+   intptr_t ldrInstAddr = reinterpret_cast<intptr_t>(callSite) - ARM64_INSTRUCTION_LENGTH;
+   int32_t ldrInst = *reinterpret_cast<int32_t *>(ldrInstAddr);
+   /* Check if the instruction before blr is 'ldrx x10, label' */
+   if ((ldrInst & 0xff00001f) == 0x5800000a)
       {
-      return false;
-      }
-   /* distance is encoded in bit 5-23 */
-   int64_t distance = ((ldrInst << 8) >> 13) * 4;
-   intptr_t secondBranchAddressSlotAddr = reinterpret_cast<intptr_t>(callSite) - ARM64_INSTRUCTION_LENGTH + distance;
-   //     The layout of the cache slots is as follows:
-   //     +---------+---------------+---------+---------------+
-   //     |  class1 |method address1|  class2 |method address2|
-   //     +---------+---------------+---------+---------------+
-   addrOfFirstClassSlot = secondBranchAddressSlotAddr - sizeof(intptr_t) * 3;
+      /* distance is encoded in bit 5-23 */
+      int64_t distance = ((ldrInst << 8) >> 13) * 4;
+      intptr_t secondBranchAddressSlotAddr = ldrInstAddr + distance;
+      //     The layout of the cache slots is as follows:
+      //     +---------+---------------+---------+---------------+
+      //     |  class1 |method address1|  class2 |method address2|
+      //     +---------+---------------+---------+---------------+
+      addrOfFirstClassSlot = secondBranchAddressSlotAddr - sizeof(intptr_t) * 3;
 
-   int32_t bneInst = *reinterpret_cast<int32_t *>(callSite - ARM64_INSTRUCTION_LENGTH * 2);
-   /* Check if the instruction before ldr is 'bne' */
-   if ((bneInst & 0xff00001f) != 0x54000001)
+      int32_t bneInst = *reinterpret_cast<int32_t *>(ldrInstAddr - ARM64_INSTRUCTION_LENGTH);
+      /* Check if the instruction before ldr is 'bne' */
+      if ((bneInst & 0xff00001f) != 0x54000001)
+         {
+         return false;
+         }
+
+      int32_t cmpInst = *reinterpret_cast<int32_t *>(ldrInstAddr - ARM64_INSTRUCTION_LENGTH * 2);
+      /* Check if the instruction before bne is 'cmp vftReg, x10' */
+      if ((cmpInst & 0xfffffc1f) != 0xeb0a001f)
+         {
+         return false;
+         }
+
+      return true;
+      }
+   else if ((ldrInst & 0xfffffc1f) == 0xf869680a) /* Check if lastITable cache sequence is generated. The instruction before blr should be `ldr x10, [vftReg, x9]` */
       {
-      return false;
+      int32_t subInst = *reinterpret_cast<int32_t *>(callSite - ARM64_INSTRUCTION_LENGTH * 2);
+      /* Check if the instruction before ldr is `sub x9, x9, x11` */
+      if (subInst != 0xcb0b0129)
+         {
+         return false;
+         }
+      ldrInst = *reinterpret_cast<int32_t *>(callSite - ARM64_INSTRUCTION_LENGTH * 3);
+      /* Check if the instruction before sub is `ldr x11, [x10, #offset]` */
+      if ((ldrInst & 0xffc003ff) != 0xf940014b)
+         {
+         return false;
+         }
+      int32_t movInst = *reinterpret_cast<int32_t *>(callSite - ARM64_INSTRUCTION_LENGTH * 4);
+      /* Check if the instruction before ldr is `mov w9, sizeof(J9Class)` */
+      if ((movInst & 0xffe0001f) != 0x52800009)
+         {
+         return false;
+         }
+      intptr_t bcondInstAddr = reinterpret_cast<intptr_t>(callSite - ARM64_INSTRUCTION_LENGTH * 5);
+      int32_t bcondInst = *reinterpret_cast<int32_t *>(bcondInstAddr);
+      /* check if the instruction before mov is `b.cond snippetLabel` */
+      if ((bcondInst & 0xff000010) != 0x54000000)
+         {
+         return false;
+         }
+      /* distance is encoded in bit 5-23 */
+      int64_t distance = ((bcondInst << 8) >> 13) * 4;
+      /* offset of the fist class slot in interface call snippet */
+      static const int64_t firstClassSlotOffset = 44;
+      addrOfFirstClassSlot = bcondInstAddr + distance + firstClassSlotOffset;
+
+      return true;
       }
 
-   int32_t cmpInst = *reinterpret_cast<int32_t *>(callSite - ARM64_INSTRUCTION_LENGTH * 3);
-   /* Check if the instruction before bne is 'cmp vftReg, x9' */
-   if ((cmpInst & 0xfffffc1f) != 0xeb09001f)
-      {
-      return false;
-      }
-
-   return true;
+   return false;
    }
 
 bool arm64CodePatching(void *callee, void *callSite, void *currentPC, void *currentTramp, void *newAddrOfCallee, void *extra)
@@ -1104,16 +1148,12 @@ bool arm64CodePatching(void *callee, void *callSite, void *currentPC, void *curr
                }
             else
                {
-#if defined(OSX) && defined(AARCH64)
-               pthread_jit_write_protect_np(0);
-#endif
+               omrthread_jit_write_protect_disable();
                *((uint64_t*)currentTramp+1) = (uint64_t)entryAddress;
 #if defined(TR_HOST_ARM64)
                arm64CodeSync((uint8_t*)currentTramp+8, 8);
 #endif
-#if defined(OSX) && defined(AARCH64)
-               pthread_jit_write_protect_np(1);
-#endif
+               omrthread_jit_write_protect_enable();
                }
             }
 
@@ -1123,16 +1163,12 @@ bool arm64CodePatching(void *callee, void *callSite, void *currentPC, void *curr
       if (currentDistance != distance)
          {
          branchInstr |= (distance >> 2) & 0x03ffffff;
-#if defined(OSX) && defined(AARCH64)
-         pthread_jit_write_protect_np(0);
-#endif
+         omrthread_jit_write_protect_disable();
          *(int32_t *)callSite = branchInstr;
 #if defined(TR_HOST_ARM64)
          arm64CodeSync((uint8_t*)callSite, 4);
 #endif
-#if defined(OSX) && defined(AARCH64)
-         pthread_jit_write_protect_np(1);
-#endif
+         omrthread_jit_write_protect_enable();
          }
       }
    else if (isInterfaceCallSite(static_cast<uint8_t *>(callSite), addrOfFirstClassSlot))
@@ -1146,9 +1182,7 @@ bool arm64CodePatching(void *callee, void *callSite, void *currentPC, void *curr
       // Throwing away the flag bits in CLASS slot
       currentReceiverJ9Class &= TR::Compiler->om.maskOfObjectVftField();
 
-#if defined(OSX) && defined(AARCH64)
-      pthread_jit_write_protect_np(0);
-#endif
+      omrthread_jit_write_protect_disable();
       if (*reinterpret_cast<intptr_t *>(addrOfFirstClassSlot) == currentReceiverJ9Class)
          {
          *reinterpret_cast<intptr_t *>(addrOfFirstClassSlot + sizeof(intptr_t)) = reinterpret_cast<intptr_t>(entryAddress);
@@ -1157,9 +1191,7 @@ bool arm64CodePatching(void *callee, void *callSite, void *currentPC, void *curr
          {
          *reinterpret_cast<intptr_t *>(addrOfFirstClassSlot + sizeof(intptr_t) * 3) = reinterpret_cast<intptr_t>(entryAddress);
          }
-#if defined(OSX) && defined(AARCH64)
-      pthread_jit_write_protect_np(1);
-#endif
+      omrthread_jit_write_protect_enable();
       }
 
    return true;
