@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "x/codegen/CallSnippet.hpp"
@@ -91,7 +91,7 @@ uint8_t *TR::X86PicDataSnippet::encodeConstantPoolInfo(uint8_t *cursor)
       info->data3 = offsetToJ2IVirtualThunk;
 
       cg()->addExternalRelocation(
-         new (cg()->trHeapMemory()) TR::ExternalRelocation(
+         TR::ExternalRelocation::create(
             cursor,
             (uint8_t *)info,
             NULL,
@@ -104,24 +104,29 @@ uint8_t *TR::X86PicDataSnippet::encodeConstantPoolInfo(uint8_t *cursor)
    else if (_thunkAddress)
       {
       TR_ASSERT(comp->target().is64Bit(), "expecting a 64-bit target for thunk relocations");
-      cg()->addExternalRelocation(new (cg()->trHeapMemory()) TR::ExternalRelocation(cursor,
-                                                                             *(uint8_t **)cursor,
-                                                                             (uint8_t *)inlinedSiteIndex,
-                                                                             TR_Thunks, cg()),
-                             __FILE__,
-                             __LINE__,
-                             _startOfPicInstruction->getNode());
+      cg()->addExternalRelocation(
+         TR::ExternalRelocation::create(
+            cursor,
+            *(uint8_t **)cursor,
+            (uint8_t *)inlinedSiteIndex,
+            TR_Thunks,
+            cg()),
+         __FILE__,
+         __LINE__,
+         _startOfPicInstruction->getNode());
       }
    else
       {
-      cg()->addExternalRelocation(new (cg()->trHeapMemory()) TR::ExternalRelocation(cursor,
-                                                                                  (uint8_t *)cpAddr,
-                                                                                   (uint8_t *)inlinedSiteIndex,
-                                                                                   TR_ConstantPool,
-                                                                                   cg()),
-                             __FILE__,
-                             __LINE__,
-                             _startOfPicInstruction->getNode());
+      cg()->addExternalRelocation(
+         TR::ExternalRelocation::create(
+            cursor,
+            (uint8_t *)cpAddr,
+            (uint8_t *)inlinedSiteIndex,
+            TR_ConstantPool,
+            cg()),
+         __FILE__,
+         __LINE__,
+         _startOfPicInstruction->getNode());
       }
 
    // DD/DQ cpIndex
@@ -186,11 +191,15 @@ uint8_t *TR::X86PicDataSnippet::emitSnippetBody()
       disp32 = cg()->branchDisplacementToHelperOrTrampoline(cursor, _dispatchSymRef);
       *(int32_t *)(++cursor) = disp32;
 
-      cg()->addExternalRelocation(new (cg()->trHeapMemory())
-         TR::ExternalRelocation(cursor,
-                                    (uint8_t *)_dispatchSymRef,
-                                    TR_HelperAddress,
-                                    cg()), __FILE__, __LINE__, _startOfPicInstruction->getNode());
+      cg()->addExternalRelocation(
+         TR::ExternalRelocation::create(
+            cursor,
+            (uint8_t *)_dispatchSymRef,
+            TR_HelperAddress,
+            cg()),
+         __FILE__,
+         __LINE__,
+         _startOfPicInstruction->getNode());
       cursor += 4;
 
       // Lookup dispatch needs its stack map here.
@@ -362,11 +371,15 @@ uint8_t *TR::X86PicDataSnippet::emitSnippetBody()
       disp32 = cg()->branchDisplacementToHelperOrTrampoline(cursor, _dispatchSymRef);
       *(int32_t *)(++cursor) = disp32;
 
-      cg()->addExternalRelocation(new (cg()->trHeapMemory())
-         TR::ExternalRelocation(cursor,
-                                    (uint8_t *)_dispatchSymRef,
-                                    TR_HelperAddress,
-                                    cg()), __FILE__, __LINE__, _startOfPicInstruction->getNode());
+      cg()->addExternalRelocation(
+         TR::ExternalRelocation::create(
+            cursor,
+            (uint8_t *)_dispatchSymRef,
+            TR_HelperAddress,
+            cg()),
+         __FILE__,
+         __LINE__,
+         _startOfPicInstruction->getNode());
       cursor += 4;
 
       // Populate vtable dispatch needs its stack map here.
@@ -426,11 +439,15 @@ uint8_t *TR::X86PicDataSnippet::emitSnippetBody()
       disp32 = cg()->branchDisplacementToHelperOrTrampoline(picSlotCursor, resolveSlotHelperSymRef);
       *(int32_t *)(++picSlotCursor) = disp32;
 
-      cg()->addExternalRelocation(new (cg()->trHeapMemory())
-         TR::ExternalRelocation(picSlotCursor,
-                                    (uint8_t *)resolveSlotHelperSymRef,
-                                    TR_HelperAddress,
-                                    cg()),  __FILE__, __LINE__, _startOfPicInstruction->getNode());
+      cg()->addExternalRelocation(
+         TR::ExternalRelocation::create(
+            picSlotCursor,
+            (uint8_t *)resolveSlotHelperSymRef,
+            TR_HelperAddress,
+            cg()),
+         __FILE__,
+         __LINE__,
+         _startOfPicInstruction->getNode());
 
          picSlotCursor = (uint8_t *)(picSlotCursor - 1 + sizeofPicSlot);
 
@@ -442,11 +459,15 @@ uint8_t *TR::X86PicDataSnippet::emitSnippetBody()
             disp32 = cg()->branchDisplacementToHelperOrTrampoline(picSlotCursor, populateSlotHelperSymRef);
             *(int32_t *)(++picSlotCursor) = disp32;
 
-            cg()->addExternalRelocation(new (cg()->trHeapMemory())
-               TR::ExternalRelocation(picSlotCursor,
-                                          (uint8_t *)populateSlotHelperSymRef,
-                                          TR_HelperAddress,
-                                          cg()), __FILE__, __LINE__, _startOfPicInstruction->getNode());
+            cg()->addExternalRelocation(
+               TR::ExternalRelocation::create(
+                  picSlotCursor,
+                  (uint8_t *)populateSlotHelperSymRef,
+                  TR_HelperAddress,
+                  cg()),
+               __FILE__,
+               __LINE__,
+               _startOfPicInstruction->getNode());
             picSlotCursor = (uint8_t *)(picSlotCursor - 1 + sizeofPicSlot);
             }
       }
@@ -783,19 +804,19 @@ uint8_t *TR::X86CallSnippet::emitSnippetBody()
    {
    TR::Compilation *comp = cg()->comp();
    TR_J9VMBase*         fej9         = (TR_J9VMBase *)(cg()->fe());
+   TR::SymbolReferenceTable *srTab   = cg()->symRefTab();
    TR::SymbolReference* methodSymRef = _realMethodSymbolReference ? _realMethodSymbolReference : getNode()->getSymbolReference();
    TR::MethodSymbol*    methodSymbol = methodSymRef->getSymbol()->castToMethodSymbol();
    uint8_t*             cursor       = cg()->getBinaryBufferCursor();
 
    bool needToSetCodeLocation = true;
    bool isJitInduceOSRCall    = false;
+   bool isJitDispatchJ9Method = false;
 
-   if (comp->target().is64Bit() &&
-       methodSymbol->isHelper() &&
-       methodSymRef->isOSRInductionHelper())
-      {
+   if (methodSymbol->isHelper() && methodSymRef->isOSRInductionHelper())
       isJitInduceOSRCall = true;
-      }
+   else if (getNode()->isJitDispatchJ9MethodCall(comp))
+      isJitDispatchJ9Method = true;
 
    if (comp->target().is64Bit())
       {
@@ -806,8 +827,9 @@ uint8_t *TR::X86CallSnippet::emitSnippetBody()
       cursor = linkage->storeArguments(getNode(), cursor, false, NULL);
       needToSetCodeLocation = false;
 
-      if (cg()->hasCodeCacheSwitched() &&
-          (methodSymRef->getReferenceNumber()>=TR_AMD64numRuntimeHelpers))
+      if (cg()->hasCodeCacheSwitched()
+          && methodSymRef->getReferenceNumber() >= TR_AMD64numRuntimeHelpers
+          && !isJitDispatchJ9Method)
          {
          fej9->reserveTrampolineIfNecessary(comp, methodSymRef, true);
          }
@@ -858,17 +880,21 @@ uint8_t *TR::X86CallSnippet::emitSnippetBody()
       TR_RuntimeHelper resolutionHelper = methodSymbol->isStatic() ?
          TR_X86interpreterUnresolvedStaticGlue : TR_X86interpreterUnresolvedSpecialGlue;
 
-      TR::SymbolReference *helperSymRef = cg()->symRefTab()->findOrCreateRuntimeHelper(resolutionHelper);
+      TR::SymbolReference *helperSymRef = srTab->findOrCreateRuntimeHelper(resolutionHelper);
 
       *cursor = 0xe8;    // CALL
       int32_t disp32 = cg()->branchDisplacementToHelperOrTrampoline(cursor, helperSymRef);
       *(int32_t *)(++cursor) = disp32;
 
-      cg()->addExternalRelocation(new (cg()->trHeapMemory()) TR::ExternalRelocation(cursor,
-                                                                                    (uint8_t *)helperSymRef,
-                                                                                    TR_HelperAddress,
-                                                                                    cg()),
-                                  __FILE__, __LINE__, getNode());
+      cg()->addExternalRelocation(
+         TR::ExternalRelocation::create(
+            cursor,
+            (uint8_t *)helperSymRef,
+            TR_HelperAddress,
+            cg()),
+         __FILE__,
+         __LINE__,
+         getNode());
       cursor += 4;
 
       if (comp->target().is64Bit())
@@ -888,17 +914,21 @@ uint8_t *TR::X86CallSnippet::emitSnippetBody()
 
       // JMP interpreterStaticAndSpecialGlue
       //
-      helperSymRef = cg()->symRefTab()->findOrCreateRuntimeHelper(TR_X86interpreterStaticAndSpecialGlue);
+      helperSymRef = srTab->findOrCreateRuntimeHelper(TR_X86interpreterStaticAndSpecialGlue);
 
       *cursor = 0xe9;    // JMP
       disp32 = cg()->branchDisplacementToHelperOrTrampoline(cursor, helperSymRef);
       *(int32_t *)(++cursor) = disp32;
 
-      cg()->addExternalRelocation(new (cg()->trHeapMemory()) TR::ExternalRelocation(cursor,
-                                                                                    (uint8_t*)helperSymRef,
-                                                                                    TR_HelperAddress,
-                                                                                    cg()),
-                                  __FILE__, __LINE__, getNode());
+      cg()->addExternalRelocation(
+         TR::ExternalRelocation::create(
+            cursor,
+            (uint8_t*)helperSymRef,
+            TR_HelperAddress,
+            cg()),
+         __FILE__,
+         __LINE__,
+         getNode());
       cursor += 4;
 
       // DW dispatch helper index for the method's return type.
@@ -910,12 +940,16 @@ uint8_t *TR::X86CallSnippet::emitSnippetBody()
       intptr_t cpAddr = (intptr_t)methodSymRef->getOwningMethod(comp)->constantPool();
       *(intptr_t *)cursor = cpAddr;
 
-      cg()->addExternalRelocation(new (cg()->trHeapMemory()) TR::ExternalRelocation(cursor,
-                                                                                    *(uint8_t **)cursor,
-                                                                                    getNode() ? (uint8_t *)(uintptr_t)getNode()->getInlinedSiteIndex() : (uint8_t *)-1,
-                                                                                    TR_ConstantPool,
-                                                                                    cg()),
-                                  __FILE__, __LINE__, getNode());
+      cg()->addExternalRelocation(
+         TR::ExternalRelocation::create(
+            cursor,
+            *(uint8_t **)cursor,
+            getNode() ? (uint8_t *)(uintptr_t)getNode()->getInlinedSiteIndex() : (uint8_t *)-1,
+            TR_ConstantPool,
+            cg()),
+         __FILE__,
+         __LINE__,
+         getNode());
       cursor += sizeof(intptr_t);
 
       // DD cpIndex
@@ -948,7 +982,11 @@ uint8_t *TR::X86CallSnippet::emitSnippetBody()
       //SD: for jitInduceOSR we don't need to set the RAM method (the method that the VM needs to start executing)
       //because VM is going to figure what method to execute by looking up the jitPC in the GC map and finding
       //the desired invoke bytecode.
-      if (!isJitInduceOSRCall)
+      //
+      // For <jitDispatchJ9Method>, the method is passed in at runtime.
+      // Private linkage has already put it into the correct register.
+      //
+      if (!isJitInduceOSRCall && !isJitDispatchJ9Method)
          {
 #if defined(J9VM_OPT_JITSERVER)
          intptr_t ramMethod = comp->isOutOfProcessCompilation() && !methodSymbol->isInterpreted() ?
@@ -976,12 +1014,16 @@ uint8_t *TR::X86CallSnippet::emitSnippetBody()
 
          if (comp->getOption(TR_UseSymbolValidationManager))
             {
-            cg()->addExternalRelocation(new (cg()->trHeapMemory()) TR::ExternalRelocation(cursor,
-                                                                                          (uint8_t *)ramMethod,
-                                                                                          (uint8_t *)TR::SymbolType::typeMethod,
-                                                                                          TR_SymbolFromManager,
-                                                                                          cg()),
-                                        __FILE__, __LINE__, getNode());
+            cg()->addExternalRelocation(
+               TR::ExternalRelocation::create(
+                  cursor,
+                  (uint8_t *)ramMethod,
+                  (uint8_t *)TR::SymbolType::typeMethod,
+                  TR_SymbolFromManager,
+                  cg()),
+               __FILE__,
+               __LINE__,
+               getNode());
             }
 
          // HCR in TR::X86CallSnippet::emitSnippetBody register the method address
@@ -996,18 +1038,40 @@ uint8_t *TR::X86CallSnippet::emitSnippetBody()
       //
       *cursor = 0xe9;
 
-      TR::SymbolReference* dispatchSymRef =
-          methodSymbol->isHelper() && methodSymRef->isOSRInductionHelper() ? methodSymRef :
-                                                                             cg()->symRefTab()->findOrCreateRuntimeHelper(TR_X86interpreterStaticAndSpecialGlue);
+      TR::SymbolReference *dispatchSymRef = NULL;
+      if (isJitInduceOSRCall)
+         {
+         dispatchSymRef = methodSymRef;
+         }
+      else
+         {
+         // For <jitDispatchJ9Method>, jump directly to j2iTransition instead
+         // of using the interpreter glue. Most of the time the glue would also
+         // go to j2iTransition, but first it would check to see if the callee
+         // is compiled, and if so, it would patch the call to target the JIT
+         // body instead of this call snippet, which is incorrect if the callee
+         // varies at runtime. Testing for a JIT body is redundant anyway
+         // because <jitDispatchJ9Method> has already just done that, and there
+         // wasn't one.
+         TR_RuntimeHelper helper = isJitDispatchJ9Method
+            ? TR_j2iTransition
+            : TR_X86interpreterStaticAndSpecialGlue;
+
+         dispatchSymRef = srTab->findOrCreateRuntimeHelper(helper);
+         }
 
       int32_t disp32 = cg()->branchDisplacementToHelperOrTrampoline(cursor, dispatchSymRef);
       *(int32_t *)(++cursor) = disp32;
 
-      cg()->addExternalRelocation(new (cg()->trHeapMemory()) TR::ExternalRelocation(cursor,
-                                                                                    (uint8_t *)dispatchSymRef,
-                                                                                    TR_HelperAddress,
-                                                                                    cg()),
-                                  __FILE__, __LINE__, getNode());
+      cg()->addExternalRelocation(
+         TR::ExternalRelocation::create(
+            cursor,
+            (uint8_t *)dispatchSymRef,
+            TR_HelperAddress,
+            cg()),
+         __FILE__,
+         __LINE__,
+         getNode());
       cursor += 4;
       }
 
