@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2019 IBM Corp. and others
+ * Copyright IBM Corp. and others 2000
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "z/codegen/S390Register.hpp"
@@ -393,7 +393,7 @@ TR_StorageReference::getMaxSharedNodeSize()
       ListIterator<TR::Node> listIt(_sharedNodes);
       for (TR::Node *listNode = listIt.getFirst(); listNode; listNode = listIt.getNext())
          {
-         int32_t nodeSize = listNode->getStorageReferenceSize();
+         int32_t nodeSize = listNode->getStorageReferenceSize(comp());
          if (nodeSize > maxSize)
             {
             if (comp()->cg()->traceBCDCodeGen())
