@@ -1,6 +1,6 @@
 
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -16,9 +16,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 
@@ -48,6 +48,7 @@
 #include "ObjectHeapIterator.hpp"
 #include "ObjectModel.hpp"
 #include "OwnableSynchronizerObjectList.hpp"
+#include "ContinuationObjectList.hpp"
 #include "PointerArrayIterator.hpp"
 #include "SegmentIterator.hpp"
 #include "StringTable.hpp"
@@ -73,6 +74,12 @@ MM_HeapRootScanner::doOwnableSynchronizerObject(J9Object *objectPtr)
 	doObject(objectPtr);
 }
 
+void
+MM_HeapRootScanner::doContinuationObject(J9Object *objectPtr)
+{
+	doObject(objectPtr);
+}
+
 #if defined(J9VM_GC_FINALIZATION)
 /**
  * @todo Provide function documentation
@@ -101,7 +108,7 @@ MM_HeapRootScanner::doFinalizableObject(J9Object *objectPtr)
 void
 MM_HeapRootScanner::doMonitorReference(J9ObjectMonitor *objectMonitor, GC_HashTableIterator *monitorReferenceIterator)
 {
-	J9ThreadAbstractMonitor * monitor = (J9ThreadAbstractMonitor *)objectMonitor->monitor;
+	J9ThreadAbstractMonitor *monitor = (J9ThreadAbstractMonitor *)objectMonitor->monitor;
 	
 	void *userData = (void *)monitor->userData;
 	
@@ -166,6 +173,7 @@ MM_HeapRootScanner::scanClasses()
 #if defined(J9VM_GC_DYNAMIC_CLASS_UNLOADING)
 	J9ClassLoader *sysClassLoader = _javaVM->systemClassLoader;
 	J9ClassLoader *appClassLoader = _javaVM->applicationClassLoader;
+	J9ClassLoader *extClassLoader = _javaVM->extensionClassLoader;
 	MM_GCExtensions::DynamicClassUnloading dynamicClassUnloadingFlag = (MM_GCExtensions::DynamicClassUnloading )_extensions->dynamicClassUnloading;
 #endif
 	
@@ -173,17 +181,18 @@ MM_HeapRootScanner::scanClasses()
 	
 	GC_SegmentIterator segmentIterator(_javaVM->classMemorySegments, MEMORY_TYPE_RAM_CLASS);
 	
-	while(J9MemorySegment *segment = segmentIterator.nextSegment()) {
+	while (J9MemorySegment *segment = segmentIterator.nextSegment()) {
 		GC_ClassHeapIterator classHeapIterator(_javaVM, segment);
 		J9Class *clazz = NULL;
-		while(NULL != (clazz = classHeapIterator.nextClass())) {
+		while (NULL != (clazz = classHeapIterator.nextClass())) {
 			RootScannerEntityReachability reachability;
 #if defined(J9VM_GC_DYNAMIC_CLASS_UNLOADING)
 			if (MM_GCExtensions::DYNAMIC_CLASS_UNLOADING_NEVER == dynamicClassUnloadingFlag) {
 				/* if -Xnoclassgc, all classes are strong */
 				reachability = RootScannerEntityReachability_Strong;
 			} else {
-				if ((sysClassLoader == clazz->classLoader) || (appClassLoader == clazz->classLoader)) {
+				J9ClassLoader *classLoader = clazz->classLoader;
+				if ((classLoader == sysClassLoader) || (classLoader == appClassLoader) || (classLoader == extClassLoader)) {
 					reachability = RootScannerEntityReachability_Strong;
 				} else {
 					reachability = RootScannerEntityReachability_Weak;
@@ -257,21 +266,22 @@ MM_HeapRootScanner::scanClassLoaders()
 #if defined(J9VM_GC_DYNAMIC_CLASS_UNLOADING)
 	J9ClassLoader *sysClassLoader = _javaVM->systemClassLoader;
 	J9ClassLoader *appClassLoader = _javaVM->applicationClassLoader;
+	J9ClassLoader *extClassLoader = _javaVM->extensionClassLoader;
 	MM_GCExtensions::DynamicClassUnloading dynamicClassUnloadingFlag = (MM_GCExtensions::DynamicClassUnloading )_extensions->dynamicClassUnloading;
 #endif
-	J9ClassLoader *classLoader;	
+	J9ClassLoader *classLoader = NULL;
 	GC_ClassLoaderIterator classLoaderIterator(_javaVM->classLoaderBlocks);
 	
 	reportScanningStarted(RootScannerEntity_ClassLoaders);
 	
-	while((classLoader = classLoaderIterator.nextSlot()) != NULL) {
+	while ((classLoader = classLoaderIterator.nextSlot()) != NULL) {
 		RootScannerEntityReachability reachability;
 #if defined(J9VM_GC_DYNAMIC_CLASS_UNLOADING)
 		if (MM_GCExtensions::DYNAMIC_CLASS_UNLOADING_NEVER == dynamicClassUnloadingFlag) {
 			/* if -Xnoclassgc, all class loader refs are strong */
 			reachability = RootScannerEntityReachability_Strong;
 		} else {
-			if (classLoader == appClassLoader || classLoader == sysClassLoader) {
+			if ((classLoader == appClassLoader) || (classLoader == sysClassLoader) || (classLoader == extClassLoader)) {
 				reachability = RootScannerEntityReachability_Strong;
 			} else {
 				reachability = RootScannerEntityReachability_Weak;	
@@ -304,7 +314,7 @@ MM_HeapRootScanner::scanThreads()
 	
 	GC_VMThreadListIterator vmThreadListIterator(_javaVM);
 	
-	while(J9VMThread *walkThread = vmThreadListIterator.nextVMThread()) {
+	while (J9VMThread *walkThread = vmThreadListIterator.nextVMThread()) {
 		if (scanOneThread(walkThread)) {
 			vmThreadListIterator.reset(_javaVM->mainThread);
 		}
@@ -330,7 +340,7 @@ MM_HeapRootScanner::scanOneThread(J9VMThread* walkThread)
 {
 	GC_VMThreadIterator vmThreadIterator(walkThread);
 	
-	while(J9Object **slot = vmThreadIterator.nextSlot()) {
+	while (J9Object **slot = vmThreadIterator.nextSlot()) {
 		doVMThreadSlot(slot, &vmThreadIterator);
 	}
 	
@@ -347,7 +357,7 @@ MM_HeapRootScanner::scanFinalizableObjects()
 	reportScanningStarted(RootScannerEntity_FinalizableObjects);
 	setReachability(RootScannerEntityReachability_Strong);
 	
-	GC_FinalizeListManager * finalizeListManager = _extensions->finalizeListManager;
+	GC_FinalizeListManager *finalizeListManager = _extensions->finalizeListManager;
 	{
 		/* walk finalizable objects created by the system class loader */
 		j9object_t systemObject = finalizeListManager->peekSystemFinalizableObject();
@@ -391,7 +401,7 @@ MM_HeapRootScanner::scanJNIGlobalReferences()
 	GC_JNIGlobalReferenceIterator jniGlobalReferenceIterator(_javaVM->jniGlobalReferences);
 	J9Object **slot;
 
-	while((slot = (J9Object **)jniGlobalReferenceIterator.nextSlot()) != NULL) {
+	while ((slot = (J9Object **)jniGlobalReferenceIterator.nextSlot()) != NULL) {
 		doJNIGlobalReferenceSlot(slot, &jniGlobalReferenceIterator);
 	}
 
@@ -415,11 +425,11 @@ MM_HeapRootScanner::scanStringTable()
 	setReachability(reachability);
 	
 	MM_StringTable *stringTable = MM_GCExtensions::getExtensions(_javaVM->omrVM)->getStringTable();
-	for (UDATA tableIndex = 0; tableIndex < stringTable->getTableCount(); tableIndex++) {
+	for (uintptr_t tableIndex = 0; tableIndex < stringTable->getTableCount(); tableIndex++) {
 		GC_HashTableIterator stringTableIterator(stringTable->getTable(tableIndex));
 		J9Object **slot;
 
-		while((slot = (J9Object **)stringTableIterator.nextSlot()) != NULL) {
+		while ((slot = (J9Object **)stringTableIterator.nextSlot()) != NULL) {
 			doStringTableSlot(slot, NULL);
 		}
 	}
@@ -447,7 +457,7 @@ MM_HeapRootScanner::scanUnfinalizedObjects()
 	MM_ObjectAccessBarrier *barrier = _extensions->accessBarrier;
 	MM_UnfinalizedObjectList *unfinalizedObjectList = _extensions->unfinalizedObjectLists;
 
-	while(NULL != unfinalizedObjectList) {
+	while (NULL != unfinalizedObjectList) {
 		J9Object *objectPtr = unfinalizedObjectList->getHeadOfList();
 		while (NULL != objectPtr) {
 			doUnfinalizedObject(objectPtr);
@@ -468,7 +478,7 @@ MM_HeapRootScanner::scanOwnableSynchronizerObjects()
 	MM_ObjectAccessBarrier *barrier = _extensions->accessBarrier;
 	MM_OwnableSynchronizerObjectList *ownableSynchronizerObjectList = _extensions->getOwnableSynchronizerObjectLists();
 
-	while(NULL != ownableSynchronizerObjectList) {
+	while (NULL != ownableSynchronizerObjectList) {
 		J9Object *objectPtr = ownableSynchronizerObjectList->getHeadOfList();
 		while (NULL != objectPtr) {
 			doOwnableSynchronizerObject(objectPtr);
@@ -479,6 +489,26 @@ MM_HeapRootScanner::scanOwnableSynchronizerObjects()
 	reportScanningEnded(RootScannerEntity_OwnableSynchronizerObjects);
 }
 
+void
+MM_HeapRootScanner::scanContinuationObjects()
+{
+	reportScanningStarted(RootScannerEntity_ContinuationObjects);
+	setReachability(RootScannerEntityReachability_Weak);
+
+	MM_ObjectAccessBarrier *barrier = _extensions->accessBarrier;
+	MM_ContinuationObjectList *continuationObjectList = _extensions->getContinuationObjectLists();
+
+	while (NULL != continuationObjectList) {
+		J9Object *objectPtr = continuationObjectList->getHeadOfList();
+		while (NULL != objectPtr) {
+			doContinuationObject(objectPtr);
+			objectPtr = barrier->getContinuationLink(objectPtr);
+		}
+		continuationObjectList = continuationObjectList->getNextList();
+	}
+	reportScanningEnded(RootScannerEntity_ContinuationObjects);
+}
+
 /**
  * @todo Provide function documentation
  */
@@ -516,7 +546,7 @@ MM_HeapRootScanner::scanJNIWeakGlobalReferences()
 	GC_JNIWeakGlobalReferenceIterator jniWeakGlobalReferenceIterator(_javaVM->jniWeakGlobalReferences);
 	J9Object **slot;
 
-	while((slot = (J9Object **)jniWeakGlobalReferenceIterator.nextSlot()) != NULL) {
+	while ((slot = (J9Object **)jniWeakGlobalReferenceIterator.nextSlot()) != NULL) {
 		doJNIWeakGlobalReference(slot);
 	}
 
@@ -538,9 +568,9 @@ MM_HeapRootScanner::scanRememberedSet()
 	
 	GC_RememberedSetIterator rememberedSetIterator(&_extensions->rememberedSet);
 	
-	while((puddle = rememberedSetIterator.nextList()) != NULL) {
+	while ((puddle = rememberedSetIterator.nextList()) != NULL) {
 		GC_RememberedSetSlotIterator rememberedSetSlotIterator(puddle);
-		while((slotPtr = (J9Object **)rememberedSetSlotIterator.nextSlot()) != NULL) {
+		while ((slotPtr = (J9Object **)rememberedSetSlotIterator.nextSlot()) != NULL) {
 			doRememberedSetSlot(slotPtr, &rememberedSetSlotIterator);
 		}
 	}
@@ -564,9 +594,9 @@ MM_HeapRootScanner::scanJVMTIObjectTagTables()
 	J9Object **slotPtr;	
 	if (NULL != jvmtiData) {
 		GC_JVMTIObjectTagTableListIterator objectTagTableList( jvmtiData->environments);
-		while(NULL != (jvmtiEnv = (J9JVMTIEnv *)objectTagTableList.nextSlot())) {
+		while (NULL != (jvmtiEnv = (J9JVMTIEnv *)objectTagTableList.nextSlot())) {
 			GC_JVMTIObjectTagTableIterator objectTagTableIterator(jvmtiEnv->objectTagTable);
-			while(NULL != (slotPtr = (J9Object **)objectTagTableIterator.nextSlot())) {
+			while (NULL != (slotPtr = (J9Object **)objectTagTableIterator.nextSlot())) {
 				doJVMTIObjectTagSlot(slotPtr, &objectTagTableIterator);
 			}
 		}
