@@ -1,6 +1,6 @@
 
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -16,9 +16,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j9.h"
@@ -44,7 +44,7 @@
 
 #if defined(J9VM_GC_REALTIME)
 extern "C" {
-void classLoaderLoadHook(J9HookInterface** hook, UDATA eventNum, void* eventData, void* userData);
+void classLoaderLoadHook(J9HookInterface** hook, uintptr_t eventNum, void *eventData, void *userData);
 }
 #endif /* defined(J9VM_GC_REALTIME) */
 
@@ -53,7 +53,7 @@ MM_ClassLoaderManager::newInstance(MM_EnvironmentBase *env, MM_GlobalCollector *
 {	
 	MM_ClassLoaderManager *classLoaderManager = (MM_ClassLoaderManager *)env->getForge()->allocate(sizeof(MM_ClassLoaderManager), MM_AllocationCategory::FIXED, J9_GET_CALLSITE());
 	if (classLoaderManager) {
-		new(classLoaderManager) MM_ClassLoaderManager(env, globalCollector);
+		new (classLoaderManager) MM_ClassLoaderManager(env, globalCollector);
 		if (!classLoaderManager->initialize(env)) {
 			classLoaderManager->kill(env);
 			classLoaderManager = NULL;   			
@@ -173,7 +173,7 @@ MM_ClassLoaderManager::flushUndeadSegments(MM_EnvironmentBase *env)
 void
 MM_ClassLoaderManager::setLastUnloadNumOfClassLoaders() 
 {
-	_lastUnloadNumOfClassLoaders =  (UDATA)pool_numElements(_javaVM->classLoaderBlocks); 
+	_lastUnloadNumOfClassLoaders =  (uintptr_t)pool_numElements(_javaVM->classLoaderBlocks);
 }
 
 void
@@ -207,8 +207,8 @@ MM_ClassLoaderManager::isTimeForClassUnloading(MM_EnvironmentBase *env)
 
 #if defined(J9VM_GC_DYNAMIC_CLASS_UNLOADING)
 
-	UDATA numClassLoaderBlocks = pool_numElements(_javaVM->classLoaderBlocks);
-	UDATA numAnonymousClasses = _javaVM->anonClassCount;
+	uintptr_t numClassLoaderBlocks = pool_numElements(_javaVM->classLoaderBlocks);
+	uintptr_t numAnonymousClasses = _javaVM->anonClassCount;
 
 	Trc_MM_GlobalCollector_isTimeForClassUnloading_Entry(
 			_extensions->dynamicClassUnloading,
@@ -225,8 +225,8 @@ MM_ClassLoaderManager::isTimeForClassUnloading(MM_EnvironmentBase *env)
 
 	Assert_MM_true(numAnonymousClasses >= _lastUnloadNumOfAnonymousClasses);
 
-	if ( _extensions->dynamicClassUnloading != MM_GCExtensions::DYNAMIC_CLASS_UNLOADING_NEVER ) {
-		UDATA recentlyLoaded = (UDATA) ((numAnonymousClasses - _lastUnloadNumOfAnonymousClasses) *  _extensions->classUnloadingAnonymousClassWeight);
+	if (_extensions->dynamicClassUnloading != MM_GCExtensions::DYNAMIC_CLASS_UNLOADING_NEVER) {
+		uintptr_t recentlyLoaded = (uintptr_t)((numAnonymousClasses - _lastUnloadNumOfAnonymousClasses) *  _extensions->classUnloadingAnonymousClassWeight);
 		/* todo aryoung: _lastUnloadNumOfClassLoaders includes the class loaders which
 		 * were unloaded but still required finalization when the last classUnloading occured.
 		 * This means that the threshold check is wrong when there are classes which require finalization.
@@ -246,7 +246,7 @@ MM_ClassLoaderManager::isTimeForClassUnloading(MM_EnvironmentBase *env)
 
 #if defined(J9VM_GC_DYNAMIC_CLASS_UNLOADING)
 J9ClassLoader *
-MM_ClassLoaderManager::identifyClassLoadersToUnload(MM_EnvironmentBase *env, MM_HeapMap *markMap, MM_ClassUnloadStats* classUnloadStats)
+MM_ClassLoaderManager::identifyClassLoadersToUnload(MM_EnvironmentBase *env, MM_HeapMap *markMap, MM_ClassUnloadStats *classUnloadStats)
 {
 	Trc_MM_identifyClassLoadersToUnload_Entry(env->getLanguageVMThread());
 	
@@ -255,25 +255,25 @@ MM_ClassLoaderManager::identifyClassLoadersToUnload(MM_EnvironmentBase *env, MM_
 	classUnloadStats->_classLoaderCandidates = 0;
 
 	GC_ClassLoaderIterator classLoaderIterator(_javaVM->classLoaderBlocks);
-	J9ClassLoader * classLoader = NULL;
-	while( NULL != (classLoader = classLoaderIterator.nextSlot()) ) {
+	J9ClassLoader *classLoader = NULL;
+	while (NULL != (classLoader = classLoaderIterator.nextSlot())) {
 		classUnloadStats->_classLoaderCandidates += 1;
 		/* Check if the class loader is already DEAD - ignore if it is */
-		if( J9_GC_CLASS_LOADER_DEAD == (classLoader->gcFlags & J9_GC_CLASS_LOADER_DEAD) ) {
+		if (J9_GC_CLASS_LOADER_DEAD == (classLoader->gcFlags & J9_GC_CLASS_LOADER_DEAD)) {
 			/* If the class loader is already dead, it should be enqueued or unloading by now */  
-			Assert_MM_true( 0 != (classLoader->gcFlags & (J9_GC_CLASS_LOADER_UNLOADING | J9_GC_CLASS_LOADER_ENQ_UNLOAD)) );
-			Assert_MM_true( 0 == (classLoader->gcFlags & J9_GC_CLASS_LOADER_SCANNED) ); 
+			Assert_MM_true(0 != (classLoader->gcFlags & (J9_GC_CLASS_LOADER_UNLOADING | J9_GC_CLASS_LOADER_ENQ_UNLOAD)));
+			Assert_MM_true(0 == (classLoader->gcFlags & J9_GC_CLASS_LOADER_SCANNED));
 		} else {
 			/* If the class loader isn't already dead, it must not be enqueued or unloading */  
-			Assert_MM_true( 0 == (classLoader->gcFlags & (J9_GC_CLASS_LOADER_UNLOADING | J9_GC_CLASS_LOADER_ENQ_UNLOAD)) );
+			Assert_MM_true(0 == (classLoader->gcFlags & (J9_GC_CLASS_LOADER_UNLOADING | J9_GC_CLASS_LOADER_ENQ_UNLOAD)));
 			Assert_MM_true(NULL == classLoader->unloadLink);
 
 			/* Is the class loader still alive? (object may be NULL while the loader is being initialized) */
 			J9Object *classLoaderObject = classLoader->classLoaderObject;
-			if( (NULL != classLoaderObject) && (!markMap->isBitSet(classLoaderObject)) ) {
+			if ((NULL != classLoaderObject) && (!markMap->isBitSet(classLoaderObject))) {
 				/* Anonymous classloader should not be unloaded */
 				Assert_MM_true(0 == (classLoader->flags & J9CLASSLOADER_ANON_CLASS_LOADER));
-				Assert_MM_true( 0 == (classLoader->gcFlags & J9_GC_CLASS_LOADER_SCANNED) ); 
+				Assert_MM_true(0 == (classLoader->gcFlags & J9_GC_CLASS_LOADER_SCANNED));
 
 				/* add this loader to the linked list of loaders being unloaded in this cycle */
 				classLoader->unloadLink = unloadLink;
@@ -300,11 +300,11 @@ MM_ClassLoaderManager::identifyClassLoadersToUnload(MM_EnvironmentBase *env, MM_
 }
 
 void
-MM_ClassLoaderManager::cleanUpClassLoadersStart(MM_EnvironmentBase *env, J9ClassLoader* classLoaderUnloadList, MM_HeapMap *markMap, MM_ClassUnloadStats *classUnloadStats)
+MM_ClassLoaderManager::cleanUpClassLoadersStart(MM_EnvironmentBase *env, J9ClassLoader *classLoaderUnloadList, MM_HeapMap *markMap, MM_ClassUnloadStats *classUnloadStats)
 {
-	UDATA classUnloadCount = 0;
-	UDATA anonymousClassUnloadCount = 0;
-	UDATA classLoaderUnloadCount = 0;
+	uintptr_t classUnloadCount = 0;
+	uintptr_t anonymousClassUnloadCount = 0;
+	uintptr_t classLoaderUnloadCount = 0;
 	J9VMThread *vmThread = (J9VMThread *)env->getLanguageVMThread();
 
 	J9Class *classUnloadList = NULL;
@@ -335,7 +335,7 @@ MM_ClassLoaderManager::cleanUpClassLoadersStart(MM_EnvironmentBase *env, J9Class
 	classUnloadCount += anonymousClassUnloadCount;
 
 	/* Count all classes loaded by dying class loaders */
-	J9ClassLoader * classLoader = classLoaderUnloadList;
+	J9ClassLoader *classLoader = classLoaderUnloadList;
 	while (NULL != classLoader) {
 		Assert_MM_true( 0 == (classLoader->gcFlags & J9_GC_CLASS_LOADER_SCANNED) );
 		classLoaderUnloadCount += 1;
@@ -365,9 +365,7 @@ MM_ClassLoaderManager::cleanUpClassLoadersStart(MM_EnvironmentBase *env, J9Class
 		TRIGGER_J9HOOK_VM_CLASS_LOADERS_UNLOAD(_javaVM->hookInterface, vmThread, classLoaderUnloadList);
 	}
 
-	classUnloadStats->_classesUnloadedCount = classUnloadCount;
-	classUnloadStats->_classLoaderUnloadedCount = classLoaderUnloadCount;
-	classUnloadStats->_anonymousClassesUnloadedCount = anonymousClassUnloadCount;
+	classUnloadStats->updateUnloadedCounters(anonymousClassUnloadCount, classUnloadCount, classLoaderUnloadCount);
 
 	/* Ensure that the vm has an accurate number of currently loaded anonymous classes  */
 	_javaVM->anonClassCount -= anonymousClassUnloadCount;
@@ -376,19 +374,19 @@ MM_ClassLoaderManager::cleanUpClassLoadersStart(MM_EnvironmentBase *env, J9Class
 }
 
 J9Class *
-MM_ClassLoaderManager::addDyingClassesToList(MM_EnvironmentBase *env, J9ClassLoader * classLoader, MM_HeapMap *markMap, bool setAll, J9Class *classUnloadListStart, UDATA *classUnloadCountResult)
+MM_ClassLoaderManager::addDyingClassesToList(MM_EnvironmentBase *env, J9ClassLoader *classLoader, MM_HeapMap *markMap, bool setAll, J9Class *classUnloadListStart, uintptr_t *classUnloadCountResult)
 {
 	J9VMThread *vmThread = (J9VMThread *)env->getLanguageVMThread();
 	J9Class *classUnloadList = classUnloadListStart;
-	UDATA classUnloadCount = 0;
+	uintptr_t classUnloadCount = 0;
 
 	if (NULL != classLoader) {
 		GC_ClassLoaderSegmentIterator segmentIterator(classLoader, MEMORY_TYPE_RAM_CLASS);
 		J9MemorySegment *segment = NULL;
-		while(NULL != (segment = segmentIterator.nextSegment())) {
+		while (NULL != (segment = segmentIterator.nextSegment())) {
 			GC_ClassHeapIterator classHeapIterator(_javaVM, segment);
 			J9Class *clazz = NULL;
-			while(NULL != (clazz = classHeapIterator.nextClass())) {
+			while (NULL != (clazz = classHeapIterator.nextClass())) {
 				J9Object *classObject = clazz->classObject;
 				if (setAll || !markMap->isBitSet(classObject)) {
 
@@ -407,11 +405,11 @@ MM_ClassLoaderManager::addDyingClassesToList(MM_EnvironmentBase *env, J9ClassLoa
 					 * field to J9_INVALID_OBJECT to investigate the origin of a class object
 					 * reference whose class has been unloaded.
 					 */
-					clazz->classObject = (j9object_t) J9_INVALID_OBJECT;
+					clazz->classObject = (j9object_t)J9_INVALID_OBJECT;
 
 					/* Call class unload hook */
 					Trc_MM_cleanUpClassLoadersStart_triggerClassUnload(env->getLanguageVMThread(),clazz,
-								(UDATA) J9UTF8_LENGTH(J9ROMCLASS_CLASSNAME(clazz->romClass)),
+								(uintptr_t)J9UTF8_LENGTH(J9ROMCLASS_CLASSNAME(clazz->romClass)),
 								J9UTF8_DATA(J9ROMCLASS_CLASSNAME(clazz->romClass)));
 					TRIGGER_J9HOOK_VM_CLASS_UNLOAD(_javaVM->hookInterface, vmThread, clazz);
 
@@ -428,7 +426,7 @@ MM_ClassLoaderManager::addDyingClassesToList(MM_EnvironmentBase *env, J9ClassLoa
 }
 
 void
-MM_ClassLoaderManager::cleanUpClassLoadersEnd(MM_EnvironmentBase *env, J9ClassLoader* unloadLink) 
+MM_ClassLoaderManager::cleanUpClassLoadersEnd(MM_EnvironmentBase *env, J9ClassLoader *unloadLink)
 {
 	J9VMThread *vmThread = (J9VMThread *)env->getLanguageVMThread();
 	J9MemorySegment *reclaimedSegments = NULL;
@@ -458,14 +456,14 @@ MM_ClassLoaderManager::cleanUpSegmentsAlongClassLoaderLink(J9JavaVM *javaVM, J9M
 {
 	while (NULL != segment) {
 		J9MemorySegment *nextSegment = segment->nextSegmentInClassLoader;
-		if (segment->type & MEMORY_TYPE_RAM_CLASS) {
+		if (MEMORY_TYPE_RAM_CLASS == (segment->type & MEMORY_TYPE_RAM_CLASS)) {
 			segment->type |= MEMORY_TYPE_UNDEAD_CLASS;
 			/* we also need to unset the fact that this is a RAM CLASS since some code which walks the segment list is looking for still-valid ones */
 			segment->type &= ~MEMORY_TYPE_RAM_CLASS;
 			segment->nextSegmentInClassLoader = *reclaimedSegments;
 			*reclaimedSegments = segment;
 			segment->classLoader = NULL;
-		} else if (!(segment->type & MEMORY_TYPE_UNDEAD_CLASS)) {
+		} else if (0 == (segment->type & MEMORY_TYPE_UNDEAD_CLASS)) {
 			javaVM->internalVMFunctions->freeMemorySegment(javaVM, segment, 1);
 		}
 		segment = nextSegment;
@@ -476,6 +474,13 @@ void
 MM_ClassLoaderManager::cleanUpSegmentsInAnonymousClassLoader(MM_EnvironmentBase *env, J9MemorySegment **reclaimedSegments)
 {
 	if (NULL != _javaVM->anonClassLoader) {
+
+		uintptr_t totalRAMClasses = 0;
+		uintptr_t deadRAMClasses = 0;
+		uintptr_t unloadableROMClasses = 0;
+		uintptr_t foundROMClassesFast = 0;
+		uintptr_t foundROMClassesSlow = 0;
+
 		J9MemorySegment **previousSegmentPointer = &_javaVM->anonClassLoader->classSegments;
 		J9MemorySegment *segment = *previousSegmentPointer;
 
@@ -489,23 +494,48 @@ MM_ClassLoaderManager::cleanUpSegmentsInAnonymousClassLoader(MM_EnvironmentBase
 				/* Anonymous classes expected to be allocated one per segment */
 				Assert_MM_true(NULL == classHeapIterator.nextClass());
 
-				if (J9AccClassDying == (J9CLASS_FLAGS(clazz) & J9AccClassDying)) {
-					/* TODO replace this to better algorithm */
-					/* Try to find ROM class for unloading anonymous RAM class if it is not an array */
-					if (!_extensions->objectModel.isIndexable(clazz)) {
-						J9ROMClass *romClass = clazz->romClass;
-						if (NULL != romClass) {
-							J9MemorySegment **previousSegmentPointerROM = &_javaVM->anonClassLoader->classSegments;
-							J9MemorySegment *segmentROM = *previousSegmentPointerROM;
+				totalRAMClasses += 1;
 
+				if (J9AccClassDying == (J9CLASS_FLAGS(clazz) & J9AccClassDying)) {
+
+					deadRAMClasses += 1;
+
+					/* Try to find ROM class for unloading anonymous RAM class if it is possible to unload */
+					if (isROMClassUnloadable(clazz)) {
+						J9ROMClass *romClass = clazz->romClass;
+
+						unloadableROMClasses += 1;
+
+						/* Check next segment, there is a high probability it is what we are looking for */
+						if ((NULL != nextSegment)
+								&& (MEMORY_TYPE_ROM_CLASS == (nextSegment->type & MEMORY_TYPE_ROM_CLASS))
+								&& ((J9ROMClass *)nextSegment->heapBase == romClass)) {
+
+							foundROMClassesFast += 1;
+
+							/* Yes, next segment is the ROM segment to be freed */
+							J9MemorySegment *segmentToFree = nextSegment;
+
+							/* correct pre-cached next for RAM iteration cycle */
+							nextSegment = segmentToFree->nextSegmentInClassLoader;
+
+							/* Free memory segment */
+							_javaVM->internalVMFunctions->freeMemorySegment(_javaVM, segmentToFree, 1);
+						} else {
 							/*
 							 *  Walk all anonymous classloader's ROM memory segments
 							 *  If ROM class is allocated there it would be one per segment
 							 */
+							J9MemorySegment **previousSegmentPointerROM = &_javaVM->anonClassLoader->classSegments;
+							J9MemorySegment *segmentROM = *previousSegmentPointerROM;
+
 							while (NULL != segmentROM) {
 								J9MemorySegment *nextSegmentROM = segmentROM->nextSegmentInClassLoader;
 								if ((MEMORY_TYPE_ROM_CLASS == (segmentROM->type & MEMORY_TYPE_ROM_CLASS)) && ((J9ROMClass *)segmentROM->heapBase == romClass)) {
+
 									/* found! */
+									foundROMClassesSlow += 1;
+
 									/* remove memory segment from list */
 									*previousSegmentPointerROM = nextSegmentROM;
 									/* correct pre-cached next for RAM iteration cycle if necessary */
@@ -532,24 +562,30 @@ MM_ClassLoaderManager::cleanUpSegmentsInAnonymousClassLoader(MM_EnvironmentBase
 					segment->nextSegmentInClassLoader = *reclaimedSegments;
 					*reclaimedSegments = segment;
 					segment->classLoader = NULL;
-					/* remove RAM memory segment from classloader segments list */
+					/* remove RAM memory segment and possibly ROM (if it is the next one) from classloader segments list */
 					*previousSegmentPointer = nextSegment;
 					removed = true;
 				}
 			}
+
 			if (!removed) {
 				previousSegmentPointer = &segment->nextSegmentInClassLoader;
 			}
 			segment = nextSegment;
 		}
+
+		uintptr_t notFoundROMClasses = unloadableROMClasses - foundROMClassesFast - foundROMClassesSlow;
+		Trc_MM_cleanUpSegmentsInAnonymousClassLoader_stats(
+				env->getLanguageVMThread(), totalRAMClasses, deadRAMClasses, unloadableROMClasses,
+				foundROMClassesFast, foundROMClassesSlow, notFoundROMClasses);
 	}
 }
 
 void
 MM_ClassLoaderManager::removeFromSubclassHierarchy(MM_EnvironmentBase *env, J9Class *clazzPtr)
 {
-	J9Class* nextLink = clazzPtr->subclassTraversalLink;
-	J9Class* reverseLink = clazzPtr->subclassTraversalReverseLink;
+	J9Class *nextLink = clazzPtr->subclassTraversalLink;
+	J9Class *reverseLink = clazzPtr->subclassTraversalReverseLink;
 	
 	reverseLink->subclassTraversalLink = nextLink;
 	nextLink->subclassTraversalReverseLink = reverseLink;
@@ -560,7 +596,7 @@ MM_ClassLoaderManager::removeFromSubclassHierarchy(MM_EnvironmentBase *env, J9Cl
 }
 
 void
-MM_ClassLoaderManager::cleanUpClassLoaders(MM_EnvironmentBase *env, J9ClassLoader *classLoadersUnloadedList, J9MemorySegment** reclaimedSegments, J9ClassLoader ** unloadLink, volatile bool* finalizationRequired)
+MM_ClassLoaderManager::cleanUpClassLoaders(MM_EnvironmentBase *env, J9ClassLoader *classLoadersUnloadedList, MM_ClassUnloadStats *classUnloadStats, J9MemorySegment **reclaimedSegments, J9ClassLoader **unloadLink, volatile bool *finalizationRequired)
 {
 	*reclaimedSegments = NULL;
 	*unloadLink = NULL;
@@ -568,7 +604,9 @@ MM_ClassLoaderManager::cleanUpClassLoaders(MM_EnvironmentBase *env, J9ClassLoade
 	/*
 	 * Cleanup segments in anonymous classloader
 	 */
-	cleanUpSegmentsInAnonymousClassLoader(env, reclaimedSegments);
+	if (0 < classUnloadStats->_anonymousClassesUnloadedCount) {
+		cleanUpSegmentsInAnonymousClassLoader(env, reclaimedSegments);
+	}
 	
 	/* For each classLoader that is not already unloading, not scanned and not enqueued for finalization:
 	 * perform classLoader-specific clean up, if it died on the current collection cycle; and either enqueue it for
@@ -596,7 +634,7 @@ MM_ClassLoaderManager::cleanUpClassLoaders(MM_EnvironmentBase *env, J9ClassLoade
 		/* Determine if the classLoader needs to be enqueued for finalization (for shared library unloading),
 		 * otherwise add it to the list of classLoaders to be unloaded by cleanUpClassLoadersEnd.
 		 */
-		if(((NULL != classLoader->sharedLibraries)
+		if (((NULL != classLoader->sharedLibraries)
 		&& (0 != pool_numElements(classLoader->sharedLibraries)))
 		|| (_extensions->fvtest_forceFinalizeClassLoaders)) {
 			/* Enqueue the class loader for the finalizer */
@@ -633,17 +671,17 @@ bool
 MM_ClassLoaderManager::tryEnterClassUnloadMutex(MM_EnvironmentBase *env)
 {
 	bool result = true;
-	
-	/* now, perform any actions that the global collector needs to take before a collection starts */
+	if (!_javaVM->isClassUnloadMutexHeldForRedefinition) {
 #if defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR)
-	if (0 != omrthread_rwmutex_try_enter_write(_javaVM->classUnloadMutex))
-#else
-	if (0 != omrthread_monitor_try_enter(_javaVM->classUnloadMutex))
-#endif /* J9VM_JIT_CLASS_UNLOAD_RWMONITOR */
-	{
-		/* The JIT currently is in the monitor */
-		/* We can simply skip unloading classes for this GC */
-		result = false;
+		if (0 != omrthread_rwmutex_try_enter_write(_javaVM->classUnloadMutex))
+#else /* defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR) */
+		if (0 != omrthread_monitor_try_enter(_javaVM->classUnloadMutex))
+#endif /* defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR) */
+		{
+			/* The JIT currently is in the monitor */
+			/* We can simply skip unloading classes for this GC */
+			result = false;
+		}
 	}
 	return result;
 }
@@ -653,25 +691,25 @@ MM_ClassLoaderManager::enterClassUnloadMutex(MM_EnvironmentBase *env)
 {
 	PORT_ACCESS_FROM_ENVIRONMENT(env);
 	U_64 quiesceTime = J9CONST64(0);
-	
-	/* now, perform any actions that the global collector needs to take before a collection starts */
+	if (!_javaVM->isClassUnloadMutexHeldForRedefinition) {
 #if defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR)
-	if (0 != omrthread_rwmutex_try_enter_write(_javaVM->classUnloadMutex))
-#else
-	if (0 != omrthread_monitor_try_enter(_javaVM->classUnloadMutex))
-#endif /* J9VM_JIT_CLASS_UNLOAD_RWMONITOR */
-	{
-		/* The JIT currently is in the monitor */
-		/* We must interrupt the JIT compilation so the GC can unload classes */
-		U_64 startTime = j9time_hires_clock();
-		TRIGGER_J9HOOK_MM_INTERRUPT_COMPILATION(_extensions->hookInterface, (J9VMThread *)env->getLanguageVMThread());
+		if (0 != omrthread_rwmutex_try_enter_write(_javaVM->classUnloadMutex))
+#else /* defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR) */
+		if (0 != omrthread_monitor_try_enter(_javaVM->classUnloadMutex))
+#endif /* defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR) */
+		{
+			/* The JIT currently is in the monitor */
+			/* We must interrupt the JIT compilation so the GC can unload classes */
+			U_64 startTime = j9time_hires_clock();
+			TRIGGER_J9HOOK_MM_INTERRUPT_COMPILATION(_extensions->hookInterface, (J9VMThread *)env->getLanguageVMThread());
 #if defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR)
-		omrthread_rwmutex_enter_write(_javaVM->classUnloadMutex);
-#else
-		omrthread_monitor_enter(_javaVM->classUnloadMutex);
-#endif /* J9VM_JIT_CLASS_UNLOAD_RWMONITOR */
-		U_64 endTime = j9time_hires_clock();
-		quiesceTime = j9time_hires_delta(startTime, endTime, J9PORT_TIME_DELTA_IN_MICROSECONDS);
+			omrthread_rwmutex_enter_write(_javaVM->classUnloadMutex);
+#else /* defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR) */
+			omrthread_monitor_enter(_javaVM->classUnloadMutex);
+#endif /* defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR) */
+			U_64 endTime = j9time_hires_clock();
+			quiesceTime = j9time_hires_delta(startTime, endTime, J9PORT_TIME_DELTA_IN_MICROSECONDS);
+		}
 	}
 	return quiesceTime;
 }
@@ -680,11 +718,13 @@ void
 MM_ClassLoaderManager::exitClassUnloadMutex(MM_EnvironmentBase *env)
 {
 	/* If we allowed class unloading during this gc, we must release the classUnloadMutex */
+	if (!_javaVM->isClassUnloadMutexHeldForRedefinition) {
 #if defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR)
-	omrthread_rwmutex_exit_write(_javaVM->classUnloadMutex);
-#else
-	omrthread_monitor_exit(_javaVM->classUnloadMutex);
-#endif /* J9VM_JIT_CLASS_UNLOAD_RWMONITOR */
+		omrthread_rwmutex_exit_write(_javaVM->classUnloadMutex);
+#else /* defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR) */
+		omrthread_monitor_exit(_javaVM->classUnloadMutex);
+#endif /* defined(J9VM_JIT_CLASS_UNLOAD_RWMONITOR) */
+	}
 }
 
 #endif /* J9VM_GC_DYNAMIC_CLASS_UNLOADING */
@@ -696,9 +736,9 @@ MM_ClassLoaderManager::exitClassUnloadMutex(MM_EnvironmentBase *env)
  */
 extern "C" {
 	
-void classLoaderLoadHook(J9HookInterface** hook, UDATA eventNum, void* eventData, void* userData)
+void classLoaderLoadHook(J9HookInterface** hook, uintptr_t eventNum, void *eventData, void *userData)
 {
-	J9VMClassLoaderInitializedEvent* event = (J9VMClassLoaderInitializedEvent*)eventData;
+	J9VMClassLoaderInitializedEvent *event = (J9VMClassLoaderInitializedEvent *)eventData;
 	MM_ClassLoaderManager *manager = (MM_ClassLoaderManager *)userData;
 	/* TODO CRGTMP should we be setting the SCANNED bit on the classloader if we are in trace phase? */
 	manager->linkClassLoader(event->classLoader);
