@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "ArrayletObjectModel.hpp"
@@ -25,12 +25,13 @@
 #include "IndexableObjectAllocationModel.hpp"
 #include "Math.hpp"
 #include "MemorySpace.hpp"
-#if defined(J9VM_GC_ENABLE_DOUBLE_MAP)
+#if defined(J9VM_GC_SPARSE_HEAP_ALLOCATION) || defined(J9VM_GC_ENABLE_DOUBLE_MAP)
 #include "ArrayletLeafIterator.hpp"
 #include "HeapRegionManager.hpp"
 #include "HeapRegionDescriptorVLHGC.hpp"
 #include "Heap.hpp"
-#endif /* J9VM_GC_ENABLE_DOUBLE_MAP */
+#include "SparseVirtualMemory.hpp"
+#endif /* defined(J9VM_GC_SPARSE_HEAP_ALLOCATION) || defined(J9VM_GC_ENABLE_DOUBLE_MAP)*/
 
 /**
  * Allocation description and layout initialization. This is called before OMR allocates
@@ -44,7 +45,7 @@ MM_IndexableObjectAllocationModel::initializeAllocateDescription(MM_EnvironmentB
 		return false;
 	}
 
-	/* continue, with reservations */
+	/* Continue, with reservations */
 	setAllocatable(false);
 	MM_GCExtensions *extensions = MM_GCExtensions::getExtensions(env);
 	uintptr_t spineBytes = extensions->indexableObjectModel.getSpineSize(_class, _layout, _numberOfArraylets, _dataSize, _alignSpineDataSection);
@@ -58,7 +59,11 @@ MM_IndexableObjectAllocationModel::initializeAllocateDescription(MM_EnvironmentB
 #endif /* defined (J9VM_GC_MODRON_COMPACTION) || defined (J9VM_GC_GENERATIONAL) */
 	spineBytes = extensions->objectModel.adjustSizeInBytes(spineBytes);
 
-	/* determine size of layout overhead (additional to spine bytes) and finalize allocation description */
+	bool isVirtualLargeObjectHeapEnabled = extensions->indexableObjectModel.isVirtualLargeObjectHeapEnabled();
+
+	/* Determine size of layout overhead (additional to spine bytes) and finalize
+	   allocation description. It is used to track amount of array bytes not
+	   included to spine and set final total number of bytes allocated for array. */
 	uintptr_t layoutSizeInBytes = 0;
 	switch (_layout) {
 	case GC_ArrayletObjectModel::Illegal:
@@ -67,11 +72,22 @@ MM_IndexableObjectAllocationModel::initializeAllocateDescription(MM_EnvironmentB
 		break;
 
 	case GC_ArrayletObjectModel::InlineContiguous:
-		/* all good */
-		setAllocatable(true);
+		/* Check if we're dealing with a camouflaged discontiguous array - these arrays will require slow-path allocate */
+		if (isVirtualLargeObjectHeapEnabled && (!_isDataAdjacent)) {
+			if (isGCAllowed()) {
+				layoutSizeInBytes = _dataSize;
+				setAllocatable(true);
+			}
+		} else {
+			setAllocatable(true);
+		}
 		break;
 
 	case GC_ArrayletObjectModel::Discontiguous:
+		/* If sparse heap is enabled, the only way to get here is through zero sized arrays. */
+		if (isVirtualLargeObjectHeapEnabled) {
+			Assert_MM_true((0 == _dataSize) && (0 == _numberOfArraylets));
+		}
 		/* non-empty discontiguous arrays require slow-path allocate */
 		if (isGCAllowed() || (0 == _numberOfIndexedFields)) {
 			/* _numberOfArraylets discontiguous leaves, all contains leaf size bytes */
@@ -84,6 +100,9 @@ MM_IndexableObjectAllocationModel::initializeAllocateDescription(MM_EnvironmentB
 		break;
 
 	case GC_ArrayletObjectModel::Hybrid:
+		if (isVirtualLargeObjectHeapEnabled) {
+			Assert_MM_unreachable();
+		}
 		Assert_MM_true(0 < _numberOfArraylets);
 		/* hybrid arrays always require slow-path allocate */
 		if (isGCAllowed()) {
@@ -102,7 +121,11 @@ MM_IndexableObjectAllocationModel::initializeAllocateDescription(MM_EnvironmentB
 	}
 
 	if (isAllocatable()) {
-		/* set total request size and layout metadata to finalize the description */
+		/* Set total request size and layout metadata to finalize the description.
+		   This logic mimics out-of-spine behaviour for Discontiguous/Hybrid.
+		   allocDescription->getBytesRequested() is used across the code as
+		   total amount of memory required for this allocation.
+		   For example, it is used to report amount failed to be allocated in case of OOM. */
 		_allocateDescription.setBytesRequested(spineBytes + layoutSizeInBytes);
 		_allocateDescription.setNumArraylets(_numberOfArraylets);
 		_allocateDescription.setSpineBytes(spineBytes);
@@ -122,33 +145,57 @@ MM_IndexableObjectAllocationModel::initializeIndexableObject(MM_EnvironmentBase
 	GC_ArrayObjectModel *indexableObjectModel = &extensions->indexableObjectModel;
 	J9IndexableObject *spine = (J9IndexableObject*)initializeJavaObject(env, allocatedBytes);
 	_allocateDescription.setSpine(spine);
+	bool isVirtualLargeObjectHeapEnabled = indexableObjectModel->isVirtualLargeObjectHeapEnabled();
+
 	if (NULL != spine) {
-		/* Set the array size */
+		/* Set the array size. */
 		if (getAllocateDescription()->isChunkedArray()) {
+			/* !off-heap or 0-length arrays */
+			Assert_MM_true(!isVirtualLargeObjectHeapEnabled || (0 == _numberOfIndexedFields));
 			indexableObjectModel->setSizeInElementsForDiscontiguous(spine, _numberOfIndexedFields);
 #if defined(J9VM_ENV_DATA64)
-			indexableObjectModel->setDataAddrForDiscontiguous(spine, NULL);
-#endif /* J9VM_ENV_DATA64 */
+			if (((J9JavaVM *)env->getLanguageVM())->isIndexableDataAddrPresent) {
+				indexableObjectModel->setDataAddrForDiscontiguous(spine, NULL);
+			}
+#endif /* defined(J9VM_ENV_DATA64) */
 		} else {
 			indexableObjectModel->setSizeInElementsForContiguous(spine, _numberOfIndexedFields);
 #if defined(J9VM_ENV_DATA64)
-			indexableObjectModel->setDataAddrForContiguous(spine);
-#endif /* J9VM_ENV_DATA64 */
+			if (((J9JavaVM *)env->getLanguageVM())->isIndexableDataAddrPresent) {
+				if (_isDataAdjacent) {
+					indexableObjectModel->setDataAddrForContiguous(spine);
+				} else {
+					/* Set NULL temporarily to avoid possible complication with GC occurring while the object is partially initialized. */
+					indexableObjectModel->setDataAddrForContiguous(spine, NULL);
+				}
+			}
+#endif /* defined(J9VM_ENV_DATA64) */
 		}
 	}
 
-
 	/* Lay out arraylet and arrayoid pointers */
 	switch (_layout) {
 	case GC_ArrayletObjectModel::InlineContiguous:
-		Assert_MM_true(1 == _numberOfArraylets);
+#if defined(J9VM_GC_SPARSE_HEAP_ALLOCATION)
+		if (isVirtualLargeObjectHeapEnabled && !_isDataAdjacent) {
+			/* We still need to create leaves for discontiguous arrays that will be allocated at off-heap. */
+			spine = getSparseAddressAndDecommitLeaves(env, spine);
+			if (NULL != spine) {
+				Assert_MM_true(1 <= _numberOfArraylets);
+			}
+		}
+#endif /* defined (J9VM_GC_SPARSE_HEAP_ALLOCATION) */
+		if (!isVirtualLargeObjectHeapEnabled || _isDataAdjacent) {
+			Assert_MM_true(1 == _numberOfArraylets);
+		}
 		break;
 
 	case GC_ArrayletObjectModel::Discontiguous:
 	case GC_ArrayletObjectModel::Hybrid:
 		if (NULL != spine) {
+			/* The array size has been set earlier. */
 			if(0 == _numberOfIndexedFields) {
-				/* Don't try to initialize the arrayoid for an empty NUA */
+				/* Don't try to initialize the arrayoid for an empty NUA. */
 				Trc_MM_allocateAndConnectNonContiguousArraylet_Exit(env->getLanguageVMThread(), spine);
 				break;
 			}
@@ -167,7 +214,7 @@ MM_IndexableObjectAllocationModel::initializeIndexableObject(MM_EnvironmentBase
 	}
 
 	if (NULL != spine) {
-		/* Initialize hashcode slot */
+		/* Initialize hashcode slot. */
 		if (getAllocateDescription()->getPreHashFlag()) {
 			env->getExtensions()->objectModel.initializeHashSlot((J9JavaVM*)env->getLanguageVM(), (omrobjectptr_t)spine);
 		}
@@ -178,35 +225,6 @@ MM_IndexableObjectAllocationModel::initializeIndexableObject(MM_EnvironmentBase
 	return (omrobjectptr_t)spine;
 }
 
-/**
- * For contiguous arraylet all data is in the spine but arrayoid pointers must still be laid down.
- *
- * @return initialized arraylet spine with its arraylet pointers initialized.
- */
-MMINLINE J9IndexableObject *
-MM_IndexableObjectAllocationModel::layoutContiguousArraylet(MM_EnvironmentBase *env, J9IndexableObject *spine)
-{
-	Assert_MM_true(_numberOfArraylets == _allocateDescription.getNumArraylets());
-	MM_GCExtensions *extensions = MM_GCExtensions::getExtensions(env);
-	bool const compressed = env->compressObjectReferences();
-
-	/* set arraylet pointers in the spine. these all point into the data part of the spine */
-	fj9object_t *arrayoidPtr = extensions->indexableObjectModel.getArrayoidPointer(spine);
-	uintptr_t leafOffset = (uintptr_t)GC_SlotObject::addToSlotAddress(arrayoidPtr, _numberOfArraylets, compressed);
-	if (_alignSpineDataSection) {
-		leafOffset = MM_Math::roundToCeiling(sizeof(uint64_t), leafOffset);
-	}
-	uintptr_t arrayletLeafSize = env->getOmrVM()->_arrayletLeafSize;
-	for (uintptr_t i = 0; i < _numberOfArraylets; i++) {
-		GC_SlotObject slotObject(env->getOmrVM(), arrayoidPtr);
-		slotObject.writeReferenceToSlot((omrobjectptr_t)leafOffset);
-		leafOffset += arrayletLeafSize;
-		arrayoidPtr = GC_SlotObject::addToSlotAddress(arrayoidPtr, 1, compressed);
-	}
-
-	return spine;
-}
-
 /**
  * For discontiguous or hybrid arraylet spine is allocated first and leaves are sequentially
  * allocated and attached to the spine. The allocation description saves and restores the
@@ -226,14 +244,14 @@ MM_IndexableObjectAllocationModel::layoutDiscontiguousArraylet(MM_EnvironmentBas
 	GC_ArrayObjectModel *indexableObjectModel = &extensions->indexableObjectModel;
 	bool const compressed = env->compressObjectReferences();
 
-	/* determine how many bytes to allocate outside of the spine (in arraylet leaves) */
+	/* Determine how many bytes to allocate outside of the spine (in arraylet leaves). */
 	const uintptr_t arrayletLeafSize = env->getOmrVM()->_arrayletLeafSize;
 	Assert_MM_true(_allocateDescription.getBytesRequested() >= _allocateDescription.getContiguousBytes());
 	uintptr_t bytesRemaining = _allocateDescription.getBytesRequested() - _allocateDescription.getContiguousBytes();
 	Assert_MM_true((0 == (bytesRemaining % arrayletLeafSize)) || (GC_ArrayletObjectModel::Hybrid != _layout));
-	/* hybrid arraylets store _dataSize % arrayletLeafSize bytes in the spine, remainder in _numberOfArraylets-1 leaves */
+	/* Hybrid arraylets store _dataSize % arrayletLeafSize bytes in the spine, remainder in _numberOfArraylets-1 leaves. */
 
-	/* allocate leaf for each arraylet and attach it to its leaf pointer in the spine */
+	/* Allocate leaf for each arraylet and attach it to its leaf pointer in the spine. */
 	uintptr_t arrayoidIndex = 0;
 	fj9object_t *arrayoidPtr = indexableObjectModel->getArrayoidPointer(spine);
 	while (0 < bytesRemaining) {
@@ -241,20 +259,20 @@ MM_IndexableObjectAllocationModel::layoutDiscontiguousArraylet(MM_EnvironmentBas
 		void *leaf = env->_objectAllocationInterface->allocateArrayletLeaf(env, &_allocateDescription,
 				_allocateDescription.getMemorySpace(), true);
 
-		/* if leaf allocation failed set the result to NULL and return */
+		/* If leaf allocation failed set the result to NULL and return. */
 		if (NULL == leaf) {
-			/* spine and preceding arraylets are now floating garbage */
+			/* Spine and preceding arraylets are now floating garbage. */
 			Trc_MM_allocateAndConnectNonContiguousArraylet_leafFailure(env->getLanguageVMThread());
 			_allocateDescription.setSpine(NULL);
 			spine = NULL;
 			break;
 		}
 
-		/* refresh the spine -- it might move if we GC while allocating the leaf */
+		/* Refresh the spine -- it might move if we GC while allocating the leaf. */
 		spine = _allocateDescription.getSpine();
 		arrayoidPtr = indexableObjectModel->getArrayoidPointer(spine);
 
-		/* set the arrayoid pointer in the spine to point to the new leaf */
+		/* Set the arrayoid pointer in the spine to point to the new leaf. */
 		GC_SlotObject slotObject(env->getOmrVM(), GC_SlotObject::addToSlotAddress(arrayoidPtr, arrayoidIndex, compressed));
 		slotObject.writeReferenceToSlot((omrobjectptr_t)leaf);
 
@@ -262,11 +280,13 @@ MM_IndexableObjectAllocationModel::layoutDiscontiguousArraylet(MM_EnvironmentBas
 		arrayoidIndex += 1;
 	}
 
+	bool isVirtualLargeObjectHeapEnabled = indexableObjectModel->isVirtualLargeObjectHeapEnabled();
 	if (NULL != spine) {
 		switch (_layout) {
 		case GC_ArrayletObjectModel::Discontiguous:
 			indexableObjectModel->AssertArrayletIsDiscontiguous(spine);
 			Assert_MM_true(arrayoidIndex == _numberOfArraylets);
+			Assert_MM_false(isVirtualLargeObjectHeapEnabled);
 #if defined(J9VM_GC_ENABLE_DOUBLE_MAP)
 			if (indexableObjectModel->isDoubleMappingEnabled()) {
 				/**
@@ -281,22 +301,23 @@ MM_IndexableObjectAllocationModel::layoutDiscontiguousArraylet(MM_EnvironmentBas
 			break;
 
 		case GC_ArrayletObjectModel::Hybrid:
+			/* Unreachable if off-heap is enabled. */
+			Assert_MM_false(isVirtualLargeObjectHeapEnabled);
 #if defined(J9VM_GC_ENABLE_DOUBLE_MAP)
 			/* Unreachable if double map is enabled */
 			if (indexableObjectModel->isDoubleMappingEnabled()) {
 				Assert_MM_double_map_unreachable();
 			}
 #endif /* J9VM_GC_ENABLE_DOUBLE_MAP */
-			/* last arrayoid points to end of arrayoid array in spine header (object-aligned if
-			 * required). (data size % leaf size) bytes of data are stored here (may be empty).
-			 */
+			/* Last arrayoid points to end of arrayoid array in spine header (object-aligned if
+			   required). (data size % leaf size) bytes of data are stored here (may be empty). */
 			Assert_MM_true(arrayoidIndex == (_numberOfArraylets - 1));
 			{
 				uintptr_t leafOffset = (uintptr_t)GC_SlotObject::addToSlotAddress(arrayoidPtr, _numberOfArraylets, compressed);
 				if (_alignSpineDataSection) {
 					leafOffset = MM_Math::roundToCeiling(env->getObjectAlignmentInBytes(), leafOffset);
 				}
-				/* set the last arrayoid pointer to point to remainder data */
+				/* Set the last arrayoid pointer to point to remainder data. */
 				GC_SlotObject slotObject(env->getOmrVM(), GC_SlotObject::addToSlotAddress(arrayoidPtr, arrayoidIndex, compressed));
 				slotObject.writeReferenceToSlot((omrobjectptr_t)leafOffset);
 			}
@@ -311,12 +332,88 @@ MM_IndexableObjectAllocationModel::layoutDiscontiguousArraylet(MM_EnvironmentBas
 	return spine;
 }
 
+#if defined(J9VM_GC_SPARSE_HEAP_ALLOCATION)
+MMINLINE J9IndexableObject *
+MM_IndexableObjectAllocationModel::getSparseAddressAndDecommitLeaves(MM_EnvironmentBase *env, J9IndexableObject *spine)
+{
+	Assert_MM_true(_numberOfArraylets == _allocateDescription.getNumArraylets());
+
+	MM_GCExtensions *extensions = MM_GCExtensions::getExtensions(env);
+	GC_ArrayObjectModel *indexableObjectModel = &extensions->indexableObjectModel;
+	const UDATA arrayletLeafSize = env->getOmrVM()->_arrayletLeafSize;
+	uintptr_t byteAmount = 0;
+	UDATA arrayletLeafCount = MM_Math::roundToCeiling(arrayletLeafSize, _dataSize) / arrayletLeafSize;
+
+	/* Determine how many bytes to allocate outside of the spine (in arraylet leaves). */
+	Assert_MM_true(_allocateDescription.getBytesRequested() >= _allocateDescription.getContiguousBytes());
+	uintptr_t bytesRemaining = _allocateDescription.getBytesRequested() - _allocateDescription.getContiguousBytes();
+
+	/* Allocate leaf for each arraylet and attach it to its leaf pointer in the spine. */
+	uintptr_t arrayoidIndex = 0;
+	Trc_MM_getSparseAddressAndDecommitLeaves_Entry(env->getLanguageVMThread(), spine, (void *)bytesRemaining, arrayletLeafCount, (void *)arrayletLeafSize);
+	while (0 < bytesRemaining) {
+		/* Allocate the next arraylet leaf - leaves are allocated solely for the purpose of
+		   decommitting the memory later on in this function. */
+		void *leaf = env->_objectAllocationInterface->allocateArrayletLeaf(
+				env, &_allocateDescription, _allocateDescription.getMemorySpace(), true);
+
+		/* If leaf allocation failed set the result to NULL and return. */
+		if (NULL == leaf) {
+			/* Spine and preceding arraylets are now floating garbage. */
+			Trc_MM_allocateAndConnectNonContiguousArraylet_leafFailure(env->getLanguageVMThread());
+			_allocateDescription.setSpine(NULL);
+			spine = NULL;
+			break;
+		}
+
+		if (0 == arrayoidIndex) {
+			MM_HeapRegionDescriptorVLHGC *firstLeafRegionDescriptor = (MM_HeapRegionDescriptorVLHGC *)extensions->getHeap()->getHeapRegionManager()->tableDescriptorForAddress(leaf);
+			firstLeafRegionDescriptor->_sparseHeapAllocation = true;
+		}
+
+		/* Disable region for reads and writes, since that'll be done through the contiguous double mapped region */
+		void *highAddress = (void *)((uintptr_t)leaf + arrayletLeafSize);
+		bool ret = extensions->heap->decommitMemory(leaf, arrayletLeafSize, leaf, highAddress);
+		if (!ret) {
+			Trc_MM_VirtualMemory_decommitMemory_failure(leaf, arrayletLeafSize);
+		}
+
+		/* Refresh the spine -- it might move if we GC while allocating the leaf */
+		spine = _allocateDescription.getSpine();
+
+		bytesRemaining -= OMR_MIN(bytesRemaining, arrayletLeafSize);
+		arrayoidIndex += 1;
+	}
+
+
+	if (NULL != spine) {
+		Assert_MM_true(_layout == GC_ArrayletObjectModel::InlineContiguous);
+		Assert_MM_true(indexableObjectModel->isVirtualLargeObjectHeapEnabled());
+		/* Number of arraylet leaves in the iterator must match the number of leaves calculated */
+		Assert_MM_true(arrayletLeafCount == arrayoidIndex);
+
+		byteAmount = _dataSize;
+		void *virtualLargeObjectHeapAddress = extensions->largeObjectVirtualMemory->allocateSparseFreeEntryAndMapToHeapObject(spine, byteAmount);
+		if (NULL != virtualLargeObjectHeapAddress) {
+			indexableObjectModel->setDataAddrForContiguous((J9IndexableObject *)spine, virtualLargeObjectHeapAddress);
+		} else {
+			_allocateDescription.setSpine(NULL);
+			spine = NULL;
+		}
+	}
+
+	Trc_MM_getSparseAddressAndDecommitLeaves_Exit(env->getLanguageVMThread(), spine, (void *)bytesRemaining);
+
+	return spine;
+}
+#endif /* defined(J9VM_GC_SPARSE_HEAP_ALLOCATION) */
+
 #if defined(J9VM_GC_ENABLE_DOUBLE_MAP)
 #if !((defined(LINUX) || defined(OSX)) && defined(J9VM_ENV_DATA64))
 /* Double map is only supported on LINUX 64 bit Systems for now */
 #error "Platform not supported by Double Map API"
 #endif /* !((defined(LINUX) || defined(OSX)) && defined(J9VM_ENV_DATA64)) */
-void * 
+void *
 MM_IndexableObjectAllocationModel::doubleMapArraylets(MM_EnvironmentBase *env, J9Object *objectPtr, void *preferredAddress)
 {
 	MM_GCExtensions *extensions = MM_GCExtensions::getExtensions(env);
@@ -390,4 +487,3 @@ MM_IndexableObjectAllocationModel::doubleMapArraylets(MM_EnvironmentBase *env, J
 	return result;
 }
 #endif /* J9VM_GC_ENABLE_DOUBLE_MAP */
-
