@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2001, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2001
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,21 +15,20 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
+#include "StringTable.hpp"
+
 #include "hashtable_api.h"
 #include "j2sever.h"
 #include "j9consts.h"
-#include "j9protos.h"
 #include "objhelp.h"
-#include "ModronAssertions.h"
 
 #include "EnvironmentBase.hpp"
 #include "GCExtensions.hpp"
-#include "StringTable.hpp"
 #include "VMHelpers.hpp"
 
 /* the following is all zeros except the least significant bit */
@@ -556,6 +555,7 @@ j9gc_createJavaLangString(J9VMThread *vmThread, U_8 *data, UDATA length, UDATA s
 	bool anonClassName = J9_ARE_ANY_BITS_SET(stringFlags, J9_STR_ANON_CLASS_NAME);
 	bool internString = J9_ARE_ANY_BITS_SET(stringFlags, J9_STR_INTERN);
 	UDATA unicodeLength = 0;
+	UDATA zerosFound = 0;
 
 	Trc_MM_createJavaLangString_Entry(vmThread, length, data, stringFlags);
 
@@ -575,7 +575,7 @@ j9gc_createJavaLangString(J9VMThread *vmThread, U_8 *data, UDATA length, UDATA s
 			for (UDATA i = 0; i < unicodeLength; ++i) {
 				if (unicodeData[i] > 0x7F) {
 					isASCII = false;
-					if (J2SE_VERSION(vm) >= J2SE_V17) {
+					if (J2SE_VERSION(vm) >= J2SE_V11) {
 						for (UDATA j = i; j < unicodeLength; ++j) {
 							if (unicodeData[j] > 0xFF) {
 								isASCIIorLatin1 = false;
@@ -596,8 +596,14 @@ j9gc_createJavaLangString(J9VMThread *vmThread, U_8 *data, UDATA length, UDATA s
 	} else {
 		for (UDATA i = 0; i < length; ++i) {
 			if (data[i] > 0x7F) {
+				/* Check for 0 in modified UTF8. */
+				if ((0xC0 == data[i]) && ((i + 1) < length) && (0x80 == data[i + 1])) {
+					zerosFound += 1;
+					i += 1;
+					continue;
+				}
 				isASCII = false;
-				if (compressStrings && (J2SE_VERSION(vm) >= J2SE_V17)) {
+				if (compressStrings && (J2SE_VERSION(vm) >= J2SE_V11)) {
 					U_8 *dataTmp = data + i;
 					UDATA lengthTmp = length - i;
 					isASCIIorLatin1 = VM_VMHelpers::isLatin1String(dataTmp, lengthTmp);
@@ -623,7 +629,13 @@ j9gc_createJavaLangString(J9VMThread *vmThread, U_8 *data, UDATA length, UDATA s
 
 		if (isASCII) {
 			for (UDATA i = 0; i < length; ++i) {
-				hash = (hash << 5) - hash + data[i];
+				U_8 c = data[i];
+				/* Look for the start of the modified UTF8 sequence for 0, which is validated when isASCII is set. */
+				if (0xC0 == c) {
+					c = 0;
+					i += 1;
+				}
+				hash = (hash << 5) - hash + c;
 			}
 		} else {
 			hash = VM_VMHelpers::computeHashForUTF8(data, length);
@@ -653,7 +665,7 @@ j9gc_createJavaLangString(J9VMThread *vmThread, U_8 *data, UDATA length, UDATA s
 
 		if (!isUnicode) {
 			if (isASCII) {
-				unicodeLength = length;
+				unicodeLength = length - zerosFound;
 			} else {
 				UDATA tempLength = length;
 				U_8 *tempData = data;
@@ -707,21 +719,35 @@ j9gc_createJavaLangString(J9VMThread *vmThread, U_8 *data, UDATA length, UDATA s
 				UDATA lastSlash = 0;
 				if (translateSlashes) {
 					if (isASCII) {
+						UDATA storeIndex = 0;
 						for (UDATA i = 0; i < length; ++i) {
 							U_8 c = data[i];
 							if ('/' == c) {
 								lastSlash = i;
 								c = '.';
+							/* Look for the start of the modified UTF8 sequence for 0, which is validated when isASCII is set. */
+							} else if (0xC0 == c) {
+								c = 0;
+								i += 1;
 							}
-							J9JAVAARRAYOFBYTE_STORE(vmThread, charArray, i, c);
+							J9JAVAARRAYOFBYTE_STORE(vmThread, charArray, storeIndex, c);
+							storeIndex += 1;
 						}
 					} else {
 						lastSlash = storeLatin1ByteArrayhelper(vmThread, data, length, charArray, true);
 					}
 				} else {
 					if (isASCII) {
+						UDATA storeIndex = 0;
 						for (UDATA i = 0; i < length; ++i) {
-							J9JAVAARRAYOFBYTE_STORE(vmThread, charArray, i, data[i]);
+							U_8 c = data[i];
+							/* Look for the start of the modified UTF8 sequence for 0, which is validated when isASCII is set. */
+							if (0xC0 == c) {
+								c = 0;
+								i += 1;
+							}
+							J9JAVAARRAYOFBYTE_STORE(vmThread, charArray, storeIndex, c);
+							storeIndex += 1;
 						}
 					} else {
 						lastSlash = storeLatin1ByteArrayhelper(vmThread, data, length, charArray, false);
@@ -738,17 +764,31 @@ j9gc_createJavaLangString(J9VMThread *vmThread, U_8 *data, UDATA length, UDATA s
 				UDATA lastSlash = 0;
 				if (isASCII) {
 					if (translateSlashes) {
+						IDATA storeIndex = 0;
 						for (UDATA i = 0; i < length; ++i) {
 							U_8 c = data[i];
 							if ('/' == c) {
 								lastSlash = i;
 								c = '.';
+							/* Look for the start of the modified UTF8 sequence for 0, which is validated when isASCII is set. */
+							} else if (0xC0 == c) {
+								c = 0;
+								i += 1;
 							}
-							J9JAVAARRAYOFCHAR_STORE(vmThread, charArray, i, c);
+							J9JAVAARRAYOFCHAR_STORE(vmThread, charArray, storeIndex, c);
+							storeIndex += 1;
 						}
 					} else {
+						UDATA storeIndex = 0;
 						for (UDATA i = 0; i < length; ++i) {
-							J9JAVAARRAYOFCHAR_STORE(vmThread, charArray, i, data[i]);
+							U_8 c = data[i];
+							/* Look for the start of the modified UTF8 sequence for 0, which is validated when isASCII is set. */
+							if (0xC0 == c) {
+								c = 0;
+								i += 1;
+							}
+							J9JAVAARRAYOFCHAR_STORE(vmThread, charArray, storeIndex, c);
+							storeIndex += 1;
 						}
 					}
 				} else {
@@ -801,12 +841,16 @@ j9gc_createJavaLangString(J9VMThread *vmThread, U_8 *data, UDATA length, UDATA s
 					 * jitHookClassPreinitialize will process initialization events for String compression sideEffectGuards
 					 * so we must initialize the class if this is the first time we are loading it
 					 */
+					PUSH_OBJECT_IN_SPECIAL_FRAME(vmThread, result);
 					J9Class* flagClass = vmFuncs->internalFindKnownClass(vmThread, J9VMCONSTANTPOOL_JAVALANGSTRINGSTRINGCOMPRESSIONFLAG, J9_FINDKNOWNCLASS_FLAG_INITIALIZE);
+					result = POP_OBJECT_IN_SPECIAL_FRAME(vmThread);
 
 					if (NULL == flagClass) {
 						goto nomem;
 					} else {
+						PUSH_OBJECT_IN_SPECIAL_FRAME(vmThread, result);
 						j9object_t flag = J9AllocateObject(vmThread, flagClass, allocateFlags);
+						result = POP_OBJECT_IN_SPECIAL_FRAME(vmThread);
 
 						if (NULL == flag) {
 							goto nomem;
