@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "ConcurrentMarkingDelegate.hpp"
@@ -31,36 +31,11 @@
 #include "HeapRegionIteratorStandard.hpp"
 #include "ReferenceObjectList.hpp"
 #include "StackSlotValidator.hpp"
+#include "VMAccess.hpp"
 #include "VMInterface.hpp"
 #include "VMThreadListIterator.hpp"
 
-/**
- * Concurrents stack slot iterator.
- * Called for each slot in a threads active stack frames which contains a object reference.
- *
- * @param objectIndirect
- * @param localdata
- * @param isDerivedPointer
- * @param objectIndirectBase
- */
-void
-concurrentStackSlotIterator(J9JavaVM *javaVM, omrobjectptr_t *objectIndirect, void *localData, J9StackWalkState *walkState, const void *stackLocation)
-{
-	MM_ConcurrentMarkingDelegate::markSchemeStackIteratorData *data = (MM_ConcurrentMarkingDelegate::markSchemeStackIteratorData *)localData;
-
-	omrobjectptr_t object = *objectIndirect;
-	if (data->env->getExtensions()->heap->objectIsInGap(object)) {
-		/* CMVC 136483:  Ensure that the object is not in the gap of a split heap (stack-allocated object) since we can't mark that part of the address space */
-		Assert_MM_validStackSlot(MM_StackSlotValidator(MM_StackSlotValidator::NOT_ON_HEAP, object, stackLocation, walkState).validate(data->env));
-	} else if (data->markingScheme->isHeapObject(object)) {
-		/* heap object - validate and mark */
-		Assert_MM_validStackSlot(MM_StackSlotValidator(0, object, stackLocation, walkState).validate(data->env));
-		data->markingScheme->markObject(data->env, object);
-	} else if (NULL != object) {
-		/* stack object - just validate */
-		Assert_MM_validStackSlot(MM_StackSlotValidator(MM_StackSlotValidator::NOT_ON_HEAP, object, stackLocation, walkState).validate(data->env));
-	}
-}
+#include "MarkingDelegate.hpp"
 
 bool
 MM_ConcurrentMarkingDelegate::initialize(MM_EnvironmentBase *env, MM_ConcurrentGC *collector)
@@ -69,6 +44,7 @@ MM_ConcurrentMarkingDelegate::initialize(MM_EnvironmentBase *env, MM_ConcurrentG
 	_javaVM = (J9JavaVM *)extensions->getOmrVM()->_language_vm;
 	_objectModel = &(extensions->objectModel);
 	_markingScheme = collector->getMarkingScheme();
+	_markingDelegate = _markingScheme->getMarkingDelegate();
 	_collector = collector;
 	return true;
 }
@@ -148,10 +124,15 @@ MM_ConcurrentMarkingDelegate::scanThreadRoots(MM_EnvironmentBase *env)
 		}
 	}
 
-	markSchemeStackIteratorData localData;
-	localData.markingScheme = _markingScheme;
-	localData.env = env;
-	GC_VMThreadStackSlotIterator::scanSlots(vmThread, vmThread, (void *)&localData, concurrentStackSlotIterator, true, false);
+	MM_MarkingDelegate::StackIteratorData localData(env, _markingDelegate);
+	const bool stackFrameClassWalkNeeded = true;
+	GC_VMThreadStackSlotIterator::scanSlots(vmThread, vmThread, (void *)&localData, (J9MODRON_OSLOTITERATOR *)MM_MarkingDelegate::stackSlotIterator, stackFrameClassWalkNeeded, false);
+
+#if JAVA_SPEC_VERSION >= 19
+	if (NULL != vmThread->currentContinuation) {
+		_markingDelegate->scanContinuationNativeSlotsNoSync(env, vmThread, vmThread->currentContinuation, stackFrameClassWalkNeeded);
+	}
+#endif /* JAVA_SPEC_VERSION >= 19 */
 
 	return true;
 }
@@ -416,10 +397,8 @@ MM_ConcurrentMarkingDelegate::concurrentClassMark(MM_EnvironmentBase *env, bool
 					J9Module **modulePtr = (J9Module**)hashTableStartDo(classLoader->moduleHashTable, &moduleWalkState);
 					while (NULL != modulePtr) {
 						J9Module * const module = *modulePtr;
-
-						_markingScheme->markObject(env, (j9object_t)module->moduleObject);
-						if (NULL != module->moduleName) {
-							_markingScheme->markObject(env, (j9object_t)module->moduleName);
+						if (NULL != module->moduleObject) {
+							_markingScheme->markObject(env, (j9object_t)module->moduleObject);
 						}
 						if (NULL != module->version) {
 							_markingScheme->markObject(env, (j9object_t)module->version);
@@ -431,7 +410,9 @@ MM_ConcurrentMarkingDelegate::concurrentClassMark(MM_EnvironmentBase *env, bool
 					}
 
 					if (classLoader == _javaVM->systemClassLoader) {
-						_markingScheme->markObject(env, _javaVM->unamedModuleForSystemLoader->moduleObject);
+						if (NULL != _javaVM->unnamedModuleForSystemLoader->moduleObject) {
+							_markingScheme->markObject(env, _javaVM->unnamedModuleForSystemLoader->moduleObject);
+						}
 					}
 				}
 
@@ -450,4 +431,19 @@ quitConcurrentClassMark:
 }
 #endif /* J9VM_GC_DYNAMIC_CLASS_UNLOADING */
 
+void
+MM_ConcurrentMarkingDelegate::acquireExclusiveVMAccessAndSignalThreadsToActivateWriteBarrier(MM_EnvironmentBase *env)
+{
+	J9VMThread *vmThread = (J9VMThread *)env->getLanguageVMThread();
+
+	VM_VMAccess::setPublicFlags(vmThread, J9_PUBLIC_FLAGS_NOT_AT_SAFE_POINT);
+	_collector->acquireExclusiveVMAccessAndSignalThreadsToActivateWriteBarrier(env);
+	VM_VMAccess::clearPublicFlags(vmThread, J9_PUBLIC_FLAGS_NOT_AT_SAFE_POINT);
+
+	if (J9_ARE_ANY_BITS_SET(vmThread->publicFlags, J9_PUBLIC_FLAGS_HALT_THREAD_ANY) && (0 == vmThread->omrVMThread->exclusiveCount)) {
+		vmThread->javaVM->internalVMFunctions->internalReleaseVMAccess(vmThread);
+		vmThread->javaVM->internalVMFunctions->internalAcquireVMAccess(vmThread);
+	}
+}
+
 #endif /* defined(OMR_GC_MODRON_CONCURRENT_MARK) */
