@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2020 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "omr.h"
@@ -105,47 +105,6 @@ MM_RealtimeGC::initialize(MM_EnvironmentBase *env)
 	_extensions->realtimeGC = this;
 	_allowGrowth = false;
 	
-	if (_extensions->gcTrigger == 0) {
-		_extensions->gcTrigger = (_extensions->memoryMax / 2);
-		_extensions->gcInitialTrigger = (_extensions->memoryMax / 2);
-	}
-
-	_extensions->distanceToYieldTimeCheck = 0;
-
-	/* Only SRT passes this check as the commandline option to specify beatMicro is only enabled on SRT */
-	if (METRONOME_DEFAULT_BEAT_MICRO != _extensions->beatMicro) {
-		/* User-specified quanta time, adjust related parameters */
-		_extensions->timeWindowMicro = 20 * _extensions->beatMicro;
-		/* Currently all supported SRT platforms - AIX and Linux, can only use HRT for alarm thread implementation.
-		 * The default value for HRT period is 1/3 of the default quanta: 1 msec for HRT period and 3 msec quanta,
-		 * we will attempt to adjust the HRT period to 1/3 of the specified quanta.
-		 */
-		uintptr_t hrtPeriodMicro = _extensions->beatMicro / 3;
-		if ((hrtPeriodMicro < METRONOME_DEFAULT_HRT_PERIOD_MICRO) && (METRONOME_DEFAULT_HRT_PERIOD_MICRO < _extensions->beatMicro)) {
-			/* If the adjusted value is too small for the hires clock resolution, we will use the default HRT period provided that
-			 * the default period is smaller than the quanta time specified.
-			 * Otherwise we fail to initialize the alarm thread with an error message.
-			 */
-			hrtPeriodMicro = METRONOME_DEFAULT_HRT_PERIOD_MICRO;
-		}
-		Assert_MM_true(0 != hrtPeriodMicro);
-		_extensions->hrtPeriodMicro = hrtPeriodMicro;
-
-		/* On Windows SRT we still use interrupt-based alarm. Set the interrupt period the same as hires timer period.
-		 * We will fail to init the alarm if this is too small a resolution for Windows.
-		 */
-		_extensions->itPeriodMicro = _extensions->hrtPeriodMicro;
-
-		/* if the pause time user specified is larger than the default value, calculate if there is opportunity
-		 * for the GC to do time checking less often inside condYieldFromGC.
-		 */
-		if (METRONOME_DEFAULT_BEAT_MICRO < _extensions->beatMicro) {
-			uintptr_t intervalToSkipYieldCheckMicro = _extensions->beatMicro - METRONOME_DEFAULT_BEAT_MICRO;
-			uintptr_t maxInterYieldTimeMicro = INTER_YIELD_MAX_NS / 1000;
-			_extensions->distanceToYieldTimeCheck = (U_32)(intervalToSkipYieldCheckMicro / maxInterYieldTimeMicro);
-		}
-	}
-
 	_osInterface = MM_OSInterface::newInstance(env);
 	if (_osInterface == NULL) {
 		return false;
@@ -402,6 +361,7 @@ MM_RealtimeGC::internalPreCollect(MM_EnvironmentBase *env, MM_MemorySubSpace *su
 	env->_cycleState->_gcCode = MM_GCCode(gcCode);
 	env->_cycleState->_type = _cycleType;
 	env->_cycleState->_activeSubSpace = subSpace;
+	env->_cycleState->_collectionStatistics = &_collectionStatistics;
 
 	/* If we are in an excessiveGC level beyond normal then an aggressive GC is
 	 * conducted to free up as much space as possible
@@ -428,6 +388,7 @@ MM_RealtimeGC::internalPreCollect(MM_EnvironmentBase *env, MM_MemorySubSpace *su
 	}
 	/* we are about to collect so generate the appropriate cycle start and increment start events */
 	reportGCCycleStart(rtEnv);
+	reportGCIncrementStart(rtEnv);
 	_sched->reportStartGCIncrement(rtEnv);
 }
 
@@ -488,6 +449,7 @@ MM_RealtimeGC::internalPostCollect(MM_EnvironmentBase *env, MM_MemorySubSpace *s
 	 * the METRONOME_INCREMENT_START/END events become out of order and verbose GC will fail.
 	 */
 	reportGCCycleFinalIncrementEnding(env);
+	reportGCIncrementEnd(env);
 
 	MM_EnvironmentRealtime *rtEnv = MM_EnvironmentRealtime::getEnvironment(env);
 	_sched->reportStopGCIncrement(rtEnv, true);
@@ -654,6 +616,24 @@ MM_RealtimeGC::reportGCCycleEnd(MM_EnvironmentBase *env)
 	omrthread_monitor_exit(env->getOmrVM()->_gcCycleOnMonitor);
 }
 
+void
+MM_RealtimeGC::reportGCIncrementStart(MM_EnvironmentBase *env)
+{
+	PORT_ACCESS_FROM_ENVIRONMENT(env);
+	MM_CollectionStatistics *stats = (MM_CollectionStatistics *)env->_cycleState->_collectionStatistics;
+	stats->_startTime = j9time_hires_clock();
+}
+
+void
+MM_RealtimeGC::reportGCIncrementEnd(MM_EnvironmentBase *env)
+{
+	PORT_ACCESS_FROM_ENVIRONMENT(env);
+	MM_CollectionStatistics *stats = (MM_CollectionStatistics *)env->_cycleState->_collectionStatistics;
+
+	stats->_endTime = j9time_hires_clock();
+	stats->processPauseDuration();
+}
+
 /**
  * @todo Provide method documentation
  * @ingroup GC_Metronome methodGroup
