@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 /**
  * @file
@@ -47,6 +47,7 @@
 
 MM_InterRegionRememberedSet::MM_InterRegionRememberedSet(MM_HeapRegionManager *heapRegionManager)
 	: _heapRegionManager(heapRegionManager)
+	, _rsclBufferControlBlockPoolMemoryHandle()
 	, _rsclBufferControlBlockPool(NULL)
 	, _rsclBufferControlBlockHead(NULL)
 	, _freeBufferCount(0)
@@ -239,7 +240,7 @@ MM_InterRegionRememberedSet::allocateRegionBuffers(MM_EnvironmentVLHGC* env, MM_
 }
 
 bool
-MM_InterRegionRememberedSet::initialize(MM_EnvironmentVLHGC* env)
+MM_InterRegionRememberedSet::initialize(MM_EnvironmentVLHGC *env)
 {
 	MM_GCExtensions *ext = MM_GCExtensions::getExtensions(env);
 
@@ -255,8 +256,17 @@ MM_InterRegionRememberedSet::initialize(MM_EnvironmentVLHGC* env)
 	UDATA bufferSize = MM_RememberedSetCardBucket::MAX_BUFFER_SIZE * cardSize;
 	Assert_MM_true(((UDATA)1 << MM_Bits::leadingZeroes(bufferSize)) == bufferSize);
 
-	/* All Buffer Control Block are pre-allocated on startup. Buffers themselves are allocated as regions are commited */
-	_rsclBufferControlBlockPool = (MM_CardBufferControlBlock *)ext->getForge()->allocate(rsclBufferControlBlockPoolSize, MM_AllocationCategory::REMEMBERED_SET, J9_GET_CALLSITE());
+	/* All Buffer Control Block are pre-allocated on startup, based on maximum heap size. Buffers themselves are allocated as regions are used first time.
+	 * Since Control Blocks consume considerable amount of memory, they are allocated as virtual memory,
+	 * so they can be lazily committed as regions are effectively used (when their RSCL is first time initialized). */
+	MM_MemoryManager *memoryManager = ext->memoryManager;
+	if (memoryManager->createVirtualMemoryForMetadata(env, &_rsclBufferControlBlockPoolMemoryHandle, ext->heapAlignment, rsclBufferControlBlockPoolSize)) {
+		_rsclBufferControlBlockPool = (MM_CardBufferControlBlock *)memoryManager->getHeapBase(&_rsclBufferControlBlockPoolMemoryHandle);
+		if (!memoryManager->commitMemory(&_rsclBufferControlBlockPoolMemoryHandle, _rsclBufferControlBlockPool, rsclBufferControlBlockPoolSize)) {
+			_rsclBufferControlBlockPool = NULL;
+		}
+	}
+
 	if (NULL == _rsclBufferControlBlockPool) {
 		return false;
 	}
@@ -468,15 +478,15 @@ MM_InterRegionRememberedSet::releaseCardBufferControlBlockLocalPools(MM_Environm
 
 
 void
-MM_InterRegionRememberedSet::tearDown(MM_EnvironmentBase* env)
+MM_InterRegionRememberedSet::tearDown(MM_EnvironmentBase *env)
 {
 	MM_GCExtensions *ext = MM_GCExtensions::getExtensions(env);
 
 	if (NULL != _rsclBufferControlBlockPool) {
-		ext->getForge()->free(_rsclBufferControlBlockPool);
+		ext->memoryManager->destroyVirtualMemory(env, &_rsclBufferControlBlockPoolMemoryHandle);
+		_rsclBufferControlBlockPool = NULL;
 	}
 
-	/* TODO: _lock initialize might have failed */
 	_lock.tearDown();
 }
 
