@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -1358,16 +1358,17 @@ MM_SchedulingDelegate::calculateEdenSize(MM_EnvironmentVLHGC *env)
 	uintptr_t maxEdenRegionCount = _extensions->getHeap()->getHeapRegionManager()->getTableRegionCount();
 	bool edenIsVerySmall = (_edenRegionCount * 64) < maxEdenRegionCount;
 
+	/* Eden will be stealing free regions from the entire heap, without telling the heap to grow.
+	 * Note: the eden sizing logic knows how much free memory is available in the heap, and knows to not grow too much.
+	 * eden size can not be bigger than free region size.
+	 */
+	maxEdenChange = freeRegions - _edenRegionCount;
+
 	if (0 == maxHeapExpansionRegions) {
-		/*
-		 * The heap is fully expanded. Eden will be stealing free regions from the entire heap, without telling the heap to grow.
-		 * Note: When heap is fully expanded, the eden sizing logic knows how much free memory is available in the heap, and knows to not grow too much
-		 */
-		maxEdenChange = freeRegions;
 		_extensions->globalVLHGCStats._heapSizingData.edenRegionChange = 0;
 	} else {
 		/* Eden will inform the total heap resizing logic, that it needs to change total heap size in order to maintain same "tenure" size */
-		maxEdenChange = maxHeapExpansionRegions;
+		maxEdenChange += maxHeapExpansionRegions;
 		intptr_t edenChangeWithSurvivorHeadroom = desiredEdenChangeSize;
 
 		/* Total heap needs to be aware that by changing eden size, the amount of survivor space might also need to change */
