@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2002, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 2002
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -29,7 +29,6 @@
 #include <stdlib.h>
 #include <assert.h>
 
-
 #include "j9consts.h"
 #include "jclprots.h"
 #include "j9protos.h"
@@ -60,8 +59,8 @@ extern void initializeVMI(void);
 static J9ThreadEnv*
 getJ9ThreadEnv(JNIEnv* env)
 {
-	JavaVM* jniVM;
-	static J9ThreadEnv* threadEnv;
+	JavaVM* jniVM = NULL;
+	static J9ThreadEnv* threadEnv = NULL;
 
 	if (NULL != threadEnv) {
 		return threadEnv;
@@ -88,9 +87,9 @@ getJ9ThreadEnv(JNIEnv* env)
 void JNICALL
 JVM_ArrayCopy(JNIEnv *env, jclass ignored, jobject src, jint src_pos, jobject dst, jint dst_pos, jint length)
 {
-	J9VMThread *currentThread;
-	J9JavaVM *vm;
-	J9InternalVMFunctions *vmFuncs;
+	J9VMThread *currentThread = NULL;
+	J9JavaVM *vm = NULL;
+	J9InternalVMFunctions *vmFuncs = NULL;
 
 	Assert_SC_notNull(env);
 
@@ -332,6 +331,8 @@ JVM_DisableCompiler(jint arg0, jint arg1)
 	return NULL;
 }
 
+#if (JAVA_SPEC_VERSION == 8) && defined(WIN32)
+/* Do not remove: this method is required by Windows IBM Java 8 builds. */
 static jclass
 java_lang_J9VMInternals(JNIEnv* env)
 {
@@ -355,6 +356,7 @@ java_lang_J9VMInternals(JNIEnv* env)
 	return cached;
 }
 
+/* Do not remove: this method is required by Windows IBM Java 8 builds. */
 static jmethodID
 java_lang_J9VMInternals_doPrivileged(JNIEnv* env)
 {
@@ -368,6 +370,7 @@ java_lang_J9VMInternals_doPrivileged(JNIEnv* env)
 	return cached;
 }
 
+/* Do not remove: this method is required by Windows IBM Java 8 builds. */
 static jmethodID
 java_lang_J9VMInternals_doPrivilegedWithException(JNIEnv* env)
 {
@@ -381,7 +384,7 @@ java_lang_J9VMInternals_doPrivilegedWithException(JNIEnv* env)
 	return cached;
 }
 
-
+/* Do not remove: this method is required by Windows IBM Java 8 builds. */
 jobject JNICALL
 JVM_DoPrivileged(JNIEnv* env, jobject java_security_AccessController, jobject action, jboolean unknown, jboolean isExceptionAction)
 {
@@ -404,8 +407,7 @@ JVM_DoPrivileged(JNIEnv* env, jobject java_security_AccessController, jobject ac
 			methodID,
 			action);
 }
-
-
+#endif /* (JAVA_SPEC_VERSION == 8) && defined(WIN32) */
 
 jobject JNICALL
 JVM_EnableCompiler(jint arg0, jint arg1)
@@ -501,16 +503,16 @@ done:
 }
 
 /**
- * Find the specified class in given class loader 
+ * Find the specified class in given class loader
  *
  * @param env
  * @param className    null-terminated class name string.
  * @param init         initialize the class when set
- * @param classLoader  classloader of the class 
+ * @param classLoader  classloader of the class
  * @param throwError   set to true in order to throw errors
  * @return Assumed to be a jclass.
  *
- * Note: this call is implemented from info provided via CMVC 154874. 
+ * Note: this call is implemented from info provided via CMVC 154874.
  */
 jobject JNICALL
 JVM_FindClassFromClassLoader(JNIEnv* env, char* className, jboolean init, jobject classLoader, jboolean throwError)
@@ -625,9 +627,9 @@ JVM_FindPrimitiveClass(JNIEnv* env, char* name)
 jobject JNICALL
 JVM_GetArrayElement(JNIEnv *env, jobject array, jint index)
 {
-	J9VMThread *currentThread;
-	J9JavaVM *vm;
-	J9InternalVMFunctions *vmFuncs;
+	J9VMThread *currentThread = NULL;
+	J9JavaVM *vm = NULL;
+	J9InternalVMFunctions *vmFuncs = NULL;
 	jobject elementJNIRef = NULL;
 
 	Assert_SC_notNull(env);
@@ -749,9 +751,9 @@ JVM_GetArrayElement(JNIEnv *env, jobject array, jint index)
 jint JNICALL
 JVM_GetArrayLength(JNIEnv *env, jobject array)
 {
-	J9VMThread *currentThread;
-	J9JavaVM *vm;
-	J9InternalVMFunctions *vmFuncs;
+	J9VMThread *currentThread = NULL;
+	J9JavaVM *vm = NULL;
+	J9InternalVMFunctions *vmFuncs = NULL;
 	jsize arrayLength = 0;
 
 	Assert_SC_notNull(env);
@@ -765,8 +767,8 @@ JVM_GetArrayLength(JNIEnv *env, jobject array)
 	if (NULL == array) {
 		vmFuncs->setCurrentException(currentThread, J9VMCONSTANTPOOL_JAVALANGNULLPOINTEREXCEPTION, NULL);
 	} else {
-		J9Class *ramClass;
-		j9object_t j9array;
+		J9Class *ramClass = NULL;
+		j9object_t j9array = NULL;
 
 		j9array = J9_JNI_UNWRAP_REFERENCE(array);
 		ramClass = J9OBJECT_CLAZZ(currentThread, j9array);
@@ -787,11 +789,11 @@ J9Class*
 java_lang_Class_vmRef(JNIEnv* env, jobject clazz)
 {
 	J9VMThread* currentThread = (J9VMThread*) env;
-	J9JavaVM*  vm = currentThread->javaVM;
-	J9Class* ramClass;
+	J9JavaVM* vm = currentThread->javaVM;
+	J9Class* ramClass = NULL;
 
 	vm->internalVMFunctions->internalEnterVMFromJNI(currentThread);
-	ramClass = J9VMJAVALANGCLASS_VMREF(currentThread, J9_JNI_UNWRAP_REFERENCE(clazz) );
+	ramClass = J9VMJAVALANGCLASS_VMREF(currentThread, J9_JNI_UNWRAP_REFERENCE(clazz));
 	vm->internalVMFunctions->internalExitVMToJNI(currentThread);
 
 	return ramClass;
@@ -831,10 +833,10 @@ utf8_to_java_lang_String(JNIEnv* env, J9UTF8* utf)
 jobject JNICALL
 JVM_GetClassDeclaredConstructors(JNIEnv* env, jclass clazz, jboolean unknown)
 {
-	J9Class* ramClass;
-	J9ROMClass* romClass;
-	jclass jlrConstructor;
-	jobjectArray result;
+	J9Class* ramClass = NULL;
+	J9ROMClass* romClass = NULL;
+	jclass jlrConstructor = NULL;
+	jobjectArray result = NULL;
 	char* eyecatcher = "<init>";
 	U_16 initLength = 6;
 	jsize size = 0;
@@ -887,7 +889,7 @@ JVM_GetClassDeclaredConstructors(JNIEnv* env, jclass clazz, jboolean unknown)
 				char* name = utf8_to_cstring(env, nameUTF);
 				char* signature = utf8_to_cstring(env, signatureUTF);
 				jmethodID methodID = (*env)->GetMethodID(env, clazz, name, signature);
-				jobject reflectedMethod;
+				jobject reflectedMethod = NULL;
 
 				assert(methodID != NULL);
 				if (NULL != name) {
@@ -912,12 +914,12 @@ JVM_GetClassDeclaredConstructors(JNIEnv* env, jclass clazz, jboolean unknown)
 jobject JNICALL
 JVM_GetClassDeclaredFields(JNIEnv* env, jobject clazz, jint arg2)
 {
-	J9Class* ramClass;
-	J9ROMClass* romClass;
-	jclass jlrField;
-	jsize size;
-	jobjectArray result;
-	J9ROMFieldShape *field;
+	J9Class* ramClass = NULL;
+	J9ROMClass* romClass = NULL;
+	jclass jlrField = NULL;
+	jsize size = 0;
+	jobjectArray result = NULL;
+	J9ROMFieldShape *field = NULL;
 	J9ROMFieldWalkState walkState;
 	jsize index = 0;
 	PORT_ACCESS_FROM_ENV(env);
@@ -946,17 +948,17 @@ JVM_GetClassDeclaredFields(JNIEnv* env, jobject clazz, jint arg2)
 
 	/* Iterate through the fields */
 	field = romFieldsStartDo( romClass, &walkState );
-	while( field != NULL ) {
+	while (NULL != field) {
 		U_32 modifiers = field->modifiers;
-		jfieldID fieldID;
-		jobject reflectedField;
-		jboolean isStatic;
+		jfieldID fieldID = 0;
+		jobject reflectedField = NULL;
+		jboolean isStatic = JNI_FALSE;
 		J9UTF8* nameUTF =  J9ROMFIELDSHAPE_NAME(field);
 		J9UTF8* signatureUTF = J9ROMFIELDSHAPE_SIGNATURE(field);
 		char* name = utf8_to_cstring(env, nameUTF);
 		char* signature = utf8_to_cstring(env, signatureUTF);
 
-		if(modifiers & J9AccStatic) {
+		if (J9_ARE_ANY_BITS_SET(modifiers, J9AccStatic)) {
 			fieldID  = (*env)->GetStaticFieldID(env, clazz, name, signature);
 			isStatic = JNI_TRUE;
 		} else {
@@ -975,7 +977,7 @@ JVM_GetClassDeclaredFields(JNIEnv* env, jobject clazz, jint arg2)
 		reflectedField = (*env)->ToReflectedField(env, clazz, fieldID, isStatic);
 		assert(reflectedField != NULL);
 		(*env)->SetObjectArrayElement(env, result, index++, reflectedField);
-		field = romFieldsNextDo( &walkState );
+		field = romFieldsNextDo(&walkState);
 	}
 
 	return result;
@@ -986,10 +988,10 @@ JVM_GetClassDeclaredFields(JNIEnv* env, jobject clazz, jint arg2)
 jobject JNICALL
 JVM_GetClassDeclaredMethods(JNIEnv* env, jobject clazz, jboolean unknown)
 {
-	J9Class* ramClass;
-	J9ROMClass* romClass;
-	jclass jlrMethod;
-	jobjectArray result;
+	J9Class* ramClass = NULL;
+	J9ROMClass* romClass = NULL;
+	jclass jlrMethod = NULL;
+	jobjectArray result = NULL;
 	char* eyecatcher = "<init>";
 	U_16 initLength = 6;
 	jsize size = 0;
@@ -1042,11 +1044,11 @@ JVM_GetClassDeclaredMethods(JNIEnv* env, jobject clazz, jboolean unknown)
 				char* name = utf8_to_cstring(env, nameUTF);
 				char* signature = utf8_to_cstring(env, signatureUTF);
 				U_32 modifiers = romMethod->modifiers;
-				jmethodID methodID;
-				jobject reflectedMethod;
-				jboolean isStatic;
+				jmethodID methodID = NULL;
+				jobject reflectedMethod = NULL;
+				jboolean isStatic = JNI_FALSE;
 
-				if(modifiers & J9AccStatic) {
+				if (J9_ARE_ANY_BITS_SET(modifiers, J9AccStatic)) {
 					methodID = (*env)->GetStaticMethodID(env, clazz, name, signature);
 					isStatic = JNI_TRUE;
 				} else {
@@ -1054,7 +1056,6 @@ JVM_GetClassDeclaredMethods(JNIEnv* env, jobject clazz, jboolean unknown)
 					isStatic = JNI_FALSE;
 				}
 
-
 				assert(methodID != NULL);
 				if (NULL != name) {
 					j9mem_free_memory(name);
@@ -1082,16 +1083,15 @@ JVM_GetClassInterfaces(jint arg0, jint arg1)
 	return NULL;
 }
 
-
-
+#if JAVA_SPEC_VERSION < 25
 jint JNICALL
-JVM_GetClassModifiers(JNIEnv* env, jclass clazz)
+JVM_GetClassModifiers(JNIEnv *env, jclass clazz)
 {
-	J9Class* ramClass = java_lang_Class_vmRef(env, clazz);
-	J9ROMClass* romClass = ramClass->romClass;
+	J9Class *ramClass = java_lang_Class_vmRef(env, clazz);
+	J9ROMClass *romClass = ramClass->romClass;
 
 	if (J9ROMCLASS_IS_ARRAY(romClass)) {
-		J9ArrayClass* arrayClass = (J9ArrayClass*)ramClass;
+		J9ArrayClass *arrayClass = (J9ArrayClass *)ramClass;
 		jint result = 0;
 		J9ROMClass *leafRomClass = arrayClass->leafComponentType->romClass;
 		if (J9_ARE_ALL_BITS_SET(leafRomClass->extraModifiers, J9AccClassInnerClass)) {
@@ -1110,8 +1110,7 @@ JVM_GetClassModifiers(JNIEnv* env, jclass clazz)
 		}
 	}
 }
-
-
+#endif /* JAVA_SPEC_VERSION < 25 */
 
 jobject JNICALL
 JVM_GetClassSigners(jint arg0, jint arg1)
@@ -1154,14 +1153,14 @@ JVM_GetDeclaringClass(jint arg0, jint arg1)
 }
 
 
-
+#if JAVA_SPEC_VERSION < 24
 jobject JNICALL
 JVM_GetInheritedAccessControlContext(jint arg0, jint arg1)
 {
 	assert(!"JVM_GetInheritedAccessControlContext() stubbed!");
 	return NULL;
 }
-
+#endif /* JAVA_SPEC_VERSION < 24 */
 
 
 /**
@@ -1175,9 +1174,9 @@ JVM_GetInheritedAccessControlContext(jint arg0, jint arg1)
 jvalue JNICALL
 JVM_GetPrimitiveArrayElement(JNIEnv *env, jobject array, jint index, jint wCode)
 {
-	J9VMThread *currentThread;
-	J9JavaVM *vm;
-	J9InternalVMFunctions *vmFuncs;
+	J9VMThread *currentThread = NULL;
+	J9JavaVM *vm = NULL;
+	J9InternalVMFunctions *vmFuncs = NULL;
 	jvalue value;
 
 	Assert_SC_notNull(env);
@@ -1343,20 +1342,22 @@ JVM_GetPrimitiveArrayElement(JNIEnv *env, jobject array, jint index, jint wCode)
 	return value;
 }
 
-
-
+#if JAVA_SPEC_VERSION < 25
 jobject JNICALL
 JVM_GetProtectionDomain(jint arg0, jint arg1)
 {
 	assert(!"JVM_GetProtectionDomain() stubbed!");
 	return NULL;
 }
+#endif /* JAVA_SPEC_VERSION < 25 */
 
+#if JAVA_SPEC_VERSION < 24
 jobject JNICALL
-JVM_GetStackAccessControlContext(JNIEnv* env, jclass java_security_AccessController)
+JVM_GetStackAccessControlContext(JNIEnv *env, jclass java_security_AccessController)
 {
 	return NULL;
 }
+#endif /* JAVA_SPEC_VERSION < 24 */
 
 jint JNICALL
 JVM_GetStackTraceDepth(JNIEnv* env, jobject throwable)
@@ -1364,7 +1365,7 @@ JVM_GetStackTraceDepth(JNIEnv* env, jobject throwable)
 	J9VMThread* currentThread = (J9VMThread*) env;
 	J9JavaVM * vm = currentThread->javaVM;
 	J9InternalVMFunctions * vmfns = vm->internalVMFunctions;
-	jint numberOfFrames;
+	jint numberOfFrames = 0;
 	UDATA pruneConstructors = 0;
 	/* If -XX:+ShowHiddenFrames option has not been set, skip hidden method frames */
 	UDATA skipHiddenFrames = J9_ARE_NO_BITS_SET(vm->runtimeFlags, J9_RUNTIME_SHOW_HIDDEN_FRAMES);
@@ -1427,7 +1428,7 @@ typedef struct GetStackTraceElementUserData {
 
 /* Return TRUE to keep iterating, FALSE to halt the walk. */
 static UDATA
-getStackTraceElementIterator(J9VMThread * vmThread, void * voidUserData, UDATA bytecodeOffset, J9ROMClass * romClass, J9ROMMethod * romMethod, J9UTF8 * fileName, UDATA lineNumber, J9ClassLoader* classLoader, J9Class* ramClass)
+getStackTraceElementIterator(J9VMThread * vmThread, void * voidUserData, UDATA bytecodeOffset, J9ROMClass * romClass, J9ROMMethod * romMethod, J9UTF8 * fileName, UDATA lineNumber, J9ClassLoader* classLoader, J9Class* ramClass, UDATA frameType)
 {
 	GetStackTraceElementUserData * userData = voidUserData;
 
@@ -1452,7 +1453,7 @@ JVM_GetStackTraceElement(JNIEnv* env, jobject throwable, jint index)
 	J9VMThread* currentThread = (J9VMThread*) env;
 	J9JavaVM * vm = currentThread->javaVM;
 	J9InternalVMFunctions * vmfns = vm->internalVMFunctions;
-	jobject stackTraceElement;
+	jobject stackTraceElement = NULL;
 	UDATA pruneConstructors = 0;
 	GetStackTraceElementUserData userData;
 	jobject declaringClass = NULL;
@@ -1532,14 +1533,10 @@ JVM_IHashCode(JNIEnv *env, jobject obj)
 jobject JNICALL
 JVM_InitProperties(JNIEnv* env, jobject properties)
 {
-	/* This JVM method is invoked by JCL native Java_java_lang_System_initProperties
-	 * only for initialization of platform encoding.
-	 * This is only required by Java 11 raw builds.
-	 * This method is not invoked by other Java levels.
-	 */
-#if JAVA_SPEC_VERSION < 11
+	/* This JVM method is invoked by JCL native Java_java_lang_System_initProperties. */
+#if !defined(OPENJ9_BUILD)
 	assert(!"JVM_InitProperties should not be called!");
-#endif /* JAVA_SPEC_VERSION < 11 */
+#endif /* !defined(OPENJ9_BUILD) */
 	return properties;
 }
 
@@ -1560,7 +1557,7 @@ JVM_InternString(JNIEnv *env, jstring str)
 		J9VMThread* currentThread = (J9VMThread*) env;
 		J9JavaVM* javaVM = currentThread->javaVM;
 		J9InternalVMFunctions* vmfns = javaVM->internalVMFunctions;
-		j9object_t stringObject;
+		j9object_t stringObject = NULL;
 
 		vmfns->internalEnterVMFromJNI(currentThread);
 		stringObject = J9_JNI_UNWRAP_REFERENCE(str);
@@ -1581,42 +1578,30 @@ JVM_Interrupt(jint arg0, jint arg1)
 	return NULL;
 }
 
-
+#if JAVA_SPEC_VERSION < 25
+jboolean JNICALL
+JVM_IsArrayClass(JNIEnv *env, jclass clazz)
+{
+	J9Class *ramClass = java_lang_Class_vmRef(env, clazz);
+	return J9ROMCLASS_IS_ARRAY(ramClass->romClass) ? JNI_TRUE : JNI_FALSE;
+}
+#endif /* JAVA_SPEC_VERSION < 25 */
 
 jboolean JNICALL
-JVM_IsArrayClass(JNIEnv* env, jclass clazz)
+JVM_IsInterface(JNIEnv *env, jclass clazz)
 {
-	J9Class * ramClass = java_lang_Class_vmRef(env, clazz);
-	if (J9ROMCLASS_IS_ARRAY(ramClass->romClass)) {
-		return JNI_TRUE;
-	} else {
-		return JNI_FALSE;
-	}
+	J9Class *ramClass = java_lang_Class_vmRef(env, clazz);
+	return J9ROMCLASS_IS_INTERFACE(ramClass->romClass) ? JNI_TRUE : JNI_FALSE;
 }
 
-
-
-jboolean JNICALL
-JVM_IsInterface(JNIEnv* env, jclass clazz)
-{
-	J9Class * ramClass = java_lang_Class_vmRef(env, clazz);
-	if (J9ROMCLASS_IS_INTERFACE(ramClass->romClass)) {
-		return JNI_TRUE;
-	} else {
-		return JNI_FALSE;
-	}
-}
-
-
-
 jboolean JNICALL
 JVM_IsInterrupted(JNIEnv* env, jobject thread, jboolean unknown)
 {
 	J9VMThread *currentThread = (J9VMThread *)env;
-	J9VMThread *targetThread;
+	J9VMThread *targetThread = NULL;
 	J9JavaVM* vm = currentThread->javaVM;
 	J9ThreadEnv* threadEnv = getJ9ThreadEnv(env);
-	UDATA rcClear;
+	UDATA rcClear = 0;
 
 	vm->internalVMFunctions->internalEnterVMFromJNI(currentThread);
 	targetThread = J9VMJAVALANGTHREAD_THREADREF(currentThread, J9_JNI_UNWRAP_REFERENCE(thread) );
@@ -1636,32 +1621,26 @@ JVM_IsInterrupted(JNIEnv* env, jobject thread, jboolean unknown)
 	}
 }
 
-
-
+#if JAVA_SPEC_VERSION < 25
 jboolean JNICALL
-JVM_IsPrimitiveClass(JNIEnv* env, jclass clazz)
+JVM_IsPrimitiveClass(JNIEnv *env, jclass clazz)
 {
-	J9Class * ramClass = java_lang_Class_vmRef(env, clazz);
-	if (J9ROMCLASS_IS_PRIMITIVE_TYPE(ramClass->romClass)) {
-		return JNI_TRUE;
-	} else {
-		return JNI_FALSE;
-	}
+	J9Class *ramClass = java_lang_Class_vmRef(env, clazz);
+	return J9ROMCLASS_IS_PRIMITIVE_TYPE(ramClass->romClass) ? JNI_TRUE : JNI_FALSE;
 }
-
+#endif /* JAVA_SPEC_VERSION < 25 */
 
 /**
- * Check whether the jni version is supported.
- * This function may not lock, gc or throw exception.
+ * Check whether the JNI version is supported.
+ * This function may not lock, GC or throw an exception.
  * @param version
- * @return true if version is JNI_VERSION_1_1, JNI_VERSION_1_2, JNI_VERSION_1_4, JNI_VERSION_1_6, or
- * 		   JNI_VERSION_1_8, JNI_VERSION_9, JNI_VERSION_10; false if not.
+ * @return true if version is supported; false if not
  * @careful
  */
 jboolean JNICALL
 JVM_IsSupportedJNIVersion(jint version)
 {
-	switch(version) {
+	switch (version) {
 	case JNI_VERSION_1_1:
 	case JNI_VERSION_1_2:
 	case JNI_VERSION_1_4:
@@ -1673,6 +1652,18 @@ JVM_IsSupportedJNIVersion(jint version)
 #if JAVA_SPEC_VERSION >= 10
 	case JNI_VERSION_10:
 #endif /* JAVA_SPEC_VERSION >= 10 */
+#if JAVA_SPEC_VERSION >= 19
+	case JNI_VERSION_19:
+#endif /* JAVA_SPEC_VERSION >= 19 */
+#if JAVA_SPEC_VERSION >= 20
+	case JNI_VERSION_20:
+#endif /* JAVA_SPEC_VERSION >= 20 */
+#if JAVA_SPEC_VERSION >= 21
+	case JNI_VERSION_21:
+#endif /* JAVA_SPEC_VERSION >= 21 */
+#if JAVA_SPEC_VERSION >= 24
+	case JNI_VERSION_24:
+#endif /* JAVA_SPEC_VERSION >= 24 */
 		return JNI_TRUE;
 
 	default:
@@ -1681,7 +1672,7 @@ JVM_IsSupportedJNIVersion(jint version)
 }
 
 
-
+#if JAVA_SPEC_VERSION < 17
 jboolean JNICALL
 JVM_IsThreadAlive(JNIEnv* jniEnv, jobject targetThread)
 {
@@ -1696,7 +1687,7 @@ JVM_IsThreadAlive(JNIEnv* jniEnv, jobject targetThread)
 	/* Assume that a non-null threadRef indicates the thread is alive */
 	return (NULL == vmThread) ? JNI_FALSE : JNI_TRUE;
 }
-
+#endif /* JAVA_SPEC_VERSION < 17 */
 
 
 jobject JNICALL
@@ -1707,8 +1698,8 @@ JVM_NewArray(JNIEnv* jniEnv, jclass componentType, jint dimension)
 	J9MemoryManagerFunctions * mmfns = vm->memoryManagerFunctions;
 	J9Class* ramClass = java_lang_Class_vmRef(jniEnv, componentType);
 	J9ROMClass* romClass = ramClass->romClass;
-	j9object_t newArray;
-	jobject arrayRef;
+	j9object_t newArray = NULL;
+	jobject arrayRef = NULL;
 
 	vm->internalVMFunctions->internalEnterVMFromJNI(currentThread);
 	if (ramClass->arrayClass == NULL) {
@@ -1775,7 +1766,7 @@ JVM_NewMultiArray(JNIEnv *env, jclass eltClass, jintArray dim)
 			vmFuncs->setCurrentException(currentThread, J9VMCONSTANTPOOL_JAVALANGILLEGALARGUMENTEXCEPTION, NULL);
 		} else {
 			j9object_t componentTypeClassObject = J9_JNI_UNWRAP_REFERENCE(eltClass);
-	
+
 			if (NULL != componentTypeClassObject) {
 				J9Class *componentTypeClass = J9VM_J9CLASS_FROM_HEAPCLASS(currentThread, componentTypeClassObject);
 
@@ -1783,25 +1774,25 @@ JVM_NewMultiArray(JNIEnv *env, jclass eltClass, jintArray dim)
 				UDATA count = dimensions;
 				J9Class *componentArrayClass = componentTypeClass;
 				BOOLEAN exceptionIsPending = FALSE;
-	
+
 				while ((count > 0) && (!exceptionIsPending)) {
 					componentArrayClass = fetchArrayClass(currentThread, componentArrayClass);
 					exceptionIsPending = (NULL != currentThread->currentException);
 					count -= 1;
 				}
-				
+
 				if (!exceptionIsPending) {
 					/* make a copy of the dimensions array in non-object memory */
 					I_32 onStackDimensions[MAX_DIMENSIONS];
 					j9object_t directObject = NULL;
 					UDATA i = 0;
-					
+
 					memset(onStackDimensions, 0, sizeof(onStackDimensions));
 					dimensionsArrayObject = J9_JNI_UNWRAP_REFERENCE(dim);
 					for (i = 0; i < dimensions; i++) {
 						onStackDimensions[i] = J9JAVAARRAYOFINT_LOAD(currentThread, dimensionsArrayObject, i);
 					}
-	
+
 					directObject = vmFuncs->helperMultiANewArray(currentThread, (J9ArrayClass *)componentArrayClass, (UDATA)dimensions, onStackDimensions, J9_GC_ALLOCATE_OBJECT_NON_INSTRUMENTABLE);
 					if (NULL != directObject) {
 						result = vmFuncs->j9jni_createLocalRef(env, directObject);
@@ -1825,17 +1816,6 @@ JVM_ResolveClass(jint arg0, jint arg1)
 	return NULL;
 }
 
-
-
-jobject JNICALL
-JVM_ResumeThread(jint arg0, jint arg1)
-{
-	assert(!"JVM_ResumeThread() stubbed!");
-	return NULL;
-}
-
-
-
 /**
  * Set the val to the array at the index.
  * This function may lock, gc or throw exception.
@@ -1846,9 +1826,9 @@ JVM_ResumeThread(jint arg0, jint arg1)
 void JNICALL
 JVM_SetArrayElement(JNIEnv *env, jobject array, jint index, jobject value)
 {
-	J9VMThread *currentThread;
-	J9JavaVM *vm;
-	J9InternalVMFunctions *vmFuncs;
+	J9VMThread *currentThread = NULL;
+	J9JavaVM *vm = NULL;
+	J9InternalVMFunctions *vmFuncs = NULL;
 
 	Assert_SC_notNull(env);
 
@@ -2047,9 +2027,9 @@ JVM_SetClassSigners(jint arg0, jint arg1, jint arg2)
 void JNICALL
 JVM_SetPrimitiveArrayElement(JNIEnv *env, jobject array, jint index, jvalue value, unsigned char vCode)
 {
-	J9VMThread *currentThread;
-	J9JavaVM *vm;
-	J9InternalVMFunctions *vmFuncs;
+	J9VMThread *currentThread = NULL;
+	J9JavaVM *vm = NULL;
+	J9InternalVMFunctions *vmFuncs = NULL;
 
 	Assert_SC_notNull(env);
 
@@ -2228,7 +2208,7 @@ JVM_SetThreadPriority(JNIEnv* env, jobject thread, jint priority)
 	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM* vm = currentThread->javaVM;
 	const UDATA *prioMap = currentThread->javaVM->java2J9ThreadPriorityMap;
-	J9VMThread *vmThread;
+	J9VMThread *vmThread = NULL;
 
 	if (currentThread->javaVM->runtimeFlags & J9_RUNTIME_NO_PRIORITIES) {
 		return;
@@ -2244,7 +2224,7 @@ JVM_SetThreadPriority(JNIEnv* env, jobject thread, jint priority)
 	vmThread = J9VMJAVALANGTHREAD_THREADREF(currentThread, J9_JNI_UNWRAP_REFERENCE(thread) );
 	vm->internalVMFunctions->internalExitVMToJNI(currentThread);
 
-	if ( (NULL != vmThread) && (NULL != vmThread->osThread)) {
+	if ((NULL != vmThread) && (NULL != vmThread->osThread)) {
 		J9ThreadEnv* threadEnv = getJ9ThreadEnv(env);
 		threadEnv->set_priority(vmThread->osThread, prioMap[priority]);
 	}
@@ -2257,25 +2237,38 @@ JVM_StartThread(JNIEnv* jniEnv, jobject newThread)
 {
 	J9VMThread* currentThread = (J9VMThread*)jniEnv;
 	J9JavaVM* javaVM = currentThread->javaVM;
-	UDATA priority, isDaemon, privateFlags;
-	j9object_t newThreadObject;
-	UDATA result;
+	UDATA priority = J9THREAD_PRIORITY_NORMAL;
+	UDATA isDaemon = FALSE;
+	UDATA privateFlags = 0;
+	j9object_t newThreadObject = NULL;
+	UDATA result = 0;
 
 	javaVM->internalVMFunctions->internalEnterVMFromJNI(currentThread);
 
 	newThreadObject = J9_JNI_UNWRAP_REFERENCE(newThread);
+#if JAVA_SPEC_VERSION >= 19
+	j9object_t threadHolder = J9VMJAVALANGTHREAD_HOLDER(currentThread, newThreadObject);
+#endif /* JAVA_SPEC_VERSION >= 19 */
 
-	if (0 != (javaVM->runtimeFlags & J9RuntimeFlagNoPriorities)) {
-		priority = J9THREAD_PRIORITY_NORMAL;
-	} else {
+	if (J9_ARE_NO_BITS_SET(javaVM->runtimeFlags, J9RuntimeFlagNoPriorities)) {
+#if JAVA_SPEC_VERSION >= 19
+		if (NULL != threadHolder) {
+			priority = J9VMJAVALANGTHREADFIELDHOLDER_PRIORITY(currentThread, threadHolder);
+		}
+#else /* JAVA_SPEC_VERSION >= 19 */
 		priority = J9VMJAVALANGTHREAD_PRIORITY(currentThread, newThreadObject);
+#endif /* JAVA_SPEC_VERSION >= 19 */
 	}
 
+#if JAVA_SPEC_VERSION >= 19
+	if (NULL != threadHolder) {
+		isDaemon = J9VMJAVALANGTHREADFIELDHOLDER_DAEMON(currentThread, threadHolder);
+	}
+#else /* JAVA_SPEC_VERSION >= 19 */
 	isDaemon = J9VMJAVALANGTHREAD_ISDAEMON(currentThread, newThreadObject);
+#endif /* JAVA_SPEC_VERSION >= 19 */
 	if (isDaemon) {
 		privateFlags = J9_PRIVATE_FLAGS_DAEMON_THREAD;
-	} else {
-		privateFlags = 0;
 	}
 
 	result = javaVM->internalVMFunctions->startJavaThread(
@@ -2297,7 +2290,13 @@ JVM_StartThread(JNIEnv* jniEnv, jobject newThread)
 	return;
 }
 
-
+#if JAVA_SPEC_VERSION < 20
+jobject JNICALL
+JVM_ResumeThread(jint arg0, jint arg1)
+{
+	assert(!"JVM_ResumeThread() stubbed!");
+	return NULL;
+}
 
 jobject JNICALL
 JVM_StopThread(jint arg0, jint arg1, jint arg2)
@@ -2306,39 +2305,13 @@ JVM_StopThread(jint arg0, jint arg1, jint arg2)
 	return NULL;
 }
 
-
-
 jobject JNICALL
 JVM_SuspendThread(jint arg0, jint arg1)
 {
 	assert(!"JVM_SuspendThread() stubbed!");
 	return NULL;
 }
-
-
-/* NOTE this is required by JDK15+ jdk.internal.loader.NativeLibraries.unload().
- */
-#if JAVA_SPEC_VERSION >= 15
-void JNICALL JVM_UnloadLibrary(void *handle)
-#else /* JAVA_SPEC_VERSION >= 15 */
-jobject JNICALL JVM_UnloadLibrary(jint arg0)
-#endif /* JAVA_SPEC_VERSION >= 15 */
-{
-#if JAVA_SPEC_VERSION >= 15
-	Trc_SC_UnloadLibrary_Entry(handle);
-#if defined(WIN32)
-	FreeLibrary((HMODULE)handle);
-#elif defined(J9UNIX) || defined(J9ZOS390) /* defined(WIN32) */
-	dlclose(handle);
-#else /* defined(WIN32) */
-#error "Please implement J7vmi.c:JVM_UnloadLibrary(void *handle)"
-#endif /* defined(WIN32) */
-#else /* JAVA_SPEC_VERSION >= 15 */
-	assert(!"JVM_UnloadLibrary() stubbed!");
-	return NULL;
-#endif /* JAVA_SPEC_VERSION >= 15 */
-}
-
+#endif /* JAVA_SPEC_VERSION < 20 */
 
 
 jobject JNICALL
@@ -2355,8 +2328,8 @@ JVM_Yield(jint arg0, jint arg1)
 jint JNICALL
 JVM_SetSockOpt(jint fd, int level, int optname, const char *optval, int optlen)
 {
-	jint retVal;
-	
+	jint retVal = 0;
+
 #if defined(WIN32)
 	retVal = setsockopt(fd, level, optname, optval, optlen);
 #elif defined(J9ZTPF)
@@ -2364,7 +2337,7 @@ JVM_SetSockOpt(jint fd, int level, int optname, const char *optval, int optlen)
 #else
 	retVal = setsockopt(fd, level, optname, optval, (socklen_t)optlen);
 #endif
-	
+
 	return retVal;
 }
 
@@ -2372,7 +2345,7 @@ JVM_SetSockOpt(jint fd, int level, int optname, const char *optval, int optlen)
 jint JNICALL
 JVM_GetSockOpt(jint fd, int level, int optname, char *optval, int *optlen)
 {
-	jint retVal;
+	jint retVal = 0;
 
 #if defined(WIN32)
 	retVal = getsockopt(fd, level, optname, optval, optlen);
@@ -2381,7 +2354,7 @@ JVM_GetSockOpt(jint fd, int level, int optname, char *optval, int *optlen)
 #else
 	retVal = getsockopt(fd, level, optname, optval, (socklen_t *)optlen);
 #endif
-	
+
 	return retVal;
 }
 
@@ -2391,7 +2364,7 @@ JVM_GetSockOpt(jint fd, int level, int optname, char *optval, int *optlen)
 jint JNICALL
 JVM_SocketShutdown(jint fd, jint howto)
 {
-	jint retVal = JNI_FALSE;
+	jint retVal = 0;
 
 #if defined(J9UNIX)
 	retVal = shutdown(fd, howto);
@@ -2400,7 +2373,7 @@ JVM_SocketShutdown(jint fd, jint howto)
 #else /* defined(J9UNIX) */
 	assert(!"JVM_SocketShutdown() stubbed!");
 #endif /* defined(J9UNIX) */
-	
+
 	return retVal;
 }
 
@@ -2411,14 +2384,14 @@ JVM_SocketShutdown(jint fd, jint howto)
 jint JNICALL
 JVM_GetSockName(jint fd, struct sockaddr *him, int *len)
 {
-	jint retVal;
+	jint retVal = 0;
 
 #if defined(WIN32)
 	retVal = getsockname(fd, him, len);
 #else
 	retVal = getsockname(fd, him, (socklen_t *)len);
 #endif
-	
+
 	return retVal;
 }
 
@@ -2428,9 +2401,7 @@ JVM_GetSockName(jint fd, struct sockaddr *him, int *len)
 int JNICALL
 JVM_GetHostName(char* name, int namelen)
 {
-	jint retVal;
-
-	retVal = gethostname(name, namelen);
+	jint retVal = gethostname(name, namelen);
 
 	return retVal;
 }
@@ -2446,11 +2417,11 @@ JVM_GetHostName(char* name, int namelen)
  *  sun.jvm.args =
  */
 /*
- * Notes: 
- * 	Redirector has an implementation of JVM_InitAgentProperties. 
- * 	This method is still kept within the actual jvm dll in case that a launcher uses this jvm dll directly without going through the redirector. 
- * 	If this method need to be modified, the changes have to be synchronized for both versions. 
- */ 
+ * Notes:
+ * 	Redirector has an implementation of JVM_InitAgentProperties.
+ * 	This method is still kept within the actual jvm dll in case that a launcher uses this jvm dll directly without going through the redirector.
+ * 	If this method need to be modified, the changes have to be synchronized for both versions.
+ */
 jobject JNICALL
 JVM_InitAgentProperties(JNIEnv *env, jobject agent_props)
 {
@@ -2501,7 +2472,7 @@ void
 throwNewNullPointerException(JNIEnv *env, char *message)
 {
 	jclass exceptionClass = (*env)->FindClass(env, "java/lang/NullPointerException");
-	if (exceptionClass == 0) {
+	if (NULL == exceptionClass) {
 		/* Just return if we can't load the exception class. */
 		return;
 	}
@@ -2515,7 +2486,7 @@ void
 throwNewIndexOutOfBoundsException(JNIEnv *env, char *message)
 {
 	jclass exceptionClass = (*env)->FindClass(env, "java/lang/IndexOutOfBoundsException");
-	if (exceptionClass == 0) {
+	if (NULL == exceptionClass) {
 		/* Just return if we can't load the exception class. */
 		return;
 	}
@@ -2530,7 +2501,7 @@ void
 throwNewInternalError(JNIEnv *env, char *message)
 {
 	jclass exceptionClass = (*env)->FindClass(env, "java/lang/InternalError");
-	if (exceptionClass == 0) {
+	if (NULL == exceptionClass) {
 		/* Just return if we can't load the exception class. */
 		return;
 	}
@@ -2580,14 +2551,32 @@ jvmDefineClassHelper(JNIEnv *env, jobject classLoaderObject,
 
 	if (NULL != className) {
 		j9object_t classNameObject = J9_JNI_UNWRAP_REFERENCE(className);
+
+		/* Perform maximum length check to avoid copy in extreme cases. */
+		if (J9VMJAVALANGSTRING_LENGTH(currentThread, classNameObject) > J9VM_MAX_CLASS_NAME_LENGTH) {
+			vmFuncs->setCurrentExceptionNLS(currentThread,
+				J9VMCONSTANTPOOL_JAVALANGCLASSNOTFOUNDEXCEPTION,
+				J9NLS_VM_CLASS_NAME_EXCEEDS_MAX_LENGTH);
+			goto done;
+		}
+
 		utf8Name = (U_8*)vmFuncs->copyStringToUTF8WithMemAlloc(currentThread, classNameObject, J9_STR_NULL_TERMINATE_RESULT, "", 0, utf8NameStackBuffer, J9VM_PACKAGE_NAME_BUFFER_LENGTH, &utf8Length);
 		if (NULL == utf8Name) {
 			vmFuncs->setNativeOutOfMemoryError(currentThread, 0, 0);
 			goto done;
 		}
 
+		if (utf8Length > J9VM_MAX_CLASS_NAME_LENGTH) {
+			vmFuncs->setCurrentExceptionNLS(currentThread,
+				J9VMCONSTANTPOOL_JAVALANGCLASSNOTFOUNDEXCEPTION,
+				J9NLS_VM_CLASS_NAME_EXCEEDS_MAX_LENGTH);
+			goto done;
+		}
+
 		if (CLASSNAME_INVALID == vmFuncs->verifyQualifiedName(currentThread, utf8Name, utf8Length, CLASSNAME_VALID_NON_ARRARY)) {
-			vmFuncs->setCurrentException(currentThread, J9VMCONSTANTPOOL_JAVALANGNOCLASSDEFFOUNDERROR, (UDATA *)*(j9object_t*)className);
+			vmFuncs->setCurrentException(currentThread,
+				J9VMCONSTANTPOOL_JAVALANGNOCLASSDEFFOUNDERROR,
+				(UDATA *)*(j9object_t *)className);
 			goto done;
 		}
 	}
@@ -2720,11 +2709,10 @@ JVM_DTraceIsProbeEnabled(jint arg0, jint arg1)
 	return NULL;
 }
 
-jobject JNICALL
-JVM_DTraceIsSupported(jint arg0)
+jboolean JNICALL
+JVM_DTraceIsSupported(JNIEnv *env)
 {
-	assert(!"JVM_DTraceIsSupported() stubbed!");
-	return NULL;
+	return JNI_FALSE;
 }
 
 #endif /* JAVA_SPEC_VERSION < 17 */
@@ -2750,13 +2738,6 @@ JVM_EnqueueOperation(jint arg0, jint arg1, jint arg2, jint arg3, jint arg4)
 	return NULL;
 }
 
-jobject JNICALL
-JVM_Exit(jint arg0)
-{
-	assert(!"JVM_Exit() stubbed!");
-	return NULL;
-}
-
 jobject JNICALL
 JVM_GetCPFieldNameUTF(jint arg0, jint arg1, jint arg2)
 {
