@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2021, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 2021
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j9vm31.h"
@@ -173,12 +173,26 @@ GetStringUTFLength(JNIEnv *env, jstring string)
 	const jint NUM_ARGS = 2;
 	J9_CEL4RO64_ArgType argTypes[NUM_ARGS] = { CEL4RO64_type_JNIEnv64, CEL4RO64_type_jstring };
 	uint64_t argValues[NUM_ARGS] = { JNIENV64_FROM_JNIENV31(env), string };
-	jsize returnValue = NULL;
+	jsize returnValue = 0;
 	FUNCTION_DESCRIPTOR_FROM_JNIENV31(env, GetStringUTFLength);
 	j9_cel4ro64_call_function(functionDescriptor, argTypes, argValues, NUM_ARGS, CEL4RO64_type_jsize, &returnValue);
 	return returnValue;
 }
 
+#if JAVA_SPEC_VERSION >= 24
+jlong JNICALL
+GetStringUTFLengthAsLong(JNIEnv *env, jstring string)
+{
+	const jint NUM_ARGS = 2;
+	J9_CEL4RO64_ArgType argTypes[NUM_ARGS] = { CEL4RO64_type_JNIEnv64, CEL4RO64_type_jstring };
+	uint64_t argValues[NUM_ARGS] = { JNIENV64_FROM_JNIENV31(env), string };
+	jlong returnValue = 0;
+	FUNCTION_DESCRIPTOR_FROM_JNIENV31(env, GetStringUTFLengthAsLong);
+	j9_cel4ro64_call_function(functionDescriptor, argTypes, argValues, NUM_ARGS, CEL4RO64_type_jlong, &returnValue);
+	return returnValue;
+}
+#endif /* JAVA_SPEC_VERSION >= 24 */
+
 const char* JNICALL
 GetStringUTFChars(JNIEnv *env, jstring string, jboolean *isCopy)
 {
@@ -362,8 +376,23 @@ GetStringPlatform(JNIEnv* env, jstring instr, char* outstr, jint outlen, const c
 	J9_CEL4RO64_ArgType argTypes[NUM_ARGS] = { CEL4RO64_type_JNIEnv64, CEL4RO64_type_jstring, CEL4RO64_type_uint32_ptr, CEL4RO64_type_jint, CEL4RO64_type_uint32_ptr };
 	uint64_t argValues[NUM_ARGS] = { JNIENV64_FROM_JNIENV31(env), instr, (uint64_t)outstr, outlen, (uint64_t)encoding };
 	jint returnValue = NULL;
-	uint32_t cel4ro64ReturnCode = j9_cel4ro64_load_query_call_function(
-			LIBJVM_NAME, "GetStringPlatform",
+	uint32_t cel4ro64ReturnCode = J9_CEL4RO64_RETCODE_OK;
+
+	static uint64_t GetStringPlatformFD = 0; /* Cached value of GetStringPlatform function descriptor. */
+
+	/* Lazy initialization of function descriptor on first invocation. */
+	if (0 == GetStringPlatformFD) {
+		cel4ro64ReturnCode = j9_cel4ro64_load_query_function_descriptor(
+			LIBJVM_NAME, "GetStringPlatform", &GetStringPlatformFD);
+
+		if (J9_CEL4RO64_RETCODE_OK != cel4ro64ReturnCode) {
+			fprintf(stderr, "GetStringPlatform function query failed. CEL4RO64 rc: %d - %s\n", cel4ro64ReturnCode, j9_cel4ro64_get_error_message(cel4ro64ReturnCode));
+			return JNI_ERR;
+		}
+	}
+
+	cel4ro64ReturnCode = j9_cel4ro64_call_function(
+			GetStringPlatformFD,
 			argTypes, argValues, NUM_ARGS, CEL4RO64_type_jint, &returnValue);
 	if (J9_CEL4RO64_RETCODE_OK != cel4ro64ReturnCode) {
 		fprintf(stderr, "GetStringPlatform failed. CEL4RO64 rc: %d - %s\n", cel4ro64ReturnCode, j9_cel4ro64_get_error_message(cel4ro64ReturnCode));
@@ -379,8 +408,23 @@ GetStringPlatformLength(JNIEnv* env, jstring instr, jint* outlen, const char* en
 	J9_CEL4RO64_ArgType argTypes[NUM_ARGS] = { CEL4RO64_type_JNIEnv64, CEL4RO64_type_jstring, CEL4RO64_type_uint32_ptr, CEL4RO64_type_uint32_ptr };
 	uint64_t argValues[NUM_ARGS] = { JNIENV64_FROM_JNIENV31(env), instr, (uint64_t)outlen, (uint64_t)encoding };
 	jint returnValue = NULL;
-	uint32_t cel4ro64ReturnCode = j9_cel4ro64_load_query_call_function(
-			LIBJVM_NAME, "GetStringPlatformLength",
+	uint32_t cel4ro64ReturnCode = J9_CEL4RO64_RETCODE_OK;
+
+	static uint64_t GetStringPlatformLengthFD = 0; /* Cached value of GetStringPlatformLength function descriptor. */
+
+	/* Lazy initialization of function descriptor on first invocation. */
+	if (0 == GetStringPlatformLengthFD) {
+		cel4ro64ReturnCode = j9_cel4ro64_load_query_function_descriptor(
+			LIBJVM_NAME, "GetStringPlatformLength", &GetStringPlatformLengthFD);
+
+		if (J9_CEL4RO64_RETCODE_OK != cel4ro64ReturnCode) {
+			fprintf(stderr, "GetStringPlatformLength function query failed. CEL4RO64 rc: %d - %s\n", cel4ro64ReturnCode, j9_cel4ro64_get_error_message(cel4ro64ReturnCode));
+			return JNI_ERR;
+		}
+	}
+
+	cel4ro64ReturnCode = j9_cel4ro64_call_function(
+			GetStringPlatformLengthFD,
 			argTypes, argValues, NUM_ARGS, CEL4RO64_type_jint, &returnValue);
 	if (J9_CEL4RO64_RETCODE_OK != cel4ro64ReturnCode) {
 		fprintf(stderr, "GetStringPlatformLength failed. CEL4RO64 rc: %d - %s\n", cel4ro64ReturnCode, j9_cel4ro64_get_error_message(cel4ro64ReturnCode));
@@ -396,8 +440,23 @@ NewStringPlatform(JNIEnv* env, const char* instr, jstring* outstr, const char* e
 	J9_CEL4RO64_ArgType argTypes[NUM_ARGS] = { CEL4RO64_type_JNIEnv64, CEL4RO64_type_uint32_ptr, CEL4RO64_type_uint32_ptr, CEL4RO64_type_uint32_ptr };
 	uint64_t argValues[NUM_ARGS] = { JNIENV64_FROM_JNIENV31(env), (uint64_t)instr, (uint64_t)outstr, (uint64_t)encoding };
 	jint returnValue = NULL;
-	uint32_t cel4ro64ReturnCode = j9_cel4ro64_load_query_call_function(
-			LIBJVM_NAME, "NewStringPlatform",
+	uint32_t cel4ro64ReturnCode = J9_CEL4RO64_RETCODE_OK;
+
+	static uint64_t NewStringPlatformFD = 0; /* Cached value of NewStringPlatform function descriptor. */
+
+	/* Lazy initialization of function descriptor on first invocation. */
+	if (0 == NewStringPlatformFD) {
+		cel4ro64ReturnCode = j9_cel4ro64_load_query_function_descriptor(
+			LIBJVM_NAME, "NewStringPlatform", &NewStringPlatformFD);
+
+		if (J9_CEL4RO64_RETCODE_OK != cel4ro64ReturnCode) {
+			fprintf(stderr, "NewStringPlatform function query failed. CEL4RO64 rc: %d - %s\n", cel4ro64ReturnCode, j9_cel4ro64_get_error_message(cel4ro64ReturnCode));
+			return JNI_ERR;
+		}
+	}
+
+	cel4ro64ReturnCode = j9_cel4ro64_call_function(
+			NewStringPlatformFD,
 			argTypes, argValues, NUM_ARGS, CEL4RO64_type_jint, &returnValue);
 	if (J9_CEL4RO64_RETCODE_OK != cel4ro64ReturnCode) {
 		fprintf(stderr, "NewStringPlatform failed. CEL4RO64 rc: %d - %s\n", cel4ro64ReturnCode, j9_cel4ro64_get_error_message(cel4ro64ReturnCode));
