@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2019 IBM Corp. and others
+ * Copyright IBM Corp. and others 1998
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include <stdlib.h>
@@ -260,7 +260,7 @@ printHeapStatistics(JNIEnv *env,J9HeapStatisticsTableEntry **statsArray,
 
 	PORT_ACCESS_FROM_ENV(env);
 
-	result = j9str_printf(PORTLIB, bufferCursor, bufferSize,
+	result = j9str_printf(bufferCursor, bufferSize,
 			"%5s %14s %14s    %s\n-------------------------------------------------\n",
 			"num", "object count", "total size", "class name"
 	);
@@ -268,7 +268,7 @@ printHeapStatistics(JNIEnv *env,J9HeapStatisticsTableEntry **statsArray,
 	bufferSize -= result;
 	for (classCursor = 0; (result > 0) && (classCursor < numClasses); ++classCursor) {
 		J9Class *currentClass = statsArray[classCursor]->clazz;
-		result = j9str_printf(PORTLIB, bufferCursor, bufferSize,
+		result = j9str_printf(bufferCursor, bufferSize,
 				"%5d %14zu %14zu    ",
 				classCursor + 1, statsArray[classCursor]->objectCount,
 				statsArray[classCursor]->aggregateSize
@@ -284,20 +284,20 @@ printHeapStatistics(JNIEnv *env,J9HeapStatisticsTableEntry **statsArray,
 			UDATA isPrimitive = J9ROMCLASS_IS_PRIMITIVE_TYPE(leafROMClass);
 			UDATA i = 0;
 			for (i = 0; i < arity; ++i) {
-				result = j9str_printf(PORTLIB, bufferCursor, bufferSize, "[");
+				result = j9str_printf(bufferCursor, bufferSize, "[");
 				bufferCursor += result;
 				bufferSize -= result;
 			}
 			if (isPrimitive) {
-				result = j9str_printf(PORTLIB, bufferCursor, bufferSize, "%c\n",
+				result = j9str_printf(bufferCursor, bufferSize, "%c\n",
 						J9UTF8_DATA(J9ROMCLASS_CLASSNAME(leafComponentType->arrayClass->romClass))[1]);
 			} else {
-				result = j9str_printf(PORTLIB, bufferCursor, bufferSize, "L%.*s;\n",
+				result = j9str_printf(bufferCursor, bufferSize, "L%.*s;\n",
 						J9UTF8_LENGTH(leafName), J9UTF8_DATA(leafName));
 			}
 		} else {
 			J9UTF8 *className = J9ROMCLASS_CLASSNAME(currentClass->romClass);
-			result = j9str_printf(PORTLIB, bufferCursor, bufferSize, "%.*s\n",
+			result = j9str_printf(bufferCursor, bufferSize, "%.*s\n",
 					J9UTF8_LENGTH(className), J9UTF8_DATA(className));
 		}
 		bufferCursor += result;
@@ -305,7 +305,7 @@ printHeapStatistics(JNIEnv *env,J9HeapStatisticsTableEntry **statsArray,
 		cumulativeCount += statsArray[classCursor]->objectCount;
 		cumulativeSize += statsArray[classCursor]->aggregateSize;
 	}
-	result = j9str_printf(PORTLIB, bufferCursor, bufferSize,
+	result = j9str_printf(bufferCursor, bufferSize,
 			"%5s %14zd %14zd\n",
 			"Total", cumulativeCount, cumulativeSize
 	);
@@ -569,3 +569,16 @@ Java_com_ibm_oti_vm_VM_dumpString(JNIEnv * env, jclass clazz, jstring str)
 		}
 	}
 }
+
+#if defined(J9VM_OPT_JFR)
+jstring JNICALL
+Java_com_ibm_oti_vm_VM_getjfrCMDLineOption(JNIEnv *env, jclass clazz)
+{
+	J9JavaVM *javaVM = ((J9VMThread *) env)->javaVM;
+	jstring jfrOption = NULL;
+	if (NULL != javaVM->jfrState.jfrCMDLineOption) {
+		jfrOption = (*env)->NewStringUTF(env, javaVM->jfrState.jfrCMDLineOption);
+	}
+	return jfrOption;
+}
+#endif /* defined(J9VM_OPT_JFR) */
