@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1998
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j9.h"
@@ -38,6 +38,7 @@
 #include "ArrayCopyHelpers.hpp"
 #include "AtomicSupport.hpp"
 #include "ObjectMonitor.hpp"
+#include "UnsafeAPI.hpp"
 #include "VMHelpers.hpp"
 
 extern "C" {
@@ -104,7 +105,7 @@ Java_sun_misc_Unsafe_defineAnonymousClass(JNIEnv *env, jobject receiver, jclass
 	PORT_ACCESS_FROM_ENV(env);
 
 	U_16 indexMap[BUFFER_SIZE];
-	if (constPatches != NULL) {
+	if (NULL != constPatches) {
 		j9array_t patchArray = (j9array_t)J9_JNI_UNWRAP_REFERENCE(constPatches);
 		cpPatchMap.size = (U_16)J9INDEXABLEOBJECT_SIZE(currentThread, patchArray);
 		if (cpPatchMap.size <= BUFFER_SIZE) {
@@ -112,7 +113,7 @@ Java_sun_misc_Unsafe_defineAnonymousClass(JNIEnv *env, jobject receiver, jclass
 		} else {
 			cpPatchMap.indexMap = (U_16 *)j9mem_allocate_memory(cpPatchMap.size * sizeof(U_16), J9MEM_CATEGORY_VM);
 
-			if (cpPatchMap.indexMap == NULL) {
+			if (NULL == cpPatchMap.indexMap) {
 				vmFuncs->setNativeOutOfMemoryError(currentThread, 0, 0);
 				vmFuncs->internalExitVMToJNI(currentThread);
 				return NULL;
@@ -137,7 +138,7 @@ Java_sun_misc_Unsafe_defineAnonymousClass(JNIEnv *env, jobject receiver, jclass
 		goto exit;
 	}
 
-	if (constPatches != NULL) {
+	if (NULL != constPatches) {
 		vmFuncs->internalEnterVMFromJNI(currentThread);
 		j9array_t patchArray = (j9array_t)J9_JNI_UNWRAP_REFERENCE(constPatches);
 		J9Class *clazz = J9VM_J9CLASS_FROM_HEAPCLASS(currentThread, J9_JNI_UNWRAP_REFERENCE(anonClass));
@@ -150,9 +151,8 @@ Java_sun_misc_Unsafe_defineAnonymousClass(JNIEnv *env, jobject receiver, jclass
 			/* Check if a valid mapping exist for cp entry */
 			if (cpPatchMap.indexMap[i] != 0) {
 				j9object_t item = J9JAVAARRAYOFOBJECT_LOAD(currentThread, patchArray, i);
-				if (item != NULL) {
+				if (NULL != item) {
 					if (J9_CP_TYPE(cpShapeDescription, cpPatchMap.indexMap[i]) == J9CPTYPE_STRING) {
-
 						J9UTF8 *romString = J9ROMSTRINGREF_UTF8DATA((J9ROMStringRef *)&romCP[cpPatchMap.indexMap[i]]);
 
 						/* For each patch object, search the RAM constantpool for identical string entries */
@@ -191,14 +191,12 @@ exit:
 void JNICALL
 Java_sun_misc_Unsafe_registerNatives(JNIEnv *env, jclass clazz)
 {
-	jfieldID fid;
-
 	Trc_JCL_sun_misc_Unsafe_registerNatives_Entry(env);
 
 	/* If Unsafe has a static int field called INVALID_FIELD_OFFSET, set it to -1 */
+	jfieldID fid = env->GetStaticFieldID(clazz, "INVALID_FIELD_OFFSET", "I");
 
-	fid = env->GetStaticFieldID(clazz, "INVALID_FIELD_OFFSET", "I");
-	if (fid == NULL) {
+	if (NULL == fid) {
 		env->ExceptionClear();
 	} else {
 		env->SetStaticIntField(clazz, fid, -1);
@@ -207,7 +205,6 @@ Java_sun_misc_Unsafe_registerNatives(JNIEnv *env, jclass clazz)
 	Trc_JCL_sun_misc_Unsafe_registerNatives_Exit(env);
 }
 
-
 jint JNICALL
 Java_sun_misc_Unsafe_pageSize(JNIEnv *env, jobject receiver)
 {
@@ -216,7 +213,6 @@ Java_sun_misc_Unsafe_pageSize(JNIEnv *env, jobject receiver)
 	return (jint)j9vmem_supported_page_sizes()[0];
 }
 
-
 jint JNICALL
 Java_sun_misc_Unsafe_getLoadAverage(JNIEnv *env, jobject receiver, jdoubleArray loadavg, jint nelems)
 {
@@ -225,39 +221,37 @@ Java_sun_misc_Unsafe_getLoadAverage(JNIEnv *env, jobject receiver, jdoubleArray
 	return -1;
 }
 
-
 jlong JNICALL
 Java_sun_misc_Unsafe_allocateMemory(JNIEnv *env, jobject receiver, jlong size)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	jlong result = 0;
-	UDATA actualSize = (UDATA)size;
+	UDATA actualSize = (UDATA)(U_64)size;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
-	if ((size < 0) || (size != (jlong)actualSize)) {
+	if ((size < 0) || (size != (jlong)(IDATA)actualSize)) {
 		vmFuncs->setCurrentExceptionUTF(currentThread, J9VMCONSTANTPOOL_JAVALANGILLEGALARGUMENTEXCEPTION, NULL);
 	} else {
-		result = (jlong)(UDATA)unsafeAllocateMemory(currentThread, actualSize, TRUE);
+		result = JLONG_FROM_POINTER(unsafeAllocateMemory(currentThread, actualSize, TRUE));
 	}
 	vmFuncs->internalExitVMToJNI(currentThread);
 	return result;
 }
 
-
 jlong JNICALL
 Java_sun_misc_Unsafe_allocateDBBMemory(JNIEnv *env, jobject receiver, jlong size)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	jlong result = 0;
-	UDATA actualSize = (UDATA)size;
+	UDATA actualSize = (UDATA)(U_64)size;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
-	if ((size < 0) || (size != (jlong)actualSize)) {
+	if ((size < 0) || (size != (jlong)(IDATA)actualSize)) {
 		vmFuncs->setCurrentExceptionUTF(currentThread, J9VMCONSTANTPOOL_JAVALANGILLEGALARGUMENTEXCEPTION, NULL);
 	} else {
-		result = (jlong)(UDATA)unsafeAllocateDBBMemory(currentThread, actualSize, TRUE);
+		result = JLONG_FROM_POINTER(unsafeAllocateDBBMemory(currentThread, actualSize, TRUE));
 	}
 	vmFuncs->internalExitVMToJNI(currentThread);
 	return result;
@@ -269,20 +263,18 @@ Java_jdk_internal_misc_Unsafe_allocateDBBMemory(JNIEnv *env, jobject receiver, j
 	return Java_sun_misc_Unsafe_allocateDBBMemory(env, receiver, size);
 }
 
-
 void JNICALL
 Java_sun_misc_Unsafe_freeMemory(JNIEnv *env, jobject receiver, jlong address)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
-	unsafeFreeMemory(currentThread, (void*)(UDATA)address);
+	J9VMThread *currentThread = (J9VMThread *)env;
+	unsafeFreeMemory(currentThread, JLONG_TO_POINTER(address));
 }
 
-
 void JNICALL
 Java_sun_misc_Unsafe_freeDBBMemory(JNIEnv *env, jobject receiver, jlong address)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
-	unsafeFreeDBBMemory(currentThread, (void*)(UDATA)address);
+	J9VMThread *currentThread = (J9VMThread *)env;
+	unsafeFreeDBBMemory(currentThread, JLONG_TO_POINTER(address));
 }
 
 void JNICALL
@@ -294,35 +286,34 @@ Java_jdk_internal_misc_Unsafe_freeDBBMemory(JNIEnv *env, jobject receiver, jlong
 jlong JNICALL
 Java_sun_misc_Unsafe_reallocateMemory(JNIEnv *env, jobject receiver, jlong address, jlong size)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	jlong result = 0;
 	UDATA actualSize = (UDATA)size;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
-	if ((size < 0) || (size != (jlong)actualSize)) {
+	if ((size < 0) || (size != (jlong)(IDATA)actualSize)) {
 		vmFuncs->setCurrentExceptionUTF(currentThread, J9VMCONSTANTPOOL_JAVALANGILLEGALARGUMENTEXCEPTION, NULL);
 	} else {
-		result = (jlong)(UDATA)unsafeReallocateMemory(currentThread, (void*)(UDATA)address, actualSize);
+		result = JLONG_FROM_POINTER(unsafeReallocateMemory(currentThread, JLONG_TO_POINTER(address), actualSize));
 	}
 	vmFuncs->internalExitVMToJNI(currentThread);
 	return result;
 }
 
-
 jlong JNICALL
 Java_sun_misc_Unsafe_reallocateDBBMemory(JNIEnv *env, jobject receiver, jlong address, jlong size)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	jlong result = 0;
 	UDATA actualSize = (UDATA)size;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
-	if ((size < 0) || (size != (jlong)actualSize)) {
+	if ((size < 0) || (size != (jlong)(IDATA)actualSize)) {
 		vmFuncs->setCurrentExceptionUTF(currentThread, J9VMCONSTANTPOOL_JAVALANGILLEGALARGUMENTEXCEPTION, NULL);
 	} else {
-		result = (jlong)(UDATA)unsafeReallocateDBBMemory(currentThread, (void*)(UDATA)address, actualSize);
+		result = JLONG_FROM_POINTER(unsafeReallocateDBBMemory(currentThread, JLONG_TO_POINTER(address), actualSize));
 	}
 	vmFuncs->internalExitVMToJNI(currentThread);
 	return result;
@@ -337,7 +328,7 @@ Java_jdk_internal_misc_Unsafe_reallocateDBBMemory(JNIEnv *env, jobject receiver,
 void JNICALL
 Java_sun_misc_Unsafe_ensureClassInitialized(JNIEnv *env, jobject receiver, jclass clazz)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
@@ -353,23 +344,21 @@ Java_sun_misc_Unsafe_ensureClassInitialized(JNIEnv *env, jobject receiver, jclas
 	vmFuncs->internalExitVMToJNI(currentThread);
 }
 
-
 void JNICALL
 Java_sun_misc_Unsafe_park(JNIEnv *env, jobject receiver, jboolean isAbsolute, jlong time)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
-	vmFuncs->threadParkImpl(currentThread, (IDATA)isAbsolute, (I_64)time);
+	vmFuncs->threadParkImpl(currentThread, isAbsolute ? TRUE : FALSE, time);
 	vmFuncs->internalExitVMToJNI(currentThread);
 }
 
-
 void JNICALL
 Java_sun_misc_Unsafe_unpark(JNIEnv *env, jobject receiver, jthread thread)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
@@ -377,11 +366,10 @@ Java_sun_misc_Unsafe_unpark(JNIEnv *env, jobject receiver, jthread thread)
 	vmFuncs->internalExitVMToJNI(currentThread);
 }
 
-
 void JNICALL
 Java_sun_misc_Unsafe_throwException(JNIEnv *env, jobject receiver, jthrowable exception)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
@@ -393,11 +381,10 @@ Java_sun_misc_Unsafe_throwException(JNIEnv *env, jobject receiver, jthrowable ex
 	vmFuncs->internalExitVMToJNI(currentThread);
 }
 
-
 void JNICALL
 Java_sun_misc_Unsafe_monitorEnter(JNIEnv *env, jobject receiver, jobject obj)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
@@ -405,9 +392,16 @@ Java_sun_misc_Unsafe_monitorEnter(JNIEnv *env, jobject receiver, jobject obj)
 		vmFuncs->setCurrentExceptionUTF(currentThread, J9VMCONSTANTPOOL_JAVALANGNULLPOINTEREXCEPTION, NULL);
 	} else {
 		UDATA monresult = vmFuncs->objectMonitorEnter(currentThread, J9_JNI_UNWRAP_REFERENCE(obj));
-		if (0 == monresult) {
+		if (J9_OBJECT_MONITOR_ENTER_FAILED(monresult)) {
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+			if (J9_OBJECT_MONITOR_CRIU_SINGLE_THREAD_MODE_THROW == monresult) {
+				vmFuncs->setCRIUSingleThreadModeJVMCRIUException(currentThread, 0, 0);
+			} else
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+			if (J9_OBJECT_MONITOR_OOM == monresult) {
 oom:
-			vmFuncs->setNativeOutOfMemoryError(currentThread, 0, 0);
+				vmFuncs->setNativeOutOfMemoryError(currentThread, 0, 0);
+			}
 		} else if (J9_UNEXPECTED(!VM_ObjectMonitor::recordJNIMonitorEnter(currentThread, (j9object_t)monresult))) {
 			vmFuncs->objectMonitorExit(currentThread, (j9object_t)monresult);
 			goto oom;
@@ -416,11 +410,10 @@ oom:
 	vmFuncs->internalExitVMToJNI(currentThread);
 }
 
-
 void JNICALL
 Java_sun_misc_Unsafe_monitorExit(JNIEnv *env, jobject receiver, jobject obj)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
@@ -438,11 +431,10 @@ Java_sun_misc_Unsafe_monitorExit(JNIEnv *env, jobject receiver, jobject obj)
 	vmFuncs->internalExitVMToJNI(currentThread);
 }
 
-
 jboolean JNICALL
 Java_sun_misc_Unsafe_tryMonitorEnter(JNIEnv *env, jobject receiver, jobject obj)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	jboolean entered = JNI_TRUE;
@@ -502,11 +494,11 @@ determineCommonAlignment(UDATA sourceOffset, UDATA destOffset, UDATA size)
 {
 	UDATA alignment = determineAlignment(sourceOffset);
 	UDATA destAlignment = determineAlignment(destOffset);
-	if (destAlignment < alignment) {
+	if (alignment > destAlignment) {
 		alignment = destAlignment;
 	}
 	UDATA sizeAlignment = determineAlignment(size);
-	if (sizeAlignment < alignment) {
+	if (alignment > sizeAlignment) {
 		alignment = sizeAlignment;
 	}
 	return alignment;
@@ -528,7 +520,7 @@ determineCommonAlignment(UDATA sourceOffset, UDATA destOffset, UDATA size)
  * @param elementCount elements to copy in array
  */
 static VMINLINE void
-copyMemorySub(J9VMThread* currentThread, j9object_t sourceObject, UDATA sourceOffset, j9object_t destObject,
+copyMemorySub(J9VMThread *currentThread, j9object_t sourceObject, UDATA sourceOffset, j9object_t destObject,
 		UDATA destOffset, UDATA actualSize, UDATA logElementSize, UDATA sourceIndex, UDATA destIndex,
 		UDATA elementCount)
 {
@@ -536,15 +528,15 @@ copyMemorySub(J9VMThread* currentThread, j9object_t sourceObject, UDATA sourceOf
 		if (NULL == destObject) {
 			UDATA sourceEnd = sourceOffset + actualSize;
 			if ((destOffset > sourceOffset) && (destOffset < sourceEnd)) {
-				alignedBackwardsMemcpy(currentThread, (void*)(destOffset + actualSize), (void*)sourceEnd, actualSize, logElementSize);
+				alignedBackwardsMemcpy(currentThread, (void *)(destOffset + actualSize), (void *)sourceEnd, actualSize, logElementSize);
 			} else {
-				alignedMemcpy(currentThread, (void*)destOffset, (void*)sourceOffset, actualSize, logElementSize);
+				alignedMemcpy(currentThread, (void *)destOffset, (void *)sourceOffset, actualSize, logElementSize);
 			}
 		} else {
-			VM_ArrayCopyHelpers::memcpyToArray(currentThread, destObject, logElementSize, destIndex, elementCount, (void*)sourceOffset);
+			VM_ArrayCopyHelpers::memcpyToArray(currentThread, destObject, logElementSize, destIndex, elementCount, (void *)sourceOffset);
 		}
 	} else if (NULL == destObject) {
-		VM_ArrayCopyHelpers::memcpyFromArray(currentThread, sourceObject, logElementSize, sourceIndex, elementCount, (void*)destOffset);
+		VM_ArrayCopyHelpers::memcpyFromArray(currentThread, sourceObject, logElementSize, sourceIndex, elementCount, (void *)destOffset);
 	} else {
 		VM_ArrayCopyHelpers::primitiveArrayCopy(currentThread, sourceObject, sourceIndex, destObject, destIndex, elementCount, logElementSize);
 	}
@@ -561,11 +553,11 @@ copyMemorySub(J9VMThread* currentThread, j9object_t sourceObject, UDATA sourceOf
  * @param actualSize the number of bytes to be copied
  */
 static VMINLINE void
-copyMemory(J9VMThread* currentThread, j9object_t sourceObject, UDATA sourceOffset, j9object_t destObject,
+copyMemory(J9VMThread *currentThread, j9object_t sourceObject, UDATA sourceOffset, j9object_t destObject,
 		UDATA destOffset, UDATA actualSize)
 {
 	/* Because array data is always 8-aligned, only the alignment of the offsets (and byte size) need be considered */
-	UDATA const headerSize = J9VMTHREAD_CONTIGUOUS_HEADER_SIZE(currentThread);
+	UDATA const headerSize = VM_UnsafeAPI::arrayBase(currentThread);
 	UDATA logElementSize = determineCommonAlignment(sourceOffset, destOffset, actualSize);
 	UDATA sourceIndex = (sourceOffset - headerSize) >> logElementSize;
 	UDATA destIndex = (destOffset - headerSize) >> logElementSize;
@@ -585,10 +577,10 @@ copyMemory(J9VMThread* currentThread, j9object_t sourceObject, UDATA sourceOffse
  * @param destOffset location in destObject to start copy
  */
 static VMINLINE void
-copyMemoryByte(J9VMThread* currentThread, j9object_t sourceObject, UDATA sourceOffset, j9object_t destObject,
+copyMemoryByte(J9VMThread *currentThread, j9object_t sourceObject, UDATA sourceOffset, j9object_t destObject,
 		UDATA destOffset)
 {
-	UDATA const headerSize = J9VMTHREAD_CONTIGUOUS_HEADER_SIZE(currentThread);
+	UDATA const headerSize = VM_UnsafeAPI::arrayBase(currentThread);
 	UDATA sourceIndex = sourceOffset - headerSize;
 	UDATA destIndex = destOffset - headerSize;
 
@@ -599,7 +591,7 @@ copyMemoryByte(J9VMThread* currentThread, j9object_t sourceObject, UDATA sourceO
 void JNICALL
 Java_sun_misc_Unsafe_copyMemory__Ljava_lang_Object_2JLjava_lang_Object_2JJ(JNIEnv *env, jobject receiver, jobject srcBase, jlong srcOffset, jobject dstBase, jlong dstOffset, jlong size)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	UDATA sourceOffset = (UDATA)srcOffset;
@@ -618,7 +610,7 @@ illegal:
 			if (!J9CLASS_IS_ARRAY(clazz)) {
 				goto illegal;
 			}
-			if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass*)clazz)->componentType->romClass)) {
+			if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass *)clazz)->componentType->romClass)) {
 				goto illegal;
 			}
 		}
@@ -628,7 +620,7 @@ illegal:
 			if (!J9CLASS_IS_ARRAY(clazz)) {
 				goto illegal;
 			}
-			if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass*)clazz)->componentType->romClass)) {
+			if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass *)clazz)->componentType->romClass)) {
 				goto illegal;
 			}
 		}
@@ -638,11 +630,10 @@ illegal:
 	vmFuncs->internalExitVMToJNI(currentThread);
 }
 
-
 jlong JNICALL
 Java_sun_misc_Unsafe_objectFieldOffset(JNIEnv *env, jobject receiver, jobject field)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	jlong offset = 0;
@@ -664,41 +655,39 @@ Java_sun_misc_Unsafe_objectFieldOffset(JNIEnv *env, jobject receiver, jobject fi
 	return offset;
 }
 
-
 void JNICALL
 Java_sun_misc_Unsafe_setMemory__Ljava_lang_Object_2JJB(JNIEnv *env, jobject receiver, jobject obj, jlong offset, jlong size, jbyte value)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
-	UDATA actualSize = (UDATA)size;
-	if ((size < 0) || (size != (jlong)actualSize)) {
+	UDATA actualSize = (UDATA)(U_64)size;
+	if ((size < 0) || (size != (jlong)(IDATA)actualSize)) {
 illegal:
 		vmFuncs->setCurrentExceptionUTF(currentThread, J9VMCONSTANTPOOL_JAVALANGILLEGALARGUMENTEXCEPTION, NULL);
 	} else if (NULL == obj) {
 		/* NULL object - raw memset */
-		memset((void*)(UDATA)offset, (int)value, actualSize);
+		memset((void *)(IDATA)offset, (int)value, actualSize);
 	} else {
 		j9object_t object = J9_JNI_UNWRAP_REFERENCE(obj);
 		J9Class *clazz = J9OBJECT_CLAZZ(currentThread, object);
 		if (!J9CLASS_IS_ARRAY(clazz)) {
 			goto illegal;
 		}
-		if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass*)clazz)->componentType->romClass)) {
+		if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass *)clazz)->componentType->romClass)) {
 			goto illegal;
 		}
-		offset -= J9VMTHREAD_CONTIGUOUS_HEADER_SIZE(currentThread);
+		offset -= VM_UnsafeAPI::arrayBase(currentThread);
 		VM_ArrayCopyHelpers::primitiveArrayFill(currentThread, object, (UDATA)offset, actualSize, (U_8)value);
 	}
 	vmFuncs->internalExitVMToJNI(currentThread);
 }
 
-
 jobject JNICALL
 Java_sun_misc_Unsafe_staticFieldBase__Ljava_lang_reflect_Field_2(JNIEnv *env, jobject receiver, jobject field)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	jobject base = NULL;
@@ -720,11 +709,10 @@ Java_sun_misc_Unsafe_staticFieldBase__Ljava_lang_reflect_Field_2(JNIEnv *env, jo
 	return base;
 }
 
-
 jlong JNICALL
 Java_sun_misc_Unsafe_staticFieldOffset(JNIEnv *env, jobject receiver, jobject field)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	jlong offset = 0;
@@ -761,9 +749,9 @@ Java_sun_misc_Unsafe_isBigEndian0(JNIEnv *env, jobject receiver)
 {
 #if defined(J9VM_ENV_LITTLE_ENDIAN)
 	return JNI_FALSE;
-#else
+#else /* defined(J9VM_ENV_LITTLE_ENDIAN) */
 	return JNI_TRUE;
-#endif
+#endif /* defined(J9VM_ENV_LITTLE_ENDIAN) */
 }
 
 jobject JNICALL
@@ -803,8 +791,8 @@ Java_sun_misc_Unsafe_shouldBeInitialized(JNIEnv *env, jobject receiver, jclass c
 		vmFuncs->setCurrentExceptionUTF(currentThread, J9VMCONSTANTPOOL_JAVALANGNULLPOINTEREXCEPTION, NULL);
 	} else {
 		j9object_t classObject = J9_JNI_UNWRAP_REFERENCE(clazz);
-		J9Class *j9clazz =  J9VM_J9CLASS_FROM_HEAPCLASS(currentThread, classObject);
-		if (VM_VMHelpers::classRequiresInitialization(currentThread, j9clazz)) {
+		J9Class *j9clazz = J9VM_J9CLASS_FROM_HEAPCLASS(currentThread, classObject);
+		if (J9ClassInitSucceeded != j9clazz->initializeStatus) {
 			result = JNI_TRUE;
 		}
 	}
@@ -813,10 +801,11 @@ Java_sun_misc_Unsafe_shouldBeInitialized(JNIEnv *env, jobject receiver, jclass c
 	return result;
 }
 
+#if JAVA_SPEC_VERSION > 8
 jlong JNICALL
 Java_jdk_internal_misc_Unsafe_objectFieldOffset1(JNIEnv *env, jobject receiver, jclass clazz, jstring name)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
 	j9object_t fieldObj = NULL;
@@ -825,11 +814,8 @@ Java_jdk_internal_misc_Unsafe_objectFieldOffset1(JNIEnv *env, jobject receiver,
 	vmFuncs->internalEnterVMFromJNI(currentThread);
 	fieldObj = getFieldObjHelper(currentThread, clazz, name);
 	if (NULL != fieldObj) {
-		J9JNIFieldID *fieldID = NULL;
-		J9ROMFieldShape *romField = NULL;
-
-		fieldID = vm->reflectFunctions.idFromFieldObject(currentThread, J9_JNI_UNWRAP_REFERENCE(clazz), fieldObj);
-		romField = fieldID->field;
+		J9JNIFieldID *fieldID = vm->reflectFunctions.idFromFieldObject(currentThread, J9_JNI_UNWRAP_REFERENCE(clazz), fieldObj);
+		J9ROMFieldShape *romField = fieldID->field;
 		if (NULL == romField) {
 			vmFuncs->setCurrentExceptionUTF(currentThread, J9VMCONSTANTPOOL_JAVALANGINTERNALERROR, NULL);
 		} else if (J9_ARE_ANY_BITS_SET(romField->modifiers, J9AccStatic)) {
@@ -846,6 +832,9 @@ Java_jdk_internal_misc_Unsafe_objectFieldOffset1(JNIEnv *env, jobject receiver,
 
 	return offset;
 }
+#endif /* JAVA_SPEC_VERSION > 8 */
+
+#if JAVA_SPEC_VERSION >= 14
 
 /**
  * Writes modified memory in an address range from cache to main memory.
@@ -879,19 +868,19 @@ Java_jdk_internal_misc_Unsafe_writebackMemory(JNIEnv *env, jobject receiver, jlo
 
 		VM_AtomicSupport::readWriteBarrier();
 		switch (vm->cpuCacheWritebackCapabilities) {
-		case J9PORT_X86_FEATURE_CLWB:
+		case OMR_FEATURE_X86_CLWB:
 			do {
 				cacheLine += cacheLineSize;
 				asm volatile("clwb %0" : "+m" (*(U_8 *)cacheLine));
 			} while (lastCacheLine != cacheLine);
 			break;
-		case J9PORT_X86_FEATURE_CLFLUSHOPT:
+		case OMR_FEATURE_X86_CLFLUSHOPT:
 			do {
 				cacheLine += cacheLineSize;
 				asm volatile("clflushopt %0" : "+m" (*(U_8 *)cacheLine));
 			} while (lastCacheLine != cacheLine);
 			break;
-		case J9PORT_X86_FEATURE_CLFSH:
+		case OMR_FEATURE_X86_CLFSH:
 			do {
 				cacheLine += cacheLineSize;
 				asm volatile("clflush %0" : "+m" (*(U_8 *)cacheLine));
@@ -904,7 +893,7 @@ Java_jdk_internal_misc_Unsafe_writebackMemory(JNIEnv *env, jobject receiver, jlo
 		return;
 	}
 error:
-#endif /* x86 */
+#endif /* (defined(J9X86) || defined(J9HAMMER)) && !defined(WIN32) */
 
 	jclass exceptionClass = env->FindClass("java/lang/RuntimeException");
 	if (NULL == exceptionClass) {
@@ -929,189 +918,163 @@ Java_jdk_internal_misc_Unsafe_isWritebackEnabled(JNIEnv *env, jclass clazz)
 	J9JavaVM *vm = currentThread->javaVM;
 	if (vm->dCacheLineSize > 0) {
 		switch (vm->cpuCacheWritebackCapabilities) {
-		case J9PORT_X86_FEATURE_CLWB:
-		case J9PORT_X86_FEATURE_CLFLUSHOPT:
-		case J9PORT_X86_FEATURE_CLFSH:
+		case OMR_FEATURE_X86_CLFLUSHOPT:
+		case OMR_FEATURE_X86_CLFSH:
+		case OMR_FEATURE_X86_CLWB:
 			result = JNI_TRUE;
 			break;
 		default:
 			break;
 		}
 	}
-#endif /* x86 */
+#endif /* (defined(J9X86) || defined(J9HAMMER)) && !defined(WIN32) */
 	return result;
 }
 
-/* register jdk.internal.misc.Unsafe natives common to Java 9, 10 and beyond */
-static void
-registerJdkInternalMiscUnsafeNativesCommon(JNIEnv *env, jclass clazz) {
-	/* clazz can't be null */
-	JNINativeMethod natives[] = {
-		{
-			(char*)"defineClass0",
-			(char*)"(Ljava/lang/String;[BIILjava/lang/ClassLoader;Ljava/security/ProtectionDomain;)Ljava/lang/Class;",
-			(void*)&Java_sun_misc_Unsafe_defineClass__Ljava_lang_String_2_3BIILjava_lang_ClassLoader_2Ljava_security_ProtectionDomain_2
-		},
+#endif /* JAVA_SPEC_VERSION >= 14 */
+
+#if JAVA_SPEC_VERSION >= 9
+
+static JNINativeMethod nativeMethods[] = {
+	{
+		(char *)"defineClass0",
+		(char *)"(Ljava/lang/String;[BIILjava/lang/ClassLoader;Ljava/security/ProtectionDomain;)Ljava/lang/Class;",
+		(void *)&Java_sun_misc_Unsafe_defineClass__Ljava_lang_String_2_3BIILjava_lang_ClassLoader_2Ljava_security_ProtectionDomain_2
+	},
 #if JAVA_SPEC_VERSION < 17
-		{
-			(char*)"defineAnonymousClass0",
-			(char*)"(Ljava/lang/Class;[B[Ljava/lang/Object;)Ljava/lang/Class;",
-			(void*)&Java_sun_misc_Unsafe_defineAnonymousClass
-		},
+	{
+		(char *)"defineAnonymousClass0",
+		(char *)"(Ljava/lang/Class;[B[Ljava/lang/Object;)Ljava/lang/Class;",
+		(void *)&Java_sun_misc_Unsafe_defineAnonymousClass
+	},
 #endif /* JAVA_SPEC_VERSION < 17 */
-		{
-			(char*)"pageSize",
-			(char*)"()I",
-			(void*)&Java_sun_misc_Unsafe_pageSize
-		},
-		{
-			(char*)"getLoadAverage0",
-			(char*)"([DI)I",
-			(void*)&Java_sun_misc_Unsafe_getLoadAverage
-		},
-		{
-			(char*)"shouldBeInitialized0",
-			(char*)"(Ljava/lang/Class;)Z",
-			(void*)&Java_sun_misc_Unsafe_shouldBeInitialized
-		},
-		{
-			(char*)"allocateMemory0",
-			(char*)"(J)J",
-			(void*)&Java_sun_misc_Unsafe_allocateMemory
-		},
-		{
-			(char*)"freeMemory0",
-			(char*)"(J)V",
-			(void*)&Java_sun_misc_Unsafe_freeMemory
-		},
-		{
-			(char*)"reallocateMemory0",
-			(char*)"(JJ)J",
-			(void*)&Java_sun_misc_Unsafe_reallocateMemory
-		},
-		{
-			(char*)"ensureClassInitialized0",
-			(char*)"(Ljava/lang/Class;)V",
-			(void*)&Java_sun_misc_Unsafe_ensureClassInitialized
-		},
-		{
-			(char*)"park",
-			(char*)"(ZJ)V",
-			(void*)&Java_sun_misc_Unsafe_park
-		},
-		{
-			(char*)"unpark",
-			(char*)"(Ljava/lang/Object;)V",
-			(void*)&Java_sun_misc_Unsafe_unpark
-		},
-		{
-			(char*)"throwException",
-			(char*)"(Ljava/lang/Throwable;)V",
-			(void*)&Java_sun_misc_Unsafe_throwException
-		},
-		{
-			(char*)"copyMemory0",
-			(char*)"(Ljava/lang/Object;JLjava/lang/Object;JJ)V",
-			(void*)&Java_sun_misc_Unsafe_copyMemory__Ljava_lang_Object_2JLjava_lang_Object_2JJ
-		},
-		{
-			(char*)"objectFieldOffset0",
-			(char*)"(Ljava/lang/reflect/Field;)J",
-			(void*)&Java_sun_misc_Unsafe_objectFieldOffset
-		},
-		{
-			(char*)"setMemory0",
-			(char*)"(Ljava/lang/Object;JJB)V",
-			(void*)&Java_sun_misc_Unsafe_setMemory__Ljava_lang_Object_2JJB
-		},
-		{
-			(char*)"staticFieldBase0",
-			(char*)"(Ljava/lang/reflect/Field;)Ljava/lang/Object;",
-			(void*)&Java_sun_misc_Unsafe_staticFieldBase__Ljava_lang_reflect_Field_2
-		},
-		{
-			(char*)"staticFieldOffset0",
-			(char*)"(Ljava/lang/reflect/Field;)J",
-			(void*)&Java_sun_misc_Unsafe_staticFieldOffset
-		},
-		{
-			(char*)"unalignedAccess0",
-			(char*)"()Z",
-			(void*)&Java_sun_misc_Unsafe_unalignedAccess0
-		},
-		{
-			(char*)"isBigEndian0",
-			(char*)"()Z",
-			(void*)&Java_sun_misc_Unsafe_isBigEndian0
-		},
-		{
-			(char*)"getUncompressedObject",
-			(char*)"(J)Ljava/lang/Object;",
-			(void*)&Java_sun_misc_Unsafe_getUncompressedObject
-		},
-	};
-	jint numNatives = sizeof(natives)/sizeof(JNINativeMethod);
-	env->RegisterNatives(clazz, natives, numNatives);
-#if defined(J9VM_OPT_JAVA_OFFLOAD_SUPPORT)
-	clearNonZAAPEligibleBit(env, clazz, natives, numNatives);
-#endif /* J9VM_OPT_JAVA_OFFLOAD_SUPPORT */
-}
+	{
+		(char *)"pageSize",
+		(char *)"()I",
+		(void *)&Java_sun_misc_Unsafe_pageSize
+	},
+	{
+		(char *)"getLoadAverage0",
+		(char *)"([DI)I",
+		(void *)&Java_sun_misc_Unsafe_getLoadAverage
+	},
+	{
+		(char *)"shouldBeInitialized0",
+		(char *)"(Ljava/lang/Class;)Z",
+		(void *)&Java_sun_misc_Unsafe_shouldBeInitialized
+	},
+	{
+		(char *)"allocateMemory0",
+		(char *)"(J)J",
+		(void *)&Java_sun_misc_Unsafe_allocateMemory
+	},
+	{
+		(char *)"freeMemory0",
+		(char *)"(J)V",
+		(void *)&Java_sun_misc_Unsafe_freeMemory
+	},
+	{
+		(char *)"reallocateMemory0",
+		(char *)"(JJ)J",
+		(void *)&Java_sun_misc_Unsafe_reallocateMemory
+	},
+	{
+		(char *)"ensureClassInitialized0",
+		(char *)"(Ljava/lang/Class;)V",
+		(void *)&Java_sun_misc_Unsafe_ensureClassInitialized
+	},
+	{
+		(char *)"park",
+		(char *)"(ZJ)V",
+		(void *)&Java_sun_misc_Unsafe_park
+	},
+	{
+		(char *)"unpark",
+		(char *)"(Ljava/lang/Object;)V",
+		(void *)&Java_sun_misc_Unsafe_unpark
+	},
+	{
+		(char *)"throwException",
+		(char *)"(Ljava/lang/Throwable;)V",
+		(void *)&Java_sun_misc_Unsafe_throwException
+	},
+	{
+		(char *)"copyMemory0",
+		(char *)"(Ljava/lang/Object;JLjava/lang/Object;JJ)V",
+		(void *)&Java_sun_misc_Unsafe_copyMemory__Ljava_lang_Object_2JLjava_lang_Object_2JJ
+	},
+	{
+		(char *)"objectFieldOffset0",
+		(char *)"(Ljava/lang/reflect/Field;)J",
+		(void *)&Java_sun_misc_Unsafe_objectFieldOffset
+	},
+	{
+		(char *)"setMemory0",
+		(char *)"(Ljava/lang/Object;JJB)V",
+		(void *)&Java_sun_misc_Unsafe_setMemory__Ljava_lang_Object_2JJB
+	},
+	{
+		(char *)"staticFieldBase0",
+		(char *)"(Ljava/lang/reflect/Field;)Ljava/lang/Object;",
+		(void *)&Java_sun_misc_Unsafe_staticFieldBase__Ljava_lang_reflect_Field_2
+	},
+	{
+		(char *)"staticFieldOffset0",
+		(char *)"(Ljava/lang/reflect/Field;)J",
+		(void *)&Java_sun_misc_Unsafe_staticFieldOffset
+	},
+	{
+		(char *)"unalignedAccess0",
+		(char *)"()Z",
+		(void *)&Java_sun_misc_Unsafe_unalignedAccess0
+	},
+	{
+		(char *)"isBigEndian0",
+		(char *)"()Z",
+		(void *)&Java_sun_misc_Unsafe_isBigEndian0
+	},
+	{
+		(char *)"getUncompressedObject",
+		(char *)"(J)Ljava/lang/Object;",
+		(void *)&Java_sun_misc_Unsafe_getUncompressedObject
+	},
+#if JAVA_SPEC_VERSION >= 10
+	{
+		(char *)"objectFieldOffset1",
+		(char *)"(Ljava/lang/Class;Ljava/lang/String;)J",
+		(void *)&Java_jdk_internal_misc_Unsafe_objectFieldOffset1
+	},
+#endif /* JAVA_SPEC_VERSION >= 10 */
+#if JAVA_SPEC_VERSION >= 14
+	{
+		(char *)"writebackMemory",
+		(char *)"(JJ)V",
+		(void *)&Java_jdk_internal_misc_Unsafe_writebackMemory
+	},
+	{
+		(char *)"isWritebackEnabled",
+		(char *)"()Z",
+		(void *)&Java_jdk_internal_misc_Unsafe_isWritebackEnabled
+	},
+#endif /* JAVA_SPEC_VERSION >= 14 */
+};
 
-/* register jdk.internal.misc.Unsafe natives for Java 10 */
-static void
-registerJdkInternalMiscUnsafeNativesJava10(JNIEnv *env, jclass clazz) {
-	/* clazz can't be null */
-	JNINativeMethod natives[] = {
-		{
-			(char*)"objectFieldOffset1",
-			(char*)"(Ljava/lang/Class;Ljava/lang/String;)J",
-			(void*)&Java_jdk_internal_misc_Unsafe_objectFieldOffset1
-		}
-	};
-	jint numNatives = sizeof(natives)/sizeof(JNINativeMethod);
-	env->RegisterNatives(clazz, natives, numNatives);
-#if defined(J9VM_OPT_JAVA_OFFLOAD_SUPPORT)
-	clearNonZAAPEligibleBit(env, clazz, natives, numNatives);
-#endif /* J9VM_OPT_JAVA_OFFLOAD_SUPPORT */
-}
-
-/* register jdk.internal.misc.Unsafe natives for Java 14 */
-static void
-registerJdkInternalMiscUnsafeNativesJava14(JNIEnv *env, jclass clazz) {
-	/* clazz can't be null */
-	JNINativeMethod natives[] = {
-		{
-			(char*)"writebackMemory",
-			(char*)"(JJ)V",
-			(void*)&Java_jdk_internal_misc_Unsafe_writebackMemory
-		},
-		{
-			(char*)"isWritebackEnabled",
-			(char*)"()Z",
-			(void*)&Java_jdk_internal_misc_Unsafe_isWritebackEnabled
-		}
-	};
-	jint numNatives = sizeof(natives)/sizeof(JNINativeMethod);
-	env->RegisterNatives(clazz, natives, numNatives);
-#if defined(J9VM_OPT_JAVA_OFFLOAD_SUPPORT)
-	clearNonZAAPEligibleBit(env, clazz, natives, numNatives);
-#endif /* J9VM_OPT_JAVA_OFFLOAD_SUPPORT */
-}
-
-/* class jdk.internal.misc.Unsafe only presents in Java 9 and beyond */
+/* Class jdk.internal.misc.Unsafe is only present in Java 9 and beyond. */
 void JNICALL
 Java_jdk_internal_misc_Unsafe_registerNatives(JNIEnv *env, jclass clazz)
 {
+	jint numNatives = sizeof(nativeMethods) / sizeof(nativeMethods[0]);
+
+	/* clazz can't be null */
 	Java_sun_misc_Unsafe_registerNatives(env, clazz);
-	registerJdkInternalMiscUnsafeNativesCommon(env, clazz);
-	if (JAVA_SPEC_VERSION >= 10) {
-		registerJdkInternalMiscUnsafeNativesJava10(env, clazz);
-	}
-	if (JAVA_SPEC_VERSION >= 14) {
-		registerJdkInternalMiscUnsafeNativesJava14(env, clazz);
-	}
+	env->RegisterNatives(clazz, nativeMethods, numNatives);
+#if defined(J9VM_OPT_JAVA_OFFLOAD_SUPPORT)
+	clearNonZAAPEligibleBit(env, clazz, nativeMethods, numNatives);
+#endif /* defined(J9VM_OPT_JAVA_OFFLOAD_SUPPORT) */
 }
 
+#endif /* JAVA_SPEC_VERSION >= 9 */
+
 /*
  * Determine if memory addresses overlap.
  * Will always return false if either object is an array.
@@ -1124,11 +1087,12 @@ Java_jdk_internal_misc_Unsafe_registerNatives(JNIEnv *env, jclass clazz)
  * @return true for no memory overlap, otherwise false
  */
 jboolean
- memOverlapIsNone(j9object_t sourceObject, UDATA sourceOffset, j9object_t destObject, UDATA destOffset, UDATA actualCopySize) {
+memOverlapIsNone(j9object_t sourceObject, UDATA sourceOffset, j9object_t destObject, UDATA destOffset, UDATA actualCopySize)
+{
 	jboolean result = JNI_FALSE;
 	if (sourceObject != destObject) {
 		result = JNI_TRUE;
-	} else if ((sourceObject == NULL) && (destObject == NULL)) {
+	} else if ((NULL == sourceObject) && (NULL == destObject)) {
 		if (sourceOffset > (destOffset + actualCopySize)) {
 			result = JNI_TRUE;
 		} else if (destOffset > (sourceOffset + actualCopySize)) {
@@ -1148,27 +1112,28 @@ jboolean
  * @return true for no memory overlap, otherwise false
  */
 jboolean
- memOverlapIsUnaligned(UDATA sourceOffset, UDATA destOffset) {
-	return (sourceOffset != destOffset);
+memOverlapIsUnaligned(UDATA sourceOffset, UDATA destOffset)
+{
+	return sourceOffset != destOffset;
 }
 
 void JNICALL
 Java_jdk_internal_misc_Unsafe_copySwapMemory0(JNIEnv *env, jobject receiver, jobject srcBase, jlong srcOffset,
 		jobject dstBase, jlong dstOffset, jlong copySize, jlong elemSize)
 {
-	J9VMThread *currentThread = (J9VMThread*)env;
+	J9VMThread *currentThread = (J9VMThread *)env;
 	J9JavaVM *vm = currentThread->javaVM;
 	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
-	UDATA sourceOffset = (UDATA)srcOffset;
-	UDATA destOffset = (UDATA)dstOffset;
+	UDATA sourceOffset = (UDATA)(U_64)srcOffset;
+	UDATA destOffset = (UDATA)(U_64)dstOffset;
 	vmFuncs->internalEnterVMFromJNI(currentThread);
-	UDATA actualCopySize = (UDATA)copySize;
-	UDATA actualElementSize = (UDATA)elemSize;
+	UDATA actualCopySize = (UDATA)(U_64)copySize;
+	UDATA actualElementSize = (UDATA)(U_64)elemSize;
 
-	if ((copySize < 0) || (copySize != (jlong)actualCopySize) || (elemSize != (jlong)actualElementSize)) {
+	if ((copySize < 0) || (copySize != (jlong)(IDATA)actualCopySize) || (elemSize != (jlong)(IDATA)actualElementSize)) {
 illegal:
 		vmFuncs->setCurrentExceptionUTF(currentThread, J9VMCONSTANTPOOL_JAVALANGILLEGALARGUMENTEXCEPTION, NULL);
-	} else if ( ! (2 == elemSize || 4 == elemSize || 8 == elemSize)) {
+	} else if (!((2 == elemSize) || (4 == elemSize) || (8 == elemSize))) {
 		/* verify that element size is supported for swapping */
 		goto illegal;
 	} else if (0 != (copySize % elemSize)) {
@@ -1183,7 +1148,7 @@ illegal:
 			if (!J9CLASS_IS_ARRAY(clazz)) {
 				goto illegal;
 			}
-			if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass*)clazz)->componentType->romClass)) {
+			if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass *)clazz)->componentType->romClass)) {
 				goto illegal;
 			}
 		}
@@ -1193,7 +1158,7 @@ illegal:
 			if (!J9CLASS_IS_ARRAY(clazz)) {
 				goto illegal;
 			}
-			if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass*)clazz)->componentType->romClass)) {
+			if (!J9ROMCLASS_IS_PRIMITIVE_TYPE(((J9ArrayClass *)clazz)->componentType->romClass)) {
 				goto illegal;
 			}
 		}
