@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2019 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include <stdlib.h>
@@ -78,10 +78,10 @@ UDATA signalProtectedMain(struct J9PortLibrary *portLibrary, void *arg)
 	PORT_ACCESS_FROM_PORT(args->portLibrary);
 
 	if (argc <= 1) {
-		j9file_printf(PORTLIB, J9PORT_TTY_OUT, "jniinv test program\n");
-		j9file_printf(PORTLIB, J9PORT_TTY_OUT, J9_COPYRIGHT_STRING "\n");
-		j9file_printf(PORTLIB, J9PORT_TTY_OUT, "usage: jniinv [-D... [-X...]] [-io:\n");
-		j9file_printf(PORTLIB, J9PORT_TTY_OUT, "minimally, you need to used -Djava.home=..\n");
+		j9file_printf(J9PORT_TTY_OUT, "jniinv test program\n");
+		j9file_printf(J9PORT_TTY_OUT, J9_COPYRIGHT_STRING "\n");
+		j9file_printf(J9PORT_TTY_OUT, "usage: jniinv [-D... [-X...]] [-io:\n");
+		j9file_printf(J9PORT_TTY_OUT, "minimally, you need to used -Djava.home=..\n");
 		return 0;
 	}
 
