@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "jvmtiHelpers.h"
@@ -98,6 +98,11 @@ dumpCapabilities(J9JavaVM * vm, const jvmtiCapabilities *capabilities, const cha
 #if JAVA_SPEC_VERSION >= 11
 	PRINT_CAPABILITY(can_generate_sampled_object_alloc_events);
 #endif /* JAVA_SPEC_VERSION >= 11 */
+
+	/* JVMTI 19 */
+#if JAVA_SPEC_VERSION >= 19
+	PRINT_CAPABILITY(can_support_virtual_threads);
+#endif /* JAVA_SPEC_VERSION >= 19 */
 #undef PRINT_CAPABILITY
 }
 
@@ -110,7 +115,8 @@ jvmtiGetPotentialCapabilities(jvmtiEnv* env, jvmtiCapabilities* capabilities_ptr
 	J9JavaVM * vm = j9env->vm;
 	J9JVMTIData * jvmtiData = J9JVMTI_DATA_FROM_VM(vm);
 	jvmtiCapabilities rv_capabilities;
-	J9HookInterface ** vmHook = vm->internalVMFunctions->getVMHookInterface(vm);
+	J9InternalVMFunctions *vmFuncs = vm->internalVMFunctions;
+	J9HookInterface **vmHook = vmFuncs->getVMHookInterface(vm);
 	jvmtiError rc;
 
 	Trc_JVMTI_jvmtiGetPotentialCapabilities_Entry(env);
@@ -182,6 +188,14 @@ jvmtiGetPotentialCapabilities(jvmtiEnv* env, jvmtiCapabilities* capabilities_ptr
 	}
 #endif /* JAVA_SPEC_VERSION >= 11 */
 
+#if JAVA_SPEC_VERSION >= 19
+	if (isEventHookable(j9env, JVMTI_EVENT_VIRTUAL_THREAD_START)
+	&& isEventHookable(j9env, JVMTI_EVENT_VIRTUAL_THREAD_END)
+	) {
+		rv_capabilities.can_support_virtual_threads = 1;
+	}
+#endif /* JAVA_SPEC_VERSION >= 19 */
+
 	if (isEventHookable(j9env, JVMTI_EVENT_NATIVE_METHOD_BIND)) {
 		rv_capabilities.can_generate_native_method_bind_events = 1;
 	}
@@ -193,8 +207,13 @@ jvmtiGetPotentialCapabilities(jvmtiEnv* env, jvmtiCapabilities* capabilities_ptr
 	}
 
 	if ((*vmHook)->J9HookIsEnabled(vmHook, J9HOOK_VM_POP_FRAMES_INTERRUPT)) {
-		rv_capabilities.can_pop_frame = 1;
-		rv_capabilities.can_force_early_return = 1;
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+		if (!vmFuncs->isDebugAgentDisabled(vm))
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+		{
+			rv_capabilities.can_pop_frame = 1;
+			rv_capabilities.can_force_early_return = 1;
+		}
 	}
 
 	if ((*vmHook)->J9HookIsEnabled(vmHook, J9HOOK_VM_REQUIRED_DEBUG_ATTRIBUTES) ||
@@ -218,7 +237,12 @@ jvmtiGetPotentialCapabilities(jvmtiEnv* env, jvmtiCapabilities* capabilities_ptr
 	if ((*vmHook)->J9HookIsEnabled(vmHook, J9HOOK_VM_REQUIRED_DEBUG_ATTRIBUTES) ||
 		(vm->requiredDebugAttributes & J9VM_DEBUG_ATTRIBUTE_CAN_ACCESS_LOCALS)
 	) {
-		rv_capabilities.can_access_local_variables = 1;
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+		if (!vmFuncs->isDebugAgentDisabled(vm))
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+		{
+			rv_capabilities.can_access_local_variables = 1;
+		}
 	}
 
 	rv_capabilities.can_tag_objects = 1;
@@ -351,7 +375,9 @@ jvmtiAddCapabilities(jvmtiEnv* env,
 
 			/* Make sure all of the requested capabilities are available */
 
-			if ((byte & ~(((U_8 *) &potentialCapabilities)[i])) != 0) {
+			if (0 != (byte & ~(((U_8 *) &potentialCapabilities)[i]))) {
+				Trc_JVMTI_jvmtiAddCapabilities_no_capability(currentThread,
+					i, ((U_8 *) &potentialCapabilities)[i], (byte & ~(((U_8 *) &potentialCapabilities)[i])));
 				goto fail;
 			}
 
@@ -369,6 +395,7 @@ jvmtiAddCapabilities(jvmtiEnv* env,
 					vm->requiredDebugAttributes |= J9VM_DEBUG_ATTRIBUTE_MAINTAIN_ORIGINAL_METHOD_ORDER;
 				} else {
 					rc = JVMTI_ERROR_NOT_AVAILABLE;
+					Trc_JVMTI_jvmtiAddCapabilities_not_onload(currentThread);
 					goto fail;
 				}
 			}
@@ -378,6 +405,7 @@ jvmtiAddCapabilities(jvmtiEnv* env,
 		if (newCapabilities.can_generate_sampled_object_alloc_events) {
 			if (J9_ARE_ANY_BITS_SET(jvmtiData->flags, J9JVMTI_FLAG_SAMPLED_OBJECT_ALLOC_ENABLED)) {
 				rc = JVMTI_ERROR_NOT_AVAILABLE;
+				Trc_JVMTI_jvmtiAddCapabilities_object_alloc_set_already(currentThread);
 				goto fail;
 			}
 
@@ -398,13 +426,15 @@ jvmtiAddCapabilities(jvmtiEnv* env,
 
 		/* Reserve hooks for any events now allowed by the new capabilities */
 
-		if (mapCapabilitiesToEvents(j9env, &newCapabilities, reserveEvent) != 0) {
+		if (0 != mapCapabilitiesToEvents(j9env, &newCapabilities, reserveEvent)) {
+			Trc_JVMTI_jvmtiAddCapabilities_mapCapabilitiesToEvents_failed(currentThread);
 			goto fail;
 		}
 
 		/* Handle non-event hooks */
 
-		if (hookNonEventCapabilities(j9env, &newCapabilities) != 0) {
+		if (0 != hookNonEventCapabilities(j9env, &newCapabilities)) {
+			Trc_JVMTI_jvmtiAddCapabilities_hookNonEventCapabilities_failed(currentThread);
 			goto fail;
 		}
 
@@ -527,55 +557,78 @@ mapCapabilitiesToEvents(J9JVMTIEnv * j9env, jvmtiCapabilities * capabilities, J9
 {
 	IDATA rc = 0;
 
-	if (capabilities->can_generate_single_step_events) {
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_SINGLE_STEP);
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+	J9JavaVM *vm = j9env->vm;
+	/* Skip J9HookReserve for the events required by JDWP agent pre-checkpoint when isDebugAgentDisabled() returns TRUE,
+	 * these events will be registered post-restore if a JDWP agent is specified in the restore option file,
+	 * otherwise they are going to be unregistered by J9HookUnregister() which only clears J9HOOK_FLAG_HOOKED,
+	 * but not J9HOOK_FLAG_RESERVED.
+	 * J9HookUnreserve() might clear the flag set by other callers.
+	 */
+	if (!vm->internalVMFunctions->isDebugAgentDisabled(vm))
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT)*/
+	{
+		if (capabilities->can_generate_single_step_events) {
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_SINGLE_STEP);
+		}
+		if (capabilities->can_generate_breakpoint_events) {
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_BREAKPOINT);
+		}
+		if (capabilities->can_generate_field_access_events) {
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_FIELD_ACCESS);
+		}
+		if (capabilities->can_generate_field_modification_events) {
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_FIELD_MODIFICATION);
+		}
+		if (capabilities->can_generate_frame_pop_events) {
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_FRAME_POP);
+		}
+		if (capabilities->can_generate_monitor_events) {
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_MONITOR_CONTENDED_ENTER);
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_MONITOR_CONTENDED_ENTERED);
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_MONITOR_WAIT);
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_MONITOR_WAITED);
+		}
+#if JAVA_SPEC_VERSION >= 19
+		if (capabilities->can_support_virtual_threads) {
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_VIRTUAL_THREAD_START);
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_VIRTUAL_THREAD_END);
+		}
+#endif /* JAVA_SPEC_VERSION >= 19 */
+		if (capabilities->can_generate_garbage_collection_events) {
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_GARBAGE_COLLECTION_START);
+			rc |= eventHookFunction(j9env, JVMTI_EVENT_GARBAGE_COLLECTION_FINISH);
+		}
 	}
-
-	if (capabilities->can_generate_breakpoint_events) {
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_BREAKPOINT);
+	/* CRIU can't skip J9HookReserve for the following events when they are requested via
+	 * jvmtiAddCapabilities(), otherwise, the corresponding hooks will be disabled by JIT.
+	 * For example, can_generate_method_exit_events/JVMTI_EVENT_METHOD_EXIT related hooks
+	 * are J9HOOK_VM_METHOD_RETURN/J9HOOK_VM_NATIVE_METHOD_RETURN, the hook
+	 * J9HOOK_VM_METHOD_RETURN is to be disabled if not reserved in advance.
+	 * Similarly for the other events, if they are not reserved and a jdwp agent is
+	 * specified at the restore option file, the capability can't be added,
+	 * and the debugger won't be able to run.
+	 */
+	if (capabilities->can_generate_method_exit_events) {
+		rc |= eventHookFunction(j9env, JVMTI_EVENT_METHOD_EXIT);
 	}
-
-	if (capabilities->can_generate_field_access_events) {
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_FIELD_ACCESS);
-	}
-
-	if (capabilities->can_generate_field_modification_events) {
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_FIELD_MODIFICATION);
-	}
-
-	if (capabilities->can_generate_frame_pop_events) {
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_FRAME_POP);
-	}
-
 	if (capabilities->can_generate_method_entry_events) {
 		rc |= eventHookFunction(j9env, JVMTI_EVENT_METHOD_ENTRY);
 	}
-
-	if (capabilities->can_generate_method_exit_events) {
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_METHOD_EXIT);
+	if (capabilities->can_generate_exception_events) {
+		rc |= eventHookFunction(j9env, JVMTI_EVENT_EXCEPTION);
+		rc |= eventHookFunction(j9env, JVMTI_EVENT_EXCEPTION_CATCH);
 	}
 
 	if (capabilities->can_generate_native_method_bind_events) {
 		rc |= eventHookFunction(j9env, JVMTI_EVENT_NATIVE_METHOD_BIND);
 	}
 
-	if (capabilities->can_generate_exception_events) {
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_EXCEPTION);
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_EXCEPTION_CATCH);
-	}
-
 	if (capabilities->can_generate_compiled_method_load_events) {
 		rc |= eventHookFunction(j9env, JVMTI_EVENT_COMPILED_METHOD_LOAD);
 		rc |= eventHookFunction(j9env, JVMTI_EVENT_COMPILED_METHOD_UNLOAD);
 	}
 
-	if (capabilities->can_generate_monitor_events) {
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_MONITOR_CONTENDED_ENTER);
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_MONITOR_CONTENDED_ENTERED);
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_MONITOR_WAIT);
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_MONITOR_WAITED);
-	}
-
 	if (capabilities->can_generate_vm_object_alloc_events) {
 		rc |= eventHookFunction(j9env, JVMTI_EVENT_VM_OBJECT_ALLOC);
 	}
@@ -590,11 +643,6 @@ mapCapabilitiesToEvents(J9JVMTIEnv * j9env, jvmtiCapabilities * capabilities, J9
 		rc |= eventHookFunction(j9env, JVMTI_EVENT_OBJECT_FREE);
 	}
 
-	if (capabilities->can_generate_garbage_collection_events) {
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_GARBAGE_COLLECTION_START);
-		rc |= eventHookFunction(j9env, JVMTI_EVENT_GARBAGE_COLLECTION_FINISH);
-	}
-
 	if (capabilities->can_generate_resource_exhaustion_heap_events ||
 		capabilities->can_generate_resource_exhaustion_threads_events) {
 		rc |= eventHookFunction(j9env, JVMTI_EVENT_RESOURCE_EXHAUSTED);	
@@ -602,5 +650,3 @@ mapCapabilitiesToEvents(J9JVMTIEnv * j9env, jvmtiCapabilities * capabilities, J9
 
 	return rc;
 }
-
-
