@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #if !defined(OBJECTMONITOR_HPP_)
@@ -262,6 +262,9 @@ done:
 		if (lock == compareAndSwapLockword(currentThread, lockEA, lock, mine, readBeforeCAS)) {
 			VM_AtomicSupport::readBarrier();
 			locked = true;
+#if JAVA_SPEC_VERSION >= 19
+			currentThread->ownedMonitorCount += 1;
+#endif /* JAVA_SPEC_VERSION >= 19 */
 		}
 		return locked;
 	}
@@ -307,6 +310,9 @@ done:
 				VM_AtomicSupport::writeBarrier();
 				J9_STORE_LOCKWORD(currentThread, lockEA, 0);
 				unlocked = true;
+#if JAVA_SPEC_VERSION >= 19
+				currentThread->ownedMonitorCount -= 1;
+#endif /* JAVA_SPEC_VERSION >= 19 */
 			}
 		}
 		return unlocked;
