@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef CFR_H
@@ -241,6 +241,9 @@ typedef struct J9CfrAttribute {
 #define CFR_ATTRIBUTE_NestHost 26
 #define CFR_ATTRIBUTE_Record 27
 #define CFR_ATTRIBUTE_PermittedSubclasses 28
+#define CFR_ATTRIBUTE_LoadableDescriptors 29
+#define CFR_ATTRIBUTE_ImplicitCreation 30
+#define CFR_ATTRIBUTE_NullRestricted 31
 #define CFR_ATTRIBUTE_StrippedLocalVariableTypeTable  122
 #define CFR_ATTRIBUTE_StrippedSourceDebugExtension  123
 #define CFR_ATTRIBUTE_StrippedInnerClasses  124
@@ -531,6 +534,33 @@ typedef struct J9CfrAttributePermittedSubclasses {
     U_16* classes;
 } J9CfrAttributePermittedSubclasses;
 
+#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
+typedef struct J9CfrAttributeLoadableDescriptors {
+    U_8 tag;
+    U_16 nameIndex;
+    U_32 length;
+    UDATA romAddress;
+    U_16 numberOfDescriptors;
+    U_16 *descriptors;
+} J9CfrAttributeLoadableDescriptors;
+#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
+#if defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES)
+typedef struct J9CfrAttributeImplicitCreation {
+    U_8 tag;
+    U_16 nameIndex;
+    U_32 length;
+    UDATA romAddress;
+    U_16 implicitCreationFlags;
+} J9CfrAttributeImplicitCreation;
+
+typedef struct J9CfrAttributeNullRestricted {
+    U_8 tag;
+    U_16 nameIndex;
+    U_32 length;
+    UDATA romAddress;
+} J9CfrAttributeNullRestricted;
+#endif /* defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES) */
+
 /* @ddr_namespace: map_to_type=J9CfrConstantPoolInfo */
 
 typedef struct J9CfrConstantPoolInfo {
@@ -807,13 +837,7 @@ typedef struct J9CfrMethod {
 #define CFR_BC_goto_w 200
 #define CFR_BC_jsr_w 201
 #define CFR_BC_breakpoint 202  			/* Reserved opcodes */
-#define CFR_BC_aconst_init 203
-#define CFR_BC_withfield 204
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-#define CFR_BC_MaxDefined CFR_BC_withfield
-#else /* J9VM_OPT_VALHALLA_VALUE_TYPES */
 #define CFR_BC_MaxDefined CFR_BC_jsr_w
-#endif /* J9VM_OPT_VALHALLA_VALUE_TYPES */
 #define CFR_BC_impdep1 254
 #define CFR_BC_impdep2 255
 #define CFR_BC_invokehandle 232 		/* JSR 292 internals */
@@ -853,18 +877,17 @@ typedef struct J9CfrClassFile {
 #define CFR_ACC_STATIC  					0x00000008
 #define CFR_ACC_FINAL  						0x00000010
 #define CFR_ACC_SUPER  						0x00000020
+/* From the spec, CFR_ACC_SUPER semantics is mandatory since Java 8 and the bit takes no effect, so it is repurposed as CFR_ACC_IDENTITY in Valhalla */
+#define CFR_ACC_IDENTITY  					0x00000020
 #define CFR_ACC_SYNCHRONIZED  				0x00000020
 #define CFR_ACC_BRIDGE  					0x00000040
 #define CFR_ACC_VOLATILE  					0x00000040
-#define CFR_ACC_PERMITS_VALUE  					0x00000040
 #define CFR_ACC_TRANSIENT  					0x00000080
-#define CFR_ACC_VALUE_TYPE					0x00000100
 #define CFR_ACC_VARARGS  					0x00000080
 #define CFR_ACC_NATIVE  					0x00000100
 #define CFR_ACC_INTERFACE  					0x00000200
 #define CFR_ACC_ABSTRACT  					0x00000400
 #define CFR_ACC_STRICT  					0x00000800
-#define CFR_ACC_PRIMITIVE_VALUE_TYPE  		0x00000800
 #define CFR_ACC_SYNTHETIC  					0x00001000
 #define CFR_ACC_ANNOTATION 					0x00002000
 #define CFR_ACC_ENUM  						0x00004000
@@ -891,24 +914,23 @@ typedef struct J9CfrClassFile {
 #define CFR_MAJOR_VERSION  45
 #define CFR_MINOR_VERSION  3
 #define CFR_PUBLIC_PRIVATE_PROTECTED_MASK	(CFR_ACC_PUBLIC | CFR_ACC_PRIVATE | CFR_ACC_PROTECTED)
+
+#define J9_IS_CLASSFILE_VALUETYPE(classfile) (J9_IS_CLASSFILE_OR_ROMCLASS_VALUETYPE_VERSION(classfile) && J9_ARE_NO_BITS_SET((classfile)->accessFlags, CFR_ACC_IDENTITY | CFR_ACC_INTERFACE))
+
 #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-#define CFR_CLASS_ACCESS_MASK					(CFR_ACC_PUBLIC | CFR_ACC_FINAL | CFR_ACC_SUPER | CFR_ACC_INTERFACE | CFR_ACC_ABSTRACT | CFR_ACC_SYNTHETIC | CFR_ACC_ANNOTATION | CFR_ACC_ENUM | CFR_ACC_VALUE_TYPE | CFR_ACC_PRIMITIVE_VALUE_TYPE | CFR_ACC_PERMITS_VALUE)
-#define J9_IS_CLASSFILE_VALUETYPE(classfile)     J9_ARE_ALL_BITS_SET((classfile)->accessFlags, CFR_ACC_VALUE_TYPE)
-#define J9_IS_CLASSFILE_PRIMITIVE_VALUETYPE(classfile)     J9_ARE_ALL_BITS_SET((classfile)->accessFlags, CFR_ACC_PRIMITIVE_VALUE_TYPE)
+#define CFR_CLASS_ACCESS_MASK					(CFR_ACC_PUBLIC | CFR_ACC_FINAL | CFR_ACC_IDENTITY | CFR_ACC_INTERFACE | CFR_ACC_ABSTRACT | CFR_ACC_SYNTHETIC | CFR_ACC_ANNOTATION | CFR_ACC_ENUM)
+#define CFR_FIELD_ACCESS_MASK  					(CFR_ACC_PUBLIC | CFR_ACC_PRIVATE | CFR_ACC_PROTECTED | CFR_ACC_STATIC | CFR_ACC_FINAL | CFR_ACC_VOLATILE | CFR_ACC_TRANSIENT | CFR_ACC_SYNTHETIC | CFR_ACC_ENUM | CFR_ACC_STRICT)
 #else /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
 #define CFR_CLASS_ACCESS_MASK					(CFR_ACC_PUBLIC | CFR_ACC_FINAL | CFR_ACC_SUPER | CFR_ACC_INTERFACE | CFR_ACC_ABSTRACT | CFR_ACC_SYNTHETIC | CFR_ACC_ANNOTATION | CFR_ACC_ENUM)
-#define J9_IS_CLASSFILE_VALUETYPE(classfile)     FALSE
-#define J9_IS_CLASSFILE_PRIMITIVE_VALUETYPE(classfile) FALSE
+#define CFR_FIELD_ACCESS_MASK  					(CFR_ACC_PUBLIC | CFR_ACC_PRIVATE | CFR_ACC_PROTECTED | CFR_ACC_STATIC | CFR_ACC_FINAL | CFR_ACC_VOLATILE | CFR_ACC_TRANSIENT | CFR_ACC_SYNTHETIC | CFR_ACC_ENUM)
 #endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
 #define CFR_INTERFACE_CLASS_ACCESS_MASK			(CFR_ACC_PUBLIC | CFR_ACC_INTERFACE | CFR_ACC_ABSTRACT | CFR_ACC_SYNTHETIC | CFR_ACC_ANNOTATION)
-#define CFR_FIELD_ACCESS_MASK  					(CFR_ACC_PUBLIC | CFR_ACC_PRIVATE | CFR_ACC_PROTECTED | CFR_ACC_STATIC | CFR_ACC_FINAL | CFR_ACC_VOLATILE | CFR_ACC_TRANSIENT | CFR_ACC_SYNTHETIC | CFR_ACC_ENUM)
 #define CFR_INTERFACE_FIELD_ACCESS_MASK  		(CFR_ACC_PUBLIC | CFR_ACC_STATIC | CFR_ACC_FINAL | CFR_ACC_SYNTHETIC)
 #define CFR_INTERFACE_FIELD_ACCESS_REQUIRED  	(CFR_ACC_PUBLIC | CFR_ACC_STATIC | CFR_ACC_FINAL)
 #define CFR_METHOD_ACCESS_MASK  				(CFR_ACC_PUBLIC | CFR_ACC_PRIVATE | CFR_ACC_PROTECTED | CFR_ACC_STATIC | CFR_ACC_FINAL | CFR_ACC_SYNCHRONIZED | CFR_ACC_BRIDGE | CFR_ACC_VARARGS | CFR_ACC_NATIVE | CFR_ACC_STRICT | CFR_ACC_ABSTRACT | CFR_ACC_SYNTHETIC)
 #define CFR_ABSTRACT_METHOD_ACCESS_MASK  		(CFR_ACC_PUBLIC | CFR_ACC_PROTECTED | CFR_ACC_BRIDGE | CFR_ACC_VARARGS | CFR_ACC_ABSTRACT | CFR_ACC_SYNTHETIC)
 #define CFR_INTERFACE_METHOD_ACCESS_MASK  		(CFR_ACC_PUBLIC | CFR_ACC_BRIDGE | CFR_ACC_VARARGS | CFR_ACC_ABSTRACT | CFR_ACC_SYNTHETIC)
 #define CFR_INTERFACE_METHOD_ACCESS_REQUIRED	(CFR_ACC_PUBLIC | CFR_ACC_ABSTRACT)
-#define CFR_INIT_VT_METHOD_ACCESS_MASK  (CFR_ACC_PUBLIC | CFR_ACC_PRIVATE | CFR_ACC_STATIC | CFR_ACC_SYNTHETIC | CFR_ACC_VARARGS | CFR_ACC_STRICT)
 #define CFR_INIT_METHOD_ACCESS_MASK		(CFR_ACC_PUBLIC | CFR_ACC_PRIVATE | CFR_ACC_PROTECTED | CFR_ACC_SYNTHETIC | CFR_ACC_VARARGS | CFR_ACC_STRICT)
 #define CFR_CLINIT_METHOD_ACCESS_MASK	(CFR_ACC_STRICT | CFR_ACC_STATIC)
 #define CFR_CLASS_ACCESS_NEWJDK5_MASK	(CFR_ACC_SYNTHETIC | CFR_ACC_ANNOTATION | CFR_ACC_ENUM)  /* Defines access flags not available prior to JDK 5.0 */
@@ -957,16 +979,15 @@ typedef struct J9CfrClassFile {
 #if JAVA_SPEC_VERSION >= 16
 #define CFR_METHOD_EXT_HAS_SCOPED_ANNOTATION 0x08
 #endif /* JAVA_SPEC_VERSION >= 16 */
+#define CFR_METHOD_EXT_NOT_CHECKPOINT_SAFE_ANNOTATION 0x10
+#if JAVA_SPEC_VERSION >= 20
+#define CFR_METHOD_EXT_JVMTIMOUNTTRANSITION_ANNOTATION 0x20
+#endif /* JAVA_SPEC_VERSION >= 20 */
 
 #define ANON_CLASSNAME_CHARACTER_SEPARATOR '/'
 
 #define CFR_FOUND_CHARS_IN_EXTENDED_MUE_FORM 0x1
 #define CFR_FOUND_SEPARATOR_IN_MUE_FORM 0x2
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-#define CFR_CLASS_FILE_VERSION_SUPPORT_VALUE_TYPE 0x4
-#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-
-#define IDENTITY_OBJECT_NAME "java/lang/IdentityObject"
 
 #ifdef __cplusplus
 }
