@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef J9CONSTS_H
@@ -159,8 +159,11 @@ extern "C" {
 #define J9_PRIVATE_FLAGS_FILL_EXISTING_TRACE 0x4000
 #define J9_PRIVATE_FLAGS_REPORT_EXCEPTION_THROW 0x8000
 #define J9_PRIVATE_FLAGS_REALTIME_THREAD 0x10000
-#define J9_PRIVATE_FLAGS_UNUSED 0x20000
-#define J9_PRIVATE_FLAGS_STACK_CORRUPT 0x40000
+#if JAVA_SPEC_VERSION >= 16
+#define J9_PRIVATE_FLAGS_FFI_UPCALL_THREAD 0x20000
+#endif /* JAVA_SPEC_VERSION >= 16 */
+#define J9_PRIVATE_FLAGS_UNUSED_0x40000 0x40000
+#define J9_PRIVATE_FLAGS_VIRTUAL_THREAD_HIDDEN_FRAMES 0x80000
 #define J9_PRIVATE_FLAGS_CONCURRENT_MARK_ACTIVE 0x100000
 #define J9_PRIVATE_FLAGS_USE_BOOTSTRAP_LOADER 0x200000
 #define J9_PRIVATE_FLAGS_NO_EXCEPTION_IN_START_JAVA_THREAD 0x400000
@@ -178,6 +181,8 @@ extern "C" {
 #define J9_PRIVATE_FLAGS2_CHECK_PACKAGE_ACCESS 0x2
 #define J9_PRIVATE_FLAGS2_UNSAFE_HANDLE_SIGBUS 0x4
 #define J9_PRIVATE_FLAGS2_ASYNC_GET_CALL_TRACE 0x8 /* DO NOT set this anywhere but AsyncGetCallTrace */
+#define J9_PRIVATE_FLAGS2_REENTER_INTERPRETER 0x10
+#define J9_PRIVATE_FLAGS2_DELAY_HALT_FOR_CHECKPOINT 0x20
 
 #define J9_PUBLIC_FLAGS_HALT_THREAD_EXCLUSIVE 0x1
 #define J9_PUBLIC_FLAGS_DEBUG_VM_ACCESS 0x2
@@ -197,9 +202,9 @@ extern "C" {
 #define J9_PUBLIC_FLAGS_HALT_THREAD_INSPECTION 0x8000
 #define J9_PUBLIC_FLAGS_HALT_THREAD_HCR 0x10000
 #define J9_PUBLIC_FLAGS_THREAD_PARKED 0x20000
-#define J9_PUBLIC_FLAGS_UNUSED_0x40000 0x40000
+#define J9_PUBLIC_FLAGS_EXCLUSIVE_SET_NOT_SAFE 0x40000
 #define J9_PUBLIC_FLAGS_THREAD_TIMED 0x80000
-#define J9_PUBLIC_FLAGS_UNUSED_0x100000 0x100000
+#define J9_PUBLIC_FLAGS_CLOSE_SCOPE 0x100000
 #define J9_PUBLIC_FLAGS_HALT_THREAD_FOR_CHECKPOINT 0x200000
 #define J9_PUBLIC_FLAGS_JNI_CRITICAL_REGION 0x400000
 #define J9_PUBLIC_FLAGS_POP_FRAMES_INTERRUPT 0x800000
@@ -257,7 +262,6 @@ extern "C" {
 #define J9_RESOLVE_FLAG_JCL_CONSTANT_POOL 0x100
 #define J9_RESOLVE_FLAG_REDEFINE_CLASS 0x200
 #define J9_RESOLVE_FLAG_NO_CLASS_INIT 0x400
-#define J9_RESOLVE_FLAG_WITH_FIELD 0x800
 
 #define J9_RESOLVE_STATIC_FIELD_TAG_FROM_CLINIT 0x1
 #define J9_RESOLVE_STATIC_FIELD_TAG_IS_ISOLATED 0x2
@@ -281,7 +285,7 @@ extern "C" {
 #define J9_RUNTIME_ARGENCODING_UNICODE 0x2000
 #define J9_RUNTIME_ARGENCODING_LATIN 0x4000
 #define J9_RUNTIME_ARGENCODING_UTF8 0x8000
-#define J9_RUNTIME_UNUSED_0x10000 0x10000
+#define J9_RUNTIME_ALLOW_DYNAMIC_AGENT 0x10000
 #define J9_RUNTIME_UNUSED_0x20000 0x20000
 #define J9_RUNTIME_UNUSED_0x40000 0x40000
 #define J9_RUNTIME_UNUSED_0x80000 0x80000
@@ -332,8 +336,7 @@ extern "C" {
 #define J9_EXTENDED_RUNTIME_REDUCE_CPU_MONITOR_OVERHEAD 0x80000000
 
 /* constants for J9JavaVM.extendedRuntimeFlags2 */
-/* Bit 0x1 is free to use. */
-#define J9_EXTENDED_RUNTIME2_UNUSED_0x1 0x1
+#define J9_EXTENDED_RUNTIME2_JFR_ENABLED 0x1
 #define J9_EXTENDED_RUNTIME2_COMPRESS_OBJECT_REFERENCES 0x2
 #define J9_EXTENDED_RUNTIME2_ENABLE_PREVIEW 0x4
 #define J9_EXTENDED_RUNTIME2_LOAD_AGENT_MODULE 0x8
@@ -354,6 +357,23 @@ extern "C" {
 #define J9_EXTENDED_RUNTIME2_TUNE_QUICKSTART 0x40000
 #define J9_EXTENDED_RUNTIME2_DISABLE_FINALIZATION 0x80000
 #define J9_EXTENDED_RUNTIME2_CRIU_SINGLE_THREAD_MODE 0x100000
+#define J9_EXTENDED_RUNTIME2_USE_CONTAINER_SUPPORT 0x200000
+#define J9_EXTENDED_RUNTIME2_SHOW_CARRIER_FRAMES 0x400000
+#define J9_EXTENDED_RUNTIME2_CRIU_SINGLE_THROW_BLOCKING_EXCEPTIONS 0x800000
+#define J9_EXTENDED_RUNTIME2_CPU_LOAD_COMPATIBILITY 0x1000000
+#define J9_EXTENDED_RUNTIME2_ALWAYS_KEEP_JNI_IDS 0x2000000
+#define J9_EXTENDED_RUNTIME2_NEVER_KEEP_JNI_IDS 0x4000000
+#define J9_EXTENDED_RUNTIME2_FFI_PROTO 0x8000000
+#define J9_EXTENDED_RUNTIME2_DISABLE_EXTENDED_HCR 0x10000000
+#define J9_EXTENDED_RUNTIME2_ENABLE_CONTINUATION_CACHE_SUMMARY 0x20000000
+#define J9_EXTENDED_RUNTIME2_RAMSTATE_SNAPSHOT_RUN 0x40000000
+#define J9_EXTENDED_RUNTIME2_RAMSTATE_RESTORE_RUN 0x80000000
+
+/* constants for J9JavaVM.extendedRuntimeFlags3 */
+#define J9_EXTENDED_RUNTIME3_START_FLIGHT_RECORDING 0x1
+#define J9_EXTENDED_RUNTIME3_YIELD_PINNED_CONTINUATION 0x2
+#define J9_EXTENDED_RUNTIME3_CACHE_MAPS 0x4
+#define J9_EXTENDED_RUNTIME3_MODULE_PACKAGES_INITIALIZED 0x8
 
 #define J9_OBJECT_HEADER_AGE_DEFAULT 0xA /* OBJECT_HEADER_AGE_DEFAULT */
 #define J9_OBJECT_HEADER_SHAPE_MASK 0xE /* OBJECT_HEADER_SHAPE_MASK */
@@ -371,16 +391,14 @@ extern "C" {
 #define J9_STARTPC_DLT_READY 0x8
 #define J9_STARTPC_STATUS 0xF
 
-/* Frame iterator return values */
-#define J9_STACKWALK_STOP_ITERATING 0x0
-#define J9_STACKWALK_KEEP_ITERATING 0x1
-
 /* Stackwalk return values */
 #define J9_STACKWALK_RC_NONE 0x0
-#define J9_STACKWALK_RC_NO_MEMORY 0x1
-#define J9_STACKWALK_RC_FRAME_NOT_FOUND 0x2
-#define J9_STACKWALK_RC_BAD_STATE_BUFFER 0x3
-#define J9_STACKWALK_RC_STACK_CORRUPT 0x4
+#define J9_STACKWALK_STOP_ITERATING 0x0 /* Must be the same as no error */
+#define J9_STACKWALK_KEEP_ITERATING 0x1
+#define J9_STACKWALK_RC_NO_MEMORY 0x2
+#define J9_STACKWALK_RC_FRAME_NOT_FOUND 0x3
+#define J9_STACKWALK_RC_BAD_STATE_BUFFER 0x4
+#define J9_STACKWALK_RC_STACK_CORRUPT 0x5
 
 /* Tag for JIT artifact search cache */
 #define J9_STACKWALK_NO_JIT_CACHE 0x1
@@ -397,7 +415,7 @@ extern "C" {
 #define J9_STACKWALK_CACHE_METHODS 0x400
 #define J9_STACKWALK_CACHE_MASK 0x700
 #define J9_STACKWALK_SKIP_HIDDEN_FRAMES 0x800
-#define J9_STACKWALK_INCLUDE_ARRAYLET_LEAVES 0x10000
+#define J9_STACKWALK_PREPARE_FOR_YIELD 0x10000
 #define J9_STACKWALK_LINEAR 0x20000
 #define J9_STACKWALK_VISIBLE_ONLY 0x40000
 #define J9_STACKWALK_INCLUDE_NATIVES 0x80000
@@ -438,6 +456,9 @@ extern "C" {
 #define J9_FINDCLASS_FLAG_CLASS_OPTION_NESTMATE 0x40000
 #define J9_FINDCLASS_FLAG_CLASS_OPTION_STRONG 0x80000
 #define J9_FINDCLASS_FLAG_DO_NOT_SHARE 0x100000
+#define J9_FINDCLASS_FLAG_LAMBDA 0x200000
+#define J9_FINDCLASS_FLAG_LAMBDAFORM 0x400000
+#define J9_FINDCLASS_FLAG_CLASS_OPTION_NULL_RESTRICTED_ARRAY 0x800000
 
 #define J9_FINDKNOWNCLASS_FLAG_INITIALIZE 0x1
 #define J9_FINDKNOWNCLASS_FLAG_EXISTING_ONLY 0x2
@@ -481,7 +502,6 @@ extern "C" {
 
 #define J9_GC_MINIMUM_OBJECT_SIZE 0x10
 #define J9_GC_INDEXABLE_DATA_FIELD_SIZE sizeof(void *)
-#define J9_GC_MINIMUM_INDEXABLE_OBJECT_SIZE (J9_GC_MINIMUM_OBJECT_SIZE + J9_GC_INDEXABLE_DATA_FIELD_SIZE)
 #if defined(J9VM_ENV_DATA64)
 #define J9_GC_MARK_MAP_LOG_SIZEOF_UDATA 0x6
 #define J9_GC_MARK_MAP_UDATA_MASK 0x3F
@@ -489,6 +509,15 @@ extern "C" {
 #define J9_GC_MARK_MAP_LOG_SIZEOF_UDATA 0x5
 #define J9_GC_MARK_MAP_UDATA_MASK 0x1F
 #endif /* J9VM_ENV_DATA64 */
+#define J9_GC_CONTINUATION_STATE_CONCURRENT_SCAN_NONE 0
+#define J9_GC_CONTINUATION_STATE_STARTED 0x1
+#define J9_GC_CONTINUATION_STATE_FINISHED 0x2
+#define J9_GC_CONTINUATION_STATE_PENDING_TO_BE_MOUNTED 0x4
+#define J9_GC_CONTINUATION_STATE_CONCURRENT_SCAN_LOCAL 0x8
+#define J9_GC_CONTINUATION_STATE_CONCURRENT_SCAN_GLOBAL 0x10
+#define J9_GC_CONTINUATION_STATE_LAST_UNMOUNT 0x20
+#define J9_GC_CONTINUATION_STATE_CONCURRENT_SCAN_ANY (J9_GC_CONTINUATION_STATE_CONCURRENT_SCAN_LOCAL | J9_GC_CONTINUATION_STATE_CONCURRENT_SCAN_GLOBAL)
+#define J9_GC_CONTINUATION_STATE_CARRIERID_MASK (~(uintptr_t)0xff)
 
 #define J9VMGC_SIZECLASSES_MIN 0x1
 #define J9VMGC_SIZECLASSES_MIN_SMALL 0x1
@@ -563,6 +592,7 @@ extern "C" {
 #define BCV_ERR_INACCESSIBLE_CLASS						-33
 #define BCV_ERR_BYTECODE_ERROR							-34
 #define BCV_ERR_NEW_OJBECT_MISMATCH						-35
+#define BCV_ERR_INIT_FLAGS_MISMATCH						-36
 
 #define J9_GC_OBJ_HEAP_HOLE 0x1
 #define J9_GC_MULTI_SLOT_HOLE 0x1
@@ -574,7 +604,7 @@ extern "C" {
 #define J9_GC_COLLECT_REF_WEAK 0x1
 #define J9_GC_COLLECT_REF_PHANTOM 0x2
 
-#define J9_LOOK_ALLOW_FWD 0x1
+#define J9_LOOK_UNUSED_0x1 0x1
 #define J9_LOOK_INTERFACE 0x2
 #define J9_LOOK_STATIC 0x4
 #define J9_LOOK_NEW_INSTANCE 0x8
@@ -619,8 +649,10 @@ extern "C" {
 #define J9_SIG_XRS_SYNC 0x10
 #define J9_SIG_XRS_ASYNC 0x20
 #define J9_SIG_ZOS_CEEHDLR 0x40
+#define J9_SIG_NO_SIG_USR2 0x80
 #define J9_SIG_PERCOLATE_CONDITIONS 0x100
 #define J9_SIG_POSIX_COOPERATIVE_SHUTDOWN 0x200
+#define J9_SIG_WINDOWS_MITIGATION_POLICY_CFG_ENABLED 0x400
 
 #define J9_THREAD_START_NO_ERROR 0x0
 #define J9_THREAD_START_FAILED_VMTHREAD_ALLOC 0x1
@@ -630,6 +662,15 @@ extern "C" {
 #define J9_THREAD_START_UNSPECIFIED_ERROR 0x5
 #define J9_THREAD_START_THROW_CURRENT_EXCEPTION 0x6
 
+#if JAVA_SPEC_VERSION >= 19
+/* J9VMThread address is aligned using J9VMTHREAD_ALIGNMENT (0x100)
+ * so last 8 bits can be used as bit flags.
+ */
+#define J9_VIRTUALTHREAD_INTERNAL_STATE_NONE      0x0
+#define J9_VIRTUALTHREAD_INTERNAL_STATE_SUSPENDED 0x1
+#define J9_VIRTUALTHREAD_INTERNAL_STATE_CARRIERID_MASK (~(uintptr_t)0xff)
+#endif /* JAVA_SPEC_VERSION >= 19 */
+
 #define J9_GC_ALLOCATE_OBJECT_NON_INSTRUMENTABLE OMR_GC_ALLOCATE_OBJECT_NON_INSTRUMENTABLE
 #define J9_GC_ALLOCATE_OBJECT_INSTRUMENTABLE OMR_GC_ALLOCATE_OBJECT_INSTRUMENTABLE
 #define J9_GC_ALLOCATE_OBJECT_TENURED OMR_GC_ALLOCATE_OBJECT_TENURED
@@ -659,6 +700,7 @@ extern "C" {
 #define J9_NATIVE_LIBRARY_SWITCH_REQUIRED 0x1
 #define J9_NATIVE_LIBRARY_SWITCH_JDBC 0x2
 #define J9_NATIVE_LIBRARY_SWITCH_WITH_SUBTASKS 0x4
+#define J9_NATIVE_LIBRARY_SWITCH_MASK 0x7
 
 #define J9_METHOD_HANDLE_COMPILE_SHARED 0x0
 #define J9_METHOD_HANDLE_COMPILE_CUSTOM 0x1
@@ -669,6 +711,11 @@ extern "C" {
 #define J9_METHOD_TYPE_ORIGIN_INVOKE_EXACT 0x2
 #define J9_METHOD_TYPE_ORIGIN_INVOKE 0x4
 
+/* J9Module::moduleName was previously of type j9object_t. This constant signifies
+ * to DDR whether the moduleNames in a core file are of type J9UTF8 *.
+ */
+#define J9_MODULE_NAME_IS_J9UTF8 0x1
+
 #define J9_CHECK_ASYNC_NO_ACTION 0x0
 #define J9_CHECK_ASYNC_THROW_EXCEPTION 0x1
 #define J9_CHECK_ASYNC_POP_FRAMES 0x4
@@ -699,6 +746,11 @@ extern "C" {
 #define J9_BCLOOP_CHECK_ASYNC 0x15
 #define J9_BCLOOP_EXIT_INTERPRETER 0x16
 #define J9_BCLOOP_FILL_OSR_BUFFER 0x17
+#if JAVA_SPEC_VERSION >= 16
+#define J9_BCLOOP_N2I_TRANSITION 0x18
+#endif /* JAVA_SPEC_VERSION >= 16 */
+#define J9_BCLOOP_REENTER_INTERPRETER 0x19
+#define J9_BCLOOP_YIELD_FOR_JIT_MONENT 0x1A
 
 #define J9_RAS_METHOD_UNSEEN 0x0
 #define J9_RAS_METHOD_SEEN 0x1
@@ -731,51 +783,53 @@ extern "C" {
 /* Jazz 99339: The reason code for JVMTI event callbacks on zOS.
  * Note: refer to jvmtiEventCallbacks (/runtime/include/jvmti.h) for reserved JVMTI events.
  */
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_VM_INIT								0x16
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_VM_DEATH							0x17
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_THREAD_START						0x18
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_THREAD_END							0x19
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_CLASS_FILE_LOAD_HOOK				0x1A
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_CLASS_LOAD							0x1B
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_CLASS_PREPARE						0x1C
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_VM_START							0x1D
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_EXCEPTION							0x1E
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_EXCEPTION_CATCH						0x1F
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_SINGLE_STEP							0x20
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_FRAME_POP							0x21
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_BREAKPOINT							0x22
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_FIELD_ACCESS						0x23
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_FIELD_MODIFICATION					0x24
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_METHOD_ENTRY						0x25
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_METHOD_EXIT							0x26
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_NATIVE_METHOD_BIND					0x27
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_COMPILED_METHOD_LOAD				0x28
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_COMPILED_METHOD_UNLOAD				0x29
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_DYNAMIC_CODE_GENERATED				0x2A
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_DATA_DUMP_REQUEST					0x2B
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_VM_INIT 0x16
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_VM_DEATH 0x17
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_THREAD_START 0x18
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_THREAD_END 0x19
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_CLASS_FILE_LOAD_HOOK 0x1A
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_CLASS_LOAD 0x1B
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_CLASS_PREPARE 0x1C
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_VM_START 0x1D
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_EXCEPTION 0x1E
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_EXCEPTION_CATCH 0x1F
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_SINGLE_STEP 0x20
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_FRAME_POP 0x21
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_BREAKPOINT 0x22
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_FIELD_ACCESS 0x23
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_FIELD_MODIFICATION 0x24
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_METHOD_ENTRY 0x25
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_METHOD_EXIT 0x26
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_NATIVE_METHOD_BIND 0x27
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_COMPILED_METHOD_LOAD 0x28
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_COMPILED_METHOD_UNLOAD 0x29
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_DYNAMIC_CODE_GENERATED 0x2A
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_DATA_DUMP_REQUEST 0x2B
 /* 0x2C Reserved for JVMTI event */
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_MONITOR_WAIT						0x2D
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_MONITOR_WAITED						0x2E
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_MONITOR_CONTENDED_ENTER				0x2F
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_MONITOR_CONTENDED_ENTERED			0x30
-/* 0x31 Reserved for JVMTI event */
-/* 0x32 Reserved for JVMTI event */
-/* 0x33 Reserved for JVMTI event */
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_RESOURCE_EXHAUSTED					0x34
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_GC_START							0x35
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_GC_FINISH							0x36
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_OBJECT_FREE							0x37
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_VM_OBJECT_ALLOC						0x38
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_MONITOR_WAIT 0x2D
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_MONITOR_WAITED 0x2E
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_MONITOR_CONTENDED_ENTER 0x2F
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_MONITOR_CONTENDED_ENTERED 0x30
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_VIRTUAL_THREAD_MOUNT 0x31
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_VIRTUAL_THREAD_UNMOUNT 0x32
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_VIRTUAL_THREAD_DESTROY 0x33
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_RESOURCE_EXHAUSTED 0x34
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_GC_START 0x35
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_GC_FINISH 0x36
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_OBJECT_FREE 0x37
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_VM_OBJECT_ALLOC 0x38
 /* 0x39 Reserved for JVMTI event */
-#define J9_JNI_OFFLOAD_SWITCH_JVMTI_SAMPLED_OBJECT_ALLOC				0x3A
-#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_COMPILING_START					0x3B
-#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_COMPILING_END						0x3C
-#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_INSTRUMENTABLE_OBJECT_ALLOC		0x3D
-#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_VM_DUMP_START						0x3E
-#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_VM_DUMP_END						0x3F
-#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_GC_CYCLE_START					0x40
-#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_GC_CYCLE_FINISH					0x41
-#define J9_JNI_OFFLOAD_SWITCH_TRACE_SUBSCRIBER_THREAD					0x42
+#define J9_JNI_OFFLOAD_SWITCH_JVMTI_SAMPLED_OBJECT_ALLOC 0x3A
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_COMPILING_START 0x3B
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_COMPILING_END 0x3C
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_INSTRUMENTABLE_OBJECT_ALLOC 0x3D
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_VM_DUMP_START 0x3E
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_VM_DUMP_END 0x3F
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_GC_CYCLE_START 0x40
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_GC_CYCLE_FINISH 0x41
+#define J9_JNI_OFFLOAD_SWITCH_TRACE_SUBSCRIBER_THREAD 0x42
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_EVENT_OPENJ9_VM_CHECKPOINT 0x43
+#define J9_JNI_OFFLOAD_SWITCH_J9JVMTI_EVENT_OPENJ9_VM_RESTORE 0x44
 
 #define J9_JNI_OFFLOAD_SWITCH_THRESHOLD 0x4000
 #define J9_JNI_OFFLOAD_WITH_SUBTASKS_FLAG 0x40000000
@@ -886,12 +940,49 @@ extern "C" {
 #define J9_JAVA_NATIVE 0x100
 #define J9_JAVA_STATIC 0x8
 
+/* Stack walker frame Types */
+#define J9VM_STACK_FRAME_INTERPRETER 0
+#define J9VM_STACK_FRAME_JIT 1
+#define J9VM_STACK_FRAME_JIT_INLINE 2
+#define J9VM_STACK_FRAME_NATIVE 3
+
+/* JFR event types */
+
+#define J9JFR_EVENT_TYPE_EXECUTION_SAMPLE 0
+#define J9JFR_EVENT_TYPE_THREAD_START 1
+#define J9JFR_EVENT_TYPE_THREAD_END 2
+#define J9JFR_EVENT_TYPE_THREAD_SLEEP 3
+#define J9JFR_EVENT_TYPE_OBJECT_WAIT 4
+#define J9JFR_EVENT_TYPE_CPU_LOAD 5
+#define J9JFR_EVENT_TYPE_THREAD_CPU_LOAD 6
+#define J9JFR_EVENT_TYPE_CLASS_LOADING_STATISTICS 7
+#define J9JFR_EVENT_TYPE_THREAD_CONTEXT_SWITCH_RATE 8
+#define J9JFR_EVENT_TYPE_THREAD_PARK 9
+#define J9JFR_EVENT_TYPE_THREAD_STATISTICS 10
+#define J9JFR_EVENT_TYPE_MONITOR_ENTER 11
+#define J9JFR_EVENT_TYPE_SYSTEM_GC 12
+
+/* JFR thread states */
+
+#define J9JFR_THREAD_STATE_RUNNING 0
+
 /* Constants for JIT flattened field resolution */
 
-#define J9TR_FLAT_RESOLVE_WITHFIELD 0
 #define J9TR_FLAT_RESOLVE_GETFIELD 1
 #define J9TR_FLAT_RESOLVE_PUTFIELD 2
 
+/* Class version minimum for value type support. */
+
+#define VALUE_TYPES_MAJOR_VERSION 69
+#define PREVIEW_MINOR_VERSION 65535
+#define J9_IS_CLASSFILE_OR_ROMCLASS_VALUETYPE_VERSION(classfileOrRomClass) (((classfileOrRomClass)->majorVersion >= VALUE_TYPES_MAJOR_VERSION) && (PREVIEW_MINOR_VERSION == (classfileOrRomClass)->minorVersion))
+
+#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
+/* Constants for java.lang.reflect.Field flags */
+#define TRUST_FINAL 0x10
+#define NULL_RESTRICTED 0x20
+#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES)*/
+
 #ifdef __cplusplus
 }
 #endif
