@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2020 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,13 +15,14 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #if !defined(OTI_J9PORT_H_)
 #define OTI_J9PORT_H_
+
 /* @ddr_namespace: map_to_type=J9PortLibrary */
 
 /* NOTE:  j9port_generated.h include is at the bottom of this file until its dependencies on this file can be relaxed */
@@ -233,6 +234,7 @@
 #define J9SH_BASEDIR "javasharedresources\\"
 #else
 #define J9SH_BASEDIR "javasharedresources/"
+#define J9SH_HIDDENDIR ".cache/"
 #endif
 
 /**
@@ -337,353 +339,6 @@ typedef struct J9GuestMemoryUsage {
 	int64_t maxMemLimit;
 } J9GuestMemoryUsage;
 
-/* j9sysinfo_get_processor_description
- */
-typedef enum J9ProcessorArchitecture {
-
-	PROCESSOR_UNDEFINED,
-
-	PROCESSOR_S390_UNKNOWN,
-	PROCESSOR_S390_GP6,
-	PROCESSOR_S390_GP7,
-	PROCESSOR_S390_GP8,
-	PROCESSOR_S390_GP9,
-	PROCESSOR_S390_GP10,
-	PROCESSOR_S390_GP11,
-	PROCESSOR_S390_GP12,
-	PROCESSOR_S390_GP13,
-	PROCESSOR_S390_GP14,
-
-	PROCESSOR_PPC_UNKNOWN,
-	PROCESSOR_PPC_7XX,
-	PROCESSOR_PPC_GP,
-	PROCESSOR_PPC_GR,
-	PROCESSOR_PPC_NSTAR,
-	PROCESSOR_PPC_PULSAR,
-	PROCESSOR_PPC_PWR403,
-	PROCESSOR_PPC_PWR405,
-	PROCESSOR_PPC_PWR440,
-	PROCESSOR_PPC_PWR601,
-	PROCESSOR_PPC_PWR602,
-	PROCESSOR_PPC_PWR603,
-	PROCESSOR_PPC_PWR604,
-	PROCESSOR_PPC_PWR620,
-	PROCESSOR_PPC_PWR630,
-	PROCESSOR_PPC_RIOS1,
-	PROCESSOR_PPC_RIOS2,
-	PROCESSOR_PPC_P6,
-	PROCESSOR_PPC_P7,
-	PROCESSOR_PPC_P8,
-	PROCESSOR_PPC_P9,
-	PROCESSOR_PPC_P10,
-
-	PROCESSOR_X86_UNKNOWN,
-	PROCESSOR_X86_INTELPENTIUM,
-	PROCESSOR_X86_INTELP6,
-	PROCESSOR_X86_INTELPENTIUM4,
-	PROCESSOR_X86_INTELCORE2,
-	PROCESSOR_X86_INTELTULSA,
-	PROCESSOR_X86_INTELNEHALEM,
-	PROCESSOR_X86_INTELWESTMERE,
-	PROCESSOR_X86_INTELSANDYBRIDGE,
-	PROCESSOR_X86_INTELHASWELL,
-	PROCESSOR_X86_AMDK5,
-	PROCESSOR_X86_AMDK6,
-	PROCESSOR_X86_AMDATHLONDURON,
-	PROCESSOR_X86_AMDOPTERON,
-
-	PROCESOR_RISCV64_UNKNOWN,
-
-	PROCESSOR_DUMMY = 0x40000000 /* force wide enums */
-
-} J9ProcessorArchitecture;
-
-/* Holds processor type and features used with j9sysinfo_get_processor_description
- * and j9sysinfo_processor_has_feature
- */
-#define J9PORT_SYSINFO_FEATURES_SIZE 5
-typedef struct J9ProcessorDesc {
-	J9ProcessorArchitecture processor;
-	J9ProcessorArchitecture physicalProcessor;
-	uint32_t features[J9PORT_SYSINFO_FEATURES_SIZE];
-} J9ProcessorDesc;
-
-/* PowerPC features
- * Auxiliary Vector Hardware Capability (AT_HWCAP) features for PowerPC.
- */
-#define J9PORT_PPC_FEATURE_32                    31 /* 32-bit mode.  */
-#define J9PORT_PPC_FEATURE_64                    30 /* 64-bit mode.  */
-#define J9PORT_PPC_FEATURE_601_INSTR             29 /* 601 chip, Old POWER ISA.  */
-#define J9PORT_PPC_FEATURE_HAS_ALTIVEC           28 /* SIMD/Vector Unit.  */
-#define J9PORT_PPC_FEATURE_HAS_FPU               27 /* Floating Point Unit.  */
-#define J9PORT_PPC_FEATURE_HAS_MMU               26 /* Memory Management Unit.  */
-#define J9PORT_PPC_FEATURE_HAS_4xxMAC            25 /* 4xx Multiply Accumulator.  */
-#define J9PORT_PPC_FEATURE_UNIFIED_CACHE         24 /* Unified I/D cache.  */
-#define J9PORT_PPC_FEATURE_HAS_SPE               23 /* Signal Processing ext.  */
-#define J9PORT_PPC_FEATURE_HAS_EFP_SINGLE        22 /* SPE Float.  */
-#define J9PORT_PPC_FEATURE_HAS_EFP_DOUBLE        21 /* SPE Double.  */
-#define J9PORT_PPC_FEATURE_NO_TB                 20 /* 601/403gx have no timebase.  */
-#define J9PORT_PPC_FEATURE_POWER4                19 /* POWER4 ISA 2.01.  */
-#define J9PORT_PPC_FEATURE_POWER5                18 /* POWER5 ISA 2.02.  */
-#define J9PORT_PPC_FEATURE_POWER5_PLUS           17 /* POWER5+ ISA 2.03.  */
-#define J9PORT_PPC_FEATURE_CELL_BE               16 /* CELL Broadband Engine */
-#define J9PORT_PPC_FEATURE_BOOKE                 15 /* ISA Embedded Category.  */
-#define J9PORT_PPC_FEATURE_SMT                   14 /* Simultaneous Multi-Threading.  */
-#define J9PORT_PPC_FEATURE_ICACHE_SNOOP          13
-#define J9PORT_PPC_FEATURE_ARCH_2_05             12 /* ISA 2.05.  */
-#define J9PORT_PPC_FEATURE_PA6T                  11 /* PA Semi 6T Core.  */
-#define J9PORT_PPC_FEATURE_HAS_DFP               10 /* Decimal FP Unit.  */
-#define J9PORT_PPC_FEATURE_POWER6_EXT             9 /* P6 + mffgpr/mftgpr.  */
-#define J9PORT_PPC_FEATURE_ARCH_2_06              8 /* ISA 2.06.  */
-#define J9PORT_PPC_FEATURE_HAS_VSX                7 /* P7 Vector Scalar Extension.  */
-#define J9PORT_PPC_FEATURE_PSERIES_PERFMON_COMPAT 6 /* Has ISA >= 2.05 PMU basic subset support.  */
-#define J9PORT_PPC_FEATURE_TRUE_LE                1 /* Processor in true Little Endian mode.  */
-#define J9PORT_PPC_FEATURE_PPC_LE                 0 /* Processor emulates Little Endian Mode.  */
-
-#define J9PORT_PPC_FEATURE_ARCH_2_07             32 + 31
-#define J9PORT_PPC_FEATURE_HTM                   32 + 30
-#define J9PORT_PPC_FEATURE_DSCR                  32 + 29
-#define J9PORT_PPC_FEATURE_EBB                   32 + 28
-#define J9PORT_PPC_FEATURE_ISEL                  32 + 27
-#define J9PORT_PPC_FEATURE_TAR                   32 + 26
-
-/* s390 features
- * z/Architecture Principles of Operation 4-69
- * STORE FACILITY LIST EXTENDED (STFLE)
- */
-#define J9PORT_S390_FEATURE_ESAN3      0 /* STFLE bit 0 */
-#define J9PORT_S390_FEATURE_ZARCH      1 /* STFLE bit 2 */
-#define J9PORT_S390_FEATURE_STFLE      2 /* STFLE bit 7 */
-#define J9PORT_S390_FEATURE_MSA        3 /* STFLE bit 17 */
-#define J9PORT_S390_FEATURE_DFP        6 /* STFLE bit 42 & 44 */
-#define J9PORT_S390_FEATURE_HPAGE      7
-#define J9PORT_S390_FEATURE_TE        10 /* STFLE bit 50 & 73 */
-#define J9PORT_S390_FEATURE_MSA_EXTENSION3                      11 /* STFLE bit 76 */
-#define J9PORT_S390_FEATURE_MSA_EXTENSION4                      12 /* STFLE bit 77 */
-
-#define J9PORT_S390_FEATURE_COMPARE_AND_SWAP_AND_STORE          32 + 0  /* STFLE bit 32 */
-#define J9PORT_S390_FEATURE_COMPARE_AND_SWAP_AND_STORE2         32 + 1  /* STFLE bit 33 */
-#define J9PORT_S390_FEATURE_EXECUTE_EXTENSIONS                  32 + 3  /* STFLE bit 35 */
-#define J9PORT_S390_FEATURE_FPE                                 32 + 9  /* STFLE bit 41 */
-
-#define J9PORT_S390_FEATURE_RI            64 + 0 /* STFLE bit 64 */
-
-/* z990 facilities */
-
-/* STFLE bit 19 - Long-displacement facility */
-#define J9PORT_S390_FEATURE_LONG_DISPLACEMENT 19
-
-/* z9 facilities */
-
-/* STFLE bit 21 - Extended-immediate facility */
-#define J9PORT_S390_FEATURE_EXTENDED_IMMEDIATE 21
-
-/* STFLE bit 22 - Extended-translation facility 3 */
-#define J9PORT_S390_FEATURE_EXTENDED_TRANSLATION_3 22
-
-/* STFLE bit 30 - ETF3-enhancement facility */
-#define J9PORT_S390_FEATURE_ETF3_ENHANCEMENT 30
-
-/* z10 facilities */
-
-/* STFLE bit 34 - General-instructions-extension facility */
-#define J9PORT_S390_FEATURE_GENERAL_INSTRUCTIONS_EXTENSIONS 34
-
-/* z196 facilities */
-
-/* STFLE bit 45 - High-word facility */
-#define J9PORT_S390_FEATURE_HIGH_WORD 45
-
-/* STFLE bit 45 - Load/store-on-condition facility 1 */
-#define J9PORT_S390_FEATURE_LOAD_STORE_ON_CONDITION_1 45
-
-/* zEC12 facilities */
-
-/* STFLE bit 49 - Miscellaneous-instruction-extension facility */
-#define J9PORT_S390_FEATURE_MISCELLANEOUS_INSTRUCTION_EXTENSION 49
-
-/* z13 facilities */
-
-/* STFLE bit 53 - Load/store-on-condition facility 2 */
-#define J9PORT_S390_FEATURE_LOAD_STORE_ON_CONDITION_2 53
-
-/* STFLE bit 53 - Load-and-zero-rightmost-byte facility */
-#define J9PORT_S390_FEATURE_LOAD_AND_ZERO_RIGHTMOST_BYTE 53
-
-/* STFLE bit 129 - Vector facility */
-#define J9PORT_S390_FEATURE_VECTOR_FACILITY 129
-
-/* z14 facilities */
-
-/* STFLE bit 58 - Miscellaneous-instruction-extensions facility 2 */
-#define J9PORT_S390_FEATURE_MISCELLANEOUS_INSTRUCTION_EXTENSION_2 58
-
-/* STFLE bit 59 - Semaphore-assist facility */
-#define J9PORT_S390_FEATURE_SEMAPHORE_ASSIST 59
-
-/* STFLE bit 131 - Side-effect-access facility */
-#define J9PORT_S390_FEATURE_SIDE_EFFECT_ACCESS 131
-
-/* STFLE bit 133 - Guarded-storage facility */
-#define J9PORT_S390_FEATURE_GUARDED_STORAGE 133
-
-/* STFLE bit 134 - Vector packed decimal facility */
-#define J9PORT_S390_FEATURE_VECTOR_PACKED_DECIMAL 134
-
-/* STFLE bit 135 - Vector enhancements facility 1 */
-#define J9PORT_S390_FEATURE_VECTOR_FACILITY_ENHANCEMENT_1 135
-
-/* STFLE bit 146 - Message-security-assist-extension-8 facility */
-#define J9PORT_S390_FEATURE_MSA_EXTENSION_8 146
-
-/* STFLE bit 57 - Message-security-assist-extension-5 facility */
-#define J9PORT_S390_FEATURE_MSA_EXTENSION_5 57
-
-/* z15 facilities */
-
-/* STFLE bit 61 - Miscellaneous-instruction-extensions facility 3 */ 
-#define J9PORT_S390_FEATURE_MISCELLANEOUS_INSTRUCTION_EXTENSION_3 61
-
-/* STFLE bit 148 - Vector enhancements facility 2 */
-#define J9PORT_S390_FEATURE_VECTOR_FACILITY_ENHANCEMENT_2 148
-
-/* STFLE bit 152 - Vector packed decimal enhancement facility */
-#define J9PORT_S390_FEATURE_VECTOR_PACKED_DECIMAL_ENHANCEMENT_FACILITY 152
-
-
-/*  Linux on Z features
- *  Auxiliary Vector Hardware Capability (AT_HWCAP) features for Linux on Z.
- *  Obtained from: https://github.com/torvalds/linux/blob/050cdc6c9501abcd64720b8cc3e7941efee9547d/arch/s390/include/asm/elf.h#L94-L109.
- *  If new facility support is required, then it must be defined there (and here), before we can check for it consistently.
- *
- *  The linux kernel will use the defines in the above link to set HWCAP features. This is done inside "setup_hwcaps(void)" routine found
- *  in arch/s390/kernel/setup.c in the linux kernel source tree.
- */
-#define J9PORT_HWCAP_S390_ESAN3     0x1
-#define J9PORT_HWCAP_S390_ZARCH     0x2
-#define J9PORT_HWCAP_S390_STFLE     0x4
-#define J9PORT_HWCAP_S390_MSA       0x8
-#define J9PORT_HWCAP_S390_LDISP     0x10
-#define J9PORT_HWCAP_S390_EIMM      0x20
-#define J9PORT_HWCAP_S390_DFP       0x40
-#define J9PORT_HWCAP_S390_HPAGE     0x80
-#define J9PORT_HWCAP_S390_ETF3EH    0x100
-#define J9PORT_HWCAP_S390_HIGH_GPRS 0x200
-#define J9PORT_HWCAP_S390_TE        0x400
-#define J9PORT_HWCAP_S390_VXRS      0x800
-#define J9PORT_HWCAP_S390_VXRS_BCD  0x1000
-#define J9PORT_HWCAP_S390_VXRS_EXT  0x2000
-#define J9PORT_HWCAP_S390_GS        0x4000
-
-/* x86 features
- * INTEL INSTRUCTION SET REFERENCE, A-M
- * 3-170 Vol. 2A Table 3-21. More on Feature Information Returned in the EDX Register
- */
-#define J9PORT_X86_FEATURE_FPU     0 /* Floating Point Unit On-Chip. */
-#define J9PORT_X86_FEATURE_VME     1 /* Virtual 8086 Mode Enhancements. */
-#define J9PORT_X86_FEATURE_DE      2 /* DE Debugging Extensions. */
-#define J9PORT_X86_FEATURE_PSE     3 /* Page Size Extension. */
-#define J9PORT_X86_FEATURE_TSC     4 /* Time Stamp Counter. */
-#define J9PORT_X86_FEATURE_MSR     5 /* Model Specific Registers RDMSR and WRMSR Instructions. */
-#define J9PORT_X86_FEATURE_PAE     6 /* Physical Address Extension. */
-#define J9PORT_X86_FEATURE_MCE     7 /* Machine Check Exception. */
-#define J9PORT_X86_FEATURE_CX8     8 /* Compare-and-exchange 8 bytes (64 bits) instruction */
-#define J9PORT_X86_FEATURE_APIC    9 /* APIC On-Chip. */
-#define J9PORT_X86_FEATURE_10     10 /* Reserved */
-#define J9PORT_X86_FEATURE_SEP    11 /* SYSENTER and SYSEXIT Instructions. */
-#define J9PORT_X86_FEATURE_MTRR   12 /* Memory Type Range Registers. */
-#define J9PORT_X86_FEATURE_PGE    13 /* Page Global Bit. */
-#define J9PORT_X86_FEATURE_MCA    14 /* Machine Check Architecture. */
-#define J9PORT_X86_FEATURE_CMOV   15 /* Conditional Move Instructions. */
-#define J9PORT_X86_FEATURE_PAT    16 /* Page Attribute Table. */
-#define J9PORT_X86_FEATURE_PSE_36 17 /* 36-Bit Page Size Extension. */
-#define J9PORT_X86_FEATURE_PSN    18 /* Processor Serial Number. */
-#define J9PORT_X86_FEATURE_CLFSH  19 /* CLFLUSH Instruction. */
-#define J9PORT_X86_FEATURE_20     20 /* Reserved */
-#define J9PORT_X86_FEATURE_DS     21 /* Debug Store. */
-#define J9PORT_X86_FEATURE_ACPI   22 /* Thermal Monitor and Software Controlled Clock Facilities. */
-#define J9PORT_X86_FEATURE_MMX    23 /* Intel MMX Technology. */
-#define J9PORT_X86_FEATURE_FXSR   24 /* FXSAVE and FXRSTOR Instructions. */
-#define J9PORT_X86_FEATURE_SSE    25 /* The processor supports the SSE extensions. */
-#define J9PORT_X86_FEATURE_SSE2   26 /* The processor supports the SSE2 extensions. */
-#define J9PORT_X86_FEATURE_SS     27 /* Self Snoop. */
-#define J9PORT_X86_FEATURE_HTT    28 /* Hyper Threading. */
-#define J9PORT_X86_FEATURE_TM     29 /* Thermal Monitor. */
-#define J9PORT_X86_FEATURE_30     30 /* Reserved */
-#define J9PORT_X86_FEATURE_PBE    31 /* Pending Break Enable. */
-
-/* INTEL INSTRUCTION SET REFERENCE, A-M
- * Vol. 2A 3-167 Table 3-20. Feature Information Returned in the ECX Register
- */
-#define J9PORT_X86_FEATURE_SSE3         32 + 0 /* Streaming SIMD Extensions 3 */
-#define J9PORT_X86_FEATURE_PCLMULQDQ    32 + 1 /* PCLMULQDQ. */
-#define J9PORT_X86_FEATURE_DTES64       32 + 2 /* 64-bit DS Area. */
-#define J9PORT_X86_FEATURE_MONITOR      32 + 3 /* MONITOR/MWAIT. */
-#define J9PORT_X86_FEATURE_DS_CPL       32 + 4 /* CPL Qualified Debug Store. */
-#define J9PORT_X86_FEATURE_VMX          32 + 5 /* Virtual Machine Extensions. */
-#define J9PORT_X86_FEATURE_SMX          32 + 6 /* Safer Mode Extensions. */
-#define J9PORT_X86_FEATURE_EIST         32 + 7 /* Enhanced Intel SpeedStep technology. */
-#define J9PORT_X86_FEATURE_TM2          32 + 8 /* Thermal Monitor 2. */
-#define J9PORT_X86_FEATURE_SSSE3        32 + 9 /* Supplemental Streaming SIMD Extensions 3 */
-#define J9PORT_X86_FEATURE_CNXT_ID      32 + 10 /* L1 Context ID. */
-#define J9PORT_X86_FEATURE_11           32 + 11 /* Reserved */
-#define J9PORT_X86_FEATURE_FMA          32 + 12 /* FMA extensions using YMM state. */
-#define J9PORT_X86_FEATURE_CMPXCHG16B   32 + 13 /* CMPXCHG16B Available. */
-#define J9PORT_X86_FEATURE_XTPR         32 + 14 /* xTPR Update Control. */
-#define J9PORT_X86_FEATURE_PDCM         32 + 15 /* Perfmon and Debug Capability. */
-#define J9PORT_X86_FEATURE_16           32 + 16 /* Reserved. */
-#define J9PORT_X86_FEATURE_PCID         32 + 17 /* Process-context identifiers. */
-#define J9PORT_X86_FEATURE_DCA          32 + 18 /* Processor supports the ability to prefetch data from a memory mapped device. */
-#define J9PORT_X86_FEATURE_SSE4_1       32 + 19 /* Processor supports SSE4.1. */
-#define J9PORT_X86_FEATURE_SSE4_2       32 + 20 /* Processor supports SSE4.2. */
-#define J9PORT_X86_FEATURE_X2APIC       32 + 21 /* Processor supports x2APIC feature. */
-#define J9PORT_X86_FEATURE_MOVBE        32 + 22 /* Processor supports MOVBE instruction. */
-#define J9PORT_X86_FEATURE_POPCNT       32 + 23 /* Processor supports the POPCNT instruction. */
-#define J9PORT_X86_FEATURE_TSC_DEADLINE 32 + 24 /* Processor's local APIC timer supports one-shot operation using a TSC deadline value. */
-#define J9PORT_X86_FEATURE_AESNI        32 + 25 /* Processor supports the AESNI instruction extensions. */
-#define J9PORT_X86_FEATURE_XSAVE        32 + 26 /* Processor supports the XSAVE/XRSTOR processor extended states. */
-#define J9PORT_X86_FEATURE_OSXSAVE      32 + 27 /* OS has enabled XSETBV/XGETBV instructions to access XCR0, and support for processor extended state management using XSAVE/XRSTOR. */
-#define J9PORT_X86_FEATURE_AVX          32 + 28 /* Processor supports the AVX instruction extensions. */
-#define J9PORT_X86_FEATURE_F16C         32 + 29 /* 16-bit floating-point conversion instructions. */
-#define J9PORT_X86_FEATURE_RDRAND       32 + 30 /* Processor supports RDRAND instruction. */
-
-
-/* INTEL INSTRUCTION SET REFERENCE, A-L May 2019
- * Vol. 2 3-197 Table 3-8. Structured Feature Information Returned in the EBX Register by CPUID instruction
- */
-#define J9PORT_X86_FEATURE_FSGSBASE 		96 + 0	/* fsgsbase instructions support */
-#define J9PORT_X86_FEATURE_IA32_TSC_ADJUST	96 + 1	/* IA32_TSC_ADJUST MSR support */
-#define J9PORT_X86_FEATURE_SGX				96 + 2	/* Intel Software Guard Extensions */
-#define J9PORT_X86_FEATURE_BMI1				96 + 3	/* Bit Manipulation Instructions 1 */
-#define J9PORT_X86_FEATURE_HLE				96 + 4	/* Hardware Lock Elison */
-#define J9PORT_X86_FEATURE_AVX2				96 + 5	/* AVX2 support */
-#define J9PORT_X86_FEATURE_FDP_EXCPTN_ONLY	96 + 6	/* x87 FPU data pointer updated only on exceptions */
-#define J9PORT_X86_FEATURE_SMEP				96 + 7	/* Supervsior-Mode Execution Prevention */
-#define J9PORT_X86_FEATURE_BMI2				96 + 8	/* Bit Manipulation Instructions 2 */
-#define J9PORT_X86_FEATURE_ERMSB			96 + 9	/* Enhanced REP MOVSB/STOSB */
-#define J9PORT_X86_FEATURE_INVPCID			96 + 10	/* Invalidate Process-Context Identifier instruction */
-#define J9PORT_X86_FEATURE_RTM				96 + 11	/* Restricted Transactional Memory */
-#define J9PORT_X86_FEATURE_RDT_M			96 + 12	/* Intel RDT Monitoring */
-#define J9PORT_X86_FEATURE_DEPRECATE_FPUCS	96 + 13	/* Deprecates FPU CS and FPU DS when set */
-#define J9PORT_X86_FEATURE_MPX				96 + 14	/* Intel Memory Protextion Extensions */
-#define J9PORT_X86_FEATURE_RDT_A			96 + 15	/* Intel RDT Allocation */
-#define J9PORT_X86_FEATURE_AVX512F			96 + 16	/* AVX512 Foundation */
-#define J9PORT_X86_FEATURE_AVX512DQ			96 + 17	/* AVX512 Doubleword & Quadword */
-#define J9PORT_X86_FEATURE_RDSEED			96 + 18	/* RDSEED instruction support */
-#define J9PORT_X86_FEATURE_ADX				96 + 19	/* Intel ADX (multi-precision arithmetic) */
-#define J9PORT_X86_FEATURE_SMAP				96 + 20	/* Supervisor-Mode Access Prevention */
-#define J9PORT_X86_FEATURE_AVX512_IFMA		96 + 21	/* AVX512 Integer Fused Multiply Add */
-#define J9PORT_X86_FEATURE_22				96 + 22	/* reserved */
-#define J9PORT_X86_FEATURE_CLFLUSHOPT		96 + 23	/* cache flush optimized */
-#define J9PORT_X86_FEATURE_CLWB				96 + 24	/* cache line write back */
-#define J9PORT_X86_FEATURE_IPT				96 + 25	/* Intel Processor Trace */
-#define J9PORT_X86_FEATURE_AVX512PF			96 + 26	/* AVX512 Prefetch */
-#define J9PORT_X86_FEATURE_AVX512ER			96 + 27	/* AVX512 Exponential and Reciprocal */
-#define J9PORT_X86_FEATURE_AVX512CD			96 + 28	/* AVX512 Conflict Detection */
-#define J9PORT_X86_FEATURE_SHA				96 + 29	/* Intel SHA Extensions */
-#define J9PORT_X86_FEATURE_AVX512BW			96 + 30	/* AVX512 Byte and Word */
-#define J9PORT_X86_FEATURE_AVX512VL			96 + 31	/* AVX512 Vector Length */
-
 /* cache types */
 #define J9PORT_CACHEINFO_ICACHE 0x01
 #define J9PORT_CACHEINFO_DCACHE 0x02
@@ -758,6 +413,7 @@ typedef struct J9CacheInfoQuery {
 #define J9PORT_SLOPEN_LAZY OMRPORT_SLOPEN_LAZY
 #define J9PORT_SLOPEN_NO_LOOKUP_MSG_FOR_NOT_FOUND OMRPORT_SLOPEN_NO_LOOKUP_MSG_FOR_NOT_FOUND
 #define J9PORT_SLOPEN_OPEN_EXECUTABLE OMRPORT_SLOPEN_OPEN_EXECUTABLE
+#define J9PORT_SLOPEN_NO_LOAD OMRPORT_SLOPEN_NO_LOAD
 
 #define J9PORT_SL_FOUND OMRPORT_SL_FOUND
 #define J9PORT_SL_NOT_FOUND OMRPORT_SL_NOT_FOUND
@@ -780,6 +436,7 @@ typedef struct J9CacheInfoQuery {
 #define J9PORT_CTLDATA_VECTOR_REGS_SUPPORT_ON OMRPORT_CTLDATA_VECTOR_REGS_SUPPORT_ON
 #define J9PORT_CTLDATA_VMEM_ADVISE_HUGEPAGE OMRPORT_CTLDATA_VMEM_ADVISE_HUGEPAGE
 #define J9PORT_CTLDATA_VMEM_HUGE_PAGES_MMAP_ENABLED OMRPORT_CTLDATA_VMEM_HUGE_PAGES_MMAP_ENABLED
+#define J9PORT_CTLDATA_CRIU_SUPPORT_FLAGS "CRIU_SUPPORT_FLAGS"
 
 #define J9PORT_CPU_ONLINE OMRPORT_CPU_ONLINE
 #define J9PORT_CPU_TARGET OMRPORT_CPU_TARGET
@@ -963,19 +620,13 @@ typedef struct J9CacheInfoQuery {
 #define J9PORT_MMAP_FLAG_SHARED OMRPORT_MMAP_FLAG_SHARED
 #define J9PORT_MMAP_FLAG_COPYONWRITE OMRPORT_MMAP_FLAG_COPYONWRITE
 #define J9PORT_MMAP_SYNC_WAIT OMRPORT_MMAP_SYNC_WAIT
+#if defined(J9ZOS39064)
+#define J9PORT_MMAP_FLAG_ZOS_64BIT OMRPORT_MMAP_FLAG_ZOS_64BIT
+#endif /* defined(J9ZOS39064) */
 
 #define J9PORT_RESOURCE_SHARED_MEMORY OMRPORT_RESOURCE_SHARED_MEMORY
 #define J9PORT_RESOURCE_ADDRESS_SPACE OMRPORT_RESOURCE_ADDRESS_SPACE
 
-#define J9PORT_ARCH_X86 OMRPORT_ARCH_X86
-#define J9PORT_ARCH_HAMMER OMRPORT_ARCH_HAMMER
-#define J9PORT_ARCH_PPC OMRPORT_ARCH_PPC
-#define J9PORT_ARCH_PPC64 OMRPORT_ARCH_PPC64
-#define J9PORT_ARCH_PPC64LE OMRPORT_ARCH_PPC64LE
-#define J9PORT_ARCH_S390 OMRPORT_ARCH_S390
-#define J9PORT_ARCH_S390X OMRPORT_ARCH_S390X
-#define J9PORT_ARCH_RISCV OMRPORT_ARCH_RISCV
-
 #define J9PORT_RESOURCE_CORE_FLAGS OMRPORT_RESOURCE_CORE_FLAGS
 #define J9PORT_RESOURCE_FILE_DESCRIPTORS OMRPORT_RESOURCE_FILE_DESCRIPTORS
 
@@ -984,4 +635,7 @@ typedef struct J9CacheInfoQuery {
 #define J9PORT_PROCINFO_PROC_ONLINE OMRPORT_PROCINFO_PROC_ONLINE
 #define J9PORT_PROCINFO_NOT_AVAILABLE OMRPORT_PROCINFO_NOT_AVAILABLE
 
+#define J9OMRPORT_CRIU_SUPPORT_ENABLED OMRPORT_CRIU_SUPPORT_ENABLED
+#define J9OMRPORT_CRIU_SUPPORT_FINAL_RESTORE OMRPORT_CRIU_SUPPORT_FINAL_RESTORE
+
 #endif /* !defined(OTI_J9PORT_H_) */
