@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /**
@@ -47,6 +47,7 @@
 #include "portpriv.h"
 #include "j9sharedhelper.h"
 #include "j9shmem.h"
+#include "omrutil.h"
 #include "protect_helpers.h"
 #include "j9SysvIPCWrappers.h"
 #include "shchelp.h"
@@ -142,6 +143,8 @@
 #define __errno2() 0
 #endif
 
+#define SHM_STRING_ENDS_WITH_CHAR(str, ch) ((ch) == *((str) + strlen(str) - 1))
+
 static int32_t createSharedMemory(J9PortLibrary *portLibrary, intptr_t fd, BOOLEAN isReadOnlyFD, const char *baseFile, int32_t size, int32_t perm, struct j9shmem_controlFileFormat * controlinfo, uintptr_t groupPerm, struct j9shmem_handle *handle);
 static intptr_t checkSize(J9PortLibrary *portLibrary, int shmid, int64_t size);
 static intptr_t openSharedMemory (J9PortLibrary *portLibrary, intptr_t fd, const char *baseFile, int32_t perm, struct j9shmem_controlFileFormat * controlinfo, uintptr_t groupPerm, struct j9shmem_handle *handle, BOOLEAN *canUnlink, uintptr_t cacheFileType);
@@ -1268,8 +1271,10 @@ j9shmem_getDir(struct J9PortLibrary* portLibrary, const char* ctrlDirName, uint3
 		rootDir = ctrlDirName;
 	} else {
 		if (J9_ARE_ALL_BITS_SET(flags, J9SHMEM_GETDIR_USE_USERHOME)) {
+			/* If J9SHMEM_GETDIR_USE_USERHOME is set, try setting cache directory to $HOME/.cache/javasharedresources/ */
 			Assert_PRT_true(TRUE == appendBaseDir);
-			uintptr_t baseDirLen = strlen(J9SH_BASEDIR);
+			uintptr_t baseDirLen = sizeof(J9SH_HIDDENDIR) - 1 + sizeof(J9SH_BASEDIR) - 1;
+			uintptr_t minBufLen = bufLength < J9SH_MAXPATH ? bufLength : J9SH_MAXPATH;
 			/*
 			 *  User could define/change their notion of the home directory by setting environment variable "HOME".
 			 *  So check "HOME" first, if failed, then check getpwuid(getuid())->pw_dir.
@@ -1277,37 +1282,52 @@ j9shmem_getDir(struct J9PortLibrary* portLibrary, const char* ctrlDirName, uint3
 			if (0 == omrsysinfo_get_env("HOME", homeDirBuf, J9SH_MAXPATH)) {
 				uintptr_t dirLen = strlen((const char*)homeDirBuf);
 				if (0 < dirLen
-					&& ((dirLen + baseDirLen) < bufLength))
+					&& ((dirLen + baseDirLen) < minBufLen))
 				{
 					homeDir = homeDirBuf;
 				} else {
-					Trc_PRT_j9shmem_getDir_tryHomeDirFailed_homeDirTooLong(dirLen, bufLength - baseDirLen);
+					Trc_PRT_j9shmem_getDir_tryHomeDirFailed_homeDirTooLong(dirLen, minBufLen - baseDirLen);
 				}
 			} else {
 				Trc_PRT_j9shmem_getDir_tryHomeDirFailed_getEnvHomeFailed();
 			}
 			if (NULL == homeDir) {
-				struct passwd *pwent = getpwuid(getuid());
+				struct passwd *pwent = NULL;
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+				/* isCheckPointAllowed is equivalent to isCheckpointAllowed().
+				 * https://github.com/eclipse-openj9/openj9/issues/15800
+				 */
+				if (portLibrary->isCheckPointAllowed) {
+					Trc_PRT_j9shmem_getDir_tryHomeDirFailed_notFinalRestore();
+				} else
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+				{
+					pwent = getpwuid(getuid());
+					if (NULL == pwent) {
+						Trc_PRT_j9shmem_getDir_tryHomeDirFailed_getpwuidFailed();
+					}
+				}
 				if (NULL != pwent) {
 					uintptr_t dirLen = strlen((const char*)pwent->pw_dir);
-					if (0 < dirLen
-						&& ((dirLen + baseDirLen) < bufLength))
-					{
+					if ((0 < dirLen)
+						&& ((dirLen + baseDirLen) < minBufLen)
+					) {
 						homeDir = pwent->pw_dir;
 					} else {
 						rc = J9PORT_ERROR_SHMEM_GET_DIR_HOME_BUF_OVERFLOW;
-						Trc_PRT_j9shmem_getDir_tryHomeDirFailed_pw_dirDirTooLong(dirLen, bufLength - baseDirLen);
+						Trc_PRT_j9shmem_getDir_tryHomeDirFailed_pw_dirDirTooLong(dirLen, minBufLen - baseDirLen);
 					}
 				} else {
 					rc = J9PORT_ERROR_SHMEM_GET_DIR_FAILED_TO_GET_HOME;
-					Trc_PRT_j9shmem_getDir_tryHomeDirFailed_getpwuidFailed();
 				}
 			}
 			if (NULL != homeDir) {
 				struct J9FileStat statBuf = {0};
 				if (0 == omrfile_stat(homeDir, 0, &statBuf)) {
 					if (!statBuf.isRemote) {
-						rootDir = homeDir;
+						uintptr_t charWritten = omrstr_printf(homeDirBuf, J9SH_MAXPATH, SHM_STRING_ENDS_WITH_CHAR(homeDir, '/') ? "%s%s" : "%s/%s", homeDir, J9SH_HIDDENDIR);
+						Assert_PRT_true(charWritten < J9SH_MAXPATH);
+						rootDir = homeDirBuf;
 					} else {
 						rc = J9PORT_ERROR_SHMEM_GET_DIR_HOME_ON_NFS;
 						Trc_PRT_j9shmem_getDir_tryHomeDirFailed_homeOnNFS(homeDir);
@@ -1326,7 +1346,7 @@ j9shmem_getDir(struct J9PortLibrary* portLibrary, const char* ctrlDirName, uint3
 		Assert_PRT_true(rc < 0);
 	} else {
 		if (appendBaseDir) {
-			if (omrstr_printf(buffer, bufLength, "%s/%s", rootDir, J9SH_BASEDIR) == bufLength - 1) {
+			if (omrstr_printf(buffer, bufLength, SHM_STRING_ENDS_WITH_CHAR(rootDir, '/') ? "%s%s" : "%s/%s", rootDir, J9SH_BASEDIR) >= bufLength) {
 				Trc_PRT_j9shmem_getDir_ExitFailedOverflow();
 				rc = J9PORT_ERROR_SHMEM_GET_DIR_BUF_OVERFLOW;
 			}
@@ -1334,8 +1354,8 @@ j9shmem_getDir(struct J9PortLibrary* portLibrary, const char* ctrlDirName, uint3
 			/* Avoid appending two slashes; this leads to problems in matching full file names. */
 			if (omrstr_printf(buffer,
 								bufLength,
-								('/' == rootDir[strlen(rootDir)-1]) ? "%s" : "%s/",
-										rootDir) == bufLength - 1) {
+								SHM_STRING_ENDS_WITH_CHAR(rootDir, '/') ? "%s" : "%s/",
+								rootDir) >= bufLength) {
 				Trc_PRT_j9shmem_getDir_ExitFailedOverflow();
 				rc = J9PORT_ERROR_SHMEM_GET_DIR_BUF_OVERFLOW;
 			}
@@ -1707,7 +1727,7 @@ openSharedMemory (J9PortLibrary *portLibrary, intptr_t fd, const char *baseFile,
 				goto failDontUnlink;
 			}
 		} else {
-#if defined(__GNUC__) || defined(AIXPPC)
+#if (defined(__GNUC__) && !defined(J9ZOS390)) || defined(AIXPPC)
 #if defined(OSX)
 			/*Use ._key for OSX*/
 			if (buf.shm_perm._key != controlinfo->common.ftok_key)
