@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "jnitest_internal.h"
@@ -807,7 +807,7 @@ BOOLEAN isSystemPropertyEqual(JNIEnv* env, const char* utfPropName,  const char*
 jobject JNICALL
 Java_j9vm_utils_JNI_NewDirectByteBuffer(JNIEnv *env, jclass clazz, jlong address, jlong capacity)
 {
-	return (*env)->NewDirectByteBuffer(env, (void*)(UDATA)address, capacity);
+	return (*env)->NewDirectByteBuffer(env, JLONG_TO_POINTER(address), capacity);
 }
 
 /*
@@ -818,7 +818,7 @@ Java_j9vm_utils_JNI_NewDirectByteBuffer(JNIEnv *env, jclass clazz, jlong address
 jlong JNICALL
 Java_j9vm_utils_JNI_GetDirectBufferAddress(JNIEnv *env, jclass clazz, jobject buffer)
 {
-	return (jlong)(UDATA)((*env)->GetDirectBufferAddress(env, buffer));
+	return JLONG_FROM_POINTER((*env)->GetDirectBufferAddress(env, buffer));
 }
 
 /*
@@ -1022,3 +1022,18 @@ Java_j9vm_test_jni_PthreadTest_attachAndDetach(JNIEnv *env, jclass clazz)
     return worked;
 #endif /* defined(WIN32) */
 }
+
+jboolean JNICALL
+Java_org_openj9_test_jep425_VirtualThreadTests_lockSupportPark(JNIEnv *env, jclass cls)
+{
+	jboolean result = JNI_FALSE;
+	jclass clazz = (*env)->FindClass(env, "java/util/concurrent/locks/LockSupport");
+	if (NULL != clazz) {
+		jmethodID method = (*env)->GetStaticMethodID(env, clazz, "park", "()V");
+		if (NULL != method) {
+			(*env)->CallStaticObjectMethod(env, clazz, method);
+			result = JNI_TRUE;
+		}
+	}
+	return result;
+}
