@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2020 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #ifndef jnitest_internal_h
@@ -1742,6 +1742,15 @@ Java_jit_test_vich_JNIObjectArray_getObjectArrayElement(JNIEnv *env, jobject obj
 
 /* ---------------- jnitest.c ---------------- */
 
+/**
+ * @brief Invoke LockSupport.park().
+ * @param env
+ * @param cls
+ * @return JNI_TRUE if LockSupport.park() is invoked; otherwise, JNI_FALSE.
+ */
+jboolean JNICALL
+Java_org_openj9_test_jep425_VirtualThreadTests_lockSupportPark(JNIEnv *env, jclass cls);
+
 /**
 * @brief
 * @param env
@@ -2069,7 +2078,6 @@ Java_j9vm_test_thread_NativeHelpers_abort(JNIEnv *env, jclass cls, jobject threa
 void JNICALL 
 Java_j9vm_test_thread_NativeHelpers_priorityInterrupt(JNIEnv *env, jclass cls, jobject thread);
 
-
 /* ---------------- jniReturnInvalidReference.c ---------------- */
 jobject JNICALL 
 Java_j9vm_test_jnichk_ReturnInvalidReference_deletedLocalRef(JNIEnv * env, jobject caller);
