@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j9consts.h"
@@ -46,6 +46,9 @@ static void allSlotsInBytecodesDo (J9ROMClass* romClass, J9ROMMethod* method, J9
 static void allSlotsInCPShapeDescriptionDo (J9ROMClass* romClass, J9ROMClassWalkCallbacks* callbacks, void* userData);
 static void allSlotsInCallSiteDataDo (J9ROMClass* romClass, J9ROMClassWalkCallbacks* callbacks, void* userData);
 static UDATA allSlotsInMethodParametersDataDo(J9ROMClass* romClass, U_8* cursor, J9ROMClassWalkCallbacks* callbacks, void* userData);
+#if defined(J9VM_OPT_METHOD_HANDLE)
+static void allSlotsInVarHandleMethodTypeLookupTableDo(J9ROMClass* romClass, J9ROMClassWalkCallbacks* callbacks, void* userData);
+#endif /* defined(J9VM_OPT_METHOD_HANDLE) */
 static void allSlotsInStaticSplitMethodRefIndexesDo (J9ROMClass* romClass, J9ROMClassWalkCallbacks* callbacks, void* userData);
 static void allSlotsInSpecialSplitMethodRefIndexesDo (J9ROMClass* romClass, J9ROMClassWalkCallbacks* callbacks, void* userData);
 static void allSlotsInSourceDebugExtensionDo (J9ROMClass* romClass, J9SourceDebugExtension* sde, J9ROMClassWalkCallbacks* callbacks, void* userData);
@@ -55,7 +58,7 @@ static void nullSectionCallback(J9ROMClass*, void*, UDATA, const char*, void*);
 static BOOLEAN defaultValidateRangeCallback(J9ROMClass*, void*, UDATA, void*);
 
 #define SLOT_CALLBACK(romClassPtr, slotType, structure, slotName) callbacks->slotCallback(romClassPtr, slotType, &structure->slotName, #slotName, userData)
-/* 
+/*
  * Walk all slots in the J9ROMClass, calling callback for each slot.
  * Identify the type of slot using one of the J9ROM_ constants defined in the header file.
  * Note that NAS and UTF8 slots are treated specially, since, as invariants, they may
@@ -155,11 +158,20 @@ void allSlotsInROMClassDo(J9ROMClass* romClass,
 	SLOT_CALLBACK(romClass, J9ROM_U32,  romClass, invokeCacheCount);
 #else /* defined(J9VM_OPT_OPENJDK_METHODHANDLE) */
 	SLOT_CALLBACK(romClass, J9ROM_U32,  romClass, methodTypeCount);
+	SLOT_CALLBACK(romClass, J9ROM_U32,  romClass, varHandleMethodTypeCount);
 #endif /* defined(J9VM_OPT_OPENJDK_METHODHANDLE) */
+	SLOT_CALLBACK(romClass, J9ROM_U32,  romClass, bsmCount);
+	SLOT_CALLBACK(romClass, J9ROM_U32,  romClass, callSiteCount);
+	SLOT_CALLBACK(romClass, J9ROM_SRP,  romClass, callSiteData);
+	SLOT_CALLBACK(romClass, J9ROM_U32,  romClass, classFileSize);
+	SLOT_CALLBACK(romClass, J9ROM_U32,  romClass, classFileCPCount);
 	SLOT_CALLBACK(romClass, J9ROM_U16,  romClass, staticSplitMethodRefCount);
 	SLOT_CALLBACK(romClass, J9ROM_U16,  romClass, specialSplitMethodRefCount);
 	SLOT_CALLBACK(romClass, J9ROM_SRP,  romClass, staticSplitMethodRefIndexes);
 	SLOT_CALLBACK(romClass, J9ROM_SRP,  romClass, specialSplitMethodRefIndexes);
+#if defined(J9VM_OPT_METHOD_HANDLE)
+	SLOT_CALLBACK(romClass, J9ROM_SRP,  romClass, varHandleMethodTypeLookupTable);
+#endif /* defined(J9VM_OPT_METHOD_HANDLE) */
 
 	/* walk interfaces SRPs block */
 	srpCursor = J9ROMCLASS_INTERFACES(romClass);
@@ -210,8 +222,12 @@ void allSlotsInROMClassDo(J9ROMClass* romClass,
 #endif /* JAVA_SPEC_VERSION >= 11 */
 
 	/* add CP NAS section */
-	firstMethod = J9ROMCLASS_ROMMETHODS(romClass);
-	count = (U_32)(((UDATA)firstMethod - (UDATA)srpCursor) / (sizeof(J9SRP) * 2));
+	if (0 != romClass->romMethodCount) {
+		firstMethod = J9ROMCLASS_ROMMETHODS(romClass);
+		count = (U_32)(((UDATA)firstMethod - (UDATA)srpCursor) / (sizeof(J9SRP) * 2));
+	} else {
+		count = 0;
+	}
 	rangeValid = callbacks->validateRangeCallback(romClass, srpCursor, count * sizeof(J9SRP) * 2, userData);
 	if (rangeValid) {
 		callbacks->sectionCallback(romClass, srpCursor, count * sizeof(J9SRP) * 2, "cpNamesAndSignaturesSRPs", userData);
@@ -223,6 +239,9 @@ void allSlotsInROMClassDo(J9ROMClass* romClass,
 	allSlotsInConstantPoolDo(romClass, callbacks, userData);
 	allSlotsInCallSiteDataDo(romClass, callbacks, userData);
 	allSlotsInOptionalInfoDo(romClass, callbacks, userData);
+#if defined(J9VM_OPT_METHOD_HANDLE)
+	allSlotsInVarHandleMethodTypeLookupTableDo(romClass, callbacks, userData);
+#endif /* defined(J9VM_OPT_METHOD_HANDLE) */
 	allSlotsInStaticSplitMethodRefIndexesDo(romClass, callbacks, userData);
 	allSlotsInSpecialSplitMethodRefIndexesDo(romClass, callbacks, userData);
 }
@@ -315,7 +334,7 @@ static J9ROMMethod* allSlotsInROMMethodDo(J9ROMClass* romClass, J9ROMMethod* met
 		cursor += allSlotsInAnnotationDo(romClass, (U_32 *)cursor, "methodAnnotation", callbacks, userData);
 	}
 
-	
+
 	if (J9ROMMETHOD_HAS_PARAMETER_ANNOTATIONS(method)) {
 		cursor += allSlotsInAnnotationDo(romClass, (U_32 *)cursor, "parameterAnnotations", callbacks, userData);
 	}
@@ -323,7 +342,7 @@ static J9ROMMethod* allSlotsInROMMethodDo(J9ROMClass* romClass, J9ROMMethod* met
 	if (J9ROMMETHOD_HAS_DEFAULT_ANNOTATION(method)) {
 		cursor += allSlotsInAnnotationDo(romClass, (U_32 *)cursor, "defaultAnnotation", callbacks, userData);
 	}
-	
+
 	if (J9ROMMETHOD_HAS_METHOD_TYPE_ANNOTATIONS(extendedModifiers)) {
 		cursor += allSlotsInAnnotationDo(romClass, (U_32 *)cursor, "methodTypeAnnotations", callbacks, userData);
 	}
@@ -349,7 +368,7 @@ static J9ROMMethod* allSlotsInROMMethodDo(J9ROMClass* romClass, J9ROMMethod* met
 	}
 
 	if (J9ROMMETHOD_HAS_METHOD_PARAMETERS(method)) {
-	
+
 		cursor += allSlotsInMethodParametersDataDo(romClass, (U_8*)cursor, callbacks, userData);
 	}
 
@@ -452,7 +471,7 @@ static void allSlotsInBytecodesDo(J9ROMClass* romClass, J9ROMMethod* method, J9R
 	BOOLEAN rangeValid;
 
 	/* bytecodeSizeLow already walked */
-	length = J9_BYTECODE_SIZE_FROM_ROM_METHOD(method); 
+	length = J9_BYTECODE_SIZE_FROM_ROM_METHOD(method);
 
 	if (length == 0) return;
 	pc = bytecodes = J9_BYTECODE_START_FROM_ROM_METHOD(method); /* endian safe */
@@ -484,7 +503,7 @@ static void allSlotsInBytecodesDo(J9ROMClass* romClass, J9ROMMethod* method, J9R
 				callbacks->slotCallback(romClass, J9ROM_U8, pc, "bcArg8", userData);
 				pc += 1;
 				break;
-				
+
 			/* Single 16-bit argument */
 			case JBinvokeinterface2:
 				callbacks->slotCallback(romClass, J9ROM_U8, pc, "bcArg8", userData);
@@ -527,9 +546,6 @@ static void allSlotsInBytecodesDo(J9ROMClass* romClass, J9ROMMethod* method, J9R
 			case JBputstatic:
 			case JBgetfield:
 			case JBputfield:
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-			case JBwithfield:
-#endif
 			case JBinvokevirtual:
 			case JBinvokespecial:
 			case JBinvokestatic:
@@ -538,9 +554,6 @@ static void allSlotsInBytecodesDo(J9ROMClass* romClass, J9ROMMethod* method, J9R
 			case JBinvokedynamic:
 			case JBinvokeinterface:
 			case JBnew:
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-			case JBaconst_init:
-#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
 			case JBnewdup:
 			case JBanewarray:
 			case JBcheckcast:
@@ -605,7 +618,7 @@ static void allSlotsInBytecodesDo(J9ROMClass* romClass, J9ROMMethod* method, J9R
 					pc += 4;
 				}
 				break;
-				
+
 
 			case JBlookupswitch:
 				switch((pc - bytecodes - 1) % 4) {
@@ -810,7 +823,7 @@ static void allSlotsInRecordDo(J9ROMClass* romClass, U_32* recordPointer, J9ROMC
 	callbacks->sectionCallback(romClass, recordPointer, (UDATA)cursor - (UDATA)recordPointer, "recordInfo", userData);
 }
 
-static void 
+static void
 allSlotsInPermittedSubclassesDo(J9ROMClass* romClass, U_32* permittedSubclassesPointer, J9ROMClassWalkCallbacks* callbacks, void* userData)
 {
 	BOOLEAN rangeValid = FALSE;
@@ -825,7 +838,7 @@ allSlotsInPermittedSubclassesDo(J9ROMClass* romClass, U_32* permittedSubclassesP
 	callbacks->slotCallback(romClass, J9ROM_U32, cursor, "permittedSubclassesCount", userData);
 	cursor += 1;
 	permittedSubclassesCount = *permittedSubclassesPointer;
-	
+
 	for (; permittedSubclassesCount > 0; permittedSubclassesCount--) {
 		rangeValid = callbacks->validateRangeCallback(romClass, cursor, sizeof(U_32), userData);
 		if (FALSE == rangeValid) {
@@ -838,10 +851,58 @@ allSlotsInPermittedSubclassesDo(J9ROMClass* romClass, U_32* permittedSubclassesP
 	callbacks->sectionCallback(romClass, permittedSubclassesPointer, (UDATA)cursor - (UDATA)permittedSubclassesPointer, "permittedSubclassesInfo", userData);
 }
 
+#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
+static void
+allSlotsInloadableDescriptorsAttributeDo(J9ROMClass *romClass, U_32 *loadableDescriptorsAttributePointer, J9ROMClassWalkCallbacks *callbacks, void *userData)
+{
+	BOOLEAN rangeValid = FALSE;
+	U_32 *cursor = loadableDescriptorsAttributePointer;
+	U_32 loadableDescriptorsAttributeCount = 0;
+
+	rangeValid = callbacks->validateRangeCallback(romClass, cursor, sizeof(U_32), userData);
+	if (FALSE == rangeValid) {
+		return;
+	}
+
+	callbacks->slotCallback(romClass, J9ROM_U32, cursor, "loadableDescriptorsAttributeCount", userData);
+	cursor += 1;
+	loadableDescriptorsAttributeCount = *loadableDescriptorsAttributePointer;
+
+	for (; loadableDescriptorsAttributeCount > 0; loadableDescriptorsAttributeCount--) {
+		rangeValid = callbacks->validateRangeCallback(romClass, cursor, sizeof(U_32), userData);
+		if (FALSE == rangeValid) {
+			return;
+		}
+		callbacks->slotCallback(romClass, J9ROM_UTF8, cursor, "descriptorName", userData);
+		cursor += 1;
+	}
+
+	callbacks->sectionCallback(romClass, loadableDescriptorsAttributePointer, (UDATA)cursor - (UDATA)loadableDescriptorsAttributePointer, "loadableDescriptorsAttributeInfo", userData);
+}
+#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
+
+#if defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES)
+static void
+allSlotsInImplicitCreationAttributeDo(J9ROMClass* romClass, U_32* implicitCreationAttributePointer, J9ROMClassWalkCallbacks* callbacks, void* userData)
+{
+	BOOLEAN rangeValid = FALSE;
+	U_32 *cursor = implicitCreationAttributePointer;
+
+	rangeValid = callbacks->validateRangeCallback(romClass, cursor, sizeof(U_32), userData);
+	if (FALSE == rangeValid) {
+		return;
+	}
+
+	callbacks->slotCallback(romClass, J9ROM_U32, cursor, "implicitCreationFlags", userData);
+	cursor += 1;
+
+	callbacks->sectionCallback(romClass, implicitCreationAttributePointer, (UDATA)cursor - (UDATA)implicitCreationAttributePointer, "implicitCreationAttributeInfo", userData);
+}
+#endif /* defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES) */
 /*
  * See ROMClassWriter::writeOptionalInfo for illustration of the layout.
  */
-static void 
+static void
 allSlotsInOptionalInfoDo(J9ROMClass* romClass, J9ROMClassWalkCallbacks* callbacks, void* userData)
 {
 	U_32 *optionalInfo;
@@ -935,11 +996,53 @@ allSlotsInOptionalInfoDo(J9ROMClass* romClass, J9ROMClassWalkCallbacks* callback
 		}
 		cursor++;
 	}
-#endif /* J9VM_OPT_VALHALLA_VALUE_TYPES */
+
+	if (J9_ARE_ANY_BITS_SET(romClass->optionalFlags, J9_ROMCLASS_OPTINFO_LOADABLEDESCRIPTORS_ATTRIBUTE)) {
+		rangeValid = callbacks->validateRangeCallback(romClass, cursor, sizeof(J9SRP), userData);
+		if (rangeValid) {
+			callbacks->slotCallback(romClass, J9ROM_SRP, cursor, "loadableDescriptorsAttributeSRP", userData);
+			allSlotsInloadableDescriptorsAttributeDo(romClass, SRP_PTR_GET(cursor, U_32*), callbacks, userData);
+		}
+		cursor++;
+	}
+#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
+#if defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES)
+	if (J9_ARE_ANY_BITS_SET(romClass->optionalFlags, J9_ROMCLASS_OPTINFO_IMPLICITCREATION_ATTRIBUTE)) {
+		rangeValid = callbacks->validateRangeCallback(romClass, cursor, sizeof(J9SRP), userData);
+		if (rangeValid) {
+			callbacks->slotCallback(romClass, J9ROM_SRP, cursor, "implicitCreationAttributeSRP", userData);
+			allSlotsInImplicitCreationAttributeDo(romClass, SRP_PTR_GET(cursor, U_32*), callbacks, userData);
+		}
+		cursor++;
+	}
+#endif /* defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES) */
 	callbacks->sectionCallback(romClass, optionalInfo, (UDATA)cursor - (UDATA)optionalInfo, "optionalInfo", userData);
 }
 
-static void 
+#if defined(J9VM_OPT_METHOD_HANDLE)
+static void
+allSlotsInVarHandleMethodTypeLookupTableDo(J9ROMClass* romClass, J9ROMClassWalkCallbacks* callbacks, void* userData)
+{
+	U_32 count = romClass->varHandleMethodTypeCount;
+
+	if (count > 0) {
+		U_16 *cursor = J9ROMCLASS_VARHANDLEMETHODTYPELOOKUPTABLE(romClass);
+		BOOLEAN rangeValid = callbacks->validateRangeCallback(romClass, cursor, count * sizeof(U_16), userData);
+
+		if (rangeValid) {
+			U_32 i = 0;
+
+			callbacks->sectionCallback(romClass, cursor, count * sizeof(U_16), "varHandleMethodTypeLookupTable", userData);
+			for (i = 0; i < count; i++) {
+				callbacks->slotCallback(romClass, J9ROM_U16, cursor, "cpIndex", userData);
+				cursor += 1;
+			}
+		}
+	}
+}
+#endif /* defined(J9VM_OPT_METHOD_HANDLE) */
+
+static void
 allSlotsInStaticSplitMethodRefIndexesDo(J9ROMClass* romClass, J9ROMClassWalkCallbacks* callbacks, void* userData)
 {
 	U_16 count = romClass->staticSplitMethodRefCount;
@@ -960,7 +1063,7 @@ allSlotsInStaticSplitMethodRefIndexesDo(J9ROMClass* romClass, J9ROMClassWalkCall
 	}
 }
 
-static void 
+static void
 allSlotsInSpecialSplitMethodRefIndexesDo(J9ROMClass* romClass, J9ROMClassWalkCallbacks* callbacks, void* userData)
 {
 	U_16 count = romClass->specialSplitMethodRefCount;
@@ -1361,7 +1464,7 @@ static void allSlotsInCallSiteDataDo (J9ROMClass* romClass, J9ROMClassWalkCallba
 	 *			U_16 argument[argumentCount];
 	 *		} bootStrapMethodData[romClass->bsmCount];
 	 * }
-	 * 
+	 *
 	 * Note: SRP is 32 bits
 	 */
 	BOOLEAN rangeValid;
