@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include <string.h>
@@ -95,7 +95,7 @@ findArgInVMArgs(J9PortLibrary *portLibrary, J9VMInitArgs* j9vm_args, UDATA match
 
 	PORT_ACCESS_FROM_PORT(portLibrary);
 
-	if (optionName==NULL || 0==j9vm_args->nOptions) {
+	if ((NULL == optionName) || (NULL == j9vm_args) || (0 == j9vm_args->nOptions)) {
 		return returnVal;
 	}
 
@@ -341,7 +341,8 @@ optionValueOperations(J9PortLibrary *portLibrary, J9VMInitArgs* j9vm_args, IDATA
 	char *values, *scanStart;
 	char *cursor;
 	IDATA sepCount = 0;
-	UDATA value, oldValue, done;
+	UDATA value = 0;
+	UDATA done = 0;
 	IDATA errorFound = 0;
 	UDATA mapType;
 	double dvalue = 0.0;
@@ -577,9 +578,17 @@ optionValueOperations(J9PortLibrary *portLibrary, J9VMInitArgs* j9vm_args, IDATA
 				} else {
 					switch (*cursor) {
 						case '\0':
-							oldValue = value;
-							value = (value +  sizeof(UDATA) - 1) & ~(sizeof(UDATA) - 1);		/* round to nearest pointer value */
-							if (value < oldValue) return OPTION_OVERFLOW;
+#if defined(J9VM_ENV_DATA64)
+#define MEM_MAX_LIMIT I_64_MAX
+#else /* defined(J9VM_ENV_DATA64) */
+#define MEM_MAX_LIMIT U_32_MAX
+#endif /* defined(J9VM_ENV_DATA64) */
+							/* Round (up if possible) to a multiple of the pointer width. */
+							if (value < ((UDATA)MEM_MAX_LIMIT & ~(sizeof(UDATA) - 1))) {
+								value += sizeof(UDATA) - 1;
+							}
+							value &= ~(sizeof(UDATA) - 1);
+#undef MEM_MAX_LIMIT
 							break;
 						case 'k':
 						case 'K':
@@ -648,7 +657,7 @@ optionValueOperations(J9PortLibrary *portLibrary, J9VMInitArgs* j9vm_args, IDATA
 	return OPTION_OK;
 }
 
-static char * 
+static char *
 getStartOfOptionValue(J9VMInitArgs* j9vm_args, IDATA element, const char *optionName)
 {
 	const char *option = optionName;
@@ -659,7 +668,7 @@ getStartOfOptionValue(J9VMInitArgs* j9vm_args, IDATA element, const char *option
 	}
 
 	Assert_Util_true(NULL != option);
-	
+
 	value = j9vm_args->actualVMArgs->options[element].optionString + strlen(option);
 
 	return value;
@@ -674,42 +683,13 @@ addOptionsDefaultFile(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsL
 	UDATA resultLength  = 0;
 	PORT_ACCESS_FROM_PORT(portLib);
 
-	resultLength  = j9str_printf(PORTLIB, optionsArgumentBuffer, sizeof(optionsArgumentBuffer), VMOPT_XOPTIONSFILE_EQUALS "%s" DIR_SEPARATOR_STR OPTIONS_DEFAULT, optionsDirectory);
+	resultLength  = j9str_printf(optionsArgumentBuffer, sizeof(optionsArgumentBuffer), VMOPT_XOPTIONSFILE_EQUALS "%s" DIR_SEPARATOR_STR OPTIONS_DEFAULT, optionsDirectory);
 	if (resultLength > (sizeof(VMOPT_XOPTIONSFILE_EQUALS) + MAX_PATH)) {
 		return -1; /* overflow */
 	}
 	return addXOptionsFile(portLib, optionsArgumentBuffer, vmArgumentsList, verboseFlags);
 }
 
-IDATA
-addXjcl(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList, UDATA j2seVersion)
-{
-	char *dllName = J9_JAVA_SE_DLL_NAME;
-	size_t dllNameLength = sizeof(J9_JAVA_SE_DLL_NAME);
-	size_t argumentLength = -1;
-	char *argString = NULL;
-	UDATA j2seReleaseValue = j2seVersion & J2SE_RELEASE_MASK;
-	J9JavaVMArgInfo *optArg = NULL;
-	
-	PORT_ACCESS_FROM_PORT(portLib);
-#ifdef J9VM_IVE_RAW_BUILD /* J9VM_IVE_RAW_BUILD is not enabled by default */
-	Assert_Util_unreachable();
-#endif /* J9VM_IVE_RAW_BUILD */
-
-	argumentLength = sizeof(VMOPT_XJCL_COLON) + dllNameLength - 1; /* sizeof called twice, each includes the \0 */
-	argString = j9mem_allocate_memory(argumentLength, OMRMEM_CATEGORY_VM);
-	if (NULL == argString) {
-		return -1;
-	}
-	j9str_printf(PORTLIB, argString, argumentLength, VMOPT_XJCL_COLON "%s", dllName);
-	optArg = newJavaVMArgInfo(vmArgumentsList, argString, ARG_MEMORY_ALLOCATION | CONSUMABLE_ARG);
-	if (NULL == optArg) {
-		j9mem_free_memory(argString);
-		return -1;
-	}
-	return 0;
-}
-
 IDATA
 addBootLibraryPath(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList, char *propertyNameEquals, char *j9binPath, char *jrebinPath)
 {
@@ -726,7 +706,7 @@ addBootLibraryPath(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList
 		return -1;
 	}
 
-	j9str_printf(PORTLIB, optionsArgumentBuffer, argumentLength, "%s%s" J9JAVA_PATH_SEPARATOR "%s", propertyNameEquals, j9binPath, jrebinPath);
+	j9str_printf(optionsArgumentBuffer, argumentLength, "%s%s" J9JAVA_PATH_SEPARATOR "%s", propertyNameEquals, j9binPath, jrebinPath);
 
 	optArg = newJavaVMArgInfo(vmArgumentsList, optionsArgumentBuffer, ARG_MEMORY_ALLOCATION|CONSUMABLE_ARG);
 	if (NULL == optArg) {
@@ -863,6 +843,8 @@ addJavaLibraryPath(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList
 		substringLength += strlen(pathBuffer);
 		substringIndex;
 		substringIndex += 1;
+	} else {
+		j9mem_free_memory(pathBuffer);
 	}
 
 #else /* defined(J9UNIX) || defined(J9ZOS390) */
@@ -984,7 +966,7 @@ addJavaHome(J9PortLibrary *portLib, J9JavaVMArgInfoList *vmArgumentsList, UDATA
 		if (NULL == optionsArgumentBuffer) {
 			return -1;
 		}
-		j9str_printf(PORTLIB, optionsArgumentBuffer, argumentLength, JAVA_HOME_EQUALS "%s" J9JAVA_PATH_SEPARATOR "%s", altJavaHomeBuffer, jrelibPath);
+		j9str_printf(optionsArgumentBuffer, argumentLength, JAVA_HOME_EQUALS "%s" J9JAVA_PATH_SEPARATOR "%s", altJavaHomeBuffer, jrelibPath);
 	} else
 #endif /* WIN32 */
 	{
@@ -1058,7 +1040,7 @@ addExtDir(J9PortLibrary *portLib, J9JavaVMArgInfoList *vmArgumentsList, char *jr
 }
 
 IDATA
-addUserDir(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList, char *cwd) 
+addUserDir(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList, char *cwd)
 {
 	size_t argumentLength = 1; /* space for the \0 */
 	char *optionsArgumentBuffer = NULL;
@@ -1073,7 +1055,7 @@ addUserDir(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList, char *
 		return -1;
 	}
 
-	j9str_printf(PORTLIB, optionsArgumentBuffer, argumentLength, JAVA_USER_DIR_EQUALS "%s", cwd);
+	j9str_printf(optionsArgumentBuffer, argumentLength, JAVA_USER_DIR_EQUALS "%s", cwd);
 
 	optArg = newJavaVMArgInfo(vmArgumentsList, optionsArgumentBuffer, ARG_MEMORY_ALLOCATION|CONSUMABLE_ARG);
 	if (NULL == optArg) {
@@ -1302,7 +1284,7 @@ addJarArguments(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList, c
  * @return 0 on success, negative on failure
  */
 static IDATA
-mapEnvVarToArgument(J9PortLibrary* portLibrary, const char* envVar, const char* j9opt, J9JavaVMArgInfoList *vmArgumentsList, UDATA mapType, UDATA verboseFlags) 
+mapEnvVarToArgument(J9PortLibrary* portLibrary, const char* envVar, const char* j9opt, J9JavaVMArgInfoList *vmArgumentsList, UDATA mapType, UDATA verboseFlags)
 {
 	J9JavaVMArgInfo *optArg = NULL;
 	IDATA valueSize = 0;
@@ -1363,7 +1345,7 @@ mapEnvVarToArgument(J9PortLibrary* portLibrary, const char* envVar, const char*
 
 
 IDATA
-addEnvironmentVariableArguments(J9PortLibrary *portLib, const char *envVarName, J9JavaVMArgInfoList *vmArgumentsList, UDATA verboseFlags) 
+addEnvironmentVariableArguments(J9PortLibrary *portLib, const char *envVarName, J9JavaVMArgInfoList *vmArgumentsList, UDATA verboseFlags)
 {
 	PORT_ACCESS_FROM_PORT(portLib);
 	IDATA envVarSize = j9sysinfo_get_env(envVarName, NULL, 0);
@@ -1383,12 +1365,14 @@ addEnvironmentVariableArguments(J9PortLibrary *portLib, const char *envVarName,
 }
 
 IDATA
-addEnvironmentVariables(J9PortLibrary *portLib, JavaVMInitArgs *launcherArgs, J9JavaVMArgInfoList *vmArgumentsList, UDATA verboseFlags) 
+addEnvironmentVariables(J9PortLibrary *portLib, JavaVMInitArgs *launcherArgs, J9JavaVMArgInfoList *vmArgumentsList, UDATA verboseFlags)
 {
 	IDATA status = 0;
 	if (
 			(0 != mapEnvVarToArgument(portLib, ENVVAR_IBM_MIXED_MODE_THRESHOLD, MAPOPT_XJIT_COUNT_EQUALS, vmArgumentsList, EXACT_MAP_WITH_OPTIONS, verboseFlags))
+#if JAVA_SPEC_VERSION < 21
 			|| (0 != mapEnvVarToArgument(portLib, ENVVAR_JAVA_COMPILER, SYSPROP_DJAVA_COMPILER_EQUALS, vmArgumentsList, EXACT_MAP_WITH_OPTIONS, verboseFlags))
+#endif /* JAVA_SPEC_VERSION < 21 */
 			|| (0 != mapEnvVarToArgument(portLib, ENVVAR_IBM_NOSIGHANDLER, VMOPT_XRS, vmArgumentsList, EXACT_MAP_NO_OPTIONS, verboseFlags))
 #if defined(J9ZOS390)
 			|| (0 != mapEnvVarToArgument(portLib, ENVVAR_JAVA_THREAD_MODEL, MAPOPT_XTHR_TW_EQUALS, vmArgumentsList, EXACT_MAP_WITH_OPTIONS, verboseFlags))
@@ -1536,7 +1520,7 @@ addLauncherArgs(J9PortLibrary * portLib, JavaVMInitArgs *launcherArgs, UDATA lau
 }
 
 IDATA
-addXserviceArgs(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList, char *xServiceBuffer, UDATA verboseFlags) 
+addXserviceArgs(J9PortLibrary * portLib, J9JavaVMArgInfoList *vmArgumentsList, char *xServiceBuffer, UDATA verboseFlags)
 {
 
 	PORT_ACCESS_FROM_PORT(portLib);
