@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2015, 2020 IBM Corp. and others
+ * Copyright IBM Corp. and others 2015
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "errormessage_internal.h"
@@ -111,12 +111,12 @@ getJ9RtvUTF8StringfromCP(J9UTF8Ref* buf, void* constantPool, UDATA index)
  * @param classIndex - class index in the classNameList array
  */
 static void
-getJ9RtvStringfromClassNameList(J9UTF8Ref* buf, void* classNameListPtr, void* romClass, UDATA classIndex)
+getJ9RtvStringfromClassNameList(J9UTF8Ref *buf, void *classNameListPtr, void *romClass, UDATA classIndex)
 {
-	struct J9UTF8** classNameList = (struct J9UTF8**)classNameListPtr;
-	U_32* offset = (U_32*)classNameList[BCV_INDEX_FROM_TYPE(classIndex)];
+	J9UTF8 **classNameList = (J9UTF8 **)classNameListPtr;
+	U_32 *offset = (U_32 *)classNameList[BCV_INDEX_FROM_TYPE(classIndex)];
 
-	buf->length = J9UTF8_LENGTH(offset + 1);
+	buf->length = J9UTF8_LENGTH((J9UTF8 *)(offset + 1));
 
 	/* addClassName() in vrfyhelp.c:
 	 * Note: offset[0] is set up in addClassName() to determine where to obtain the class name (classNameSegment or romClass).
@@ -125,10 +125,10 @@ getJ9RtvStringfromClassNameList(J9UTF8Ref* buf, void* classNameListPtr, void* ro
 	 */
 	if (0 == offset[0]) {
 		/* class name exists in the classNameSegment */
-		buf->bytes = (U_8*)J9UTF8_DATA(offset + 1);
+		buf->bytes = J9UTF8_DATA((J9UTF8 *)(offset + 1));
 	} else {
 		/* class name exists in the ROM class */
-		buf->bytes = (U_8*)((UDATA) offset[0] + (UDATA) romClass);
+		buf->bytes = (U_8 *)((UDATA)offset[0] + (UDATA)romClass);
 	}
 
 	buf->arity = (U_8)BCV_ARITY_FROM_TYPE(classIndex);
@@ -222,8 +222,8 @@ pushLiveStackToVerificationTypeBuffer(StackMapFrame* stackMapFrame, J9BytecodeVe
 	U_16 maxLocals = methodInfo->maxLocals;
 	IDATA lastIndex = 0;
 	IDATA slot = 0;
-
-	IDATA errorPositionInclusive = 1;
+	/* Don't print anything on the empty stack. */
+	IDATA errorPositionInclusive = (BCV_ERR_STACK_UNDERFLOW == verifyData->errorDetailCode) ? 0 : 1;
 	IDATA wideType = DATATYPE_1_SLOT;
 	BOOLEAN result = TRUE;
 	BOOLEAN nonTopFound = FALSE;
@@ -498,6 +498,7 @@ adjustStackTopForWrongDataType(J9BytecodeVerificationData *verifyData)
 	case BCV_ERR_STACKMAP_FRAME_LOCALS_UNDERFLOW:	/* FALLTHROUGH */
 	case BCV_ERR_STACKMAP_FRAME_LOCALS_OVERFLOW:	/* FALLTHROUGH */
 	case BCV_ERR_STACKMAP_FRAME_STACK_OVERFLOW:		/* FALLTHROUGH */
+	case BCV_ERR_INIT_FLAGS_MISMATCH:				/* FALLTHROUGH */
 	case BCV_ERR_INIT_NOT_CALL_INIT:				/* FALLTHROUGH */
 	case BCV_ERR_WRONG_TOP_TYPE:					/* FALLTHROUGH */
 	case BCV_ERR_INVALID_ARRAY_REFERENCE:			/* FALLTHROUGH */
@@ -766,14 +767,13 @@ printReasonForFlagMismatch(MessageBuffer *msgBuf, J9BytecodeVerificationData *ve
 				goto exit;
 			}
 
-			/* Compare the flag value if the bci value matches */
-			if (currentFrame->bci == targetFrame->bci) {
-				BOOLEAN matchFlag = compareStackMapFrameFlag(currentFrame, targetFrame);
-				if (!matchFlag) {
-					printMessage(msgBuf, "Current frame's flags are not assignable to stack map frame's.");
-					result = TRUE;
-					goto exit;
-				}
+			/* Compare the flag value regardless of the bci value as the comparison
+			 * might happen to the branch bytecode specified in the stackmap frame.
+			 */
+			if (FALSE == compareStackMapFrameFlag(currentFrame, targetFrame)) {
+				printMessage(msgBuf, "Current frame's flags are not assignable to stack map frame's.");
+				result = TRUE;
+				goto exit;
 			}
 		}
 	}
@@ -865,6 +865,7 @@ generateJ9RtvExceptionDetails(J9BytecodeVerificationData* verifyData, U_8* initM
 		printMessage(&msgBuf, "Current frame's stack size doesn't match stackmap.");
 		printStackFrame = setStackMapFrameWithIndex(verifyData, &methodInfo, &stackMapFrameTarget);
 		break;
+	case BCV_ERR_INIT_FLAGS_MISMATCH: /* FALLTHROUGH */
 	case BCV_ERR_INIT_NOT_CALL_INIT:
 		printStackFrame = printReasonForFlagMismatch(&msgBuf, verifyData, &methodInfo, &stackMapFrameCurrent, &stackMapFrameTarget);
 		printCurrentStack = printStackFrame;
