@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2001, 2020 IBM Corp. and others
+ * Copyright IBM Corp. and others 2001
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j9protos.h"
@@ -32,6 +32,7 @@
 #include "VMHelpers.hpp"
 #include "AtomicSupport.hpp"
 #include "ObjectMonitor.hpp"
+#include "VMAccess.hpp"
 
 extern "C" {
 
@@ -139,6 +140,12 @@ objectMonitorEnterBlocking(J9VMThread *currentThread)
 	UDATA result = 0;
 	j9object_t object = J9VMTHREAD_BLOCKINGENTEROBJECT(currentThread, currentThread);
 	J9Class *ramClass = J9OBJECT_CLAZZ(currentThread, object);
+	J9JavaVM *vm = currentThread->javaVM;
+	PORT_ACCESS_FROM_JAVAVM(vm);
+	I_64 startTicks = 0;
+	if (J9_EVENT_IS_HOOKED(vm->hookInterface, J9HOOK_VM_MONITOR_CONTENDED_ENTERED)) {
+		startTicks = j9time_nano_time();
+	}
 	/* Throughout this function, note that inlineGetLockAddress cannot run into out of memory case because
 	 * an entry in monitor table will have been created by the earlier call in objectMonitorEnterNonBlocking.
 	 */
@@ -158,28 +165,40 @@ objectMonitorEnterBlocking(J9VMThread *currentThread)
 	}
 #endif /* J9VM_THR_LOCK_RESERVATION */
 	{
-		J9JavaVM *vm = currentThread->javaVM;
 		J9ObjectMonitor *objectMonitor = monitorTableAt(currentThread, object);
 		/* Table entry was created by the nonblocking case, so this peek cannot fail */
 		Assert_VM_notNull(objectMonitor);
 		object = NULL; // for safety, since object may be moved by the GC at various points after this
 		/* Ensure object monitor isn't deflated while we block */
 		omrthread_monitor_t monitor = objectMonitor->monitor;
+		J9VMThread *previousOwner = NULL;
+		omrthread_t previousOMRThreadOwner = ((J9ThreadMonitor *)monitor)->owner;
 		VM_AtomicSupport::add(&monitor->pinCount, 1);
 		/* Initialize our wait time to 1ms. Increase it as we have to wait more and more
 		 * using the sequence 1, 4, 16, 64 and then 64 thereafter.
 		 */
 		IDATA waitTime = 1;
+
+		if ((NULL != previousOMRThreadOwner) && !IS_J9_OBJECT_MONITOR_OWNER_DETACHED(previousOMRThreadOwner)) {
+			previousOwner = getVMThreadFromOMRThread(vm, previousOMRThreadOwner);
+		}
+
 		if (J9_EVENT_IS_HOOKED(vm->hookInterface, J9HOOK_VM_MONITOR_CONTENDED_ENTER)) {
 			bool frameBuilt = saveBlockingEnterObject(currentThread);
+			VM_VMAccess::setPublicFlags(currentThread, J9_PUBLIC_FLAGS_THREAD_BLOCKED);
 			ALWAYS_TRIGGER_J9HOOK_VM_MONITOR_CONTENDED_ENTER(vm->hookInterface, currentThread, monitor);
 			restoreBlockingEnterObject(currentThread, frameBuilt);
 		}
 		omrthread_t const osThread = currentThread->osThread;
 		/* Update j.l.management info */
 		currentThread->mgmtBlockedCount += 1;
+		if (J9_ARE_ALL_BITS_SET(currentThread->publicFlags, J9_PUBLIC_FLAGS_THREAD_BLOCKED)) {
+			internalReleaseVMAccess(currentThread);
+			goto releasedAccess;
+		}
 restart:
 		internalReleaseVMAccessSetStatus(currentThread, J9_PUBLIC_FLAGS_THREAD_BLOCKED);
+releasedAccess:
 		omrthread_monitor_enter_using_threadId(monitor, osThread);
 #if defined(J9VM_THR_SMART_DEFLATION)
 		/* Update the anti-deflation vote because we had to block */
@@ -190,6 +209,9 @@ restart:
 			if (J9_ARE_ANY_BITS_SET(((J9ThreadMonitor*)monitor)->flags, J9THREAD_MONITOR_INFLATED)) {
 				Trc_VM_objectMonitorEnterBlocking_alreadyInflated(currentThread);
 				internalAcquireVMAccess(currentThread);
+#if JAVA_SPEC_VERSION >= 19
+				currentThread->ownedMonitorCount += 1;
+#endif /* JAVA_SPEC_VERSION >= 19 */
 				goto done;
 			}
 			if (0 != internalTryAcquireVMAccess(currentThread)) {
@@ -280,7 +302,7 @@ done:
 		VM_AtomicSupport::subtract(&monitor->pinCount, 1);
 		if (J9_EVENT_IS_HOOKED(vm->hookInterface, J9HOOK_VM_MONITOR_CONTENDED_ENTERED)) {
 			bool frameBuilt = saveBlockingEnterObject(currentThread);
-			ALWAYS_TRIGGER_J9HOOK_VM_MONITOR_CONTENDED_ENTERED(vm->hookInterface, currentThread, monitor);
+			ALWAYS_TRIGGER_J9HOOK_VM_MONITOR_CONTENDED_ENTERED(vm->hookInterface, currentThread, monitor, startTicks, ramClass, previousOwner);
 			restoreBlockingEnterObject(currentThread, frameBuilt);
 		}
 	}
@@ -308,23 +330,27 @@ UDATA
 objectMonitorEnterNonBlocking(J9VMThread *currentThread, j9object_t object)
 {
 	UDATA result = (UDATA)object;
+	J9JavaVM *vm = currentThread->javaVM;
 	j9objectmonitor_t volatile *lwEA = VM_ObjectMonitor::inlineGetLockAddress(currentThread, object);
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES) || (JAVA_SPEC_VERSION >= 16)
+#if JAVA_SPEC_VERSION >= 16
 	J9Class * objClass = J9OBJECT_CLAZZ(currentThread, object);
-#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) || (JAVA_SPEC_VERSION >= 16) */
+#endif /* JAVA_SPEC_VERSION >= 16 */
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+	BOOLEAN retry = FALSE;
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
 
 #if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
 	if (J9_IS_J9CLASS_VALUETYPE(objClass)) {
 		result = J9_OBJECT_MONITOR_VALUE_TYPE_IMSE;
 		goto done;
 	}
-#endif /* J9VM_OPT_VALHALLA_VALUE_TYPES */	
+#endif /* J9VM_OPT_VALHALLA_VALUE_TYPES */
 #if JAVA_SPEC_VERSION >= 16
 	if (J9_IS_J9CLASS_VALUEBASED(objClass)) {
-		U_32 runtimeFlags2 = currentThread->javaVM->extendedRuntimeFlags2;
+		U_32 runtimeFlags2 = vm->extendedRuntimeFlags2;
 		if (J9_ARE_ALL_BITS_SET(runtimeFlags2, J9_EXTENDED_RUNTIME2_VALUE_BASED_EXCEPTION)) {
 			result = J9_OBJECT_MONITOR_VALUE_TYPE_IMSE;
-			goto done;	
+			goto done;
 		} else if (J9_ARE_ALL_BITS_SET(runtimeFlags2, J9_EXTENDED_RUNTIME2_VALUE_BASED_WARNING)) {
 			PORT_ACCESS_FROM_VMC(currentThread);
 			const J9UTF8* className = J9ROMCLASS_CLASSNAME(J9OBJECT_CLAZZ(currentThread, object)->romClass);
@@ -332,7 +358,7 @@ objectMonitorEnterNonBlocking(J9VMThread *currentThread, j9object_t object)
 		}
 	}
 #endif /* JAVA_SPEC_VERSION >= 16 */
-	
+
 restart:
 	if (NULL == lwEA) {
 		/* out of memory */
@@ -367,7 +393,7 @@ restart:
 				 * When the Learning Counter reaches reservedTransitionThreshold, the object transitions from Learning to Reserved.
 				 * reservedTransitionThreshold is being checked against a 2 bit value so values of 4 or higher will prevent transitions to Reserved.
 				 */
-				U_32 reservedTransitionThreshold = currentThread->javaVM->reservedTransitionThreshold;
+				U_32 reservedTransitionThreshold = vm->reservedTransitionThreshold;
 				bool reservedTransition = false;
 				if (((lock & OBJECT_HEADER_LOCK_LEARNING_LC_MASK) >> OBJECT_HEADER_LOCK_LEARNING_LC_OFFSET) >= reservedTransitionThreshold) {
 					/*
@@ -403,6 +429,9 @@ restart:
 					goto restart;
 				}
 			}
+#if JAVA_SPEC_VERSION >= 19
+			currentThread->ownedMonitorCount += 1;
+#endif /* JAVA_SPEC_VERSION >= 19 */
 			/* no barrier is required in the recursive case */
 		} else {
 			/* check to see if object is unlocked (JIT did not do initial inline sequence due to insufficient static type info) */
@@ -484,7 +513,19 @@ restart:
 wouldBlock:
 	/* unable to get thin lock by spinning - follow blocking path */
 	J9VMTHREAD_SET_BLOCKINGENTEROBJECT(currentThread, currentThread, object);
-	result = J9_OBJECT_MONITOR_BLOCKING;
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+	if (J9_THROW_BLOCKING_EXCEPTION_IN_SINGLE_THREAD_MODE(vm)) {
+		if (OBJECT_HEADER_LOCK_RESERVED == (J9_LOAD_LOCKWORD(currentThread, lwEA) & (OBJECT_HEADER_LOCK_RESERVED + OBJECT_HEADER_LOCK_INFLATED)) && !retry) {
+			cancelLockReservation(currentThread);
+			retry = TRUE;
+			goto restart;
+		}
+		result = J9_OBJECT_MONITOR_CRIU_SINGLE_THREAD_MODE_THROW;
+	} else
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
+	{
+		result = J9_OBJECT_MONITOR_BLOCKING;
+	}
 done:
 	return result;
 }
@@ -553,6 +594,9 @@ spinOnFlatLock(J9VMThread *currentThread, j9objectmonitor_t volatile *lwEA, j9ob
 						if (lock == VM_ObjectMonitor::compareAndSwapLockword(currentThread, lwEA, lock, (j9objectmonitor_t)(UDATA)currentThread, false)) {
 							/* compare and swap succeeded */
 							VM_AtomicSupport::readBarrier();
+#if JAVA_SPEC_VERSION >= 19
+							currentThread->ownedMonitorCount += 1;
+#endif /* JAVA_SPEC_VERSION >= 19 */
 							rc = true;
 
 							/* Transition from Learning to Flat occurred so the Cancel Counter in the object's J9Class is incremented by 1. */
@@ -696,6 +740,9 @@ spinOnTryEnter(J9VMThread *currentThread, J9ObjectMonitor *objectMonitor, j9obje
 				if (J9_LOCK_IS_INFLATED(J9_LOAD_LOCKWORD(currentThread, lwEA))) {
 					/* try_enter succeeded - monitor is inflated */
 					rc = true;
+#if JAVA_SPEC_VERSION >= 19
+					currentThread->ownedMonitorCount += 1;
+#endif /* JAVA_SPEC_VERSION >= 19 */
 				} else {
 					/* try_enter succeeded - monitor is not inflated - would block */
 					SET_IGNORE_ENTER(monitor);
