@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,25 +15,27 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
+#include "bcnames.h"
+#include "j2sever.h"
 #include "j9.h"
 #include "j9cfg.h"
-#include "j9protos.h"
 #include "j9consts.h"
-#include "j2sever.h"
-#include "vm_internal.h"
 #include "j9cp.h"
-#include "ut_j9vm.h"
-#include "objhelp.h"
+#include "j9protos.h"
 #include "j9vmnls.h"
-#include <string.h>
+#include "objhelp.h"
 #include "pcstack.h"
 #include "rommeth.h"
-#include "bcnames.h"
+#include "ut_j9vm.h"
+#include "vm_api.h"
+#include "vm_internal.h"
+
+#include <string.h>
 
 static UDATA classAndLoaderHashFn (void *key, void *userData);
 static UDATA classAndLoaderHashEqualFn (void *leftKey, void *rightKey, void *userData);
@@ -72,7 +74,7 @@ totalStaticSlotsForClass( J9ROMClass *romClass ) {
 /**
  * @internal
  *
- * This function should only be called by internalFindClassUTF8()!
+ * This method has potential GC point.
  *
  * Load the array class with the specified name.
  * If nameData does not refer to a valid array, return NULL.
@@ -108,7 +110,7 @@ internalFindArrayClass(J9VMThread* vmThread, J9Module *j9module, UDATA arity, U_
 		/* the first level of arity is already present in the array class */
 		arity -= 1;
 
-	} else if (IS_REF_OR_VAL_SIGNATURE(firstChar) && lastChar == ';') {
+	} else if (IS_CLASS_SIGNATURE(firstChar) && lastChar == ';') {
 
 		name += arity + 1; /* 1 for 'L' */
 		length -= arity + 2; /* 2 for 'L and ';' */
@@ -188,30 +190,29 @@ findPrimitiveArrayClass(J9JavaVM* vm, jchar sigChar)
 	}
 }
 
-
-/**
- * @internal
- *
- * Allocates and fills in the relevant fields of an array class
- */
-J9Class* 
-internalCreateArrayClass(J9VMThread* vmThread, J9ROMArrayClass* romClass, J9Class* elementClass)
+static J9Class *
+internalCreateArrayClassHelper(J9VMThread *vmThread, J9ROMArrayClass *romClass, J9Class *elementClass, UDATA options)
 {
 	J9Class *result = NULL;
 	j9object_t heapClass = J9VM_J9CLASS_TO_HEAPCLASS(elementClass);
 	j9object_t protectionDomain = NULL;
-	J9ROMClass* arrayRomClass = (J9ROMClass*) romClass;
+	J9ROMClass *arrayRomClass = (J9ROMClass *)romClass;
 	J9JavaVM *const javaVM = vmThread->javaVM;
-	UDATA options = 0;
 	BOOLEAN elementInitSuccess = TRUE;
 
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-	/* When creating an array of valuetype elements, the array elements are initialized to the defaultValue of the
-	 * element type. As a result the element type must be fully initialized (if its a valuetype) before creating an
-	 * instance of the array. Element class init must be done before the arrayClass is created so that in the case
-	 * of an init failure the arrayClass is not temporarily exposed.
+#if defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES)
+	/* When creating an array of implicitly constructible valuetype elements,
+	 * the array elements are initialized to the defaultValue of the element
+	 * type. As a result, the element type must be fully initialized before
+	 * creating an instance of the array. Element class init must be done
+	 * before the arrayClass is created so that in the case of an init failure
+	 * the arrayClass is not temporarily exposed.
+	 * Don't try to initialize class for null-restricted array creation since
+	 * the regular arrayClass will always be created first.
 	 */
-	if (J9_IS_J9CLASS_PRIMITIVE_VALUETYPE(elementClass)) {
+	if (J9_IS_J9CLASS_ALLOW_DEFAULT_VALUE(elementClass)
+		&& J9_ARE_NO_BITS_SET(options, J9_FINDCLASS_FLAG_CLASS_OPTION_NULL_RESTRICTED_ARRAY)
+	) {
 		UDATA initStatus = elementClass->initializeStatus;
 		if ((J9ClassInitSucceeded != initStatus) && ((UDATA)vmThread != initStatus)) {
 			initializeClass(vmThread, elementClass);
@@ -220,7 +221,7 @@ internalCreateArrayClass(J9VMThread* vmThread, J9ROMArrayClass* romClass, J9Clas
 			}
 		}
 	}
-#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
+#endif /* defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES) */
 
 	if (elementInitSuccess) {
 		if (J9ROMCLASS_IS_HIDDEN(elementClass->romClass)) {
@@ -260,6 +261,31 @@ internalCreateArrayClass(J9VMThread* vmThread, J9ROMArrayClass* romClass, J9Clas
 	return result;
 }
 
+/**
+ * @internal
+ *
+ * Allocates and fills in the relevant fields of an array class
+ */
+J9Class *
+internalCreateArrayClass(J9VMThread *vmThread, J9ROMArrayClass *romClass, J9Class *elementClass)
+{
+	return internalCreateArrayClassHelper(vmThread, romClass, elementClass, 0);
+}
+
+J9Class *
+internalCreateArrayClassWithOptions(J9VMThread *vmThread, J9ROMArrayClass *romClass, J9Class *elementClass, UDATA options)
+{
+	/* Create elementClass's arrayClass as a prerequisite to nullRestrictedArrayClass */
+	if (J9_ARE_ALL_BITS_SET(options, J9_FINDCLASS_FLAG_CLASS_OPTION_NULL_RESTRICTED_ARRAY)
+		&& (NULL == elementClass->arrayClass)
+	) {
+		if (NULL == internalCreateArrayClassHelper(vmThread, romClass, elementClass, 0)) {
+			return NULL;
+		}
+	}
+	return internalCreateArrayClassHelper(vmThread, romClass, elementClass, options);
+}
+
 /**
  * Peek the classHashTable to see if the `className` class has already been loaded by `classLoader`.
  *
@@ -287,7 +313,7 @@ peekClassHashTable(J9VMThread* currentThread, J9ClassLoader* classLoader, U_8* c
 	return ramClass;
 }
 
-J9Class*  
+J9Class*
 internalFindClassString(J9VMThread* currentThread, j9object_t moduleName, j9object_t className, J9ClassLoader* classLoader, UDATA options, UDATA allowedBitsForClassName)
 {
 	J9Class *result = NULL;
@@ -299,6 +325,13 @@ internalFindClassString(J9VMThread* currentThread, j9object_t moduleName, j9obje
 		omrthread_monitor_enter(vm->classTableMutex);
 	}
 	result = hashClassTableAtString(classLoader, (j9object_t) className);
+#if defined(J9VM_OPT_SNAPSHOTS)
+	if ((NULL != result) && IS_RESTORE_RUN(vm)) {
+		if (!loadWarmClassFromSnapshot(currentThread, classLoader, result)) {
+			result = NULL;
+		}
+	}
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
 	if (!fastMode) {
 		omrthread_monitor_exit(vm->classTableMutex);
 	}
@@ -310,12 +343,21 @@ internalFindClassString(J9VMThread* currentThread, j9object_t moduleName, j9obje
 		U_8 *utf8Name = NULL;
 		UDATA utf8Length = 0;
 		UDATA stringFlags = J9_STR_NULL_TERMINATE_RESULT;
+		J9InternalVMFunctions const *const vmFuncs = vm->internalVMFunctions;
 		PORT_ACCESS_FROM_JAVAVM(vm);
 
 		if (CLASSNAME_INVALID == allowedBitsForClassName) {
 			stringFlags |= J9_STR_XLAT;
 		}
 
+		/* Perform maximum length check to avoid copy in extreme cases. */
+		if (J9VMJAVALANGSTRING_LENGTH(currentThread, className) > J9VM_MAX_CLASS_NAME_LENGTH) {
+			setCurrentExceptionNLS(currentThread,
+				J9VMCONSTANTPOOL_JAVALANGCLASSNOTFOUNDEXCEPTION,
+				J9NLS_VM_CLASS_NAME_EXCEEDS_MAX_LENGTH);
+			return NULL;
+		}
+
 		utf8Name = (U_8*)copyStringToUTF8WithMemAlloc(currentThread, className, stringFlags, "", 0, (char *)localBuf, J9VM_PACKAGE_NAME_BUFFER_LENGTH, &utf8Length);
 		if (NULL == utf8Name) {
 			/* Throw out-of-memory */
@@ -324,22 +366,37 @@ internalFindClassString(J9VMThread* currentThread, j9object_t moduleName, j9obje
 		}
 
 		/* Make sure the name is legal */
-		if ((CLASSNAME_INVALID == allowedBitsForClassName)
+		if (utf8Length > J9VM_MAX_CLASS_NAME_LENGTH) {
+			if (CLASSNAME_INVALID != allowedBitsForClassName) {
+				setCurrentExceptionNLS(currentThread,
+					J9VMCONSTANTPOOL_JAVALANGCLASSNOTFOUNDEXCEPTION,
+					J9NLS_VM_CLASS_NAME_EXCEEDS_MAX_LENGTH);
+			}
+		} else if ((CLASSNAME_INVALID == allowedBitsForClassName)
 			|| (CLASSNAME_INVALID != verifyQualifiedName(currentThread, utf8Name, utf8Length, allowedBitsForClassName))
 		) {
 			if (NULL != moduleName) {
 				J9Module module = {0};
 				J9Module *modulePtr = &module;
 
-				modulePtr->moduleName = moduleName;
+				module.moduleName = vmFuncs->copyStringToJ9UTF8WithMemAlloc(
+						currentThread, moduleName, J9_STR_NULL_TERMINATE_RESULT, "", 0, NULL, 0);
+				if (NULL == module.moduleName) {
+					vmFuncs->setNativeOutOfMemoryError(currentThread, 0, 0);
+					goto done;
+				}
+				omrthread_monitor_enter(vm->classLoaderModuleAndLocationMutex);
 				findResult = hashTableFind(classLoader->moduleHashTable, &modulePtr);
+				omrthread_monitor_exit(vm->classLoaderModuleAndLocationMutex);
 				if (NULL != findResult) {
 					j9module = *findResult;
 				}
+				j9mem_free_memory(module.moduleName);
 			}
 
 			result = internalFindClassInModule(currentThread, j9module, utf8Name, utf8Length, classLoader, options);
 		}
+done:
 		if (utf8Name != localBuf) {
 			j9mem_free_memory(utf8Name);
 		}
@@ -366,7 +423,7 @@ internalRunPreInitInstructions(J9Class * ramClass, J9VMThread * vmThread)
 		UDATA descriptionCount = 0;
 		U_32 description = 0;
 		UDATA i;
-		
+
 		BOOLEAN isAnonClass = J9_ARE_ANY_BITS_SET(romClass->extraModifiers, J9AccClassAnonClass | J9AccClassHidden);
 
 		for (i = 0; i < ramConstantPoolCount; ++i) {
@@ -453,7 +510,7 @@ internalRunPreInitInstructions(J9Class * ramClass, J9VMThread * vmThread)
 
 
 
-J9Class*  
+J9Class*
 resolveKnownClass(J9JavaVM * vm, UDATA index)
 {
 	J9VMThread * currentThread = currentVMThread(vm);
@@ -555,13 +612,14 @@ callFindLocallyDefinedClass(J9VMThread* vmThread, J9Module *j9module, U_8* class
 
 		findResult = (IDATA) returnVal;
 		if (0 == findResult) {
-			TRIGGER_J9HOOK_VM_FIND_LOCALLY_DEFINED_CLASS_FROM_FILESYSTEM(vmThread->javaVM->hookInterface, 
-																		 vmThread, 
-																		 (const char *)className, 
-																		 classNameLength,
-																		 classLoader,
-																		 &classFound,
-																		 returnPointer);
+			TRIGGER_J9HOOK_VM_FIND_LOCALLY_DEFINED_CLASS_FROM_FILESYSTEM(
+					vmThread->javaVM->hookInterface,
+					vmThread,
+					(const char *)className,
+					classNameLength,
+					classLoader,
+					&classFound,
+					returnPointer);
 
 			if (returnVal != NULL) {
 				findResult = classFound;
@@ -601,7 +659,7 @@ callFindLocallyDefinedClass(J9VMThread* vmThread, J9Module *j9module, U_8* class
  * Postcondition: class table mutex unlocked (NB)
  */
 
-static J9Class*  
+static J9Class*
 attemptDynamicClassLoad(J9VMThread* vmThread, J9Module *j9module, U_8* className, UDATA classNameLength, J9ClassLoader* classLoader, UDATA options)
 {
 	J9Class *foundClass = NULL;
@@ -687,11 +745,12 @@ attemptDynamicClassLoad(J9VMThread* vmThread, J9Module *j9module, U_8* className
  * Postcondition: no mutexes locked, vm access, loader object locked
 */
 
-static VMINLINE J9Class*   
+static VMINLINE J9Class*
 callLoadClass(J9VMThread* vmThread, U_8* className, UDATA classNameLength, J9ClassLoader* classLoader, j9object_t * classNotFoundException)
 {
 	j9object_t classNameString, sendLoadClassResult;
 	J9Class *foundClass = NULL;
+	BOOLEAN addedClassToInitiatingLoader = FALSE;
 
 	Assert_VM_mustHaveVMAccess(vmThread);
 
@@ -766,7 +825,9 @@ callLoadClass(J9VMThread* vmThread, U_8* className, UDATA classNameLength, J9Cla
 
 					/* Add the found class to our class table since we are an initiating loader */
 
-					if (hashClassTableAtPut(vmThread, classLoader, className, classNameLength, foundClass)) {
+					if (!hashClassTableAtPut(vmThread, classLoader, className, classNameLength, foundClass)) {
+						addedClassToInitiatingLoader = TRUE;
+					} else {
 						/* Failed to store the class - GC and retry */
 
 						omrthread_monitor_exit(vm->classTableMutex);
@@ -777,7 +838,9 @@ callLoadClass(J9VMThread* vmThread, U_8* className, UDATA classNameLength, J9Cla
 
 						ramClass = hashClassTableAt(classLoader, className, classNameLength);
 						if (NULL == ramClass) {
-							if (hashClassTableAtPut(vmThread, classLoader, className, classNameLength, foundClass)) {
+							if (!hashClassTableAtPut(vmThread, classLoader, className, classNameLength, foundClass)) {
+								addedClassToInitiatingLoader = TRUE;
+							} else {
 								/* Add failed again, throw native OOM */
 
 								omrthread_monitor_exit(vm->classTableMutex);
@@ -800,6 +863,11 @@ callLoadClass(J9VMThread* vmThread, U_8* className, UDATA classNameLength, J9Cla
 					}
 				}
 			}
+
+			if (addedClassToInitiatingLoader) {
+				addOutlivingLoader(vmThread, classLoader, foundClass->classLoader);
+			}
+
 			omrthread_monitor_exit(vm->classTableMutex);
  		}
 	} else {
@@ -816,7 +884,7 @@ callLoadClass(J9VMThread* vmThread, U_8* className, UDATA classNameLength, J9Cla
  * and class loader object
  * @param className: pointer to this thread's copy of the classname string
  * @param classNameLength: length of the classname string
- * 
+ *
  * @return loaded class object if success, NULL if fail
  *
  * Precondition: classTableMutex locked, vm access
@@ -825,20 +893,21 @@ callLoadClass(J9VMThread* vmThread, U_8* className, UDATA classNameLength, J9Cla
  */
 
 
-static J9Class*   
+static J9Class*
 waitForContendedLoadClass(J9VMThread* vmThread, J9ContendedLoadTableEntry *tableEntry, U_8* className, UDATA classNameLength)
 {
-	UDATA recursionCount  = 0;
-	UDATA i;
-	J9Class* foundClass = NULL;
-	J9VMThread*  monitorOwner = NULL;
+	UDATA recursionCount = 0;
+	UDATA i = 0;
+	J9Class *foundClass = NULL;
+	J9VMThread *monitorOwner = NULL;
 	UDATA status = CLASSLOADING_DUMMY;
+	J9JavaVM *vm = vmThread->javaVM;
 
 	Trc_VM_waitForContendedLoadClass_getObjectMonitorOwner(vmThread, vmThread, tableEntry->classLoader, classNameLength, className);
 	Assert_VM_mustHaveVMAccess(vmThread);
 	/* get here if and only if someone else is loading the class */
 	/* give up the classloader monitor to allow other threads to run */
-	monitorOwner = getObjectMonitorOwner(vmThread->javaVM, tableEntry->classLoader->classLoaderObject, &recursionCount);
+	monitorOwner = getObjectMonitorOwner(vm, tableEntry->classLoader->classLoaderObject, &recursionCount);
 	if (monitorOwner == vmThread) {
 		Trc_VM_waitForContendedLoadClass_release_object_monitor(vmThread, vmThread, tableEntry->classLoader, classNameLength, className);
 		for (i = 0; i < recursionCount; ++i) {
@@ -849,19 +918,26 @@ waitForContendedLoadClass(J9VMThread* vmThread, J9ContendedLoadTableEntry *table
 	}
 	internalReleaseVMAccess(vmThread);
 	do {
-		omrthread_monitor_wait(vmThread->javaVM->classTableMutex);
+		omrthread_monitor_wait(vm->classTableMutex);
 	} while ((status = tableEntry->status) == CLASSLOADING_LOAD_IN_PROGRESS);
 	/* still have classTableMutex here */
 	Trc_VM_waitForContendedLoadClass_waited(vmThread, vmThread, tableEntry->classLoader, classNameLength, className, status);
 	foundClass = hashClassTableAt(tableEntry->classLoader, className, classNameLength);
-	omrthread_monitor_exit(vmThread->javaVM->classTableMutex);
+#if defined(J9VM_OPT_SNAPSHOTS)
+	if ((NULL != foundClass) && IS_RESTORE_RUN(vm)) {
+		if (!loadWarmClassFromSnapshot(vmThread, tableEntry->classLoader, foundClass)) {
+			foundClass = NULL;
+		}
+	}
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+	omrthread_monitor_exit(vm->classTableMutex);
 	internalAcquireVMAccess(vmThread);
 	Trc_VM_waitForContendedLoadClass_acquired_vm_access(vmThread, vmThread, tableEntry->classLoader, classNameLength, className);
 	for (i = 0; i < recursionCount; ++i) {
 		objectMonitorEnter(vmThread, tableEntry->classLoader->classLoaderObject);
 	}
 	Assert_VM_mustHaveVMAccess(vmThread);
-	omrthread_monitor_enter(vmThread->javaVM->classTableMutex);
+	omrthread_monitor_enter(vm->classTableMutex);
 	return foundClass;
 }
 
@@ -886,7 +962,7 @@ waitForContendedLoadClass(J9VMThread* vmThread, J9ContendedLoadTableEntry *table
  * get classTable mutex
  */
 
-static VMINLINE J9Class*   
+static VMINLINE J9Class*
 arbitratedLoadClass(J9VMThread* vmThread, U_8* className, UDATA classNameLength, J9ClassLoader* classLoader, j9object_t * classNotFoundException)
 {
 	J9Class* foundClass;
@@ -950,6 +1026,108 @@ arbitratedLoadClass(J9VMThread* vmThread, U_8* className, UDATA classNameLength,
 	return foundClass;
 }
 
+#if defined(J9VM_OPT_SNAPSHOTS)
+BOOLEAN
+loadWarmClassFromSnapshotInternal(J9VMThread *vmThread, J9ClassLoader *classLoader, J9Class *clazz)
+{
+	BOOLEAN rc = FALSE;
+	BOOLEAN failed = FALSE;
+
+	if (J9_ARE_NO_BITS_SET(clazz->classFlags, J9ClassIsLoadedFromSnapshot)) {
+		J9JavaVM *vm = vmThread->javaVM;
+		J9Class *superClazz = clazz->superclasses[J9CLASS_DEPTH(clazz) - 1];
+		J9ITable *itable = (J9ITable *)clazz->iTable;
+		j9object_t classObject = NULL;
+		const char *className = (const char *)J9UTF8_DATA(J9ROMCLASS_CLASSNAME(clazz->romClass));
+
+		clazz->classFlags |= J9ClassIsLoadedFromSnapshot;
+
+		/* Load superclasses and interfaces first. */
+		if (NULL != superClazz) {
+			if (!loadWarmClassFromSnapshot(vmThread, classLoader, superClazz)) {
+				goto done;
+			}
+		}
+
+		while (NULL != itable) {
+			J9Class *interface = itable->interfaceClass;
+			if (NULL != interface) {
+				if (!loadWarmClassFromSnapshot(vmThread, classLoader, interface)) {
+					goto done;
+				}
+			}
+			itable = itable->next;
+		}
+
+		initializeSnapshotJ9Class(vm, clazz);
+		TRIGGER_J9HOOK_VM_INTERNAL_CLASS_LOAD(vm->hookInterface, vmThread, clazz, failed);
+		TRIGGER_J9HOOK_VM_CLASS_LOAD(vm->hookInterface, vmThread, clazz);
+
+		if (failed) {
+			/* The class-load hooks above may necessitate reloading className. */
+			className = (const char *)J9UTF8_DATA(J9ROMCLASS_CLASSNAME(clazz->romClass));
+			Trc_VM_snapshot_loadWarmClassFromSnapshot_ClassLoadHookFailed(vmThread, clazz, className);
+			goto done;
+		}
+
+		/* TODO: This is only a temporary fix for arrays.
+		 * Pre-emptively load arrays to ensure that clazz->arrayClass returns a valid class.
+		 */
+		if (NULL != clazz->arrayClass) {
+			if (!loadWarmClassFromSnapshot(vmThread, classLoader, clazz->arrayClass)) {
+				goto done;
+			}
+		}
+		if (J9_ARE_ANY_BITS_SET(vmThread->javaVM->extendedRuntimeFlags, J9_EXTENDED_RUNTIME_CLASS_OBJECT_ASSIGNED)) {
+			Assert_VM_Null(clazz->classObject);
+			clazz = initializeSnapshotClassObject(vm, classLoader, clazz);
+			if (NULL == clazz) {
+				goto done;
+			}
+		}
+
+		/* TODO: Handle/trace error/NULL paths. */
+		classObject = clazz->classObject;
+		if (NULL != classObject) {
+			J9Module *module = clazz->module;
+			if (NULL != module) {
+				j9object_t moduleObject = module->moduleObject;
+				if (NULL != moduleObject) {
+					J9VMJAVALANGCLASS_SET_MODULE(vmThread, classObject, moduleObject);
+				}
+			} else {
+				/* Unnamed module. */
+				J9VMJAVALANGCLASS_SET_MODULE(vmThread, classObject, J9VMJAVALANGCLASSLOADER_UNNAMEDMODULE(vmThread, clazz->classLoader->classLoaderObject));
+			}
+		}
+
+		Trc_VM_snapshot_loadWarmClassFromSnapshot_ClassInfo(vmThread, clazz, className);
+	}
+	clazz->classFlags &= ~J9ClassIsFrozen;
+	rc = TRUE;
+
+done:
+	return rc;
+}
+
+BOOLEAN
+loadWarmClassFromSnapshot(J9VMThread *currentThread, J9ClassLoader *classLoader, J9Class *clazz)
+{
+	BOOLEAN rc = TRUE;
+	J9JavaVM *vm = currentThread->javaVM;
+
+	if (J9_ARE_ANY_BITS_SET(clazz->classFlags, J9ClassIsFrozen)) {
+		omrthread_monitor_enter(vm->rcpCacheMutex);
+		if (J9_ARE_ANY_BITS_SET(clazz->classFlags, J9ClassIsFrozen)) {
+			rc = loadWarmClassFromSnapshotInternal(currentThread, classLoader, clazz);
+		}
+		omrthread_monitor_exit(vm->rcpCacheMutex);
+	}
+
+	return rc;
+}
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+
 /**
  * Load non-array class.
  * @param vmThread Current VM thread
@@ -965,7 +1143,7 @@ arbitratedLoadClass(J9VMThread* vmThread, U_8* className, UDATA classNameLength,
  * Postcondition: either J9_FINDCLASS_FLAG_EXISTING_ONLY set or classTableMutex not owned by this thread; this thread has vm access
  */
 
-static VMINLINE J9Class *  
+static VMINLINE J9Class *
 loadNonArrayClass(J9VMThread* vmThread, J9Module *j9module, U_8* className, UDATA classNameLength, J9ClassLoader* classLoader, UDATA options, j9object_t *exception)
 {
 	J9JavaVM * const vm = vmThread->javaVM;
@@ -974,13 +1152,6 @@ loadNonArrayClass(J9VMThread* vmThread, J9Module *j9module, U_8* className, UDAT
 	BOOLEAN fastMode = J9_ARE_ALL_BITS_SET(vm->extendedRuntimeFlags, J9_EXTENDED_RUNTIME_FAST_CLASS_HASH_TABLE);
 	BOOLEAN loaderMonitorLocked = FALSE;
 
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-	if (J9_IS_STRING_DESCRIPTOR(className, classNameLength)) {
-		className += 1; /* 1 for 'L' or 'Q' */
-		classNameLength -= 2; /* 2 for 'L'/'Q' and ';' */
-	}
-#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-
 	vmThread->privateFlags &= ~J9_PRIVATE_FLAGS_CLOAD_NO_MEM;
 
 	if (J9CLASSLOADER_PARALLEL_CAPABLE == (J9CLASSLOADER_PARALLEL_CAPABLE & classLoader->flags)) {
@@ -1012,6 +1183,13 @@ loadNonArrayClass(J9VMThread* vmThread, J9Module *j9module, U_8* className, UDAT
 
 	foundClass = hashClassTableAt(classLoader, className, classNameLength);
 	if (NULL != foundClass) {
+#if defined(J9VM_OPT_SNAPSHOTS)
+		if (IS_RESTORE_RUN(vm)) {
+			if (!loadWarmClassFromSnapshot(vmThread, classLoader, foundClass)) {
+				foundClass = NULL;
+			}
+		}
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
 		if (!fastMode) {
 			omrthread_monitor_exit(vm->classTableMutex);
 		}
@@ -1036,8 +1214,17 @@ loadNonArrayClass(J9VMThread* vmThread, J9Module *j9module, U_8* className, UDAT
 				/* check again if somebody else already loaded the class */
 				foundClass = hashClassTableAt(classLoader, className, classNameLength);
 				if (NULL != foundClass) {
-					omrthread_monitor_exit(vm->classTableMutex);
-					goto done;
+#if defined(J9VM_OPT_SNAPSHOTS)
+					if (IS_RESTORE_RUN(vm)
+						&& !loadWarmClassFromSnapshot(vmThread, classLoader, foundClass)
+					) {
+						foundClass = NULL;
+					} else
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+					{
+						omrthread_monitor_exit(vm->classTableMutex);
+						goto done;
+					}
 				}
 			}
 			/* Do not do the primitive type optimization if -Xfuture is on */
@@ -1119,13 +1306,13 @@ done:
 /**
  * It is a wrapper around internalFindClassInModule().
  */
-J9Class*  
+J9Class*
 internalFindClassUTF8(J9VMThread* vmThread, U_8* className, UDATA classNameLength, J9ClassLoader* classLoader, UDATA options)
 {
 	return internalFindClassInModule(vmThread, NULL, className, classNameLength, classLoader, options);
 }
 
-J9Class*  
+J9Class*
 internalFindClassInModule(J9VMThread* vmThread, J9Module *j9module, U_8* className, UDATA classNameLength, J9ClassLoader* classLoader, UDATA options)
 {
 	J9JavaVM *vm = vmThread->javaVM;
@@ -1208,7 +1395,7 @@ internalFindClassInModule(J9VMThread* vmThread, J9Module *j9module, U_8* classNa
 	return foundClass;
 }
 
-J9Class*  
+J9Class*
 internalFindKnownClass(J9VMThread *vmThread, UDATA index, UDATA flags)
 {
 	J9JavaVM *vm = vmThread->javaVM;
