@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2020 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j9.h"
@@ -33,21 +33,22 @@
 IDATA
 objectMonitorExit(J9VMThread* vmStruct, j9object_t object)
 {
+	IDATA rc = J9THREAD_ILLEGAL_MONITOR_STATE;
 	j9objectmonitor_t *lockEA = NULL;
 	j9objectmonitor_t lock = 0;
 
 	Assert_VM_true(vmStruct != NULL);
-	Assert_VM_true(0 == ((UDATA)vmStruct & OBJECT_HEADER_LOCK_BITS_MASK));
+	Assert_VM_true(J9_ARE_NO_BITS_SET((UDATA)vmStruct, OBJECT_HEADER_LOCK_BITS_MASK));
 
-	Trc_VM_objectMonitorExit_Entry(vmStruct,object);
+	Trc_VM_objectMonitorExit_Entry(vmStruct, object);
 
-	if (!LN_HAS_LOCKWORD(vmStruct,object)) {
+	if (!LN_HAS_LOCKWORD(vmStruct, object)) {
 		J9ObjectMonitor *objectMonitor = NULL;
 
 		objectMonitor = monitorTableAt(vmStruct, object);
-		if (objectMonitor == NULL) {
+		if (NULL == objectMonitor) {
 			Trc_VM_objectMonitorExit_Exit_IllegalNullMonitor(vmStruct, object);
-			return J9THREAD_ILLEGAL_MONITOR_STATE;
+			goto done;
 		}
 
 		lockEA = &(objectMonitor->alternateLockword);
@@ -58,7 +59,7 @@ restart:
 	lock = J9_LOAD_LOCKWORD(vmStruct, lockEA);
 
 	if (J9_FLATLOCK_OWNER(lock) == vmStruct) {
-		/* the current thread owns the monitor, and it's a flat lock */
+		/* The current thread owns the monitor, and it's a flat lock. */
 		UDATA count = (UDATA)lock & OBJECT_HEADER_LOCK_BITS_MASK;
 		/*
 		 * If the learning bit is set, the lock is in the learning state. CAS should be used to decrement the RC field.
@@ -73,26 +74,91 @@ restart:
 		 *   >=16 (RC >= 1)
 		 */
 
-		Assert_VM_false(lock & OBJECT_HEADER_LOCK_INFLATED);
+		Assert_VM_false(J9_ARE_ANY_BITS_SET(lock, OBJECT_HEADER_LOCK_INFLATED));
 #ifndef J9VM_THR_LOCK_RESERVATION
-		Assert_VM_false(lock & OBJECT_HEADER_LOCK_RESERVED);
+		Assert_VM_false(J9_ARE_ANY_BITS_SET(lock, OBJECT_HEADER_LOCK_RESERVED));
 
 		/* Global Lock Reservation is currently only supported on Power. Learning bit should be clear on other platforms. */
 #if !(defined(AIXPPC) || defined(LINUXPPC))
-		Assert_VM_false(lock & OBJECT_HEADER_LOCK_LEARNING);
+		Assert_VM_false(J9_ARE_ANY_BITS_SET(lock, OBJECT_HEADER_LOCK_LEARNING));
 #endif /* if !(defined(AIXPPC) || defined(LINUXPPC)) */
 #endif /* ifndef J9VM_THR_LOCK_RESERVATION */
+#if JAVA_SPEC_VERSION >= 24
+		if (OBJECT_HEADER_LOCK_FLC == count) {
+			/* FLC set, non-recursive. */
+			J9ObjectMonitor *objectMonitor = objectMonitorInflate(vmStruct, object, lock);
 
-		if (count == 0x00) {
-			/* just release the flatlock */
+			if (NULL == objectMonitor) {
+				/* Out of memory - impossible? */
+				monitorExitWriteBarrier();
+				J9_STORE_LOCKWORD(vmStruct, lockEA, 0);
+			} else {
+				omrthread_monitor_t monitor = objectMonitor->monitor;
+				J9JavaVM *vm = vmStruct->javaVM;
+
+				TRIGGER_J9HOOK_VM_MONITOR_CONTENDED_EXIT(vm->hookInterface, vmStruct, monitor);
+
+				omrthread_monitor_exit(monitor);
+
+				if (J9_ARE_ANY_BITS_SET(vm->extendedRuntimeFlags3, J9_EXTENDED_RUNTIME3_YIELD_PINNED_CONTINUATION)
+				&& (0 != objectMonitor->virtualThreadWaitCount)
+				) {
+					J9VM_SEND_VIRTUAL_UNBLOCKER_THREAD_SIGNAL(vm);
+				}
+			}
+		} else {
+			j9objectmonitor_t newLock = 0;
+			BOOLEAN casSuccess = FALSE;
+			if (0x00 == count) {
+				/* Just release the flatlock. */
+			} else if (J9_ARE_ANY_BITS_SET(count, OBJECT_HEADER_LOCK_LEARNING)
+				&& J9_ARE_ANY_BITS_SET(count, OBJECT_HEADER_LOCK_LEARNING_RECURSION_MASK)
+			) {
+				/* Learning state case. */
+				newLock = lock - OBJECT_HEADER_LOCK_LEARNING_FIRST_RECURSION_BIT;
+			} else if (J9_ARE_ANY_BITS_SET(count, OBJECT_HEADER_LOCK_LEARNING)) {
+				/* Lock is in Learning state but unowned.
+				 * (if it were owned it would have been caught by the first Learning state check)
+				 */
+				goto done;
+			} else if (count >= OBJECT_HEADER_LOCK_FIRST_RECURSION_BIT) {
+				/* Just decrement the flatlock recursion count. */
+				newLock = lock - OBJECT_HEADER_LOCK_FIRST_RECURSION_BIT;
+#ifdef J9VM_THR_LOCK_RESERVATION
+			} else if (J9_ARE_ANY_BITS_SET(count, OBJECT_HEADER_LOCK_RESERVED)) {
+				/* Lock is reserved but unowned.
+				 * (if it were owned the count would be >= OBJECT_HEADER_LOCK_FIRST_RECURSION_BIT)
+				 */
+				Trc_VM_objectMonitorExit_Exit_ReservedButUnownedFlatLock(vmStruct, lock, object);
+				goto done;
+#endif /* J9VM_THR_LOCK_RESERVATION */
+			}
+
+			monitorExitWriteBarrier();
+			if (J9VMTHREAD_COMPRESS_OBJECT_REFERENCES(vmStruct)) {
+				casSuccess = (lock == compareAndSwapU32((uint32_t *)lockEA, (uint32_t)lock, (uint32_t)newLock));
+			} else {
+				casSuccess = (lock == compareAndSwapUDATA((uintptr_t *)lockEA, (uintptr_t)lock, (uintptr_t)newLock));
+			}
+
+			if (!casSuccess) {
+				/* Another thread has attempted to get the lock and may have update the FLC flag. */
+				goto restart;
+			}
+		}
+#else /* JAVA_SPEC_VERSION >= 24 */
+		if (0x00 == count) {
+			/* Just release the flatlock. */
 			monitorExitWriteBarrier();
 			J9_STORE_LOCKWORD(vmStruct, lockEA, 0);
-		} else if ((count & OBJECT_HEADER_LOCK_LEARNING) && (count & OBJECT_HEADER_LOCK_LEARNING_RECURSION_MASK)) {
-			/* Learning state case */
-			I_32 casSuccess = FALSE;
+		} else if (J9_ARE_ANY_BITS_SET(count, OBJECT_HEADER_LOCK_LEARNING)
+			&& J9_ARE_ANY_BITS_SET(count, OBJECT_HEADER_LOCK_LEARNING_RECURSION_MASK)
+		) {
+			/* Learning state case. */
+			BOOLEAN casSuccess = FALSE;
 
 			if (1 == J9_FLATLOCK_COUNT(lock)) {
-				/* if RC field is 1, this fully unlocks the object so a write barrier is needed */
+				/* If RC field is 1, this fully unlocks the object so a write barrier is needed. */
 				monitorExitWriteBarrier();
 			}
 
@@ -112,75 +178,82 @@ restart:
 				 */
 				goto restart;
 			}
-		} else if (count & OBJECT_HEADER_LOCK_LEARNING) {
-			/* Lock is in Learning state but unowned (if it were owned it would have been caught by the first Learning state check) */
-			return J9THREAD_ILLEGAL_MONITOR_STATE;
+		} else if (J9_ARE_ANY_BITS_SET(count, OBJECT_HEADER_LOCK_LEARNING)) {
+			/* Lock is in Learning state but unowned.
+			 * (if it were owned it would have been caught by the first Learning state check)
+			 */
+			Trc_VM_objectMonitorExit_Exit_LearningStateUnowned(vmStruct, lock, object);
+			goto done;
 		} else if (count >= OBJECT_HEADER_LOCK_FIRST_RECURSION_BIT) {
-			/* just decrement the flatlock recursion count */
+			/* Just decrement the flatlock recursion count. */
 			lock -= OBJECT_HEADER_LOCK_FIRST_RECURSION_BIT;
 			J9_STORE_LOCKWORD(vmStruct, lockEA, lock);
 #ifdef J9VM_THR_LOCK_RESERVATION
-		} else if (count & OBJECT_HEADER_LOCK_RESERVED) {
-			/* lock is reserved but unowned (if it were owned the count would be >= OBJECT_HEADER_LOCK_FIRST_RECURSION_BIT) */
+		} else if (J9_ARE_ANY_BITS_SET(count, OBJECT_HEADER_LOCK_RESERVED)) {
+			/* Lock is reserved but unowned.
+			 * (if it were owned the count would be >= OBJECT_HEADER_LOCK_FIRST_RECURSION_BIT)
+			 */
 			Trc_VM_objectMonitorExit_Exit_ReservedButUnownedFlatLock(vmStruct, lock, object);
-			return J9THREAD_ILLEGAL_MONITOR_STATE;
-#endif
+			goto done;
+#endif /* J9VM_THR_LOCK_RESERVATION */
 		} else {
-			/* FLC set, non-recursive */
+			/* FLC set, non-recursive. */
 			J9ObjectMonitor *objectMonitor = objectMonitorInflate(vmStruct, object, lock);
 
-			if (objectMonitor == NULL) {
-				/* out of memory - impossible? */
+			if (NULL == objectMonitor) {
+				/* Out of memory - impossible? */
 				monitorExitWriteBarrier();
 				J9_STORE_LOCKWORD(vmStruct, lockEA, 0);
 			} else {
 				omrthread_monitor_t monitor = objectMonitor->monitor;
-
 				TRIGGER_J9HOOK_VM_MONITOR_CONTENDED_EXIT(vmStruct->javaVM->hookInterface, vmStruct, monitor);
-
 				omrthread_monitor_exit(monitor);
 			}
 		}
+#endif /* JAVA_SPEC_VERSION >= 24 */
 		Trc_VM_objectMonitorExit_Exit_FCBSet(vmStruct);
-		return 0;
+		rc = 0;
+		goto done;
 	} else if (J9_LOCK_IS_INFLATED(lock)) {
-		/* Dealing with an inflated monitor */
+		/* Dealing with an inflated monitor. */
 		J9ObjectMonitor *objectMonitor = NULL;
 		J9ThreadAbstractMonitor *monitor = NULL;
-		IDATA rc = 0;
 		IDATA deflate = 1;
-		
-		objectMonitor = J9_INFLLOCK_OBJECT_MONITOR(lock);		
+		J9JavaVM *vm = vmStruct->javaVM;
+
+		objectMonitor = J9_INFLLOCK_OBJECT_MONITOR(lock);
 		monitor = (J9ThreadAbstractMonitor *)objectMonitor->monitor;
 		Assert_VM_notNull(monitor);
+		Assert_VM_false(IS_J9_OBJECT_MONITOR_OWNER_DETACHED(monitor->owner));
 
 #ifdef OMR_THR_ADAPTIVE_SPIN
-		/* for now we don't allow deflation if spinning has been disabled for this monitor
-		 * because it has a longer hold time */
-		if ((NULL != monitor->tracing) && (0 != (monitor->flags & J9THREAD_MONITOR_DISABLE_SPINNING))) {
+		/* For now we don't allow deflation if spinning has been disabled
+		 * for this monitor because it has a longer hold time.
+		 */
+		if ((NULL != monitor->tracing) && J9_ARE_ANY_BITS_SET(monitor->flags, J9THREAD_MONITOR_DISABLE_SPINNING)) {
 			deflate = 0;
 		}
-#endif
+#endif /* OMR_THR_ADAPTIVE_SPIN */
 
 		if (monitor->owner != vmStruct->osThread) {
 			Trc_VM_objectMonitorExit_Exit_IllegalInflatedLock(vmStruct, monitor->owner, vmStruct->osThread);
-			return J9THREAD_ILLEGAL_MONITOR_STATE;
+			goto done;
 		}
 
 		/*
 		 * Are we releasing the inflated monitor? (count is 1)
-		 * If we are, the question becomes: 
-		 *    May we deflate? If so, CAN we deflate? 
-		 * 
+		 * If we are, the question becomes:
+		 *    May we deflate? If so, CAN we deflate?
+		 *
 		 * We may deflate if the user has specified that we may. (thrDeflationPolicy)
 		 * We can deflate if no thread is blocked on this inflated monitor (pin count is 0) and
 		 *   iff the deflation policy in effect decides it's ok.
 		 */
-		if (monitor->count == 1) {
+		if (1 == monitor->count) {
 			if (0 == monitor->pinCount) {
 				if (deflate) {
 					deflate = 0;
-					switch (vmStruct->javaVM->thrDeflationPolicy) {
+					switch (vm->thrDeflationPolicy) {
 					case J9VM_DEFLATION_POLICY_NEVER:
 						break;
 					case J9VM_DEFLATION_POLICY_ASAP:
@@ -206,30 +279,47 @@ restart:
 					}
 				}
 			} else {
-				if (J9_EVENT_IS_HOOKED(vmStruct->javaVM->hookInterface, J9HOOK_VM_MONITOR_CONTENDED_EXIT)) {
+				if (J9_EVENT_IS_HOOKED(vm->hookInterface, J9HOOK_VM_MONITOR_CONTENDED_EXIT)) {
 					/* Are CONTENDED_EXIT events suppressed? */
 					if (!(monitor->flags & J9THREAD_MONITOR_SUPPRESS_CONTENDED_EXIT)) {
-						/* Any threads blocked on this monitor? If so, report contended exit */
+						/* Any threads blocked on this monitor? If so, report contended exit. */
 						if ((monitor->pinCount - omrthread_monitor_num_waiting((omrthread_monitor_t)monitor)) > 0) {
-							ALWAYS_TRIGGER_J9HOOK_VM_MONITOR_CONTENDED_EXIT(vmStruct->javaVM->hookInterface, vmStruct, (omrthread_monitor_t)monitor);
+							ALWAYS_TRIGGER_J9HOOK_VM_MONITOR_CONTENDED_EXIT(vm->hookInterface, vmStruct, (omrthread_monitor_t)monitor);
 							monitor->flags |= J9THREAD_MONITOR_SUPPRESS_CONTENDED_EXIT;
 						}
 					}
 				}
 			}
 		}
-		rc =  omrthread_monitor_exit((omrthread_monitor_t)monitor);
+#if JAVA_SPEC_VERSION >= 24
+		Trc_VM_objectMonitorExit_OMRThread_Monitor_Exit(vmStruct, monitor, monitor->count);
+#endif /*  JAVA_SPEC_VERSION >= 24 */
+		rc = omrthread_monitor_exit((omrthread_monitor_t)monitor);
+#if JAVA_SPEC_VERSION >= 24
+		if (J9_ARE_ANY_BITS_SET(vm->extendedRuntimeFlags3, J9_EXTENDED_RUNTIME3_YIELD_PINNED_CONTINUATION)
+		&& (0 != objectMonitor->virtualThreadWaitCount)
+		) {
+			J9VM_SEND_VIRTUAL_UNBLOCKER_THREAD_SIGNAL(vm);
+		}
+#endif /* JAVA_SPEC_VERSION >= 24 */
 		Trc_VM_objectMonitorExit_Exit_InflatedLock(vmStruct, rc);
-		return rc;
+		goto done;
 	} else {
-		/* flat lock, but wrong thread */
+		/* Flat lock, but wrong thread. */
 		Assert_VM_true( (lock == 0) || (lock == OBJECT_HEADER_LOCK_LEARNING) || (lock == OBJECT_HEADER_LOCK_RESERVED) || ((UDATA)lock & ~(UDATA)OBJECT_HEADER_LOCK_BITS_MASK) );
 		Trc_VM_objectMonitorExit_Exit_IllegalFlatLock(vmStruct, lock, object);
-		return J9THREAD_ILLEGAL_MONITOR_STATE;
+		goto done;
 	}
 
 	Assert_VM_unreachable();
 
+done:
+#if JAVA_SPEC_VERSION >= 19
+	if (0 == rc) {
+		vmStruct->ownedMonitorCount -= 1;
+	}
+#endif /* JAVA_SPEC_VERSION >= 19 */
+	return rc;
 }
 
 
@@ -240,8 +330,8 @@ restart:
 	NULL if out of memory
 	the inflated monitor, if successful
  */
-J9ObjectMonitor * 
-objectMonitorInflate(J9VMThread* vmStruct, j9object_t object, UDATA lock) 
+J9ObjectMonitor *
+objectMonitorInflate(J9VMThread* vmStruct, j9object_t object, UDATA lock)
 {
 	J9ObjectMonitor *objectMonitor = monitorTableAt(vmStruct, object);
 	omrthread_monitor_t monitor = NULL;
@@ -254,7 +344,7 @@ objectMonitorInflate(J9VMThread* vmStruct, j9object_t object, UDATA lock)
 	omrthread_monitor_enter(monitor);
 
 	/* set the count to be the current thread's count */
-	((J9ThreadAbstractMonitor*)monitor)->count = J9_FLATLOCK_COUNT(lock);	
+	((J9ThreadAbstractMonitor*)monitor)->count = J9_FLATLOCK_COUNT(lock);
 
 	if (!LN_HAS_LOCKWORD(vmStruct,object)) {
 		J9_STORE_LOCKWORD(vmStruct, &objectMonitor->alternateLockword, (j9objectmonitor_t)((UDATA)objectMonitor | OBJECT_HEADER_LOCK_INFLATED));
@@ -274,7 +364,7 @@ objectMonitorInflate(J9VMThread* vmStruct, j9object_t object, UDATA lock)
 
 
 UDATA
-objectMonitorEnter(J9VMThread* vmStruct, j9object_t object) 
+objectMonitorEnter(J9VMThread* vmStruct, j9object_t object)
 {
 	UDATA rc = objectMonitorEnterNonBlocking(vmStruct, object);
 
@@ -355,7 +445,7 @@ cancelLockReservation(J9VMThread* vmStruct)
 					compareAndSwapUDATA((uintptr_t*)lockEA, (uintptr_t)oldLock, (uintptr_t)newLock);
 				}
 
-				/* 
+				/*
 				 * CAS can only fail if another canceller has modified the lockword, in which case the
 				 * object is either no longer reserved or reserved by a different thread.
 				 * Such cases should be detected by the calling function when it re-attempts to enter the monitor.
@@ -376,13 +466,13 @@ cancelLockReservation(J9VMThread* vmStruct)
 
 /**
  * Destroy a monitor as appropriate for the given platform.
- *  For example, destroying a monitor may free the internal resources associated with the monitor. 
- *  Alternatively, destroying a monitor may return the monitor to a monitor pool for subsequent reuse. 
+ *  For example, destroying a monitor may free the internal resources associated with the monitor.
+ *  Alternatively, destroying a monitor may return the monitor to a monitor pool for subsequent reuse.
  *
  * @note A monitor must NOT be destroyed if the monitor is in use - if threads are waiting on
  * it, or if it is currently owned.
  *
- * @pre the caller must have VM access. 
+ * @pre the caller must have VM access.
  *
  * @pre Destroying a monitor modifies the hash key used by the VM's MonitorTable.
  * It must not be possible for another thread to be searching for this node in the
@@ -391,13 +481,13 @@ cancelLockReservation(J9VMThread* vmStruct)
  * thread has references to it, and the monitor is unreachable. Since the
  * monitor is marked allocated, no other thread will be attempting to reuse
  * it.
- * 
+ *
  * @param[in] vm Java VM
  * @param[in] vmThread the J9VMThread calling this function
  * @param[in] monitor a monitor to be destroyed
  * @return  0 on success or non-0 on failure (the monitor is in use)
  */
-IDATA 
+IDATA
 objectMonitorDestroy(J9JavaVM *vm, J9VMThread* vmThread, omrthread_monitor_t monitor)
 {
 	return (IDATA)omrthread_monitor_destroy_nolock(vmThread->osThread, monitor);
@@ -409,8 +499,8 @@ objectMonitorDestroy(J9JavaVM *vm, J9VMThread* vmThread, omrthread_monitor_t mon
  * @note A monitor must NOT be destroyed if the monitor is in use - if threads are waiting on
  * it, or if it is currently owned.
  *
- * @pre the caller must have VM access. 
- * 
+ * @pre the caller must have VM access.
+ *
  * @param[in] vm Java VM
  * @param[in] vmThread the J9VMThread calling this function
  */
