@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j2sever.h"
@@ -129,13 +129,18 @@ isMethodHandleINL(U_8 *methodName, U_16 methodNameLength)
 		}
 		break;
 	case 12:
-		if (J9UTF8_LITERAL_EQUALS(methodName, methodNameLength, "linkToStatic")) {
+		if (J9UTF8_LITERAL_EQUALS(methodName, methodNameLength, "linkToStatic")
+#if JAVA_SPEC_VERSION >= 22
+			|| J9UTF8_LITERAL_EQUALS(methodName, methodNameLength, "linkToNative")
+#endif /* JAVA_SPEC_VERSION >= 22 */
+		) {
 			isMethodHandle = TRUE;
 		}
 		break;
 	case 13:
 		if (J9UTF8_LITERAL_EQUALS(methodName, methodNameLength, "linkToSpecial")
-		||  J9UTF8_LITERAL_EQUALS(methodName, methodNameLength, "linkToVirtual")) {
+			|| J9UTF8_LITERAL_EQUALS(methodName, methodNameLength, "linkToVirtual")
+		) {
 			isMethodHandle = TRUE;
 		}
 		break;
@@ -152,7 +157,8 @@ isMethodHandleINL(U_8 *methodName, U_16 methodNameLength)
 }
 #endif /* defined(J9VM_OPT_OPENJDK_METHODHANDLE) */
 
-UDATA   
+#if JAVA_SPEC_VERSION < 24
+UDATA
 packageAccessIsLegal(J9VMThread *currentThread, J9Class *targetClass, j9object_t protectionDomain, UDATA canRunJavaCode)
 {
 	UDATA legal = FALSE;
@@ -187,11 +193,12 @@ requirePackageAccessCheck(J9JavaVM *vm, J9ClassLoader *srcClassLoader, J9Module
 			}
 		}
 	}
-	
+
 	return checkFlag;
 }
+#endif /* JAVA_SPEC_VERSION < 24 */
 
-j9object_t   
+j9object_t
 resolveStringRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags)
 {
 	J9UTF8 *utf8Wrapper;
@@ -202,12 +209,12 @@ resolveStringRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDA
 
 	romStringRef = (J9ROMStringRef *)&ramCP->romConstantPool[cpIndex];
 	utf8Wrapper = J9ROMSTRINGREF_UTF8DATA(romStringRef);
-	
+
 	Trc_VM_resolveStringRef_utf8(vmStruct, &utf8Wrapper, J9UTF8_LENGTH(utf8Wrapper), J9UTF8_DATA(utf8Wrapper));
 
 	/* Create a new string */
 	stringRef = vmStruct->javaVM->memoryManagerFunctions->j9gc_createJavaLangString(vmStruct, J9UTF8_DATA(utf8Wrapper), J9UTF8_LENGTH(utf8Wrapper), J9_STR_TENURE | J9_STR_INTERN);
-	
+
 	/* If stringRef is NULL, the exception has already been set. */
 	if ((NULL != stringRef) && J9_ARE_NO_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_NO_CP_UPDATE)) {
 		J9Class *clazz = J9_CLASS_FROM_CP(ramCP);
@@ -223,7 +230,7 @@ resolveStringRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDA
 }
 
 
-J9Class *   
+J9Class *
 findFieldSignatureClass(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA fieldRefCpIndex)
 {
 	J9ROMFieldRef *romFieldRef;
@@ -245,7 +252,7 @@ findFieldSignatureClass(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA field
 	if ('[' == J9UTF8_DATA(signature)[0]) {
 		resolvedClass = internalFindClassUTF8(vmStruct, J9UTF8_DATA(signature), J9UTF8_LENGTH(signature), classLoader, J9_FINDCLASS_FLAG_THROW_ON_FAIL);
 	} else {
-		Assert_VM_true('L' == J9UTF8_DATA(signature)[0]);
+		Assert_VM_true(IS_CLASS_SIGNATURE(J9UTF8_DATA(signature)[0]));
 		/* skip fieldSignature's L and ; to have only CLASSNAME required for internalFindClassUTF8 */
 		resolvedClass = internalFindClassUTF8(vmStruct, &J9UTF8_DATA(signature)[1], J9UTF8_LENGTH(signature)-2, classLoader, J9_FINDCLASS_FLAG_THROW_ON_FAIL);
 	}
@@ -253,7 +260,7 @@ findFieldSignatureClass(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA field
 	return resolvedClass;
 }
 
-J9Class *   
+J9Class *
 resolveClassRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags)
 {
 	J9JavaVM *vm = vmStruct->javaVM;
@@ -273,7 +280,7 @@ resolveClassRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDAT
 	UDATA lookupOptions = 0;
 	if (canRunJavaCode) {
 		if (!throwException) {
-			lookupOptions = J9_LOOK_NO_THROW;			
+			lookupOptions = J9_LOOK_NO_THROW;
 		}
 	} else {
 		lookupOptions = J9_LOOK_NO_JAVA;
@@ -311,7 +318,7 @@ tryAgain:
 			findClassFlags |= J9_FINDCLASS_FLAG_THROW_ON_FAIL;
 		}
 		if (J9_ARE_ANY_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_NO_CLASS_LOAD)) {
-			findClassFlags |= J9_FINDCLASS_FLAG_EXISTING_ONLY;		
+			findClassFlags |= J9_FINDCLASS_FLAG_EXISTING_ONLY;
 		}
 	} else {
 		findClassFlags |= J9_FINDCLASS_FLAG_EXISTING_ONLY;
@@ -331,16 +338,6 @@ tryAgain:
 		goto done;
 	}
 
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-	if ((';' == *(char *)(classNameData + (classNameLength - 1)))
-		&& ('Q' == *(char *)classNameData)
-	) {
-		/* we are dealing with signature envelope, extract the name from it */
-		classNameData += 1;
-		classNameLength -= 2;
-	}
-#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-
 	resolvedClass = internalFindClassUTF8(vmStruct, classNameData, classNameLength,
 			classLoader, findClassFlags);
 
@@ -362,13 +359,14 @@ tryAgain:
 		goto done;
 	}
 
+#if JAVA_SPEC_VERSION < 24
 	/* Perform a package access check from the current class to the resolved class.
 	 * No check is required if any of the following is true:
 	 * 		- the current class and resolved class are identical
 	 * 		- the current class was loaded by the bootstrap class loader
 	 * 		- the current class and resolved class are in same module
 	 */
-	if ((currentClass != resolvedClass) 
+	if ((currentClass != resolvedClass)
 		&& (classLoader != bootstrapClassLoader)
 		&& requirePackageAccessCheck(vm, classLoader, currentClass->module, resolvedClass)
 	) {
@@ -387,6 +385,7 @@ tryAgain:
 			goto bail;
 		}
 	}
+#endif /* JAVA_SPEC_VERSION < 24 */
 
 	if (jitCompileTimeResolve) {
 		if (J9_ARE_NO_BITS_SET(resolvedClass->romClass->modifiers, J9AccInterface)) {
@@ -433,7 +432,7 @@ tryAgain:
 			goto bail;
 		}
 	}
-	
+
 	if (J9_ARE_ANY_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_INIT_CLASS)) {
 		UDATA initStatus = resolvedClass->initializeStatus;
 		if ((J9ClassInitSucceeded != initStatus) && ((UDATA)vmStruct != initStatus)) {
@@ -462,14 +461,14 @@ tryAgain:
 
 done:
 	Trc_VM_resolveClassRef_Exit(vmStruct, resolvedClass);
-	return resolvedClass; 
+	return resolvedClass;
 bail:
 	resolvedClass = NULL;
 	goto done;
 }
 
 
-J9Method *   
+J9Method *
 resolveStaticMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9RAMStaticMethodRef *ramCPEntry)
 {
 	J9ROMMethodRef *romMethodRef;
@@ -485,7 +484,7 @@ resolveStaticMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cp
 	UDATA lookupOptions = 0;
 	if (canRunJavaCode) {
 		if (!throwException) {
-			lookupOptions = J9_LOOK_NO_THROW;			
+			lookupOptions = J9_LOOK_NO_THROW;
 		}
 	} else {
 		lookupOptions = J9_LOOK_NO_JAVA;
@@ -592,7 +591,7 @@ incompat:
 	}
 
 	/* Initialize the defining class of the method. */
-	
+
 	if (J9_ARE_NO_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_NO_CLASS_INIT)) {
 		J9Class *methodClass = J9_CLASS_FROM_METHOD(method);
 		UDATA initStatus = methodClass->initializeStatus;
@@ -630,7 +629,7 @@ done:
 }
 
 
-J9Method *   
+J9Method *
 resolveStaticMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags)
 {
 	/* Bit of magic here, the resulting method must be in vmStruct->floatTemp1 in the CLINIT case
@@ -640,7 +639,7 @@ resolveStaticMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpInde
 	J9Method *method;
 
 	method = resolveStaticMethodRefInto(vmStruct, ramCP, cpIndex, resolveFlags, ramStaticMethodRef);
-	
+
 	if (method != NULL) {
 		/* Check for <clinit> case. */
 		if (((resolveFlags & J9_RESOLVE_FLAG_CHECK_CLINIT) == J9_RESOLVE_FLAG_CHECK_CLINIT)
@@ -656,7 +655,7 @@ resolveStaticMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpInde
 }
 
 
-J9Method *   
+J9Method *
 resolveStaticSplitMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA splitTableIndex, UDATA resolveFlags)
 {
 	J9RAMStaticMethodRef *ramStaticMethodRef = (J9RAMStaticMethodRef *)&vmStruct->floatTemp1;
@@ -672,7 +671,7 @@ resolveStaticSplitMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA s
 				&& (J9_CLASS_FROM_METHOD(method)->initializeStatus == (UDATA)vmStruct)
 			) {
 				return (J9Method *) -1;
-			} else if (J9_ARE_NO_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_NO_CP_UPDATE)) {			
+			} else if (J9_ARE_NO_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_NO_CP_UPDATE)) {
 				ramCP->ramClass->staticSplitMethodTable[splitTableIndex] = method;
 			}
 		}
@@ -682,7 +681,7 @@ resolveStaticSplitMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA s
 }
 
 
-void *    
+void *
 resolveStaticFieldRefInto(J9VMThread *vmStruct, J9Method *method, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9ROMFieldShape **resolvedField, J9RAMStaticFieldRef *ramCPEntry)
 {
 	void *staticAddress;
@@ -694,7 +693,7 @@ resolveStaticFieldRefInto(J9VMThread *vmStruct, J9Method *method, J9ConstantPool
 	UDATA lookupOptions = 0;
 	if (canRunJavaCode) {
 		if (!throwException) {
-			lookupOptions = J9_LOOK_NO_THROW;			
+			lookupOptions = J9_LOOK_NO_THROW;
 		}
 	} else {
 		lookupOptions = J9_LOOK_NO_JAVA;
@@ -804,7 +803,7 @@ illegalAccess:
 					VM_VMHelpers::reportFinalFieldModified(vmStruct, definingClass);
 				}
 			}
-			
+
 			/* check class constraints */
 			if ((javaVM->runtimeFlags & J9_RUNTIME_VERIFY) != 0) {
 				J9ClassLoader *cl1 = classFromCP->classLoader;
@@ -815,7 +814,7 @@ illegalAccess:
 				}
 				if (cl1 != cl2) {
 					J9UTF8 *fieldSignature = J9ROMFIELDSHAPE_SIGNATURE(field);
-					if (j9bcv_checkClassLoadingConstraintsForSignature(vmStruct, cl1, cl2, signature, fieldSignature) != 0) {
+					if (0 != j9bcv_checkClassLoadingConstraintsForSignature(vmStruct, cl1, cl2, signature, fieldSignature, FALSE)) {
 						if (throwException) {
 							setCurrentException(vmStruct, J9VMCONSTANTPOOL_JAVALANGVERIFYERROR, NULL);
 						}
@@ -824,7 +823,7 @@ illegalAccess:
 					}
 				}
 			}
-			
+
 			/* If this is a JIT compile-time resolve, do not allow fields declared in uninitialized classes. */
 			if (jitCompileTimeResolve && (J9ClassInitSucceeded != initStatus)) {
 				goto done;
@@ -832,13 +831,31 @@ illegalAccess:
 
 			if ((NULL != ramCPEntry) && J9_ARE_NO_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_NO_CP_UPDATE)) {
 				UDATA localClassAndFlagsData = (UDATA)localClassAndFlags;
-				/* Add bits to the class for base type and double wide field. */
-				if ((modifiers & J9FieldFlagObject) != J9FieldFlagObject) {
-					localClassAndFlagsData |= J9StaticFieldRefBaseType;
-					if ((modifiers & J9FieldSizeDouble) == J9FieldSizeDouble) {
-						localClassAndFlagsData |= J9StaticFieldRefDouble;
-					} else if (J9FieldTypeBoolean == (modifiers & J9FieldTypeMask)) {
-						localClassAndFlagsData |= J9StaticFieldRefBoolean;
+				/* Add bits to the class for the field type */
+				if ((modifiers & J9FieldFlagObject) == J9FieldFlagObject) {
+					localClassAndFlagsData |= J9StaticFieldRefTypeObject;
+				} else {
+					switch(modifiers & J9FieldTypeMask) {
+					case J9FieldTypeBoolean:
+						localClassAndFlagsData |= J9StaticFieldRefTypeBoolean;
+						break;
+					case J9FieldTypeByte:
+						localClassAndFlagsData |= J9StaticFieldRefTypeByte;
+						break;
+					case J9FieldTypeChar:
+						localClassAndFlagsData |= J9StaticFieldRefTypeChar;
+						break;
+					case J9FieldTypeShort:
+						localClassAndFlagsData |= J9StaticFieldRefTypeShort;
+						break;
+					case J9FieldTypeInt:
+					case J9FieldTypeFloat:
+						localClassAndFlagsData |= J9StaticFieldRefTypeIntFloat;
+						break;
+					case J9FieldTypeLong:
+					case J9FieldTypeDouble:
+						localClassAndFlagsData |= J9StaticFieldRefTypeLongDouble;
+						break;
 					}
 				}
 				/* Set the volatile, final and setter bits in the flags as needed */
@@ -851,6 +868,11 @@ illegalAccess:
 				if (0 != (resolveFlags & J9_RESOLVE_FLAG_FIELD_SETTER)) {
 					localClassAndFlagsData |= J9StaticFieldRefPutResolved;
 				}
+#if defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES)
+				if (J9_ARE_ALL_BITS_SET(modifiers, J9FieldFlagIsNullRestricted)) {
+					localClassAndFlagsData |= J9StaticFieldIsNullRestricted;
+				}
+#endif /* defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES) */
 				/* Swap the class address bits and the flag bits. */
 				ramCPEntry->flagsAndClass = J9FLAGSANDCLASS_FROM_CLASSANDFLAGS(localClassAndFlagsData);
 				/* Set the high bit in valueOffset to ensure that a resolved static field ref is always interpreted as an unresolved instance fieldref. */
@@ -865,7 +887,7 @@ done:
 }
 
 
-void *   
+void *
 resolveStaticFieldRef(J9VMThread *vmStruct, J9Method *method, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9ROMFieldShape **resolvedField)
 {
 	/* Bit of magic here, the resulting field must be in vmStruct->floatTemp1 in the CLINIT case
@@ -875,7 +897,7 @@ resolveStaticFieldRef(J9VMThread *vmStruct, J9Method *method, J9ConstantPool *ra
 	void *staticAddress;
 
 	staticAddress = resolveStaticFieldRefInto(vmStruct, method, ramCP, cpIndex, resolveFlags, resolvedField, ramStaticFieldRef);
-	
+
 	if (staticAddress != NULL) {
 		/* Check for <clinit> case. */
 		if ((resolveFlags & J9_RESOLVE_FLAG_CHECK_CLINIT) == J9_RESOLVE_FLAG_CHECK_CLINIT) {
@@ -894,7 +916,7 @@ resolveStaticFieldRef(J9VMThread *vmStruct, J9Method *method, J9ConstantPool *ra
 }
 
 
-IDATA   
+IDATA
 resolveInstanceFieldRefInto(J9VMThread *vmStruct, J9Method *method, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9ROMFieldShape **resolvedField, J9RAMFieldRef *ramCPEntry)
 {
 	IDATA fieldOffset = -1;
@@ -906,19 +928,19 @@ resolveInstanceFieldRefInto(J9VMThread *vmStruct, J9Method *method, J9ConstantPo
 	UDATA lookupOptions = 0;
 	if (canRunJavaCode) {
 		if (!throwException) {
-			lookupOptions = J9_LOOK_NO_THROW;			
+			lookupOptions = J9_LOOK_NO_THROW;
 		}
 	} else {
 		lookupOptions = J9_LOOK_NO_JAVA;
 	}
 
 	Trc_VM_resolveInstanceFieldRef_Entry(vmStruct, method, ramCP, cpIndex, resolveFlags, resolvedField);
-	
+
 	/* Get the class.  Stop immediately if an exception occurs. */
 	romFieldRef = (J9ROMFieldRef *)&ramCP->romConstantPool[cpIndex];
 	/* Resolve the class. */
 	resolvedClass = resolveClassRef(vmStruct, ramCP, romFieldRef->classRefCPIndex, resolveFlags);
-	
+
 	/* If clazz is NULL, the exception has already been set. */
 	if (resolvedClass != NULL) {
 		J9JavaVM *javaVM = vmStruct->javaVM;
@@ -933,13 +955,12 @@ resolveInstanceFieldRefInto(J9VMThread *vmStruct, J9Method *method, J9ConstantPo
 		J9Class *targetClass = NULL;
 		UDATA modifiers = 0;
 		char *nlsStr = NULL;
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
+#if defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES)
 		UDATA fieldIndex = 0;
 		bool fccEntryFieldNotSet = true;
 		J9Class *flattenableClass = NULL;
 		J9FlattenedClassCache *flattenedClassCache = NULL;
-#endif		
-		bool isWithField = false;
+#endif /* J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES */
 		J9Class *currentTargetClass = NULL;
 		J9Class *currentSenderClass = NULL;
 
@@ -957,20 +978,18 @@ resolveInstanceFieldRefInto(J9VMThread *vmStruct, J9Method *method, J9ConstantPo
 		nameAndSig = J9ROMFIELDREF_NAMEANDSIGNATURE(romFieldRef);
 		name = J9ROMNAMEANDSIGNATURE_NAME(nameAndSig);
 		signature = J9ROMNAMEANDSIGNATURE_SIGNATURE(nameAndSig);
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-		isWithField = J9_ARE_ANY_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_WITH_FIELD);
 		/**
-		 * This is an optimization that searches for a field offset in the FCC. 
-		 * If the offset is found there is no need to repeat the process. 
-		 * Also, since this optimization is only done for ValueTypes, 
-		 * the resolvedClass will always be the class that owns the field, 
+		 * This is an optimization that searches for a field offset in the FCC.
+		 * If the offset is found there is no need to repeat the process.
+		 * Also, since this optimization is only done for ValueTypes,
+		 * the resolvedClass will always be the class that owns the field,
 		 * since ValueType superclasses can not have fields.
 		 */
+#if defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES)
 		if (J9_IS_J9CLASS_VALUETYPE(resolvedClass)) {
-			if ('Q' == J9UTF8_DATA(signature)[0]) {
-				flattenedClassCache = resolvedClass->flattenedClassCache;
-				fieldIndex = findIndexInFlattenedClassCache(flattenedClassCache, nameAndSig);
-				Assert_VM_false(UDATA_MAX == fieldIndex);
+			flattenedClassCache = resolvedClass->flattenedClassCache;
+			fieldIndex = findIndexInFlattenedClassCache(flattenedClassCache, nameAndSig);
+			if (UDATA_MAX != fieldIndex) {
 				J9FlattenedClassCacheEntry * flattenedClassCacheEntry = J9_VM_FCC_ENTRY_FROM_FCC(flattenedClassCache, fieldIndex);
 				fieldOffset = flattenedClassCacheEntry->offset;
 				if (-1 != fieldOffset) {
@@ -981,8 +1000,8 @@ resolveInstanceFieldRefInto(J9VMThread *vmStruct, J9Method *method, J9ConstantPo
 				}
 			}
 		}
-		if (fccEntryFieldNotSet) 
-#endif
+		if (fccEntryFieldNotSet)
+#endif /* defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES) */
 		{
 			fieldOffset = instanceFieldOffsetWithSourceClass(vmStruct, resolvedClass, J9UTF8_DATA(name), J9UTF8_LENGTH(name), J9UTF8_DATA(signature), J9UTF8_LENGTH(signature), &definingClass, (UDATA *)&field, lookupOptions, classFromCP);
 		}
@@ -1009,12 +1028,8 @@ resolveInstanceFieldRefInto(J9VMThread *vmStruct, J9Method *method, J9ConstantPo
 			}
 
 			if ((modifiers & J9AccFinal) != 0) {
-				if (J9_ARE_ANY_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_FIELD_SETTER)
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-					|| isWithField
-#endif /* defined(J9VM_OPT_VALHALLA_VALUE_TYPES) */
-				) {
-					checkResult = checkVisibility(vmStruct, classFromCP, definingClass, J9AccPrivate, isWithField ? lookupOptions : (lookupOptions | J9_LOOK_NO_NESTMATES));
+				if (J9_ARE_ANY_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_FIELD_SETTER)) {
+					checkResult = checkVisibility(vmStruct, classFromCP, definingClass, J9AccPrivate, lookupOptions | J9_LOOK_NO_NESTMATES);
 					if (checkResult < J9_VISIBILITY_ALLOWED) {
 						badMemberModifier = J9AccPrivate;
 						targetClass = definingClass;
@@ -1032,15 +1047,7 @@ illegalAccess:
 						}
 						goto done;
 					}
-					if (
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-							/* The withfield bytecode is allowed to set a final field. However, the invoker of withfield must have private access
-							 * to the field (similar to a constructor setting a final field). The private access check is done above, so if we get
-							 * to this point we can skip the finalFieldSetAllowed() if we know its a withfield */
-							!isWithField &&
-#endif
-							!finalFieldSetAllowed(vmStruct, false, method, definingClass, classFromCP, field, canRunJavaCode)
-					) {
+					if (!finalFieldSetAllowed(vmStruct, false, method, definingClass, classFromCP, field, canRunJavaCode)) {
 						fieldOffset = -1;
 						goto done;
 					}
@@ -1057,7 +1064,7 @@ illegalAccess:
 				}
 				if (cl1 != cl2) {
 					J9UTF8 *fieldSignature = J9ROMFIELDSHAPE_SIGNATURE(field);
-					if (j9bcv_checkClassLoadingConstraintsForSignature(vmStruct, cl1, cl2, signature, fieldSignature) != 0) {
+					if (0 != j9bcv_checkClassLoadingConstraintsForSignature(vmStruct, cl1, cl2, signature, fieldSignature, FALSE)) {
 						if (throwException) {
 							setCurrentException(vmStruct, J9VMCONSTANTPOOL_JAVALANGVERIFYERROR, NULL);
 						}
@@ -1066,11 +1073,11 @@ illegalAccess:
 					}
 				}
 			}
-		
+
 			if ((NULL != ramCPEntry) && J9_ARE_NO_BITS_SET(resolveFlags, J9_RESOLVE_FLAG_NO_CP_UPDATE)) {
 				UDATA valueOffset = fieldOffset;
-#if defined(J9VM_OPT_VALHALLA_VALUE_TYPES)
-				if ('Q' == J9UTF8_DATA(signature)[0]) {
+#if defined(J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES)
+				if (J9ROMFIELD_IS_NULL_RESTRICTED(field)) {
 					if (fccEntryFieldNotSet) {
 						flattenedClassCache = definingClass->flattenedClassCache;
 						fieldIndex = findIndexInFlattenedClassCache(flattenedClassCache, nameAndSig);
@@ -1086,7 +1093,7 @@ illegalAccess:
 						issueWriteBarrier();
 					}
 				}
-#endif
+#endif /* J9VM_OPT_VALHALLA_FLATTENABLE_VALUE_TYPES */
 				/* Sign extend the resolved constant to make sure that it is always larger than valueOffset field */
 				modifiers |= (UDATA)(IDATA)(I_32) J9FieldFlagResolved;
 				if (0 != (resolveFlags & J9_RESOLVE_FLAG_FIELD_SETTER)) {
@@ -1108,7 +1115,7 @@ done:
 }
 
 
-IDATA   
+IDATA
 resolveInstanceFieldRef(J9VMThread *vmStruct, J9Method *method, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9ROMFieldShape **resolvedField)
 {
 	J9RAMFieldRef *ramFieldRef = (J9RAMFieldRef *)&ramCP[cpIndex];
@@ -1116,7 +1123,7 @@ resolveInstanceFieldRef(J9VMThread *vmStruct, J9Method *method, J9ConstantPool *
 }
 
 
-J9Method *   
+J9Method *
 resolveInterfaceMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9RAMInterfaceMethodRef *ramCPEntry)
 {
 	J9Method *returnValue = NULL;
@@ -1132,7 +1139,7 @@ resolveInterfaceMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA
 	UDATA lookupOptions = 0;
 	if (canRunJavaCode) {
 		if (!throwException) {
-			lookupOptions = J9_LOOK_NO_THROW;			
+			lookupOptions = J9_LOOK_NO_THROW;
 		}
 	} else {
 		lookupOptions = J9_LOOK_NO_JAVA;
@@ -1146,7 +1153,7 @@ resolveInterfaceMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA
 
 	/* Resolve the class. */
 	interfaceClass = resolveClassRef(vmStruct, ramCP, romMethodRef->classRefCPIndex, resolveFlags);
-	
+
 	/* If interfaceClass is NULL, the exception has already been set. */
 	if (interfaceClass == NULL) {
 		goto done;
@@ -1160,7 +1167,7 @@ resolveInterfaceMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA
 		}
 		goto done;
 	}
-	
+
 	nameAndSig = J9ROMFIELDREF_NAMEANDSIGNATURE(romMethodRef);
 	lookupOptions |= J9_LOOK_INTERFACE;
 	if ((resolveFlags & J9_RESOLVE_FLAG_JCL_CONSTANT_POOL) == J9_RESOLVE_FLAG_JCL_CONSTANT_POOL) {
@@ -1172,7 +1179,7 @@ resolveInterfaceMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA
 	method = (J9Method *)javaLookupMethod(vmStruct, interfaceClass, nameAndSig, cpClass, lookupOptions);
 
 	Trc_VM_resolveInterfaceMethodRef_lookupMethod(vmStruct, method);
-	
+
 	/* If method is NULL, the exception has already been set. */
 	if (method != NULL) {
 		if (ramCPEntry != NULL) {
@@ -1244,7 +1251,7 @@ done:
 }
 
 
-J9Method *   
+J9Method *
 resolveInterfaceMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags)
 {
 	J9RAMInterfaceMethodRef *ramInterfaceMethodRef = (J9RAMInterfaceMethodRef *)&ramCP[cpIndex];
@@ -1252,7 +1259,7 @@ resolveInterfaceMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpI
 }
 
 
-J9Method *   
+J9Method *
 resolveSpecialMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9RAMSpecialMethodRef *ramCPEntry)
 {
 	J9ROMMethodRef *romMethodRef;
@@ -1266,7 +1273,7 @@ resolveSpecialMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA c
 	UDATA lookupOptions = 0;
 	if (canRunJavaCode) {
 		if (!throwException) {
-			lookupOptions = J9_LOOK_NO_THROW;			
+			lookupOptions = J9_LOOK_NO_THROW;
 		}
 	} else {
 		lookupOptions = J9_LOOK_NO_JAVA;
@@ -1280,7 +1287,7 @@ resolveSpecialMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA c
 
 	/* Resolve the class. */
 	resolvedClass = resolveClassRef(vmStruct, ramCP, romMethodRef->classRefCPIndex, resolveFlags);
-	
+
 	/* If resolvedClass is NULL, the exception has already been set. */
 	if (resolvedClass == NULL) {
 		goto done;
@@ -1301,7 +1308,7 @@ resolveSpecialMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA c
 	 * Special invocations (defender supersends) will not look at the receiver's vtable, but instead invoke the result of javaLookupMethod.
 	 * Default method conflicts must therefore be handled by the lookup code.
 	 */
-	lookupOptions |= (J9_LOOK_VIRTUAL | J9_LOOK_ALLOW_FWD | J9_LOOK_HANDLE_DEFAULT_METHOD_CONFLICTS);
+	lookupOptions |= (J9_LOOK_VIRTUAL | J9_LOOK_HANDLE_DEFAULT_METHOD_CONFLICTS);
 
 	if ((resolveFlags & J9_RESOLVE_FLAG_JCL_CONSTANT_POOL) != J9_RESOLVE_FLAG_JCL_CONSTANT_POOL) {
 		lookupOptions |= J9_LOOK_CLCONSTRAINTS;
@@ -1342,9 +1349,9 @@ incompat:
 	}
 
 	method = (J9Method *)javaLookupMethod(vmStruct, resolvedClass, nameAndSig, currentClass, lookupOptions);
-	
+
 	Trc_VM_resolveSpecialMethodRef_lookupMethod(vmStruct, method);
-	
+
 	if (method == NULL) {
 		goto done;
 	} else {
@@ -1376,7 +1383,7 @@ done:
 }
 
 
-J9Method *   
+J9Method *
 resolveSpecialMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags)
 {
 	J9RAMSpecialMethodRef *ramSpecialMethodRef = (J9RAMSpecialMethodRef *)&ramCP[cpIndex];
@@ -1384,12 +1391,12 @@ resolveSpecialMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpInd
 }
 
 
-J9Method *   
+J9Method *
 resolveSpecialSplitMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA splitTableIndex, UDATA resolveFlags)
 {
 	U_16 cpIndex = *(U_16 *)(J9ROMCLASS_SPECIALSPLITMETHODREFINDEXES(ramCP->ramClass->romClass) + splitTableIndex);
 	J9Method *method = ramCP->ramClass->specialSplitMethodTable[splitTableIndex];
-	
+
 	if (method == (J9Method*)vmStruct->javaVM->initialMethods.initialSpecialMethod) {
 		method = resolveSpecialMethodRefInto(vmStruct, ramCP, cpIndex, resolveFlags, NULL);
 
@@ -1401,7 +1408,7 @@ resolveSpecialSplitMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA
 }
 
 
-UDATA   
+UDATA
 resolveVirtualMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9Method **resolvedMethod, J9RAMVirtualMethodRef *ramCPEntry)
 {
 	UDATA vTableOffset = 0;
@@ -1414,7 +1421,7 @@ resolveVirtualMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA c
 	UDATA lookupOptions = 0;
 	if (canRunJavaCode) {
 		if (!throwException) {
-			lookupOptions = J9_LOOK_NO_THROW;			
+			lookupOptions = J9_LOOK_NO_THROW;
 		}
 	} else {
 		lookupOptions = J9_LOOK_NO_JAVA;
@@ -1652,7 +1659,7 @@ resolveVirtualMethodRefInto(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA c
 		if (NULL != method) {
 			J9ROMMethod* romMethod = J9_ROM_METHOD_FROM_RAM_METHOD(method);
 			/* Only allow non-interface method to call invokePrivate, private interface method should use "invokeInterface" bytecode
-			 * The else case will throw ICCE for private interface method 
+			 * The else case will throw ICCE for private interface method
 			 */
 			if (!J9ROMMETHOD_HAS_VTABLE(romMethod) && J9_ARE_NO_BITS_SET(resolvedClass->romClass->modifiers, J9AccInterface)) {
 				/* Private method found, will not be in vTable, point vTable index to invokePrivate */
@@ -1698,8 +1705,8 @@ done:
 	return vTableOffset;
 }
 
-	
-UDATA   
+
+UDATA
 resolveVirtualMethodRef(J9VMThread *vmStruct, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9Method **resolvedMethod)
 {
 	J9RAMVirtualMethodRef *ramVirtualMethodRef = (J9RAMVirtualMethodRef *)&ramCP[cpIndex];
@@ -1829,7 +1836,7 @@ resolveMethodTypeRefInto(J9VMThread *vmThread, J9ConstantPool *ramCP, UDATA cpIn
 	return methodType;
 }
 
-j9object_t   
+j9object_t
 resolveMethodTypeRef(J9VMThread *vmThread, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags)
 {
 	J9RAMMethodTypeRef *ramMethodTypeRef = (J9RAMMethodTypeRef *)&ramCP[cpIndex];
@@ -1837,7 +1844,7 @@ resolveMethodTypeRef(J9VMThread *vmThread, J9ConstantPool *ramCP, UDATA cpIndex,
 
 }
 
-j9object_t   
+j9object_t
 resolveMethodHandleRefInto(J9VMThread *vmThread, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags, J9RAMMethodHandleRef *ramCPEntry)
 {
 	J9ROMMethodHandleRef *romMethodHandleRef;
@@ -1995,8 +2002,8 @@ resolveMethodHandleRefInto(J9VMThread *vmThread, J9ConstantPool *ramCP, UDATA cp
 	/* Only write the value in if its not null */
 	if (NULL != methodHandle) {
 		methodHandle = vmThread->javaVM->memoryManagerFunctions->j9gc_objaccess_asConstantPoolObject(
-																		vmThread, 
-																		methodHandle, 
+																		vmThread,
+																		methodHandle,
 																		J9_GC_ALLOCATE_OBJECT_TENURED | J9_GC_ALLOCATE_OBJECT_NON_INSTRUMENTABLE | J9_GC_ALLOCATE_OBJECT_HASHED);
 		if (NULL == methodHandle) {
 			if (throwException) {
@@ -2014,7 +2021,7 @@ _done:
 	return methodHandle;
 }
 
-j9object_t   
+j9object_t
 resolveMethodHandleRef(J9VMThread *vmThread, J9ConstantPool *ramCP, UDATA cpIndex, UDATA resolveFlags)
 {
 	J9RAMMethodHandleRef *ramMethodHandleRef = (J9RAMMethodHandleRef *)&ramCP[cpIndex];
@@ -2231,24 +2238,23 @@ resolveInvokeDynamic(J9VMThread *vmThread, J9ConstantPool *ramCP, UDATA callSite
 		/* Check if an exception is already pending */
 		if (vmThread->currentException != NULL) {
 			/* Already a pending exception */
-			result = vmThread->currentException;
+			result = NULL;
 		} else if (NULL == result) {
 			setCurrentExceptionUTF(vmThread, J9VMCONSTANTPOOL_JAVALANGNULLPOINTEREXCEPTION, NULL);
-			result = vmThread->currentException;
-		}
-
-		/* The result can be an array or exception. Ensure that the result and its elements are
-		 * written/published before the result reference is stored.
-		 */
-		VM_AtomicSupport::writeBarrier();
-
-		J9MemoryManagerFunctions *gcFuncs = vmThread->javaVM->memoryManagerFunctions;
-		J9Class *j9class = J9_CLASS_FROM_CP(ramCP);
-		if (0 == gcFuncs->j9gc_objaccess_staticCompareAndSwapObject(vmThread, j9class, callSite, NULL, result)) {
-			/* Another thread beat this thread to updating the call site, ensure both threads
-			 * return the same method handle.
+		} else {
+			/* The result is an array. Ensure that the result and its elements are
+			 * written/published before the result reference is stored.
 			 */
-			result = *callSite;
+			VM_AtomicSupport::writeBarrier();
+
+			J9MemoryManagerFunctions *gcFuncs = vmThread->javaVM->memoryManagerFunctions;
+			J9Class *j9class = J9_CLASS_FROM_CP(ramCP);
+			if (0 == gcFuncs->j9gc_objaccess_staticCompareAndSwapObject(vmThread, j9class, callSite, NULL, result)) {
+				/* Another thread beat this thread to updating the call site, ensure both threads
+				 * return the same method handle.
+				 */
+				result = *callSite;
+			}
 		}
 #else /* defined(J9VM_OPT_OPENJDK_METHODHANDLE) */
 		/* Check if an exception is already pending */
@@ -2280,3 +2286,32 @@ resolveInvokeDynamic(J9VMThread *vmThread, J9ConstantPool *ramCP, UDATA callSite
 	Trc_VM_resolveInvokeDynamic_Exit(vmThread, result);
 	return result;
 }
+
+#if JAVA_SPEC_VERSION >= 16
+/**
+ * @brief The function calls into the interpreter via sendResolveFfiCallInvokeHandle()
+ * to fetch the MemberName object plus appendix intended for the downcall/upcall method.
+ *
+ * @param vmThread the pointer to J9VMThread
+ * @param handle the downcall/upcall MethodHandle object
+ * @return the cache array that stores MemberName and appendix
+ */
+j9object_t
+resolveFfiCallInvokeHandle(J9VMThread *vmThread, j9object_t handle)
+{
+	j9object_t invokeCache = NULL;
+	Trc_VM_resolveFfiCallInvokeHandle_Entry(vmThread);
+
+	sendResolveFfiCallInvokeHandle(vmThread, handle);
+	invokeCache = (j9object_t)vmThread->returnValue;
+	if (VM_VMHelpers::exceptionPending(vmThread)) {
+		/* Already a pending exception */
+		invokeCache = NULL;
+	} else if (NULL == invokeCache) {
+		setCurrentExceptionUTF(vmThread, J9VMCONSTANTPOOL_JAVALANGNULLPOINTEREXCEPTION, NULL);
+	}
+
+	Trc_VM_resolveFfiCallInvokeHandle_Exit(vmThread, invokeCache);
+	return invokeCache;
+}
+#endif /* JAVA_SPEC_VERSION >= 16 */
