@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include <stdio.h>
@@ -27,11 +27,15 @@
 #include "j9protos.h"
 #include "j9cfg.h"
 #include "j9port.h"
+#if defined(J9VM_OPT_SNAPSHOTS)
+#include "j9port_generated.h"
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
 #include "j9consts.h"
 #include "ut_j9vm.h"
 #include "vm_internal.h"
 #include "segment.h"
 #include "j9modron.h"
+#include "omrutilbase.h"
 
 #define ROUND_TO(granularity, number) (((number) + (granularity) - 1) & ~((UDATA)(granularity) - 1))
 
@@ -120,6 +124,9 @@ void freeMemorySegment(J9JavaVM *javaVM, J9MemorySegment *segment, BOOLEAN freeD
 	segmentList->totalSegmentSize -= segment->size;
 
 	if (segment->type & MEMORY_TYPE_ALLOCATED) {
+#if defined(J9VM_OPT_SNAPSHOTS)
+		VMSNAPSHOTIMPLPORT_ACCESS_FROM_JAVAVM(javaVM);
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
 		BOOLEAN useAdvise = (J9_EXTENDED_RUNTIME_FLAG_JSCRATCH_ADV_ON_FREE == (javaVM->extendedRuntimeFlags & J9_EXTENDED_RUNTIME_FLAG_JSCRATCH_ADV_ON_FREE));
 
 		/* The order of these checks is important.
@@ -130,11 +137,28 @@ void freeMemorySegment(J9JavaVM *javaVM, J9MemorySegment *segment, BOOLEAN freeD
 		} else if ((useAdvise) && (MEMORY_TYPE_JIT_SCRATCH_SPACE & segment->type)) {
 			j9mem_advise_and_free_memory(segment->baseAddress);
 		} else if (segment->type & (MEMORY_TYPE_RAM_CLASS | MEMORY_TYPE_UNDEAD_CLASS)) {
-			if (J9JAVAVM_COMPRESS_OBJECT_REFERENCES(javaVM)) {
-				j9mem_free_memory32(segment->baseAddress);
-			} else {
-				j9mem_free_memory(segment->baseAddress);
+#if defined(J9VM_OPT_SNAPSHOTS)
+			if (IS_SNAPSHOTTING_ENABLED(javaVM)) {
+				if (J9JAVAVM_COMPRESS_OBJECT_REFERENCES(javaVM)) {
+					vmsnapshot_free_memory32(segment->baseAddress);
+				} else {
+					vmsnapshot_free_memory(segment->baseAddress);
+				}
+			} else
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+			{
+				if (J9JAVAVM_COMPRESS_OBJECT_REFERENCES(javaVM)) {
+					j9mem_free_memory32(segment->baseAddress);
+				} else {
+					j9mem_free_memory(segment->baseAddress);
+				}
 			}
+#if defined(J9VM_OPT_SNAPSHOTS)
+		} else if (J9_ARE_ANY_BITS_SET(segment->type, MEMORY_TYPE_ROM_CLASS)
+			&& IS_SNAPSHOTTING_ENABLED(javaVM)
+		) {
+			vmsnapshot_free_memory(segment->baseAddress);
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
 		} else {
 			j9mem_free_memory(segment->baseAddress);
 		}
@@ -180,20 +204,36 @@ void freeMemorySegmentListEntry(J9MemorySegmentList *segmentList, J9MemorySegmen
 void freeMemorySegmentList(J9JavaVM *javaVM, J9MemorySegmentList *segmentList)
 {
 	PORT_ACCESS_FROM_JAVAVM(javaVM);
-	J9MemorySegment *currentSegment;
+#if defined(J9VM_OPT_SNAPSHOTS)
+	VMSNAPSHOTIMPLPORT_ACCESS_FROM_JAVAVM(javaVM);
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+	J9MemorySegment *currentSegment = NULL;
 
 	do {
-		currentSegment = segmentList -> nextSegment ;
-		if ( currentSegment )
+		currentSegment = segmentList->nextSegment;
+		if (NULL != currentSegment)
 			freeMemorySegment(javaVM, currentSegment, 1);
-	} while ( currentSegment );
+	} while (NULL != currentSegment);
 	pool_kill(segmentList->segmentPool);
 
 #if defined(J9VM_THR_PREEMPTIVE)
 	if(segmentList->segmentMutex) omrthread_monitor_destroy(segmentList->segmentMutex);
 #endif
 
-	j9mem_free_memory(segmentList);
+	/* It is guaranteed that classMemorySegments were allocated on the VMSnapshotImpl heap. */
+	/* TODO: In J9MemorySegmentList flags, add an option to signify allocation from the
+	 * VMSnapshotImpl to replace this check (See @ref omr:j9nongenerated.h).
+	 */
+#if defined(J9VM_OPT_SNAPSHOTS)
+	if (IS_SNAPSHOTTING_ENABLED(javaVM)
+		&& ((javaVM->classMemorySegments == segmentList) || (javaVM->memorySegments == segmentList))
+	) {
+		vmsnapshot_free_memory(segmentList);
+	} else
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+	{
+		j9mem_free_memory(segmentList);
+	}
 }
 
 void
@@ -227,8 +267,11 @@ U_32 memorySegmentListSize (J9MemorySegmentList *segmentList)
 static void *
 allocateMemoryForSegment(J9JavaVM *javaVM,J9MemorySegment *segment, J9PortVmemParams *vmemParams, U_32 memoryCategory)
 {
-	void *tmpAddr;
+	void *tmpAddr = NULL;
 	PORT_ACCESS_FROM_GINFO(javaVM);
+#if defined(J9VM_OPT_SNAPSHOTS)
+	VMSNAPSHOTIMPLPORT_ACCESS_FROM_JAVAVM(javaVM);
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
 
 	/* The order of these checks is important.
 	 * MEMORY_TYPE_VIRTUAL is expected to be used along with another bit, like MEMORY_TYPE_JIT_SCRATCH_SPACE.
@@ -239,21 +282,50 @@ allocateMemoryForSegment(J9JavaVM *javaVM,J9MemorySegment *segment, J9PortVmemPa
 		 */
 		Assert_VM_true(J9_ARE_NO_BITS_SET(segment->type, MEMORY_TYPE_VIRTUAL) || J9_ARE_ANY_BITS_SET(segment->type, ~MEMORY_TYPE_VIRTUAL));
 
+		/* Memory segments can be marked as MEMORY_TYPE_DISCLAIMABLE_TO_FILE
+		 * to indicate the intent of allocating memory backed-up by a file.
+		 * This information is passed to the omr port library by setting the
+		 * flag OMRPORT_VMEM_MEMORY_MODE_SHARE_TMP_FILE_OPEN.
+		 * Note that when setting the OMRPORT_VMEM_MEMORY_MODE_SHARE_TMP_FILE_OPEN bit,
+		 * we also have to set the OMRPORT_VMEM_MEMORY_MODE_SHARE_FILE_OPEN bit.
+		 */
+		if (J9_ARE_ALL_BITS_SET(segment->type, MEMORY_TYPE_DISCLAIMABLE_TO_FILE)) {
+			vmemParams->mode |= (OMRPORT_VMEM_MEMORY_MODE_SHARE_TMP_FILE_OPEN | OMRPORT_VMEM_MEMORY_MODE_SHARE_FILE_OPEN);
+		}
+
 		tmpAddr = j9vmem_reserve_memory_ex(&segment->vmemIdentifier, vmemParams);
 	} else if (J9_ARE_ALL_BITS_SET(segment->type, MEMORY_TYPE_FIXED_RAM_CLASS)) {
 		tmpAddr = j9vmem_reserve_memory_ex(&segment->vmemIdentifier, vmemParams);
 		Trc_VM_virtualRAMClassAlloc(tmpAddr);
 	} else if (J9_ARE_ALL_BITS_SET(segment->type, MEMORY_TYPE_RAM_CLASS)) {
-		if (J9JAVAVM_COMPRESS_OBJECT_REFERENCES(javaVM)) {
-			tmpAddr = j9mem_allocate_memory32(segment->size, memoryCategory);
-		} else {
-			tmpAddr = j9mem_allocate_memory(segment->size, memoryCategory);
+#if defined(J9VM_OPT_SNAPSHOTS)
+		if (IS_SNAPSHOTTING_ENABLED(javaVM)) {
+			if (J9JAVAVM_COMPRESS_OBJECT_REFERENCES(javaVM)) {
+				tmpAddr = vmsnapshot_allocate_memory32(segment->size, memoryCategory);
+			} else {
+				tmpAddr = vmsnapshot_allocate_memory(segment->size, memoryCategory);
+			}
+		} else
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+		{
+			if (J9JAVAVM_COMPRESS_OBJECT_REFERENCES(javaVM)) {
+				tmpAddr = j9mem_allocate_memory32(segment->size, memoryCategory);
+			} else {
+				tmpAddr = j9mem_allocate_memory(segment->size, memoryCategory);
+			}
 		}
+#if defined(J9VM_OPT_SNAPSHOTS)
+	} else if (J9_ARE_ALL_BITS_SET(segment->type, MEMORY_TYPE_ROM_CLASS) && IS_SNAPSHOTTING_ENABLED(javaVM)) {
+		tmpAddr = vmsnapshot_allocate_memory(segment->size, memoryCategory);
+		/* TODO: Add Memory type for allocation inside the snapshot. (MEMORY_TYPE_IMAGE_ALLOCATED)
+		 * (See @ref omr:j9nongenerated.h).
+		 */
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
 	} else {
 		tmpAddr = j9mem_allocate_memory(segment->size, memoryCategory);
 	}
 
-	if(tmpAddr) {
+	if (NULL != tmpAddr) {
 		segment->type = segment->type | MEMORY_TYPE_ALLOCATED;
 	}
 
@@ -341,6 +413,7 @@ J9MemorySegment * allocateFixedMemorySegmentInList(J9JavaVM *javaVM, J9MemorySeg
 	} else if (J9_ARE_ALL_BITS_SET(type, MEMORY_TYPE_VIRTUAL)) {
 
 		flags = J9PORT_VMEM_MEMORY_MODE_READ | J9PORT_VMEM_MEMORY_MODE_WRITE | J9PORT_VMEM_MEMORY_MODE_VIRTUAL;
+
 		if (J9_ARE_NO_BITS_SET(type, MEMORY_TYPE_UNCOMMITTED)) {
 			flags |= J9PORT_VMEM_MEMORY_MODE_COMMIT;
 		}
@@ -404,6 +477,15 @@ static J9MemorySegment * allocateVirtualMemorySegmentInListInternal(J9JavaVM *ja
 			freeMemorySegmentListEntry(segmentList, segment);
 			segment = NULL;
 		} else {
+			if (J9_ARE_ALL_BITS_SET(type, MEMORY_TYPE_CODE)) {
+				/* For CodeCache segments the JIT will later write a TR::CodeCache structure pointer at the begining of the segment.
+				 * Until then, make sure that a potential reader sees a NULL pointer.
+				 */
+				omrthread_jit_write_protect_disable();
+				*((UDATA**)allocatedBase) = NULL;
+				issueWriteBarrier();
+				omrthread_jit_write_protect_enable();
+			}
 			segment->baseAddress = allocatedBase;
 			segment->heapBase = allocatedBase;
 			segment->heapTop = (U_8 *)&(segment->heapBase)[size];
@@ -433,16 +515,55 @@ static J9MemorySegment * allocateVirtualMemorySegmentInListInternal(J9JavaVM *ja
 
 J9MemorySegmentList *allocateMemorySegmentListWithSize(J9JavaVM * javaVM, U_32 numberOfMemorySegments, UDATA sizeOfElements, U_32 memoryCategory)
 {
-	J9MemorySegmentList *segmentList;
+	J9MemorySegmentList *segmentList = NULL;
 	PORT_ACCESS_FROM_JAVAVM(javaVM);
 
-	if (NULL == (segmentList = j9mem_allocate_memory(sizeof(J9MemorySegmentList), memoryCategory)))
-		return NULL;
+#if defined(J9VM_OPT_SNAPSHOTS)
+	VMSNAPSHOTIMPLPORT_ACCESS_FROM_JAVAVM(javaVM);
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
 
-	segmentList->segmentPool = pool_new(sizeOfElements, numberOfMemorySegments, 0, 0, J9_GET_CALLSITE(), memoryCategory, POOL_FOR_PORT(PORTLIB));
-	if (!(segmentList->segmentPool)) {
-		j9mem_free_memory(segmentList);
-		return NULL;
+#if defined(J9VM_OPT_SNAPSHOTS)
+	/* Check if the current run is a snapshot run, to ensure correct allocation for the
+	 * class memory segments list.
+	 */
+	if (IS_SNAPSHOT_RUN(javaVM)
+		&& ((J9MEM_CATEGORY_CLASSES == memoryCategory) || (OMRMEM_CATEGORY_VM == memoryCategory))
+	) {
+		segmentList = vmsnapshot_allocate_memory(sizeof(J9MemorySegmentList), memoryCategory);
+		if (NULL == segmentList) {
+			return NULL;
+		}
+		segmentList->segmentPool = pool_new(
+				sizeOfElements,
+				numberOfMemorySegments,
+				0,
+				0,
+				J9_GET_CALLSITE(),
+				memoryCategory,
+				POOL_FOR_PORT(VMSNAPSHOTIMPL_OMRPORT_FROM_JAVAVM(javaVM)));
+		if (NULL == segmentList->segmentPool) {
+			vmsnapshot_free_memory(segmentList);
+			return NULL;
+		}
+	} else
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+	{
+		segmentList = j9mem_allocate_memory(sizeof(J9MemorySegmentList), memoryCategory);
+		if (NULL == segmentList) {
+			return NULL;
+		}
+		segmentList->segmentPool = pool_new(
+				sizeOfElements,
+				numberOfMemorySegments,
+				0,
+				0,
+				J9_GET_CALLSITE(),
+				memoryCategory,
+				POOL_FOR_PORT(PORTLIB));
+		if (NULL == segmentList->segmentPool) {
+			j9mem_free_memory(segmentList);
+			return NULL;
+		}
 	}
 	segmentList->nextSegment = NULL;
 	segmentList->totalSegmentSize = 0;
@@ -464,6 +585,27 @@ J9MemorySegmentList *allocateMemorySegmentListWithSize(J9JavaVM * javaVM, U_32 n
 	return segmentList;
 }
 
+J9Class *
+segmentIteratorNextClass(J9Class **nextClass)
+{
+	for (;;) {
+		J9Class *current = *nextClass;
+
+		if (NULL == current) {
+			break;
+		}
+		*nextClass = current->nextClassInSegment;
+#if defined(J9VM_OPT_SNAPSHOTS)
+		if (J9_ARE_NO_BITS_SET(current->classFlags, J9ClassIsFrozen))
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+		{
+			return current;
+		}
+	}
+
+	return NULL;
+}
+
 static void
 prepareClassWalkState(J9ClassWalkState* state, J9JavaVM* vm, J9ClassLoader* classLoader)
 {
@@ -490,18 +632,18 @@ allClassesStartDo(J9ClassWalkState* state, J9JavaVM* vm, J9ClassLoader* classLoa
 	return allClassesNextDo(state);
 }
 
-J9Class*
-allClassesNextDo(J9ClassWalkState* state)
+static J9Class *
+allClassesNextDoInternal(J9ClassWalkState *state)
 {
-	J9Class* clazzPtr = NULL;
+	J9Class *clazzPtr = NULL;
 	while ((state->nextSegment != NULL) && (clazzPtr == NULL)) {
-		J9MemorySegment* nextSegment = state->nextSegment;
+		J9MemorySegment *nextSegment = state->nextSegment;
 		if ((nextSegment->type & MEMORY_TYPE_RAM_CLASS) == MEMORY_TYPE_RAM_CLASS) {
 			if (state->heapPtr < nextSegment->heapBase || state->heapPtr > nextSegment->heapAlloc) {
 				state->heapPtr = *(U_8 **)nextSegment->heapBase;
 			}
 			if (state->heapPtr != NULL) {
-				clazzPtr = (J9Class*)state->heapPtr;
+				clazzPtr = (J9Class *)state->heapPtr;
 				state->heapPtr = (U_8 *)clazzPtr->nextClassInSegment;
 			}
 			if (state->heapPtr != NULL) {
@@ -518,6 +660,22 @@ allClassesNextDo(J9ClassWalkState* state)
 	return clazzPtr;
 }
 
+J9Class *
+allClassesNextDo(J9ClassWalkState *state)
+{
+	J9Class *clazz = allClassesNextDoInternal(state);
+
+#if defined(J9VM_OPT_SNAPSHOTS)
+	if (IS_RESTORE_RUN(state->vm)) {
+		while ((NULL != clazz) && J9_ARE_ANY_BITS_SET(clazz->classFlags, J9ClassIsFrozen)) {
+			clazz = allClassesNextDoInternal(state);
+		}
+	}
+#endif /* defined(J9VM_OPT_SNAPSHOTS) */
+
+	return clazz;
+}
+
 void allClassesEndDo(J9ClassWalkState* state)
 {
 #if defined(J9VM_THR_PREEMPTIVE)
