@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2022 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 #include "j9.h"
@@ -42,7 +42,7 @@
 #endif
 
 #ifndef J9VM_INTERP_STACKWALK_TRACING
-#define jitWalkStackFrames walkState->walkThread->javaVM->jitWalkStackFrames
+#define jitWalkStackFrames walkState->javaVM->jitWalkStackFrames
 #endif
 
 #include "rommeth.h"
@@ -91,13 +91,47 @@ static void dropToCurrentFrame (J9StackWalkState * walkState);
 /* The minimum number of stack slots that a stack frame can occupy */
 #define J9_STACKWALK_MIN_FRAME_SLOTS (OMR_MIN(sizeof(J9JITFrame), sizeof(J9SFStackFrame)) / sizeof(UDATA))
 
+static VMINLINE UDATA
+maxFramesOnStack(J9StackWalkState *walkState)
+{
+	J9VMThread *walkThread = walkState->walkThread;
+	J9JITConfig *jitConfig = walkThread->javaVM->jitConfig;
+	UDATA * endOfStack = walkThread->stackObject->end;
+	UDATA * sp = walkState->walkThread->sp;
+	UDATA maxFrames = (endOfStack - sp) / J9_STACKWALK_MIN_FRAME_SLOTS;
+
+	if (NULL != jitConfig) {
+		if (J9_ARE_NO_BITS_SET(walkState->flags, J9_STACKWALK_SKIP_INLINES)) {
+			/* It's possible every frame on the stack is a fully inlined JIT frame */
+			maxFrames *= (jitConfig->maxInlineDepth + 1);
+		}
+	}
+
+	if (J9_ARE_NO_BITS_SET(walkState->flags, J9_STACKWALK_NO_ERROR_REPORT)) {
+		UDATA * stackStart = J9_LOWEST_STACK_SLOT(walkState->walkThread);
+#if defined(J9VM_INTERP_STACKWALK_TRACING)
+		Assert_VRB_true(sp >= stackStart);
+		Assert_VRB_true(sp <= endOfStack);
+#else /* J9VM_INTERP_STACKWALK_TRACING */
+		Assert_VM_true(sp >= stackStart);
+		Assert_VM_true(sp <= endOfStack);
+#endif /* J9VM_INTERP_STACKWALK_TRACING */
+	}
+
+	return maxFrames;
+}
+
 UDATA  walkStackFrames(J9VMThread *currentThread, J9StackWalkState *walkState)
 {
-	UDATA rc = (walkState->walkThread->privateFlags & J9_PRIVATE_FLAGS_STACK_CORRUPT) ? J9_STACKWALK_RC_STACK_CORRUPT : J9_STACKWALK_RC_NONE;
+	UDATA rc = J9_STACKWALK_RC_NONE;
+	UDATA walkRC = 0;
 	J9Method * nextLiterals;
 	UDATA * nextA0;
 	UDATA savedFlags = walkState->flags;
 	J9StackWalkState * oldState = NULL;
+	J9JavaVM *vm = walkState->walkThread->javaVM;
+	U_8 *impdep2PC = vm->callInReturnPC;
+
 #if defined(J9VM_INTERP_STACKWALK_TRACING) 
 	void  (*savedOSlotIterator) (struct J9VMThread * vmThread, struct J9StackWalkState * walkState, j9object_t * objectSlot, const void * stackLocation) =
 		walkState->objectSlotWalkFunction;
@@ -120,12 +154,18 @@ UDATA  walkStackFrames(J9VMThread *currentThread, J9StackWalkState *walkState)
 		walkState->flags &= ~J9_STACKWALK_RESUME;
 		goto resumeInterpreterWalk;
 	}
-	if (currentThread != NULL) {
+
+	walkState->loopBreaker = 0;
+	if (NULL != currentThread) {
 		oldState = currentThread->activeWalkState;
 		currentThread->activeWalkState = walkState;
+		if (J9_ARE_ANY_BITS_SET(currentThread->privateFlags2, J9_PRIVATE_FLAGS2_ASYNC_GET_CALL_TRACE)) {
+			/* Add one because walkFrames decrements loopBreaker and bails when it reaches 0 */
+			walkState->loopBreaker = maxFramesOnStack(walkState) + 1;
+		}
 	}
 
-	walkState->javaVM = walkState->walkThread->javaVM;
+	walkState->javaVM = vm;
 	walkState->currentThread = currentThread;
 	walkState->cache = NULL;
 	walkState->framesWalked = 0;
@@ -155,7 +195,6 @@ UDATA  walkStackFrames(J9VMThread *currentThread, J9StackWalkState *walkState)
 #endif
 #endif
 
-
 #ifdef J9VM_INTERP_STACKWALK_TRACING
 	swPrintf(walkState, 1, "\n");
 	swPrintf(walkState, 1, "*** BEGIN STACK WALK, flags = %p, walkThread = %p, walkState = %p ***\n", walkState->flags, walkState->walkThread, walkState);
@@ -167,7 +206,6 @@ UDATA  walkStackFrames(J9VMThread *currentThread, J9StackWalkState *walkState)
 	if (walkState->flags & J9_STACKWALK_CACHE_PCS) swPrintf(walkState, 2, "\tCACHE_PCS\n");
 	if (walkState->flags & J9_STACKWALK_SKIP_HIDDEN_FRAMES) swPrintf(walkState, 2, "\tSKIP_HIDDEN_FRAME\n");
 	if (walkState->flags & J9_STACKWALK_COUNT_SPECIFIED) swPrintf(walkState, 2, "\tCOUNT_SPECIFIED\n");
-	if (walkState->flags & J9_STACKWALK_INCLUDE_ARRAYLET_LEAVES) swPrintf(walkState, 2, "\tINCLUDE_ARRAYLET_LEAVES\n");
 	if (walkState->flags & J9_STACKWALK_INCLUDE_NATIVES) swPrintf(walkState, 2, "\tINCLUDE_NATIVES\n");
 	if (walkState->flags & J9_STACKWALK_INCLUDE_CALL_IN_FRAMES) swPrintf(walkState, 2, "\tINCLUDE_CALL_IN_FRAMES\n");
 	if (walkState->flags & J9_STACKWALK_ITERATE_FRAMES) swPrintf(walkState, 2, "\tITERATE_FRAMES\n");
@@ -184,19 +222,23 @@ UDATA  walkStackFrames(J9VMThread *currentThread, J9StackWalkState *walkState)
 	if (walkState->flags & J9_STACKWALK_CHECK_I_SLOTS_FOR_OBJECTS) swPrintf(walkState, 2, "\tCHECK_I_SLOTS_FOR_OBJECTS\n");
 	if (walkState->flags & J9_STACKWALK_SAVE_STACKED_REGISTERS) swPrintf(walkState, 2, "\tSAVE_STACKED_REGISTERS\n");
 
-	if ((walkState->flags & (J9_STACKWALK_MAINTAIN_REGISTER_MAP | J9_STACKWALK_INCLUDE_CALL_IN_FRAMES)) == (J9_STACKWALK_MAINTAIN_REGISTER_MAP | J9_STACKWALK_INCLUDE_CALL_IN_FRAMES)) {
-		PORT_ACCESS_FROM_WALKSTATE(walkState);
-		J9UTF8 * className = J9ROMCLASS_CLASSNAME(((J9Class *) walkState->userData4)->romClass);
-		char detailStackBuffer[J9VM_PACKAGE_NAME_BUFFER_LENGTH];
-		char * detail = NULL;
-		j9object_t detailMessage = J9VMJAVALANGTHROWABLE_DETAILMESSAGE(currentThread, walkState->restartException);
+	if (J9_ARE_ALL_BITS_SET(walkState->flags, J9_STACKWALK_MAINTAIN_REGISTER_MAP | J9_STACKWALK_INCLUDE_CALL_IN_FRAMES)) {
+		J9Class * clazz = (J9Class *) walkState->userData4;
 
-		if (NULL != detailMessage) {
-			detail = walkState->walkThread->javaVM->internalVMFunctions->copyStringToUTF8WithMemAlloc(currentThread, detailMessage, J9_STR_NULL_TERMINATE_RESULT, ": ", 2, detailStackBuffer, J9VM_PACKAGE_NAME_BUFFER_LENGTH, NULL);
-		}
-		swPrintf(walkState, 2, "\tThrowing exception: %.*s%s\n", J9UTF8_LENGTH(className), J9UTF8_DATA(className), detail ? detail : "");
-		if (detailStackBuffer != detail) {
-			j9mem_free_memory(detail);
+		if (NULL != clazz) {
+			PORT_ACCESS_FROM_WALKSTATE(walkState);
+			J9UTF8 * className = J9ROMCLASS_CLASSNAME(clazz->romClass);
+			char detailStackBuffer[J9VM_PACKAGE_NAME_BUFFER_LENGTH];
+			char * detail = NULL;
+			j9object_t detailMessage = J9VMJAVALANGTHROWABLE_DETAILMESSAGE(currentThread, walkState->restartException);
+
+			if (NULL != detailMessage) {
+				detail = vm->internalVMFunctions->copyStringToUTF8WithMemAlloc(currentThread, detailMessage, J9_STR_NULL_TERMINATE_RESULT, ": ", 2, detailStackBuffer, J9VM_PACKAGE_NAME_BUFFER_LENGTH, NULL);
+			}
+			swPrintf(walkState, 2, "\tThrowing exception: %.*s%s\n", J9UTF8_LENGTH(className), J9UTF8_DATA(className), detail ? detail : "");
+			if (detailStackBuffer != detail) {
+				j9mem_free_memory(detail);
+			}
 		}
 	}
 
@@ -249,7 +291,7 @@ UDATA  walkStackFrames(J9VMThread *currentThread, J9StackWalkState *walkState)
 
 #ifdef J9VM_INTERP_LINEAR_STACKWALK_TRACING
 	if (walkState->flags & J9_STACKWALK_LINEAR) {
-		rc = lswInitialize(walkState->walkThread->javaVM, walkState);
+		rc = lswInitialize(vm, walkState);
 		if (rc != SW_ERROR_NONE) {
 			rc = J9_STACKWALK_RC_NO_MEMORY;
 			goto terminationPoint;
@@ -268,7 +310,7 @@ UDATA  walkStackFrames(J9VMThread *currentThread, J9StackWalkState *walkState)
 		walkState->bytecodePCOffset = -1;
 
 #ifdef J9VM_INTERP_LINEAR_STACKWALK_TRACING
-		lswFrameNew(walkState->walkThread->javaVM, walkState, (UDATA)walkState->pc);
+		lswFrameNew(vm, walkState, (UDATA)walkState->pc);
 #endif	
 
 		switch((UDATA) walkState->pc) {
@@ -306,7 +348,7 @@ UDATA  walkStackFrames(J9VMThread *currentThread, J9StackWalkState *walkState)
 				}
 #endif
 
-				if (*(walkState->pc) == 0xFF) /* impdep2 = 0xFF - indicates a JNI call-in frame */
+				if ((impdep2PC == walkState->pc) || ((impdep2PC + 3) == walkState->pc)) /* impdep2 = 0xFF - indicates a JNI call-in frame */
 				{
 					walkJNICallInFrame(walkState);
 				} else {
@@ -331,7 +373,9 @@ UDATA  walkStackFrames(J9VMThread *currentThread, J9StackWalkState *walkState)
 
 		/* Walk the frame */
 
-		if (walkFrame(walkState) != J9_STACKWALK_KEEP_ITERATING) {
+		walkRC = walkFrame(walkState);
+		if (J9_STACKWALK_KEEP_ITERATING != walkRC) {
+			rc = walkRC;
 			goto terminationPoint;
 		}
 resumeInterpreterWalk:
@@ -343,7 +387,9 @@ resumeInterpreterWalk:
 #ifdef J9VM_INTERP_NATIVE_SUPPORT
 		if (walkState->frameFlags & J9_STACK_FLAGS_JIT_TRANSITION_TO_INTERPRETER_MASK) {
 resumeJitWalk:
-			if (jitWalkStackFrames(walkState) != J9_STACKWALK_KEEP_ITERATING) {
+			walkRC = jitWalkStackFrames(walkState);
+			if (J9_STACKWALK_KEEP_ITERATING != walkRC) {
+				rc = walkRC;
 				goto terminationPoint;
 			}
 			walkState->decompilationRecord = NULL;
@@ -375,7 +421,7 @@ endOfStack:
 #if defined(J9VM_INTERP_LINEAR_STACKWALK_TRACING)
 	if (walkState->linearSlotWalker) {
 		lswPrintFrames(walkState->walkThread, walkState);
-		lswCleanup(walkState->walkThread->javaVM, walkState);
+		lswCleanup(vm, walkState);
 	}
 #endif	
 
@@ -407,6 +453,13 @@ terminationPoint:
 
 UDATA walkFrame(J9StackWalkState * walkState)
 {
+	if (0 != walkState->loopBreaker) {
+		walkState->loopBreaker -= 1;
+		if (0 == walkState->loopBreaker) {
+			return J9_STACKWALK_RC_STACK_CORRUPT;
+		}
+	}
+
 	if (walkState->flags & J9_STACKWALK_VISIBLE_ONLY) {
 
 		if ((((UDATA) walkState->pc == J9SF_FRAME_TYPE_NATIVE_METHOD) || ((UDATA) walkState->pc == J9SF_FRAME_TYPE_JNI_NATIVE_METHOD)) && !(walkState->flags & J9_STACKWALK_INCLUDE_NATIVES)) {
@@ -417,7 +470,7 @@ UDATA walkFrame(J9StackWalkState * walkState)
 		if (!walkState->jitInfo) {
 #endif
 			if (*walkState->bp & J9SF_A0_INVISIBLE_TAG) {
-				if (!(walkState->flags & J9_STACKWALK_INCLUDE_CALL_IN_FRAMES) || (walkState->pc != walkState->walkThread->javaVM->callInReturnPC)) {
+				if (!(walkState->flags & J9_STACKWALK_INCLUDE_CALL_IN_FRAMES) || (walkState->pc != walkState->javaVM->callInReturnPC)) {
 					return J9_STACKWALK_KEEP_ITERATING;
 				}
 			}
@@ -476,7 +529,7 @@ UDATA walkFrame(J9StackWalkState * walkState)
 					/* If the PC is within the romClass of the method, it's correct (not breakpointed) */
 
 					if ((cachePC < (U_8 *) romClass) || (cachePC >= (((U_8 *) romClass) + romClass->romSize))) {
-						TRIGGER_J9HOOK_VM_PERMANENT_PC(walkState->walkThread->javaVM->hookInterface, walkState->walkThread, cachePC);
+						TRIGGER_J9HOOK_VM_PERMANENT_PC(walkState->javaVM->hookInterface, walkState->walkThread, cachePC);
 					}
 				}
 			}
@@ -520,42 +573,21 @@ walkIt:
 static UDATA allocateCache(J9StackWalkState * walkState)
 {
 	PORT_ACCESS_FROM_WALKSTATE(walkState);
-	UDATA * endOfStack = walkState->walkThread->stackObject->end;
-	UDATA framesPresent = 0;
+	UDATA framesPresent = maxFramesOnStack(walkState);
 	UDATA cacheElementSize = 0;
 	UDATA cacheSize = 0;
 	UDATA * stackStart = J9_LOWEST_STACK_SLOT(walkState->walkThread);
 	UDATA * sp = walkState->walkThread->sp;
 
-	if (J9_ARE_NO_BITS_SET(walkState->flags, J9_STACKWALK_NO_ERROR_REPORT)) {
-#if defined(J9VM_INTERP_STACKWALK_TRACING)
-		Assert_VRB_true(sp >= stackStart);
-		Assert_VRB_true(sp <= endOfStack);
-#else /* J9VM_INTERP_STACKWALK_TRACING */
-		Assert_VM_true(sp >= stackStart);
-		Assert_VM_true(sp <= endOfStack);
-#endif /* J9VM_INTERP_STACKWALK_TRACING */
-	}
-
-	framesPresent = (endOfStack - sp) / J9_STACKWALK_MIN_FRAME_SLOTS;
-
 	if (walkState->flags & J9_STACKWALK_CACHE_PCS) ++cacheElementSize;
 	if (walkState->flags & J9_STACKWALK_CACHE_CPS) ++cacheElementSize;
 	if (walkState->flags & J9_STACKWALK_CACHE_METHODS) ++cacheElementSize;
 
 	cacheSize = framesPresent * cacheElementSize;
-#ifdef J9VM_INTERP_NATIVE_SUPPORT
-	if (walkState->walkThread->javaVM->jitConfig) {
-		if (!(walkState->flags & J9_STACKWALK_SKIP_INLINES)) {
-			/* computations above assume 1 cacheElement per frame, in reality there may be (maxInlinedMethods + 1) elements per frame */
-			cacheSize *= (walkState->walkThread->javaVM->jitConfig->maxInlineDepth + 1);
-		}
-	}
-#endif
 
 	if ((walkState != walkState->walkThread->stackWalkState) || ((UDATA) (sp - stackStart) < cacheSize)
 #if defined (J9VM_INTERP_VERBOSE) || defined (J9VM_PROF_EVENT_REPORTING)
-		|| (walkState->walkThread->javaVM->runtimeFlags & J9_RUNTIME_PAINT_STACK)
+		|| (walkState->javaVM->runtimeFlags & J9_RUNTIME_PAINT_STACK)
 #endif
 	) {
 #ifdef J9VM_INTERP_STACKWALK_TRACING
@@ -637,10 +669,10 @@ static void walkDescribedPushes(J9StackWalkState * walkState, UDATA * highestSlo
 			char indexedTag[64];
 			PORT_ACCESS_FROM_WALKSTATE(walkState);
 			if (walkState->slotType == J9_STACKWALK_SLOT_TYPE_METHOD_LOCAL) {
-				j9str_printf(PORTLIB, indexedTag, 64, "%s-Slot: %s%d",
-						 (description & 1) ? "O" : "I", (walkState->slotIndex >= (IDATA)argCount) ? "t" : "a", walkState->slotIndex);
+				j9str_printf(indexedTag, 64, "%s-Slot: %s%d",
+						(description & 1) ? "O" : "I", (walkState->slotIndex >= (IDATA)argCount) ? "t" : "a", walkState->slotIndex);
 			} else {
-				j9str_printf(PORTLIB, indexedTag, 64, "%s-Slot: p%d", (description & 1) ? "O" : "I", walkState->slotIndex);
+				j9str_printf(indexedTag, 64, "%s-Slot: p%d", (description & 1) ? "O" : "I", walkState->slotIndex);
 			}
 
 			if (description & 1) {
@@ -671,6 +703,7 @@ static void
 walkMethodFrame(J9StackWalkState * walkState)
 {
 	J9SFMethodFrame * methodFrame = (J9SFMethodFrame *) ((U_8*) walkState->walkSP + (UDATA) walkState->literals);
+	BOOLEAN isJNINative = (J9SF_FRAME_TYPE_JNI_NATIVE_METHOD == (UDATA)walkState->pc);
 
 	walkState->bp = (UDATA *) &(methodFrame->savedA0);
 	walkState->frameFlags = methodFrame->specialFrameFlags;
@@ -704,7 +737,7 @@ walkMethodFrame(J9StackWalkState * walkState)
 #endif
 
 	if ((walkState->flags & J9_STACKWALK_ITERATE_O_SLOTS) && walkState->literals) {
-		if (walkState->frameFlags & J9_SSF_JNI_REFS_REDIRECTED) {
+		if ((walkState->frameFlags & J9_SSF_JNI_REFS_REDIRECTED) || isJNINative) {
 			walkPushedJNIRefs(walkState);
 		} else {
 			walkObjectPushes(walkState);
@@ -722,11 +755,12 @@ walkMethodFrame(J9StackWalkState * walkState)
 			if (walkState->argCount) {
 				/* Max size as argCount always <= 255 */
 				U_32 result[8];
+				J9Class *methodClass = UNTAGGED_METHOD_CP(walkState->method)->ramClass;
 
 #ifdef J9VM_INTERP_STACKWALK_TRACING
 				swPrintf(walkState, 4, "\tUsing signature mapper\n");
 #endif
-				j9localmap_ArgBitsForPC0(UNTAGGED_METHOD_CP(walkState->method)->ramClass->romClass, romMethod, result);
+				j9cached_ArgBitsForPC0(methodClass->romClass, romMethod, result, walkState->javaVM, methodClass->classLoader);
 
 #ifdef J9VM_INTERP_STACKWALK_TRACING
 				swPrintf(walkState, 4, "\tArguments starting at %p for %d slots\n", walkState->arg0EA, walkState->argCount);
@@ -843,7 +877,7 @@ static void walkGenericSpecialFrame(J9StackWalkState * walkState)
 void
 walkBytecodeFrameSlots(J9StackWalkState *walkState, J9Method *method, UDATA offsetPC, UDATA *pendingBase, UDATA pendingStackHeight, UDATA *localBase, UDATA numberOfLocals, UDATA alwaysLocalMap)
 {
-	J9JavaVM *vm = walkState->walkThread->javaVM;
+	J9JavaVM *vm = walkState->javaVM;
 	PORT_ACCESS_FROM_JAVAVM(vm);
 	UDATA *bp = localBase - numberOfLocals;
 	J9Class *ramClass = J9_CLASS_FROM_METHOD(method);
@@ -947,7 +981,7 @@ walkBytecodeFrame(J9StackWalkState * walkState)
 		}
 	} else {
 #if defined(J9VM_OPT_METHOD_HANDLE)
-		J9JavaVM *vm = walkState->walkThread->javaVM;
+		J9JavaVM *vm = walkState->javaVM;
 #endif /* defined(J9VM_OPT_METHOD_HANDLE) */
 		UDATA argTempCount = 0;
 		J9ROMMethod * romMethod = J9_ROM_METHOD_FROM_RAM_METHOD(walkState->method);
@@ -1039,8 +1073,8 @@ static void walkJNICallInFrame(J9StackWalkState * walkState)
 
 		if (walkState->walkSP != walkState->unwindSP) {
 #ifdef J9VM_INTERP_STACKWALK_TRACING
-			if (walkState->pc != (walkState->walkThread->javaVM->callInReturnPC + 3)) {
-				swPrintf(walkState, 0, "Error: PC should have been advanced in order to push return value, pc = %p, cipc = %p !!!\n", walkState->pc, walkState->walkThread->javaVM->callInReturnPC);
+			if (walkState->pc != (walkState->javaVM->callInReturnPC + 3)) {
+				swPrintf(walkState, 0, "Error: PC should have been advanced in order to push return value, pc = %p, cipc = %p !!!\n", walkState->pc, walkState->javaVM->callInReturnPC);
 			}
 #endif
 			if (walkState->frameFlags & J9_SSF_RETURNS_OBJECT) {
@@ -1225,7 +1259,7 @@ static void walkIndirectDescribedPushes(J9StackWalkState * walkState, UDATA * hi
 #if (defined(J9VM_INTERP_STACKWALK_TRACING)) 
 void swPrintf(J9StackWalkState * walkState, UDATA level, char * format, ...)
 {
-	if (walkState->walkThread->javaVM->stackWalkVerboseLevel >= level) {
+	if (walkState->javaVM->stackWalkVerboseLevel >= level) {
 		PORT_ACCESS_FROM_WALKSTATE(walkState);
 		va_list args;
 		char buf[1024];
@@ -1297,7 +1331,7 @@ static void initializeObjectSlotBitVector(J9StackWalkState * walkState)
 	walkState->objectSlotBitVector = NULL;
 	walkState->elsBitVector = 0;
 
-	if ((walkState->walkThread->javaVM->runtimeFlags & J9_RUNTIME_SNIFF_AND_WHACK) && !(walkState->flags & J9_STACKWALK_DO_NOT_SNIFF_AND_WHACK)) {
+	if ((walkState->javaVM->runtimeFlags & J9_RUNTIME_SNIFF_AND_WHACK) && !(walkState->flags & J9_STACKWALK_DO_NOT_SNIFF_AND_WHACK)) {
 		PORT_ACCESS_FROM_WALKSTATE(walkState);
 		UDATA slotCount = walkState->walkThread->stackObject->end - walkState->walkThread->sp;
 		UDATA byteCount = (slotCount + 7) >> 3;
@@ -1380,17 +1414,17 @@ static void sniffAndWhackPointer(J9StackWalkState * walkState, j9object_t *slotP
 	j9object_t object = *slotPointer;
 
 	if (object) {
-		J9JavaVM * vm = walkState->walkThread->javaVM;
+		J9JavaVM * vm = walkState->javaVM;
 		UDATA rc = vm->memoryManagerFunctions->j9gc_ext_check_is_valid_heap_object(vm, object, 0);
 
 		if ((rc == J9OBJECTCHECK_OBJECT) || (rc == J9OBJECTCHECK_FORWARDED)) {
 #ifdef J9VM_ENV_DATA64
-			((U_32 *) slotPointer)[1] = (U_32) 0xDEAD0000 + (U_16) (walkState->walkThread->javaVM->whackedPointerCounter);
+			((U_32 *) slotPointer)[1] = (U_32) 0xDEAD0000 + (U_16) (vm->whackedPointerCounter);
 #else
-			((U_32 *) slotPointer)[0] = (U_32) 0xDEAD0000 + (U_16) (walkState->walkThread->javaVM->whackedPointerCounter);
+			((U_32 *) slotPointer)[0] = (U_32) 0xDEAD0000 + (U_16) (vm->whackedPointerCounter);
 #endif
 			swPrintf(walkState, 1, "WHACKING I-Slot[%p] = %p --now--> %p\n", slotPointer, object, *slotPointer);
-			walkState->walkThread->javaVM->whackedPointerCounter += 2;
+			vm->whackedPointerCounter += 2;
 		}
 	}
 }
@@ -1488,8 +1522,7 @@ getLocalsMap(J9StackWalkState * walkState, J9ROMClass * romClass, J9ROMMethod *
 			/* j9localmap_ArgBitsForPC0 only deals with args, so zero out the result array to make sure the temps are non-object */
 
 			memset(result, 0, ((argTempCount + 31) / 32) * sizeof(U_32));
-
-			j9localmap_ArgBitsForPC0(romClass, romMethod, result);
+			j9cached_ArgBitsForPC0(romClass, romMethod, result, walkState->javaVM, UNTAGGED_METHOD_CP(walkState->method)->ramClass->classLoader);
 			return;
 		}
 	}
@@ -1497,7 +1530,7 @@ getLocalsMap(J9StackWalkState * walkState, J9ROMClass * romClass, J9ROMMethod *
 #ifdef J9VM_INTERP_STACKWALK_TRACING
 	swPrintf(walkState, 4, "\tUsing local mapper\n");
 #endif
-	errorCode = vm->localMapFunction(PORTLIB, romClass, romMethod, offsetPC, result, vm, j9mapmemory_GetBuffer, j9mapmemory_ReleaseBuffer);
+	errorCode = j9cached_LocalBitsForPC(romClass, romMethod, offsetPC, result, vm, j9mapmemory_GetBuffer, j9mapmemory_ReleaseBuffer, vm, J9_CLASS_FROM_METHOD(walkState->method)->classLoader);
 
 	if (errorCode < 0) {
 		if (J9_ARE_NO_BITS_SET(walkState->flags, J9_STACKWALK_NO_ERROR_REPORT)) {
@@ -1521,7 +1554,7 @@ getStackMap(J9StackWalkState * walkState, J9ROMClass * romClass, J9ROMMethod * r
 	PORT_ACCESS_FROM_WALKSTATE(walkState);
 	IDATA errorCode;
 
-	errorCode = j9stackmap_StackBitsForPC(PORTLIB, offsetPC, romClass, romMethod, result, pushCount, walkState->walkThread->javaVM, j9mapmemory_GetBuffer, j9mapmemory_ReleaseBuffer);
+	errorCode = j9cached_StackBitsForPC(offsetPC, romClass, romMethod, result, pushCount, walkState->javaVM, j9mapmemory_GetBuffer, j9mapmemory_ReleaseBuffer, walkState->javaVM, UNTAGGED_METHOD_CP(walkState->method)->ramClass->classLoader);
 	if (errorCode < 0) {
 		if (J9_ARE_NO_BITS_SET(walkState->flags, J9_STACKWALK_NO_ERROR_REPORT)) {
 			/* Local map failed, result = %p - aborting VM */
