@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1991, 2021 IBM Corp. and others
+ * Copyright IBM Corp. and others 1991
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,9 +15,9 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
  *******************************************************************************/
 
 /* #define J9VM_DBG */
@@ -80,7 +80,7 @@ static UDATA javaProtectedThreadProc (J9PortLibrary* portLibrary, void * entryar
 
 static UDATA    startJavaThreadInternal(J9VMThread * currentThread, UDATA privateFlags, UDATA osStackSize, UDATA priority, omrthread_entrypoint_t entryPoint, void * entryArg, UDATA setException);
 
-#if (defined(J9VM_DBG)) 
+#if (defined(J9VM_DBG))
 static void badness (char *description);
 #endif /* J9VM_DBG */
 
@@ -91,7 +91,7 @@ static void printCustomSpinOptions(void *element, void *userData);
 #endif /* J9VM_INTERP_CUSTOM_SPIN_OPTIONS */
 
 J9VMThread *
-allocateVMThread(J9JavaVM * vm, omrthread_t osThread, UDATA privateFlags, void * memorySpace, J9Object * threadObject)
+allocateVMThread(J9JavaVM *vm, omrthread_t osThread, UDATA privateFlags, void *memorySpace, J9Object *threadObject)
 {
 	PORT_ACCESS_FROM_PORT(vm->portLibrary);
 	J9JavaStack *stack = NULL;
@@ -160,7 +160,7 @@ allocateVMThread(J9JavaVM * vm, omrthread_t osThread, UDATA privateFlags, void *
 
 		/* Reuse a dead vmThread */
 		threadIsRecycled = TRUE;
-		
+
 		/* Grab the first dead thread (already reinitialized) */
 		J9_LINKED_LIST_REMOVE_FIRST(vm->deadThreadList, newThread);
 
@@ -191,6 +191,14 @@ allocateVMThread(J9JavaVM * vm, omrthread_t osThread, UDATA privateFlags, void *
 	newThread->threadObject = threadObject;
 	newThread->stackWalkState = &(newThread->inlineStackWalkState);
 	newThread->javaVM = vm;
+
+	newThread->contiguousIndexableHeaderSize = vm->contiguousIndexableHeaderSize;
+	newThread->discontiguousIndexableHeaderSize = vm->discontiguousIndexableHeaderSize;
+	newThread->unsafeIndexableHeaderSize = vm->unsafeIndexableHeaderSize;
+#if defined(J9VM_ENV_DATA64)
+	newThread->indexableObjectLayout = vm->indexableObjectLayout;
+#endif /* defined(J9VM_ENV_DATA64) */
+
 	newThread->privateFlags = privateFlags;
 	if (vm->extendedRuntimeFlags & J9_EXTENDED_RUNTIME_DEBUG_VM_ACCESS) {
 		setEventFlag(newThread, J9_PUBLIC_FLAGS_DEBUG_VM_ACCESS);
@@ -198,6 +206,22 @@ allocateVMThread(J9JavaVM * vm, omrthread_t osThread, UDATA privateFlags, void *
 	newThread->stackObject = stack;
 	newThread->stackOverflowMark = newThread->stackOverflowMark2 = J9JAVASTACK_STACKOVERFLOWMARK(stack);
 	newThread->osThread = osThread;
+#if defined(J9VM_OPT_CRIU_SUPPORT)
+	/* JDWP threads need to remain live while checkpoint/restore hooks run, so add
+	 * J9_PRIVATE_FLAGS2_DELAY_HALT_FOR_CHECKPOINT to the new J9VMThread created from a
+	 * JDWP java thread object.
+	 */
+	if (J9_ARE_ANY_BITS_SET(vm->checkpointState.flags, J9VM_CRIU_IS_JDWP_ENABLED)) {
+		for (UDATA i = 0; i < vm->checkpointState.javaDebugThreadCount; i++) {
+			j9object_t jdwpThreadObject = J9_JNI_UNWRAP_REFERENCE(vm->checkpointState.javaDebugThreads[i]);
+			if (jdwpThreadObject == threadObject) {
+				Trc_VM_criu_allocateVMThread_set_delayflag(i);
+				newThread->privateFlags2 |= J9_PRIVATE_FLAGS2_DELAY_HALT_FOR_CHECKPOINT;
+				break;
+			}
+		}
+	}
+#endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
 
 #ifdef J9VM_OPT_JAVA_OFFLOAD_SUPPORT
 	newThread->javaOffloadState = 0;
@@ -258,7 +282,24 @@ allocateVMThread(J9JavaVM * vm, omrthread_t osThread, UDATA privateFlags, void *
 #if JAVA_SPEC_VERSION >= 16
 	newThread->ffiArgs = NULL;
 	newThread->ffiArgCount = 0;
+	newThread->jmpBufEnvPtr = NULL;
 #endif /* JAVA_SPEC_VERSION >= 16 */
+#if JAVA_SPEC_VERSION >= 21
+	newThread->isInCriticalDownCall = FALSE;
+#endif /* JAVA_SPEC_VERSION >= 21 */
+
+#if JAVA_SPEC_VERSION >= 19
+	newThread->currentContinuation = NULL;
+	newThread->continuationPinCount = 0;
+	newThread->ownedMonitorCount = 0;
+	newThread->callOutCount = 0;
+	newThread->carrierThreadObject = threadObject;
+	newThread->scopedValueCache = NULL;
+#endif /* JAVA_SPEC_VERSION >= 19 */
+
+#if defined(J9VM_OPT_JFR)
+	newThread->threadJfrState.prevTimestamp = -1;
+#endif /* defined(J9VM_OPT_JFR) */
 
 	/* If an exclusive access request is in progress, mark this thread */
 
@@ -275,7 +316,12 @@ allocateVMThread(J9JavaVM * vm, omrthread_t osThread, UDATA privateFlags, void *
 	}
 #if defined(J9VM_OPT_CRIU_SUPPORT)
 	if (VM_CRIUHelpers::isJVMInSingleThreadMode(vm) && VM_VMHelpers::threadCanRunJavaCode(newThread)) {
-		setHaltFlag(newThread, J9_PUBLIC_FLAGS_HALT_THREAD_FOR_CHECKPOINT);
+		/* New threads with the delay halt flag should not be halted here. */
+		Trc_VM_criu_allocateVMThread_check_delayflag(newThread);
+		if (J9_ARE_NO_BITS_SET(newThread->privateFlags2, J9_PRIVATE_FLAGS2_DELAY_HALT_FOR_CHECKPOINT)) {
+			Trc_VM_criu_allocateVMThread_set_haltflag(newThread);
+			setHaltFlag(newThread, J9_PUBLIC_FLAGS_HALT_THREAD_FOR_CHECKPOINT);
+		}
 	}
 #endif /* defined(J9VM_OPT_CRIU_SUPPORT) */
 	omrthread_monitor_exit(vm->exclusiveAccessMutex);
@@ -301,6 +347,11 @@ allocateVMThread(J9JavaVM * vm, omrthread_t osThread, UDATA privateFlags, void *
 	/* Update counters for total # of threads and daemon threads and notify anyone waiting */
 
 	++(vm->totalThreadCount);
+	++(vm->accumulatedThreadCount);
+	if (vm->totalThreadCount > vm->peakThreadCount) {
+		vm->peakThreadCount = vm->totalThreadCount;
+	}
+
 	if (privateFlags & J9_PRIVATE_FLAGS_DAEMON_THREAD) {
 		++(vm->daemonThreadCount);
 	}
@@ -336,6 +387,9 @@ fail:
 			freeVMThread(vm, newThread);
 		}
 		newThread->threadObject = NULL;
+#if JAVA_SPEC_VERSION >= 19
+		newThread->carrierThreadObject = NULL;
+#endif /* JAVA_SPEC_VERSION >= 19 */
 		/* Detach the thread from OMR */
 		detachVMThreadFromOMR(vm, newThread);
 		if (!threadIsRecycled) {
@@ -354,12 +408,12 @@ IDATA J9THREAD_PROC javaThreadProc(void *entryarg)
 	J9VMThread* vmThread = currentVMThread(vm);
 	PORT_ACCESS_FROM_JAVAVM(vm);
 	UDATA result;
-	
+
 	vmThread->gpProtected = 1;
 
-	j9sig_protect(javaProtectedThreadProc, vmThread, 
+	j9sig_protect(javaProtectedThreadProc, vmThread,
 		structuredSignalHandler, vmThread,
-		J9PORT_SIG_FLAG_SIGALLSYNC | J9PORT_SIG_FLAG_MAY_CONTINUE_EXECUTION, 
+		J9PORT_SIG_FLAG_SIGALLSYNC | J9PORT_SIG_FLAG_MAY_CONTINUE_EXECUTION,
 		&result);
 
 	exitJavaThread(vm);
@@ -369,7 +423,7 @@ IDATA J9THREAD_PROC javaThreadProc(void *entryarg)
 
 
 
-#if (defined(J9VM_DBG)) 
+#if (defined(J9VM_DBG))
 static void badness(char *description)
 {
 	printf("\n<badness: %s>\n", description);
@@ -406,7 +460,7 @@ void threadCleanup(J9VMThread * vmThread, UDATA forkedByVM)
 		internalExceptionDescribe(vmThread);
 	}
 	releaseVMAccess(vmThread);
-	
+
 	/* Mark this thread as dead */
 	setEventFlag(vmThread, J9_PUBLIC_FLAGS_STOPPED);
 
@@ -430,11 +484,11 @@ void threadCleanup(J9VMThread * vmThread, UDATA forkedByVM)
 	omrthread_monitor_exit(vm->vmThreadListMutex);
 
 	/* Do the java dance to indicate thread death */
-
 	acquireVMAccess(vmThread);
 	cleanUpAttachedThread(vmThread);
 	releaseVMAccess(vmThread);
-	
+
+
 #if defined(OMR_GC_CONCURRENT_SCAVENGER) && defined(J9VM_ARCH_S390)
 	/* Concurrent scavenge enabled and JIT loaded implies running on supported h/w.
 	 * As such, per-thread deinitialization must occur
@@ -472,7 +526,7 @@ printCustomSpinOptions(void *element, void *userData)
 	j9tty_printf(PORTLIB, ",\n" LEADING_SPACE_EXTRA "customTryEnterSpin2=%zu", j9monitorOptions->thrMaxTryEnterSpins2BeforeBlocking);
 	j9tty_printf(PORTLIB, ",\n" LEADING_SPACE_EXTRA "customTryEnterYield=%zu", j9monitorOptions->thrMaxTryEnterYieldsBeforeBlocking);
 #if defined(OMR_THR_CUSTOM_SPIN_OPTIONS)
-#if defined(OMR_THR_THREE_TIER_LOCKING)	
+#if defined(OMR_THR_THREE_TIER_LOCKING)
 	j9tty_printf(PORTLIB, ",\n" LEADING_SPACE_EXTRA "customThreeTierSpinCount1=%zu", j9threadOptions->customThreeTierSpinCount1);
 	j9tty_printf(PORTLIB, ",\n" LEADING_SPACE_EXTRA "customThreeTierSpinCount2=%zu", j9threadOptions->customThreeTierSpinCount2);
 	j9tty_printf(PORTLIB, ",\n" LEADING_SPACE_EXTRA "customThreeTierSpinCount3=%zu", j9threadOptions->customThreeTierSpinCount3);
@@ -486,10 +540,10 @@ printCustomSpinOptions(void *element, void *userData)
 
 /**
  * Initializes VM thread options and parses suboptions of -Xthr:
- * 
- * Note that omrthread defaults are set at omrthread_init(), and not in 
+ *
+ * Note that omrthread defaults are set at omrthread_init(), and not in
  * this function.
- * 
+ *
  * @param[in] vm JavaVM.
  * @param[in] optArg Suboption string of format subopt[=val][,[subopt[=val]]...
  * @returns JNI error code
@@ -513,14 +567,30 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 #endif /* J9VM_INTERP_CUSTOM_SPIN_OPTIONS */
 	PORT_ACCESS_FROM_JAVAVM(vm);
 
-	cpus = j9sysinfo_get_number_CPUs_by_type(J9PORT_CPU_ONLINE);
+	cpus = j9sysinfo_get_number_CPUs_by_type(J9PORT_CPU_TARGET);
 
 	/* initialize defaults, first */
 	vm->thrMaxYieldsBeforeBlocking = 45;
 	vm->thrMaxTryEnterYieldsBeforeBlocking = 45;
 	vm->thrNestedSpinning = 1;
 	vm->thrTryEnterNestedSpinning = 1;
-	vm->thrDeflationPolicy = J9VM_DEFLATION_POLICY_ASAP;
+
+#if JAVA_SPEC_VERSION >= 24
+	/* Currently, there are timing holes between JVM_TakeVirtualThreadListToUnblock and monitor deflation.
+	 * A monitor can be deflated while it is being accessed in JVM_TakeVirtualThreadListToUnblock. This
+	 * leads to a NULL dereference causing a segfault. Adding more synchronization will cause a significant
+	 * overhead in the object monitor exit path. Until an efficient solution is developed, the policy to never
+	 * deflate will be employed in order to support Synchronize Virtual Threads without Pinning (JEP491).
+	 * Since the current JEP491 implementation always inflates monitors before usage, deflating will be
+	 * counter-productive.
+	 */
+	if (J9_ARE_ANY_BITS_SET(vm->extendedRuntimeFlags3, J9_EXTENDED_RUNTIME3_YIELD_PINNED_CONTINUATION)) {
+		vm->thrDeflationPolicy = J9VM_DEFLATION_POLICY_NEVER;
+	} else
+#endif /* JAVA_SPEC_VERSION >= 24 */
+	{
+		vm->thrDeflationPolicy = J9VM_DEFLATION_POLICY_ASAP;
+	}
 
 	if (cpus > 1) {
 #if (defined(LINUXPPC)) && !defined(J9VM_ENV_LITTLE_ENDIAN)
@@ -577,7 +647,7 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 	**(UDATA**)omrthread_global((char*)"yieldAlgorithm") = J9THREAD_LIB_YIELD_ALGORITHM_SCHED_YIELD;
 	**(UDATA**)omrthread_global((char*)"yieldUsleepMultiplier") = 1;
 #endif /* defined(OMR_THR_YIELD_ALG) */
-	
+
 #if defined(LINUX)
 	/* Check the sched_compat_yield setting for the versions of the Completely Fair Scheduler (CFS) which
 	 * have broken the thread_yield behavior. If running in CFS and sched_compat_yield=0, the CPU yielding
@@ -640,6 +710,10 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 #endif /* OMR_THR_ADAPTIVE_SPIN */
 
 #if defined(OMR_THR_THREE_TIER_LOCKING)
+	/* There is no advantage from spinning when running on a single processor. */
+	if (cpus <= 1) {
+		**(UDATA**)omrthread_global((char*)"defaultMonitorSpinCount1") = 1;
+	}
 	omrthread_lib_clear_flags(J9THREAD_LIB_FLAG_SECONDARY_SPIN_OBJECT_MONITORS_ENABLED | J9THREAD_LIB_FLAG_FAST_NOTIFY);
 #if defined(OMR_THR_SPIN_WAKE_CONTROL)
 	{
@@ -661,6 +735,15 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 #endif /* defined(OMR_THR_SPIN_WAKE_CONTROL) */
 #endif /* defined(OMR_THR_THREE_TIER_LOCKING) */
 
+
+#if defined(OMR_THR_YIELD_ALG)
+	**(UDATA **)omrthread_global((char *)"parkPolicy") = 0;
+	**(UDATA **)omrthread_global((char *)"parkSleepMultiplier") = 0;
+	**(UDATA **)omrthread_global((char *)"parkSleepTime") = 0;
+	**(UDATA **)omrthread_global((char *)"parkSpinCount") = 0;
+	**(UDATA **)omrthread_global((char *)"parkSleepCount") = 0;
+#endif /* defined(OMR_THR_YIELD_ALG) */
+
 	/* parse arguments */
 	if (optArg == NULL) {
 		return JNI_OK;
@@ -736,22 +819,22 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 		}
 
 		if (try_scan(&scan_start, "nestedSpinning")) {
-			vm->thrNestedSpinning = 1;			
+			vm->thrNestedSpinning = 1;
 			continue;
 		}
 
 		if (try_scan(&scan_start, "noNestedSpinning")) {
-			vm->thrNestedSpinning = 0;			
+			vm->thrNestedSpinning = 0;
 			continue;
 		}
 
 		if (try_scan(&scan_start, "tryEnterNestedSpinning")) {
-			vm->thrTryEnterNestedSpinning = 1;			
+			vm->thrTryEnterNestedSpinning = 1;
 			continue;
 		}
 
 		if (try_scan(&scan_start, "noTryEnterNestedSpinning")) {
-			vm->thrTryEnterNestedSpinning = 0;			
+			vm->thrTryEnterNestedSpinning = 0;
 			continue;
 		}
 
@@ -874,7 +957,7 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 #ifdef OMR_THR_JLM_HOLD_TIMES
 		if (try_scan(&scan_start, "clockSkewHi=")) {
 				/* upper 32 bits of 64 bit clock */
-				UDATA clockSkewHi; 
+				UDATA clockSkewHi;
 				if (scan_hex(&scan_start, &clockSkewHi)) {
 					goto _error;
 				}
@@ -888,7 +971,7 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 			char *policy = scan_to_delim(PORTLIB, &scan_start, ',');
 			if (NULL != policy) {
 				if (0 == strcmp(policy, "never")) {
-					vm->thrDeflationPolicy = J9VM_DEFLATION_POLICY_NEVER;						
+					vm->thrDeflationPolicy = J9VM_DEFLATION_POLICY_NEVER;
 				} else if (0 == strcmp(policy, "asap")) {
 					vm->thrDeflationPolicy = J9VM_DEFLATION_POLICY_ASAP;
 				}
@@ -899,7 +982,7 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 #endif
 				else {
 					/* restore for better error message */
-					scan_start = oldScanStart; 
+					scan_start = oldScanStart;
 					j9mem_free_memory(policy);
 					goto _error;
 				}
@@ -1181,7 +1264,7 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 #endif /* defined(OMR_THR_YIELD_ALG) */
 
 
-#if defined(J9ZOS390) 
+#if defined(J9ZOS390)
 		if (try_scan(&scan_start, "tw=")) {
 			char *old_scan_start = scan_start; /* for error handling */
 			char *tw = scan_to_delim(PORTLIB, &scan_start, ',');
@@ -1190,17 +1273,17 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 				tw = "heavy";
 			} else if (tw && (!strcmp(tw, "MEDIUM") || !strcmp(tw, "NATIVE") || !strcmp(tw, "native") || !strcmp(tw, ""))) {
 				tw = "medium";
-			}			
-			*gtw = (UDATA)tw; 
+			}
+			*gtw = (UDATA)tw;
 			if (tw && !strcmp(tw, "heavy")) {
-				continue; 
+				continue;
 			}
 			if (tw && !strcmp(tw, "medium")) {
-				continue; 
+				continue;
 			}
-			*gtw = 0; 
+			*gtw = 0;
 			scan_start = old_scan_start;
-			goto _error; 
+			goto _error;
 		}
 #endif
 
@@ -1211,7 +1294,7 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 			continue;
 		}
 #endif
-		
+
 		if (try_scan(&scan_start, "destroyMutexOnMonitorFree")) {
 			omrthread_lib_set_flags(J9THREAD_LIB_FLAG_DESTROY_MUTEX_ON_MONITOR_FREE);
 			continue;
@@ -1222,6 +1305,53 @@ threadParseArguments(J9JavaVM *vm, char *optArg)
 			continue;
 		}
 
+#if defined(OMR_THR_YIELD_ALG)
+		if (try_scan(&scan_start, "parkPolicy=")) {
+			UDATA parkPolicy = 0;
+			if (scan_udata(&scan_start, &parkPolicy)) {
+				goto _error;
+			}
+			**(UDATA **)omrthread_global((char *)"parkPolicy") = parkPolicy;
+			continue;
+		}
+
+		if (try_scan(&scan_start, "parkSleepMultiplier=")) {
+			UDATA parkSleepMultiplier = 0;
+			if (scan_udata(&scan_start, &parkSleepMultiplier)) {
+				goto _error;
+			}
+			**(UDATA **)omrthread_global((char *)"parkSleepMultiplier") = parkSleepMultiplier;
+			continue;
+		}
+
+		if (try_scan(&scan_start, "parkSleepTime=")) {
+			UDATA parkSleepTime = 0;
+			if (scan_udata(&scan_start, &parkSleepTime)) {
+				goto _error;
+			}
+			**(UDATA **)omrthread_global((char *)"parkSleepTime") = parkSleepTime;
+			continue;
+		}
+
+		if (try_scan(&scan_start, "parkSpinCount=")) {
+			UDATA parkSpinCount = 0;
+			if (scan_udata(&scan_start, &parkSpinCount)) {
+				goto _error;
+			}
+			**(UDATA **)omrthread_global((char *)"parkSpinCount") = parkSpinCount;
+			continue;
+		}
+
+		if (try_scan(&scan_start, "parkSleepCount=")) {
+			UDATA parkSleepCount = 0;
+			if (scan_udata(&scan_start, &parkSleepCount)) {
+				goto _error;
+			}
+			**(UDATA **)omrthread_global((char *)"parkSleepCount") = parkSleepCount;
+			continue;
+		}
+#endif /* defined(OMR_THR_YIELD_ALG) */
+
 		/* Couldn't find a match for arguments */
 _error:
 		j9nls_printf(PORTLIB, J9NLS_ERROR, J9NLS_VM_UNRECOGNIZED_XTHR_OPTION, scan_start);
@@ -1301,7 +1431,7 @@ dumpThreadingInfo(J9JavaVM* jvm)
 		envbuf = (char*)j9mem_allocate_memory(envbuflen + 1, OMRMEM_CATEGORY_VM);
 		if (envbuf != NULL) {
 			if (j9sysinfo_get_env("_EDC_PTHREAD_YIELD", envbuf, envbuflen) != 0) {
-				envbuf[0] = '\0';				
+				envbuf[0] = '\0';
 			} else {
 				envbuf[envbuflen] = '\0';
 			}
@@ -1323,7 +1453,7 @@ dumpThreadingInfo(J9JavaVM* jvm)
 	j9tty_printf(PORTLIB, LEADING_SPACE "tryEnterYield=%zu,\n", jvm->thrMaxTryEnterYieldsBeforeBlocking);
 	j9tty_printf(PORTLIB, LEADING_SPACE "%sestedSpinning,\n", (jvm->thrNestedSpinning) ? "n" : "noN");
 	j9tty_printf(PORTLIB, LEADING_SPACE "%sryEnterNestedSpinning,\n", (jvm->thrTryEnterNestedSpinning) ? "t" : "noT");
-	j9tty_printf(PORTLIB, LEADING_SPACE "%sestroyMutexOnMonitorFree,\n", 
+	j9tty_printf(PORTLIB, LEADING_SPACE "%sestroyMutexOnMonitorFree,\n",
 		J9_ARE_ALL_BITS_SET(omrthread_lib_get_flags(), J9THREAD_LIB_FLAG_DESTROY_MUTEX_ON_MONITOR_FREE) ? "d" : "noD");
 #if !defined(WIN32) && defined(OMR_NOTIFY_POLICY_CONTROL)
 	j9tty_printf(PORTLIB, LEADING_SPACE "notifyPolicy=%s,\n",
@@ -1355,7 +1485,7 @@ dumpThreadingInfo(J9JavaVM* jvm)
 		}
 	}
 #endif
-	
+
 	if (J9_ARE_ALL_BITS_SET(omrthread_lib_get_flags(), J9THREAD_LIB_FLAG_NO_SCHEDULING)) {
 		j9tty_printf(PORTLIB, ",\n");
 		j9tty_printf(PORTLIB, LEADING_SPACE "noThreadScheduling");
@@ -1365,7 +1495,7 @@ dumpThreadingInfo(J9JavaVM* jvm)
 	j9tty_printf(PORTLIB, ",\n" LEADING_SPACE "%secondarySpinForObjectMonitors",
 		J9_ARE_ALL_BITS_SET(omrthread_lib_get_flags(), J9THREAD_LIB_FLAG_SECONDARY_SPIN_OBJECT_MONITORS_ENABLED) ? "s" : "noS");
 #endif /* defined(OMR_THR_THREE_TIER_LOCKING) */
-	
+
 #ifdef OMR_THR_ADAPTIVE_SPIN
 	if ((0 != *(UDATA*)omrthread_global((char*)"adaptSpinHoldtimeEnable")) || (0 != *(UDATA*)omrthread_global((char*)"adaptSpinSlowPercentEnable"))) {
 		j9tty_printf(PORTLIB, ",\n" LEADING_SPACE "adaptSpin");
@@ -1392,7 +1522,7 @@ dumpThreadingInfo(J9JavaVM* jvm)
 
 
 
-J9JavaStack * 
+J9JavaStack *
 allocateJavaStack(J9JavaVM * vm, UDATA stackSize, J9JavaStack * previousStack)
 {
 	PORT_ACCESS_FROM_JAVAVM(vm);
@@ -1424,9 +1554,9 @@ allocateJavaStack(J9JavaVM * vm, UDATA stackSize, J9JavaStack * previousStack)
 			end += sizeof(UDATA);
 		}
 #if 0
-		j9tty_printf(PORTLIB, "Allocated stack ending at 16r%p (stagger = %d, alignment = %d)\n", 
-			end, 
-			stagger, 
+		j9tty_printf(PORTLIB, "Allocated stack ending at 16r%p (stagger = %d, alignment = %d)\n",
+			end,
+			stagger,
 			vm->thrStaggerMax ? end % vm->thrStaggerMax : 0);
 #endif
 
@@ -1435,6 +1565,9 @@ allocateJavaStack(J9JavaVM * vm, UDATA stackSize, J9JavaStack * previousStack)
 		stack->previous = previousStack;
 		stack->firstReferenceFrame = 0;
 
+#if JAVA_SPEC_VERSION >= 19
+		stack->isVirtual = FALSE;
+#endif /* JAVA_SPEC_VERSION >= 19 */
 
 		/* If this is a profiling VM, or verbose:stack is enabled, paint the stack with a distinctive pattern so we can
 			determine how much has been used */
@@ -1442,7 +1575,7 @@ allocateJavaStack(J9JavaVM * vm, UDATA stackSize, J9JavaStack * previousStack)
 #if defined (J9VM_INTERP_VERBOSE) || defined (J9VM_PROF_EVENT_REPORTING)
 		if (vm->runtimeFlags & J9_RUNTIME_PAINT_STACK) {
 			UDATA * currentSlot = (UDATA *) (stack + 1);
-	
+
 			while (currentSlot != stack->end)
 				*currentSlot++ = J9_RUNTIME_STACK_FILL;
 		}
@@ -1453,13 +1586,13 @@ allocateJavaStack(J9JavaVM * vm, UDATA stackSize, J9JavaStack * previousStack)
 }
 
 
-void 
+void
 freeJavaStack(J9JavaVM * vm, J9JavaStack * stack)
 {
 	PORT_ACCESS_FROM_JAVAVM(vm);
 
 	if (J9JAVAVM_COMPRESS_OBJECT_REFERENCES(vm)) {
-		j9mem_free_memory32(stack);			
+		j9mem_free_memory32(stack);
 	} else {
 		j9mem_free_memory(stack);
 	}
@@ -1487,12 +1620,12 @@ static UDATA printMethodInfo(J9VMThread *currentThread , J9StackWalkState *state
 	char *cursor = buf;
 	char *end = buf + sizeof(buf);
 	PORT_ACCESS_FROM_VMC(currentThread);
-	
-	cursor += j9str_printf(PORTLIB, cursor, end - cursor, "\tat %.*s.%.*s%.*s", J9UTF8_LENGTH(className), J9UTF8_DATA(className), J9UTF8_LENGTH(methodName), J9UTF8_DATA(methodName), J9UTF8_LENGTH(sig), J9UTF8_DATA(sig));
+
+	cursor += j9str_printf(cursor, end - cursor, "\tat %.*s.%.*s%.*s", J9UTF8_LENGTH(className), J9UTF8_DATA(className), J9UTF8_LENGTH(methodName), J9UTF8_DATA(methodName), J9UTF8_LENGTH(sig), J9UTF8_DATA(sig));
 
 	if (romMethod->modifiers & J9AccNative) {
 	/*increment cursor here by the return of j9str_printf if it needs to be used further*/
-		j9str_printf(PORTLIB, cursor, end - cursor, " (Native Method)");
+		j9str_printf(cursor, end - cursor, " (Native Method)");
 	} else {
 		UDATA offsetPC = state->bytecodePCOffset;
 #ifdef J9VM_OPT_DEBUG_INFO_SERVER
@@ -1501,25 +1634,25 @@ static UDATA printMethodInfo(J9VMThread *currentThread , J9StackWalkState *state
 		if (sourceFile) {
 			IDATA lineNumber = getLineNumberForROMClass(vm, method, offsetPC);
 
-			cursor += j9str_printf(PORTLIB, cursor, end - cursor, " (%.*s", J9UTF8_LENGTH(sourceFile), J9UTF8_DATA(sourceFile));
+			cursor += j9str_printf(cursor, end - cursor, " (%.*s", J9UTF8_LENGTH(sourceFile), J9UTF8_DATA(sourceFile));
 			if (lineNumber != -1) {
-				cursor += j9str_printf(PORTLIB, cursor, end - cursor, ":%zu", lineNumber);
+				cursor += j9str_printf(cursor, end - cursor, ":%zu", lineNumber);
 			}
-			cursor += j9str_printf(PORTLIB, cursor, end - cursor, ")");
+			cursor += j9str_printf(cursor, end - cursor, ")");
 		} else
 #endif
 		{
-			cursor += j9str_printf(PORTLIB, cursor, end - cursor, " (Bytecode PC: %zu)", offsetPC);
+			cursor += j9str_printf(cursor, end - cursor, " (Bytecode PC: %zu)", offsetPC);
 		}
 
 #ifdef J9VM_INTERP_NATIVE_SUPPORT
 		if (state->jitInfo != NULL) {
 		/*increment cursor here by the return of j9str_printf if it needs to be used further*/
-			j9str_printf(PORTLIB, cursor, end - cursor, " (Compiled Code)");
+			j9str_printf(cursor, end - cursor, " (Compiled Code)");
 		}
 #endif
 	}
-	
+
 	trace_printf(PORTLIB, tracefd, (char*)"%s\n", buf);
 
 	return J9_STACKWALK_KEEP_ITERATING;
@@ -1539,7 +1672,7 @@ void printThreadInfo(J9JavaVM *vm, J9VMThread *self, char *toFile, BOOLEAN allTh
 
 	if ( !vm->mainThread ) {
 		/* No main thread, so not much we can do here */
-		j9tty_err_printf(PORTLIB, "Thread info not available\n");
+		j9tty_err_printf("Thread info not available\n");
 		return;
 	}
 
@@ -1567,13 +1700,13 @@ void printThreadInfo(J9JavaVM *vm, J9VMThread *self, char *toFile, BOOLEAN allTh
 	if (toFile != NULL) {
 		strcpy(fileName, toFile);
 		if ((tracefd = j9file_open(fileName, EsOpenWrite | EsOpenCreate | EsOpenTruncate, 0666))==-1) {
-			j9tty_err_printf(PORTLIB, "Error: Failed to open dump file %s.\nWill output to stderr instead:\n", fileName);
+			j9tty_err_printf("Error: Failed to open dump file %s.\nWill output to stderr instead:\n", fileName);
 		}
 	} else if (vm->sigquitToFileDir != NULL) {
-		j9str_printf(PORTLIB, fileName, EsMaxPath, "%s%s%s%d%s", 
-							vm->sigquitToFileDir, DIR_SEPARATOR_STR, SIGQUIT_FILE_NAME, j9time_usec_clock(), SIGQUIT_FILE_EXT);
+		j9str_printf(fileName, EsMaxPath, "%s%s%s%d%s",
+				vm->sigquitToFileDir, DIR_SEPARATOR_STR, SIGQUIT_FILE_NAME, j9time_usec_clock(), SIGQUIT_FILE_EXT);
 		if ((tracefd = j9file_open(fileName, EsOpenWrite | EsOpenCreate | EsOpenTruncate, 0666))==-1) {
-			j9tty_err_printf(PORTLIB, "Error: Failed to open trace file %s.\nWill output to stderr instead:\n", fileName);
+			j9tty_err_printf("Error: Failed to open trace file %s.\nWill output to stderr instead:\n", fileName);
 		}
 	}
 
@@ -1601,7 +1734,7 @@ void printThreadInfo(J9JavaVM *vm, J9VMThread *self, char *toFile, BOOLEAN allTh
 
 				vm->walkStackFrames(firstThread, &walkState);
 			}
-			
+
 			if (allThreads) {
 				trace_printf(PORTLIB, tracefd, (char*)"\n");
 				currentThread = currentThread->linkNext;
@@ -1611,7 +1744,7 @@ void printThreadInfo(J9JavaVM *vm, J9VMThread *self, char *toFile, BOOLEAN allTh
 
 	if (tracefd != -1) {
 		j9file_close(tracefd);
-		j9tty_err_printf(PORTLIB, "Thread info written to: %s\n", fileName);
+		j9tty_err_printf("Thread info written to: %s\n", fileName);
 	}
 
 	if(exclusiveRequestedLocally) {
@@ -1671,11 +1804,11 @@ printJ9ThreadStatusMonitorInfo(J9VMThread *vmThread, IDATA tracefd)
 	const char* name;
 
 	(void)getVMThreadRawState(vmThread, NULL, (omrthread_monitor_t *)&blockingMonitor, &owner, &count);
-	
+
 	if (blockingMonitor == NULL) {
 		return;
 	}
-	
+
 	if ( (((J9ThreadAbstractMonitor*)blockingMonitor)->flags & J9THREAD_MONITOR_OBJECT) == J9THREAD_MONITOR_OBJECT ) {
 		j9object_t object = (j9object_t)((J9ThreadAbstractMonitor*)blockingMonitor)->userData;
 		J9ROMClass *romClass;
@@ -1726,11 +1859,11 @@ static void trace_printf(struct J9PortLibrary *portLib, IDATA tracefd, char * fo
 	va_start(args, format);
 	j9str_vprintf(buffer, sizeof(buffer), format, args);
 	va_end(args);
-	
+
 	if (tracefd != -1)
 		wroteToFile = (j9file_write_text(tracefd, buffer, strlen(buffer)) != -1);
 	if (!wroteToFile)
-		j9tty_err_printf(PORTLIB, buffer);
+		j9tty_err_printf(buffer);
 }
 
 j9object_t
@@ -1772,7 +1905,7 @@ createCachedOutOfMemoryError(J9VMThread * currentThread, j9object_t threadObject
 }
 
 
-UDATA   
+UDATA
 startJavaThread(J9VMThread * currentThread, j9object_t threadObject, UDATA privateFlags,
 	UDATA osStackSize, UDATA priority, omrthread_entrypoint_t entryPoint, void * entryArg, j9object_t schedulingParameters)
 {
@@ -1792,7 +1925,7 @@ startJavaThread(J9VMThread * currentThread, j9object_t threadObject, UDATA priva
 	 * Oracle class libraries don't have the 'started' field */
 	J9VMJAVALANGTHREAD_SET_STARTED(currentThread, threadObject, TRUE);
 #endif /* !J9VM_IVE_RAW_BUILD */
-	
+
 	/* Save objects on the stack in case we GC */
 
 	PUSH_OBJECT_IN_SPECIAL_FRAME(currentThread, (j9object_t) threadObject);
@@ -1846,14 +1979,14 @@ popNone:
 }
 
 /* On entry the stack is:
- * 
+ *
  * 0: lock
  * 1: OOM
  * 2: schedulingParameters
  * 3: threadObject
  */
 
-static UDATA   
+static UDATA
 startJavaThreadInternal(J9VMThread * currentThread, UDATA privateFlags, UDATA osStackSize, UDATA priority,
 		omrthread_entrypoint_t entryPoint, void * entryArg, UDATA setException)
 {
@@ -1865,7 +1998,7 @@ startJavaThreadInternal(J9VMThread * currentThread, UDATA privateFlags, UDATA os
 	j9object_t cachedOutOfMemoryError;
 	j9object_t threadObject;
 	char *threadName = NULL;
-	
+
 	/* Fork the OS thread */
 
 	retVal = vm->internalVMFunctions->createThreadWithCategory(
@@ -1878,12 +2011,12 @@ startJavaThreadInternal(J9VMThread * currentThread, UDATA privateFlags, UDATA os
 				J9THREAD_CATEGORY_APPLICATION_THREAD);
 	if (retVal != J9THREAD_SUCCESS) {
 		if (retVal & J9THREAD_ERR_OS_ERRNO_SET) {
-#if defined(J9ZOS390)		
+#if defined(J9ZOS390)
 			omrthread_os_errno_t os_errno = omrthread_get_os_errno();
 			omrthread_os_errno_t os_errno2 = omrthread_get_os_errno2();
 			Trc_VM_startJavaThread_failedToCreateOSThreadWithErrno2(currentThread, -retVal,
 				omrthread_get_errordesc(-retVal), os_errno, os_errno, os_errno2, os_errno2);
-			if (setException) {	
+			if (setException) {
 				if (0 == setFailedToForkThreadException(currentThread, -retVal, os_errno, os_errno2)) {
 					return J9_THREAD_START_THROW_CURRENT_EXCEPTION;
 				}
@@ -1899,7 +2032,7 @@ startJavaThreadInternal(J9VMThread * currentThread, UDATA privateFlags, UDATA os
 			}
 #endif /* !J9ZOS390 */
 		} else {
-			Trc_VM_startJavaThread_failedToCreateOSThread(currentThread, -retVal, 
+			Trc_VM_startJavaThread_failedToCreateOSThread(currentThread, -retVal,
 				omrthread_get_errordesc(-retVal));
 		}
 		return J9_THREAD_START_FAILED_TO_FORK_THREAD;
@@ -1921,14 +2054,14 @@ startJavaThreadInternal(J9VMThread * currentThread, UDATA privateFlags, UDATA os
 		Trc_VM_startJavaThread_failedVMThreadAlloc(currentThread);
 		omrthread_cancel(osThread);
 		return J9_THREAD_START_FAILED_VMTHREAD_ALLOC;
-	} 
-		
+	}
+
 	/* Create the vmThread */
 
 	newThread = allocateVMThread(vm, osThread, privateFlags, currentThread->omrVMThread->memorySpace, threadObject);
 	if (newThread == NULL) {
 		PORT_ACCESS_FROM_PORT(vm->portLibrary);
-		
+
 		Trc_VM_startJavaThread_failedVMThreadAlloc(currentThread);
 		omrthread_cancel(osThread);
 		j9mem_free_memory(threadName);
@@ -1964,6 +2097,8 @@ startJavaThreadInternal(J9VMThread * currentThread, UDATA privateFlags, UDATA os
 
 	omrthread_resume(osThread);
 
+	TRIGGER_J9HOOK_VM_THREAD_STARTING(vm->hookInterface, currentThread, newThread);
+
 	return J9_THREAD_START_NO_ERROR;
 }
 
@@ -1992,12 +2127,12 @@ setFailedToForkThreadException(J9VMThread *currentThread, IDATA retVal, omrthrea
 			J9NLS_VM_THREAD_CREATE_FAILED_WITH_32BIT_ERRNO2, NULL);
 
 	if (errorMessage) {
-		bufLen = j9str_printf(PORTLIB, NULL, 0, errorMessage, retVal, os_errno, os_errno, (U_32)(UDATA)os_errno2);
+		bufLen = j9str_printf(NULL, 0, errorMessage, retVal, os_errno, os_errno, (U_32)(UDATA)os_errno2);
 		if (bufLen > 0) {
 			buf = (char*)j9mem_allocate_memory(bufLen, OMRMEM_CATEGORY_VM);
 			if (buf) {
 				/* j9str_printf return value doesn't include the NUL terminator */
-				if ((bufLen - 1) == j9str_printf(PORTLIB, buf, bufLen, errorMessage, retVal, os_errno, os_errno, os_errno2, os_errno2)) {
+				if ((bufLen - 1) == j9str_printf(buf, bufLen, errorMessage, retVal, os_errno, os_errno, os_errno2, os_errno2)) {
 					setCurrentExceptionUTF(currentThread, J9_EX_OOM_THREAD | J9VMCONSTANTPOOL_JAVALANGOUTOFMEMORYERROR, buf);
 					rc = 0;
 				}
@@ -2031,12 +2166,12 @@ setFailedToForkThreadException(J9VMThread *currentThread, IDATA retVal, omrthrea
 		J9NLS_VM_THREAD_CREATE_FAILED_WITH_ERRNO, NULL);
 
 	if (errorMessage) {
-		bufLen = j9str_printf(PORTLIB, NULL, 0, errorMessage, retVal, os_errno);
+		bufLen = j9str_printf(NULL, 0, errorMessage, retVal, os_errno);
 		if (bufLen > 0) {
 			buf = (char*)j9mem_allocate_memory(bufLen, OMRMEM_CATEGORY_VM);
 			if (buf) {
 				/* j9str_printf return value doesn't include the NUL terminator */
-				if ((bufLen - 1) == j9str_printf(PORTLIB, buf, bufLen, errorMessage, retVal, os_errno)) {
+				if ((bufLen - 1) == j9str_printf(buf, bufLen, errorMessage, retVal, os_errno)) {
 					setCurrentExceptionUTF(currentThread, J9_EX_OOM_THREAD | J9VMCONSTANTPOOL_JAVALANGOUTOFMEMORYERROR, buf);
 					rc = 0;
 				}
@@ -2056,7 +2191,6 @@ setFailedToForkThreadException(J9VMThread *currentThread, IDATA retVal, omrthrea
 static UDATA
 javaProtectedThreadProc(J9PortLibrary* portLibrary, void * entryarg)
 {
-	
 	J9VMThread * vmThread = (J9VMThread *) entryarg;
 	J9JavaVM *vm = vmThread->javaVM;
 
@@ -2064,26 +2198,20 @@ javaProtectedThreadProc(J9PortLibrary* portLibrary, void * entryarg)
 
 #if defined(LINUX)
 	omrthread_set_name(vmThread->osThread, (char*)vmThread->omrVMThread->threadName);
-#endif
+#endif /* defined(LINUX) */
 
 	threadAboutToStart(vmThread);
 
 	TRIGGER_J9HOOK_VM_THREAD_STARTED(vm->hookInterface, vmThread, vmThread);
 
-
 	acquireVMAccess(vmThread);
-#ifdef J9VM_OPT_DEPRECATED_METHODS
-	if (!(J9VMJAVALANGTHREAD_STOPCALLED(vmThread,vmThread->threadObject))){
-#endif
-
-		{
-			/* Start running the thread */
-			runJavaThread(vmThread);
-		}
-
-#ifdef J9VM_OPT_DEPRECATED_METHODS
+#if defined(J9VM_OPT_DEPRECATED_METHODS) && (JAVA_SPEC_VERSION < 20)
+	if (!(J9VMJAVALANGTHREAD_STOPCALLED(vmThread, vmThread->threadObject)))
+#endif /* defined(J9VM_OPT_DEPRECATED_METHODS) && (JAVA_SPEC_VERSION < 20) */
+	{
+		/* Start running the thread. */
+		runJavaThread(vmThread);
 	}
-#endif	
 	releaseVMAccess(vmThread);
 
 	/* Perform thread cleanup */
@@ -2106,13 +2234,13 @@ typedef struct ThreadMap {
 } ThreadMap;
 
 static UDATA
-tmapHash(void *key, void *userData) 
+tmapHash(void *key, void *userData)
 {
 	return (UDATA)((ThreadMap*)key)->thread;
 }
 
 static UDATA
-tmapHashEqual(void *leftKey, void *rightKey, void *userData) 
+tmapHashEqual(void *leftKey, void *rightKey, void *userData)
 {
 	return ((ThreadMap*)leftKey)->thread == ((ThreadMap*)rightKey)->thread;
 }
@@ -2120,14 +2248,14 @@ tmapHashEqual(void *leftKey, void *rightKey, void *userData)
 /*
  * This function finds java threads that are deadlocked on object monitors.
  * The caller must have VM access.
- * The caller is responsible for freeing the memory allocated for 
+ * The caller is responsible for freeing the memory allocated for
  * deadlockedThreads and blockingObjects (using portlib j9mem_free_memory()).
  * deadlockedThreads, blockingObjects are not changed if an error occurs.
- * 
+ *
  * [in] currentThread		Must be non-NULL
  * [out]deadlockedThreads	Where to store array of returned vmThreads.
  * 								If NULL, this info is not returned.
- * [out]blockingObjects		Where to store the corresponding monitor that each deadlockedThread is blocked on. 
+ * [out]blockingObjects		Where to store the corresponding monitor that each deadlockedThread is blocked on.
  * 								If NULL, this info is not returned.
  * [in] flags				See J9VMTHREAD_FINDDEADLOCKFLAG_xxx in vm_api.h
  * Returns:
@@ -2170,7 +2298,12 @@ findObjectDeadlockedThreads(J9VMThread *currentThread,
 	threadCount = 0;
 	vmThread = currentThread;
 	do {
-		if (vmThread->threadObject != NULL) {
+		if ((NULL != vmThread->threadObject)
+#if JAVA_SPEC_VERSION >= 19
+		/* Exclude J9VMThreads if a continuation is mounted to match the RI. */
+		&& (NULL == vmThread->currentContinuation)
+#endif /* JAVA_SPEC_VERSION >= 19 */
+		) {
 			++threadCount;
 		}
 	} while ((vmThread = vmThread->linkNext) != currentThread);
@@ -2186,7 +2319,7 @@ findObjectDeadlockedThreads(J9VMThread *currentThread,
 		goto findDeadlock_fail;
 	}
 
-	/* 
+	/*
 	 * Use a hash-table to map J9VMThreads to their index in the thrChain array
 	 * (this allows us to walk the thrChain array in monitor-ownership chain order)
 	 */
@@ -2203,12 +2336,17 @@ findObjectDeadlockedThreads(J9VMThread *currentThread,
 	threadCount = 0;
 	vmThread = currentThread;
 	do {
-		if (vmThread->threadObject != NULL) {
+		if ((NULL != vmThread->threadObject)
+#if JAVA_SPEC_VERSION >= 19
+		/* Exclude J9VMThreads if a continuation is mounted to match the RI. */
+		&& (NULL == vmThread->currentContinuation)
+#endif /* JAVA_SPEC_VERSION >= 19 */
+		) {
 			UDATA state;
 			J9VMThread *owner;
 			j9object_t monitorObject;
 			BOOLEAN isBlocked = FALSE;
-			
+
 			state = getVMThreadObjectState(vmThread, &monitorObject, &owner, NULL);
 			switch (state) {
 			case J9VMTHREAD_STATE_BLOCKED:
@@ -2225,18 +2363,18 @@ findObjectDeadlockedThreads(J9VMThread *currentThread,
 				break;
 			case J9VMTHREAD_STATE_PARKED:
 				if (flags & J9VMTHREAD_FINDDEADLOCKFLAG_INCLUDESYNCHRONIZERS) {
-					if (monitorObject && owner) {		
+					if (monitorObject && owner) {
 						isBlocked = TRUE;
-					}	
+					}
 				}
 				break;
 			default:
-				isBlocked = FALSE;	
+				isBlocked = FALSE;
 			}
-			
+
 			if (TRUE == isBlocked) {
 				ThreadMap tmap;
-				
+
 				thrChain[threadCount].thread = vmThread;
 				thrChain[threadCount].threadobj = (j9object_t)vmThread->threadObject;
 				thrChain[threadCount].object = monitorObject;
@@ -2250,7 +2388,7 @@ findObjectDeadlockedThreads(J9VMThread *currentThread,
 			}
 		}
 	} while ((vmThread = vmThread->linkNext) != currentThread);
-	
+
 	if (!(flags & J9VMTHREAD_FINDDEADLOCKFLAG_ALREADYHAVEEXCLUSIVE)) {
 		releaseExclusiveVMAccess(currentThread);
 	}
@@ -2259,7 +2397,7 @@ findObjectDeadlockedThreads(J9VMThread *currentThread,
 	for (i = 0; i < threadCount; ++i) {
 		ThreadMap tmap;
 		ThreadMap *p_tmap;
-		
+
 		/* Fake-up a ThreadMap for query purposes only */
 		tmap.thread = thrChain[i].owner;
 		p_tmap = (ThreadMap *)hashTableFind(hashTable, &(tmap));
@@ -2299,7 +2437,7 @@ findObjectDeadlockedThreads(J9VMThread *currentThread,
 			}
 			thrChain[index].visit = visit;
 			index = thrChain[index].owner_index;
-		} while (index != -1); 
+		} while (index != -1);
 	}
 
 	if (deadCount <= 0) {
@@ -2350,7 +2488,7 @@ findDeadlock_done:
 		}
 	}
 	return deadCount;
-	
+
 findDeadlock_fail:
 	if (!(flags & J9VMTHREAD_FINDDEADLOCKFLAG_ALREADYHAVEEXCLUSIVE)) {
 		releaseExclusiveVMAccess(currentThread);
@@ -2361,31 +2499,41 @@ findDeadlock_fail:
 /*
  * This function returns the Java priority for the thread specified.  For RealtimeThreads
  * the priority comes from the priority field in the PriorityParameters object for the thread, while
- * for regular Java threads or any thread in a non-realtime vm the priority comes from the priority field 
- * in the Java Thread object 
- * 
+ * for regular Java threads or any thread in a non-realtime vm the priority comes from the priority field
+ * in the Java Thread object
+ *
  * Callers of this method must ensure that the Java Thread object for the thread is not NULL and
  * that this cannot change while the call is being made
- * 
- * This method should also only be called when we don't need barrier checks such as call from 
+ *
+ * This method should also only be called when we don't need barrier checks such as call from
  * jvmti and ras
- * 
+ *
  * @param vm the vm to be used to do the lookup
  * @param thread the thread for which to get the priority
- * 
+ *
  * @retval the Java priority for the thread
  */
 UDATA
 getJavaThreadPriority(struct J9JavaVM *vm, J9VMThread* thread )
 {
+#if JAVA_SPEC_VERSION >= 19
+	j9object_t threadHolder = J9VMJAVALANGTHREAD_HOLDER(thread, thread->threadObject);
+	UDATA priority = 0;
+	if (NULL != threadHolder) {
+		priority = J9VMJAVALANGTHREADFIELDHOLDER_PRIORITY(thread, threadHolder);
+	}
+
+	return priority;
+#else /* JAVA_SPEC_VERSION >= 19 */
 	return J9VMJAVALANGTHREAD_PRIORITY(thread, thread->threadObject);
+#endif /* JAVA_SPEC_VERSION >= 19 */
 }
 
 
 /**
  * Perform thread setup before any java code is run on the thread.
  * Triggers J9HOOK_THREAD_ABOUT_TO_START.
- * 
+ *
  * @param currentThread the current J9VMThread
  */
 void
