@@ -1,5 +1,5 @@
-/*******************************************************************************
- * Copyright (c) 2001, 2018 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2001
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,10 +15,10 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package com.ibm.j9.uma.configuration;
 
 import java.io.BufferedReader;
@@ -27,8 +27,6 @@ import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStreamReader;
-import java.util.Calendar;
-import java.util.GregorianCalendar;
 import java.util.Hashtable;
 import java.util.List;
 import java.util.Map;
@@ -308,14 +306,8 @@ public class ConfigurationImpl implements IConfiguration, ISinglePredicateEvalua
 		return platform;
 	}
 
-	String rawCopyrightNotice;
-
 	public String getRawCopyrightNotice() throws UMAException {
-		if (rawCopyrightNotice == null) {
-			GregorianCalendar calendar = new GregorianCalendar();
-			rawCopyrightNotice = "Copyright (c) 2000, " + calendar.get(Calendar.YEAR) + " IBM Corp. and others";
-		}
-		return rawCopyrightNotice;
+		return "Copyright IBM Corp. and others 2000";
 	}
 
 	String makefileCopyrightNotice;
@@ -348,9 +340,9 @@ public class ConfigurationImpl implements IConfiguration, ISinglePredicateEvalua
 				"# OpenJDK Assembly Exception [2].\n" +
 				"#\n" +
 				"# [1] https://www.gnu.org/software/classpath/license.html\n" +
-				"# [2] http://openjdk.java.net/legal/assembly-exception.html\n" +
+				"# [2] https://openjdk.org/legal/assembly-exception.html\n" +
 				"#\n" +
-				"# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception\n" +
+				"# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0\n" +
 				"\n" +
 				"# File generated in stream: " + majorVersion + "." + minorVersion + "\n" +
 				"#\n" +
@@ -380,9 +372,9 @@ public class ConfigurationImpl implements IConfiguration, ISinglePredicateEvalua
 				" * OpenJDK Assembly Exception [2].\n" +
 				" *\n" +
 				" * [1] https://www.gnu.org/software/classpath/license.html\n" +
-				" * [2] http://openjdk.java.net/legal/assembly-exception.html\n" +
+				" * [2] https://openjdk.org/legal/assembly-exception.html\n" +
 				" *\n" +
-				" * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception\n" +
+				" * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0\n" +
 				" */\n";
 		}
 		return cCopyrightNotice;
