@@ -1,5 +1,5 @@
-/*******************************************************************************
- * Copyright (c) 2004, 2019 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2004
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,10 +15,10 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package com.ibm.oti.NLSTool;
 
 import java.io.BufferedReader;
@@ -776,7 +776,7 @@ public class J9NLS implements NLSConstants {
 		public void writeHeader(String headerName, String moduleName) {
 			String header = headerName.replace('.', '_');
 			append("/*\n");
-			append(" * Copyright (c) 1991, 2017 IBM Corp. and others\n");
+			append(" * Copyright IBM Corp. and others 1991\n");
 			append(" *\n");
 			append(" * This program and the accompanying materials are made available under\n");
 			append(" * the terms of the Eclipse Public License 2.0 which accompanies this\n");
@@ -792,9 +792,9 @@ public class J9NLS implements NLSConstants {
 			append(" * OpenJDK Assembly Exception [2].\n");
 			append(" *\n");
 			append(" * [1] https://www.gnu.org/software/classpath/license.html\n");
-			append(" * [2] http://openjdk.java.net/legal/assembly-exception.html\n");
+			append(" * [2] https://openjdk.org/legal/assembly-exception.html\n");
 			append(" *\n");
-			append(" * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception\n");
+			append(" * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0\n");
 			append(" */\n\n");
 			append("#ifndef " + header + "\n");
 			append("#define " + header + "\n\n");
