@@ -1,5 +1,5 @@
-/*******************************************************************************
- * Copyright (c) 2005, 2021 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2005
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,10 +15,10 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 
 package org.openj9.test.java.lang.management;
 
@@ -114,7 +114,6 @@ public class TestGarbageCollectorMXBean {
 	public final void testGetLastGcInfo() {
 		AssertJUnit.assertTrue(gcb instanceof com.sun.management.GarbageCollectorMXBean);
 		System.gc();
-		System.gc();
 		AssertJUnit.assertTrue(((com.sun.management.GarbageCollectorMXBean)gcb).getLastGcInfo() != null);
 	}
 
