@@ -1,5 +1,5 @@
-/*******************************************************************************
- * Copyright (c) 2005, 2020 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2005
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,10 +15,10 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package org.openj9.test.java.lang.management;
 
 import org.testng.annotations.AfterClass;
@@ -189,20 +189,22 @@ public class TestOperatingSystemMXBean {
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "SystemLoadAverage") instanceof Double);
 
 			// The good IBM attributes ...
-			AssertJUnit.assertTrue(mbs.getAttribute(objName, "TotalPhysicalMemory") instanceof Long);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "ProcessingCapacity") instanceof Integer);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "ProcessCpuTime") instanceof Long);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "FreePhysicalMemorySize") instanceof Long);
-			AssertJUnit.assertTrue(mbs.getAttribute(objName, "ProcessVirtualMemorySize") instanceof Long);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "ProcessPrivateMemorySize") instanceof Long);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "ProcessPhysicalMemorySize") instanceof Long);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "TotalPhysicalMemorySize") instanceof Long);
-			AssertJUnit.assertTrue(mbs.getAttribute(objName, "ProcessCpuTimeByNS") instanceof Long);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "SystemCpuLoad") instanceof Double);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "CommittedVirtualMemorySize") instanceof Long);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "TotalSwapSpaceSize") instanceof Long);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "FreeSwapSpaceSize") instanceof Long);
 			AssertJUnit.assertTrue(mbs.getAttribute(objName, "ProcessCpuLoad") instanceof Double);
+			if (verMajor < 19) {
+				AssertJUnit.assertTrue(mbs.getAttribute(objName, "TotalPhysicalMemory") instanceof Long);
+				AssertJUnit.assertTrue(mbs.getAttribute(objName, "ProcessCpuTimeByNS") instanceof Long);
+				AssertJUnit.assertTrue(mbs.getAttribute(objName, "ProcessVirtualMemorySize") instanceof Long);
+			}
 			if (verMajor >= 14) {
 				AssertJUnit.assertTrue(mbs.getAttribute(objName, "CpuLoad") instanceof Double);
 				AssertJUnit.assertTrue(mbs.getAttribute(objName, "FreeMemorySize") instanceof Long);
@@ -289,14 +291,6 @@ public class TestOperatingSystemMXBean {
 		}
 	}
 
-	/**
-	 * IBM attribute
-	 */
-	@Test
-	public void testGetTotalPhysicalMemory() {
-		AssertJUnit.assertTrue(osb.getTotalPhysicalMemory() > 0);
-	}
-
 	/**
 	 * IBM attribute
 	 */
@@ -467,7 +461,7 @@ public class TestOperatingSystemMXBean {
 				} else if (name.equals("ProcessCpuLoad")) {
 					AssertJUnit.assertTrue(value instanceof Double);
 				} else if (name.equals("HardwareModel")) {
-					AssertJUnit.assertTrue(value instanceof String);
+					AssertJUnit.assertTrue((value == null) || (value instanceof String));
 				} else if (name.equals("HardwareEmulated")) {
 					AssertJUnit.assertTrue(value instanceof Boolean);
 				} else if (name.equals("OpenFileDescriptorCount")) {
@@ -580,6 +574,10 @@ public class TestOperatingSystemMXBean {
 			} else {
 				attrNbr = 25;
 			}
+			if (verMajor >= 19) {
+				// deprecated APIs are removed in 19
+				attrNbr -= 3;
+			}
 		} else {
 			// Java 8 - 13
 			if (TestUtil.isRunningOnUnix()) {
