@@ -1,5 +1,5 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2020 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 1998
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,10 +15,10 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package org.openj9.test.java.lang;
 
 import org.openj9.test.util.VersionCheck;
@@ -932,8 +932,23 @@ public class Test_String {
 	 */
 	@Test
 	public void test_indexOf3() {
-		AssertJUnit.assertTrue("Failed to find string", hw1.indexOf("World") > 0);
-		AssertJUnit.assertTrue("Failed to find string", !(hw1.indexOf("ZZ") > 0));
+		AssertJUnit.assertEquals("Failed to find string 1", 5, hw1.indexOf("World"));
+		AssertJUnit.assertEquals("Failed to find string 2", -1, hw1.indexOf("ZZ"));
+		String needle = "\u00b0\u00b1";
+		String hay = new StringBuilder("a").append(needle).toString();
+		AssertJUnit.assertEquals("Failed to find string 3", 1, hay.indexOf(needle));
+
+		String multi = "\u0100:abc";
+		String[] splits = multi.split(":");
+		AssertJUnit.assertEquals("Failed to find string 4", 3, "123abc".indexOf(splits[1]));
+		String sub = multi.substring(2);
+		AssertJUnit.assertEquals("Failed to find string 5", 3, "123abc".indexOf(sub));
+		String r1 = multi.replace('\u0100', '1');
+		AssertJUnit.assertEquals("Failed to find string 6", 0, "1:abc".indexOf(r1));
+		String r2 = multi.replaceAll("\u0100", "1");
+		AssertJUnit.assertEquals("Failed to find string 7", 0, "1:abc".indexOf(r2));
+		String r3 = multi.replaceAll("\u0100", "");
+		AssertJUnit.assertEquals("Failed to find string 8", 0, ":abc".indexOf(r3));
 	}
 
 	/**
@@ -1007,6 +1022,11 @@ public class Test_String {
 	public void test_lastIndexOf3() {
 		AssertJUnit.assertTrue("Returned incorrect index", hw1.lastIndexOf("World") == 5);
 		AssertJUnit.assertTrue("Found String outside of index", hw1.lastIndexOf("HeKKKKKKKK") == -1);
+
+		/* test https://github.com/eclipse-openj9/openj9/issues/19273 */
+		String s1 = "a";
+		String s2 = "b";
+		AssertJUnit.assertTrue("Incorrect index of \\u0000", (s1 + "\u0000" + s2).lastIndexOf("\u0000") == 1);
 	}
 
 	/**
@@ -1566,28 +1586,41 @@ public class Test_String {
 	@Test
 	public void test_offsetByCodePoints() {
 		String s1 = "A\ud800\udc00C";
-		AssertJUnit.assertTrue("wrong offset 0, 0", s1.offsetByCodePoints(0, 0) == 0);
-		AssertJUnit.assertTrue("wrong offset 0, 1", s1.offsetByCodePoints(0, 1) == 1);
-		AssertJUnit.assertTrue("wrong offset 0, 2", s1.offsetByCodePoints(0, 2) == 3);
-		AssertJUnit.assertTrue("wrong offset 0, 3", s1.offsetByCodePoints(0, 3) == 4);
-		AssertJUnit.assertTrue("wrong offset 1, 0", s1.offsetByCodePoints(1, 0) == 1);
-		AssertJUnit.assertTrue("wrong offset 1, 1", s1.offsetByCodePoints(1, 1) == 3);
-		AssertJUnit.assertTrue("wrong offset 1, 2", s1.offsetByCodePoints(1, 2) == 4);
-		AssertJUnit.assertTrue("wrong offset 2, 0", s1.offsetByCodePoints(2, 0) == 2);
-		AssertJUnit.assertTrue("wrong offset 2, 1", s1.offsetByCodePoints(2, 1) == 3);
-		AssertJUnit.assertTrue("wrong offset 2, 2", s1.offsetByCodePoints(2, 2) == 4);
-		AssertJUnit.assertTrue("wrong offset 3, 0", s1.offsetByCodePoints(3, 0) == 3);
-		AssertJUnit.assertTrue("wrong offset 3, 1", s1.offsetByCodePoints(3, 1) == 4);
-		AssertJUnit.assertTrue("wrong offset 4, 0", s1.offsetByCodePoints(4, 0) == 4);
+		AssertJUnit.assertEquals("wrong offset 0, 0", 0, s1.offsetByCodePoints(0, 0));
+		AssertJUnit.assertEquals("wrong offset 0, 1", 1, s1.offsetByCodePoints(0, 1));
+		AssertJUnit.assertEquals("wrong offset 0, 2", 3, s1.offsetByCodePoints(0, 2));
+		AssertJUnit.assertEquals("wrong offset 0, 3", 4, s1.offsetByCodePoints(0, 3));
+		AssertJUnit.assertEquals("wrong offset 1, 0", 1, s1.offsetByCodePoints(1, 0));
+		AssertJUnit.assertEquals("wrong offset 1, 1", 3, s1.offsetByCodePoints(1, 1));
+		AssertJUnit.assertEquals("wrong offset 1, 2", 4, s1.offsetByCodePoints(1, 2));
+		AssertJUnit.assertEquals("wrong offset 2, 0", 2, s1.offsetByCodePoints(2, 0));
+		AssertJUnit.assertEquals("wrong offset 2, 1", 3, s1.offsetByCodePoints(2, 1));
+		AssertJUnit.assertEquals("wrong offset 2, 2", 4, s1.offsetByCodePoints(2, 2));
+		AssertJUnit.assertEquals("wrong offset 3, 0", 3, s1.offsetByCodePoints(3, 0));
+		AssertJUnit.assertEquals("wrong offset 3, 1", 4, s1.offsetByCodePoints(3, 1));
+		AssertJUnit.assertEquals("wrong offset 4, 0", 4, s1.offsetByCodePoints(4, 0));
 
-		AssertJUnit.assertTrue("wrong offset 4, -1", s1.offsetByCodePoints(4, -1) == 3);
-		AssertJUnit.assertTrue("wrong offset 4, -2", s1.offsetByCodePoints(4, -2) == 1);
-		AssertJUnit.assertTrue("wrong offset 4, -3", s1.offsetByCodePoints(4, -3) == 0);
-		AssertJUnit.assertTrue("wrong offset 3, -1", s1.offsetByCodePoints(3, -1) == 1);
-		AssertJUnit.assertTrue("wrong offset 3, -2", s1.offsetByCodePoints(3, -2) == 0);
-		AssertJUnit.assertTrue("wrong offset 2, -1", s1.offsetByCodePoints(2, -1) == 1);
-		AssertJUnit.assertTrue("wrong offset 2, -2", s1.offsetByCodePoints(2, -2) == 0);
-		AssertJUnit.assertTrue("wrong offset 1, -1", s1.offsetByCodePoints(1, -1) == 0);
+		AssertJUnit.assertEquals("wrong offset 4, -1", 3, s1.offsetByCodePoints(4, -1));
+		AssertJUnit.assertEquals("wrong offset 4, -2", 1, s1.offsetByCodePoints(4, -2));
+		AssertJUnit.assertEquals("wrong offset 4, -3", 0, s1.offsetByCodePoints(4, -3));
+		AssertJUnit.assertEquals("wrong offset 3, -1", 1, s1.offsetByCodePoints(3, -1));
+		AssertJUnit.assertEquals("wrong offset 3, -2", 0, s1.offsetByCodePoints(3, -2));
+		AssertJUnit.assertEquals("wrong offset 2, -1", 1, s1.offsetByCodePoints(2, -1));
+		AssertJUnit.assertEquals("wrong offset 2, -2", 0, s1.offsetByCodePoints(2, -2));
+		AssertJUnit.assertEquals("wrong offset 1, -1", 0, s1.offsetByCodePoints(1, -1));
+
+		try {
+			s1.offsetByCodePoints(s1.length(), 1);
+			AssertJUnit.fail("returned index greater than length should fail");
+		} catch (IndexOutOfBoundsException e) {
+			// correct exception thrown
+		}
+		try {
+			s1.offsetByCodePoints(0, -1);
+			AssertJUnit.fail("returned index less than zero should fail");
+		} catch (IndexOutOfBoundsException e) {
+			// correct exception thrown
+		}
 	}
 
 	/**
@@ -1827,6 +1860,19 @@ public class Test_String {
 		}
 	}
 
+	/**
+	 * @tests java.lang.String#replaceAll(String, String)
+	 */
+	@Test
+	public void test_replaceAll() {
+		String replace1 = "1a2a3a\u0011";
+		String result1 = replace1.replaceAll("\u1161", "[");
+		AssertJUnit.assertSame("replaceAll() compact result should be identical", replace1, result1);
+		String replace2 = "1a2b3c\u1161";
+		String result2 = replace2.replaceAll("\u1162", "\u1234");
+		AssertJUnit.assertSame("replaceAll() non-compact result should be identical", replace2, result2);
+	}
+
 	/**
 	 * @tests java.lang.String#replaceAll(String, String)
 	 */
@@ -1839,6 +1885,7 @@ public class Test_String {
 			// expected
 		}
 	}
+
 	/**
 	 * @tests java.lang.String#replaceAll(String, String)
 	 */
