@@ -1,5 +1,5 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2022 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 1998
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,17 +15,18 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package org.openj9.test.java.lang;
 
+import java.lang.ref.WeakReference;
+import org.openj9.test.util.VersionCheck;
 import org.testng.annotations.AfterMethod;
 import org.testng.annotations.Test;
 import org.testng.Assert;
 import org.testng.AssertJUnit;
-import java.lang.ref.WeakReference;
 
 @Test(groups = { "level.sanity" })
 public class Test_Thread {
@@ -33,6 +34,7 @@ public class Test_Thread {
 	static class SimpleThread implements Runnable {
 		int delay;
 
+		@Override
 		public void run() {
 			try {
 				synchronized (this) {
@@ -42,18 +44,17 @@ public class Test_Thread {
 			} catch (InterruptedException e) {
 				return;
 			}
-
 		}
 
 		public SimpleThread(int d) {
-			if (d >= 0)
-				delay = d;
+			delay = Math.max(d, 0);
 		}
 	}
 
 	static class YieldThread implements Runnable {
 		volatile int delay;
 
+		@Override
 		public void run() {
 			int x = 0;
 			while (true) {
@@ -62,8 +63,7 @@ public class Test_Thread {
 		}
 
 		public YieldThread(int d) {
-			if (d >= 0)
-				delay = d;
+			delay = Math.max(d, 0);
 		}
 	}
 
@@ -71,6 +71,7 @@ public class Test_Thread {
 		Thread parent;
 		volatile int checkVal = -1;
 
+		@Override
 		public void run() {
 			try {
 				synchronized (this) {
@@ -194,20 +195,20 @@ public class Test_Thread {
 		tg.destroy();
 
 		Runnable r = new Runnable() {
+			@Override
 			public void run() {
 			}
 		};
 
-		ThreadGroup foo = null;
+		ThreadGroup foo = new ThreadGroup("foo");
 		try {
-			new Thread(foo = new ThreadGroup("foo"), r, null);
+			new Thread(foo, r, null);
 			// Should not get here
 			AssertJUnit.assertTrue("Null cannot be accepted as Thread name", false);
 		} catch (NullPointerException npe) {
-			AssertJUnit.assertTrue("Null cannot be accepted as Thread name", true);
+			// expected: null cannot be accepted as thread name
 			foo.destroy();
 		}
-
 	}
 
 	/**
@@ -235,31 +236,6 @@ public class Test_Thread {
 		}
 	}
 
-	/**
-	 * @tests java.lang.Thread#countStackFrames()
-	 */
-	@Test
-	public void test_countStackFrames() {
-		int versionMajor = org.openj9.test.util.VersionCheck.major();
-		if (versionMajor < 13) {
-			AssertJUnit.assertTrue("Test failed.", Thread.currentThread().countStackFrames() == 0);
-		} else if (versionMajor < 14) {
-			try {
-				Thread.currentThread().countStackFrames();
-				Assert.fail("Should thrown IllegalThreadStateException!");
-			} catch (IllegalThreadStateException itse) {
-				// pass with expected exception
-			}
-		} else {
-			try {
-				Thread.currentThread().countStackFrames();
-				Assert.fail("Should thrown UnsupportedOperationException!");
-			} catch (UnsupportedOperationException uoe) {
-				// pass with expected exception
-			}
-		}
-	}
-
 	/**
 	 * @tests java.lang.Thread#currentThread()
 	 */
@@ -287,6 +263,7 @@ public class Test_Thread {
 				return enumerateCount;
 			}
 
+			@Override
 			public synchronized void run() {
 				try {
 					this.notify();
@@ -421,6 +398,7 @@ public class Test_Thread {
 		/* [PR CMVC 88976] the Thread is alive until cleanup() is called */
 		final Object lock = new Object();
 		Thread t = new Thread() {
+			@Override
 			public void run() {
 				synchronized (lock) {
 					lock.notifyAll();
@@ -451,6 +429,7 @@ public class Test_Thread {
 			Thread parent;
 			boolean sync;
 
+			@Override
 			public void run() {
 				if (sync) {
 					synchronized (lock) {
@@ -552,11 +531,14 @@ public class Test_Thread {
 		class SpinThread implements Runnable {
 			public volatile boolean done = false;
 
+			@Override
 			public void run() {
-				while (!Thread.currentThread().isInterrupted())
-					;
-				while (!done)
-					;
+				while (!Thread.currentThread().isInterrupted()) {
+					/* wait to be interrupted */
+				}
+				while (!done) {
+					/* wait to be told we're done */
+				}
 			}
 		}
 
@@ -583,12 +565,8 @@ public class Test_Thread {
 		SimpleThread simple;
 		try {
 			st = new Thread(simple = new SimpleThread(100));
-			AssertJUnit.assertTrue("Thread is alive", !st.isAlive()); // cause isAlive() to
-			// be compiled by
-			// the jit, as it
-			// must be called
-			// within 100ms
-			// below
+			AssertJUnit.assertTrue("Thread is alive", !st.isAlive()); // cause isAlive()
+			// to be compiled by the jit, as it must be called within 100ms below
 			synchronized (simple) {
 				st.start();
 				simple.wait();
@@ -615,12 +593,8 @@ public class Test_Thread {
 		SimpleThread simple;
 		try {
 			st = new Thread(simple = new SimpleThread(1000), "SimpleThread12");
-			AssertJUnit.assertTrue("Thread is alive", !st.isAlive()); // cause isAlive() to
-			// be compiled by
-			// the jit, as it
-			// must be called
-			// within 100ms
-			// below
+			AssertJUnit.assertTrue("Thread is alive", !st.isAlive()); // cause isAlive()
+			// to be compiled by the jit, as it must be called within 100ms below
 			synchronized (simple) {
 				st.start();
 				simple.wait();
@@ -648,6 +622,7 @@ public class Test_Thread {
 		final Object lock = new Object();
 		final Thread main = Thread.currentThread();
 		Thread killer = new Thread(new Runnable() {
+			@Override
 			public void run() {
 				try {
 					synchronized (lock) {
@@ -680,9 +655,9 @@ public class Test_Thread {
 	 */
 	@Test
 	public void test_join3() {
-		SimpleThread simple;
 		try {
-			Thread t = new Thread(simple = new SimpleThread(3000), "Squawk1");
+			SimpleThread simple = new SimpleThread(60 * 1000);
+			Thread t = new Thread(simple, "Squawk1");
 			synchronized (simple) {
 				t.start();
 				simple.wait();
@@ -690,8 +665,9 @@ public class Test_Thread {
 			long firstRead = System.currentTimeMillis();
 			t.join(100, 999999);
 			long secondRead = System.currentTimeMillis();
-			AssertJUnit.assertTrue("Did not join by appropriate time: " + secondRead + "-" + firstRead + "="
-					+ (secondRead - firstRead), secondRead - firstRead >= 100);
+			long delta = secondRead - firstRead;
+			AssertJUnit.assertTrue("Did not join by appropriate time: " + secondRead + "-" + firstRead + "=" + delta,
+					delta >= 100);
 			AssertJUnit.assertTrue("Joined thread is not alive", t.isAlive());
 			t.interrupt();
 		} catch (Exception e) {
@@ -701,6 +677,7 @@ public class Test_Thread {
 		final Object lock = new Object();
 		final Thread main = Thread.currentThread();
 		Thread killer = new Thread(new Runnable() {
+			@Override
 			public void run() {
 				try {
 					synchronized (lock) {
@@ -733,29 +710,38 @@ public class Test_Thread {
 	 */
 	@Test
 	public void test_resume() {
-		int orgval;
-		ResSupThread t;
-		try {
-			t = new ResSupThread(Thread.currentThread());
-			synchronized (t) {
-				ct = new Thread(t, "Interupt Test2");
-				ct.start();
-				t.wait();
+		if (VersionCheck.major() < 20) {
+			int orgval;
+			ResSupThread t;
+			try {
+				t = new ResSupThread(Thread.currentThread());
+				synchronized (t) {
+					ct = new Thread(t, "Interupt Test2");
+					ct.start();
+					t.wait();
+				}
+				ct.suspend();
+				// Wait to be sure the suspend has occurred
+				Thread.sleep(500);
+				orgval = t.getCheckVal();
+				// Wait to be sure the thread is suspended
+				Thread.sleep(500);
+				AssertJUnit.assertTrue("Failed to suspend thread", orgval == t.getCheckVal());
+				ct.resume();
+				// Wait to be sure the resume has occurred.
+				Thread.sleep(500);
+				AssertJUnit.assertTrue("Failed to resume thread", orgval != t.getCheckVal());
+				ct.interrupt();
+			} catch (InterruptedException e) {
+				Assert.fail("Unexpected interrupt occurred", e);
+			}
+		} else {
+			try {
+				Thread.currentThread().resume();
+				Assert.fail("expected UnsupportedOperationException");
+			} catch (UnsupportedOperationException e) {
+				// expected
 			}
-			ct.suspend();
-			// Wait to be sure the suspend has occurred
-			Thread.sleep(500);
-			orgval = t.getCheckVal();
-			// Wait to be sure the thread is suspended
-			Thread.sleep(500);
-			AssertJUnit.assertTrue("Failed to suspend thread", orgval == t.getCheckVal());
-			ct.resume();
-			// Wait to be sure the resume has occurred.
-			Thread.sleep(500);
-			AssertJUnit.assertTrue("Failed to resume thread", orgval != t.getCheckVal());
-			ct.interrupt();
-		} catch (InterruptedException e) {
-			Assert.fail("Unexpected interrupt occurred", e);
 		}
 	}
 
@@ -767,6 +753,7 @@ public class Test_Thread {
 		class RunThread implements Runnable {
 			boolean didThreadRun = false;
 
+			@Override
 			public void run() {
 				didThreadRun = true;
 			}
@@ -783,7 +770,7 @@ public class Test_Thread {
 			AssertJUnit.assertTrue("Thread did not run", rt.didThreadRun);
 			t.join();
 		} catch (InterruptedException e) {
-			AssertJUnit.assertTrue("Joined thread was interrupted", true);
+			// expected: joined thread was interrupted
 		}
 		AssertJUnit.assertTrue("Joined thread is still alive", !t.isAlive());
 	}
@@ -811,8 +798,7 @@ public class Test_Thread {
 			st.setName(null);
 			AssertJUnit.assertTrue("Null should not be accepted as a valid name", false);
 		} catch (NullPointerException e) {
-			// success
-			AssertJUnit.assertTrue("Null should not be accepted as a valid name", true);
+			// expected: null should not be accepted as a valid name
 		}
 		st.start();
 	}
@@ -863,8 +849,9 @@ public class Test_Thread {
 			/* [PR CMVC 133063] timing related test failure */
 			for (int i = 0; i < 5; i++) {
 				Thread.sleep(150);
-				if (orgval != t.getCheckVal())
+				if (orgval != t.getCheckVal()) {
 					break;
+				}
 			}
 			AssertJUnit.assertTrue("Thread is not running2", orgval != t.getCheckVal());
 			ct.interrupt();
@@ -897,7 +884,7 @@ public class Test_Thread {
 				} catch (IllegalThreadStateException e) {
 					/* IllegalThreadStateException is expected
 					 *
-					 *  This thread's group should only have this thread as child since it is alive and still running.
+					 * This thread's group should only have this thread as child since it is alive and still running.
 					 */
 					if (myGroup.activeCount() != 1) {
 						testResult = "This running thread's thread group does not have one child which should be this thread";
@@ -931,13 +918,14 @@ public class Test_Thread {
 	public void test_start_WeakReference() {
 		/* [PR CVMC 118827] references are not removed in dead threads */
 		Object o = new Object();
-		final WeakReference ref = new WeakReference(o);
+		final WeakReference<Object> ref = new WeakReference<>(o);
 		Thread t = new Thread(new Runnable() {
 			// If Thread.runnable isn't cleared, "save" holds a reference
 			Object save;
 			ThreadLocal tl = new ThreadLocal();
 			InheritableThreadLocal itl = new InheritableThreadLocal();
 
+			@Override
 			public void run() {
 				save = ref.get();
 				tl.set(save);
@@ -961,32 +949,40 @@ public class Test_Thread {
 	 */
 	@Test
 	public void test_stop() {
-		try {
-			Runnable r = new ResSupThread(null);
-			synchronized (r) {
-				st = new Thread(r, "Interupt Test5");
-				st.start();
-				r.wait();
+		if (VersionCheck.major() < 20) {
+			try {
+				Runnable r = new ResSupThread(null);
+				synchronized (r) {
+					st = new Thread(r, "Interupt Test5");
+					st.start();
+					r.wait();
+				}
+			} catch (InterruptedException e) {
+				Assert.fail("Unexpected interrupt received", e);
 			}
+			st.stop();
 
-		} catch (InterruptedException e) {
-			Assert.fail("Unexpected interrupt received", e);
+			/*
+			 * Thread.stop() implementation is changed from 1.1.7 to the 1.2
+			 * behaviour. now stop() doesn't ensure immediate thread death, so we
+			 * have to wait till st thread joins back to current thread, before
+			 * making an isAlive() check.
+			 */
+			try {
+				st.join(10000);
+			} catch (InterruptedException e1) {
+				st.interrupt();
+				Assert.fail("Failed to stopThread before 10000 timeout", e1);
+			}
+			AssertJUnit.assertTrue("Failed to stopThread", !st.isAlive());
+		} else {
+			try {
+				Thread.currentThread().stop();
+				Assert.fail("expected UnsupportedOperationException");
+			} catch (UnsupportedOperationException e) {
+				// expected
+			}
 		}
-		st.stop();
-
-		/*
-		 * Thread.stop() implementation is changed from 1.1.7 to the 1.2
-		 * behaviour. now stop() doesn't ensure immediate thread death, so we
-		 * have to wait till st thread joins back to current thread, before
-		 * making an isAlive() check.
-		 */
-		try {
-			st.join(10000);
-		} catch (InterruptedException e1) {
-			st.interrupt();
-			Assert.fail("Failed to stopThread before 10000 timeout", e1);
-		}
-		AssertJUnit.assertTrue("Failed to stopThread", !st.isAlive());
 	}
 
 	/**
@@ -997,22 +993,32 @@ public class Test_Thread {
 		class StopBeforeStartThread extends Thread {
 			public boolean failed = false;
 
+			@Override
 			public void run() {
 				synchronized (this) {
 					failed = true;
 				}
 			}
 		}
-		try {
-			StopBeforeStartThread t = new StopBeforeStartThread();
-			t.stop();
-			t.start();
-			t.join();
-			synchronized (t) {
-				AssertJUnit.assertFalse("thread should not run if stop called before start, stop()", t.failed);
+		if (VersionCheck.major() < 20) {
+			try {
+				StopBeforeStartThread t = new StopBeforeStartThread();
+				t.stop();
+				t.start();
+				t.join();
+				synchronized (t) {
+					AssertJUnit.assertFalse("thread should not run if stop called before start, stop()", t.failed);
+				}
+			} catch (Exception e) {
+				Assert.fail("Unexpected exception:", e);
+			}
+		} else {
+			try {
+				Thread.currentThread().stop();
+				Assert.fail("expected UnsupportedOperationException");
+			} catch (UnsupportedOperationException e) {
+				// expected
 			}
-		} catch (Exception e) {
-			Assert.fail("Unexpected exception:", e);
 		}
 	}
 
@@ -1021,54 +1027,64 @@ public class Test_Thread {
 	 */
 	@Test
 	public void test_suspend() {
-		int orgval;
-		ResSupThread t = new ResSupThread(Thread.currentThread());
-		try {
-			synchronized (t) {
-				ct = new Thread(t, "Interupt Test6");
-				ct.start();
-				t.wait();
-			}
-			ct.suspend();
-			// Wait to be sure the suspend has occurred
-			Thread.sleep(500);
-			orgval = t.getCheckVal();
-			// Wait to be sure the thread is suspended
-			Thread.sleep(500);
-			AssertJUnit.assertTrue("Failed to suspend thread", orgval == t.getCheckVal());
-			ct.resume();
-			// Wait to be sure the resume has occurred.
-			Thread.sleep(500);
-			AssertJUnit.assertTrue("Failed to resume thread", orgval != t.getCheckVal());
-			ct.interrupt();
-		} catch (InterruptedException e) {
-			Assert.fail("Unexpected interrupt occurred", e);
-		}
-
-		/*
-		 * [PR 106321] suspend() must not synchronize when a Thread is
-		 * suspending itself
-		 */
-		final Object notify = new Object();
-		Thread t1 = new Thread(new Runnable() {
-			public void run() {
-				synchronized (notify) {
-					notify.notify();
+		if (VersionCheck.major() < 20) {
+			int orgval;
+			ResSupThread t = new ResSupThread(Thread.currentThread());
+			try {
+				synchronized (t) {
+					ct = new Thread(t, "Interupt Test6");
+					ct.start();
+					t.wait();
 				}
+				ct.suspend();
+				// Wait to be sure the suspend has occurred
+				Thread.sleep(500);
+				orgval = t.getCheckVal();
+				// Wait to be sure the thread is suspended
+				Thread.sleep(500);
+				AssertJUnit.assertTrue("Failed to suspend thread", orgval == t.getCheckVal());
+				ct.resume();
+				// Wait to be sure the resume has occurred.
+				Thread.sleep(500);
+				AssertJUnit.assertTrue("Failed to resume thread", orgval != t.getCheckVal());
+				ct.interrupt();
+			} catch (InterruptedException e) {
+				Assert.fail("Unexpected interrupt occurred", e);
+			}
+
+			/*
+			 * [PR 106321] suspend() must not synchronize when a Thread is
+			 * suspending itself
+			 */
+			final Object notify = new Object();
+			Thread t1 = new Thread(new Runnable() {
+				@Override
+				public void run() {
+					synchronized (notify) {
+						notify.notify();
+					}
+					Thread.currentThread().suspend();
+				}
+			});
+			try {
+				synchronized (notify) {
+					t1.start();
+					notify.wait();
+				}
+				// wait for Thread to suspend
+				Thread.sleep(500);
+				AssertJUnit.assertTrue("Thread should be alive", t1.isAlive());
+				t1.resume();
+				t1.join();
+			} catch (InterruptedException e) {
+			}
+		} else {
+			try {
 				Thread.currentThread().suspend();
+				Assert.fail("expected UnsupportedOperationException");
+			} catch (UnsupportedOperationException e) {
+				// expected
 			}
-		});
-		try {
-			synchronized (notify) {
-				t1.start();
-				notify.wait();
-			}
-			// wait for Thread to suspend
-			Thread.sleep(500);
-			AssertJUnit.assertTrue("Thread should be alive", t1.isAlive());
-			t1.resume();
-			t1.join();
-		} catch (InterruptedException e) {
 		}
 	}
 
@@ -1098,18 +1114,21 @@ public class Test_Thread {
 	@AfterMethod
 	protected void tearDown() {
 		try {
-			if (st != null)
+			if (st != null) {
 				st.interrupt();
+			}
 		} catch (Exception e) {
 		}
 		try {
-			if (spinner != null)
+			if (spinner != null) {
 				spinner.interrupt();
+			}
 		} catch (Exception e) {
 		}
 		try {
-			if (ct != null)
+			if (ct != null) {
 				ct.interrupt();
+			}
 		} catch (Exception e) {
 		}
 
@@ -1121,13 +1140,14 @@ public class Test_Thread {
 		} catch (Exception e) {
 		}
 
-		/*Make sure that the current thread is not interrupted. If it is, set it back to false.*/
-		/*Otherwise any call to join, wait, sleep from the current thread throws either InterruptedException or IllegalMonitorStateException*/
+		/* Make sure that the current thread is not interrupted. If it is, set it back to false. */
+		/* Otherwise any call to join, wait, sleep from the current thread throws either
+		 * InterruptedException or IllegalMonitorStateException. */
 		if (Thread.currentThread().isInterrupted()) {
 			try {
 				Thread.currentThread().join(10);
 			} catch (InterruptedException e) {
-				//If we are here, current threads interrupted status is set to false.
+				// If we are here, current threads interrupted status is set to false.
 			}
 		}
 	}
