@@ -1,7 +1,7 @@
 package org.openj9.test.java.lang.invoke;
 
-/*******************************************************************************
- * Copyright (c) 1998, 2022 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 1998
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -17,10 +17,10 @@ package org.openj9.test.java.lang.invoke;
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 
 import org.testng.annotations.Test;
 import org.testng.Assert;
@@ -28,6 +28,7 @@ import org.testng.AssertJUnit;
 import java.lang.invoke.*;
 import java.lang.reflect.*;
 import org.openj9.test.java.lang.invoke.helpers.*;
+import org.openj9.test.util.VersionCheck;
 import static java.lang.invoke.MethodHandles.*;
 import static java.lang.invoke.MethodType.*;
 
@@ -690,5 +691,24 @@ public class Test_MethodHandleInfo {
 		AssertJUnit.assertTrue(IAEThrown);
 	}
 
+	/**
+	 * Check that IMPL_LOOKUP.findGetter() works for an instance field at offset 0.
+	 *
+	 * @throws Throwable
+	 */
+	@Test(groups = { "level.sanity" })
+	public void test_ReflectAs_Field_Offset0() throws Throwable {
+		Class<?> fieldType;
+		if (VersionCheck.major() > 8) {
+			fieldType = byte[].class;
+		} else {
+			fieldType = char[].class;
+		}
+		Field field = MethodHandles.Lookup.class.getDeclaredField("IMPL_LOOKUP");
+		field.setAccessible(true);
+		MethodHandles.Lookup lookup = (MethodHandles.Lookup) field.get(null);
+		System.out.println(lookup.findGetter(String.class, "value", fieldType));
+	}
+
 	enum EnumTest {TEST1, TEST2}
 }
