@@ -1,5 +1,5 @@
-/*******************************************************************************
- * Copyright (c) 2016, 2021 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2016
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,16 +15,15 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package com.ibm.dump.tests.jdmpview_heapdump;
 
 public class CreateDumps {
 
 	public static void main(String[] args) {
-		System.gc();
 		System.gc();
 		com.ibm.jvm.Dump.HeapDump();	
 		com.ibm.jvm.Dump.SystemDump();
