@@ -1,5 +1,5 @@
-/*******************************************************************************
- * Copyright (c) 2001, 2012 IBM Corp. and others
+/*
+ * Copyright IBM Corp. and others 2001
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -15,10 +15,10 @@
  * OpenJDK Assembly Exception [2].
  *
  * [1] https://www.gnu.org/software/classpath/license.html
- * [2] http://openjdk.java.net/legal/assembly-exception.html
+ * [2] https://openjdk.org/legal/assembly-exception.html
  *
- * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
- *******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
+ */
 package j9vm.test.jni;
 
 import java.util.Random;
@@ -138,10 +138,8 @@ public class CriticalRegionTest
 	
 	private static void discardAndGC()
 	{
-		// Double-GC to try for an aggressive collect
 		garbage = null;
 		System.gc();
-		System.gc();
 	}
 	
 	private static void reportJNIError()
@@ -868,8 +866,8 @@ public class CriticalRegionTest
 		try {
 			//lock.wait(sleepTime * 2);
 			for(int i = 0; i < threadCount; i++) {
-				/* CMVC 195423 : Increase timeout to 5 seconds to allow all child threads to complete */
-				threads[i].join(sleepTime * 5);
+				/* Increase timeout to 10 seconds to allow all child threads to complete */
+				threads[i].join(sleepTime * 10);
 			}
 		} catch (InterruptedException e) {}
 			
