@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,7 @@
 
 /*
  * ===========================================================================
- * (c) Copyright IBM Corp. 1997, 2020 All Rights Reserved
+ * (c) Copyright IBM Corp. 1997, 2022 All Rights Reserved
  * ===========================================================================
  */
 
@@ -59,11 +59,9 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.List;
 import java.util.NoSuchElementException;
 import java.util.Properties;
-import java.util.Set;
 import java.util.StringTokenizer;
 import java.util.jar.JarFile;
 import java.util.zip.CRC32;
@@ -78,8 +76,6 @@ import jdk.internal.access.JavaNetURLAccess;
 import jdk.internal.access.JavaUtilZipFileAccess;
 import jdk.internal.access.SharedSecrets;
 import jdk.internal.misc.VM;                                    //OpenJ9-shared_classes_misc
-import jdk.internal.util.jar.InvalidJarIndexError;
-import jdk.internal.util.jar.JarIndex;
 import sun.net.util.URLUtil;
 import sun.net.www.ParseUtil;
 import com.ibm.sharedclasses.spi.SharedClassProvider;           //OpenJ9-shared_classes_misc
@@ -100,7 +96,6 @@ public class URLClassPath {
     private static final boolean DISABLE_ACC_CHECKING;
     private static final boolean DISABLE_CP_URL_CHECK;
     private static final boolean DEBUG_CP_URL_CHECK;
-    private static final boolean ENABLE_JAR_INDEX;
 
     static {
         Properties props = GetPropertyAction.privilegedGetProperties();
@@ -120,9 +115,6 @@ public class URLClassPath {
         // the check is not disabled).
         p = props.getProperty("jdk.net.URLClassPath.showIgnoredClassPathEntries");
         DEBUG_CP_URL_CHECK = p != null ? p.equals("true") || p.isEmpty() : false;
-
-        p = props.getProperty("jdk.net.URLClassPath.enableJarIndex");
-        ENABLE_JAR_INDEX = p != null ? p.equals("true") || p.isEmpty() : false;
     }
 
     /* The original search path of URLs. */
@@ -317,7 +309,9 @@ public class URLClassPath {
 
         this.unopenedUrls = unopenedUrls;
         this.path = path;
-        this.jarHandler = null;
+        // the application class loader uses the built-in protocol handler to avoid protocol
+        // handler lookup when opening JAR files on the class path.
+        this.jarHandler = new sun.net.www.protocol.jar.Handler();
         this.acc = null;
     }
 
@@ -618,6 +612,7 @@ public class URLClassPath {
                                         isDefaultJarHandler(url) &&
                                         file.endsWith("!/")) {
                                     // extract the nested URL
+                                    @SuppressWarnings("deprecation")
                                     URL nestedUrl = new URL(file.substring(0, file.length() - 2));
                                     return new JarLoader(nestedUrl, jarHandler, lmap, acc, usingSharedClasses());   //OpenJ9-shared_classes_misc
                                 } else {
@@ -730,7 +725,8 @@ public class URLClassPath {
         URL findResource(final String name, boolean check) {
             URL url;
             try {
-                url = new URL(base, ParseUtil.encodePath(name, false));
+                @SuppressWarnings("deprecation")
+                var _unused = url = new URL(base, ParseUtil.encodePath(name, false));
             } catch (MalformedURLException e) {
                 return null;
             }
@@ -766,7 +762,8 @@ public class URLClassPath {
         Resource getResource(final String name, boolean check) {
             final URL url;
             try {
-                url = new URL(base, ParseUtil.encodePath(name, false));
+                @SuppressWarnings("deprecation")
+                var _unused = url = new URL(base, ParseUtil.encodePath(name, false));
             } catch (MalformedURLException e) {
                 return null;
             }
@@ -836,7 +833,6 @@ public class URLClassPath {
     private static class JarLoader extends Loader {
         private JarFile jar;
         private final URL csu;
-        private JarIndex index;
         private URLStreamHandler handler;
         private final HashMap<String, Loader> lmap;
         private boolean usingSharedClasses;                                     //OpenJ9-shared_classes_misc
@@ -855,7 +851,7 @@ public class URLClassPath {
                           @SuppressWarnings("removal") AccessControlContext acc, boolean usingSharedClasses)   //OpenJ9-shared_classes_misc
             throws IOException
         {
-            super(new URL("jar", "", -1, url + "!/", jarHandler));
+            super(newURL("jar", "", -1, url + "!/", jarHandler));
             this.usingSharedClasses = usingSharedClasses;                       //OpenJ9-shared_classes_misc
             csu = url;
             handler = jarHandler;
@@ -865,6 +861,13 @@ public class URLClassPath {
             ensureOpen();
         }
 
+        @SuppressWarnings("deprecation")
+        private static URL newURL(String protocol, String host, int port, String file, URLStreamHandler handler)
+                throws MalformedURLException
+        {
+            return new URL(protocol, host, port, file, handler);
+        }
+
         @Override
         public void close () throws IOException {
             // closing is synchronized at higher level
@@ -896,35 +899,6 @@ public class URLClassPath {
                                     Thread.dumpStack();
                                 }
                                 jar = getJarFile(csu);
-                                if (!ENABLE_JAR_INDEX) {
-                                    return null;
-                                }
-                                if (usingSharedClasses) {                       //OpenJ9-shared_classes_misc
-                                    /* do not use Jar indexing with shared classes */ //OpenJ9-shared_classes_misc
-                                    index = null;                                   //OpenJ9-shared_classes_misc
-                                } else {                                         //OpenJ9-shared_classes_misc
-                                    index = JarIndex.getJarIndex(jar); //OpenJ9-shared_classes_misc
-                                }                                                //OpenJ9-shared_classes_misc
-                                if (index != null) {
-                                    String[] jarfiles = index.getJarFiles();
-                                // Add all the dependent URLs to the lmap so that loaders
-                                // will not be created for them by URLClassPath.getLoader(int)
-                                // if the same URL occurs later on the main class path.  We set
-                                // Loader to null here to avoid creating a Loader for each
-                                // URL until we actually need to try to load something from them.
-                                    for (int i = 0; i < jarfiles.length; i++) {
-                                        try {
-                                            URL jarURL = new URL(csu, jarfiles[i]);
-                                            // If a non-null loader already exists, leave it alone.
-                                            String urlNoFragString = URLUtil.urlNoFragString(jarURL);
-                                            if (!lmap.containsKey(urlNoFragString)) {
-                                                lmap.put(urlNoFragString, null);
-                                            }
-                                        } catch (MalformedURLException e) {
-                                            continue;
-                                        }
-                                    }
-                                }
                                 return null;
                             }
                         }, acc);
@@ -961,24 +935,13 @@ public class URLClassPath {
                 return checkJar(new JarFile(new File(p.getPath()), true, ZipFile.OPEN_READ,
                         JarFile.runtimeVersion()));
             }
+            @SuppressWarnings("deprecation")
             URLConnection uc = (new URL(getBaseURL(), "#runtime")).openConnection();
             uc.setRequestProperty(USER_AGENT_JAVA_VERSION, JAVA_VERSION);
             JarFile jarFile = ((JarURLConnection)uc).getJarFile();
             return checkJar(jarFile);
         }
 
-        /*
-         * Returns the index of this JarLoader if it exists.
-         */
-        JarIndex getIndex() {
-            try {
-                ensureOpen();
-            } catch (IOException e) {
-                throw new InternalError(e);
-            }
-            return index;
-        }
-
         /*
          * Creates the resource and if the check flag is set to true, checks if
          * is its okay to return the resource.
@@ -994,16 +957,12 @@ public class URLClassPath {
                 } else {
                     nm = name;
                 }
-                url = new URL(getBaseURL(), ParseUtil.encodePath(nm, false));
+                @SuppressWarnings("deprecation")
+                var _unused = url = new URL(getBaseURL(), ParseUtil.encodePath(nm, false));
                 if (check) {
                     URLClassPath.check(url);
                 }
-            } catch (MalformedURLException e) {
-                return null;
-                // throw new IllegalArgumentException("name");
-            } catch (IOException e) {
-                return null;
-            } catch (@SuppressWarnings("removal") AccessControlException e) {
+            } catch (@SuppressWarnings("removal") AccessControlException | IOException e) {
                 return null;
             }
 
@@ -1092,139 +1051,15 @@ public class URLClassPath {
             if (entry != null)
                 return checkResource(name, check, entry);
 
-            if (index == null)
-                return null;
 
-            HashSet<String> visited = new HashSet<>();
-            return getResource(name, check, visited);
-        }
-
-        /*
-         * Version of getResource() that tracks the jar files that have been
-         * visited by linking through the index files. This helper method uses
-         * a HashSet to store the URLs of jar files that have been searched and
-         * uses it to avoid going into an infinite loop, looking for a
-         * non-existent resource.
-         */
-        @SuppressWarnings("removal")
-        Resource getResource(final String name, boolean check,
-                             Set<String> visited) {
-            Resource res;
-            String[] jarFiles;
-            int count = 0;
-            List<String> jarFilesList;
-
-            /* If there no jar files in the index that can potential contain
-             * this resource then return immediately.
-             */
-            if ((jarFilesList = index.get(name)) == null)
-                return null;
-
-            do {
-                int size = jarFilesList.size();
-                jarFiles = jarFilesList.toArray(new String[size]);
-                /* loop through the mapped jar file list */
-                while (count < size) {
-                    String jarName = jarFiles[count++];
-                    JarLoader newLoader;
-                    final URL url;
-
-                    try{
-                        url = new URL(csu, jarName);
-                        String urlNoFragString = URLUtil.urlNoFragString(url);
-                        if ((newLoader = (JarLoader)lmap.get(urlNoFragString)) == null) {
-                            /* no loader has been set up for this jar file
-                             * before
-                             */
-                            newLoader = AccessController.doPrivileged(
-                                new PrivilegedExceptionAction<>() {
-                                    public JarLoader run() throws IOException {
-                                        return new JarLoader(url, handler,
-                                            lmap, acc, usingSharedClasses);   //OpenJ9-shared_classes_misc
-                                    }
-                                }, acc);
-
-                            /* this newly opened jar file has its own index,
-                             * merge it into the parent's index, taking into
-                             * account the relative path.
-                             */
-                            JarIndex newIndex = newLoader.getIndex();
-                            if (newIndex != null) {
-                                int pos = jarName.lastIndexOf('/');
-                                newIndex.merge(this.index, (pos == -1 ?
-                                    null : jarName.substring(0, pos + 1)));
-                            }
-
-                            /* put it in the global hashtable */
-                            lmap.put(urlNoFragString, newLoader);
-                        }
-                    } catch (PrivilegedActionException pae) {
-                        continue;
-                    } catch (MalformedURLException e) {
-                        continue;
-                    }
-
-                    /* Note that the addition of the url to the list of visited
-                     * jars incorporates a check for presence in the hashmap
-                     */
-                    boolean visitedURL = !visited.add(URLUtil.urlNoFragString(url));
-                    if (!visitedURL) {
-                        try {
-                            newLoader.ensureOpen();
-                        } catch (IOException e) {
-                            throw new InternalError(e);
-                        }
-                        final JarEntry entry = newLoader.jar.getJarEntry(name);
-                        if (entry != null) {
-                            return newLoader.checkResource(name, check, entry);
-                        }
-
-                        /* Verify that at least one other resource with the
-                         * same package name as the lookedup resource is
-                         * present in the new jar
-                         */
-                        if (!newLoader.validIndex(name)) {
-                            /* the mapping is wrong */
-                            throw new InvalidJarIndexError("Invalid index");
-                        }
-                    }
-
-                    /* If newLoader is the current loader or if it is a
-                     * loader that has already been searched or if the new
-                     * loader does not have an index then skip it
-                     * and move on to the next loader.
-                     */
-                    if (visitedURL || newLoader == this ||
-                            newLoader.getIndex() == null) {
-                        continue;
-                    }
-
-                    /* Process the index of the new loader
-                     */
-                    if ((res = newLoader.getResource(name, check, visited))
-                            != null) {
-                        return res;
-                    }
-                }
-                // Get the list of jar files again as the list could have grown
-                // due to merging of index files.
-                jarFilesList = index.get(name);
-
-            // If the count is unchanged, we are done.
-            } while (count < jarFilesList.size());
             return null;
         }
 
-
         /*
          * Returns the JAR file local class path, or null if none.
          */
         @Override
         URL[] getClassPath() throws IOException {
-            if (index != null) {
-                return null;
-            }
-
             ensureOpen();
 
             // Only get manifest when necessary
@@ -1255,6 +1090,7 @@ public class URLClassPath {
             int i = 0;
             while (st.hasMoreTokens()) {
                 String path = st.nextToken();
+                @SuppressWarnings("deprecation")
                 URL url = DISABLE_CP_URL_CHECK ? new URL(base, path) : tryResolve(base, path);
                 if (url != null) {
                     urls[i] = url;
@@ -1291,6 +1127,7 @@ public class URLClassPath {
          * @throws MalformedURLException
          */
         static URL tryResolveFile(URL base, String input) throws MalformedURLException {
+            @SuppressWarnings("deprecation")
             URL retVal = new URL(base, input);
             if (input.indexOf(':') >= 0 &&
                     !"file".equalsIgnoreCase(retVal.getProtocol())) {
@@ -1312,6 +1149,7 @@ public class URLClassPath {
         static URL tryResolveNonFile(URL base, String input) throws MalformedURLException {
             String child = input.replace(File.separatorChar, '/');
             if (isRelative(child)) {
+                @SuppressWarnings("deprecation")
                 URL url = new URL(base, child);
                 String bp = base.getPath();
                 String urlp = url.getPath();
@@ -1356,7 +1194,8 @@ public class URLClassPath {
             String path = url.getFile().replace('/', File.separatorChar);
             path = ParseUtil.decode(path);
             dir = (new File(path)).getCanonicalFile();
-            normalizedBase = new URL(getBaseURL(), ".");
+            @SuppressWarnings("deprecation")
+            var _unused = normalizedBase = new URL(getBaseURL(), ".");
         }
 
         /*
@@ -1375,7 +1214,8 @@ public class URLClassPath {
         Resource getResource(final String name, boolean check) {
             final URL url;
             try {
-                url = new URL(getBaseURL(), ParseUtil.encodePath(name, false));
+                @SuppressWarnings("deprecation")
+                var _unused = url = new URL(getBaseURL(), ParseUtil.encodePath(name, false));
 
                 if (url.getFile().startsWith(normalizedBase.getFile()) == false) {
                     // requested resource had ../..'s in path
