@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,7 @@
 
 /*
  * ===========================================================================
- * (c) Copyright IBM Corp. 2022, 2022 All Rights Reserved
+ * (c) Copyright IBM Corp. 2022, 2024 All Rights Reserved
  * ===========================================================================
  */
 
@@ -57,6 +57,9 @@
  */
 
 
+#include <assert.h>
+
+#include "criuhelpers.h"
 #include "java.h"
 #include "jni.h"
 
@@ -127,7 +130,7 @@ static void TranslateApplicationArgs(int jargc, const char **jargv, int *pargc,
 static jboolean AddApplicationOptions(int cpathc, const char **cpathv);
 static void SetApplicationClassPath(const char**);
 
-static void PrintJavaVersion(JNIEnv *env, jboolean extraLF);
+static void PrintJavaVersion(JNIEnv *env);
 static void PrintUsage(JNIEnv* env, jboolean doXUsage);
 static void ShowSettings(JNIEnv* env, char *optString);
 static void ShowResolvedModules(JNIEnv* env);
@@ -137,6 +140,8 @@ static jboolean ValidateModules(JNIEnv* env);
 
 static void SetPaths(int argc, char **argv);
 
+static int parse_size(const char *s, jlong *result);
+
 static void DumpState();
 
 enum OptionKind {
@@ -224,6 +229,53 @@ static jlong initialHeapSize    = 0;  /* initial heap size */
 #define STACK_SIZE_MINIMUM (64 * KB)
 #endif
 
+static jboolean
+parseXmso(JLI_List openj9Args)
+{
+    jboolean result = JNI_FALSE;
+    size_t i = openj9Args->size;
+    while (i > 0) {
+        i -= 1;
+        if (JLI_StrCCmp(openj9Args->elements[i], "-Xmso") == 0) {
+            jlong tmp = 0;
+            if (parse_size(openj9Args->elements[i] + 5, &tmp)) {
+                threadStackSize = tmp;
+                result = JNI_TRUE;
+                if (threadStackSize > 0 && threadStackSize < (jlong)STACK_SIZE_MINIMUM) {
+                    threadStackSize = STACK_SIZE_MINIMUM;
+                }
+            }
+            break;
+        }
+    }
+    JLI_List_free(openj9Args);
+    return result;
+}
+
+static void
+parseXmsoInFile(const char *filename)
+{
+    JLI_List openj9Args = JLI_ParseOpenJ9ArgsFile(filename);
+    if (openj9Args != NULL) {
+        jboolean result = parseXmso(openj9Args);
+        if (JLI_IsTraceLauncher() && result) {
+            printf("Set -Xmso%ld from file %s\n", (long)threadStackSize, filename);
+        }
+    }
+}
+
+static void
+parseXmsoInEnv(const char *envVar)
+{
+    JLI_List openj9Args = JLI_List_new(8); /* 8 is arbitrary */
+    if (JLI_ParseOpenJ9ArgsFromEnvVar(openj9Args, envVar)) {
+        jboolean result = parseXmso(openj9Args);
+        if (JLI_IsTraceLauncher() && result) {
+            printf("Set -Xmso%ld from env var %s\n", (long)threadStackSize, envVar);
+        }
+    }
+}
+
 /*
  * Entry point.
  */
@@ -323,6 +375,19 @@ JLI_Launch(int argc, char ** argv,              /* main argc, argv */
         }
     }
 
+    /* Process -Xmso to set the main thread stack size. May be overridden by
+     * a later command line option.
+     */
+    {
+#define OPTIONS_DEFAULT_PATH "/lib/options.default"
+        char optionsfile[sizeof(jrepath) + sizeof(OPTIONS_DEFAULT_PATH) - 1];
+        JLI_Snprintf(optionsfile, sizeof(optionsfile), "%s" OPTIONS_DEFAULT_PATH, jrepath);
+        parseXmsoInFile(optionsfile);
+        parseXmsoInEnv("JAVA_TOOL_OPTIONS");
+        parseXmsoInEnv("OPENJ9_JAVA_OPTIONS");
+        parseXmsoInEnv("IBM_JAVA_OPTIONS");
+    }
+
     /* Parse command line options; if the return value of
      * ParseArguments is false, the program should exit.
      */
@@ -405,11 +470,17 @@ JavaMain(void* _args)
     JNIEnv *env = 0;
     jclass mainClass = NULL;
     jclass appClass = NULL; // actual application class being launched
-    jmethodID mainID;
     jobjectArray mainArgs;
+    jmethodID mainID;
+    jmethodID constructor;
+    jobject mainObject;
     int ret = 0;
     jlong start = 0, end = 0;
 
+#if defined(J9VM_OPT_CRAC_SUPPORT)
+    handleCRaCRestore(argc, argv);
+#endif /* defined(J9VM_OPT_CRAC_SUPPORT) */
+
     RegisterThread();
 
     /* Initialize the virtual machine */
@@ -445,7 +516,7 @@ JavaMain(void* _args)
     }
 
     if (printVersion || showVersion) {
-        PrintJavaVersion(env, showVersion);
+        PrintJavaVersion(env);
         CHECK_EXCEPTION_LEAVE(0);
         if (printVersion) {
             LEAVE();
@@ -543,12 +614,55 @@ JavaMain(void* _args)
      * is not required. The main method is invoked here so that extraneous java
      * stacks are not in the application stack trace.
      */
-    mainID = (*env)->GetStaticMethodID(env, mainClass, "main",
-                                       "([Ljava/lang/String;)V");
-    CHECK_EXCEPTION_NULL_LEAVE(mainID);
+#define MAIN_WITHOUT_ARGS 1
+#define MAIN_NONSTATIC 2
 
-    /* Invoke main method. */
-    (*env)->CallStaticVoidMethod(env, mainClass, mainID, mainArgs);
+    jclass helperClass = GetLauncherHelperClass(env);
+    jmethodID getMainType = (*env)->GetStaticMethodID(env, helperClass,
+                                                      "getMainType",
+                                                      "()I");
+    CHECK_EXCEPTION_NULL_LEAVE(getMainType);
+    int mainType = (*env)->CallStaticIntMethod(env, helperClass, getMainType);
+    CHECK_EXCEPTION_LEAVE(mainType);
+
+    switch (mainType) {
+    case 0: {
+        mainID = (*env)->GetStaticMethodID(env, mainClass, "main",
+                                           "([Ljava/lang/String;)V");
+        CHECK_EXCEPTION_NULL_LEAVE(mainID);
+        (*env)->CallStaticVoidMethod(env, mainClass, mainID, mainArgs);
+        break;
+        }
+    case MAIN_WITHOUT_ARGS: {
+        mainID = (*env)->GetStaticMethodID(env, mainClass, "main",
+                                           "()V");
+        CHECK_EXCEPTION_NULL_LEAVE(mainID);
+        (*env)->CallStaticVoidMethod(env, mainClass, mainID);
+        break;
+        }
+    case MAIN_NONSTATIC: {
+        constructor = (*env)->GetMethodID(env, mainClass, "<init>", "()V");
+        CHECK_EXCEPTION_NULL_LEAVE(constructor);
+        mainObject = (*env)->NewObject(env, mainClass, constructor);
+        CHECK_EXCEPTION_NULL_LEAVE(mainObject);
+        mainID = (*env)->GetMethodID(env, mainClass, "main",
+                                     "([Ljava/lang/String;)V");
+        CHECK_EXCEPTION_NULL_LEAVE(mainID);
+        (*env)->CallVoidMethod(env, mainObject, mainID, mainArgs);
+        break;
+        }
+    case MAIN_NONSTATIC | MAIN_WITHOUT_ARGS: {
+        constructor = (*env)->GetMethodID(env, mainClass, "<init>", "()V");
+        CHECK_EXCEPTION_NULL_LEAVE(constructor);
+        mainObject = (*env)->NewObject(env, mainClass, constructor);
+        CHECK_EXCEPTION_NULL_LEAVE(mainObject);
+        mainID = (*env)->GetMethodID(env, mainClass, "main",
+                                     "()V");
+        CHECK_EXCEPTION_NULL_LEAVE(mainID);
+        (*env)->CallVoidMethod(env, mainObject, mainID);
+        break;
+        }
+    }
 
     /*
      * The launcher's exit code (in the absence of calls to
@@ -557,7 +671,14 @@ JavaMain(void* _args)
     ret = (*env)->ExceptionOccurred(env) == NULL ? 0 : 1;
 
     LEAVE();
+#if defined(__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wreturn-type"
+#endif
 }
+#if defined(__GNUC__)
+#pragma GCC diagnostic pop
+#endif
 
 /*
  * Test if the given name is one of the class path options.
@@ -872,8 +993,13 @@ AddOption(char *str, void *info)
      */
 
     /* In OpenJ9 -Xmso is used to set native stack size instead of -Xss. -Xss is used to
-     * set Java thread size only, which is handled in the JVM code.
+     * set Java thread size only, which is handled in the JVM code. Check for -Xmso in any
+     * -Xoptionsfile= and on the command line itself. The default.options file and relevent
+     * environment variables are checked earlier.
      */
+    if (JLI_StrCCmp(str, "-Xoptionsfile=") == 0) {
+        parseXmsoInFile(str + 14);
+    }
     if (JLI_StrCCmp(str, "-Xmso") == 0) {
         jlong tmp;
         if (parse_size(str + 5, &tmp)) {
@@ -881,6 +1007,9 @@ AddOption(char *str, void *info)
             if (threadStackSize > 0 && threadStackSize < (jlong)STACK_SIZE_MINIMUM) {
                 threadStackSize = STACK_SIZE_MINIMUM;
             }
+            if (JLI_IsTraceLauncher()) {
+                printf("Set -Xmso%ld from command line\n", (long)threadStackSize);
+            }
         }
     }
 
@@ -916,10 +1045,11 @@ SetClassPath(const char *s)
     if (sizeof(format) - 2 + JLI_StrLen(s) < JLI_StrLen(s))
         // s is became corrupted after expanding wildcards
         return;
-    def = JLI_MemAlloc(sizeof(format)
+    size_t defSize = sizeof(format)
                        - 2 /* strlen("%s") */
-                       + JLI_StrLen(s));
-    sprintf(def, format, s);
+                       + JLI_StrLen(s);
+    def = JLI_MemAlloc(defSize);
+    snprintf(def, defSize, format, s);
     AddOption(def, NULL);
     if (s != orig)
         JLI_MemFree((char *) s);
@@ -974,13 +1104,9 @@ SelectVersion(int argc, char **argv, char **main_class)
 {
     char    *arg;
     char    *operand;
-    char    *version = NULL;
-    char    *jre = NULL;
     int     jarflag = 0;
     int     headlessflag = 0;
-    int     restrict_search = -1;               /* -1 implies not known */
     manifest_info info;
-    char    env_entry[MAXNAMELEN + 24] = ENV_ENTRY "=";
     char    *splash_file_name = NULL;
     char    *splash_jar_name = NULL;
     char    *env_in;
@@ -1019,7 +1145,7 @@ SelectVersion(int argc, char **argv, char **main_class)
 
     argc--;
     argv++;
-    while ((arg = *argv) != 0 && *arg == '-') {
+    while (argc > 0 && *(arg = *argv) == '-') {
         has_arg = IsOptionWithArgument(argc, argv);
         if (JLI_StrCCmp(arg, "-version:") == 0) {
             JLI_ReportErrorMessage(SPC_ERROR1);
@@ -1217,11 +1343,11 @@ ParseArguments(int *pargc, char ***pargv,
     int argc = *pargc;
     char **argv = *pargv;
     int mode = LM_UNKNOWN;
-    char *arg;
+    char *arg = NULL;
 
     *pret = 0;
 
-    while ((arg = *argv) != 0 && *arg == '-') {
+    while (argc > 0 && *(arg = *argv) == '-') {
         char *option = NULL;
         char *value = NULL;
         int kind = GetOpt(&argc, &argv, &option, &value);
@@ -1376,8 +1502,9 @@ ParseArguments(int *pargc, char ***pargv,
                    JLI_StrCCmp(arg, "-oss") == 0 ||
                    JLI_StrCCmp(arg, "-ms") == 0 ||
                    JLI_StrCCmp(arg, "-mx") == 0) {
-            char *tmp = JLI_MemAlloc(JLI_StrLen(arg) + 6);
-            sprintf(tmp, "-X%s", arg + 1); /* skip '-' */
+            size_t tmpSize = JLI_StrLen(arg) + 6;
+            char *tmp = JLI_MemAlloc(tmpSize);
+            snprintf(tmp, tmpSize, "-X%s", arg + 1); /* skip '-' */
             AddOption(tmp, NULL);
         } else if (JLI_StrCmp(arg, "-checksource") == 0 ||
                    JLI_StrCmp(arg, "-cs") == 0 ||
@@ -1386,6 +1513,8 @@ ParseArguments(int *pargc, char ***pargv,
             JLI_ReportErrorMessage(ARG_WARN, arg);
         } else if (JLI_StrCCmp(arg, "-splash:") == 0) {
             ; /* Ignore machine independent options already handled */
+        } else if (JLI_StrCmp(arg, "--disable-@files") == 0) {
+            ; /* Ignore --disable-@files option already handled */
         } else if (ProcessPlatformOption(arg)) {
             ; /* Processing of platform dependent options */
         } else {
@@ -1636,7 +1765,8 @@ TranslateApplicationArgs(int jargc, const char **jargv, int *pargc, char ***parg
     for (i = 0; i < jargc; i++) {
         const char *arg = jargv[i];
         if (arg[0] == '-' && arg[1] == 'J') {
-            *nargv++ = ((arg + 2) == NULL) ? NULL : JLI_StringDup(arg + 2);
+            assert(arg[2] != '\0' && "Invalid JAVA_ARGS or EXTRA_JAVA_ARGS defined by build");
+            *nargv++ = JLI_StringDup(arg + 2);
         }
     }
 
@@ -1708,8 +1838,9 @@ AddApplicationOptions(int cpathc, const char **cpathv)
             s = (char *) JLI_WildcardExpandClasspath(s);
             /* 40 for -Denv.class.path= */
             if (JLI_StrLen(s) + 40 > JLI_StrLen(s)) { // Safeguard from overflow
-                envcp = (char *)JLI_MemAlloc(JLI_StrLen(s) + 40);
-                sprintf(envcp, "-Denv.class.path=%s", s);
+                size_t envcpSize = JLI_StrLen(s) + 40;
+                envcp = (char *)JLI_MemAlloc(envcpSize);
+                snprintf(envcp, envcpSize, "-Denv.class.path=%s", s);
                 AddOption(envcp, NULL);
             }
         }
@@ -1721,8 +1852,9 @@ AddApplicationOptions(int cpathc, const char **cpathv)
     }
 
     /* 40 for '-Dapplication.home=' */
-    apphome = (char *)JLI_MemAlloc(JLI_StrLen(home) + 40);
-    sprintf(apphome, "-Dapplication.home=%s", home);
+    size_t apphomeSize = JLI_StrLen(home) + 40;
+    apphome = (char *)JLI_MemAlloc(apphomeSize);
+    snprintf(apphome, apphomeSize, "-Dapplication.home=%s", home);
     AddOption(apphome, NULL);
 
     /* How big is the application's classpath? */
@@ -1811,7 +1943,7 @@ static void SetJavaLauncherProp() {
  * Prints the version information from the java.version and other properties.
  */
 static void
-PrintJavaVersion(JNIEnv *env, jboolean extraLF)
+PrintJavaVersion(JNIEnv *env)
 {
     jclass ver;
     jmethodID print;
@@ -1819,7 +1951,7 @@ PrintJavaVersion(JNIEnv *env, jboolean extraLF)
     NULL_CHECK(ver = FindBootStrapClass(env, "java/lang/VersionProps"));
     NULL_CHECK(print = (*env)->GetStaticMethodID(env,
                                                  ver,
-                                                 (extraLF == JNI_TRUE) ? "println" : "print",
+                                                 "print",
                                                  "(Z)V"
                                                  )
               );
@@ -2014,7 +2146,6 @@ ReadKnownVMs(const char *jvmCfgName, jboolean speculative)
     int vmType;
     char *tmpPtr;
     char *altVMName = NULL;
-    char *serverClassVMName = NULL;
     static char *whiteSpace = " \t";
     if (JLI_IsTraceLauncher()) {
         start = CurrentTimeMicros();
