@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,7 +28,7 @@
 
 /*
  * @test
- * @bug 8271820
+ * @bug 8271820 8300924
  * @modules java.base/jdk.internal.reflect
  * @summary Test compliance of ConstructorAccessor, FieldAccessor, MethodAccessor implementations
  * @run testng/othervm --add-exports java.base/jdk.internal.reflect=ALL-UNNAMED -Djdk.reflect.useDirectMethodHandle=true -XX:-ShowCodeDetailsInExceptionMessages MethodHandleAccessorsTest
@@ -60,6 +60,8 @@ import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
 public class MethodHandleAccessorsTest {
+    static final boolean newImpl = Boolean.getBoolean("jdk.reflect.useDirectMethodHandle");
+
 
     public static void public_static_V() {}
 
@@ -73,6 +75,12 @@ public class MethodHandleAccessorsTest {
 
     public static int public_static_I_I(int i) { return i; }
 
+    public static void public_static_V_L3(Object o1, Object o2, Object o3) { }
+
+    public static void public_static_V_L4(Object o1, Object o2, Object o3, Object o4) { }
+
+    public static void public_V_L5(Object o1, Object o2, Object o3, Object o4, Object o5) { }
+
     public void public_I_V(int i) {}
 
     public int public_I_I(int i) { return i; }
@@ -151,6 +159,14 @@ public class MethodHandleAccessorsTest {
             this(varargs_object(first, rest));
         }
 
+        public Public(Object o1, Object o2, Object o3) {
+            this("3-arg constructor");
+        }
+
+        public Public(Object o1, Object o2, Object o3, Object o4) {
+            this("4-arg constructor");
+        }
+
         public Public(RuntimeException exc) {
             throw exc;
         }
@@ -407,6 +423,10 @@ public class MethodHandleAccessorsTest {
     private static final Throwable[] wrong_argument_count_no_details = new Throwable[] {
             new IllegalArgumentException("wrong number of arguments")
     };
+
+    private static final Throwable[] wrong_argument_count_zero_args = new Throwable[] {
+            new IllegalArgumentException("wrong number of arguments: 0 expected:")
+    };
     private static final Throwable[] wrong_argument_count = new Throwable[] {
             new IllegalArgumentException("wrong number of arguments"),
             new IllegalArgumentException("array is not of length 1")
@@ -464,7 +484,6 @@ public class MethodHandleAccessorsTest {
     private Object[][] testOneArgMethods() {
         MethodHandleAccessorsTest inst = new MethodHandleAccessorsTest();
         Object wrongInst = new Object();
-        boolean newImpl = Boolean.getBoolean("jdk.reflect.useDirectMethodHandle");
         return new Object[][]{
             new Object[] {"public_static_I_V",  int.class, null, new Object[]{12}, null, noException},
             new Object[] {"public_static_I_I",  int.class, null, new Object[]{12}, 12, noException},
@@ -495,6 +514,27 @@ public class MethodHandleAccessorsTest {
         };
     }
 
+    @DataProvider(name = "testMultiArgMethods")
+    private Object[][] testMultiArgMethods() {
+        MethodHandleAccessorsTest inst = new MethodHandleAccessorsTest();
+        Class<?>[] params_L3 = new Class<?>[] { Object.class, Object.class, Object.class};
+        Class<?>[] params_L4 = new Class<?>[] { Object.class, Object.class, Object.class, Object.class};
+        Class<?>[] params_L5 = new Class<?>[] { Object.class, Object.class, Object.class, Object.class, Object.class};
+
+        return new Object[][]{
+                new Object[]{"public_static_V_L3", params_L3, null, new Object[3], null, noException},
+                new Object[]{"public_static_V_L4", params_L4, null, new Object[4], null, noException},
+                new Object[]{"public_V_L5", params_L5, inst, new Object[5], null, noException},
+                // wrong arguments
+                new Object[]{"public_static_V_L3", params_L3, null, null, null,
+                             newImpl ? wrong_argument_count_zero_args : wrong_argument_count_no_details},
+                new Object[]{"public_static_V_L4", params_L4, null, new Object[0], null,
+                             newImpl ? wrong_argument_count_zero_args : wrong_argument_count_no_details},
+                new Object[]{"public_V_L5", params_L5, inst, null, null,
+                             newImpl ? wrong_argument_count_zero_args : wrong_argument_count_no_details},
+        };
+    }
+
     @DataProvider(name = "testMethodsWithVarargs")
     private Object[][] testMethodsWithVarargs() {
         Class<?>[] paramTypes = new Class<?>[] { int[].class };
@@ -525,6 +565,12 @@ public class MethodHandleAccessorsTest {
         doTest(MethodHandleAccessorsTest.class.getDeclaredMethod(methodname, paramType), target, args, expectedReturn, expectedExpections);
     }
 
+    @Test(dataProvider = "testMultiArgMethods")
+    public void testMultiArgMethod(String methodname, Class<?>[] paramTypes, Object target, Object[] args,
+                                 Object expectedReturn, Throwable[] expectedExpections) throws Exception {
+        doTest(MethodHandleAccessorsTest.class.getDeclaredMethod(methodname, paramTypes), target, args, expectedReturn, expectedExpections);
+    }
+
     @Test(dataProvider = "testMethodsWithVarargs")
     public void testMethodsWithVarargs(String methodname, Class<?>[] paramTypes, Object target, Object[] args,
                                        Object expectedReturn, Throwable[] expectedExpections) throws Exception {
@@ -563,6 +609,26 @@ public class MethodHandleAccessorsTest {
         doTest(Public.class.getDeclaredConstructor(paramTypes), args, expectedReturn, expectedExpections);
     }
 
+    @DataProvider(name = "testMultiArgConstructors")
+    private Object[][] testMultiArgConstructors() {
+        Class<?>[] params_L3 = new Class<?>[] { Object.class, Object.class, Object.class};
+        Class<?>[] params_L4 = new Class<?>[] { Object.class, Object.class, Object.class, Object.class};
+        Object o = "arg";
+        return new Object[][]{
+                new Object[]{params_L3, new Object[3], new Public(o, o, o), noException},
+                new Object[]{params_L4, new Object[4], new Public(o, o, o, o), noException},
+                new Object[]{params_L3, new Object[]{}, null,
+                             newImpl ? wrong_argument_count_zero_args : wrong_argument_count_no_details},
+                new Object[]{params_L4, null, null,
+                             newImpl ? wrong_argument_count_zero_args : wrong_argument_count_no_details},
+        };
+    }
+
+    @Test(dataProvider = "testMultiArgConstructors")
+    public void testMultiArgConstructors(Class<?>[] paramTypes, Object[] args, Object expectedReturn, Throwable[] expectedExpections) throws Exception {
+        doTest(Public.class.getDeclaredConstructor(paramTypes), args, expectedReturn, expectedExpections);
+    }
+
     @Test
     public void testOtherConstructors() throws Exception {
         doTest(Private.class.getDeclaredConstructor(), new Object[]{}, new Private());
